@@ -30,6 +30,7 @@ server.  I don't exactly remember the steps, so ping him for help.
 
     --jderose 2009-02-13
 """
+from __future__ import print_function
 
 from os import path
 import sys
@@ -72,16 +73,15 @@ api.finalize()
 ra = api.Backend.ra
 
 def assert_equal(trial, reference):
-    keys = reference.keys()
-    keys.sort()
+    keys = sorted(reference)
     for key in keys:
         reference_val = reference[key]
         trial_val = trial[key]
 
-        if reference_decode.has_key(key):
+        if key in reference_decode:
             reference_val = reference_decode[key](reference_val)
 
-        if trial_decode.has_key(key):
+        if key in trial_decode:
             trial_val = trial_decode[key](trial_val)
 
         assert reference_val == trial_val, \
@@ -90,15 +90,15 @@ def assert_equal(trial, reference):
 
 
 api.log.info('******** Testing ra.request_certificate() ********')
-request_result = ra.request_certificate(csr)
-if verbose: print "request_result=\n%s" % request_result
+request_result = ra.request_certificate(csr, ra.DEFAULT_PROFILE)
+if verbose: print("request_result=\n%s" % request_result)
 assert_equal(request_result,
              {'subject' : subject,
               })
 
 api.log.info('******** Testing ra.check_request_status() ********')
 status_result = ra.check_request_status(request_result['request_id'])
-if verbose: print "status_result=\n%s" % status_result
+if verbose: print("status_result=\n%s" % status_result)
 assert_equal(status_result,
              {'serial_number'       : request_result['serial_number'],
               'request_id'          : request_result['request_id'],
@@ -107,7 +107,7 @@ assert_equal(status_result,
 
 api.log.info('******** Testing ra.get_certificate() ********')
 get_result = ra.get_certificate(request_result['serial_number'])
-if verbose: print "get_result=\n%s" % get_result
+if verbose: print("get_result=\n%s" % get_result)
 assert_equal(get_result,
              {'serial_number' : request_result['serial_number'],
               'certificate'   : request_result['certificate'],
@@ -116,7 +116,7 @@ assert_equal(get_result,
 api.log.info('******** Testing ra.revoke_certificate() ********')
 revoke_result = ra.revoke_certificate(request_result['serial_number'],
                                       revocation_reason=6)  # Put on hold
-if verbose: print "revoke_result=\n%s" % revoke_result
+if verbose: print("revoke_result=\n%s" % revoke_result)
 assert_equal(revoke_result,
              {'revoked' : True
               })
@@ -124,7 +124,7 @@ assert_equal(revoke_result,
 
 api.log.info('******** Testing ra.take_certificate_off_hold() ********')
 unrevoke_result = ra.take_certificate_off_hold(request_result['serial_number'])
-if verbose: print "unrevoke_result=\n%s" % unrevoke_result
+if verbose: print("unrevoke_result=\n%s" % unrevoke_result)
 assert_equal(unrevoke_result,
              {'unrevoked' : True
               })
