@@ -56,6 +56,7 @@ static void ipadb_context_free(krb5_context kcontext,
             ldap_unbind_ext_s((*ctx)->lcontext, NULL, NULL);
         }
         free((*ctx)->supp_encs);
+        free((*ctx)->def_encs);
         ipadb_mspac_struct_free(&(*ctx)->mspac);
         krb5_free_default_realm(kcontext, (*ctx)->realm);
 
@@ -260,12 +261,13 @@ static int ipadb_load_global_config(struct ipadb_context *ipactx)
                             vals[i]->bv_val, vals[i]->bv_len) == 0) {
                 ipactx->config.disable_last_success = true;
                 continue;
-            }
-
-            if (strncasecmp("KDC:Disable Lockout",
-                            vals[i]->bv_val, vals[i]->bv_len) == 0) {
+            } else if (strncasecmp("KDC:Disable Lockout",
+                                   vals[i]->bv_val, vals[i]->bv_len) == 0) {
                 ipactx->config.disable_lockout = true;
                 continue;
+            } else if (strncasecmp("KDC:Disable Default Preauth for SPNs",
+                                   vals[i]->bv_val, vals[i]->bv_len) == 0) {
+                ipactx->config.disable_preauth_for_spns = true;
             }
         }
     }
@@ -316,19 +318,68 @@ ipadb_get_global_config(struct ipadb_context *ipactx)
     return &ipactx->config;
 }
 
-int ipadb_get_connection(struct ipadb_context *ipactx)
+int ipadb_get_enc_salt_types(struct ipadb_context *ipactx,
+                             LDAPMessage *entry, char *attr,
+                             krb5_key_salt_tuple **enc_salt_types,
+                             int *n_enc_salt_types)
 {
     struct berval **vals = NULL;
+    char **cvals = NULL;
+    int c = 0;
+    int i;
+    int ret = 0;
+    krb5_key_salt_tuple *kst;
+    int n_kst;
+
+    vals = ldap_get_values_len(ipactx->lcontext, entry, attr);
+    if (!vals || !vals[0]) {
+        goto done;
+    }
+
+    for (c = 0; vals[c]; c++) /* count */ ;
+    cvals = calloc(c, sizeof(char *));
+    if (!cvals) {
+        ret = ENOMEM;
+        goto done;
+    }
+    for (i = 0; i < c; i++) {
+        cvals[i] = strndup(vals[i]->bv_val, vals[i]->bv_len);
+        if (!cvals[i]) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    ret = parse_bval_key_salt_tuples(ipactx->kcontext,
+                                     (const char * const *)cvals, c,
+                                     &kst, &n_kst);
+    if (ret) {
+        goto done;
+    }
+
+    if (*enc_salt_types) {
+        free(*enc_salt_types);
+    }
+
+    *enc_salt_types = kst;
+    *n_enc_salt_types = n_kst;
+
+done:
+    ldap_value_free_len(vals);
+    for (i = 0; i < c && cvals[i]; i++) {
+        free(cvals[i]);
+    }
+    free(cvals);
+    return ret;
+}
+
+int ipadb_get_connection(struct ipadb_context *ipactx)
+{
     struct timeval tv = { 5, 0 };
     LDAPMessage *res = NULL;
     LDAPMessage *first;
-    krb5_key_salt_tuple *kst;
-    int n_kst;
     int ret;
     int v3;
-    int i;
-    char **cvals = NULL;
-    int c = 0;
 
     if (!ipactx->uri) {
         return EINVAL;
@@ -383,38 +434,21 @@ int ipadb_get_connection(struct ipadb_context *ipactx)
         goto done;
     }
 
-    vals = ldap_get_values_len(ipactx->lcontext, first,
-                               "krbSupportedEncSaltTypes");
-    if (!vals || !vals[0]) {
-        goto done;
-    }
-
-    for (c = 0; vals[c]; c++) /* count */ ;
-    cvals = calloc(c, sizeof(char *));
-    if (!cvals) {
-        ret = ENOMEM;
-        goto done;
-    }
-    for (i = 0; i < c; i++) {
-        cvals[i] = strndup(vals[i]->bv_val, vals[i]->bv_len);
-        if (!cvals[i]) {
-            ret = ENOMEM;
-            goto done;
-        }
-    }
-
-    ret = parse_bval_key_salt_tuples(ipactx->kcontext,
-                                     (const char * const *)cvals, c,
-                                     &kst, &n_kst);
+    /* defaults first, this is used to tell what default enc:salts to use
+     * for kadmin password changes */
+    ret = ipadb_get_enc_salt_types(ipactx, first,  "krbDefaultEncSaltTypes",
+                                   &ipactx->def_encs, &ipactx->n_def_encs);
     if (ret) {
         goto done;
     }
 
-    if (ipactx->supp_encs) {
-        free(ipactx->supp_encs);
+    /* supported enc salt types, use to tell kadmin what to accept
+     * but also to detect if kadmin is requesting the default set */
+    ret = ipadb_get_enc_salt_types(ipactx, first, "krbSupportedEncSaltTypes",
+                                   &ipactx->supp_encs, &ipactx->n_supp_encs);
+    if (ret) {
+        goto done;
     }
-    ipactx->supp_encs = kst;
-    ipactx->n_supp_encs = n_kst;
 
     /* get additional options */
     ret = ipadb_load_global_config(ipactx);
@@ -433,12 +467,6 @@ int ipadb_get_connection(struct ipadb_context *ipactx)
 done:
     ldap_msgfree(res);
 
-    ldap_value_free_len(vals);
-    for (i = 0; i < c && cvals[i]; i++) {
-        free(cvals[i]);
-    }
-    free(cvals);
-
     if (ret) {
         if (ipactx->lcontext) {
             ldap_unbind_ext_s(ipactx->lcontext, NULL, NULL);
