@@ -93,6 +93,7 @@ struct ipadb_global_config {
 	bool disable_lockout;
 	char **authz_data;
 	enum ipadb_user_auth user_auth;
+    bool disable_preauth_for_spns;
 };
 
 struct ipadb_context {
@@ -106,6 +107,8 @@ struct ipadb_context {
     bool override_restrictions;
     krb5_key_salt_tuple *supp_encs;
     int n_supp_encs;
+    krb5_key_salt_tuple *def_encs;
+    int n_def_encs;
     struct ipadb_mspac *mspac;
 
     /* Don't access this directly, use ipadb_get_global_config(). */
@@ -272,6 +275,14 @@ krb5_error_code ipadb_check_transited_realms(krb5_context kcontext,
 					     const krb5_data *tr_contents,
 					     const krb5_data *client_realm,
 					     const krb5_data *server_realm);
+/* Checks whether a principal's realm is one of trusted domains' realm or NetBIOS name
+ * and returns the realm of the matched trusted domain in 'trusted_domain'
+ * Returns 0 in case of success and KRB5_KDB_NOENTRY otherwise
+ * If DAL driver is not initialized, returns KRB5_KDB_DBNOTINITED */
+krb5_error_code ipadb_is_princ_from_trusted_realm(krb5_context kcontext,
+                                                  const char *test_realm, size_t size,
+                                                  char **trusted_realm);
+
 /* DELEGATION CHECKS */
 
 krb5_error_code ipadb_check_allowed_to_delegate(krb5_context kcontext,
@@ -293,3 +304,6 @@ void ipadb_parse_user_auth(LDAP *lcontext, LDAPMessage *le,
                            enum ipadb_user_auth *user_auth);
 const struct ipadb_global_config *
 ipadb_get_global_config(struct ipadb_context *ipactx);
+int ipadb_get_enc_salt_types(struct ipadb_context *ipactx, LDAPMessage *entry,
+                             char *attr, krb5_key_salt_tuple **enc_salt_types,
+                             int *n_enc_salt_types);
