@@ -229,6 +229,29 @@ done:
     return ret;
 }
 
+void set_err_msg(struct extdom_req *req, const char *format, ...)
+{
+    int ret;
+    va_list ap;
+
+    if (req == NULL) {
+        return;
+    }
+
+    if (format == NULL || req->err_msg != NULL) {
+        /* Do not override an existing error message. */
+        return;
+    }
+    va_start(ap, format);
+
+    ret = vasprintf(&req->err_msg, format, ap);
+    if (ret == -1) {
+        req->err_msg = strdup("vasprintf failed.\n");
+    }
+
+    va_end(ap);
+}
+
 int parse_request_data(struct berval *req_val, struct extdom_req **_req)
 {
     BerElement *ber = NULL;
@@ -277,26 +300,34 @@ int parse_request_data(struct berval *req_val, struct extdom_req **_req)
  * }
  */
 
+    req = calloc(sizeof(struct extdom_req), 1);
+    if (req == NULL) {
+        /* Since we return req even in the case of an error we make sure is is
+         * always safe to call free_req_data() on the returned data. */
+        *_req = NULL;
+        return LDAP_OPERATIONS_ERROR;
+    }
+
+    *_req = req;
+
     if (req_val == NULL || req_val->bv_val == NULL || req_val->bv_len == 0) {
+        set_err_msg(req, "Missing request data");
         return LDAP_PROTOCOL_ERROR;
     }
 
     ber = ber_init(req_val);
     if (ber == NULL) {
+        set_err_msg(req, "Cannot initialize BER struct");
         return LDAP_PROTOCOL_ERROR;
     }
 
     tag = ber_scanf(ber, "{ee", &input_type, &request_type);
     if (tag == LBER_ERROR) {
         ber_free(ber, 1);
+        set_err_msg(req, "Cannot read input and request type");
         return LDAP_PROTOCOL_ERROR;
     }
 
-    req = calloc(sizeof(struct extdom_req), 1);
-    if (req == NULL) {
-        return LDAP_OPERATIONS_ERROR;
-    }
-
     req->input_type = input_type;
     req->request_type = request_type;
 
@@ -320,17 +351,15 @@ int parse_request_data(struct berval *req_val, struct extdom_req **_req)
             break;
         default:
             ber_free(ber, 1);
-            free(req);
+            set_err_msg(req, "Unknown input type");
             return LDAP_PROTOCOL_ERROR;
     }
     ber_free(ber, 1);
     if (tag == LBER_ERROR) {
-        free(req);
+        set_err_msg(req, "Failed to decode BER data");
         return LDAP_PROTOCOL_ERROR;
     }
 
-    *_req = req;
-
     return LDAP_SUCCESS;
 }
 
@@ -356,6 +385,7 @@ void free_req_data(struct extdom_req *req)
         break;
     }
 
+    free(req->err_msg);
     free(req);
 }
 
@@ -370,8 +400,8 @@ int check_request(struct extdom_req *req, enum extdom_version version)
     return LDAP_SUCCESS;
 }
 
-static int get_user_grouplist(const char *name, gid_t gid,
-                              size_t *_ngroups, gid_t **_groups )
+int get_user_grouplist(const char *name, gid_t gid,
+                       size_t *_ngroups, gid_t **_groups)
 {
     int ret;
     int ngroups;
@@ -434,7 +464,7 @@ static int add_kv_list(BerElement *ber, struct sss_nss_kv *kv_list)
     return LDAP_SUCCESS;
 }
 
-static int pack_ber_sid(const char *sid, struct berval **berval)
+int pack_ber_sid(const char *sid, struct berval **berval)
 {
     BerElement *ber = NULL;
     int ret;
@@ -461,13 +491,13 @@ static int pack_ber_sid(const char *sid, struct berval **berval)
 
 #define SSSD_SYSDB_SID_STR "objectSIDString"
 
-static int pack_ber_user(struct ipa_extdom_ctx *ctx,
-                         enum response_types response_type,
-                         const char *domain_name, const char *user_name,
-                         uid_t uid, gid_t gid,
-                         const char *gecos, const char *homedir,
-                         const char *shell, struct sss_nss_kv *kv_list,
-                         struct berval **berval)
+int pack_ber_user(struct ipa_extdom_ctx *ctx,
+                  enum response_types response_type,
+                  const char *domain_name, const char *user_name,
+                  uid_t uid, gid_t gid,
+                  const char *gecos, const char *homedir,
+                  const char *shell, struct sss_nss_kv *kv_list,
+                  struct berval **berval)
 {
     BerElement *ber = NULL;
     int ret;
@@ -580,10 +610,10 @@ done:
     return ret;
 }
 
-static int pack_ber_group(enum response_types response_type,
-                          const char *domain_name, const char *group_name,
-                          gid_t gid, char **members, struct sss_nss_kv *kv_list,
-                          struct berval **berval)
+int pack_ber_group(enum response_types response_type,
+                   const char *domain_name, const char *group_name,
+                   gid_t gid, char **members, struct sss_nss_kv *kv_list,
+                   struct berval **berval)
 {
     BerElement *ber = NULL;
     int ret;
@@ -664,8 +694,8 @@ done:
     return ret;
 }
 
-static int pack_ber_name(const char *domain_name, const char *name,
-                         struct berval **berval)
+int pack_ber_name(const char *domain_name, const char *name,
+                  struct berval **berval)
 {
     BerElement *ber = NULL;
     int ret;
@@ -691,6 +721,7 @@ static int pack_ber_name(const char *domain_name, const char *name,
 }
 
 static int handle_uid_request(struct ipa_extdom_ctx *ctx,
+                              struct extdom_req *req,
                               enum request_types request_type, uid_t uid,
                               const char *domain_name, struct berval **berval)
 {
@@ -714,6 +745,7 @@ static int handle_uid_request(struct ipa_extdom_ctx *ctx,
             if (ret == ENOENT) {
                 ret = LDAP_NO_SUCH_OBJECT;
             } else {
+                set_err_msg(req, "Failed to lookup SID by UID");
                 ret = LDAP_OPERATIONS_ERROR;
             }
             goto done;
@@ -736,6 +768,7 @@ static int handle_uid_request(struct ipa_extdom_ctx *ctx,
             ret = sss_nss_getorigbyname(pwd.pw_name, &kv_list, &id_type);
             if (ret != 0 || !(id_type == SSS_ID_TYPE_UID
                                 || id_type == SSS_ID_TYPE_BOTH)) {
+                set_err_msg(req, "Failed to read original data");
                 if (ret == ENOENT) {
                     ret = LDAP_NO_SUCH_OBJECT;
                 } else {
@@ -761,6 +794,7 @@ done:
 }
 
 static int handle_gid_request(struct ipa_extdom_ctx *ctx,
+                              struct extdom_req *req,
                               enum request_types request_type, gid_t gid,
                               const char *domain_name, struct berval **berval)
 {
@@ -783,6 +817,7 @@ static int handle_gid_request(struct ipa_extdom_ctx *ctx,
             if (ret == ENOENT) {
                 ret = LDAP_NO_SUCH_OBJECT;
             } else {
+                set_err_msg(req, "Failed to lookup SID by GID");
                 ret = LDAP_OPERATIONS_ERROR;
             }
             goto done;
@@ -805,6 +840,7 @@ static int handle_gid_request(struct ipa_extdom_ctx *ctx,
             ret = sss_nss_getorigbyname(grp.gr_name, &kv_list, &id_type);
             if (ret != 0 || !(id_type == SSS_ID_TYPE_GID
                                 || id_type == SSS_ID_TYPE_BOTH)) {
+                set_err_msg(req, "Failed to read original data");
                 if (ret == ENOENT) {
                     ret = LDAP_NO_SUCH_OBJECT;
                 } else {
@@ -828,6 +864,7 @@ done:
 }
 
 static int handle_sid_request(struct ipa_extdom_ctx *ctx,
+                              struct extdom_req *req,
                               enum request_types request_type, const char *sid,
                               struct berval **berval)
 {
@@ -848,6 +885,7 @@ static int handle_sid_request(struct ipa_extdom_ctx *ctx,
         if (ret == ENOENT) {
             ret = LDAP_NO_SUCH_OBJECT;
         } else {
+            set_err_msg(req, "Failed to lookup name by SID");
             ret = LDAP_OPERATIONS_ERROR;
         }
         goto done;
@@ -855,6 +893,7 @@ static int handle_sid_request(struct ipa_extdom_ctx *ctx,
 
     sep = strchr(fq_name, SSSD_DOMAIN_SEPARATOR);
     if (sep == NULL) {
+        set_err_msg(req, "Failed to split fully qualified name");
         ret = LDAP_OPERATIONS_ERROR;
         goto done;
     }
@@ -862,6 +901,7 @@ static int handle_sid_request(struct ipa_extdom_ctx *ctx,
     object_name = strndup(fq_name, (sep - fq_name));
     domain_name = strdup(sep + 1);
     if (object_name == NULL || domain_name == NULL) {
+        set_err_msg(req, "Missing name or domain");
         ret = LDAP_OPERATIONS_ERROR;
         goto done;
     }
@@ -894,6 +934,7 @@ static int handle_sid_request(struct ipa_extdom_ctx *ctx,
             ret = sss_nss_getorigbyname(pwd.pw_name, &kv_list, &id_type);
             if (ret != 0 || !(id_type == SSS_ID_TYPE_UID
                                 || id_type == SSS_ID_TYPE_BOTH)) {
+                set_err_msg(req, "Failed to read original data");
                 if (ret == ENOENT) {
                     ret = LDAP_NO_SUCH_OBJECT;
                 } else {
@@ -926,6 +967,7 @@ static int handle_sid_request(struct ipa_extdom_ctx *ctx,
             ret = sss_nss_getorigbyname(grp.gr_name, &kv_list, &id_type);
             if (ret != 0 || !(id_type == SSS_ID_TYPE_GID
                                 || id_type == SSS_ID_TYPE_BOTH)) {
+                set_err_msg(req, "Failed to read original data");
                 if (ret == ENOENT) {
                     ret = LDAP_NO_SUCH_OBJECT;
                 } else {
@@ -956,6 +998,7 @@ done:
 }
 
 static int handle_name_request(struct ipa_extdom_ctx *ctx,
+                               struct extdom_req *req,
                                enum request_types request_type,
                                const char *name, const char *domain_name,
                                struct berval **berval)
@@ -974,6 +1017,7 @@ static int handle_name_request(struct ipa_extdom_ctx *ctx,
                                        domain_name);
     if (ret == -1) {
         ret = LDAP_OPERATIONS_ERROR;
+        set_err_msg(req, "Failed to create fully qualified name");
         fq_name = NULL; /* content is undefined according to
                            asprintf(3) */
         goto done;
@@ -985,6 +1029,7 @@ static int handle_name_request(struct ipa_extdom_ctx *ctx,
             if (ret == ENOENT) {
                 ret = LDAP_NO_SUCH_OBJECT;
             } else {
+                set_err_msg(req, "Failed to lookup SID by name");
                 ret = LDAP_OPERATIONS_ERROR;
             }
             goto done;
@@ -1004,6 +1049,7 @@ static int handle_name_request(struct ipa_extdom_ctx *ctx,
                 ret = sss_nss_getorigbyname(pwd.pw_name, &kv_list, &id_type);
                 if (ret != 0 || !(id_type == SSS_ID_TYPE_UID
                                     || id_type == SSS_ID_TYPE_BOTH)) {
+                    set_err_msg(req, "Failed to read original data");
                     if (ret == ENOENT) {
                         ret = LDAP_NO_SUCH_OBJECT;
                     } else {
@@ -1044,6 +1090,7 @@ static int handle_name_request(struct ipa_extdom_ctx *ctx,
                     if (ret == ENOENT) {
                         ret = LDAP_NO_SUCH_OBJECT;
                     } else {
+                        set_err_msg(req, "Failed to read original data");
                         ret = LDAP_OPERATIONS_ERROR;
                     }
                     goto done;
@@ -1073,27 +1120,29 @@ int handle_request(struct ipa_extdom_ctx *ctx, struct extdom_req *req,
 
     switch (req->input_type) {
     case INP_POSIX_UID:
-        ret = handle_uid_request(ctx, req->request_type,
+        ret = handle_uid_request(ctx, req, req->request_type,
                                  req->data.posix_uid.uid,
                                  req->data.posix_uid.domain_name, berval);
 
         break;
     case INP_POSIX_GID:
-        ret = handle_gid_request(ctx, req->request_type,
+        ret = handle_gid_request(ctx, req, req->request_type,
                                  req->data.posix_gid.gid,
                                  req->data.posix_uid.domain_name, berval);
 
         break;
     case INP_SID:
-        ret = handle_sid_request(ctx, req->request_type, req->data.sid, berval);
+        ret = handle_sid_request(ctx, req, req->request_type, req->data.sid,
+                                 berval);
         break;
     case INP_NAME:
-        ret = handle_name_request(ctx, req->request_type,
+        ret = handle_name_request(ctx, req, req->request_type,
                                   req->data.name.object_name,
                                   req->data.name.domain_name, berval);
 
         break;
     default:
+        set_err_msg(req, "Unknown input type");
         ret = LDAP_PROTOCOL_ERROR;
         goto done;
     }
