@@ -149,12 +149,15 @@ static int ipa_extdom_extop(Slapi_PBlock *pb)
     rc = LDAP_SUCCESS;
 
 done:
-    free_req_data(req);
+    if (req->err_msg != NULL) {
+        err_msg = req->err_msg;
+    }
     if (err_msg != NULL) {
         LOG("%s", err_msg);
     }
     slapi_send_ldap_result(pb, rc, NULL, err_msg, 0, NULL);
     ber_bvfree(ret_val);
+    free_req_data(req);
     return SLAPI_PLUGIN_EXTENDED_SENT_RESULT;
 }
 
@@ -197,7 +200,7 @@ static int ipa_extdom_init_ctx(Slapi_PBlock *pb, struct ipa_extdom_ctx **_ctx)
     if (ctx->max_nss_buf_size == 0) {
         ctx->max_nss_buf_size = DEFAULT_MAX_NSS_BUFFER;
     }
-    LOG("Maximal nss buffer size set to [%d]!\n", ctx->max_nss_buf_size);
+    LOG("Maximal nss buffer size set to [%zu]!\n", ctx->max_nss_buf_size);
 
     ret = 0;
 
