@@ -102,8 +102,11 @@ void ipapwd_keyset_free(struct ipapwd_keyset **pkset)
     *pkset = NULL;
 }
 
-static Slapi_Value **encrypt_encode_key(struct ipapwd_krbcfg *krbcfg,
+Slapi_Value **ipapwd_encrypt_encode_key(struct ipapwd_krbcfg *krbcfg,
                                         struct ipapwd_data *data,
+                                        char *preferred_principal,
+                                        int num_encsalts,
+                                        krb5_key_salt_tuple *encsalts,
                                         char **errMesg)
 {
     krb5_context krbctx;
@@ -113,7 +116,7 @@ static Slapi_Value **encrypt_encode_key(struct ipapwd_krbcfg *krbcfg,
     Slapi_Value **svals = NULL;
     krb5_principal princ = NULL;
     krb5_error_code krberr;
-    krb5_data pwd;
+    krb5_data pwd = { 0 };
     struct ipapwd_keyset *kset = NULL;
 
     krbctx = krbcfg->krbctx;
@@ -126,12 +129,20 @@ static Slapi_Value **encrypt_encode_key(struct ipapwd_krbcfg *krbcfg,
 
     kvno = ipapwd_get_cur_kvno(data->target);
 
-    krbPrincipalName = slapi_entry_attr_get_charptr(data->target,
-                                                    "krbPrincipalName");
-    if (!krbPrincipalName) {
-        *errMesg = "no krbPrincipalName present in this entry\n";
-        LOG_FATAL("%s", *errMesg);
-        goto enc_error;
+    if (preferred_principal) {
+        krbPrincipalName = slapi_ch_strdup(preferred_principal);
+    } else {
+        krbPrincipalName = slapi_entry_attr_get_charptr(data->target,
+                                                        "krbCanonicalName");
+        if (!krbPrincipalName) {
+            krbPrincipalName = slapi_entry_attr_get_charptr(data->target,
+                                                        "krbPrincipalName");
+        }
+        if (!krbPrincipalName) {
+            *errMesg = "no krbPrincipalName present in this entry\n";
+            LOG_FATAL("%s", *errMesg);
+            goto enc_error;
+        }
     }
 
     krberr = krb5_parse_name(krbctx, krbPrincipalName, &princ);
@@ -141,8 +152,10 @@ static Slapi_Value **encrypt_encode_key(struct ipapwd_krbcfg *krbcfg,
         goto enc_error;
     }
 
-    pwd.data = (char *)data->password;
-    pwd.length = strlen(data->password);
+    if (data->password) {
+        pwd.data = (char *)data->password;
+        pwd.length = strlen(data->password);
+    }
 
     kset = malloc(sizeof(struct ipapwd_keyset));
     if (!kset) {
@@ -160,8 +173,7 @@ static Slapi_Value **encrypt_encode_key(struct ipapwd_krbcfg *krbcfg,
 
     krberr = ipa_krb5_generate_key_data(krbctx, princ,
                                         pwd, kvno, krbcfg->kmkey,
-                                        krbcfg->num_pref_encsalts,
-                                        krbcfg->pref_encsalts,
+                                        num_encsalts, encsalts,
                                         &kset->num_keys, &kset->keys);
     if (krberr != 0) {
         LOG_FATAL("generating kerberos keys failed [%s]\n",
@@ -212,7 +224,10 @@ int ipapwd_gen_hashes(struct ipapwd_krbcfg *krbcfg,
 
     if (is_krb) {
 
-        *svals = encrypt_encode_key(krbcfg, data, errMesg);
+        *svals = ipapwd_encrypt_encode_key(krbcfg, data, NULL,
+                                           krbcfg->num_pref_encsalts,
+                                           krbcfg->pref_encsalts,
+                                           errMesg);
 
         if (!*svals) {
             /* errMesg should have been set in encrypt_encode_key() */
