@@ -661,6 +661,7 @@ static Slapi_Entry *get_entry_by_principal(const char *principal)
     Slapi_PBlock *pb = NULL;
     char *attrlist[] = { "krbPrincipalKey", "krbLastPwdChange",
                          "userPassword", "krbPrincipalName",
+                         "krbCanonicalName",
                          "enrolledBy", NULL };
     Slapi_Entry **es = NULL;
     int res, ret, i;
@@ -1612,8 +1613,8 @@ static int ipapwd_getkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
                                            READKEYS_OP_CHECK, NULL,
                                            SLAPI_ACL_READ);
         if (!acl_ok) {
-            LOG_FATAL("Not allowed to retrieve keytab on [%s]!\n",
-                      service_name);
+            LOG_FATAL("Not allowed to retrieve keytab on [%s] as user [%s]!\n",
+                      service_name, bind_dn);
             err_msg = "Insufficient access rights\n";
             rc = LDAP_INSUFFICIENT_ACCESS;
             goto free_and_return;
@@ -1648,7 +1649,7 @@ static int ipapwd_getkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
         data.target = target_entry;
         data.password = password;
 
-        svals = ipapwd_encrypt_encode_key(krbcfg, &data,
+        svals = ipapwd_encrypt_encode_key(krbcfg, &data, service_name,
                                           kenctypes ? num_kenctypes :
                                                 krbcfg->num_pref_encsalts,
                                           kenctypes ? kenctypes :
