@@ -42,6 +42,12 @@ from ipalib import Str
 from ipalib import output
 
 
+# To make the example ready for Python 3, we alias "unicode" to strings.
+import six
+if six.PY3:
+    unicode = str
+
+
 # We're going to create an example command plugin, that takes a name as its
 # only argument. Commands in IPA support input validation by defining
 # functions we're going to call 'validators'. This is an example of such
