@@ -20,6 +20,7 @@
  */
 
 define([
+    'dojo/on',
     './metadata',
     './ipa',
     './jquery',
@@ -30,9 +31,25 @@ define([
     './search',
     './association',
     './entity'],
-        function(metadata_provider, IPA, $, phases, reg, text) {
+        function(on, metadata_provider, IPA, $, phases, reg, text) {
 
-var exp = IPA.aci = {};
+/**
+ * Widgets, entities and fields related to Access Control that means
+ * Permissions, Privilege, Role, Delegation and Self-service.
+ *
+ * @class aci
+ * @singleton
+ */
+var aci = {};
+
+/**
+ * List of fields which are disabled for managed permissions
+ * @property {Array}
+ */
+aci.managed_fields = [
+    'ipapermright', 'extratargetfilter', 'memberof', 'ipapermlocation',
+    'ipapermtarget', 'type'
+];
 
 var make_permission_spec = function() {
 
@@ -45,96 +62,142 @@ return {
             columns: [ 'cn' ]
         },
         {
-            $factory: IPA.aci.permission_details_facet,
+            $factory: aci.permission_details_facet,
             $type: 'details',
             fields: [
                 {
                     name:'cn',
                     widget: 'identity.cn'
                 },
+                {
+                    $type: 'radio',
+                    name:'ipapermbindruletype',
+                    widget: 'identity.ipapermbindruletype',
+                    flags: ['w_if_no_aci']
+                },
                 {
                     $type: 'rights',
-                    name: 'permissions',
-                    widget: 'rights.permissions'
+                    name: 'ipapermright',
+                    widget: 'identity.ipapermright',
+                    required: true,
+                    flags: ['w_if_no_aci']
                 },
                 {
-                    $type: 'select',
-                    name: 'target',
-                    widget: 'target.target',
-                    enabled: false
+                    $type: 'multivalued',
+                    name: 'extratargetfilter',
+                    widget: 'target.extratargetfilter',
+                    acl_param: 'ipapermtargetfilter',
+                    enabled: false,
+                    flags: ['w_if_no_aci']
                 },
                 {
-                    name: 'filter',
-                    widget: 'target.filter',
-                    enabled: false
-                },
-                {
-                    $type: 'entity_select',
+                    $type: 'multivalued',
                     name: 'memberof',
                     widget: 'target.memberof',
-                    enabled: false
+                    enabled: false,
+                    flags: ['w_if_no_aci']
                 },
                 {
-                    name: 'subtree',
-                    widget: 'target.subtree',
-                    enabled: false
+                    name: 'ipapermlocation',
+                    widget: 'target.ipapermlocation',
+                    enabled: false,
+                    flags: ['w_if_no_aci']
                 },
                 {
-                    $type: 'entity_select',
-                    name: 'targetgroup',
-                    widget: 'target.targetgroup',
-                    enabled: false
+                    name: 'ipapermtarget',
+                    widget: 'target.ipapermtarget',
+                    enabled: false,
+                    flags: ['w_if_no_aci']
                 },
                 {
                     $type: 'select',
                     name: 'type',
                     widget: 'target.type',
-                    enabled: false
+                    enabled: false,
+                    flags: ['w_if_no_aci']
+
                 },
                 {
                     name: 'attrs',
                     widget: 'target.attrs',
-                    enabled: false
+                    enabled: false,
+                    flags: ['w_if_no_aci']
                 },
                 {
                     name: 'attrs_multi',
                     param: 'attrs',
                     $type: 'multivalued',
                     widget: 'target.attrs_multi',
-                    enabled: false
+                    enabled: false,
+                    flags: ['w_if_no_aci']
+                },
+                {
+                    name: 'ipapermdefaultattr',
+                    $type: 'multivalued',
+                    widget: 'managed.ipapermdefaultattr'
+                },
+                {
+                    name: 'ipapermincludedattr',
+                    $type: 'multivalued',
+                    widget: 'managed.ipapermincludedattr',
+                    read_only: true
+                },
+                {
+                    name: 'ipapermexcludedattr',
+                    $type: 'multivalued',
+                    widget: 'managed.ipapermexcludedattr',
+                    read_only: true
                 }
             ],
             widgets: [
                 {
-                    $type: 'details_table_section',
+                    $type: 'details_section',
                     name: 'identity',
                     label: '@i18n:objects.permission.identity',
                     widgets: [
-                        'cn'
-                    ]
-                },
-                {
-                    $type: 'details_table_section',
-                    name: 'rights',
-                    label: '@i18n:objects.permission.rights',
-                    widgets: [
+                        'cn',
+                        {
+                            $type: 'radio',
+                            name: 'ipapermbindruletype',
+                            options: ['permission', 'all', 'anonymous']
+                        },
                         {
                             $type: 'rights',
-                            name: 'permissions'
+                            name: 'ipapermright',
+                            layout: 'columns'
                         }
                     ]
                 },
                 {
                     $type: 'permission_target',
-                    container_factory: IPA.details_table_section,
+                    container_factory: IPA.details_section,
                     label: '@i18n:objects.permission.target',
-                    name: 'target',
-                    show_target: false
+                    name: 'target'
+                },
+                {
+                    $type: 'details_section',
+                    name: 'managed',
+                    label: '@i18n:objects.permission.managed',
+                    widgets: [
+                        {
+                            $type: 'multivalued',
+                            name: 'ipapermdefaultattr'
+                        },
+                        {
+                            $type: 'multivalued',
+                            name: 'ipapermincludedattr'
+                        },
+                        {
+                            $type: 'multivalued',
+                            name: 'ipapermexcludedattr'
+                        }
+                    ]
                 }
             ],
             policies: [
+                aci.permission_managed_policy,
                 {
-                    $factory: IPA.permission_target_policy,
+                    $factory: aci.permission_target_policy,
                     widget_name: 'target'
                 }
             ]
@@ -152,37 +215,38 @@ return {
                 name:'cn',
                 widget: 'general.cn'
             },
+            {
+                $type: 'radio',
+                name:'ipapermbindruletype',
+                widget: 'general.ipapermbindruletype'
+            },
             {
                 $type: 'rights',
-                name: 'permissions',
-                widget: 'general.permissions'
+                name: 'ipapermright',
+                widget: 'general.ipapermright',
+                required: true
             },
             {
-                $type: 'select',
-                name: 'target',
-                widget: 'target.target',
+                $type: 'multivalued',
+                name: 'extratargetfilter',
+                widget: 'target.extratargetfilter',
+                acl_param: 'ipapermtargetfilter',
                 enabled: false
             },
             {
-                name: 'filter',
-                widget: 'target.filter',
-                enabled: false
-            },
-            {
-                $type: 'entity_select',
+                $type: 'multivalued',
                 name: 'memberof',
                 widget: 'target.memberof',
                 enabled: false
             },
             {
-                name: 'subtree',
-                widget: 'target.subtree',
+                name: 'ipapermlocation',
+                widget: 'target.ipapermlocation',
                 enabled: false
             },
             {
-                $type: 'entity_select',
-                name: 'targetgroup',
-                widget: 'target.targetgroup',
+                name: 'ipapermtarget',
+                widget: 'target.ipapermtarget',
                 enabled: false
             },
             {
@@ -206,32 +270,42 @@ return {
         ],
         widgets: [
             {
-                $type: 'details_table_section_nc',
+                $type: 'details_section',
                 name: 'general',
                 widgets: [
                     'cn',
+                    {
+                        $type: 'radio',
+                        name: 'ipapermbindruletype',
+                        options: ['permission', 'all', 'anonymous'],
+                        default_value: 'permission'
+                    },
                     {
                         $type: 'rights',
-                        name: 'permissions'
+                        name: 'ipapermright',
+                        layout: 'columns'
                     }
                 ]
             },
             {
                 $type: 'permission_target',
-                name:'target',
-                show_target: true
+                name:'target'
             }
         ],
         policies: [
             {
-                $factory: IPA.permission_target_policy,
+                $factory: aci.permission_target_policy,
                 widget_name: 'target'
             }
         ]
     }
 };};
 
-IPA.aci.permission_details_facet = function(spec) {
+/**
+ * @class aci.permission_details_facet
+ * @extends details.details_facet
+ */
+aci.permission_details_facet = function(spec) {
 
     var that = IPA.details_facet(spec);
 
@@ -463,13 +537,24 @@ return {
     }
 };};
 
-
-IPA.attributes_widget = function(spec) {
+/**
+ * @class aci.attributes_widget
+ * @extends IPA.checkboxes_widget
+ */
+aci.attributes_widget = function(spec) {
 
     spec = spec || {};
+    spec.layout = spec.layout || 'columns attribute_widget';
+    spec.sort = spec.sort === undefined ? true : spec.sort;
 
     var that = IPA.checkboxes_widget(spec);
 
+    /**
+     * Additional options which are not defined in metadata
+     * @property {string[]}
+     */
+    that.custom_options = spec.custom_options || [];
+
     that.object_type = spec.object_type;
     that.skip_unmatched = spec.skip_unmatched === undefined ? false : spec.skip_unmatched;
 
@@ -477,74 +562,93 @@ IPA.attributes_widget = function(spec) {
 
     that.create = function(container) {
         that.container = container;
+        that.widget_create(container);
 
-        var attr_container = $('<div/>', {
-            'class': 'aci-attribute-table-container'
-        }).appendTo(container);
-
-        that.$node = that.table = $('<table/>', {
-            id: id,
-            name: that.name,
-            'class': 'search-table aci-attribute-table scrollable'
-        }).
-            append('<thead/>').
-            append('<tbody/>').
-            appendTo(attr_container);
-
-        var tr = $('<tr></tr>').appendTo($('thead', that.table));
-
-        tr.append($('<th/>', {
-            html: $('<input/>', {
-                type: "checkbox",
-                click: function() {
-                    $('.aci-attribute', that.table).
-                        prop('checked', $(this).prop('checked'));
-                    that.value_changed.notify([], that);
-                }
-            })
-        })).append($('<th/>', {
-            'class': 'aci-attribute-column',
-            html: text.get('@i18n:objects.aci.attribute')
-        }));
-
+        that.controls = $('<div/>', {
+            'class': 'form-inline controls'
+        });
+        that.controls.appendTo(container);
+        that.create_search_filter(that.controls);
+        that.create_add_control(that.controls);
         if (that.undo) {
-            that.create_undo(container);
+            that.create_undo(that.controls);
         }
 
-        if (that.object_type) {
-            that.populate(that.object_type);
-        }
+        that.owb_create(container);
 
         that.create_error_link(container);
     };
 
-    that.create_options = function(options) {
-        var tbody = $('tbody', that.table);
+    that.create_search_filter = function(container) {
+        var filter_container = $('<div/>', {
+            'class': 'search-filter'
+        });
 
-        for (var i=0; i<options.length ; i++){
-            var value = options[i].toLowerCase();
-            var tr = $('<tr/>').appendTo(tbody);
+        that.filter = $('<input/>', {
+            type: 'text',
+            name: 'filter',
+            'class': 'form-control',
+            placeholder: text.get('@i18n:objects.permission.filter')
+        }).appendTo(filter_container);
 
-            var td =  $('<td/>').appendTo(tr);
-            var name = that.get_input_name();
-            var id = that._option_next_id + name;
-            td.append($('<input/>',{
-                id: id,
-                type: 'checkbox',
-                name: name,
-                value: value,
-                'class': 'aci-attribute',
-                change: function() {
-                    that.value_changed.notify([], that);
+        that.filter.keyup(function(e) {
+            that.filter_options();
+        });
+
+        var find_button = IPA.action_button({
+            name: 'find',
+            icon: 'fa-search',
+            click: function() {
+                that.filter_options();
+                return false;
+            }
+        }).appendTo(filter_container);
+
+        filter_container.appendTo(container);
+    };
+
+    that.create_add_control = function(container) {
+
+        that.add_button = IPA.button({
+            label: '@i18n:buttons.add',
+            click: that.show_add_dialog
+        });
+        container.append(' ');
+        that.add_button.appendTo(container);
+    };
+
+    that.show_add_dialog = function() {
+
+        var dialog = IPA.form_dialog({
+            name: "add_option",
+            title: "@i18n:objects.permission.add_custom_attr",
+            fields: [
+                {
+                    name: 'attr',
+                    label: '@i18n:objects.permission.attribute',
+                    required: true
                 }
-            }));
-            td = $('<td/>').appendTo(tr);
-            td.append($('<label/>',{
-                text: value,
-                'for': id
-            }));
-            that.new_option_id();
-        }
+            ]
+        });
+        dialog.on_confirm = function() {
+            if (!dialog.validate()) return;
+            var attr = dialog.get_field('attr');
+            var value = attr.get_value()[0];
+            that.add_custom_option(value, false, true, true);
+            dialog.close();
+        };
+        dialog.open();
+    };
+
+    that.filter_options = function() {
+        $("li", that.$node).each(function() {
+            var item = $(this);
+            if(item.find('input').val().indexOf(that.filter.val()) === -1) {
+                item.css('display','none');
+            } else {
+                item.css('display','inline');
+            }
+        });
     };
 
     that.update = function(values) {
@@ -564,13 +668,12 @@ IPA.attributes_widget = function(spec) {
 
         that.populate(that.object_type);
         that.append();
+        that.owb_create(that.container);
         that.owb_update(values);
     };
 
     that.populate = function(object_type) {
 
-        $('tbody tr', that.table).remove();
-
         if (!object_type || object_type === '') return;
 
         var metadata = metadata_provider.get('@mo:'+object_type);
@@ -579,41 +682,55 @@ IPA.attributes_widget = function(spec) {
         var aciattrs = metadata.aciattrs;
 
         that.options = that.prepare_options(aciattrs);
-        that.create_options(aciattrs);
     };
 
     that.append = function() {
 
-        if (!that.values) return;
-
         var unmatched = [];
 
-        for (var i=0; i<that.values.length; i++) {
-            var input = $('input[name="'+that.name+'"]'+
-                          '[value="'+that.values[i]+'"]', that.container);
-            if (!input.length) {
-                unmatched.push(that.values[i]);
+        function add_unmatched(source) {
+            for (var i=0, l=source.length; i<l; i++) {
+                if (!that.has_option(source[i])) {
+                    that.add_option(source[i], true /* suppress update */);
+                }
             }
         }
 
-        if (unmatched.length > 0 && !that.skip_unmatched) {
-            that.options.push.apply(that.options, that.prepare_options(unmatched));
-            that.create_options(unmatched);
+        add_unmatched(that.custom_options);
+
+        if (that.values && !that.skip_unmatched) {
+            add_unmatched(that.values);
         }
     };
 
-    that.show_undo = function() {
-        $(that.undo_span).css('display', 'inline-block');
+    that.add_custom_option = function(name, to_custom, check, update) {
+
+        var value = (name || '').toLowerCase();
+        if (to_custom) that.custom_options.push(value);
+        if (check) that.values.push(value);
+        if (update) that.update(that.values);
+    };
+
+    that.has_option = function(value) {
+        var o = that.options;
+        for (var i=0, l=o.length; i<l; i++) {
+            if (o[i].value === value) return true;
+        }
+        return false;
     };
 
     return that;
 };
 
-IPA.rights_widget = function(spec) {
+/**
+ * @class aci.rights_widget
+ * @extends IPA.checkboxes_widget
+ */
+aci.rights_widget = function(spec) {
 
     var that = IPA.checkboxes_widget(spec);
 
-    that.rights = ['write', 'add', 'delete'];
+    that.rights = ['read', 'search', 'compare', 'write', 'add', 'delete', 'all'];
     for (var i=0; i<that.rights.length; i++) {
         var right = that.rights[i];
         that.add_option({label: right, value: right});
@@ -622,113 +739,100 @@ IPA.rights_widget = function(spec) {
     return that;
 };
 
-IPA.permission_target_widget = function(spec) {
+
+/**
+ * Default target to display in `permission_target_widget`
+ * @property {string}
+ */
+aci.default_target = 'general';
+
+/**
+ * @class aci.permission_target_widget
+ * @extends IPA.details_section
+ */
+aci.permission_target_widget = function(spec) {
 
     spec = spec || {};
 
-    var factory = spec.container_factory || IPA.details_table_section_nc;
+    var factory = spec.container_factory || IPA.details_section;
 
     var that = factory(spec);
 
     that.group_entity = IPA.get_entity(spec.group_entity || 'group');
 
-    that.targets = [ 'filter', 'subtree', 'targetgroup', 'type' ];
-    that.target = that.targets[0];
-    that.show_target = spec.show_target;
+    that.target = aci.default_target;
 
     var init = function() {
 
-        that.target_select = IPA.select_widget({
-            entity: that.entity,
-            name: 'target',
-            label: '@i18n:objects.permission.target',
-            hidden: !that.show_target
-        });
-
-        for (var i=0; i<that.targets.length; i++) {
-            var target = that.targets[i];
-            var target_param = IPA.get_entity_param('permission', target);
-
-            that.target_select.options.push({
-                label: target_param.label,
-                value: target
-            });
+        var objects = metadata_provider.get('@m:objects');
+        var types = IPA.create_options(['']);
+        for (var o in objects) {
+            if (objects.hasOwnProperty(o)) {
+                var obj = objects[o];
+                if (obj.can_have_permissions) {
+                    types.push({
+                        label: obj.label_singular,
+                        value: o
+                    });
+                }
+            }
         }
 
-        that.widgets.add_widget(that.target_select);
-
-
-        that.memberof_select = IPA.entity_select_widget({
-            entity: that.entity,
-            name: 'memberof',
-            other_entity: that.group_entity,
-            other_field: 'cn',
-            hidden: true
-        });
-
-        that.widgets.add_widget(that.memberof_select);
-
-        that.filter_text = IPA.text_widget({
-            entity: that.entity,
-            name: 'filter',
-            hidden: true
-        });
-
-        that.widgets.add_widget(that.filter_text);
-
-        that.subtree_textarea = IPA.textarea_widget({
-            entity: that.entity,
-            name: 'subtree',
-            hidden: true
-        });
-
-        that.widgets.add_widget(that.subtree_textarea);
-
-        that.group_select = IPA.entity_select_widget({
-            entity: that.entity,
-            name: 'targetgroup',
-            other_entity: that.group_entity,
-            other_field: 'cn',
-            hidden: true
-        });
-
-        that.widgets.add_widget(that.group_select);
-
         that.type_select = IPA.select_widget({
             entity: that.entity,
             name: 'type',
-            hidden: true
+            visible: false,
+            options: types
         });
-
-        var type_param = IPA.get_entity_param('permission', 'type');
-
-        for (var j=0; j<type_param.values.length; j++) {
-            var type_name = type_param.values[j];
-            var type_label = metadata_provider.get('@mo:'+type_name+'.label_singular');
-
-            that.type_select.options.push({
-                label: type_label,
-                value: type_name
-            });
-        }
-
         that.widgets.add_widget(that.type_select);
 
-        that.attribute_table = IPA.attributes_widget({
+        that.ipapermlocation_text = IPA.text_widget({
+            entity: that.entity,
+            name: 'ipapermlocation',
+            visible: false
+        });
+        that.widgets.add_widget(that.ipapermlocation_text);
+
+        that.extratargetfilter_text = IPA.multivalued_widget({
+            entity: that.entity,
+            name: 'extratargetfilter',
+            visible: false
+        });
+        that.widgets.add_widget(that.extratargetfilter_text);
+
+        that.ipapermtarget_text = IPA.text_widget({
+            entity: that.entity,
+            name: 'ipapermtarget',
+            visible: false
+        });
+        that.widgets.add_widget(that.ipapermtarget_text);
+
+        that.memberof_select = IPA.multivalued_widget({
+            name: 'memberof',
+            entity: that.entity,
+            hidden: true,
+            child_spec: {
+                $type: 'entity_select',
+                other_entity: that.group_entity,
+                other_field: 'cn'
+            }
+        });
+        that.widgets.add_widget(that.memberof_select);
+
+
+        that.attribute_table = aci.attributes_widget({
             entity: that.entity,
             name: 'attrs',
-            object_type: type_param.values[0],
-            hidden: true
+            object_type: types[0].name,
+            visible: false
         });
-
         that.widgets.add_widget(that.attribute_table);
 
         that.attribute_multivalued = IPA.multivalued_widget({
             entity: that.entity,
             name: 'attrs_multi',
-            hidden: true
+            visible: false
         });
-
         that.widgets.add_widget(that.attribute_multivalued);
     };
 
@@ -737,33 +841,57 @@ IPA.permission_target_widget = function(spec) {
     return that;
 };
 
-IPA.permission_target_policy = function (spec) {
+/**
+ * Permission target policy
+ * @class aci.permission_target_policy
+ * @extends IPA.facet_policy
+ */
+aci.permission_target_policy = function (spec) {
 
     var that = IPA.facet_policy();
     that.widget_name = spec.widget_name;
+    that.managed = false;
 
     that.init = function() {
 
         that.permission_target = that.container.widgets.get_widget(that.widget_name);
+        var type_f = that.container.fields.get_field('type');
+
+        on(type_f, 'value-change', that.apply_type);
+    };
+
+    that.apply_type = function () {
+
         var widgets = that.permission_target.widgets;
-
-        var target_select = widgets.get_widget('target');
-        target_select.value_changed.attach(function() {
-            var target = target_select.save()[0];
-            that.select_target(target);
-        });
-
         var type_select = widgets.get_widget('type');
+        var type = type_select.save()[0];
+        var new_target = type === '' ? 'general' : 'type';
+        if (that.permission_target.target !== new_target) {
 
-        type_select.value_changed.attach(function() {
-            var type = type_select.save()[0];
-            that.set_attrs_type(type, true);
-        });
+            var attr_table = widgets.get_widget('attrs');
+            var attr_multi = widgets.get_widget('attrs_multi');
+            var loc_w = widgets.get_widget('ipapermlocation');
+            var loc_f = that.container.fields.get_field('ipapermlocation');
+            var attrs;
+            that.select_target(new_target);
 
-        type_select.undo_clicked.attach(function() {
-            var type = type_select.save()[0];
-            that.set_attrs_type(type, true);
-        });
+            if (new_target === 'general') {
+                attrs = attr_table.save();
+                attr_multi.update(attrs);
+                attr_multi.value_changed.notify([], attr_multi);
+
+                // permission plugin resets ipapermlocation to basedn when
+                // type is unset. -> use it as pristine value so undo will
+                // work correctly.
+                loc_f.set_value([IPA.env.basedn], true);
+            } else {
+                attrs = attr_multi.save();
+                attr_table.update(attrs);
+                // notification will be done by `set_attrs_type`
+            }
+        }
+
+        that.set_attrs_type(type, true);
     };
 
     that.set_attrs_type = function(type, skip_unmatched) {
@@ -771,6 +899,16 @@ IPA.permission_target_policy = function (spec) {
         var attribute_table = that.permission_target.widgets.get_widget('attrs');
         var skip_unmatched_org = attribute_table.skip_unmatched;
         attribute_table.object_type = type;
+
+        // UI doesn't always know what are the possible attributes.
+        // In case of managed permissions, one of the possible lists is in ipapermdefaultattr.
+        var default_attrs = that.container.fields.get_field('ipapermdefaultattr');
+        if (default_attrs && default_attrs.enabled) { // if managed permission
+            attribute_table.custom_options = default_attrs.get_value();
+        } else {
+            attribute_table.custom_options = [];
+        }
+
         // skip values which don't belong to new type. Bug #2617
         attribute_table.skip_unmatched =  skip_unmatched || skip_unmatched_org;
         attribute_field.reset();
@@ -787,16 +925,19 @@ IPA.permission_target_policy = function (spec) {
     };
 
     that.post_create = function() {
-        that.select_target(that.permission_target.targets[0]);
+        that.select_target(aci.default_target);
     };
 
     that.post_load = function(data) {
 
-        var displayed_target;
+        var displayed_target = 'general';
+        var permtype = data.result.result.ipapermissiontype;
+        that.managed = permtype && permtype.indexOf("MANAGED") > -1;
+        that.system = permtype && permtype.indexOf("SYSTEM") > -1 && permtype.length === 1;
 
         for (var target in that.target_mapping) {
-
-            if (data.result.result[target]) {
+            var property = that.target_mapping[target].property;
+            if (property && data.result.result[property]) {
                 displayed_target = target;
             } else {
                 that.set_target_visible(target, false);
@@ -818,42 +959,40 @@ IPA.permission_target_policy = function (spec) {
     that.set_target_visible = function(target, visible) {
 
         var target_info = that.target_mapping[target];
-        that.set_target_visible_core(target_info, visible);
-    };
 
-    that.set_target_visible_core = function(target_info, visible) {
-        var widget = that.permission_target.widgets.get_widget(target_info.name);
-        var field = that.container.fields.get_field(target_info.name);
-        that.permission_target.set_row_visible(target_info.name, visible);
-        field.enabled = visible;
-        field.set_required(visible && target_info.required);
-        widget.hidden = !visible;
-
-        if (target_info.additional) {
-            for (var i=0; i<target_info.additional.length; i++) {
-                var nested_info = target_info.additional[i];
-                that.set_target_visible_core(nested_info, visible);
-            }
+        for (var i=0,l=target_info.fields.length; i<l; i++) {
+            var info = target_info.fields[i];
+            that.set_target_row_visible(info, visible);
         }
 
         if (visible && target_info.action) target_info.action();
     };
 
+    that.set_target_row_visible = function(target_info, visible) {
+        var widget = that.permission_target.widgets.get_widget(target_info.name);
+        var field = that.container.fields.get_field(target_info.name);
+        that.permission_target.set_row_visible(target_info.name, visible);
+        field.set_enabled(visible);
+        field.set_required(visible && target_info.required);
+        widget.set_visible(visible);
+    };
 
     that.target_mapping = {
-        filter: {
-            name: 'filter',
-            required: true,
-            additional: [
+        general: {
+            fields: [
                 {
-                    name: 'attrs_multi'
-                }
-            ]
-        },
-        subtree: {
-            name: 'subtree',
-            required: true,
-            additional: [
+                    name: 'extratargetfilter'
+                },
+                {
+                    name: 'ipapermlocation',
+                    required: true
+                },
+                {
+                    name: 'ipapermtarget'
+                },
+                {
+                    name: 'type'
+                },
                 {
                     name: 'memberof'
                 },
@@ -862,26 +1001,23 @@ IPA.permission_target_policy = function (spec) {
                 }
             ]
         },
-        targetgroup: {
-            name: 'targetgroup',
-            required: true,
-            additional: [
-                {
-                    name: 'attrs'
-                }
-            ],
-            action: function() {
-                that.set_attrs_type('group', false);
-            }
-        },
         type: {
-            name: 'type',
-            additional: [
+            property: 'type',
+            fields: [
+                {
+                    name: 'extratargetfilter'
+                },
                 {
                     name: 'memberof'
                 },
+                {
+                    name: 'type'
+                },
                 {
                     name: 'attrs'
+                },
+                {
+                    name: 'ipapermtarget'
                 }
             ],
             action: function() {
@@ -894,31 +1030,97 @@ IPA.permission_target_policy = function (spec) {
     return that;
 };
 
-exp.permission_entity_spec = make_permission_spec();
-exp.privilege_entity_spec = make_privilege_spec();
-exp.role_entity_spec = make_role_spec();
-exp.selfservice_entity_spec = make_selfservice_spec();
-exp.delegation_entity_spec = make_delegation_spec();
+/**
+ * Facet policy which shows and hides managed section based on presence
+ * "MANAGED" in ippapermissiontype attribute
+ * @class aci.permission_managed_policy
+ * @extends IPA.facet_policy
+ */
+aci.permission_managed_policy = function (spec) {
 
-exp.register = function() {
+    var that = IPA.facet_policy();
+
+    that.post_load = function(data) {
+
+        var result = data.result.result;
+        var permtype = result.ipapermissiontype;
+        var managed = permtype && permtype.indexOf("MANAGED") > -1;
+        var system = permtype && permtype.indexOf("SYSTEM") > -1 && permtype.length === 1;
+        var m_section = that.container.widgets.get_widget("managed");
+        m_section.set_visible(managed);
+
+        var fields = that.container.fields.get_fields();
+        for (var i=0, l=fields.length; i<l; i++) {
+            var field = fields[i];
+            if (field.read_only) continue;
+            var managed_f = aci.managed_fields.indexOf(field.name) > -1;
+            field.set_writable(!system && !(managed_f && managed) && field.writable);
+        }
+
+        // Bind rule type cannot be changed if permission is in a privilege
+        var privileges = result.member_privilege;
+        if (privileges && privileges.length > 0) {
+            var f = that.container.fields.get_field('ipapermbindruletype');
+            f.set_writable(false);
+        }
+    };
+
+    return that;
+};
+
+/**
+ * Permission entity spec
+ * @member aci
+ */
+aci.permission_entity_spec = make_permission_spec();
+
+/**
+ * Privilege entity spec
+ * @member aci
+ */
+aci.privilege_entity_spec = make_privilege_spec();
+
+/**
+ * Role entity spec
+ * @member aci
+ */
+aci.role_entity_spec = make_role_spec();
+
+/**
+ * Self-service entity spec
+ * @member aci
+ */
+aci.selfservice_entity_spec = make_selfservice_spec();
+
+/**
+ * Delegation entity spec
+ * @member aci
+ */
+aci.delegation_entity_spec = make_delegation_spec();
+
+/**
+ * Register entities, widgets and fields to global registers.
+ * @member aci
+ */
+aci.register = function() {
     var e = reg.entity;
     var w = reg.widget;
     var f = reg.field;
 
-    e.register({ type: 'permission', spec: exp.permission_entity_spec });
-    e.register({ type: 'privilege', spec: exp.privilege_entity_spec });
-    e.register({ type: 'role', spec: exp.role_entity_spec });
-    e.register({ type: 'selfservice', spec: exp.selfservice_entity_spec });
-    e.register({ type: 'delegation', spec: exp.delegation_entity_spec });
+    e.register({ type: 'permission', spec: aci.permission_entity_spec });
+    e.register({ type: 'privilege', spec: aci.privilege_entity_spec });
+    e.register({ type: 'role', spec: aci.role_entity_spec });
+    e.register({ type: 'selfservice', spec: aci.selfservice_entity_spec });
+    e.register({ type: 'delegation', spec: aci.delegation_entity_spec });
 
-    w.register('attributes', IPA.attributes_widget);
-    f.register('attributes', IPA.checkboxes_field);
-    w.register('rights', IPA.rights_widget);
-    f.register('rights', IPA.checkboxes_field);
-    w.register('permission_target', IPA.permission_target_widget);
+    w.register('attributes', aci.attributes_widget);
+    f.register('attributes', IPA.field);
+    w.register('rights', aci.rights_widget);
+    f.register('rights', IPA.field);
+    w.register('permission_target', aci.permission_target_widget);
 };
 
-phases.on('registration', exp.register);
+phases.on('registration', aci.register);
 
-return exp;
-});
\ No newline at end of file
+return aci;
+});
