@@ -198,7 +198,7 @@ IPA.entity_adder_dialog = function(spec) {
             var field = fields[j];
 
             var values = record[field.param];
-            if (!values || values.length === 0) continue;
+            if (!values || values.length === 0 || !field.enabled) continue;
             if (field.flags.indexOf('no_command') > -1) continue;
 
             if (field.param === pkey_name) {
