@@ -24,10 +24,13 @@ define([
     './plugins/sync_otp',
     './plugins/login',
     './plugins/login_process',
+    './plugins/api_browser',
     // entities
     './aci',
     './automember',
     './automount',
+    './plugins/caacl',
+    './plugins/certprofile',
     './dns',
     './group',
     './hbac',
@@ -35,6 +38,7 @@ define([
     './hostgroup',
     './host',
     './idrange',
+    './idviews',
     './netgroup',
     './otptoken',
     './policy',
@@ -46,8 +50,10 @@ define([
     './service',
     './sudo',
     './trust',
+    './topology',
     './user',
+    './stageuser',
     'dojo/domReady!'
 ],function(app_container) {
     return app_container;
-});
\ No newline at end of file
+});
