@@ -28,6 +28,7 @@ define([
     './metadata',
     './ipa',
     './jquery',
+    './metadata',
     './navigation',
     './phases',
     './reg',
@@ -37,8 +38,8 @@ define([
     './facet',
     './search',
     './dialog'],
-        function(lang, Deferred, metadata_provider, IPA, $, navigation,
-                 phases, reg, rpc, su, text) {
+        function(lang, Deferred, metadata_provider, IPA, $, metadata,
+                 navigation, phases, reg, rpc, su, text) {
 
 /**
  * Association module
@@ -133,6 +134,7 @@ IPA.bulk_associator = function(spec) {
     spec = spec || {};
 
     var that = IPA.associator(spec);
+    that.options = spec.options || { 'all': true };
 
     that.execute = function() {
 
@@ -145,7 +147,7 @@ IPA.bulk_associator = function(spec) {
             entity: that.entity.name,
             method: that.method,
             args: [that.pkey],
-            options: { 'all': true },
+            options: that.options,
             on_success: that.on_success,
             on_error: that.on_error
         });
@@ -284,7 +286,7 @@ IPA.association_adder_dialog = function(spec) {
             }
         }
 
-        var options = { all: true };
+        var options = {};
         if (that.search_options) {
             lang.mixin(options, that.search_options);
         }
@@ -314,13 +316,19 @@ IPA.association_adder_dialog = function(spec) {
             }
         }
 
-        rpc.command({
+        var cmd = rpc.command({
             entity: that.other_entity.name,
             method: 'find',
             args: [that.get_filter()],
             options: options,
             on_success: on_success
-        }).execute();
+        });
+
+        var no_members = metadata.get('@mc-opt:' + cmd.get_command() + ':no_members');
+        if (no_members) {
+            cmd.set_option('no_members', true);
+        }
+        cmd.execute();
     };
 
     that.normalize_values = function(values) {
@@ -405,7 +413,7 @@ IPA.association_table_widget = function (spec) {
 
     spec = spec || {};
 
-    var index = spec.name.indexOf('_');
+    var index = spec.name.lastIndexOf('_');
     spec.attribute_member = spec.attribute_member || spec.name.substring(0, index);
     spec.other_entity = spec.other_entity || spec.name.substring(index+1);
 
@@ -588,7 +596,7 @@ IPA.association_table_widget = function (spec) {
         var i;
         var columns = that.columns.values;
         if (columns.length == 1) { // show pkey only
-            var name = columns[0].name;
+            var name = columns[0].param;
             for (i=0; i<that.values.length; i++) {
                 var record = {};
                 record[name] = that.values[i];
@@ -773,6 +781,12 @@ IPA.association_table_field = function (spec) {
 
     var that = IPA.field(spec);
 
+    that.load = function(data) {
+        that.values = that.adapter.load(data);
+        that.widget.update(that.values);
+        that.widget.unselect_all();
+    };
+
     that.refresh = function() {
 
         function on_success(data, text_status, xhr) {
@@ -816,16 +830,25 @@ exp.association_facet_pre_op = function(spec, context) {
         return has_indirect;
     };
 
-    var entity = context.entity;
     su.context_entity(spec, context);
-    spec.entity = entity;
+    var entity = reg.entity.get(spec.entity);
 
-    var index = spec.name.indexOf('_');
+    var index = spec.name.lastIndexOf('_');
     spec.attribute_member = spec.attribute_member ||
         spec.name.substring(0, index);
     spec.other_entity = spec.other_entity ||
         spec.name.substring(index+1);
 
+    if (!spec.associator) {
+        // batch associator (default) calls entity_command, serial associator
+        // calls other_entity_command --> if entity doesn't support the command,
+        // switch associators to try the other_entity
+        var add_command = spec.add_method || 'add_member';
+        if (!metadata_provider.get('@mc:'+entity.name+'_'+add_command)) {
+            spec.associator = IPA.serial_associator;
+        }
+    }
+
     spec.add_title = '@i18n:association.add.'+spec.attribute_member;
     spec.remove_title = '@i18n:association.remove.'+spec.attribute_member;
 
@@ -868,6 +891,7 @@ exp.association_facet_pre_op = function(spec, context) {
             hide_cond: ['read-only'],
             show_cond: ['direct'],
             enable_cond: ['item-selected'],
+            enabled: false,
             handler: function(facet) {
                 facet.show_remove_dialog();
             }
@@ -1266,6 +1290,7 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
         that.init_facet();
         that.init_table_columns();
         init();
+        that.policies.init();
     };
 
     if (!no_init) that.init_association_facet();
@@ -1280,8 +1305,8 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
  */
 exp.attribute_facet_pre_op = function(spec, context) {
 
-    var entity = context.entity;
     su.context_entity(spec, context);
+    var entity = reg.entity.get(spec.entity);
 
     spec.title = spec.title || entity.metadata.label_singular;
     spec.label = spec.label || entity.metadata.label_singular;
@@ -1396,6 +1421,15 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
 
     that.refresh = function() {
 
+        var command = that.get_refresh_command();
+        command.execute();
+    };
+
+    /**
+     * Create refresh command
+     */
+    that.get_refresh_command = function() {
+
         var pkey = that.get_pkeys();
 
         var command = rpc.command({
@@ -1420,8 +1454,7 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
             that.redirect_error(error_thrown);
             that.report_error(error_thrown);
         };
-
-        command.execute();
+        return command;
     };
 
     that.clear = function() {
@@ -1510,10 +1543,13 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
         that.init_facet();
         that.init_table_columns();
         that.init_table(that.entity);
+        that.policies.init();
     };
 
     if (!no_init) that.init_attribute_facet();
 
+    that.attribute_get_refresh_command = that.get_refresh_command;
+
     return that;
 };
 
