@@ -34,7 +34,8 @@ define([
         './widget',
         './facet',
         './add'],
-    function(lang, builder, IPA, $, phases, reg, rpc, su, text, widget_mod) {
+    function(lang, builder, IPA, $, phases, reg, rpc, su, text, widget_mod,
+             facet_mod) {
 
 /**
  * Details module
@@ -453,16 +454,16 @@ exp.facet_policies = IPA.facet_policies = function(spec) {
  * - sets name, title, label if not present
  * - adds default actions and related buttons
  *   - refresh
- *   - reset
- *   - update
+ *   - revert
+ *   - save
  * - adds dirty state evaluator
  *
  * @member details
  */
 exp.details_facet_pre_op = function(spec, context) {
 
-    var entity = context.entity;
     su.context_entity(spec, context);
+    var entity = reg.entity.get(spec.entity);
 
     spec.name = spec.name || 'details';
     spec.title = spec.title || entity.metadata.label_singular;
@@ -472,21 +473,21 @@ exp.details_facet_pre_op = function(spec, context) {
     spec.actions = spec.actions || [];
     spec.actions.unshift(
         'refresh',
-        'reset',
-        'update');
+        'revert',
+        'save');
 
     spec.control_buttons = spec.control_buttons || [];
 
     if (!spec.no_update) {
         spec.control_buttons.unshift(
             {
-                name: 'reset',
-                label: '@i18n:buttons.reset',
+                name: 'revert',
+                label: '@i18n:buttons.revert',
                 icon: 'fa-undo'
             },
             {
-                name: 'update',
-                label: '@i18n:buttons.update',
+                name: 'save',
+                label: '@i18n:buttons.save',
                 icon: 'fa-upload'
             });
     }
@@ -521,6 +522,15 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.entity = IPA.get_entity(spec.entity);
 
+
+    /**
+     * Name of refresh RPC command
+     *
+     * - defaults to 'show'
+     * @property {string}
+     */
+    that.refresh_command_name = spec.refresh_command_name || 'show';
+
     /**
      * Name of update command
      *
@@ -550,6 +560,15 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.check_rights = spec.check_rights !== undefined ? spec.check_rights : true;
 
+    /**
+     * Get all fields
+     *
+     * Controls obtaining of all attributes on refresh and update
+     *
+     * @property {boolean}
+     */
+    that.get_all_attrs = spec.get_all_attrs !== undefined ? spec.get_all_attrs: true;
+
     /**
      * Facet label
      * @property {string}
@@ -574,15 +593,6 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.fields = IPA.field_container({ container: that });
 
-    /**
-     * Policies
-     * @property {IPA.facet_policies}
-     */
-    that.policies = IPA.facet_policies({
-        container: that,
-        policies: spec.policies
-    });
-
     that.fields.add_field = function(field) {
 
         if (field.dirty_changed) {
@@ -634,7 +644,6 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
         }
 
         that.facet_create(container);
-        that.policies.post_create();
     };
 
     /**
@@ -877,7 +886,8 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
 
         var args = that.get_pkeys();
 
-        var options = { all: true };
+        var options = {};
+        if (that.get_all_attrs) options.all = true;
         if (that.check_rights) options.rights = true;
 
         var command = rpc.command({
@@ -991,7 +1001,7 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      * @return {string}
      */
     that.get_refresh_command_name = function() {
-        return that.entity.name+'_show';
+        return that.entity.name+'_'+that.refresh_command_name;
     };
 
     /**
@@ -1001,13 +1011,14 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.create_refresh_command = function() {
 
-        var options = { all: true };
+        var options = {};
+        if (that.get_all_attrs) options.all = true;
         if (that.check_rights) options.rights = true;
 
         var command = rpc.command({
             name: that.get_refresh_command_name(),
             entity: that.entity.name,
-            method: 'show',
+            method: that.refresh_command_name,
             options: options
         });
 
@@ -1404,8 +1415,8 @@ exp.refresh_action = IPA.refresh_action = function(spec) {
 exp.reset_action = IPA.reset_action = function(spec) {
 
     spec = spec || {};
-    spec.name = spec.name || 'reset';
-    spec.label = spec.label || '@i18n:buttons.reset';
+    spec.name = spec.name || 'revert';
+    spec.label = spec.label || '@i18n:buttons.revert';
     spec.enable_cond = spec.enable_cond || ['dirty'];
 
     var that = IPA.action(spec);
@@ -1426,8 +1437,8 @@ exp.reset_action = IPA.reset_action = function(spec) {
 exp.update_action = IPA.update_action = function(spec) {
 
     spec = spec || {};
-    spec.name = spec.name || 'update';
-    spec.label = spec.label || '@i18n:buttons.update';
+    spec.name = spec.name || 'save';
+    spec.label = spec.label || '@i18n:buttons.save';
     spec.needs_confirm = spec.needs_confirm !== undefined ? spec.needs_confirm : false;
     spec.enable_cond = spec.enable_cond || ['dirty'];
 
@@ -1967,11 +1978,11 @@ exp.delete_action = IPA.delete_action = function(spec) {
  *
  * @class details.enabled_summary_cond
  * @alternateClassName IPA.enabled_summary_cond
+ * @extends facet.summary_cond
  */
 exp.enabled_summary_cond = IPA.enabled_summary_cond = function() {
 
-    var that = IPA.object();
-    lang.mixin(that, {
+    var that = facet_mod.summary_cond ({
         pos: ['enabled'],
         neg: [],
         description: text.get('@i18n:status.enabled'),
@@ -1985,10 +1996,10 @@ exp.enabled_summary_cond = IPA.enabled_summary_cond = function() {
  *
  * @class details.disabled_summary_cond
  * @alternateClassName IPA.disabled_summary_cond
+ * @extends facet.summary_cond
  */
 exp.disabled_summary_cond = IPA.disabled_summary_cond = function() {
-    var that = IPA.object();
-    lang.mixin(that, {
+    var that = facet_mod.summary_cond({
         pos: [],
         neg: ['enabled'],
         description: text.get('@i18n:status.disabled'),
@@ -2007,8 +2018,8 @@ exp.register = function() {
     var f = reg.facet;
 
     a.register('refresh', exp.refresh_action);
-    a.register('reset', exp.reset_action);
-    a.register('update', exp.update_action);
+    a.register('revert', exp.reset_action);
+    a.register('save', exp.update_action);
     a.register('object', exp.object_action);
     a.register('enable', exp.enable_action);
     a.register('disable', exp.disable_action);
@@ -2019,11 +2030,12 @@ exp.register = function() {
         factory: IPA.details_facet,
         pre_ops: [
             exp.details_facet_pre_op
-        ]
+        ],
+        spec: { name: 'details' }
     });
 };
 
 phases.on('registration', exp.register);
 
 return exp;
-});
\ No newline at end of file
+});
