@@ -30,12 +30,14 @@ define([
        './jquery',
        './phases',
        './reg',
+       './rpc',
        './text',
        './field',
        './widget'],
-       function(lang, keys, topic, Evented, builder, IPA, $, phases, reg, text,
-        field_mod, widget_mod) {
+       function(lang, keys, topic, Evented, builder, IPA, $, phases, reg, rpc,
+        text, field_mod, widget_mod) {
 
+var dialogs = {};
 
 /**
  * Opened dialogs
@@ -221,7 +223,7 @@ IPA.dialog = function(spec) {
 
     spec = spec || {};
 
-    var that = new Evented();
+    var that = IPA.object();
 
     /** @property {entity.entity} entity Entity */
     that.entity = IPA.get_entity(spec.entity);
@@ -238,6 +240,16 @@ IPA.dialog = function(spec) {
     /** @property {boolean} Dialog is shown */
     that.is_shown = false;
 
+    /**
+     * Whether dialog should be opened.
+     *
+     * The actual display state is reflected in `is_shown` property. Dialog can
+     * be `opened` and not `is_shown` at the same time, eg., when other dialog
+     * is displayed. Opposite is invalid state.
+     * @property {boolean}
+     */
+    that.opened = false;
+
     /**
      * Close dialog on Escape key press
      * @property {boolean} close_on_escape=true
@@ -405,7 +417,7 @@ IPA.dialog = function(spec) {
 
         that.title_node = $('<h4/>', {
             'class': 'modal-title',
-            text: that.title
+            text: that.title || ''
         }).appendTo(that.header_node);
 
         return that.header_node;
@@ -556,6 +568,7 @@ IPA.dialog = function(spec) {
 
         that.register_listeners();
 
+        this.opened = true;
         this.emit('open', { source: that });
         topic.publish('dialog.open', { source: that });
 
@@ -571,6 +584,7 @@ IPA.dialog = function(spec) {
      * @param  {Function} clb Show callback, called when showing is complete.
      */
     that.show = function(clb) {
+        if (!this.opened) return;
         that.is_shown = true;
         this.dom_node.one('shown.bs.modal', clb);
 
@@ -648,6 +662,12 @@ IPA.dialog = function(spec) {
     that.close = function() {
 
         that.remove_listeners();
+        this.opened = false;
+
+        if (!this.is_shown) {
+            that.emit('closed', { source: that });
+            topic.publish('dialog.closed', { source: that });
+        }
 
         if (!that.dom_node) return;
 
@@ -796,6 +816,8 @@ IPA.adder_dialog = function(spec) {
     /** @property {number} height=300 Height */
     that.height = spec.height || 360;
 
+    that.add_button_label = spec.add_button_label || '@i18n:buttons.add';
+
     if (!that.entity) {
         var except = {
             expected: false,
@@ -1045,7 +1067,7 @@ IPA.adder_dialog = function(spec) {
 
         var add_button = that.create_button({
             name: 'add',
-            label: '@i18n:buttons.add',
+            label: that.add_button_label,
             click: function() {
                 if (!add_button.is_enabled()) return;
                 that.execute();
@@ -1551,6 +1573,200 @@ IPA.confirm_mixin = function() {
     };
 };
 
+
+/**
+ * Dialog's post_ops
+ */
+dialogs.command_dialog_post_op = function(dialog, spec) {
+    dialog.init();
+    return dialog;
+};
+
+
+/**
+ * General command dialog
+ * @class
+ * @extends IPA.dialog
+ * @mixins IPA.confirm_mixin
+ */
+IPA.command_dialog = dialogs.command_dialog = function(spec) {
+
+    var that = IPA.dialog(spec);
+
+    IPA.confirm_mixin().apply(that);
+
+    /**
+     * Method for setting password
+     * @property {string}
+     */
+    that.method = spec.method || 'mod';
+
+    /**
+     * Command args
+     * @property {string[]}
+     */
+    that.args = spec.args || [];
+
+    /**
+     * Command additional options
+     * @property {Object}
+     */
+    that.options = spec.options || {};
+
+    /**
+     * Success message
+     * @property {string}
+     */
+    that.success_message = spec.success_message || '@i18n:dialogs.success';
+
+    /**
+     * Set button label
+     * @property {string}
+     */
+    that.confirm_button_label = spec.confirm_button_label || '@i18n:buttons.ok';
+
+    /**
+     * Failed event
+     * @event
+     */
+    that.failed = IPA.observer();
+
+    /**
+     * Succeeded event
+     * @event
+     */
+    that.succeeded = IPA.observer();
+
+    /**
+     * Execute change
+     */
+    that.execute = function() {
+
+        var command = that.create_command();
+        command.execute();
+    };
+
+    /**
+     * Confirm handler
+     * @protected
+     */
+    that.on_confirm = function() {
+
+        if (!that.validate()) return;
+        that.execute();
+        that.close();
+    };
+
+    /**
+     * Create buttons
+     * @protected
+     */
+    that.create_buttons = function() {
+
+        that.create_button({
+            name: 'confirm',
+            label: that.confirm_button_label,
+            click: function() {
+                that.on_confirm();
+            }
+        });
+
+        that.create_button({
+            name: 'cancel',
+            label: '@i18n:buttons.cancel',
+            click: function() {
+                that.close();
+            }
+        });
+    };
+
+    /**
+     * Make options for command
+     * @protected
+     */
+    that.make_otions = function() {
+
+        var options = {};
+        lang.mixin(options, that.options);
+
+        var fields = that.fields.get_fields();
+        for (var j=0; j<fields.length; j++) {
+            var field = fields[j];
+            var values = field.save();
+            if (!values || values.length === 0 || !field.enabled) continue;
+            if (field.flags.indexOf('no_command') > -1) continue;
+
+            if (values.length === 1) {
+                options[field.param] = values[0];
+            } else {
+                options[field.param] = values;
+            }
+        }
+        return options;
+    };
+
+    /**
+     * Create command
+     * @protected
+     */
+    that.create_command = function() {
+
+        var options = that.make_otions();
+        var entity = null;
+        if (that.entity) entity = that.entity.name;
+        var command = rpc.command({
+            entity: entity,
+            method: that.method,
+            args: that.args,
+            options: options,
+            on_success: function(data) {
+                that.on_success(data);
+            },
+            on_error: function(data) {
+                that.on_error(data);
+            }
+        });
+        return command;
+    };
+
+    /**
+     * Get success message
+     * @protected
+     */
+    that.get_success_message = function() {
+        return text.get(that.success_message);
+    };
+
+    /**
+     * Success handler
+     * @protected
+     * @param {Object} data
+     */
+    that.on_success = function(data) {
+        that.succeeded.notify([data], that);
+        IPA.notify_success(that.get_success_message());
+    };
+
+    /**
+     * Error handler
+     * @protected
+     */
+    that.on_error = function(xhr, status, error) {
+        that.failed.notify([xhr, status, error], that);
+    };
+
+    /**
+     * Init function
+     *
+     * - should be called right after instance creation
+     */
+    that.init = function() {
+        that.create_buttons();
+    };
+
+    return that;
+};
+
 /**
  * Dialog builder
  * - added as builder for 'dialog' registry
@@ -1560,5 +1776,21 @@ var dialog_builder = builder.get('dialog');
 dialog_builder.factory = IPA.dialog;
 reg.set('dialog', dialog_builder.registry);
 
-return {};
+/**
+ * Register dialog
+ */
+dialogs.register = function() {
+
+    var d = reg.dialog;
+
+    d.register({
+        type: 'command',
+        factory: dialogs.command_dialog,
+        post_ops: [dialogs.command_dialog_post_op]
+    });
+};
+
+phases.on('registration', dialogs.register);
+
+return dialogs;
 });
