@@ -142,7 +142,10 @@ return {
                 name: 'identity',
                 fields: [
                     'idnsname',
-                    'idnssoamname',
+                    {
+                        name: 'idnssoamname',
+                        required: true
+                    },
                     'idnssoarname',
                     'idnssoaserial',
                     'idnssoarefresh',
@@ -150,13 +153,6 @@ return {
                     'idnssoaexpire',
                     'idnssoaminimum',
                     'dnsttl',
-                    {
-                        $type: 'combobox',
-                        name: 'dnsclass',
-                        options: [
-                            'IN', 'CS', 'CH', 'HS'
-                        ]
-                    },
                     {
                         $type: 'radio',
                         name: 'idnsallowdynupdate',
@@ -283,7 +279,6 @@ return {
     ],
     standard_association_facets: true,
     adder_dialog: {
-        $factory: IPA.dnszone_adder_dialog,
         height: 300,
         sections: [
             {
@@ -303,26 +298,6 @@ return {
                         validators: ['network']
                     }
                 ]
-            },
-            {
-                name: 'other',
-                fields: [
-                    'idnssoamname',
-                    {
-                        name: 'ip_address',
-                        validators: [ 'ip_address' ],
-                        metadata: '@mc-opt:dnszone_add:ip_address'
-                    },
-                    {
-                        name: 'idnssoarname',
-                        required: false
-                    },
-                    {
-                        $type: 'force_dnszone_add_checkbox',
-                        name: 'force',
-                        metadata: '@mc-opt:dnszone_add:force'
-                    }
-                ]
             }
         ],
         policies: [
@@ -541,6 +516,55 @@ IPA.dnszone_details_facet = function(spec, no_init) {
         return batch;
     };
 
+    that.update = function (on_success, on_error) {
+
+        // If update touches 'idnssoamname', open dialog to allow to skip
+        // DNS resolution check (--force option)
+
+        var command = that.create_update_command();
+
+        command.on_success = function (data, text_status, xhr) {
+            that.update_on_success(data, text_status, xhr);
+            if (on_success) on_success.call(this, data, text_status, xhr);
+        };
+
+        command.on_error = function (xhr, text_status, error_thrown) {
+            that.update_on_error(xhr, text_status, error_thrown);
+            if (on_error) on_error.call(this, xhr, text_status, error_thrown);
+        };
+
+        if (command.options.idnssoamname === undefined) {
+            command.execute();
+            return;
+        }
+
+        var dialog = IPA.confirm_dialog({
+            title: '@i18n:objects.dnszone.soamname_change_title',
+            message: '@i18n:objects.dnszone.soamname_change_message',
+            ok_label: '@i18n:objects.realmdomains.check_dns',
+            on_ok: function () {
+                command.execute();
+            }
+        });
+
+        var cancel_button = dialog.get_button('cancel');
+        dialog.buttons.remove('cancel');
+
+        dialog.create_button({
+            name: 'force',
+            label: '@i18n:objects.realmdomains.force_update',
+            visible: true,
+            click: function () {
+                command.set_option('force', true);
+                command.execute();
+                dialog.close();
+            }
+        });
+
+        dialog.add_button(cancel_button);
+        dialog.open();
+    };
+
     that.update_on_success = function(data, text_status, xhr) {
         that.refresh();
         that.on_update.notify();
@@ -666,77 +690,10 @@ IPA.dnszone_name_widget = function(spec) {
 IPA.force_dnszone_add_checkbox_widget = function(spec) {
     var metadata = IPA.get_command_option('dnszone_add', spec.name);
     spec.label = metadata.label;
-    spec.tooltip = metadata.doc;
+    spec.title = metadata.doc;
     return IPA.checkbox_widget(spec);
 };
 
-IPA.dnszone_adder_dialog = function(spec) {
-
-    spec = spec || {};
-
-    var that = IPA.entity_adder_dialog(spec);
-
-    function ends_with(str, suffix) {
-        return str.indexOf(suffix, str.length - suffix.length) !== -1;
-    }
-
-    var init = function() {
-        var zone_w = that.fields.get_field('idnsname').widget;
-        var reverse_zone_w = that.fields.get_field('name_from_ip').widget;
-        var ns_w = that.fields.get_field('idnssoamname').widget;
-
-        zone_w.value_changed.attach(that.check_ns_ip);
-        reverse_zone_w.value_changed.attach(that.check_ns_ip);
-        ns_w.value_changed.attach(that.check_ns_ip);
-    };
-
-    that.check_ns_ip = function() {
-        var ip_address_f = that.fields.get_field('ip_address');
-        var zone_w = that.fields.get_field('idnsname').widget;
-        var ns_w = that.fields.get_field('idnssoamname').widget;
-
-        var zone = zone_w.save()[0] || '';
-        var ns = ns_w.save()[0] || '';
-
-        var zone_is_reverse = !zone_w.enabled ||
-                              ends_with(zone, '.in-addr.arpa.') ||
-                              ends_with(zone, '.ip6.arpa.');
-        var relative_ns = true;
-        var ns_in_zone = false;
-
-        if (ns && ns[ns.length-1] === '.') {
-            relative_ns = false;
-            ns = ns.slice(0, -1);
-        }
-
-        if (zone && zone[zone.length-1] === '.') {
-            zone = zone.slice(0, -1);
-        }
-
-        if (ns && zone && ends_with(ns, '.' + zone)) {
-            ns_in_zone = true;
-        }
-
-        if (!zone_is_reverse && (relative_ns || ns_in_zone)) {
-            ip_address_f.set_enabled(true);
-            ip_address_f.set_required(true);
-        } else {
-            ip_address_f.reset();
-            ip_address_f.set_required(false);
-            ip_address_f.set_enabled(false);
-        }
-    };
-
-    that.create_content = function() {
-        that.entity_adder_dialog_create_content();
-        that.container.addClass('dnszone-adder-dialog');
-    };
-
-    init();
-
-    return that;
-};
-
 IPA.dns.add_permission_action = function(spec) {
 
     spec = spec || {};
@@ -1071,9 +1028,16 @@ IPA.dns.get_record_metadata = function() {
         {
             name: 'nsrecord',
             attributes: [
-                'ns_part_hostname'
+                'ns_part_hostname',
+                {
+                    $type: 'checkbox',
+                    name: 'force',
+                    label: '@i18n:objects.dnszone.skip_dns_check'
+                }
+            ],
+            adder_attributes: [
+                'force'
             ],
-            adder_attributes: [],
             columns: ['ns_part_hostname']
         },
         {
@@ -1393,8 +1357,12 @@ IPA.dns.record_prepare_editor_for_type = function(type, fields, widgets, update)
         }
 
         var metadata = IPA.get_entity_param('dnsrecord', attribute.name);
-        if (metadata && update && metadata.flags &&
-            metadata.flags.indexOf('no_update') > -1) continue;
+        var no_update = metadata && metadata.flags &&
+                        metadata.flags.indexOf('no_update') > -1;
+        var adder_attr = type.adder_attributes &&
+                         type.adder_attributes.indexOf(attribute.name) > -1;
+
+        if (update && (no_update || adder_attr)) continue;
 
         //create field
         var field = {};
@@ -1993,7 +1961,7 @@ IPA.dns.record_type_table_widget = function(spec) {
 
         dialog.create_button({
             name: 'modify',
-            label: '@i18n:buttons.update',
+            label: '@i18n:buttons.save',
             click: function() {
                 dialog.modify();
             }
