@@ -53,7 +53,12 @@ define([
  * @class facet
  * @singleton
  */
-var exp = {};
+var exp = {
+    sidebar_class: 'sidebar-pf sidebar-pf-left',
+    sidebar_width: ' col-sm-3 col-md-2 col-sm-pull-9 col-md-pull-10',
+    sidebar_content_width: ' col-sm-9 col-md-10 col-sm-push-3 col-md-push-2',
+    sidebar_content_full_width: 'col-md-12'
+};
 exp.facet_spec = {};
 
 /**
@@ -189,6 +194,19 @@ exp.facet = IPA.facet = function(spec, no_init) {
      */
     that.disable_facet_tabs = spec.disable_facet_tabs;
 
+    /**
+     * Facet tabs in sidebar
+     *
+     * There is and effort (#4625) to move all facet tabs into sidebar but it
+     * was not user tested, therefore they remain on the old place for the
+     * time  being.
+     *
+     * This option should be changed when ^^ is removed.
+     * @property {boolean}
+     */
+    that.tabs_in_sidebar = spec.tabs_in_sidebar !== undefined ?
+        spec.tabs_in_sidebar : false;
+
     /**
      * State object for actions
      * @property {facet.state}
@@ -207,6 +225,15 @@ exp.facet = IPA.facet = function(spec, no_init) {
      */
     that.header_actions = spec.header_actions || [];
 
+    /**
+     * Policies
+     * @property {IPA.facet_policies}
+     */
+    that.policies = IPA.facet_policies({
+        container: that,
+        policies: spec.policies
+    });
+
     /**
      * Facet header
      * @property {facet.facet_header}
@@ -283,6 +310,17 @@ exp.facet = IPA.facet = function(spec, no_init) {
      */
     that.dom_node = null;
 
+    /**
+     * Facet groups
+     *
+     * Entity facet groups are used if not defined
+     *
+     * @property {IPA.facet_group[]}
+     */
+    that.facet_groups = builder.build('', spec.facet_groups, {}, {
+        $factory: IPA.facet_group
+    });
+
     /**
      * Facet group name
      * @property {string}
@@ -321,6 +359,14 @@ exp.facet = IPA.facet = function(spec, no_init) {
      */
     that.state = new FacetState();
 
+
+    that.get_full_name = function() {
+        if (that.entity) {
+            return that.entity.create_facet_type(that.name);
+        }
+        return that.name;
+    };
+
     /**
      * Set and normalize pkeys. Merges with existing if present. If keys length
      * differs, the alignment is from the last one to the first one.
@@ -567,7 +613,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
             that.dom_node.detach();
         } else {
             that.dom_node = $('<div/>', {
-                'class': 'facet active-facet fluid-container',
+                'class': 'facet active-facet container-fluid',
                 name: that.name,
                 'data-name': that.name,
                 'data-entity': entity_name
@@ -583,25 +629,45 @@ exp.facet = IPA.facet = function(spec, no_init) {
         var node = dom_node[0];
         construct.place(node,that.container_node);
 
+        var row = $('<div/>', {
+            'class': 'row'
+        }).appendTo(dom_node);
+        var content_cont = row;
 
-        if (that.disable_facet_tabs) dom_node.addClass('no-facet-tabs');
+
+        // header
+        if (that.disable_facet_tabs) {
+            dom_node.addClass('no-facet-tabs');
+        } else if (that.tabs_in_sidebar) {
+            that.sidebar_content_el = $('<div/>', {
+                'class': exp.sidebar_content_width
+            }).appendTo(row);
+            content_cont = $('<div/>', {
+                'class': 'row'
+            }).appendTo(that.sidebar_content_el);
+
+            that.sidebar_el = $('<div/>', {
+                'class': exp.sidebar_class  + exp.sidebar_width
+            }).appendTo(row);
+        }
         dom_node.addClass(that.display_class);
 
         that.header_container = $('<div/>', {
             'class': 'facet-header col-sm-12'
-        }).appendTo(dom_node);
+        }).appendTo(content_cont);
         that.create_header(that.header_container);
 
         that.content = $('<div/>', {
             'class': 'facet-content col-sm-12'
-        }).appendTo(dom_node);
+        }).appendTo(content_cont);
 
         that.error_container = $('<div/>', {
             'class': 'facet-content facet-error col-sm-12'
-        }).appendTo(dom_node);
+        }).appendTo(content_cont);
 
         that.create_content(that.content);
         dom_node.removeClass('active-facet');
+        that.policies.post_create();
     };
 
     /**
@@ -657,6 +723,23 @@ exp.facet = IPA.facet = function(spec, no_init) {
         }
     };
 
+    /**
+     * Display or hide facet tabs - either in sidebar or facet header
+     * @param {boolean} visible
+     */
+    that.set_tabs_visible = function(visible) {
+
+        if (that.disable_facet_tabs) return;
+        if (that.tabs_in_sidebar && that.sidebar_el) {
+            var a = visible ? exp.sidebar_content_width : exp.sidebar_content_full_width;
+            var r = visible ? exp.sidebar_content_full_width : exp.sidebar_content_width;
+            that.sidebar_content_el.removeClass(r);
+            that.sidebar_content_el.addClass(a);
+            that.sidebar_el.css('display', visible ? '' : 'none');
+        }
+        that.header.set_tabs_visible(visible);
+    };
+
     /**
      * Update h1 element in title container
      *
@@ -1045,7 +1128,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
 };
 
 /**
- * Facet header
+ * Facet header which is intended to be used in non-entity facets
  *
  * Widget-like object which purpose is to render facet's header.
  *
@@ -1056,15 +1139,17 @@ exp.facet = IPA.facet = function(spec, no_init) {
  * - action list
  * - facet tabs
  *
- * @class facet.facet_header
- * @alternateClassName IPA.facet_header
+ * @class facet.simple_facet_header
  */
-exp.facet_header = IPA.facet_header = function(spec) {
+exp.simple_facet_header = function(spec) {
 
     spec = spec || {};
 
     var that = IPA.object();
 
+    that.init_group_names = spec.init_group_names != undefined ?
+        spec.init_group_names : false;
+
     /**
      * Facet this header belongs to
      * @property {facet.facet}
@@ -1095,6 +1180,41 @@ exp.facet_header = IPA.facet_header = function(spec) {
         that.facet.action_state.changed.attach(that.update_summary);
 
         that.title_widget = IPA.facet_title();
+
+        if (!that.facet.tabs_in_sidebar) {
+            that.tabs_widget = new exp.FacetGroupsWidget({
+                facet: this.facet,
+                init_group_names: that.init_group_names
+            });
+        } else {
+            that.tabs_widget = new exp.FacetGroupsWidget({
+                facet: this.facet,
+                init_group_names: that.init_group_names,
+                css_class: '',
+                group_el_type: '<div/>',
+                group_class: '',
+                group_label_el_type: '<div/>',
+                group_label_class: 'nav-category',
+                group_label_title_el_type: '<h2/>',
+                group_label_title_class: '',
+                tab_cont_el_type: '<div/>',
+                tab_cont_class: '',
+                tab_list_el_type: '<ul/>',
+                tab_list_class: 'nav nav-pills nav-stacked',
+                tab_el_type: '<li/>',
+                tab_class: 't',
+                selected_class: 'active'
+            });
+        }
+    };
+
+    /**
+     * Display or hide facet tabs
+     * @param {boolean} visible
+     */
+    that.set_tabs_visible = function(visible) {
+        if (!this.tabs_widget) return;
+        this.tabs_widget.set_visible(visible);
     };
 
     /**
@@ -1102,14 +1222,11 @@ exp.facet_header = IPA.facet_header = function(spec) {
      */
     that.select_tab = function() {
         if (that.facet.disable_facet_tabs) return;
-
-        $(that.facet_tabs).find('a').removeClass('selected');
         var facet_name = that.facet.name;
-
         if (!facet_name || facet_name === 'default') {
-            that.facet_tabs.find('a:first').addClass('selected');
+            that.tabs_widget.select_first();
         } else {
-            that.facet_tabs.find('a#' + facet_name ).addClass('selected');
+            that.tabs_widget.select(that.facet.get_full_name());
         }
     };
 
@@ -1137,10 +1254,135 @@ exp.facet_header = IPA.facet_header = function(spec) {
         that.title_widget.update(title_info);
     };
 
+    that.update_breadcrumb = function(pkey) {
+        // doesn't support breadcrumb
+        return;
+    };
+
+
+    that.create_facet_groups = function(container) {
+        var facet_groups = that.get_facet_groups();
+        this.tabs_widget.groups = facet_groups;
+        var tabs = this.tabs_widget.render();
+        container.append(tabs);
+    };
+
+    /**
+     * Get facet groups for current facet.
+     *
+     * @return {Array} Array of facet groups
+     */
+    that.get_facet_groups = function() {
+        if (that.facet.facet_groups) {
+            return that.facet.facet_groups;
+        }
+        return [];
+    };
+
+    /**
+     * Create header's HTML
+     * @param {jQuery} container
+     */
+    that.create = function(container) {
+
+        that.container = container;
+
+        if (!that.facet.disable_breadcrumb) {
+            that.breadcrumb = $('<ol/>', {
+                'class': 'breadcrumb'
+            }).appendTo(container);
+            that.update_breadcrumb('');
+        }
+
+        that.title_widget.create(container);
+        that.title_widget.update({ text: that.facet.label });
+
+        if (!that.facet.disable_facet_tabs) {
+
+            var tab_cont = container;
+            if (that.facet.tabs_in_sidebar) {
+                tab_cont = that.facet.sidebar_el;
+            }
+            that.create_facet_groups(tab_cont);
+            $(window).trigger('resize');
+        }
+    };
+
+    /**
+     * Reflect facet's action state summary into title widget class and icon
+     * title.
+     */
+    that.update_summary = function() {
+        var summary = that.facet.action_state.summary();
+
+        if (summary.state.length > 0) {
+            var css_class = summary.state.join(' ');
+            that.title_widget.set_class(css_class);
+            that.title_widget.set_icon_title(summary.description);
+        }
+    };
+
+    /**
+     * Compute maximum pkey length to be displayed in header
+     * @return {number} length
+     */
+    that.get_max_pkey_length = function() {
+
+        var label_w, max_pkey_w, max_pkey_l, al, al_w, icon_w, char_w, container_w;
+
+        container_w = that.container.width();
+        icon_w = that.title_widget.icon.width();
+        label_w = that.title_widget.title.width();
+        char_w = label_w / that.title_widget.title.text().length;
+        max_pkey_w = container_w - icon_w - label_w;
+        max_pkey_w -= 10; //some space correction to be safe
+        max_pkey_l = Math.ceil(max_pkey_w / char_w);
+
+        return max_pkey_l;
+    };
+
+    /**
+     * Clear displayed information
+     */
+    that.clear = function() {
+    };
+
+    that.facet_header_set_pkey = that.set_pkey;
+
+    return that;
+};
+
+/**
+ * Facet header
+ *
+ * Widget-like object which purpose is to render facet's header.
+ *
+ * By default, facet header consists of:
+ *
+ * - breadcrumb navigation
+ * - title
+ * - action list
+ * - facet tabs
+ *
+ * @class facet.facet_header
+ * @alternateClassName IPA.facet_header
+ */
+exp.facet_header = IPA.facet_header = function(spec) {
+
+    spec = spec || {};
+
+    var that = exp.simple_facet_header(spec);
+
     that.update_breadcrumb = function(pkey) {
 
         if (!that.breadcrumb) return;
 
+        if (pkey === undefined) {
+            pkey = that.facet.get_pkey();
+            var pkey_max = that.get_max_pkey_length();
+            pkey = IPA.limit_text(pkey, pkey_max);
+        }
+
         var items = [];
         var item, i, l, keys, target_facet, target_facet_keys, containing_entity;
 
@@ -1222,93 +1464,20 @@ exp.facet_header = IPA.facet_header = function(spec) {
     };
 
     /**
-     * Create link for facet tab
-     * @protected
-     * @param {jQuery} container
-     * @param {facet.facet} other_facet
+     * Get facet groups for current facet.
+     *
+     * By default facet groups are defined in entity. In certain circumstances
+     * it could be overridden, i.e., if different facet contained in the facet
+     * groups uses different entity.
+     * @return {Array} Array of facet groups
      */
-    that.create_facet_link = function(container, other_facet) {
-
-        var li = $('<li/>', {
-            name: other_facet.name,
-            click: function() {
-                if (li.hasClass('entity-facet-disabled')) {
-                    return false;
-                }
-
-                var pkeys = that.facet.get_pkeys();
-                navigation.show(other_facet, pkeys);
-
-                return false;
-            }
-        }).appendTo(container);
-
-        $('<a/>', {
-            text: other_facet.tab_label,
-            id: other_facet.name
-        }).appendTo(li);
+    that.get_facet_groups = function() {
+        if (that.facet.facet_groups) {
+            return that.facet.facet_groups;
+        }
+        return that.facet.entity.facet_groups.values;
     };
 
-    /**
-     * Create facet tab group
-     * @protected
-     * @param {jQuery} container
-     * @param {Object} facet_group
-     */
-    that.create_facet_group = function(container, facet_group) {
-
-        var section = $('<div/>', {
-            name: facet_group.name,
-            'class': 'facet-group'
-        }).appendTo(container);
-
-        $('<div/>', {
-            'class': 'facet-group-label'
-        }).appendTo(section);
-
-        var ul = $('<ul/>', {
-            'class': 'facet-tab'
-        }).appendTo(section);
-
-        var facets = facet_group.facets.values;
-        for (var i=0; i<facets.length; i++) {
-            var facet = facets[i];
-            that.create_facet_link(ul, facet);
-        }
-    };
-
-    /**
-     * Create header's HTML
-     * @param {jQuery} container
-     */
-    that.create = function(container) {
-
-        that.container = container;
-
-        if (!that.facet.disable_breadcrumb) {
-            that.breadcrumb = $('<ol/>', {
-                'class': 'breadcrumb'
-            }).appendTo(container);
-            that.update_breadcrumb('');
-        }
-
-        that.title_widget.create(container);
-        that.title_widget.update({ text: that.facet.label });
-
-        if (!that.facet.disable_facet_tabs) {
-            that.facet_tabs = $('<div/>', {
-                'class': 'facet-tabs'
-            }).appendTo(container);
-
-            var facet_groups = that.facet.entity.facet_groups.values;
-            for (var i=0; i<facet_groups.length; i++) {
-                var facet_group = facet_groups[i];
-                if (facet_group.facets.length) {
-                    that.create_facet_group(that.facet_tabs, facet_group);
-                }
-            }
-        }
-    };
 
     /**
      * Update displayed information with new data
@@ -1328,74 +1497,34 @@ exp.facet_header = IPA.facet_header = function(spec) {
         if (!that.facet.disable_facet_tabs) {
             var pkey = that.facet.get_pkey();
 
-            var facet_groups = that.facet.entity.facet_groups.values;
+            var facet_groups = that.get_facet_groups();
             for (var i=0; i<facet_groups.length; i++) {
                 var facet_group = facet_groups[i];
 
-                var span = $('.facet-group[name='+facet_group.name+']', that.facet_tabs);
-                if (!span.length) continue;
-
                 var label = facet_group.label;
                 if (pkey && label) {
                     var limited_pkey = IPA.limit_text(pkey, 20);
                     label = label.replace('${primary_key}', limited_pkey);
-                } else {
+                } else if (!label) {
                     label = '';
                 }
 
-                var label_container = $('.facet-group-label', span);
-                label_container.text(label);
-                if (pkey) label_container.attr('title', pkey);
+                this.tabs_widget.update_group(facet_group.name, label, pkey);
 
                 var facets = facet_group.facets.values;
                 for (var j=0; j<facets.length; j++) {
-                    var facet = facets[j];
-                    var link = $('li[name='+facet.name+'] a', span);
-
+                    var facet =  reg.facet.get(facets[j]);
                     var values = result ? result[facet.name] : null;
+                    label = facet.tab_label;
                     if (values) {
-                        link.text(facet.tab_label+' ('+values.length+')');
-                    } else {
-                        link.text(facet.tab_label);
+                        label = facet.tab_label + ' (' + values.length + ')';
                     }
+                    this.tabs_widget.update_tab(facet.get_full_name(), label);
                 }
             }
         }
     };
 
-    /**
-     * Reflect facet's action state summary into title widget class and icon
-     * title.
-     */
-    that.update_summary = function() {
-        var summary = that.facet.action_state.summary();
-
-        if (summary.state.length > 0) {
-            var css_class = summary.state.join(' ');
-            that.title_widget.set_class(css_class);
-            that.title_widget.set_icon_title(summary.description);
-        }
-    };
-
-    /**
-     * Compute maximum pkey length to be displayed in header
-     * @return {number} length
-     */
-    that.get_max_pkey_length = function() {
-
-        var label_w, max_pkey_w, max_pkey_l, al, al_w, icon_w, char_w, container_w;
-
-        container_w = that.container.width();
-        icon_w = that.title_widget.icon.width();
-        label_w = that.title_widget.title.width();
-        char_w = label_w / that.title_widget.title.text().length;
-        max_pkey_w = container_w - icon_w - label_w;
-        max_pkey_w -= 10; //some space correction to be safe
-        max_pkey_l = Math.ceil(max_pkey_w / char_w);
-
-        return max_pkey_l;
-    };
-
     /**
      * Clear displayed information
      */
@@ -1403,11 +1532,168 @@ exp.facet_header = IPA.facet_header = function(spec) {
         that.load();
     };
 
-    that.facet_header_set_pkey = that.set_pkey;
-
     return that;
 };
 
+exp.FacetGroupsWidget = declare([], {
+
+    facet: null,
+    groups: null,
+    group_els: null,
+    el: null,
+    visible: true,
+    init_group_names: false,
+    css_class: 'facet-tabs',
+    group_el_type: '<div/>',
+    group_class: 'facet-group',
+    group_label_el_type: '<div/>',
+    group_label_class: 'facet-group-label',
+    group_label_title_el_type: '<span/>',
+    group_label_title_class: '',
+    tab_cont_el_type: '<div/>',
+    tab_cont_class: '',
+    tab_list_el_type: '<ul/>',
+    tab_list_class: 'facet-tab',
+    tab_el_type: '<li/>',
+    tab_class: 't',
+    selected_class: 'selected',
+
+    render: function() {
+
+        this.group_els = {};
+        this.tab_els = {};
+
+        this.el = $('<div/>', { 'class': this.css_class });
+
+        for (var i=0; i<this.groups.length; i++) {
+            var group = this.groups[i];
+            if (group.facets.length) {
+                var group_el = this.render_group(group);
+                this.el.append(group_el);
+            }
+        }
+        return this.el;
+    },
+
+    render_group: function(group) {
+
+        var gr = this.group_els[group.name] = { tab_els: {}};
+
+        gr.group_el = $(this.group_el_type, {
+            'class': this.group_class,
+            name: group.name
+        });
+
+        gr.label_el = $(this.group_label_el_type, {
+            'class': this.group_label_class
+        }).appendTo(gr.group_el);
+
+        gr.label_title_el = $(this.group_label_title_el_type, {
+            'class': this.group_label_title_class,
+            text: ''
+        }).appendTo(gr.label_el);
+
+        if (this.init_group_names) {
+            gr.label_title_el.text(group.label || '');
+        }
+
+        var tab_cont = $(this.tab_cont_el_type, { 'class': this.tab_cont_class });
+        var tab_list = $(this.tab_list_el_type, { 'class': this.tab_list_class });
+        tab_list.appendTo(tab_cont);
+        var facets = group.facets.values;
+        for (var i=0,l=facets.length; i<l ;i++) {
+            var facet = reg.facet.get(facets[i]);
+            var tab_el = this.tab_els[facet.get_full_name()] = this.render_tab(facet);
+            tab_list.append(tab_el);
+        }
+        gr.group_el.append(tab_cont);
+
+        return gr.group_el;
+    },
+
+    render_tab: function(tab) {
+        var self = this;
+        var el = $(this.tab_el_type, {
+            name: tab.name,
+            'class': this.tab_class,
+            click: function() {
+                if (el.hasClass('entity-facet-disabled')) {
+                    return false;
+                }
+                self.on_click(tab);
+                return false;
+            }
+        });
+
+        $('<a/>', {
+            text: tab.tab_label,
+            'class': 'tab-link',
+            href: "#" + navigation.create_hash(tab, {}),
+            name: tab.name
+        }).appendTo(el);
+
+        return el;
+    },
+
+    on_click: function(facet) {
+        if (this.facet.get_pkeys) {
+            var pkeys = this.facet.get_pkeys();
+            navigation.show(facet, pkeys);
+        } else {
+            navigation.show(facet);
+        }
+    },
+
+    update_group: function(group_name, text, title) {
+        if (!this.group_els[group_name]) return;
+        var label_el = this.group_els[group_name].label_title_el;
+        label_el.text(text);
+        if (title) label_el.attr('title', title);
+    },
+
+    update_tab: function(tab_name, text, title) {
+        var tab_el = this.tab_els[tab_name];
+        var label_el = $('a', tab_el);
+        label_el.text(text);
+        if (title) label_el.attr('title', title);
+    },
+
+    select: function(tab_name) {
+        if (!this.el) return;
+        var cls = this.selected_class;
+        var tab_el = this.tab_els[tab_name];
+
+        this.el.find(this.tab_class).removeClass(cls);
+        this.el.find('.tab-link').removeClass(cls);
+
+        tab_el.addClass(cls);
+        tab_el.find('.tab-link').addClass(cls);
+    },
+
+    select_first: function() {
+        if (!this.el) return;
+        this.el.find('.tab-link').removeClass(this.selected_class);
+        this.el.find(this.tab_class).removeClass(this.selected_class);
+        var first = this.el.find('.tab-link:first');
+        first.addClass(this.selected_class);
+        first.parent().addClass(this.selected_class);
+    },
+
+    set_visible: function(visible) {
+        this.visible = visible;
+        this._apply_visible();
+    },
+
+    _apply_visible: function() {
+        if (!this.el) return;
+        this.el.css('display', this.visible ? '' : 'none');
+    },
+
+    constructor: function(spec) {
+        lang.mixin(this, spec);
+    }
+});
+
 /**
  * Facet title widget
  *
@@ -1542,6 +1828,16 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
      */
     that.search_all_entries = spec.search_all_entries;
 
+    /**
+     * Member resolution(no_member: true ) in rpc request is skipped by default
+     * to improve performance of getting data.
+     *
+     * Set always_request_members to true to skip this default behavior.
+     * @property {boolean}
+     */
+    that.always_request_members = spec.always_request_members !== undefined ?
+        spec.always_request_members : false;
+
     /**
      * Sort records
      */
@@ -1681,7 +1977,7 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
         }
 
         that.table.refresh_pagination();
-
+        that.policies.post_load(data);
         that.post_load.notify([data], that);
         that.clear_expired_flag();
     };
@@ -1910,7 +2206,7 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
                 args: [pkey]
             });
 
-            if (that.table.entity.has_members()) {
+            if (!that.always_request_members && that.table.entity.has_members()) {
                 command.set_options({no_members: true});
             }
 
@@ -2072,7 +2368,7 @@ exp.facet_group = IPA.facet_group = function(spec) {
      * Facet collection
      * @property {ordered_map}
      */
-    that.facets = $.ordered_map();
+    that.facets = $.ordered_map(spec.facets);
 
     /**
      * Add facet to the map
@@ -2386,7 +2682,11 @@ exp.action_holder = IPA.action_holder = function(spec) {
         }
 
         that.facet.action_state.changed.attach(that.state_changed);
-        that.facet.post_load.attach(that.on_load);
+        if (that.facet.post_load) {
+            that.facet.post_load.attach(that.on_load);
+        } else {
+            on(that.facet, 'load', that.on_load);
+        }
     };
 
     /**
@@ -2490,7 +2790,8 @@ exp.state = IPA.state = function(spec) {
      * Summary conditions
      * @property {Array.<Object>}
      */
-    that.summary_conditions = builder.build('', spec.summary_conditions) || [];
+    that.summary_conditions = builder.build('', spec.summary_conditions, {},
+                                    { $factory: exp.summary_cond }) || [];
 
     /**
      * Initializes evaluators
@@ -2517,9 +2818,16 @@ exp.state = IPA.state = function(spec) {
     that.on_eval_changed = function() {
 
         var evaluator = this;
+        that.put(evaluator.name, evaluator.state);
+    };
 
-        that.state.put(evaluator.name, evaluator.state);
-
+    /**
+     * Set state and notify
+     * @param  {string} name
+     * @param  {string} state
+     */
+    that.put = function(name, state) {
+        that.state.put(name, state);
         that.notify();
     };
 
@@ -2568,6 +2876,42 @@ exp.state = IPA.state = function(spec) {
     return that;
 };
 
+/**
+ * Summary condition base class
+ *
+ * @class facet.summary_cond
+ */
+exp.summary_cond = function(spec) {
+
+    var that = IPA.object();
+
+    /**
+     * State which must be present in order to be positively evaluated
+     * @property {string[]}
+     */
+    that.pos = spec.pos || [];
+
+    /**
+     * State which must not be present in order to be positively evaluated
+     * @property {string[]}
+     */
+    that.neg = spec.neg || [];
+
+    /**
+     * States which will be set in positive evaluation
+     * @property {string[]}
+     */
+    that.state = spec.state || [];
+
+    /**
+     * Description which will be set in positive evaluation
+     * @property {string}
+     */
+    that.description = spec.description || '';
+
+    return that;
+};
+
 /**
  * Summary evaluator for {@link facet.state}
  * @class facet.summary_evaluator
@@ -2670,8 +3014,14 @@ exp.state_evaluator = IPA.state_evaluator = function(spec) {
      */
     that.init = function(facet) {
 
-        if (that.event_name && facet[that.event_name]) {
+        if (!that.event_name) return;
+
+        if (facet[that.event_name] && facet[that.event_name].attach) {
+            // facets based on facet.facet
             facet[that.event_name].attach(that.on_event);
+        } else if (facet.emit) {
+            // facets based on facet/Facet
+            on(facet, that.event_name, that.on_event);
         }
     };
 
@@ -2700,6 +3050,37 @@ exp.state_evaluator = IPA.state_evaluator = function(spec) {
     return that;
 };
 
+/**
+ * Noop evaluator always sets the state on post_load on the first time
+ * @class facet.noop_state_evaluator
+ * @extends facet.state_evaluator
+ * @alternateClassName IPA.noop_state_evaluator
+ */
+exp.noop_state_evaluator = IPA.noop_state_evaluator = function(spec) {
+
+    spec = spec || {};
+    spec.event = spec.event || 'post_load';
+
+    var that = IPA.state_evaluator(spec);
+    that.name = spec.name || 'noop_state_evaluator';
+
+    /**
+     * States to be set
+     * @property {string[]}
+     */
+    that.state = spec.state || [];
+
+    /**
+     * @inheritDoc
+     */
+    that.on_event = function() {
+        that.notify_on_change(that.state);
+    };
+
+    return that;
+};
+
+
 /**
  * Sets 'dirty' state when facet is dirty
  * @class facet.dirty_state_evaluator
