@@ -60,7 +60,7 @@ var field = {};
 field.field = IPA.field = function(spec) {
     spec = spec || {};
 
-    var that = new Evented();
+    var that = IPA.object();
 
     /**
      * Entity
@@ -105,6 +105,16 @@ field.field = IPA.field = function(spec) {
      */
     that.acl_param = spec.acl_param || that.param;
 
+    /**
+     * Rights which determines what operation can do with this field or
+     * attribute.
+     *
+     * E.g., 'rscwo' - read, search, compare, write(mod-add), obliterate(mod-del)
+     *
+     * @property {string}
+     */
+    that.acl_rights = spec.acl_rights || 'r';
+
     /**
      * Label
      * @property {string}
@@ -112,10 +122,10 @@ field.field = IPA.field = function(spec) {
     that.label = text.get(spec.label);
 
     /**
-     * Tooltip
+     * Title
      * @property {string}
      */
-    that.tooltip = text.get(spec.tooltip);
+    that.title = text.get(spec.title);
 
     /**
      * Measurement unit
@@ -226,6 +236,18 @@ field.field = IPA.field = function(spec) {
      */
     that.metadata = spec.metadata;
 
+    /**
+     * Override of metadata.minvalue
+     * @property {number}
+     */
+    that.minvalue = spec.minvalue;
+
+    /**
+     * Override of metadata.maxvalue
+     * @property {number}
+     */
+    that.maxvalue = spec.maxvalue;
+
     /**
      * Validators
      * @property {Array.<IPA.validator>}
@@ -299,8 +321,8 @@ field.field = IPA.field = function(spec) {
             if (!that.label) {
                 that.label = that.metadata.label || '';
             }
-            if (!that.tooltip) {
-                that.tooltip = that.metadata.doc || '';
+            if (!that.title) {
+                that.title = that.metadata.doc || '';
             }
         }
 
@@ -449,6 +471,7 @@ field.field = IPA.field = function(spec) {
     that.load_writable = function(record) {
 
         var writable = true;
+        var old = that.acl_rights;
 
         function has_write(record, param) {
             var rights = record.attributelevelrights[param];
@@ -466,11 +489,17 @@ field.field = IPA.field = function(spec) {
             }
         }
 
-        if (record && record.attributelevelrights && writable) {
+        if (record && record.attributelevelrights) {
             var rights = record.attributelevelrights[that.acl_param];
             var write_attr = has_write(record, that.acl_param);
+            var all_rights = record.attributelevelrights['*'];
             var write_all = has_write(record, '*');
 
+            // don't assume any value if the rights are not defined, keep the original
+            if (rights !== undefined || all_rights !== undefined) {
+                that.acl_rights = rights || all_rights || '';
+            }
+
             // Some objects in LDAP may not have proper object class set and
             // therefore server doesn't send proper attribute rights. Flag
             // 'w_if_no_aci' should be used when we want to ensure that UI
@@ -480,10 +509,13 @@ field.field = IPA.field = function(spec) {
             var may_add_oc = !rights && write_oc && that.flags.indexOf('w_if_no_aci') > -1;
 
             // If no rights, change writable to False:
-            writable = write_attr || write_all || may_add_oc;
+            writable = writable && (write_attr || write_all || may_add_oc);
         }
 
         that.set_writable(writable);
+        if (old !== that.acl_rights) {
+            that.emit('acl-rights-change', { source: that, rights: that.acl_rights, old: old });
+        }
     };
 
     /**
@@ -789,7 +821,7 @@ field.Adapter = declare(null, {
         var dr = data.result;
         var record = null;
         if (dr) {
-            if (dr.result) record = dr.result;
+            if (IPA.defined(dr.result)) record = dr.result;
             else if (dr.results) {
                 var result = dr.results[this.result_index];
                 if (result) record = result.result;
@@ -935,6 +967,16 @@ field.metadata_validator = IPA.metadata_validator = function(spec) {
 
     var that = IPA.validator(spec);
 
+    that.get_property = function(name, obj, metadata) {
+        var prop = null;
+        if (IPA.defined(obj[name], true)) {
+            prop = obj[name];
+        } else if (IPA.defined(metadata[name], true)) {
+            prop = metadata[name];
+        }
+        return prop;
+    };
+
     /**
      * @inheritDoc
      */
@@ -960,13 +1002,17 @@ field.metadata_validator = IPA.metadata_validator = function(spec) {
 
         if (number) {
 
-            if (IPA.defined(metadata.minvalue, true) && Number(value) < Number(metadata.minvalue)) {
+            var numVal = Number(value);
+            var minvalue = that.get_property('minvalue', context, metadata);
+            var maxvalue = that.get_property('maxvalue', context, metadata);
+
+            if (IPA.defined(minvalue) &&  numVal < Number(minvalue)) {
                 message = text.get('@i18n:widget.validation.min_value');
                 message = message.replace('${value}', metadata.minvalue);
                 return that.false_result(message);
             }
 
-            if (IPA.defined(metadata.maxvalue, true) && Number(value) > Number(metadata.maxvalue)) {
+            if (IPA.defined(maxvalue) && numVal > Number(maxvalue)) {
                 message = text.get('@i18n:widget.validation.max_value');
                 message = message.replace('${value}', metadata.maxvalue);
                 return that.false_result(message);
@@ -1076,10 +1122,6 @@ field.datetime_field = IPA.datetime_field = function(spec) {
         template: datetime.templates.generalized
     };
     spec.data_parser = spec.formatter || 'datetime';
-    spec.ui_formatter = spec.ui_formatter || spec.formatter || {
-        $type: 'datetime',
-        template: datetime.templates.human
-    };
     spec.ui_parser = spec.ui_parser || 'datetime';
 
     var that = IPA.field(spec);
@@ -1448,7 +1490,9 @@ reg.set('validator', field.validator_builder.registry);
 field.adapter_builder = builder.get('adapter');
 field.adapter_builder.ctor = field.Adapter;
 field.adapter_builder.post_ops.push(function(obj, spec, context) {
-        obj.context = context.context;
+        if (context.context) {
+            obj.context = context.context;
+        }
         return obj;
     }
 );
