@@ -21,6 +21,7 @@
  */
 
 define([
+        'dojo/on',
         './ipa',
         './jquery',
         './phases',
@@ -29,7 +30,7 @@ define([
         './search',
         './association',
         './entity'],
-    function(IPA, $, phases, reg) {
+    function(on, IPA, $, phases, reg) {
 
 var exp = IPA.group = {};
 
@@ -67,7 +68,15 @@ return {
                                 posixgroup: '@i18n:objects.group.posix'
                             }
                         },
-                        'gidnumber'
+                        'gidnumber',
+                        {
+                            $type: 'link',
+                            name: 'pwpolicy',
+                            param: 'cn',
+                            label: '@mo:pwpolicy.label_singular',
+                            other_entity: 'pwpolicy',
+                            require_link: true
+                        }
                     ]
                 }
             ],
@@ -77,7 +86,7 @@ return {
                 'make_external',
                 'delete'
             ],
-            header_actions: ['select_action', 'make_posix', 'make_external', 'delete'],
+            header_actions: ['make_posix', 'make_external', 'delete'],
             state: {
                 evaluators: [
                     IPA.object_class_evaluator
@@ -197,14 +206,14 @@ IPA.group_adder_dialog = function(spec) {
     var init = function() {
 
         var type_field = that.fields.get_field('type');
-        type_field.widget.value_changed.attach(that.on_type_change);
+        on(type_field, 'value-change', that.on_type_change);
     };
 
     that.on_type_change = function() {
 
         var type_field = that.fields.get_field('type');
         var gid_field = that.fields.get_field('gidnumber');
-        var posix = type_field.save()[0] === 'posix';
+        var posix = type_field.get_value()[0] === 'posix';
 
         if (!posix) {
             gid_field.reset();
@@ -240,6 +249,7 @@ IPA.group.make_posix_action = function(spec) {
     spec.name = spec.name || 'make_posix';
     spec.method = spec.method || 'mod';
     spec.label = spec.label || '@i18n:objects.group.make_posix';
+    spec.needs_confirm = spec.needs_confirm !== undefined ? spec.needs_confirm : true;
     spec.disable_cond = spec.disable_cond || ['oc_posixgroup', 'oc_ipaexternalgroup'];
     spec.options = spec.options || {
         posix: true
@@ -256,6 +266,7 @@ IPA.group.make_external_action = function(spec) {
     spec.name = spec.name || 'make_external';
     spec.method = spec.method || 'mod';
     spec.label = spec.label || '@i18n:objects.group.make_external';
+    spec.needs_confirm = spec.needs_confirm !== undefined ? spec.needs_confirm : true;
     spec.disable_cond = spec.disable_cond || ['oc_posixgroup','oc_ipaexternalgroup'];
     spec.options = spec.options || {
         external: true
