@@ -24,13 +24,14 @@ define([
         './jquery',
         './phases',
         './reg',
+        './rpc',
         './association',
         './details',
         './entity',
         './rule',
         './search'
         ],
-            function(IPA, $, phases, reg) {
+            function(IPA, $, phases, reg, rpc) {
 
 var exp = IPA.hbac = {
     //priority of commands in details facet
@@ -61,12 +62,12 @@ var spec =  {
                 {
                     name: 'disable',
                     label: '@i18n:buttons.disable',
-                    icon: 'disabled-icon'
+                    icon: 'fa-minus'
                 },
                 {
                     name: 'enable',
                     label: '@i18n:buttons.enable',
-                    icon: 'enabled-icon'
+                    icon: 'fa-check'
                 }
             ]
         },
@@ -80,7 +81,7 @@ var spec =  {
                 'disable',
                 'delete'
             ],
-            header_actions: ['select_action', 'enable', 'disable', 'delete'],
+            header_actions: ['enable', 'disable', 'delete'],
             state: {
                 evaluators: [
                     {
@@ -224,6 +225,7 @@ return {
 };};
 
 /**
+ * @ignore
  * @param {Object} facet spec
  */
 var add_hbacrule_details_facet_widgets = function (spec) {
@@ -247,7 +249,7 @@ var add_hbacrule_details_facet_widgets = function (spec) {
 
     spec.widgets = [
         {
-            $type: 'details_table_section',
+            $type: 'details_section',
             name: 'general',
             label: '@i18n:details.general',
             widgets: [
@@ -288,7 +290,7 @@ var add_hbacrule_details_facet_widgets = function (spec) {
 
     spec.widgets.push(
         {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'user',
             label: '@i18n:objects.hbacrule.user',
             widgets: [
@@ -357,7 +359,7 @@ var add_hbacrule_details_facet_widgets = function (spec) {
 
     spec.widgets.push(
         {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'host',
             label: '@i18n:objects.hbacrule.host',
             widgets: [
@@ -430,7 +432,7 @@ var add_hbacrule_details_facet_widgets = function (spec) {
 
     spec.widgets.push(
     {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'service',
             label: '@i18n:objects.hbacrule.service',
             widgets: [
