@@ -25,13 +25,14 @@ define([
         './navigation',
         './phases',
         './reg',
+        './rpc',
         './text',
         './details',
         './search',
         './association',
         './entity',
         './hbac'],
-            function(metadata_provider, IPA, $, navigation, phases, reg, text) {
+            function(metadata_provider, IPA, $, navigation, phases, reg, rpc, text) {
 
 var exp = {};
 
@@ -182,13 +183,8 @@ IPA.hbac.test_facet = function(spec) {
             that.prev_button = IPA.button({
                 name: 'prev',
                 label: '@i18n:widget.prev',
-                icon: 'ui-icon ui-icon-triangle-1-w',
-                click: function() {
-                    if (!that.prev_button.hasClass('action-button-disabled')) {
-                        that.prev();
-                    }
-                    return false;
-                }
+                icon: 'fa-chevron-left',
+                click: that.prev
             }).appendTo(buttons);
 
             buttons.append(' ');
@@ -197,13 +193,8 @@ IPA.hbac.test_facet = function(spec) {
         that.next_button = IPA.button({
             name: 'next',
             label: '@i18n:widget.next',
-            icon: 'ui-icon ui-icon-triangle-1-e',
-            click: function() {
-                if (!that.next_button.hasClass('action-button-disabled')) {
-                    that.next();
-                }
-                return false;
-            }
+            icon: 'fa-chevron-right',
+            click: that.next
         }).appendTo(buttons);
     };
 
@@ -235,7 +226,7 @@ IPA.hbac.test_facet = function(spec) {
 
         var filter = that.state.filter;
 
-        var command = IPA.command({
+        var command = rpc.command({
             name: that.get_search_command_name(),
             entity: that.managed_entity.name,
             method: 'find',
@@ -297,6 +288,7 @@ IPA.hbac.test_select_facet = function(spec) {
 
         that.filter = $('<input/>', {
             type: 'text',
+            'class': 'form-control',
             name: 'filter'
         }).appendTo(filter_container);
 
@@ -309,15 +301,13 @@ IPA.hbac.test_select_facet = function(spec) {
 
         that.find_button = IPA.action_button({
             name: 'find',
-            icon: 'search-icon',
+            icon: 'fa-search',
             click: function() {
                 that.find();
                 return false;
             }
         }).appendTo(filter_container);
 
-        header.append(IPA.create_network_spinner());
-
         var content = $('<div/>', {
             'class': 'hbac-test-content'
         }).appendTo(container);
@@ -330,7 +320,8 @@ IPA.hbac.test_select_facet = function(spec) {
         var tr = $('<tr/>').appendTo(that.table.tfoot);
 
         var td = $('<td/>', {
-            name: 'external'
+            name: 'external',
+            colspan: that.columns.length + 1
         }).appendTo(tr);
 
         that.external_radio = $('<input/>', {
@@ -523,26 +514,22 @@ IPA.hbac.test_run_facet = function(spec) {
         }).appendTo(container);
 
         var top_panel = $('<div/>', {
-            'class': 'hbac-test-top-panel'
+            'class': 'hbac-test-top-panel row'
         }).appendTo(header);
 
         var button_panel = $('<div/>', {
-            'class': 'hbac-test-button-panel'
+            'class': 'hbac-test-button-panel col-sm-2'
         }).appendTo(top_panel);
 
         that.run_button = IPA.button({
             name: 'run_test',
             label: '@i18n:objects.hbactest.run_test',
-            click: function() {
-                if (!that.run_button.hasClass('action-button-disabled')) {
-                    that.run();
-                }
-                return false;
-            }
+            icon: 'fa-gear',
+            click: that.run
         }).appendTo(button_panel);
 
         var result_panel = $('<div/>', {
-            'class': 'hbac-test-result-panel'
+            'class': 'hbac-test-result-panel col-sm-10'
         }).appendTo(top_panel);
 
         that.test_result = $('<p/>', {
@@ -605,13 +592,8 @@ IPA.hbac.test_run_facet = function(spec) {
         that.prev_button = IPA.button({
             name: 'prev',
             label: '@i18n:widget.prev',
-            icon: 'ui-icon ui-icon-triangle-1-w',
-            click: function() {
-                if (!that.prev_button.hasClass('action-button-disabled')) {
-                    that.prev();
-                }
-                return false;
-            }
+            icon: 'fa-chevron-left',
+            click: that.prev
         }).appendTo(buttons);
 
         buttons.append(' ');
@@ -619,12 +601,8 @@ IPA.hbac.test_run_facet = function(spec) {
         that.new_test_button = IPA.button({
             name: 'new_test',
             label: '@i18n:objects.hbactest.new_test',
-            click: function() {
-                if (!that.new_test_button.hasClass('action-button-disabled')) {
-                    that.new_test();
-                }
-                return false;
-            }
+            icon: 'fa-repeat',
+            click: that.new_test
         }).appendTo(buttons);
     };
 
@@ -673,7 +651,7 @@ IPA.hbac.test_run_facet = function(spec) {
 
     that.run = function() {
 
-        var command = IPA.command({ method: 'hbactest' });
+        var command = rpc.command({ method: 'hbactest' });
 
         var options = {};
         var validation_results = {
@@ -773,10 +751,10 @@ IPA.hbac.validation_dialog = function(spec)  {
 
     that.validation_results = spec.validation_results;
 
-    that.create = function() {
+    that.create_content = function() {
 
         if (that.message) {
-            that.message_dialog_create();
+            that.message_dialog_create_content();
         }
 
         if (that.validation_results && that.validation_results.invalid_facets) {
