@@ -140,9 +140,8 @@ return {
                     name: 'certificate',
                     fields: [
                         {
-                            $type: 'certificate_status',
-                            name: 'certificate_status',
-                            label: '@i18n:objects.host.status'
+                            $type: 'certificate',
+                            name: 'usercertificate'
                         }
                     ]
                 },
@@ -265,14 +264,10 @@ return {
                     enable_cond: ['userpassword_w'],
                     show_cond: ['has_password']
                 },
-                'cert_view',
-                'cert_get',
-                'cert_request',
-                'cert_revoke',
-                'cert_restore'
+                'cert_request'
             ],
             header_actions: ['automember_rebuild', 'unprovision', 'set_otp', 'reset_otp',
-                'request_cert', 'view_cert', 'get_cert', 'revoke_cert', 'restore_cert'],
+                'request_cert'],
             state: {
                 evaluators: [
                     IPA.host.has_password_evaluator,
@@ -912,7 +907,11 @@ IPA.host.certificate_policy = function(spec) {
         return values ? values[0] : null;
     };
 
-    spec.get_hostname = spec.get_hostname || spec.get_name;
+    spec.get_cn = spec.get_cn || spec.get_name;
+
+    spec.get_cn_name = spec.get_cn_name || function(result) {
+        return "hostname";
+    };
 
     var that = IPA.cert.load_policy(spec);
     return that;
