@@ -53,33 +53,41 @@ return {
                     fields: [
                         'cn',
                         'iparangetype',
+                        {
+                            name: 'iparangetyperaw',
+                            read_only: true,
+                            visible: false
+                        },
                         {
                             name: 'ipabaseid',
                             label: '@i18n:objects.idrange.ipabaseid',
-                            tooltip: '@mo-param:idrange:ipabaseid:label'
+                            title: '@mo-param:idrange:ipabaseid:label'
                         },
                         {
                             name: 'ipaidrangesize',
                             label: '@i18n:objects.idrange.ipaidrangesize',
-                            tooltip: '@mo-param:idrange:ipaidrangesize:label'
+                            title: '@mo-param:idrange:ipaidrangesize:label'
                         },
                         {
                             name: 'ipabaserid',
                             label: '@i18n:objects.idrange.ipabaserid',
-                            tooltip: '@mo-param:idrange:ipabaserid:label'
+                            title: '@mo-param:idrange:ipabaserid:label'
                         },
                         {
                             name: 'ipasecondarybaserid',
                             label: '@i18n:objects.idrange.ipasecondarybaserid',
-                            tooltip: '@mo-param:idrange:ipasecondarybaserid:label'
+                            title: '@mo-param:idrange:ipasecondarybaserid:label'
                         },
                         {
                             name: 'ipanttrusteddomainsid',
                             label: '@i18n:objects.idrange.ipanttrusteddomainsid',
-                            tooltip: '@mo-param:idrange:ipanttrusteddomainsid:label'
+                            title: '@mo-param:idrange:ipanttrusteddomainsid:label'
                         }
                     ]
                 }
+            ],
+            policies: [
+                exp.idrange_policy
             ]
         }
     ],
@@ -88,21 +96,6 @@ return {
             {
                 name: 'cn'
             },
-            {
-                name: 'ipabaseid',
-                label: '@i18n:objects.idrange.ipabaseid',
-                tooltip: '@mo-param:idrange:ipabaseid:label'
-            },
-            {
-                name: 'ipaidrangesize',
-                label: '@i18n:objects.idrange.ipaidrangesize',
-                tooltip: '@mo-param:idrange:ipaidrangesize:label'
-            },
-            {
-                name: 'ipabaserid',
-                label: '@i18n:objects.idrange.ipabaserid',
-                tooltip: '@mo-param:idrange:ipabaserid:label'
-            },
             {
                 name: 'iparangetype',
                 $type: 'radio',
@@ -121,26 +114,31 @@ return {
                     {
                         value: 'ipa-ad-trust-posix',
                         label: '@i18n:objects.idrange.type_ad_posix'
-                    },
-                    {
-                        value: 'ipa-ad-winsync',
-                        label: '@i18n:objects.idrange.type_winsync'
-                    },
-                    {
-                        value: 'ipa-ipa-trust',
-                        label: '@i18n:objects.idrange.type_ipa'
                     }
                 ]
             },
             {
-                name: 'ipasecondarybaserid',
-                label: '@i18n:objects.idrange.ipasecondarybaserid',
-                tooltip: '@mo-param:idrange:ipasecondarybaserid:label'
+                name: 'ipabaseid',
+                label: '@i18n:objects.idrange.ipabaseid',
+                title: '@mo-param:idrange:ipabaseid:label'
             },
             {
-                name: 'ipanttrusteddomainsid',
-                label: '@i18n:objects.idrange.ipanttrusteddomainsid',
-                tooltip: '@mo-param:idrange:ipanttrusteddomainsid:label',
+                name: 'ipaidrangesize',
+                label: '@i18n:objects.idrange.ipaidrangesize',
+                title: '@mo-param:idrange:ipaidrangesize:label'
+            },
+            {
+                name: 'ipabaserid',
+                label: '@i18n:objects.idrange.ipabaserid',
+                title: '@mo-param:idrange:ipabaserid:label'
+            },
+            {
+                name: 'ipasecondarybaserid',
+                label: '@i18n:objects.idrange.ipasecondarybaserid',
+                title: '@mo-param:idrange:ipasecondarybaserid:label'
+            },
+            {
+                name: 'ipanttrusteddomainname',
                 enabled: false
             }
         ],
@@ -153,12 +151,14 @@ return {
 IPA.idrange_adder_policy = function(spec) {
     /*
     The logic for enabling/requiring ipabaserid, ipasecondarybaserid and
-    ipanttrusteddomainsid is as follows:
+    ipanttrusteddomainname is as follows:
         1) for AD ranges (range type is ipa-ad-trust or ipa-ad-trust-posix):
-           * ipabaserid and ipanttrusteddomainsid are requred
+           * ipanttrusteddomainname is required
+           * ipabaserid is required for ipa-ad-trust but disabled for
+             ipa-ad-trust-posix
            * ipasecondarybaserid is disabled
         2) for local ranges
-           *  ipanttrusteddomainsid is disabled
+           *  ipanttrusteddomainname is disabled
            A) if server has AD trust support:
               * both ipabaserid and ipasecondarybaserid are required
            B) if server does not have AD trust support:
@@ -205,7 +205,7 @@ IPA.idrange_adder_policy = function(spec) {
         var type_f = that.container.fields.get_field('iparangetype');
         var baserid_f = that.container.fields.get_field('ipabaserid');
         var secondarybaserid_f = that.container.fields.get_field('ipasecondarybaserid');
-        var trusteddomainsid_f = that.container.fields.get_field('ipanttrusteddomainsid');
+        var trusteddomainname_f = that.container.fields.get_field('ipanttrusteddomainname');
 
         var type_v = type_f.get_value()[0];
         var baserid_v = baserid_f.get_value()[0] || '';
@@ -214,11 +214,15 @@ IPA.idrange_adder_policy = function(spec) {
         var is_ad_range = (type_v === 'ipa-ad-trust' || type_v === 'ipa-ad-trust-posix');
 
         if (is_ad_range) {
-            require(baserid_f);
-            require(trusteddomainsid_f);
+            if (type_v === 'ipa-ad-trust') {
+                require(baserid_f);
+            } else {
+                disable(baserid_f);
+            }
+            require(trusteddomainname_f);
             disable(secondarybaserid_f);
         } else {
-            disable(trusteddomainsid_f);
+            disable(trusteddomainname_f);
 
             if (IPA.trust_enabled) {
                 require(baserid_f);
@@ -238,6 +242,35 @@ IPA.idrange_adder_policy = function(spec) {
     return that;
 };
 
+exp.idrange_policy = function(spec) {
+
+    spec = spec || {};
+    var that = IPA.facet_policy(spec);
+
+    that.post_load = function() {
+        var type_f = that.container.fields.get_field('iparangetyperaw');
+        var widgets = that.container.widgets;
+        var type_v = type_f.get_value()[0];
+
+        var baserid = true;
+        var secrid = true;
+        var sid = true;
+
+        if (type_v === 'ipa-local') {
+            sid = false;
+        } else if (type_v === 'ipa-ad-trust-posix') {
+            baserid = secrid = false;
+        } else if (type_v === 'ipa-ad-trust') {
+            secrid = false;
+        }
+
+        widgets.get_widget('details.ipabaserid').set_visible(baserid);
+        widgets.get_widget('details.ipasecondarybaserid').set_visible(secrid);
+        widgets.get_widget('details.ipanttrusteddomainsid').set_visible(sid);
+    };
+    return that;
+};
+
 exp.entity_spec = make_spec();
 exp.register = function() {
     var e = reg.entity;
