@@ -32,6 +32,7 @@ define([
         './json2',
         './_base/i18n',
         './auth',
+        './config',
         './datetime',
         './metadata',
         './builder',
@@ -41,7 +42,8 @@ define([
         './util',
         'exports'
     ], function(declare, Deferred, Evented, keys, topic, $, JSON, i18n, auth,
-        datetime, metadata_provider, builder, reg, rpc, text, util, exports) {
+        config, datetime, metadata_provider, builder, reg, rpc, text,
+        util, exports) {
 
 /**
  * @class
@@ -127,11 +129,9 @@ var IPA = function () {
         // if current path matches live server path, use live data
         if (that.url && window.location.pathname.substring(0, that.url.length) === that.url) {
             that.json_url = params.url || '/ipa/session/json';
-            that.login_url = params.url || '/ipa/session/login_kerberos';
 
         } else { // otherwise use fixtures
             that.json_path = params.url || "test/data";
-            // that.login_url is not needed for fixtures
         }
 
         $.ajaxSetup(that.ajax_options);
@@ -220,6 +220,26 @@ var IPA = function () {
             }
         }));
 
+        batch.add_command(rpc.command({
+            entity: 'domainlevel',
+            method: 'get',
+            retry: false,
+            on_success: function(data, text_status, xhr) {
+                that.domain_level = data.result;
+            },
+            on_error: function(xhr, text_status, error_thrown) {
+                that.domain_level = 0;
+            }
+        }));
+
+        batch.add_command(rpc.command({
+            entity: 'ca',
+            method: 'is_enabled',
+            on_success: function(data, text_status, xhr) {
+                that.ca_enabled = data.result;
+            }
+        }));
+
         batch.execute();
     };
 
@@ -377,7 +397,7 @@ IPA.get_credentials = function() {
     }
 
     var request = {
-        url: IPA.login_url,
+        url: config.krb_login_url,
         cache: false,
         type: "GET",
         success: success_handler,
@@ -409,6 +429,7 @@ IPA.logout = function() {
     }
 
     function reload () {
+        window.sessionStorage.setItem('logout', true);
         var l = window.location;
         l.assign(l.href.split('#')[0]);
     }
@@ -754,8 +775,8 @@ IPA.dirty_dialog = function(spec) {
     };
 
     that.create_button({
-        name: 'update',
-        label: '@i18n:buttons.update',
+        name: 'save',
+        label: '@i18n:buttons.save',
         click: function() {
             that.facet.update(function() {
                 that.close();
@@ -765,8 +786,8 @@ IPA.dirty_dialog = function(spec) {
     });
 
     that.create_button({
-        name: 'reset',
-        label: '@i18n:buttons.reset',
+        name: 'revert',
+        label: '@i18n:buttons.revert',
         click: function() {
             that.facet.reset();
             that.close();
