@@ -36,7 +36,22 @@ var nav = {};
             name: 'identity',
             label: '@i18n:tabs.identity',
             children: [
-                { entity: 'user' },
+                {
+                    entity: 'user',
+                    facet: 'search',
+                    children: [
+                        {
+                            entity: 'stageuser',
+                            facet: 'search',
+                            hidden: true
+                        },
+                        {
+                            entity: 'user',
+                            facet: 'search_preserved',
+                            hidden: true
+                        }
+                    ]
+                },
                 { entity: 'group' },
                 { entity: 'host' },
                 { entity: 'hostgroup' },
@@ -108,7 +123,28 @@ var nav = {};
             name: 'authentication',
             label: '@i18n:tabs.authentication',
             children: [
-                { entity: 'cert', label: '@i18n:tabs.cert' },
+                {
+                    entity: 'cert',
+                    facet: 'search',
+                    label: '@i18n:tabs.cert',
+                    children: [
+                        {
+                            entity: 'certprofile',
+                            facet: 'search',
+                            hidden: true
+                        },
+                        {
+                            entity: 'cert',
+                            facet: 'search',
+                            hidden: true
+                        },
+                        {
+                            entity: 'caacl',
+                            facet: 'search',
+                            hidden: true
+                        }
+                    ]
+                },
                 { entity: 'otptoken' },
                 { entity: 'radiusproxy' }
             ]
@@ -159,6 +195,7 @@ var nav = {};
                     ]
                 },
                 { entity: 'idrange' },
+                { entity: 'idview' },
                 { entity: 'realmdomains' },
                 {
                     name: 'trusts',
@@ -168,6 +205,38 @@ var nav = {};
                         { entity: 'trustconfig' }
                     ]
                 },
+                {
+                    entity: 'topologysuffix',
+                    label: '@i18n:tabs.topology',
+                    facet: 'search',
+                    children: [
+                        {
+                            entity: 'topologysuffix',
+                            facet: 'search',
+                            hidden: true
+                        },
+                        {
+                            entity: 'server',
+                            facet: 'search',
+                            hidden: true
+                        },
+                        {
+                            entity: 'domainlevel',
+                            facet: 'details',
+                            hidden: true
+                        },
+                        {
+                            facet: 'topology-graph',
+                            hidden: true
+                        }
+                    ]
+                },
+                {
+                    name: 'apibrowser',
+                    label: 'API browser',
+                    facet: 'apibrowser',
+                    args: { 'type': 'command' }
+                },
                 { entity: 'config' }
             ]
         }
