@@ -221,6 +221,12 @@ return {
                     ]
                 }
             ]
+        },
+        {
+            $type: 'association',
+            name: 'managedby_user',
+            add_method: 'add_managedby',
+            remove_method: 'remove_managedby'
         }
     ],
 
