@@ -22,6 +22,7 @@
  */
 
 define([
+        './builder',
         './ipa',
         './jquery',
         './phases',
@@ -30,7 +31,7 @@ define([
         './spec_util',
         './text',
         './facet'],
-    function(IPA, $, phases, reg, rpc, su, text, mod_facet) {
+    function(builder, IPA, $, phases, reg, rpc, su, text, mod_facet) {
 
 var exp = {};
 
@@ -74,8 +75,8 @@ exp.search_facet_control_buttons_pre_op = function(spec, context) {
 
 exp.search_facet_pre_op = function(spec, context) {
 
-    var entity = context.entity;
     su.context_entity(spec, context);
+    var entity = reg.entity.get(spec.entity);
 
     spec.name = spec.name || 'search';
     spec.title = spec.title || entity.metadata.label;
@@ -93,13 +94,25 @@ exp.search_facet_pre_op = function(spec, context) {
     return spec;
 };
 
+/**
+ * Search facet
+ * @class  IPA.search_facet
+ */
 IPA.search_facet = function(spec, no_init) {
 
     spec = spec || {};
 
     var that = IPA.table_facet(spec, true);
 
-    that.deleter_dialog = spec.deleter_dialog || IPA.search_deleter_dialog;
+    /**
+     * Additional command options which are added to refresh command on
+     * refresh.
+     *
+     * @property {Object}
+     */
+    that.command_options = spec.command_options || {};
+
+    that.deleter_dialog = spec.deleter_dialog;
 
     that.create_header = function(container) {
 
@@ -179,10 +192,12 @@ IPA.search_facet = function(spec, no_init) {
             return null;
         }
 
-        var dialog = that.managed_entity.get_dialog('remove');
-
+        var dialog = builder.build('', that.deleter_dialog);
         if (!dialog) {
-            dialog = that.deleter_dialog();
+            dialog = that.managed_entity.get_dialog('remove');
+        }
+        if (!dialog) {
+            dialog = IPA.search_deleter_dialog();
         }
 
         dialog.entity_name = that.managed_entity.name;
@@ -232,6 +247,15 @@ IPA.search_facet = function(spec, no_init) {
         return args;
     };
 
+    /**
+     * Provide RPC options for refresh command
+     * - override point
+     */
+    that.get_refresh_command_options = function() {
+
+        return that.command_options;
+    };
+
     that.create_refresh_command = function() {
 
         var args = that.get_refresh_command_args();
@@ -243,6 +267,8 @@ IPA.search_facet = function(spec, no_init) {
             args: args
         });
 
+        command.set_options(that.get_refresh_command_options());
+
         if (that.pagination) {
             if (!that.search_all_entries) command.set_option('pkey_only', true);
             command.set_option('sizelimit', 0);
@@ -291,6 +317,7 @@ IPA.search_facet = function(spec, no_init) {
         that.init_facet();
         that.init_table_columns();
         that.init_table(that.managed_entity);
+        that.policies.init();
     };
 
     if (!no_init) that.init_search_facet();
@@ -374,8 +401,8 @@ IPA.search_deleter_dialog = function(spec) {
 
 exp.nested_search_facet_preop = function(spec, context) {
 
-    var entity = context.entity;
     su.context_entity(spec, context);
+    var entity = reg.entity.get(spec.entity);
 
     spec.name = spec.name || 'search';
     spec.title = spec.title || entity.metadata.label_singular;
@@ -430,6 +457,7 @@ IPA.batch_remove_action = function(spec) {
     spec.name = spec.name || 'remove';
     spec.label = spec.label || '@i18n:buttons.remove';
     spec.enable_cond = spec.enable_cond || ['item-selected'];
+    spec.enabled = spec.enabled === undefined ? false : spec.enabled;
     spec.hide_cond = spec.hide_cond || ['self-service'];
 
     var that = IPA.action(spec);
@@ -535,6 +563,7 @@ IPA.batch_disable_action = function(spec) {
     spec.method = spec.method || 'disable';
     spec.needs_confirm = spec.needs_confirm === undefined ? true : spec.needs_confirm;
     spec.enable_cond = spec.enable_cond || ['item-selected'];
+    spec.enabled = spec.enabled === undefined ? false : spec.enabled;
     spec.success_msg = spec.success_msg || '@i18n:search.disabled';
     spec.confirm_msg = spec.confirm_msg || '@i18n:search.disable_confirm';
 
@@ -548,6 +577,7 @@ IPA.batch_enable_action = function(spec) {
     spec.name = spec.name || 'enable';
     spec.method = spec.method || 'enable';
     spec.needs_confirm = spec.needs_confirm === undefined ? true : spec.needs_confirm;
+    spec.enabled = spec.enabled === undefined ? false : spec.enabled;
     spec.enable_cond = spec.enable_cond || ['item-selected'];
     spec.success_msg = spec.success_msg || '@i18n:search.enabled';
     spec.confirm_msg = spec.confirm_msg || '@i18n:search.enable_confirm';
@@ -571,7 +601,8 @@ exp.register = function() {
         factory: IPA.search_facet,
         pre_ops: [
             exp.search_facet_pre_op
-        ]
+        ],
+        spec: { name: 'search' }
     });
 
     f.register({
@@ -579,7 +610,8 @@ exp.register = function() {
         factory: IPA.nested_search_facet,
         pre_ops: [
             exp.nested_search_facet_preop
-        ]
+        ],
+        spec: { name: 'nestedsearch' }
     });
 };
 
