@@ -77,7 +77,16 @@ return {
                             $type: 'checkboxes',
                             name: 'ipauserauthtype',
                             flags: ['w_if_no_aci'],
-                            options: ['password', 'radius', 'otp']
+                            options: [
+                                { label: '@i18n:authtype.type_disabled', value: 'disabled' },
+                                { label: '@i18n:authtype.type_password', value: 'password' },
+                                { label: '@i18n:authtype.type_radius', value: 'radius' },
+                                { label: '@i18n:authtype.type_otp', value: 'otp' }
+                            ],
+                            tooltip: {
+                                title: '@i18n:authtype.config_tooltip',
+                                html: true
+                            }
                         },
                         {
                             $type: 'checkbox',
