@@ -112,6 +112,11 @@ return {
                             name: 'ipakrbokasdelegate',
                             $type: 'checkbox',
                             acl_param: 'krbticketflags'
+                        },
+                        {
+                            name: 'ipakrbrequirespreauth',
+                            $type: 'checkbox',
+                            acl_param: 'krbticketflags'
                         }
                     ]
                 },
@@ -129,9 +134,8 @@ return {
                     name: 'certificate',
                     fields: [
                         {
-                            $type: 'certificate_status',
-                            name: 'certificate_status',
-                            label: '@i18n:objects.service.status'
+                            $type: 'certificate',
+                            name: 'usercertificate'
                         }
                     ]
                 },
@@ -222,19 +226,11 @@ return {
             ],
             actions: [
                 'service_unprovision',
-                'cert_view',
-                'cert_get',
-                'cert_request',
-                'cert_revoke',
-                'cert_restore'
+                'cert_request'
             ],
             header_actions: [
                 'unprovision',
-                'request_cert',
-                'view_cert',
-                'get_cert',
-                'revoke_cert',
-                'restore_cert'
+                'request_cert'
             ],
             state: {
                 evaluators: [
@@ -549,7 +545,7 @@ IPA.service.certificate_policy = function(spec) {
 
     spec.get_principal = spec.get_principal || get_pkey;
 
-    spec.get_hostname = spec.get_hostname || function(result) {
+    spec.get_cn = spec.get_cn || function(result) {
         var value = get_pkey(result);
         if (value) {
             value = value.replace(/@.*$/, '').replace(/^.*\//, '');
@@ -557,6 +553,10 @@ IPA.service.certificate_policy = function(spec) {
         return value;
     };
 
+    spec.get_cn_name = spec.get_cn_name || function(result) {
+        return "hostname";
+    };
+
     var that = IPA.cert.load_policy(spec);
     return that;
 };
