@@ -171,6 +171,12 @@ return {
                 label: '@i18n:objects.trust.domain',
                 widget: 'realm.realm_server'
             },
+            {
+                $type: 'checkbox',
+                name: 'bidirectional',
+                metadata: '@mc-opt:trust_add:bidirectional',
+                widget: 'realm.bidirectional'
+            },
             {
                 name: 'realm_admin',
                 label: '@i18n:objects.trust.account',
@@ -221,10 +227,15 @@ return {
         ],
         widgets: [
             {
-                $type: 'details_table_section_nc',
+                $type: 'details_section',
                 name: 'realm',
                 widgets: [
-                    'realm_server'
+                    'realm_server',
+                    {
+                        $type: 'checkbox',
+                        name: 'bidirectional',
+                        tooltip: '@mc-opt:trust_add:bidirectional:doc'
+                    }
                 ]
             },
             {
@@ -265,7 +276,7 @@ return {
                 ]
             },
             {
-                $type: 'details_table_section_nc',
+                $type: 'details_section',
                 name: 'range',
                 widgets: [
                     {
