@@ -29,13 +29,14 @@ define([
         './reg',
         './rpc',
         './text',
+        './dialog',
         './dialogs/password',
         './details',
         './search',
         './association',
         './entity',
         './certificate'],
-    function(builder, IPA, $, phases, reg, rpc, text, password_dialog) {
+    function(builder, IPA, $, phases, reg, rpc, text, dialogs) {
 
 /**
  * User module
@@ -43,14 +44,33 @@ define([
  * @alternateClassName IPA.user
  * @singleton
  */
-var exp = IPA.user = {};
+var exp = IPA.user = {
+    search_facet_group: {
+        name: 'search',
+        label: '@i18n:objects.stageuser.user_categories',
+        facets: {
+            search_normal: 'user_search',
+            search: 'stageuser_search',
+            search_preserved: 'user_search_preserved'
+        }
+    }
+};
 
 var make_spec = function() {
 return {
     name: 'user',
+    policies: [
+        IPA.search_facet_update_policy,
+        IPA.details_facet_update_policy
+    ],
     facets: [
         {
             $type: 'search',
+            label: '@i18n:objects.user.activeuser_label',
+            tab_label: '@i18n:objects.user.activeuser_label',
+            disable_facet_tabs: false,
+            tabs_in_sidebar: true,
+            facet_groups: [exp.search_facet_group],
             row_disabled_attribute: 'nsaccountlock',
             columns: [
                 'uid',
@@ -97,7 +117,13 @@ return {
                     label: '@i18n:buttons.enable',
                     icon: 'fa-check'
                 }
-            ]
+            ],
+            policies: [
+                IPA.user.stageuser_sidebar_policy
+            ],
+            deleter_dialog: {
+                $factory: IPA.user.deleter_dialog
+            }
         },
         {
             $type: 'details',
@@ -125,7 +151,7 @@ return {
                     fields: [
                         'uid',
                         {
-                            $factory: IPA.user_password_widget,
+                            $factory: IPA.user.password_widget,
                             name: 'has_password',
                             metadata: '@mo-param:user:userpassword'
                         },
@@ -149,6 +175,10 @@ return {
                             name: 'ipasshpubkey',
                             label: '@i18n:objects.sshkeystore.keys'
                         },
+                        {
+                            $type: 'certificate',
+                            name: 'usercertificate'
+                        },
                         {
                             $type: 'checkboxes',
                             name: 'ipauserauthtype',
@@ -158,7 +188,10 @@ return {
                                 { label: '@i18n:authtype.type_radius', value: 'radius' },
                                 { label: '@i18n:authtype.type_otp', value: 'otp' }
                             ],
-                            tooltip: '@i18n:authtype.user_tooltip'
+                            tooltip: {
+                                title: '@i18n:authtype.user_tooltip',
+                                html: true
+                            }
                         },
                         {
                             $type: 'entity_select',
@@ -280,26 +313,51 @@ return {
                 }
             ],
             actions: [
-                'add_otptoken',
-                'enable',
-                'disable',
+                {
+                    $type: 'add_otptoken',
+                    hide_cond: ['preserved-user']
+                },
+                {
+                    $type: 'enable',
+                    hide_cond: ['preserved-user']
+                },
+                {
+                    $type: 'disable',
+                    hide_cond: ['preserved-user']
+                },
+                {
+                    $type: 'enable',
+                    hide_cond: ['preserved-user']
+                },
                 'delete',
-                'reset_password',
+                {
+                    $type: 'reset_password',
+                    hide_cond: ['preserved-user']
+                },
                 {
                     $factory: IPA.object_action,
                     name: 'unlock',
                     method: 'unlock',
                     label: '@i18n:objects.user.unlock',
                     needs_confirm: true,
+                    hide_cond: ['preserved-user'],
                     confirm_msg: '@i18n:objects.user.unlock_confirm'
                 },
                 {
                     $type: 'automember_rebuild',
                     name: 'automember_rebuild',
+                    hide_cond: ['preserved-user'],
                     label: '@i18n:actions.automember_rebuild'
+                },
+                {
+                    $type: 'cert_request',
+                    hide_cond: ['preserved-user']
                 }
             ],
-            header_actions: ['reset_password', 'enable', 'disable', 'delete', 'unlock', 'add_otptoken', 'automember_rebuild'],
+            header_actions: [
+                'reset_password', 'enable', 'disable', 'delete',
+                'unlock', 'add_otptoken', 'automember_rebuild', 'request_cert'
+            ],
             state: {
                 evaluators: [
                     {
@@ -314,13 +372,25 @@ return {
                         adapter: { $type: 'batch', result_index: 0 },
                         attribute: 'userpassword'
                     },
-                    IPA.user.self_service_other_user_evaluator
+                    IPA.user.self_service_other_user_evaluator,
+                    IPA.user.preserved_user_evaluator,
+                    IPA.cert.certificate_evaluator
                 ],
                 summary_conditions: [
+                    {
+                        pos: ['preserved-user'],
+                        neg: [],
+                        state: ['preserved'],
+                        description: 'Preserved user'
+                    },
                     IPA.enabled_summary_cond,
                     IPA.disabled_summary_cond
                 ]
-            }
+            },
+            policies: [
+                IPA.user.preserved_user_policy,
+                IPA.user.certificate_policy
+            ]
         },
         {
             $type: 'association',
@@ -361,7 +431,7 @@ return {
         $pre_ops: [ IPA.user.association_facet_ss_pre_op ]
     },
     adder_dialog: {
-        $factory: IPA.user_adder_dialog,
+        $factory: IPA.user.adder_dialog,
         sections: [
             {
                 fields: [
@@ -371,7 +441,23 @@ return {
                     },
                     'givenname',
                     'sn',
-                    'userclass'
+                    'userclass',
+                    {
+                        $type: 'checkbox',
+                        name: 'noprivate',
+                        label: '@i18n:objects.user.noprivate',
+                        metadata: '@mc-opt:user_add:noprivate'
+                    },
+                    {
+                        $type: 'entity_select',
+                        name: 'gidnumber',
+                        editable: true,
+                        searchable: true,
+                        other_entity: 'group',
+                        other_field: 'gidnumber',
+                        label_field: 'cn',
+                        filter_options: {'posix': true}
+                    }
                 ]
             },
             {
@@ -392,11 +478,13 @@ return {
     }
 };};
 
-IPA.user.details_facet = function(spec) {
+IPA.user.details_facet = function(spec, no_init) {
 
     spec = spec || {};
 
-    var that = IPA.details_facet(spec);
+    var that = IPA.details_facet(spec, true);
+    that.certificate_loaded = IPA.observer();
+    that.certificate_updated = IPA.observer();
 
     that.create_refresh_command = function() {
 
@@ -454,6 +542,8 @@ IPA.user.details_facet = function(spec) {
         return batch;
     };
 
+    if (!no_init) that.init_details_facet();
+
     return that;
 };
 
@@ -472,7 +562,7 @@ IPA.user.association_facet_ss_pre_op = function(spec, context) {
 };
 
 
-IPA.user_adder_dialog = function(spec) {
+IPA.user.adder_dialog = function(spec) {
 
     var that = IPA.entity_adder_dialog(spec);
 
@@ -501,7 +591,7 @@ IPA.user_adder_dialog = function(spec) {
     return that;
 };
 
-IPA.user_password_widget = function(spec) {
+IPA.user.password_widget = function(spec) {
 
     spec = spec || {};
     spec.read_only = true;
@@ -562,7 +652,7 @@ IPA.user.password_dialog_pre_op = function(spec) {
 
 IPA.user.password_dialog = function(spec) {
 
-    var that = password_dialog.dialog(spec);
+    var that = dialogs.command_dialog(spec);
 
     that.is_self_service = function() {
         var self_service = that.args[0] === IPA.whoami.uid[0];
@@ -644,6 +734,28 @@ IPA.user.add_otptoken_action = function(spec) {
     return that;
 };
 
+IPA.user.certificate_policy = function(spec) {
+
+    spec = spec || {};
+
+    function get_pkey(result) {
+        var values = result.uid;
+        return values ? values[0] : null;
+    }
+
+    spec.adapter = { result_index: 0 };
+    spec.get_pkey = spec.get_pkey || get_pkey;
+    spec.get_name = spec.get_name || get_pkey;
+    spec.get_principal = spec.get_principal || get_pkey;
+    spec.get_cn = spec.get_cn || get_pkey;
+    spec.get_cn_name = spec.get_cn_name || function(result) {
+        return "uid";
+    };
+
+    var that = IPA.cert.load_policy(spec);
+    return that;
+};
+
 IPA.user.self_service_other_user_evaluator = function(spec) {
 
     spec = spec || {};
@@ -673,6 +785,136 @@ IPA.user.self_service_other_user_evaluator = function(spec) {
     return that;
 };
 
+/**
+ * Evaluates if user is "preserved" user
+ * @class IPA.user.preserved_user_evaluator
+ */
+IPA.user.preserved_user_evaluator = function(spec) {
+
+    spec = spec || {};
+    spec.event = spec.event || 'post_load';
+
+    var that = IPA.state_evaluator(spec);
+    that.name = spec.name || 'preserved_user_evaluator';
+    that.param = spec.param || 'dn';
+    that.adapter = builder.build('adapter', { $type: 'adapter'}, { context: that });
+
+    /**
+     * Evaluates if user is preserved, i.e. is in provisioning tree
+     */
+    that.on_event = function(data) {
+
+        var old_state = that.state;
+        that.state = [];
+
+        var dn = that.adapter.load(data)[0];
+        if (dn.indexOf('cn=provisioning') > 0) {
+            that.state.push('preserved-user');
+        }
+
+        that.notify_on_change(old_state);
+    };
+
+    return that;
+};
+
+/**
+ * Change breadcrumb navigation and therefore also target facet on first
+ * navigation item based on user state (active/preserved)
+ * @class
+ */
+IPA.user.preserved_user_policy = function(spec) {
+
+    var that = IPA.facet_policy(spec);
+    that.post_load = function(data) {
+        var adapter = builder.build('adapter', {
+            $type: 'adapter',
+            result_index: 0,
+            context: { param: 'dn' }
+        });
+        var dn = adapter.load(data)[0];
+        var preserved_user = dn.indexOf('cn=provisioning') > 0;
+        var details_facet = that.container;
+        details_facet.set_tabs_visible(!preserved_user);
+        details_facet.redirect_info = { entity: 'user', facet: 'search' };
+        if (preserved_user) {
+            details_facet.redirect_info.facet = 'search_preserved';
+        }
+        details_facet.header.update_breadcrumb();
+    };
+
+    return that;
+};
+
+
+/**
+ * Display sidebar (facet tabs) only if user can view stage and preserved user.
+ * Atm. the sidebar is hidden only in self-service. Should be extended by a
+ * check if user can actually read it.
+ * @class
+ */
+IPA.user.stageuser_sidebar_policy = function(spec) {
+
+    var that = IPA.facet_policy(spec);
+
+    that.post_create = function(data) {
+        that.container.set_tabs_visible(!IPA.is_selfservice);
+    };
+
+    return that;
+};
+
+IPA.user.deleter_dialog = function(spec) {
+
+    spec = spec || {};
+
+    var that = IPA.search_deleter_dialog(spec);
+
+    /**
+     * Adds options to user-del command
+     * @type {IPA.radio_widget}
+     */
+    that.option_radio = null;
+
+    that.create_content = function() {
+
+        that.deleter_dialog_create_content();
+
+        that.option_layout = IPA.fluid_layout({
+            label_cls: 'col-sm-3',
+            widget_cls: 'col-sm-9'
+        });
+
+        that.option_radio = IPA.radio_widget({
+            name: 'preserve',
+            label: '@i18n:objects.user.delete_mode',
+            options: [
+                { label: '@i18n:objects.user.mode_delete', value: 'false' },
+                { label: '@i18n:objects.user.mode_preserve', value: 'true' }
+            ],
+            default_value: 'false'
+        });
+
+        var html = that.option_layout.create([that.option_radio]);
+        that.container.append(html);
+        that.option_radio.set_value(['']);
+    };
+
+    that.create_command = function() {
+        var batch = that.search_deleter_dialog_create_command();
+        var preserve = that.option_radio.get_value()[0];
+
+        for (var i=0; i<batch.commands.length; i++) {
+            var command = batch.commands[i];
+            command.set_option('preserve', preserve);
+        }
+
+        return batch;
+    };
+
+    return that;
+};
+
 exp.entity_spec = make_spec();
 exp.register = function() {
     var e = reg.entity;
