@@ -292,7 +292,7 @@ does that, and the backend plugins are only installed on the server.  The
 ``user_add.execute()`` method, which is only called when in a server context,
 is implemented as a series of calls to methods on the ``ldap`` backend plugin.
 
-When `plugable.Plugin.set_api()` is called, each plugin stores a reference to
+When `plugable.Plugin.__init__()` is called, each plugin stores a reference to
 the `plugable.API` instance it has been loaded into.  So your plugin can
 access the ``my_backend`` plugin as ``self.api.Backend.my_backend``.
 
@@ -668,7 +668,7 @@ See the `ipalib.cli.textui` plugin for a description of its methods.
 Logging from your plugin
 ------------------------
 
-After `plugable.Plugin.set_api()` is called, your plugin will have a
+After `plugable.Plugin.__init__()` is called, your plugin will have a
 ``self.log`` attribute.  Plugins should only log through this attribute.
 For example:
 
@@ -711,7 +711,7 @@ assume these variables will all exist by the time the module containing their
 plugin (or plugins) is imported.
 
 `Env._bootstrap()`, which is called by `API.bootstrap()`, will create several
-run-time variables that connot be overriden in configuration files or through
+run-time variables that cannot be overridden in configuration files or through
 command-line options.  Here is an overview of this run-time information:
 
 =============  =============================  =======================
@@ -880,16 +880,16 @@ freeIPA.org:
 '''
 
 import os
-import plugable
-from backend import Backend
-from frontend import Command, LocalOrRemote, Updater, Advice
-from frontend import Object, Method
-from crud import Create, Retrieve, Update, Delete, Search
-from parameters import DefaultFrom, Bool, Flag, Int, Decimal, Bytes, Str, IA5Str, Password, DNParam, DeprecatedParam
-from parameters import (BytesEnum, StrEnum, IntEnum, AccessTime, File,
+from ipalib import plugable
+from ipalib.backend import Backend
+from ipalib.frontend import Command, LocalOrRemote, Updater
+from ipalib.frontend import Object, Method
+from ipalib.crud import Create, Retrieve, Update, Delete, Search
+from ipalib.parameters import DefaultFrom, Bool, Flag, Int, Decimal, Bytes, Str, IA5Str, Password, DNParam, DeprecatedParam
+from ipalib.parameters import (BytesEnum, StrEnum, IntEnum, AccessTime, File,
                         DateTime, DNSNameParam)
-from errors import SkipPluginModule
-from text import _, ngettext, GettextFactory, NGettextFactory
+from ipalib.errors import SkipPluginModule
+from ipalib.text import _, ngettext, GettextFactory, NGettextFactory
 
 version_info = (2, 0, 0, 'alpha', 0)
 if version_info[3] == 'final':
@@ -898,6 +898,19 @@ else:
     __version__ = '%d.%d.%d.%s.%d' % version_info
 
 
+class API(plugable.API):
+    bases = (Command, Object, Method, Backend, Updater)
+
+    @property
+    def modules(self):
+        result = ('ipalib.plugins.*',)
+        if self.env.in_server:
+            result += ('ipaserver.plugins.*',)
+        if self.env.context in ('installer', 'updates'):
+            result += ('ipaserver.install.plugins.*',)
+        return result
+
+
 def create_api(mode='dummy'):
     """
     Return standard `plugable.API` instance.
@@ -911,11 +924,9 @@ def create_api(mode='dummy'):
 
         - `frontend.Method`
 
-        - `frontend.Advice`
-
         - `backend.Backend`
     """
-    api = plugable.API(Command, Object, Method, Backend, Updater, Advice)
+    api = API()
     if mode is not None:
         api.env.mode = mode
     assert mode != 'production'
@@ -924,8 +935,8 @@ def create_api(mode='dummy'):
 api = create_api(mode=None)
 
 if os.environ.get('IPA_UNIT_TEST_MODE', None) == 'cli_test':
-    from cli import cli_plugins
-    for klass in cli_plugins:
-        api.register(klass)
+    from ipalib.cli import cli_plugins
     api.bootstrap(context='cli', in_server=False, in_tree=True)
+    for klass in cli_plugins:
+        api.add_plugin(klass)
     api.finalize()
