@@ -20,39 +20,43 @@
 """
 Functionality for Command Line Interface.
 """
+from __future__ import print_function
 
-import re
 import textwrap
 import sys
 import getpass
 import code
 import optparse
-import socket
 import fcntl
 import termios
 import struct
 import base64
-try:
-    #pylint: disable=F0401
-    import default_encoding_utf8
-except ImportError:
-    # This is a chicken-and-egg problem. The api can't be imported unless
-    # this is already installed and since it is installed with IPA therein
-    # lies the problem. Skip it for now so ipalib can be imported in-tree
-    # even in cases that IPA isn't installed on the dev machine.
-    pass
+import traceback
 
-import frontend
-import backend
-import plugable
-import util
-from errors import (PublicError, CommandError, HelpError, InternalError,
-        NoSuchNamespaceError, ValidationError, NotFound, NotConfiguredError,
-        PromptFailed, ConversionError)
-from constants import CLI_TAB
-from parameters import Password, Bytes, File, Str, StrEnum, Any
-from text import _
+import six
+from six.moves import input
+
+if six.PY3:
+    unicode = str
+
+if six.PY2:
+    reload(sys)                         # pylint: disable=reload-builtin
+    sys.setdefaultencoding('utf-8')     # pylint: disable=no-member
+
+from ipalib import frontend
+from ipalib import backend
+from ipalib import plugable
+from ipalib.errors import (PublicError, CommandError, HelpError, InternalError,
+                           NoSuchNamespaceError, ValidationError, NotFound,
+                           NotConfiguredError, PromptFailed)
+from ipalib.constants import CLI_TAB, LDAP_GENERALIZED_TIME_FORMAT
+from ipalib.parameters import File, Str, Enum, Any, Flag
+from ipalib.text import _
+from ipalib import api
 from ipapython.version import API_VERSION
+from ipapython.dnsutil import DNSName
+
+import datetime
 
 
 def to_cli(name):
@@ -99,7 +103,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> rows = [
         ...     ('a', 'package'),
         ...     ('an', 'egg'),
@@ -127,24 +131,31 @@ class textui(backend.Backend):
             return 'UTF-8'
         return stream.encoding
 
-    def decode(self, value):
-        """
-        Decode text from stdin.
-        """
-        if type(value) is str:
-            encoding = self.__get_encoding(sys.stdin)
-            return value.decode(encoding)
-        elif type(value) in (list, tuple):
-            return tuple(self.decode(v) for v in value)
-        return value
+    if six.PY2:
+        def decode(self, value):
+            """
+            Decode text from stdin.
+            """
+            if type(value) is bytes:
+                encoding = self.__get_encoding(sys.stdin)
+                return value.decode(encoding)
+            elif type(value) in (list, tuple):
+                return tuple(self.decode(v) for v in value)
+            return value
 
-    def encode(self, unicode_text):
-        """
-        Encode text for output to stdout.
-        """
-        assert type(unicode_text) is unicode
-        encoding = self.__get_encoding(sys.stdout)
-        return unicode_text.encode(encoding)
+        def encode(self, unicode_text):
+            """
+            Encode text for output to stdout.
+            """
+            assert type(unicode_text) is unicode
+            encoding = self.__get_encoding(sys.stdout)
+            return unicode_text.encode(encoding)
+    else:
+        def decode(self, value):
+            return value
+
+        def encode(self, value):
+            return value
 
     def choose_number(self, n, singular, plural=None):
         if n == 1 or plural is None:
@@ -156,8 +167,12 @@ class textui(backend.Backend):
         Convert a binary value to base64. We know a value is binary
         if it is a python str type, otherwise it is a plain string.
         """
-        if type(value) is str:
+        if type(value) is bytes:
             return base64.b64encode(value)
+        elif type(value) is datetime.datetime:
+            return value.strftime(LDAP_GENERALIZED_TIME_FORMAT)
+        elif isinstance(value, DNSName):
+            return unicode(value)
         else:
             return value
 
@@ -165,7 +180,7 @@ class textui(backend.Backend):
         """
         Print exactly like ``print`` statement would.
         """
-        print unicode(string)
+        print(unicode(string))
 
     def print_line(self, text, width=None):
         """
@@ -173,7 +188,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_line('This line can fit!', width=18)
         This line can fit!
         >>> ui.print_line('This line wont quite fit!', width=18)
@@ -187,7 +202,7 @@ class textui(backend.Backend):
             width = self.get_tty_width()
         if width is not None and width < len(text):
             text = text[:width - 3] + '...'
-        print unicode(text)
+        print(unicode(text))
 
     def print_paragraph(self, text, width=None):
         """
@@ -199,7 +214,7 @@ class textui(backend.Backend):
         ... Python is a dynamic object-oriented programming language that can
         ... be used for many kinds of software development.
         ... '''
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_paragraph(text, width=45)
         Python is a dynamic object-oriented
         programming language that can be used for
@@ -216,7 +231,7 @@ class textui(backend.Backend):
         if width is None:
             width = self.get_tty_width()
         for line in textwrap.wrap(text.strip(), width):
-            print line
+            print(line)
 
     def print_indented(self, text, indent=1):
         """
@@ -224,7 +239,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_indented('One indentation level.')
           One indentation level.
         >>> ui.print_indented('Two indentation levels.', indent=2)
@@ -232,7 +247,7 @@ class textui(backend.Backend):
         >>> ui.print_indented('No indentation.', indent=0)
         No indentation.
         """
-        print (CLI_TAB * indent + text)
+        print((CLI_TAB * indent + text))
 
     def print_keyval(self, rows, indent=1):
         """
@@ -244,7 +259,7 @@ class textui(backend.Backend):
         ...     ('in_server', True),
         ...     ('mode', u'production'),
         ... ]
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_keyval(items)
           in_server = True
           mode = u'production'
@@ -264,7 +279,7 @@ class textui(backend.Backend):
         For example:
 
         >>> attr = 'dn'
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_attribute(attr, u'dc=example,dc=com')
           dn: dc=example,dc=com
         >>> attr = 'objectClass'
@@ -274,7 +289,7 @@ class textui(backend.Backend):
           objectClass: top
           objectClass: someClass
         """
-        assert isinstance(attr, basestring)
+        assert isinstance(attr, six.string_types)
         if not isinstance(value, (list, tuple)):
             # single-value attribute
             self.print_indented(format % (attr, self.encode_binary(value)), indent)
@@ -284,7 +299,7 @@ class textui(backend.Backend):
                 for v in value:
                     self.print_indented(format % (attr, self.encode_binary(v)), indent)
             else:
-                value = map(lambda v: self.encode_binary(v), value)
+                value = [self.encode_binary(v) for v in value]
                 if len(value) > 0 and type(value[0]) in (list, tuple):
                     # This is where we print failed add/remove members
                     for l in value:
@@ -344,7 +359,7 @@ class textui(backend.Backend):
         first = True
         for entry in entries:
             if not first:
-                print ''
+                print('')
             first = False
             self.print_entry(entry, order, labels, flags, print_all, format, indent)
 
@@ -366,7 +381,8 @@ class textui(backend.Backend):
                 label = labels.get(key, key)
                 flag = flags.get(key, [])
                 value = entry[key]
-                if 'suppress_empty' in flag and value in [u'', '', [], None]:
+                if ('suppress_empty' in flag and
+                    value in [u'', '', (), [], None]):
                     continue
                 if isinstance(value, dict):
                     if frontend.entry_count(value) == 0:
@@ -401,7 +417,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_dashed('Dashed above and below.')
         -----------------------
         Dashed above and below.
@@ -413,7 +429,7 @@ class textui(backend.Backend):
         ------------------
         Only dashed above.
         """
-        assert isinstance(dash, basestring)
+        assert isinstance(dash, six.string_types)
         assert len(dash) == 1
         dashes = dash * len(string)
         if above:
@@ -428,7 +444,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_h1('A primary header')
         ================
         A primary header
@@ -442,7 +458,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_h2('A secondary header')
           ------------------
           A secondary header
@@ -458,7 +474,7 @@ class textui(backend.Backend):
         command.  For example, a hypothetical ``show_status`` command would
         output something like this:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_name('show_status')
         ------------
         show-status:
@@ -475,7 +491,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_summary('Added user "jdoe"')
         -----------------
         Added user "jdoe"
@@ -494,7 +510,7 @@ class textui(backend.Backend):
 
         For example:
 
-        >>> ui = textui()
+        >>> ui = textui(api)
         >>> ui.print_count(1, '%d goose', '%d geese')
         -------
         1 goose
@@ -515,9 +531,9 @@ class textui(backend.Backend):
         )
 
     def print_error(self, text):
-        print '  ** %s **' % unicode(text)
+        print('  ** %s **' % unicode(text))
 
-    def prompt_helper(self, prompt, label, prompt_func=raw_input):
+    def prompt_helper(self, prompt, label, prompt_func=input):
         """Prompt user for input
 
         Handles encoding the prompt and decoding the input.
@@ -526,7 +542,7 @@ class textui(backend.Backend):
         try:
             return self.decode(prompt_func(self.encode(prompt)))
         except (KeyboardInterrupt, EOFError):
-            print
+            print()
             raise PromptFailed(name=label)
 
     def print_prompt_attribute_error(self, attribute, error):
@@ -709,7 +725,7 @@ class help(frontend.Local):
         self._builtins = []
 
         # build help topics
-        for c in self.Command():
+        for c in self.api.Command():
             if c.NO_CLI:
                 continue
 
@@ -722,9 +738,14 @@ class help(frontend.Local):
                         self._topics[topic_name][2].append(c)
                     else:
                         m = '%s.%s' % (self._PLUGIN_BASE_MODULE, topic_name)
-                        doc = (
-                            unicode(_(sys.modules[m].__doc__)) or ''
-                        ).strip().split('\n', 1)[0]
+                        try:
+                            module = sys.modules[m]
+                        except KeyError:
+                            doc = ''
+                        else:
+                            doc = (
+                                unicode(_(module.__doc__)) or ''
+                            ).strip().split('\n', 1)[0]
                         self._topics[topic_name] = [doc, 0, [c]]
                     mcl = max((self._topics[topic_name][1], len(c.name)))
                     self._topics[topic_name][1] = mcl
@@ -749,9 +770,8 @@ class help(frontend.Local):
                 self._builtins.append(c)
 
         # compute maximum topic length
-        self._mtl = max(
-            len(s) for s in (self._topics.keys() + [c.name for c in self._builtins])
-        )
+        topics = list(self._topics) + [c.name for c in self._builtins]
+        self._mtl = max(len(s) for s in topics)
 
         super(help, self)._on_finalize()
 
@@ -788,7 +808,10 @@ class help(frontend.Local):
 
     def _writer(self, outfile):
         def writer(string=''):
-            print >> outfile, unicode(string)
+            try:
+                print(unicode(string), file=outfile)
+            except IOError:
+                pass
         return writer
 
     def print_topics(self, outfile):
@@ -864,19 +887,37 @@ class show_mappings(frontend.Command):
             out.append((param.cli_name, param.param_spec))
             mcl = max(mcl,len(param.cli_name))
         for item in out:
-            print to_cli(item[0]).ljust(mcl)+' : '+item[1]
+            print(to_cli(item[0]).ljust(mcl)+' : '+item[1])
 
 
 class console(frontend.Command):
-    """Start the IPA interactive Python console."""
+    """Start the IPA interactive Python console, or run a script.
 
+    An IPA API object is initialized and made available
+    in the `api` global variable.
+    """
+
+    takes_args = ('filename?',)
     has_output = tuple()
 
-    def run(self, **options):
-        code.interact(
-            '(Custom IPA interactive Python console)',
-            local=dict(api=self.api)
-        )
+    def run(self, filename=None, **options):
+        local = dict(api=self.api)
+        if filename:
+            try:
+                script = open(filename)
+            except IOError as e:
+                exit("%s: %s" % (e.filename, e.strerror))
+            try:
+                with script:
+                    exec(script, globals(), local)
+            except:
+                traceback.print_exc()
+                exit(1)
+        else:
+            code.interact(
+                '(Custom IPA interactive Python console)',
+                local=local
+            )
 
 
 class show_api(frontend.Command):
@@ -898,14 +939,14 @@ class show_api(frontend.Command):
         first = True
         for line in lines:
             if line[0] == 0 and not first:
-                print ''
+                print('')
             if first:
                 first = False
-            print '%s%s %r' % (
+            print('%s%s %r' % (
                 ' ' * line[0],
                 line[1].ljust(ml),
                 line[2],
-            )
+            ))
         if len(lines) == 1:
             s = '1 attribute shown.'
         else:
@@ -1024,8 +1065,8 @@ class cli(backend.Executioner):
         """
         if len(argv) == 0:
             self.Command.help(outfile=sys.stderr)
-            print >>sys.stderr
-            print >>sys.stderr, 'Error: Command not specified'
+            print(file=sys.stderr)
+            print('Error: Command not specified', file=sys.stderr)
             exit(2)
         (key, argv) = (argv[0], argv[1:])
         name = from_cli(key)
@@ -1050,7 +1091,6 @@ class cli(backend.Executioner):
             else:
                 for callback in callbacks:
                     callback(cmd, kw)
-        kw['version'] = API_VERSION
         self.load_files(cmd, kw)
         return kw
 
@@ -1091,7 +1131,7 @@ class cli(backend.Executioner):
         """
         Decode param values if appropriate.
         """
-        for (key, value) in kw.iteritems():
+        for (key, value) in kw.items():
             yield (key, self.Backend.textui.decode(value))
 
     def build_parser(self, cmd):
@@ -1100,7 +1140,18 @@ class cli(backend.Executioner):
             description=unicode(cmd.doc),
             formatter=IPAHelpFormatter(),
         )
+
         option_groups = {}
+
+        def _get_option_group(group_name):
+            """Get or create an option group for the given name"""
+            option_group = option_groups.get(group_name)
+            if option_group is None:
+                option_group = optparse.OptionGroup(parser, group_name)
+                parser.add_option_group(option_group)
+                option_groups[group_name] = option_group
+            return option_group
+
         for option in cmd.options():
             kw = dict(
                 dest=option.name,
@@ -1110,32 +1161,35 @@ class cli(backend.Executioner):
                 continue
             if option.password and self.env.interactive:
                 kw['action'] = 'store_true'
-            elif option.type is bool and option.autofill:
+            elif isinstance(option, Flag):
                 if option.default is True:
                     kw['action'] = 'store_false'
                 else:
                     kw['action'] = 'store_true'
-            elif isinstance(option, StrEnum):
-                kw['metavar'] = metavar=map(lambda x: str(x), option.values)
+            elif isinstance(option, Enum):
+                kw['metavar'] = list(str(x) for x in option.values)
             else:
-                kw['metavar'] = metavar=option.__class__.__name__.upper()
+                kw['metavar'] = option.__class__.__name__.upper()
 
+            cli_name = to_cli(option.cli_name)
+            option_names = ['--%s' % cli_name]
             if option.cli_short_name:
-                o = optparse.make_option('-%s' % option.cli_short_name, '--%s' % to_cli(option.cli_name), **kw)
+                option_names.append('-%s' % option.cli_short_name)
+            opt = optparse.make_option(*option_names, **kw)
+            if option.option_group is None:
+                parser.add_option(opt)
             else:
-                o = optparse.make_option('--%s' % to_cli(option.cli_name), **kw)
+                _get_option_group(option.option_group).add_option(opt)
 
-            if option.option_group is not None:
-                option_group = option_groups.get(option.option_group)
-                if option_group is None:
-                    option_group = optparse.OptionGroup(parser,
-                                                        option.option_group)
-                    parser.add_option_group(option_group)
-                    option_groups[option.option_group] = option_group
-
-                option_group.add_option(o)
-            else:
-                parser.add_option(o)
+            if option.deprecated_cli_aliases:
+                new_kw = dict(kw)
+                new_kw['help'] = _('Same as --%s') % cli_name
+                if isinstance(option, Enum):
+                    new_kw['metavar'] = 'VAL'
+                group = _get_option_group(unicode(_('Deprecated options')))
+                for alias in option.deprecated_cli_aliases:
+                    name = '--%s' % alias
+                    group.add_option(optparse.make_option(name, **new_kw))
 
         for arg in cmd.args():
             name = self.__get_arg_name(arg, format_name=False)
@@ -1237,23 +1291,25 @@ class cli(backend.Executioner):
                         f = open(fname, 'r')
                         raw = f.read()
                         f.close()
-                    except IOError, e:
+                    except IOError as e:
                         raise ValidationError(
                             name=to_cli(p.cli_name),
-                            error='%s: %s:' % (fname, e[1])
+                            error='%s: %s:' % (fname, e.args[1])
                         )
                 elif p.stdin_if_missing:
                     try:
                         raw = sys.stdin.read()
-                    except IOError, e:
+                    except IOError as e:
                         raise ValidationError(
-                            name=to_cli(p.cli_name), error=e[1]
+                            name=to_cli(p.cli_name), error=e.args[1]
                         )
-                if not raw:
-                        raise ValidationError(
-                            name=to_cli(p.cli_name), error=_('No file to read')
-                        )
-                kw[p.name] = self.Backend.textui.decode(raw)
+
+                if raw:
+                    kw[p.name] = self.Backend.textui.decode(raw)
+                elif p.required:
+                    raise ValidationError(
+                        name=to_cli(p.cli_name), error=_('No file to read')
+                    )
 
 
 class IPAHelpFormatter(optparse.IndentedHelpFormatter):
@@ -1285,18 +1341,17 @@ def run(api):
     try:
         (options, argv) = api.bootstrap_with_global_options(context='cli')
         for klass in cli_plugins:
-            api.register(klass)
-        api.load_plugins()
+            api.add_plugin(klass)
         api.finalize()
-        if not 'config_loaded' in api.env:
+        if not 'config_loaded' in api.env and not 'help' in argv:
             raise NotConfiguredError()
         sys.exit(api.Backend.cli.run(argv))
     except KeyboardInterrupt:
-        print ''
+        print('')
         api.log.info('operation aborted')
-    except PublicError, e:
+    except PublicError as e:
         error = e
-    except StandardError, e:
+    except Exception as e:
         api.log.exception('%s: %s', e.__class__.__name__, str(e))
         error = InternalError()
     if error is not None:
