@@ -29,18 +29,21 @@ of the process.
 For the per-request thread-local information, see `ipalib.request`.
 """
 
-from ConfigParser import RawConfigParser, ParsingError
-from types import NoneType
 import os
 from os import path
 import sys
-from socket import getfqdn
+
+import six
+from six.moves.urllib.parse import urlparse, urlunparse
+from six.moves.configparser import RawConfigParser, ParsingError
+
 from ipapython.dn import DN
+from ipalib.base import check_name
+from ipalib.constants import CONFIG_SECTION
+from ipalib.constants import OVERRIDE_ERROR, SET_ERROR, DEL_ERROR
 
-from base import check_name
-from constants import CONFIG_SECTION
-from constants import TYPE_ERROR, OVERRIDE_ERROR, SET_ERROR, DEL_ERROR
-
+if six.PY3:
+    unicode = str
 
 class Env(object):
     """
@@ -105,15 +108,11 @@ class Env(object):
     u'false'
 
     If an ``str`` value looks like an integer, it's automatically converted to
-    the ``int`` type.  Likewise, if an ``str`` value looks like a floating-point
-    number, it's automatically converted to the ``float`` type.  For example:
+    the ``int`` type.
 
     >>> env.lucky = '7'
     >>> env.lucky
     7
-    >>> env.three_halves = '1.5'
-    >>> env.three_halves
-    1.5
 
     Leading and trailing white-space is automatically stripped from ``str``
     values.  For example:
@@ -210,7 +209,7 @@ class Env(object):
         Prevent further changes to environment.
         """
         if self.__locked is True:
-            raise StandardError(
+            raise Exception(
                 '%s.__lock__() already called' % self.__class__.__name__
             )
         object.__setattr__(self, '_Env__locked', True)
@@ -243,9 +242,9 @@ class Env(object):
                 (self.__class__.__name__, key, self.__d[key], value)
             )
         assert not hasattr(self, key)
-        if isinstance(value, basestring):
+        if isinstance(value, six.string_types):
             value = value.strip()
-            if isinstance(value, str):
+            if isinstance(value, bytes):
                 value = value.decode('utf-8')
             m = {
                 'True': True,
@@ -257,14 +256,9 @@ class Env(object):
                 value = m[value]
             elif value.isdigit():
                 value = int(value)
-            elif key in ('basedn'):
+            elif key == 'basedn':
                 value = DN(value)
-            else:
-                try:
-                    value = float(value)
-                except (TypeError, ValueError):
-                    pass
-        assert type(value) in (unicode, int, float, bool, NoneType, DN)
+        assert type(value) in (unicode, int, float, bool, type(None), DN)
         object.__setattr__(self, key, value)
         self.__d[key] = value
 
@@ -337,7 +331,7 @@ class Env(object):
         :param kw: Variables provides as keyword arguments.
         """
         i = 0
-        for (key, value) in kw.iteritems():
+        for (key, value) in kw.items():
             if key not in self:
                 self[key] = value
                 i += 1
@@ -412,7 +406,7 @@ class Env(object):
 
     def __doing(self, name):
         if name in self.__done:
-            raise StandardError(
+            raise Exception(
                 '%s.%s() already called' % (self.__class__.__name__, name)
             )
         self.__done.add(name)
@@ -514,8 +508,8 @@ class Env(object):
                ``self.conf_default`` (if it exists) by calling
                `Env._merge_from_file()`.
 
-            4. Intelligently fill-in the *in_server* , *logdir*, and *log*
-               variables if they haven't already been set.
+            4. Intelligently fill-in the *in_server* , *logdir*, *log*, and
+               *jsonrpc_uri* variables if they haven't already been set.
 
             5. Merge-in the variables in ``defaults`` by calling `Env._merge()`.
                In normal circumstances ``defaults`` will simply be those
@@ -556,6 +550,19 @@ class Env(object):
         if 'log' not in self:
             self.log = self._join('logdir', '%s.log' % self.context)
 
+        # Derive jsonrpc_uri from xmlrpc_uri
+        if 'jsonrpc_uri' not in self:
+            if 'xmlrpc_uri' in self:
+                xmlrpc_uri = self.xmlrpc_uri
+            else:
+                xmlrpc_uri = defaults.get('xmlrpc_uri')
+            if xmlrpc_uri:
+                (scheme, netloc, uripath, params, query, fragment
+                        ) = urlparse(xmlrpc_uri)
+                uripath = uripath.replace('/xml', '/json', 1)
+                self.jsonrpc_uri = urlunparse((
+                        scheme, netloc, uripath, params, query, fragment))
+
         self._merge(**defaults)
 
     def _finalize(self, **lastchance):
