@@ -49,7 +49,7 @@ TYPE_ERROR = '%s: need a %r; got %r (a %r)'
 # Stardard format for TypeError message when a callable is expected:
 CALLABLE_ERROR = '%s: need a callable; got %r (which is a %r)'
 
-# Standard format for StandardError message when overriding an attribute:
+# Standard format for Exception message when overriding an attribute:
 OVERRIDE_ERROR = 'cannot override %s.%s value %r with %r'
 
 # Standard format for AttributeError message when a read-only attribute is
@@ -78,6 +78,8 @@ DEFAULT_CONFIG = (
     # LDAP containers:
     ('container_accounts', DN(('cn', 'accounts'))),
     ('container_user', DN(('cn', 'users'), ('cn', 'accounts'))),
+    ('container_deleteuser', DN(('cn', 'deleted users'), ('cn', 'accounts'), ('cn', 'provisioning'))),
+    ('container_stageuser',  DN(('cn', 'staged users'),  ('cn', 'accounts'), ('cn', 'provisioning'))),
     ('container_group', DN(('cn', 'groups'), ('cn', 'accounts'))),
     ('container_service', DN(('cn', 'services'), ('cn', 'accounts'))),
     ('container_host', DN(('cn', 'computers'), ('cn', 'accounts'))),
@@ -97,6 +99,7 @@ DEFAULT_CONFIG = (
     ('container_hbacservice', DN(('cn', 'hbacservices'), ('cn', 'hbac'))),
     ('container_hbacservicegroup', DN(('cn', 'hbacservicegroups'), ('cn', 'hbac'))),
     ('container_dns', DN(('cn', 'dns'))),
+    ('container_vault', DN(('cn', 'vaults'), ('cn', 'kra'))),
     ('container_virtual', DN(('cn', 'virtual operations'), ('cn', 'etc'))),
     ('container_sudorule', DN(('cn', 'sudorules'), ('cn', 'sudo'))),
     ('container_sudocmd', DN(('cn', 'sudocmds'), ('cn', 'sudo'))),
@@ -114,6 +117,10 @@ DEFAULT_CONFIG = (
     ('container_otp', DN(('cn', 'otp'))),
     ('container_radiusproxy', DN(('cn', 'radiusproxy'))),
     ('container_views', DN(('cn', 'views'), ('cn', 'accounts'))),
+    ('container_masters', DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'))),
+    ('container_certprofile', DN(('cn', 'certprofiles'), ('cn', 'ca'))),
+    ('container_topology', DN(('cn', 'topology'), ('cn', 'ipa'), ('cn', 'etc'))),
+    ('container_caacl', DN(('cn', 'caacls'), ('cn', 'ca'))),
 
     # Ports, hosts, and URIs:
     ('xmlrpc_uri', 'http://localhost:8888/ipa/xml'),
@@ -147,7 +154,7 @@ DEFAULT_CONFIG = (
     ('debug', False),
     ('startup_traceback', False),
     ('mode', 'production'),
-    ('wait_for_dns', False),
+    ('wait_for_dns', 0),
 
     # CA plugin:
     ('ca_host', FQDN),  # Set in Env._finalize_core()
@@ -160,6 +167,9 @@ DEFAULT_CONFIG = (
     ('ca_agent_install_port', None),
     ('ca_ee_install_port', None),
 
+    # Topology plugin
+    ('recommended_max_agmts', 4),  # Recommended maximum number of replication
+                                   # agreements
 
     # Special CLI:
     ('prompt_all', False),
@@ -175,6 +185,10 @@ DEFAULT_CONFIG = (
     # Used when verifying that the API hasn't changed. Not for production.
     ('validate_api', False),
 
+    # Skip client vs. server API version checking. Can lead to errors/strange
+    # behavior when newer clients talk to older servers. Use with caution.
+    ('skip_version_check', False),
+
     # ********************************************************
     #  The remaining keys are never set from the values here!
     # ********************************************************
@@ -219,3 +233,31 @@ LDAP_GENERALIZED_TIME_FORMAT = "%Y%m%d%H%M%SZ"
 
 IPA_ANCHOR_PREFIX = ':IPA:'
 SID_ANCHOR_PREFIX = ':SID:'
+
+# domains levels
+DOMAIN_LEVEL_0 = 0  # compat
+DOMAIN_LEVEL_1 = 1  # replica promotion, topology plugin
+
+MIN_DOMAIN_LEVEL = DOMAIN_LEVEL_0
+MAX_DOMAIN_LEVEL = DOMAIN_LEVEL_1
+
+# Constants used in generation of replication agreements and as topology
+# defaults
+
+# List of attributes that need to be excluded from replication initialization.
+REPL_AGMT_TOTAL_EXCLUDES = ('entryusn',
+                            'krblastsuccessfulauth',
+                            'krblastfailedauth',
+                            'krbloginfailedcount')
+
+# List of attributes that need to be excluded from normal replication.
+REPL_AGMT_EXCLUDES = ('memberof', 'idnssoaserial') + REPL_AGMT_TOTAL_EXCLUDES
+
+# List of attributes that are not updated on empty replication
+REPL_AGMT_STRIP_ATTRS = ('modifiersName',
+                         'modifyTimestamp',
+                         'internalModifiersName',
+                         'internalModifyTimestamp')
+
+DOMAIN_SUFFIX_NAME = 'domain'
+CA_SUFFIX_NAME = 'ca'
