@@ -102,13 +102,14 @@ current block assignments:
             - **5100 - 5999**  *Reserved for future use*
 """
 
-from ipalib.text import ngettext as ungettext
+import six
 
-import messages
+from ipalib.text import ngettext as ungettext
+from ipalib import messages
 from ipaplatform.paths import paths
 
 
-class PrivateError(StandardError):
+class PrivateError(Exception):
     """
     Base class for exceptions that are *never* forwarded in an RPC response.
     """
@@ -118,12 +119,17 @@ class PrivateError(StandardError):
     def __init__(self, **kw):
         self.msg = self.format % kw
         self.kw = kw
-        for (key, value) in kw.iteritems():
+        for (key, value) in kw.items():
             assert not hasattr(self, key), 'conflicting kwarg %s.%s = %r' % (
                 self.__class__.__name__, key, value,
             )
             setattr(self, key, value)
-        StandardError.__init__(self, self.msg)
+        Exception.__init__(self, self.msg)
+
+    if six.PY3:
+        @property
+        def message(self):
+            return str(self)
 
 
 class SubprocessError(PrivateError):
@@ -240,7 +246,7 @@ def _(message):
     return message
 
 
-class PublicError(StandardError):
+class PublicError(Exception):
     """
     **900** Base class for exceptions that can be forwarded in an RPC response.
     """
@@ -252,6 +258,11 @@ class PublicError(StandardError):
     rval = 1
     format = None
 
+    if six.PY3:
+        @property
+        def message(self):
+            return str(self)
+
 
 class VersionError(PublicError):
     """
@@ -1344,6 +1355,23 @@ class EmptyResult(NotFound):
 
     errno = 4031
 
+class InvalidDomainLevelError(ExecutionError):
+    """
+    **4032** Raised when a operation could not be completed due to a invalid
+             domain level.
+
+    For example:
+
+    >>> raise InvalidDomainLevelError(reason='feature requires domain level 4')
+    Traceback (most recent call last):
+      ...
+    InvalidDomainLevelError: feature requires domain level 4
+
+    """
+
+    errno = 4032
+    format = _('%(reason)s')
+
 class BuiltinError(ExecutionError):
     """
     **4100** Base class for builtin execution errors (*4100 - 4199*).
