@@ -23,20 +23,25 @@ Base classes for all front-end plugins.
 
 import re
 from distutils import version
+from textwrap import wrap
+
+import six
 
 from ipapython.version import API_VERSION
 from ipapython.ipa_log_manager import root_logger
-from base import NameSpace
-from plugable import Plugin, is_production_mode
-from parameters import create_param, Param, Str, Flag, Password
-from output import Output, Entry, ListOfEntries
-from text import _
-from errors import (ZeroArgumentError, MaxArgumentError, OverlapError,
+from ipalib.base import NameSpace
+from ipalib.plugable import Plugin
+from ipalib.parameters import create_param, Param, Str, Flag, Password
+from ipalib.output import Output, Entry, ListOfEntries
+from ipalib.text import _
+from ipalib.errors import (ZeroArgumentError, MaxArgumentError, OverlapError,
     VersionError, OptionError, InvocationError,
     ValidationError, ConversionError)
 from ipalib import messages
-from textwrap import wrap
+from ipalib.util import json_serialize
 
+if six.PY3:
+    unicode = str
 
 RULE_FLAG = 'validation_rule'
 
@@ -359,7 +364,7 @@ class HasParam(Plugin):
             self._filter_param_by_context(name, env),
             sort=False
         )
-        if not is_production_mode(self):
+        if not self.api.is_production_mode():
             check = getattr(self, 'check_' + name, None)
             if callable(check):
                 check(namespace)
@@ -422,6 +427,8 @@ class Command(HasParam):
         version_provided = 'version' in options
         if version_provided:
             self.verify_client_version(unicode(options['version']))
+        elif self.api.env.skip_version_check and not self.api.env.in_server:
+            options['version'] = u'2.0'
         else:
             options['version'] = API_VERSION
         params = self.args_options_2_params(*args, **options)
@@ -437,7 +444,9 @@ class Command(HasParam):
         self.validate(**params)
         (args, options) = self.params_2_args_options(**params)
         ret = self.run(*args, **options)
-        if not version_provided and isinstance(ret, dict):
+        if (not version_provided and isinstance(ret, dict) and
+                self.api.env.in_server):
+            # add message only on server side
             messages.add_message(
                 API_VERSION, ret,
                 messages.VersionMissing(server_version=API_VERSION))
@@ -457,7 +466,7 @@ class Command(HasParam):
             try:
                 value = values.get(p.name)
                 values[p.name] = p(value, **values)
-            except InvocationError, e:
+            except InvocationError as e:
                 errors[p.name] = str(e)
         return dict(
             values=values,
@@ -585,7 +594,7 @@ class Command(HasParam):
 
             try:
                 return param(raw, **kw)
-            except (ValidationError, ConversionError), e:
+            except (ValidationError, ConversionError) as e:
                 # Display error and prompt again
                 self.Backend.textui.print_prompt_attribute_error(unicode(label),
                                                              unicode(e.error))
@@ -608,7 +617,7 @@ class Command(HasParam):
         {'last': u'DOE', 'first': u'john'}
         """
         return dict(
-            (k, self.params[k].normalize(v)) for (k, v) in kw.iteritems()
+            (k, self.params[k].normalize(v)) for (k, v) in kw.items()
         )
 
     def convert(self, **kw):
@@ -628,7 +637,7 @@ class Command(HasParam):
         {'two': u'2', 'one': 1}
         """
         return dict(
-            (k, self.params[k].convert(v)) for (k, v) in kw.iteritems()
+            (k, self.params[k].convert(v)) for (k, v) in kw.items()
         )
 
     def __convert_iter(self, kw):
@@ -864,7 +873,7 @@ class Command(HasParam):
             if optional and arg.required:
                 raise ValueError(
                     '%s: required argument after optional in %s arguments %s' % (arg.name,
-                    self.name, map(lambda x: x.param_spec, args()))
+                    self.name, [x.param_spec for x in args()])
                 )
             if multivalue:
                 raise ValueError(
@@ -1044,31 +1053,16 @@ class Command(HasParam):
 
     # list of attributes we want exported to JSON
     json_friendly_attributes = (
-        'name', 'takes_args',
+        'name', 'doc', 'NO_CLI'
     )
 
-    # list of options we want only to mention their presence and not to write
-    # their attributes
-    json_only_presence_options = (
-        'all', 'raw', 'attrs', 'addattr', 'delattr', 'setattr', 'version',
-    )
-
-    def get_json_options(self):
-        """
-        Get only options we want exported to JSON
-        """
-        for option in self.get_options():
-            if option.name not in self.json_only_presence_options:
-                yield option
-            else:
-                yield { 'name': option.name }
-
     def __json__(self):
         json_dict = dict(
             (a, getattr(self, a)) for a in self.json_friendly_attributes
         )
 
-        json_dict['takes_options'] = list(self.get_json_options())
+        json_dict['takes_args'] = list(self.get_args())
+        json_dict['takes_options'] = list(self.get_options())
 
         return json_dict
 
@@ -1137,7 +1131,7 @@ class Object(HasParam):
             self.__get_attrs('Method'), sort=False, name_attr='attr_name'
         )
         self._create_param_namespace('params')
-        pkeys = filter(lambda p: p.primary_key, self.params())
+        pkeys = [p for p in self.params() if p.primary_key]
         if len(pkeys) > 1:
             raise ValueError(
                 '%s (Object) has multiple primary keys: %s' % (
@@ -1148,7 +1142,7 @@ class Object(HasParam):
         if len(pkeys) == 1:
             self.primary_key = pkeys[0]
             self.params_minus_pk = NameSpace(
-                filter(lambda p: not p.primary_key, self.params()), sort=False
+                [p for p in self.params() if not p.primary_key], sort=False
             )
         else:
             self.primary_key = None
@@ -1204,6 +1198,20 @@ class Object(HasParam):
                 return 1
             return 2
 
+    json_friendly_attributes = (
+        'name', 'takes_params',
+    )
+
+    def __json__(self):
+        json_dict = dict(
+            (a, json_serialize(getattr(self, a)))
+            for a in self.json_friendly_attributes
+        )
+        if self.primary_key:
+            json_dict['primary_key'] = self.primary_key.name
+        json_dict['methods'] = [m for m in self.methods]
+        return json_dict
+
 
 class Attribute(Plugin):
     """
@@ -1248,12 +1256,12 @@ class Attribute(Plugin):
     # Create stubs for attributes that are set in _on_finalize()
     __obj = Plugin.finalize_attr('_Attribute__obj')
 
-    def __init__(self):
+    def __init__(self, api):
         m = self.NAME_REGEX.match(type(self).__name__)
         assert m
         self.__obj_name = m.group('obj')
         self.__attr_name = m.group('attr')
-        super(Attribute, self).__init__()
+        super(Attribute, self).__init__(api)
 
     def __get_obj_name(self):
         return self.__obj_name
@@ -1345,9 +1353,6 @@ class Method(Attribute, Command):
     extra_options_first = False
     extra_args_first = False
 
-    def __init__(self):
-        super(Method, self).__init__()
-
     def get_output_params(self):
         for param in self.obj.params():
             if 'no_output' in param.flags:
@@ -1362,7 +1367,7 @@ class Method(Attribute, Command):
             yield param
 
 
-class Updater(Method):
+class Updater(Plugin):
     """
     An LDAP update with an associated object (always update).
 
@@ -1388,8 +1393,8 @@ class Updater(Method):
     >>> api.Updater.my_update # doctest:+ELLIPSIS
     ipalib.frontend.my_update()
     """
-    def __init__(self):
-        super(Updater, self).__init__()
+    def execute(self, **options):
+        raise NotImplementedError('%s.execute()' % self.name)
 
     def __call__(self, **options):
         self.debug(
@@ -1397,52 +1402,3 @@ class Updater(Method):
         )
 
         return self.execute(**options)
-
-
-class _AdviceOutput(object):
-
-    def __init__(self):
-        self.content = []
-        self.prefix = '# '
-        self.options = None
-
-    def comment(self, line, wrapped=True):
-        if wrapped:
-            for wrapped_line in wrap(line, 70):
-                self.content.append(self.prefix + wrapped_line)
-        else:
-            self.content.append(self.prefix + line)
-
-    def debug(self, line):
-        if self.options.verbose:
-            self.comment('DEBUG: ' + line)
-
-    def command(self, line):
-        self.content.append(line)
-
-
-class Advice(Plugin):
-    """
-    Base class for advices, plugins for ipa-advise.
-    """
-
-    options = None
-    require_root = False
-    description = ''
-
-    def __init__(self):
-        super(Advice, self).__init__()
-        self.log = _AdviceOutput()
-
-    def set_options(self, options):
-        self.options = options
-        self.log.options = options
-
-    def get_info(self):
-        """
-        This method should be overriden by child Advices.
-
-        Returns a string with instructions.
-        """
-
-        raise NotImplementedError
