@@ -29,14 +29,19 @@ range, so that it does not clash with PublicError numbers.
 Messages also have the 'type' argument, set to one of 'debug', 'info',
 'warning', 'error'. This determines the severity of themessage.
 """
+from __future__ import print_function
 
 from inspect import isclass
 
+import six
+
 from ipalib.constants import TYPE_ERROR
 from ipalib.text import _ as ugettext
 from ipalib.text import Gettext, NGettext
 from ipalib.capabilities import client_has_capability
 
+if six.PY3:
+    unicode = str
 
 def add_message(version, result, message):
     if client_has_capability(version, 'messages'):
@@ -60,14 +65,14 @@ def process_message_arguments(obj, format=None, message=None, **kw):
             obj.format = format
         obj.forwarded = False
         obj.msg = obj.format % kw
-        if isinstance(obj.format, basestring):
+        if isinstance(obj.format, six.string_types):
             obj.strerror = ugettext(obj.format) % kw
         else:
             obj.strerror = obj.format % kw
         if 'instructions' in kw:
             def convert_instructions(value):
                 if isinstance(value, list):
-                    result = u'\n'.join(map(lambda line: unicode(line), value))
+                    result = u'\n'.join(unicode(line) for line in value)
                     return result
                 return value
             instructions = u'\n'.join((unicode(_('Additional instructions:')),
@@ -83,7 +88,7 @@ def process_message_arguments(obj, format=None, message=None, **kw):
         obj.forwarded = True
         obj.msg = message
         obj.strerror = message
-    for (key, value) in kw.iteritems():
+    for (key, value) in kw.items():
         assert not hasattr(obj, key), 'conflicting kwarg %s.%s = %r' % (
             name, key, value,
         )
@@ -117,6 +122,11 @@ class PublicMessage(UserWarning):
             code=self.errno,
         )
 
+    if six.PY3:
+        @property
+        def message(self):
+            return str(self)
+
 
 class VersionMissing(PublicMessage):
     """
@@ -179,14 +189,14 @@ class OptionSemanticChangedWarning(PublicMessage):
                u"%(hint)s")
 
 
-class DNSServerNotRespondingWarning(PublicMessage):
+class DNSServerValidationWarning(PublicMessage):
     """
-    **13006**  Used when a DNS server is not responding to queries
+    **13006**  Used when a DNS server is not to able to resolve query
     """
 
     errno = 13006
     type = "warning"
-    format = _(u"DNS server %(server)s not responding.")
+    format = _(u"DNS server %(server)s: %(error)s.")
 
 
 class DNSServerDoesNotSupportDNSSECWarning(PublicMessage):
@@ -196,10 +206,11 @@ class DNSServerDoesNotSupportDNSSECWarning(PublicMessage):
 
     errno = 13007
     type = "warning"
-    format = _(u"DNS server %(server)s does not support DNSSEC. "
+    format = _(u"DNS server %(server)s does not support DNSSEC: %(error)s.\n"
                u"If DNSSEC validation is enabled on IPA server(s), "
                u"please disable it.")
 
+
 class ForwardzoneIsNotEffectiveWarning(PublicMessage):
     """
     **13008** Forwardzone is not effective, forwarding will not work because
@@ -214,6 +225,131 @@ class ForwardzoneIsNotEffectiveWarning(PublicMessage):
                u"\"%(ns_rec)s\" to parent zone \"%(authzone)s\".")
 
 
+class DNSServerDoesNotSupportEDNS0Warning(PublicMessage):
+    """
+    **13009** Used when a DNS server does not support EDNS0, required for
+    DNSSEC support
+    """
+
+    errno = 13009
+    type = "warning"
+    format = _(u"DNS server %(server)s does not support EDNS0 (RFC 6891): "
+               u"%(error)s.\n"
+               u"If DNSSEC validation is enabled on IPA server(s), "
+               u"please disable it.")
+
+
+class DNSSECValidationFailingWarning(PublicMessage):
+    """
+    **13010** Used when a DNSSEC validation failed on IPA DNS server
+    """
+
+    errno = 13010
+    type = "warning"
+    format = _(u"DNSSEC validation failed: %(error)s.\n"
+               u"Please verify your DNSSEC configuration or disable DNSSEC "
+               u"validation on all IPA servers.")
+
+
+class KerberosTXTRecordCreationFailure(PublicMessage):
+    """
+    **13011** Used when a _kerberos TXT record could not be added to
+    a DNS zone.
+    """
+
+    errno = 13011
+    type = "warning"
+    format = _(
+        "The _kerberos TXT record from domain %(domain)s could not be created "
+        "(%(error)s).\nThis can happen if the zone is not managed by IPA. "
+        "Please create the record manually, containing the following "
+        "value: '%(realm)s'"
+    )
+
+
+class KerberosTXTRecordDeletionFailure(PublicMessage):
+    """
+    **13012** Used when a _kerberos TXT record could not be removed from
+    a DNS zone.
+    """
+
+    errno = 13012
+    type = "warning"
+    format = _(
+        "The _kerberos TXT record from domain %(domain)s could not be removed "
+        "(%(error)s).\nThis can happen if the zone is not managed by IPA. "
+        "Please remove the record manually."
+    )
+
+class DNSSECMasterNotInstalled(PublicMessage):
+    """
+    **13013** Used when a DNSSEC is not installed on system (no DNSSEC
+    master server is installed).
+    """
+
+    errno = 13013
+    type = "warning"
+    format = _(
+        "No DNSSEC key master is installed. DNSSEC zone signing will not work "
+        "until the DNSSEC key master is installed."
+    )
+
+
+class DNSSuspiciousRelativeName(PublicMessage):
+    """
+    **13014** Relative name "record.zone" is being added into zone "zone.",
+    which is probably a mistake. User probably wanted to either specify
+    relative name "record" or use FQDN "record.zone.".
+    """
+
+    errno = 13014
+    type = "warning"
+    format = _(
+        "Relative record name '%(record)s' contains the zone name '%(zone)s' "
+        "as a suffix, which results in FQDN '%(fqdn)s'. This is usually a "
+        "mistake caused by a missing dot at the end of the name specification."
+    )
+
+
+class CommandDeprecatedWarning(PublicMessage):
+    """
+    **13015** Used when user uses a deprecated option
+    """
+
+    errno = 13015
+    type = "warning"
+    format = _(u"'%(command)s' is deprecated. %(additional_info)s")
+
+
+class ExternalCommandOutput(PublicMessage):
+    """
+    **13016** Line of output from an external command.
+    """
+
+    errno = 13016
+    type = "info"
+    format = _("%(line)s")
+
+
+class BrokenTrust(PublicMessage):
+    """
+    **13018** Trust for a specified domain is broken
+    """
+
+    errno = 13018
+    type = "warning"
+    format = _("Your trust to %(domain)s is broken. Please re-create it by "
+               "running 'ipa trust-add' again.")
+
+
+class ResultFormattingError(PublicMessage):
+    """
+    **13019** Unable to correctly format some part of the result
+    """
+    errno = 13019
+    type = "warning"
+
+
 def iter_messages(variables, base):
     """Return a tuple with all subclasses
     """
@@ -229,8 +365,8 @@ public_messages = tuple(sorted(
 
 def print_report(label, classes):
     for cls in classes:
-        print '%d\t%s' % (cls.errno, cls.__name__)
-    print '(%d %s)' % (len(classes), label)
+        print('%d\t%s' % (cls.errno, cls.__name__))
+    print('(%d %s)' % (len(classes), label))
 
 if __name__ == '__main__':
     print_report('public messages', public_messages)
