@@ -103,24 +103,29 @@ import re
 import decimal
 import base64
 import datetime
-from xmlrpclib import MAXINT, MININT
+from six.moves.xmlrpc_client import MAXINT, MININT
+import encodings.idna
 
-from types import NoneType
-from text import _ as ugettext
-from plugable import ReadOnly, lock, check_name
-from errors import ConversionError, RequirementError, ValidationError
-from errors import PasswordMismatch, Base64DecodeError
-from constants import TYPE_ERROR, CALLABLE_ERROR, LDAP_GENERALIZED_TIME_FORMAT
-from text import Gettext, FixMe
-from util import json_serialize, validate_idna_domain
+import dns.name
+import six
+
+from ipalib.text import _ as ugettext
+from ipalib.plugable import ReadOnly, lock, check_name
+from ipalib.errors import ConversionError, RequirementError, ValidationError
+from ipalib.errors import PasswordMismatch, Base64DecodeError
+from ipalib.constants import TYPE_ERROR, CALLABLE_ERROR, LDAP_GENERALIZED_TIME_FORMAT
+from ipalib.text import Gettext, FixMe
+from ipalib.util import json_serialize, validate_idna_domain
 from ipapython.dn import DN
 from ipapython.dnsutil import DNSName
-import dns.name
-import encodings.idna
 
 def _is_null(value):
     return not value and value != 0 # NOTE: False == 0
 
+if six.PY3:
+    unicode = str
+
+
 class DefaultFrom(ReadOnly):
     """
     Derive a default value from other supplied values.
@@ -203,7 +208,7 @@ class DefaultFrom(ReadOnly):
             )
         self.callback = callback
         if len(keys) == 0:
-            fc = callback.func_code
+            fc = callback.__code__
             if fc.co_flags & 0x0c:
                 raise ValueError("callback: variable-length argument list not allowed")
             self.keys = fc.co_varnames[:fc.co_argcount]
@@ -237,9 +242,12 @@ class DefaultFrom(ReadOnly):
             return
         try:
             return self.callback(*vals)
-        except StandardError:
+        except Exception:
             pass
 
+    def __json__(self):
+        return self.keys
+
 
 def parse_param_spec(spec):
     """
@@ -376,7 +384,7 @@ class Param(ReadOnly):
     # (direct) subclass must *always* override this class attribute.
     # If multiple types are permitted, set `type` to the canonical type and
     # `allowed_types` to a tuple of all allowed types.
-    type = NoneType  # Ouch, this wont be very useful in the real world!
+    type = type(None) # Ouch, this wont be very useful in the real world!
 
     # Subclasses should override this with something more specific:
     type_error = _('incorrect type')
@@ -390,8 +398,8 @@ class Param(ReadOnly):
         ('cli_name', str, None),
         ('cli_short_name', str, None),
         ('deprecated_cli_aliases', frozenset, frozenset()),
-        ('label', (basestring, Gettext), None),
-        ('doc', (basestring, Gettext), None),
+        ('label', (six.string_types, Gettext), None),
+        ('doc', (six.string_types, Gettext), None),
         ('required', bool, True),
         ('multivalue', bool, False),
         ('primary_key', bool, False),
@@ -732,7 +740,7 @@ class Param(ReadOnly):
             return value
         try:
             return self.normalizer(value)
-        except StandardError:
+        except Exception:
             return value
 
     def convert(self, value):
@@ -787,9 +795,8 @@ class Param(ReadOnly):
             if type(value) not in (tuple, list):
                 value = (value,)
             values = tuple(
-                self._convert_scalar(v, i) for (i, v) in filter(
-                    lambda iv: not _is_null(iv[1]), enumerate(value)
-                )
+                self._convert_scalar(v, i)
+                for (i, v) in enumerate(value) if not _is_null(v)
             )
             if len(values) == 0:
                 return
@@ -910,37 +917,32 @@ class Param(ReadOnly):
             if default is not None:
                 try:
                     return self.convert(self.normalize(default))
-                except StandardError:
+                except Exception:
                     pass
         return self.default
 
     def sort_key(self, value):
         return value
 
-    json_exclude_attrs = (
-        'alwaysask', 'autofill', 'cli_name', 'cli_short_name', 'csv',
-        'sortorder', 'falsehoods', 'truths', 'version',
-    )
-
     def __json__(self):
         json_dict = {}
         for (a, k, d) in self.kwargs:
-            if a in self.json_exclude_attrs:
-                continue
             if k in (callable, DefaultFrom):
                 continue
             elif isinstance(getattr(self, a), frozenset):
                 json_dict[a] = [k for k in getattr(self, a, [])]
             else:
                 val = getattr(self, a, '')
-                if val is None or val is False:
-                    # ignore False and not set because lack of their presence is
+                if val is None:
+                    # ignore 'not set' because lack of their presence is
                     # the information itself
-                    continue;
+                    continue
                 json_dict[a] = json_serialize(val)
+
         json_dict['class'] = self.__class__.__name__
         json_dict['name'] = self.name
         json_dict['type'] = self.type.__name__
+
         return json_dict
 
 
@@ -965,7 +967,7 @@ class Bool(Param):
         """
         if type(value) in self.allowed_types:
             return value
-        if isinstance(value, basestring):
+        if isinstance(value, six.string_types):
             value = value.lower()
         if value in self.truths:
             return True
@@ -1032,7 +1034,7 @@ class Number(Param):
         """
         if type(value) in self.allowed_types:
             return value
-        if type(value) in (unicode, int, long, float):
+        if type(value) in (unicode, float) + six.integer_types:
             try:
                 return self.type(value)
             except ValueError:
@@ -1051,12 +1053,12 @@ class Int(Number):
     """
 
     type = int
-    allowed_types = int, long
+    allowed_types = six.integer_types
     type_error = _('must be an integer')
 
     kwargs = Param.kwargs + (
-        ('minvalue', (int, long), int(MININT)),
-        ('maxvalue', (int, long), int(MAXINT)),
+        ('minvalue', six.integer_types, int(MININT)),
+        ('maxvalue', six.integer_types, int(MAXINT)),
     )
 
     @staticmethod
@@ -1070,6 +1072,9 @@ class Int(Number):
         if type(value) is unicode:
             if u'.' in value:
                 return int(float(value))
+            if six.PY3 and re.match('0[0-9]+', value):
+                # 0-prefixed octal format
+                return int(value, 8)
             return int(value, 0)
 
         raise ValueError(value)
@@ -1098,7 +1103,7 @@ class Int(Number):
         """
         Check min constraint.
         """
-        assert type(value) in (int, long)
+        assert type(value) in six.integer_types
         if value < self.minvalue:
             return _('must be at least %(minvalue)d') % dict(
                 minvalue=self.minvalue,
@@ -1108,7 +1113,7 @@ class Int(Number):
         """
         Check max constraint.
         """
-        assert type(value) in (int, long)
+        assert type(value) in six.integer_types
         if value > self.maxvalue:
             return _('can be at most %(maxvalue)d') % dict(
                 maxvalue=self.maxvalue,
@@ -1144,10 +1149,10 @@ class Decimal(Number):
             value = kw.get(kwparam)
             if value is None:
                 continue
-            if isinstance(value, (basestring, float)):
+            if isinstance(value, (six.string_types, float)):
                 try:
                     value = decimal.Decimal(value)
-                except Exception, e:
+                except Exception as e:
                     raise ValueError(
                        '%s: cannot parse kwarg %s: %s' % (
                         name, kwparam, str(e)))
@@ -1155,9 +1160,9 @@ class Decimal(Number):
 
         super(Decimal, self).__init__(name, *rules, **kw)
 
-        if (self.minvalue > self.maxvalue) \
-            and (self.minvalue is not None and \
-                 self.maxvalue is not None):
+        if (self.minvalue is not None and
+                self.maxvalue is not None and
+                self.minvalue > self.maxvalue):
             raise ValueError(
                 '%s: minvalue > maxvalue (minvalue=%s, maxvalue=%s)' % (
                     self.nice, self.minvalue, self.maxvalue)
@@ -1202,7 +1207,7 @@ class Decimal(Number):
             quantize_exp = decimal.Decimal(10) ** -self.precision
             try:
                 value = value.quantize(quantize_exp)
-            except decimal.DecimalException, e:
+            except decimal.DecimalException as e:
                 raise ConversionError(name=self.get_param_name(),
                                       error=unicode(e))
         return value
@@ -1216,7 +1221,7 @@ class Decimal(Number):
                 value = value.quantize(decimal.Decimal(1)) \
                         if value == value.to_integral() \
                         else value.normalize()
-            except decimal.DecimalException, e:
+            except decimal.DecimalException as e:
                 raise ConversionError(name=self.get_param_name(),
                                       error=unicode(e))
 
@@ -1234,10 +1239,10 @@ class Decimal(Number):
         return value
 
     def _convert_scalar(self, value, index=None):
-        if isinstance(value, (basestring, float)):
+        if isinstance(value, (six.string_types, float)):
             try:
                 value = decimal.Decimal(value)
-            except decimal.DecimalException, e:
+            except decimal.DecimalException as e:
                 raise ConversionError(name=self.get_param_name(), index=index,
                                       error=unicode(e))
 
@@ -1265,8 +1270,7 @@ class Data(Param):
         ('minlength', int, None),
         ('maxlength', int, None),
         ('length', int, None),
-        ('pattern', (basestring,), None),
-        ('pattern_errmsg', (basestring,), None),
+        ('pattern_errmsg', (six.string_types,), None),
     )
 
     re = None
@@ -1331,8 +1335,11 @@ class Bytes(Data):
     Also see the `Str` parameter.
     """
 
-    type = str
+    type = bytes
     type_error = _('must be binary data')
+    kwargs = Data.kwargs + (
+        ('pattern', (bytes,), None),
+    )
 
     def __init__(self, name, *rules, **kw):
         if kw.get('pattern', None) is None:
@@ -1346,7 +1353,7 @@ class Bytes(Data):
         """
         Check minlength constraint.
         """
-        assert type(value) is str
+        assert type(value) is bytes
         if len(value) < self.minlength:
             return _('must be at least %(minlength)d bytes') % dict(
                 minlength=self.minlength,
@@ -1356,7 +1363,7 @@ class Bytes(Data):
         """
         Check maxlength constraint.
         """
-        assert type(value) is str
+        assert type(value) is bytes
         if len(value) > self.maxlength:
             return _('can be at most %(maxlength)d bytes') % dict(
                 maxlength=self.maxlength,
@@ -1366,7 +1373,7 @@ class Bytes(Data):
         """
         Check length constraint.
         """
-        assert type(value) is str
+        assert type(value) is bytes
         if len(value) != self.length:
             return _('must be exactly %(length)d bytes') % dict(
                 length=self.length,
@@ -1376,7 +1383,7 @@ class Bytes(Data):
         if isinstance(value, unicode):
             try:
                 value = base64.b64decode(value)
-            except TypeError, e:
+            except (TypeError, ValueError) as e:
                 raise Base64DecodeError(reason=str(e))
         return super(Bytes, self)._convert_scalar(value, index)
 
@@ -1394,6 +1401,7 @@ class Str(Data):
     """
 
     kwargs = Data.kwargs + (
+        ('pattern', (six.string_types,), None),
         ('noextrawhitespace', bool, True),
     )
 
@@ -1414,7 +1422,7 @@ class Str(Data):
         """
         if type(value) in self.allowed_types:
             return value
-        if type(value) in (int, long, float, decimal.Decimal):
+        if type(value) in (float, decimal.Decimal) + six.integer_types:
             return self.type(value)
         if type(value) in (tuple, list):
             raise ConversionError(name=self.name, index=index,
@@ -1475,13 +1483,13 @@ class IA5Str(Str):
         super(IA5Str, self).__init__(name, *rules, **kw)
 
     def _convert_scalar(self, value, index=None):
-        if isinstance(value, basestring):
-            for i in xrange(len(value)):
-                if ord(value[i]) > 127:
+        if isinstance(value, six.string_types):
+            for char in value:
+                if ord(char) > 127:
                     raise ConversionError(name=self.get_param_name(),
                         index=index,
                         error=_('The character %(char)r is not allowed.') %
-                            dict(char=value[i],)
+                            dict(char=char,)
                     )
         return super(IA5Str, self)._convert_scalar(value, index)
 
@@ -1568,7 +1576,7 @@ class IntEnum(Enum):
     """
 
     type = int
-    allowed_types = int, long
+    allowed_types = six.integer_types
     type_error = Int.type_error
 
     def _convert_scalar(self, value, index=None):
@@ -1647,7 +1655,7 @@ class DateTime(Param):
     type_error = _('must be datetime value')
 
     def _convert_scalar(self, value, index=None):
-        if isinstance(value, basestring):
+        if isinstance(value, six.string_types):
             for date_format in self.accepted_formats:
                 try:
                     time = datetime.datetime.strptime(value, date_format)
@@ -1839,7 +1847,7 @@ class AccessTime(Str):
     def _rule_required(self, _, value):
         try:
             self._check(value)
-        except ValueError, e:
+        except ValueError as e:
             raise ValidationError(name=self.get_param_name(), error=e.args[0])
         except IndexError:
             raise ValidationError(
@@ -1860,7 +1868,7 @@ class DNParam(Param):
 
         try:
             dn = DN(value)
-        except Exception, e:
+        except Exception as e:
             raise ConversionError(name=self.get_param_name(), index=index,
                                   error=ugettext(e))
         return dn
