@@ -17,19 +17,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import print_function
+
 import os
 import sys
 import base64
 import nss.nss as nss
 from pyasn1.type import univ, char, namedtype, tag
 from pyasn1.codec.der import decoder
+import six
+
 from ipapython import ipautil
 from ipalib import api
 
+if six.PY3:
+    unicode = str
+
 PEM = 0
 DER = 1
 
 SAN_DNSNAME = 'DNS name'
+SAN_RFC822NAME = 'RFC822 Name'
 SAN_OTHERNAME_UPN = 'Other Name (OID.1.3.6.1.4.1.311.20.2.3)'
 SAN_OTHERNAME_KRB5PRINCIPALNAME = 'Other Name (OID.1.3.6.1.5.2.2)'
 
@@ -52,7 +60,20 @@ def get_extensions(csr, datatype=PEM):
     The return value is a tuple of strings
     """
     request = load_certificate_request(csr, datatype)
-    return tuple(nss.oid_dotted_decimal(ext.oid_tag)[4:]
+
+    # Work around a bug in python-nss where nss.oid_dotted_decimal
+    # errors on unrecognised OIDs
+    #
+    # https://bugzilla.redhat.com/show_bug.cgi?id=1246729
+    #
+    def get_prefixed_oid_str(ext):
+        """Returns a string like 'OID.1.2...'."""
+        if ext.oid_tag == 0:
+            return repr(ext)
+        else:
+            return nss.oid_dotted_decimal(ext.oid)
+
+    return tuple(get_prefixed_oid_str(ext)[4:]
                  for ext in request.extensions)
 
 class _PrincipalName(univ.Sequence):
@@ -242,7 +263,7 @@ if __name__ == '__main__':
     csrlines = sys.stdin.readlines()
     csr = ''.join(csrlines)
 
-    print load_certificate_request(csr)
-    print get_subject(csr)
-    print get_subjectaltname(csr)
-    print get_friendlyname(csr)
+    print(load_certificate_request(csr))
+    print(get_subject(csr))
+    print(get_subjectaltname(csr))
+    print(get_friendlyname(csr))
