@@ -31,33 +31,49 @@ import inspect
 import threading
 import os
 from os import path
-import subprocess
 import optparse
-import errors
 import textwrap
+import collections
+import importlib
 
-from config import Env
-import util
-import text
-from text import _
-from base import ReadOnly, NameSpace, lock, islocked, check_name
-from constants import DEFAULT_CONFIG
+import six
+
+from ipalib import errors
+from ipalib.config import Env
+from ipalib import text
+from ipalib.text import _
+from ipalib.base import ReadOnly, NameSpace, lock, islocked, check_name
+from ipalib.constants import DEFAULT_CONFIG
 from ipapython.ipa_log_manager import *
 from ipapython.version import VERSION, API_VERSION
 
+if six.PY3:
+    unicode = str
+
 # FIXME: Updated constants.TYPE_ERROR to use this clearer format from wehjit:
 TYPE_ERROR = '%s: need a %r; got a %r: %r'
 
-def is_production_mode(obj):
+
+# FIXME: This function has no unit test
+def find_modules_in_dir(src_dir):
     """
-    If the object has self.env.mode defined and that mode is
-    production return True, otherwise return False.
+    Iterate through module names found in ``src_dir``.
     """
-    if getattr(obj, 'env', None) is None:
-        return False
-    if getattr(obj.env, 'mode', None) is None:
-        return False
-    return obj.env.mode == 'production'
+    if not (os.path.abspath(src_dir) == src_dir and os.path.isdir(src_dir)):
+        return
+    if os.path.islink(src_dir):
+        return
+    suffix = '.py'
+    for name in sorted(os.listdir(src_dir)):
+        if not name.endswith(suffix):
+            continue
+        pyfile = os.path.join(src_dir, name)
+        if not os.path.isfile(pyfile):
+            continue
+        module = name[:-len(suffix)]
+        if module == '__init__':
+            continue
+        yield module
 
 
 class Registry(object):
@@ -74,129 +90,14 @@ class Registry(object):
     For forward compatibility, make sure that the module-level instance of
     this object is named "register".
     """
-    # TODO: Instead of auto-loading when plugin modules are imported,
-    # plugins should be stored in this object.
-    # The API should examine it and load plugins explicitly.
     def __call__(self):
-        from ipalib import api
-
         def decorator(cls):
-            api.register(cls)
+            API.register(cls)
             return cls
 
         return decorator
 
 
-class SetProxy(ReadOnly):
-    """
-    A read-only container with set/sequence behaviour.
-
-    This container acts as a proxy to an actual set-like object (a set,
-    frozenset, or dict) that is passed to the constructor. To the extent
-    possible in Python, this underlying set-like object cannot be modified
-    through the SetProxy... which just means you wont do it accidentally.
-    """
-    def __init__(self, s):
-        """
-        :param s: The target set-like object (a set, frozenset, or dict)
-        """
-        allowed = (set, frozenset, dict)
-        if type(s) not in allowed:
-            raise TypeError('%r not in %r' % (type(s), allowed))
-        self.__s = s
-        if not is_production_mode(self):
-            lock(self)
-
-    def __len__(self):
-        """
-        Return the number of items in this container.
-        """
-        return len(self.__s)
-
-    def __iter__(self):
-        """
-        Iterate (in ascending order) through keys.
-        """
-        for key in sorted(self.__s):
-            yield key
-
-    def __contains__(self, key):
-        """
-        Return True if this container contains ``key``.
-
-        :param key: The key to test for membership.
-        """
-        return key in self.__s
-
-
-class DictProxy(SetProxy):
-    """
-    A read-only container with mapping behaviour.
-
-    This container acts as a proxy to an actual mapping object (a dict) that
-    is passed to the constructor. To the extent possible in Python, this
-    underlying mapping object cannot be modified through the DictProxy...
-    which just means you wont do it accidentally.
-
-    Also see `SetProxy`.
-    """
-    def __init__(self, d):
-        """
-        :param d: The target mapping object (a dict)
-        """
-        if type(d) is not dict:
-            raise TypeError('%r is not %r' % (type(d), dict))
-        self.__d = d
-        super(DictProxy, self).__init__(d)
-
-    def __getitem__(self, key):
-        """
-        Return the value corresponding to ``key``.
-
-        :param key: The key of the value you wish to retrieve.
-        """
-        return self.__d[key]
-
-    def __call__(self):
-        """
-        Iterate (in ascending order by key) through values.
-        """
-        for key in self:
-            yield self.__d[key]
-
-
-class MagicDict(DictProxy):
-    """
-    A mapping container whose values can be accessed as attributes.
-
-    For example:
-
-    >>> magic = MagicDict({'the_key': 'the value'})
-    >>> magic['the_key']
-    'the value'
-    >>> magic.the_key
-    'the value'
-
-    This container acts as a proxy to an actual mapping object (a dict) that
-    is passed to the constructor. To the extent possible in Python, this
-    underlying mapping object cannot be modified through the MagicDict...
-    which just means you wont do it accidentally.
-
-    Also see `DictProxy` and `SetProxy`.
-    """
-
-    def __getattr__(self, name):
-        """
-        Return the value corresponding to ``name``.
-
-        :param name: The name of the attribute you wish to retrieve.
-        """
-        try:
-            return self[name]
-        except KeyError:
-            raise AttributeError('no magic attribute %r' % name)
-
-
 class Plugin(ReadOnly):
     """
     Base class for all plugins.
@@ -206,8 +107,9 @@ class Plugin(ReadOnly):
 
     label = None
 
-    def __init__(self):
-        self.__api = None
+    def __init__(self, api):
+        assert api is not None
+        self.__api = api
         self.__finalize_called = False
         self.__finalized = False
         self.__finalize_lock = threading.RLock()
@@ -236,14 +138,27 @@ class Plugin(ReadOnly):
                 )
             )
 
-    def __get_api(self):
+    @property
+    def api(self):
         """
-        Return `API` instance passed to `set_api()`.
-
-        If `set_api()` has not yet been called, None is returned.
+        Return `API` instance passed to `__init__()`.
         """
         return self.__api
-    api = property(__get_api)
+
+    # FIXME: for backward compatibility only
+    @property
+    def env(self):
+        return self.__api.env
+
+    # FIXME: for backward compatibility only
+    @property
+    def Backend(self):
+        return self.__api.Backend
+
+    # FIXME: for backward compatibility only
+    @property
+    def Command(self):
+        return self.__api.Command
 
     def finalize(self):
         """
@@ -262,7 +177,7 @@ class Plugin(ReadOnly):
             self.__finalize_called = True
             self._on_finalize()
             self.__finalized = True
-            if not is_production_mode(self):
+            if not self.__api.is_production_mode():
                 lock(self)
 
     def _on_finalize(self):
@@ -318,40 +233,6 @@ class Plugin(ReadOnly):
                     "attribute '%s' of plugin '%s' was not set in finalize()" % (self.name, obj.name)
                 )
 
-    def set_api(self, api):
-        """
-        Set reference to `API` instance.
-        """
-        assert self.__api is None, 'set_api() can only be called once'
-        assert api is not None, 'set_api() argument cannot be None'
-        self.__api = api
-        if not isinstance(api, API):
-            return
-        for name in api:
-            assert not hasattr(self, name)
-            setattr(self, name, api[name])
-        for name in ('env', 'context'):
-            if hasattr(api, name):
-                assert not hasattr(self, name)
-                setattr(self, name, getattr(api, name))
-
-    def call(self, executable, *args):
-        """
-        Call ``executable`` with ``args`` using subprocess.call().
-
-        If the call exits with a non-zero exit status,
-        `ipalib.errors.SubprocessError` is raised, from which you can retrieve
-        the exit code by checking the SubprocessError.returncode attribute.
-
-        This method does *not* return what ``executable`` sent to stdout... for
-        that, use `Plugin.callread()`.
-        """
-        argv = (executable,) + args
-        self.debug('Calling %r', argv)
-        code = subprocess.call(argv)
-        if code != 0:
-            raise errors.SubprocessError(returncode=code, argv=argv)
-
     def __repr__(self):
         """
         Return 'module_name.class_name()' representation.
@@ -365,7 +246,7 @@ class Plugin(ReadOnly):
         )
 
 
-class Registrar(DictProxy):
+class Registrar(collections.Mapping):
     """
     Collects plugin classes as they are registered.
 
@@ -375,47 +256,8 @@ class Registrar(DictProxy):
 
     The plugins are instantiated when `API.finalize()` is called.
     """
-    def __init__(self, *allowed):
-        """
-        :param allowed: Base classes from which plugins accepted by this
-            Registrar must subclass.
-        """
-        self.__allowed = dict((base, {}) for base in allowed)
-        self.__registered = set()
-        super(Registrar, self).__init__(
-            dict(self.__base_iter())
-        )
-
-    def __base_iter(self):
-        for (base, sub_d) in self.__allowed.iteritems():
-            if not is_production_mode(self):
-                assert inspect.isclass(base)
-            name = base.__name__
-            if not is_production_mode(self):
-                assert not hasattr(self, name)
-            setattr(self, name, MagicDict(sub_d))
-            yield (name, base)
-
-    def __findbases(self, klass):
-        """
-        Iterates through allowed bases that ``klass`` is a subclass of.
-
-        Raises `errors.PluginSubclassError` if ``klass`` is not a subclass of
-        any allowed base.
-
-        :param klass: The plugin class to find bases for.
-        """
-        if not is_production_mode(self):
-            assert inspect.isclass(klass)
-        found = False
-        for (base, sub_d) in self.__allowed.iteritems():
-            if issubclass(klass, base):
-                found = True
-                yield (base, sub_d)
-        if not found:
-            raise errors.PluginSubclassError(
-                plugin=klass, bases=self.__allowed.keys()
-            )
+    def __init__(self):
+        self.__registry = collections.OrderedDict()
 
     def __call__(self, klass, override=False):
         """
@@ -428,51 +270,97 @@ class Registrar(DictProxy):
             raise TypeError('plugin must be a class; got %r' % klass)
 
         # Raise DuplicateError if this exact class was already registered:
-        if klass in self.__registered:
+        if klass in self.__registry:
             raise errors.PluginDuplicateError(plugin=klass)
 
-        # Find the base class or raise SubclassError:
-        for (base, sub_d) in self.__findbases(klass):
-            # Check override:
-            if klass.__name__ in sub_d:
-                if not override:
-                    # Must use override=True to override:
-                    raise errors.PluginOverrideError(
-                        base=base.__name__,
-                        name=klass.__name__,
-                        plugin=klass,
-                    )
-            else:
-                if override:
-                    # There was nothing already registered to override:
-                    raise errors.PluginMissingOverrideError(
-                        base=base.__name__,
-                        name=klass.__name__,
-                        plugin=klass,
-                    )
+        # The plugin is okay, add to __registry:
+        self.__registry[klass] = dict(override=override)
 
-            # The plugin is okay, add to sub_d:
-            sub_d[klass.__name__] = klass
+    def __getitem__(self, key):
+        return self.__registry[key]
 
-        # The plugin is okay, add to __registered:
-        self.__registered.add(klass)
+    def __iter__(self):
+        return iter(self.__registry)
+
+    def __len__(self):
+        return len(self.__registry)
 
 
-class API(DictProxy):
+class API(ReadOnly):
     """
     Dynamic API object through which `Plugin` instances are accessed.
     """
 
-    def __init__(self, *allowed):
-        self.__d = dict()
+    register = Registrar()
+
+    def __init__(self):
+        super(API, self).__init__()
+        self.__plugins = {}
         self.__done = set()
-        self.register = Registrar(*allowed)
         self.env = Env()
-        super(API, self).__init__(self.__d)
+
+    @property
+    def bases(self):
+        raise NotImplementedError
+
+    @property
+    def modules(self):
+        raise NotImplementedError
+
+    def __len__(self):
+        """
+        Return the number of plugin namespaces in this API object.
+        """
+        return len(self.bases)
+
+    def __iter__(self):
+        """
+        Iterate (in ascending order) through plugin namespace names.
+        """
+        return (base.__name__ for base in self.bases)
+
+    def __contains__(self, name):
+        """
+        Return True if this API object contains plugin namespace ``name``.
+
+        :param name: The plugin namespace name to test for membership.
+        """
+        return name in set(self)
+
+    def __getitem__(self, name):
+        """
+        Return the plugin namespace corresponding to ``name``.
+
+        :param name: The name of the plugin namespace you wish to retrieve.
+        """
+        if name in self:
+            try:
+                return getattr(self, name)
+            except AttributeError:
+                pass
+
+        raise KeyError(name)
+
+    def __call__(self):
+        """
+        Iterate (in ascending order by name) through plugin namespaces.
+        """
+        for name in self:
+            try:
+                yield getattr(self, name)
+            except AttributeError:
+                raise KeyError(name)
+
+    def is_production_mode(self):
+        """
+        If the object has self.env.mode defined and that mode is
+        production return True, otherwise return False.
+        """
+        return getattr(self.env, 'mode', None) == 'production'
 
     def __doing(self, name):
         if name in self.__done:
-            raise StandardError(
+            raise Exception(
                 '%s.%s() already called' % (self.__class__.__name__, name)
             )
         self.__done.add(name)
@@ -489,16 +377,16 @@ class API(DictProxy):
         Initialize environment variables and logging.
         """
         self.__doing('bootstrap')
+        self.log_mgr = log_mgr
+        log = log_mgr.root_logger
+        self.log = log
         self.env._bootstrap(**overrides)
         self.env._finalize_core(**dict(DEFAULT_CONFIG))
-        object.__setattr__(self, 'log_mgr', log_mgr)
-        log = log_mgr.root_logger
-        object.__setattr__(self, 'log', log)
 
         # Add the argument parser
         if not parser:
             parser = self.build_global_parser()
-        object.__setattr__(self, 'parser', parser)
+        self.parser = parser
 
         # If logging has already been configured somewhere else (like in the
         # installer), don't add handlers or change levels:
@@ -518,7 +406,7 @@ class API(DictProxy):
                 else:
                     level = 'warning'
 
-        if log_mgr.handlers.has_key('console'):
+        if 'console' in log_mgr.handlers:
             log_mgr.remove_handler('console')
         log_mgr.create_log_handlers([dict(name='console',
                                           stream=sys.stderr,
@@ -546,7 +434,7 @@ class API(DictProxy):
                                               filename=self.env.log,
                                               level=level,
                                               format=LOGGING_FORMAT_FILE)])
-        except IOError, e:
+        except IOError as e:
             log.error('Cannot open log file %r: %s', self.env.log, e)
             return
 
@@ -642,60 +530,120 @@ class API(DictProxy):
         self.__do_if_not_done('bootstrap')
         if self.env.mode in ('dummy', 'unit_test'):
             return
-        self.import_plugins('ipalib')
-        if self.env.context in ('server', 'lite'):
-            self.import_plugins('ipaserver')
-        if self.env.context in ('installer', 'updates'):
-            self.import_plugins('ipaserver/install/plugins')
-        if self.env.context in ('advise'):
-            self.import_plugins('ipaserver/advise/plugins')
+        for module in self.modules:
+            self.import_plugins(module)
 
     # FIXME: This method has no unit test
-    def import_plugins(self, package):
+    def import_plugins(self, module):
         """
-        Import modules in ``plugins`` sub-package of ``package``.
+        Import plugins from ``module``.
+
+        :param module: Name of the module to import. This might be a wildcard
+                       in the form ```package.*``` in which case all modules
+                       from the given package are loaded.
         """
-        package = package.replace(os.path.sep, '.')
-        subpackage = '%s.plugins' % package
-        try:
-            parent = __import__(package)
-            parts = package.split('.')[1:]
-            if parts:
-                for part in parts:
-                    if part == 'plugins':
-                        plugins = subpackage.plugins
-                        subpackage = plugins.__name__
-                        break
-                    subpackage = parent.__getattribute__(part)
-                    parent = subpackage
-            else:
-                plugins = __import__(subpackage).plugins
-        except ImportError, e:
-            self.log.error(
-                'cannot import plugins sub-package %s: %s', subpackage, e
-            )
-            raise e
-        parent_dir = path.dirname(path.abspath(parent.__file__))
-        plugins_dir = path.dirname(path.abspath(plugins.__file__))
-        if parent_dir == plugins_dir:
-            raise errors.PluginsPackageError(
-                name=subpackage, file=plugins.__file__
-            )
-        self.log.debug('importing all plugin modules in %r...', plugins_dir)
-        for (name, pyfile) in util.find_modules_in_dir(plugins_dir):
-            fullname = '%s.%s' % (subpackage, name)
-            self.log.debug('importing plugin module %r', pyfile)
+        if module.endswith('.*'):
+            subpackage = module[:-2]
             try:
-                __import__(fullname)
-            except errors.SkipPluginModule, e:
-                self.log.debug(
-                    'skipping plugin module %s: %s', fullname, e.reason
+                plugins = importlib.import_module(subpackage)
+            except ImportError as e:
+                self.log.error("cannot import plugins sub-package %s: %s",
+                               subpackage, e)
+                raise
+            package, dot, part = subpackage.rpartition('.')
+            parent = sys.modules[package]
+
+            parent_dir = path.dirname(path.abspath(parent.__file__))
+            plugins_dir = path.dirname(path.abspath(plugins.__file__))
+            if parent_dir == plugins_dir:
+                raise errors.PluginsPackageError(
+                    name=subpackage, file=plugins.__file__
                 )
-            except StandardError, e:
+
+            self.log.debug("importing all plugin modules in %s...", subpackage)
+            modules = find_modules_in_dir(plugins_dir)
+            modules = ['.'.join((subpackage, name)) for name in modules]
+        else:
+            modules = [module]
+
+        for name in modules:
+            self.log.debug("importing plugin module %s", name)
+            try:
+                module = importlib.import_module(name)
+            except errors.SkipPluginModule as e:
+                self.log.debug("skipping plugin module %s: %s", name, e.reason)
+            except Exception as e:
                 if self.env.startup_traceback:
                     import traceback
-                    self.log.error('could not load plugin module %r\n%s', pyfile, traceback.format_exc())
+                    self.log.error("could not load plugin module %s\n%s", name,
+                                   traceback.format_exc())
                 raise
+            else:
+                self.add_module(module)
+
+    def add_module(self, module):
+        """
+        Add plugins from the ``module``.
+
+        :param module: A module from which to add plugins.
+        """
+        for name in dir(module):
+            klass = getattr(module, name)
+            if not inspect.isclass(klass):
+                continue
+            if klass not in self.register:
+                continue
+            kwargs = self.register[klass]
+            self.add_plugin(klass, **kwargs)
+
+    def add_plugin(self, klass, override=False):
+        """
+        Add the plugin ``klass``.
+
+        :param klass: A subclass of `Plugin` to attempt to add.
+        :param override: If true, override an already added plugin.
+        """
+        if not inspect.isclass(klass):
+            raise TypeError('plugin must be a class; got %r' % klass)
+
+        # Find the base class or raise SubclassError:
+        found = False
+        for base in self.bases:
+            if not issubclass(klass, base):
+                continue
+
+            sub_d = self.__plugins.setdefault(base, {})
+            found = True
+
+            if sub_d.get(klass.__name__) is klass:
+                continue
+
+            # Check override:
+            if klass.__name__ in sub_d:
+                if not override:
+                    # Must use override=True to override:
+                    raise errors.PluginOverrideError(
+                        base=base.__name__,
+                        name=klass.__name__,
+                        plugin=klass,
+                    )
+            else:
+                if override:
+                    # There was nothing already registered to override:
+                    raise errors.PluginMissingOverrideError(
+                        base=base.__name__,
+                        name=klass.__name__,
+                        plugin=klass,
+                    )
+
+            # The plugin is okay, add to sub_d:
+            sub_d[klass.__name__] = klass
+
+        if not found:
+            raise errors.PluginSubclassError(
+                plugin=klass,
+                bases=self.bases,
+            )
 
     def finalize(self):
         """
@@ -707,78 +655,42 @@ class API(DictProxy):
         self.__doing('finalize')
         self.__do_if_not_done('load_plugins')
 
-        class PluginInstance(object):
-            """
-            Represents a plugin instance.
-            """
-
-            i = 0
-
-            def __init__(self, klass):
-                self.created = self.next()
-                self.klass = klass
-                self.instance = klass()
-                self.bases = []
-
-            @classmethod
-            def next(cls):
-                cls.i += 1
-                return cls.i
-
-        class PluginInfo(ReadOnly):
-            def __init__(self, p):
-                assert isinstance(p, PluginInstance)
-                self.created = p.created
-                self.name = p.klass.__name__
-                self.module = str(p.klass.__module__)
-                self.plugin = '%s.%s' % (self.module, self.name)
-                self.bases = tuple(b.__name__ for b in p.bases)
-                if not is_production_mode(self):
-                    lock(self)
-
+        production_mode = self.is_production_mode()
         plugins = {}
-        tofinalize = set()
-        def plugin_iter(base, subclasses):
-            for klass in subclasses:
-                assert issubclass(klass, base)
-                if klass not in plugins:
-                    plugins[klass] = PluginInstance(klass)
-                p = plugins[klass]
-                if not is_production_mode(self):
-                    assert base not in p.bases
-                p.bases.append(base)
-                if klass.finalize_early or not self.env.plugins_on_demand:
-                    tofinalize.add(p)
-                yield p.instance
+        plugin_info = {}
 
-        production_mode = is_production_mode(self)
-        for name in self.register:
-            base = self.register[name]
-            magic = getattr(self.register, name)
-            namespace = NameSpace(
-                plugin_iter(base, (magic[k] for k in magic))
-            )
-            if not production_mode:
-                assert not (
-                    name in self.__d or hasattr(self, name)
-                )
-            self.__d[name] = namespace
-            object.__setattr__(self, name, namespace)
+        for base in self.bases:
+            name = base.__name__
+            sub_d = self.__plugins.get(base, {})
 
-        for p in plugins.itervalues():
-            p.instance.set_api(self)
-            if not production_mode:
-                assert p.instance.api is self
+            members = []
+            for klass in sub_d.values():
+                try:
+                    instance = plugins[klass]
+                except KeyError:
+                    instance = plugins[klass] = klass(self)
+                members.append(instance)
+                plugin_info.setdefault(
+                    '%s.%s' % (klass.__module__, klass.__name__),
+                    []).append(name)
 
-        for p in tofinalize:
-            p.instance.ensure_finalized()
             if not production_mode:
-                assert islocked(p.instance) is True
-        object.__setattr__(self, '_API__finalized', True)
-        tuple(PluginInfo(p) for p in plugins.itervalues())
-        object.__setattr__(self, 'plugins',
-            tuple(PluginInfo(p) for p in plugins.itervalues())
-        )
+                assert not hasattr(self, name)
+            setattr(self, name, NameSpace(members))
+
+        for klass, instance in plugins.items():
+            if not production_mode:
+                assert instance.api is self
+            if klass.finalize_early or not self.env.plugins_on_demand:
+                instance.ensure_finalized()
+                if not production_mode:
+                    assert islocked(instance)
+
+        self.__finalized = True
+        self.plugins = tuple((k, tuple(v)) for k, v in plugin_info.items())
+
+        if not production_mode:
+            lock(self)
 
 
 class IPAHelpFormatter(optparse.IndentedHelpFormatter):
