@@ -119,16 +119,24 @@ targetattr REPLACES the current attributes, it does not add to them.
 """
 from copy import deepcopy
 
+import six
+
 from ipalib import api, crud, errors
-from ipalib import Object, Command
-from ipalib import Flag, Int, Str, StrEnum
+from ipalib import Object
+from ipalib import Flag, Str, StrEnum, DNParam
 from ipalib.aci import ACI
 from ipalib import output
 from ipalib import _, ngettext
-from ipalib.plugins.baseldap import gen_pkey_only_option
+from ipalib.plugable import Registry
+from ipalib.plugins.baseldap import gen_pkey_only_option, pkey_to_value
 from ipapython.ipa_log_manager import *
 from ipapython.dn import DN
 
+if six.PY3:
+    unicode = str
+
+register = Registry()
+
 ACI_NAME_PREFIX_SEP = ":"
 
 _type_map = {
@@ -220,7 +228,7 @@ def _make_aci(ldap, current, aciname, kw):
     if 'aciprefix' not in kw:
         raise errors.ValidationError(name='aciprefix', error=_('ACI prefix is required'))
 
-    if sum(valid.itervalues()) == 0:
+    if sum(valid.values()) == 0:
         raise errors.ValidationError(name='target', error=_('at least one of: type, filter, subtree, targetgroup, attrs or memberof are required'))
 
     if valid['filter'] + valid['memberof'] > 1:
@@ -241,7 +249,7 @@ def _make_aci(ldap, current, aciname, kw):
         # This will raise NotFound if the permission doesn't exist
         try:
             entry_attrs = api.Command['permission_show'](kw['permission'])['result']
-        except errors.NotFound, e:
+        except errors.NotFound as e:
             if 'test' in kw and not kw.get('test'):
                 raise e
             else:
@@ -301,7 +309,7 @@ def _make_aci(ldap, current, aciname, kw):
             if not target.startswith('ldap:///'):
                 target = 'ldap:///%s' % target
             a.set_target(target)
-    except SyntaxError, e:
+    except SyntaxError as e:
         raise errors.ValidationError(name='target', error=_('Syntax Error: %(error)s') % dict(error=str(e)))
 
     return a
@@ -319,10 +327,8 @@ def _aci_to_kw(ldap, a, test=False, pkey_only=False):
         return kw
     kw['permissions'] = tuple(a.permissions)
     if 'targetattr' in a.target:
-        kw['attrs'] = list(a.target['targetattr']['expression'])
-        for i in xrange(len(kw['attrs'])):
-            kw['attrs'][i] = unicode(kw['attrs'][i])
-        kw['attrs'] = tuple(kw['attrs'])
+        kw['attrs'] = tuple(unicode(e)
+                            for e in a.target['targetattr']['expression'])
     if 'targetfilter' in a.target:
         target = a.target['targetfilter']['expression']
         if target.startswith('(memberOf=') or target.startswith('memberOf='):
@@ -364,20 +370,20 @@ def _aci_to_kw(ldap, a, test=False, pkey_only=False):
         groupdn = DN(groupdn)
         if len(groupdn) and groupdn[0].attr == 'cn':
             dn = DN()
-            entry_attrs = {}
+            entry = ldap.make_entry(dn)
             try:
-                (dn, entry_attrs) = ldap.get_entry(groupdn, ['cn'])
-            except errors.NotFound, e:
+                entry = ldap.get_entry(groupdn, ['cn'])
+            except errors.NotFound as e:
                 # FIXME, use real name here
                 if test:
                     dn = DN(('cn', 'test'), api.env.container_permission,
                             api.env.basedn)
-                    entry_attrs = {'cn': [u'test']}
-            if api.env.container_permission in dn:
-                kw['permission'] = entry_attrs['cn'][0]
+                    entry = ldap.make_entry(dn, {'cn': [u'test']})
+            if api.env.container_permission in entry.dn:
+                kw['permission'] = entry['cn'][0]
             else:
-                if 'cn' in entry_attrs:
-                    kw['group'] = entry_attrs['cn'][0]
+                if 'cn' in entry:
+                    kw['group'] = entry['cn'][0]
 
     return kw
 
@@ -386,7 +392,7 @@ def _convert_strings_to_acis(acistrs):
     for a in acistrs:
         try:
             acis.append(ACI(a))
-        except SyntaxError, e:
+        except SyntaxError as e:
             root_logger.warning("Failed to parse: %s" % a)
     return acis
 
@@ -419,6 +425,8 @@ _prefix_option = StrEnum('aciprefix',
                 values=_valid_prefix_values,
                 )
 
+
+@register()
 class aci(Object):
     """
     ACI object.
@@ -501,8 +509,8 @@ class aci(Object):
         ),
     )
 
-api.register(aci)
 
+@register()
 class aci_add(crud.Create):
     """
     Create new ACI.
@@ -532,19 +540,19 @@ class aci_add(crud.Create):
 
         newaci = _make_aci(ldap, None, aciname, kw)
 
-        (dn, entry_attrs) = ldap.get_entry(self.api.env.basedn, ['aci'])
+        entry = ldap.get_entry(self.api.env.basedn, ['aci'])
 
-        acis = _convert_strings_to_acis(entry_attrs.get('aci', []))
+        acis = _convert_strings_to_acis(entry.get('aci', []))
         for a in acis:
             # FIXME: add check for permission_group = permission_group
             if a.isequal(newaci) or newaci.name == a.name:
                 raise errors.DuplicateEntry()
 
         newaci_str = unicode(newaci)
-        entry_attrs['aci'].append(newaci_str)
+        entry.setdefault('aci', []).append(newaci_str)
 
         if not kw.get('test', False):
-            ldap.update_entry(dn, entry_attrs)
+            ldap.update_entry(entry)
 
         if kw.get('raw', False):
             result = dict(aci=unicode(newaci_str))
@@ -552,12 +560,11 @@ class aci_add(crud.Create):
             result = _aci_to_kw(ldap, newaci, kw.get('test', False))
         return dict(
             result=result,
-            value=aciname,
+            value=pkey_to_value(aciname, kw),
         )
 
-api.register(aci_add)
-
 
+@register()
 class aci_del(crud.Delete):
     """
     Delete ACI.
@@ -577,9 +584,9 @@ class aci_del(crud.Delete):
         """
         ldap = self.api.Backend.ldap2
 
-        (dn, entry_attrs) = ldap.get_entry(self.api.env.basedn, ['aci'])
+        entry = ldap.get_entry(self.api.env.basedn, ['aci'])
 
-        acistrs = entry_attrs.get('aci', [])
+        acistrs = entry.get('aci', [])
         acis = _convert_strings_to_acis(acistrs)
         aci = _find_aci_by_name(acis, aciprefix, aciname)
         for a in acistrs:
@@ -588,18 +595,17 @@ class aci_del(crud.Delete):
                 acistrs.remove(a)
                 break
 
-        entry_attrs['aci'] = acistrs
+        entry['aci'] = acistrs
 
-        ldap.update_entry(dn, entry_attrs)
+        ldap.update_entry(entry)
 
         return dict(
             result=True,
-            value=aciname,
+            value=pkey_to_value(aciname, options),
         )
 
-api.register(aci_del)
-
 
+@register()
 class aci_mod(crud.Update):
     """
     Modify ACI.
@@ -621,9 +627,9 @@ class aci_mod(crud.Update):
         aciprefix = kw['aciprefix']
         ldap = self.api.Backend.ldap2
 
-        (dn, entry_attrs) = ldap.get_entry(self.api.env.basedn, ['aci'])
+        entry = ldap.get_entry(self.api.env.basedn, ['aci'])
 
-        acis = _convert_strings_to_acis(entry_attrs.get('aci', []))
+        acis = _convert_strings_to_acis(entry.get('aci', []))
         aci = _find_aci_by_name(acis, aciprefix, aciname)
 
         # The strategy here is to convert the ACI we're updating back into
@@ -648,7 +654,7 @@ class aci_mod(crud.Update):
 
         try:
             result = self.api.Command['aci_add'](aciname, **newkw)['result']
-        except Exception, e:
+        except Exception as e:
             # ACI could not be added, try to restore the old deleted ACI and
             # report the ADD error back to user
             try:
@@ -663,12 +669,11 @@ class aci_mod(crud.Update):
             result = _aci_to_kw(ldap, newaci)
         return dict(
             result=result,
-            value=aciname,
+            value=pkey_to_value(aciname, kw),
         )
 
-api.register(aci_mod)
-
 
+@register()
 class aci_find(crud.Search):
     """
     Search for ACIs.
@@ -698,9 +703,9 @@ class aci_find(crud.Search):
     def execute(self, term, **kw):
         ldap = self.api.Backend.ldap2
 
-        (dn, entry_attrs) = ldap.get_entry(self.api.env.basedn, ['aci'])
+        entry = ldap.get_entry(self.api.env.basedn, ['aci'])
 
-        acis = _convert_strings_to_acis(entry_attrs.get('aci', []))
+        acis = _convert_strings_to_acis(entry.get('aci', []))
         results = []
 
         if term:
@@ -748,7 +753,8 @@ class aci_find(crud.Search):
                 pass
             else:
                 for a in acis:
-                    if a.bindrule['expression'] != ('ldap:///%s' % dn):
+                    uri = 'ldap:///%s' % entry.dn
+                    if a.bindrule['expression'] != uri:
                         results.remove(a)
                 acis = list(results)
 
@@ -872,9 +878,8 @@ class aci_find(crud.Search):
             truncated=False,
         )
 
-api.register(aci_find)
-
 
+@register()
 class aci_show(crud.Retrieve):
     """
     Display a single ACI given an ACI name.
@@ -887,7 +892,12 @@ class aci_show(crud.Retrieve):
         ),
     )
 
-    takes_options = (_prefix_option,)
+    takes_options = (
+        _prefix_option,
+        DNParam('location?',
+            label=_('Location of the ACI'),
+        )
+    )
 
     def execute(self, aciname, **kw):
         """
@@ -900,9 +910,10 @@ class aci_show(crud.Retrieve):
         """
         ldap = self.api.Backend.ldap2
 
-        (dn, entry_attrs) = ldap.get_entry(self.api.env.basedn, ['aci'])
+        dn = kw.get('location', self.api.env.basedn)
+        entry = ldap.get_entry(dn, ['aci'])
 
-        acis = _convert_strings_to_acis(entry_attrs.get('aci', []))
+        acis = _convert_strings_to_acis(entry.get('aci', []))
 
         aci = _find_aci_by_name(acis, kw['aciprefix'], aciname)
         if kw.get('raw', False):
@@ -911,12 +922,11 @@ class aci_show(crud.Retrieve):
             result = _aci_to_kw(ldap, aci)
         return dict(
             result=result,
-            value=aciname,
+            value=pkey_to_value(aciname, kw),
         )
 
-api.register(aci_show)
-
 
+@register()
 class aci_rename(crud.Update):
     """
     Rename an ACI.
@@ -940,9 +950,9 @@ class aci_rename(crud.Update):
     def execute(self, aciname, **kw):
         ldap = self.api.Backend.ldap2
 
-        (dn, entry_attrs) = ldap.get_entry(self.api.env.basedn, ['aci'])
+        entry = ldap.get_entry(self.api.env.basedn, ['aci'])
 
-        acis = _convert_strings_to_acis(entry_attrs.get('aci', []))
+        acis = _convert_strings_to_acis(entry.get('aci', []))
         aci = _find_aci_by_name(acis, kw['aciprefix'], aciname)
 
         for a in acis:
@@ -974,7 +984,5 @@ class aci_rename(crud.Update):
             result = _aci_to_kw(ldap, newaci)
         return dict(
             result=result,
-            value=kw['newname'],
+            value=pkey_to_value(kw['newname'], kw),
         )
-
-api.register(aci_rename)
