@@ -20,6 +20,7 @@ import uuid
 import time
 
 import ldap as _ldap
+import six
 
 from ipalib import api, errors, Str, StrEnum, DNParam, _, ngettext
 from ipalib.plugable import Registry
@@ -27,6 +28,9 @@ from ipalib.plugins.baseldap import *
 from ipalib.request import context
 from ipapython.dn import DN
 
+if six.PY3:
+    unicode = str
+
 __doc__ = _("""
 Auto Membership Rule.
 """) + _("""
