@@ -20,12 +20,18 @@
 
 import os
 
+import six
+
 from ipalib import api, errors
 from ipalib import Object, Command
 from ipalib import Flag, Str, IA5Str
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
 
+if six.PY3:
+    unicode = str
+
 __doc__ = _("""
 Automount
 
@@ -193,10 +199,13 @@ automountInformation: -ro,soft,rsize=8192,wsize=8192 nfs.example.com:/vol/arch
 
 """
 
+register = Registry()
+
 DIRECT_MAP_KEY = u'/-'
 DEFAULT_MAPS = (u'auto.direct', )
 DEFAULT_KEYS = (u'/-', )
 
+@register()
 class automountlocation(LDAPObject):
     """
     Location container for automount maps.
@@ -208,6 +217,30 @@ class automountlocation(LDAPObject):
     default_attributes = ['cn']
     label = _('Automount Locations')
     label_singular = _('Automount Location')
+    permission_filter_objectclasses = ['nscontainer']
+    managed_permissions = {
+        'System: Read Automount Configuration': {
+            # Single permission for all automount-related entries
+            'non_object': True,
+            'ipapermlocation': DN(container_dn, api.env.basedn),
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'anonymous',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'objectclass',
+                'automountinformation', 'automountkey', 'description',
+                'automountmapname', 'description',
+            },
+        },
+        'System: Add Automount Locations': {
+            'ipapermright': {'add'},
+            'default_privileges': {'Automount Administrators'},
+        },
+        'System: Remove Automount Locations': {
+            'ipapermright': {'delete'},
+            'default_privileges': {'Automount Administrators'},
+        },
+    }
 
     takes_params = (
         Str('cn',
@@ -218,9 +251,8 @@ class automountlocation(LDAPObject):
         ),
     )
 
-api.register(automountlocation)
-
 
+@register()
 class automountlocation_add(LDAPCreate):
     __doc__ = _('Create a new automount location.')
 
@@ -240,23 +272,19 @@ class automountlocation_add(LDAPCreate):
         return dn
 
 
-api.register(automountlocation_add)
-
-
+@register()
 class automountlocation_del(LDAPDelete):
     __doc__ = _('Delete an automount location.')
 
     msg_summary = _('Deleted automount location "%(value)s"')
 
-api.register(automountlocation_del)
-
 
+@register()
 class automountlocation_show(LDAPRetrieve):
     __doc__ = _('Display an automount location.')
 
-api.register(automountlocation_show)
-
 
+@register()
 class automountlocation_find(LDAPSearch):
     __doc__ = _('Search for an automount location.')
 
@@ -265,9 +293,8 @@ class automountlocation_find(LDAPSearch):
         '%(count)d automount locations matched', 0
     )
 
-api.register(automountlocation_find)
-
 
+@register()
 class automountlocation_tofiles(LDAPQuery):
     __doc__ = _('Generate automount files for a specific location.')
 
@@ -361,9 +388,8 @@ class automountlocation_tofiles(LDAPQuery):
                         )
                     )
 
-api.register(automountlocation_tofiles)
-
 
+@register()
 class automountlocation_import(LDAPQuery):
     __doc__ = _('Import automount files for a specific location.')
 
@@ -386,7 +412,7 @@ class automountlocation_import(LDAPQuery):
             fp = open(filename, 'r')
             map = fp.readlines()
             fp.close()
-        except IOError, e:
+        except IOError as e:
             if e.errno == 2:
                 raise errors.NotFound(
                     reason=_('File %(file)s not found') % {'file': filename}
@@ -432,7 +458,7 @@ class automountlocation_import(LDAPQuery):
                             automountkey=unicode(am[0]),
                             automountinformation=unicode(' '.join(am[1:])))
                 result['keys'].append([am[0], u'auto.master'])
-            except errors.DuplicateEntry, e:
+            except errors.DuplicateEntry as e:
                 if unicode(am[0]) in DEFAULT_KEYS:
                     # ignore conflict when the key was pre-created by the framework
                     pass
@@ -448,7 +474,7 @@ class automountlocation_import(LDAPQuery):
                 try:
                     api.Command['automountmap_add'](args[0], unicode(am[1]))
                     result['maps'].append(am[1])
-                except errors.DuplicateEntry, e:
+                except errors.DuplicateEntry as e:
                     if unicode(am[1]) in DEFAULT_MAPS:
                         # ignore conflict when the map was pre-created by the framework
                         pass
@@ -494,7 +520,7 @@ class automountlocation_import(LDAPQuery):
                             automountkey=key,
                             automountinformation=unicode(' '.join(am[1:])))
                     result['keys'].append([key,m])
-                except errors.DuplicateEntry, e:
+                except errors.DuplicateEntry as e:
                     if options.get('continue', False):
                         result['duplicatekeys'].append(am[0])
                         pass
@@ -554,8 +580,7 @@ class automountlocation_import(LDAPQuery):
                 )
 
 
-api.register(automountlocation_import)
-
+@register()
 class automountmap(LDAPObject):
     """
     Automount map object.
@@ -565,6 +590,7 @@ class automountmap(LDAPObject):
     object_name = _('automount map')
     object_name_plural = _('automount maps')
     object_class = ['automountmap']
+    permission_filter_objectclasses = ['automountmap']
     default_attributes = ['automountmapname', 'description']
 
     takes_params = (
@@ -580,20 +606,43 @@ class automountmap(LDAPObject):
         ),
     )
 
+    managed_permissions = {
+        'System: Add Automount Maps': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Add Automount maps";allow (add) groupdn = "ldap:///cn=Add Automount maps,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Automount Administrators'},
+        },
+        'System: Modify Automount Maps': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'automountmapname', 'description'},
+            'replaces': [
+                '(targetattr = "automountmapname || description")(target = "ldap:///automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Modify Automount maps";allow (write) groupdn = "ldap:///cn=Modify Automount maps,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Automount Administrators'},
+        },
+        'System: Remove Automount Maps': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Remove Automount maps";allow (delete) groupdn = "ldap:///cn=Remove Automount maps,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Automount Administrators'},
+        },
+    }
+
     label = _('Automount Maps')
     label_singular = _('Automount Map')
 
-api.register(automountmap)
-
 
+@register()
 class automountmap_add(LDAPCreate):
     __doc__ = _('Create a new automount map.')
 
     msg_summary = _('Added automount map "%(value)s"')
 
-api.register(automountmap_add)
-
 
+@register()
 class automountmap_del(LDAPDelete):
     __doc__ = _('Delete an automount map.')
 
@@ -603,26 +652,24 @@ class automountmap_del(LDAPDelete):
         assert isinstance(dn, DN)
         # delete optional parental connection (direct maps may not have this)
         try:
-            (dn_, entry_attrs) = ldap.find_entry_by_attr(
+            entry_attrs = ldap.find_entry_by_attr(
                 'automountinformation', keys[0], 'automount',
                 base_dn=DN(self.obj.container_dn, api.env.basedn)
             )
-            ldap.delete_entry(dn_)
+            ldap.delete_entry(entry_attrs)
         except errors.NotFound:
             pass
         return True
 
-api.register(automountmap_del)
-
 
+@register()
 class automountmap_mod(LDAPUpdate):
     __doc__ = _('Modify an automount map.')
 
     msg_summary = _('Modified automount map "%(value)s"')
 
-api.register(automountmap_mod)
-
 
+@register()
 class automountmap_find(LDAPSearch):
     __doc__ = _('Search for an automount map.')
 
@@ -631,15 +678,13 @@ class automountmap_find(LDAPSearch):
         '%(count)d automount maps matched', 0
     )
 
-api.register(automountmap_find)
-
 
+@register()
 class automountmap_show(LDAPRetrieve):
     __doc__ = _('Display an automount map.')
 
-api.register(automountmap_show)
-
 
+@register()
 class automountkey(LDAPObject):
     __doc__ = _('Automount key object.')
 
@@ -648,6 +693,7 @@ class automountkey(LDAPObject):
     object_name = _('automount key')
     object_name_plural = _('automount keys')
     object_class = ['automount']
+    permission_filter_objectclasses = ['automount']
     default_attributes = [
         'automountkey', 'automountinformation', 'description'
     ]
@@ -674,6 +720,35 @@ class automountkey(LDAPObject):
         ),
     )
 
+    managed_permissions = {
+        'System: Add Automount Keys': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///automountkey=*,automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Add Automount keys";allow (add) groupdn = "ldap:///cn=Add Automount keys,cn=permissions,cn=pbac,$SUFFIX";)',
+                '(targetfilter = "(objectclass=automount)")(target = "ldap:///automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Add Automount keys";allow (add) groupdn = "ldap:///cn=Add Automount keys,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Automount Administrators'},
+        },
+        'System: Modify Automount Keys': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'automountinformation', 'automountkey', 'description',
+            },
+            'replaces': [
+                '(targetattr = "automountkey || automountinformation || description")(targetfilter = "(objectclass=automount)")(target = "ldap:///automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Modify Automount keys";allow (write) groupdn = "ldap:///cn=Modify Automount keys,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Automount Administrators'},
+        },
+        'System: Remove Automount Keys': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///automountkey=*,automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Remove Automount keys";allow (delete) groupdn = "ldap:///cn=Remove Automount keys,cn=permissions,cn=pbac,$SUFFIX";)',
+                '(targetfilter = "(objectclass=automount)")(target = "ldap:///automountmapname=*,cn=automount,$SUFFIX")(version 3.0;acl "permission:Remove Automount keys";allow (delete) groupdn = "ldap:///cn=Remove Automount keys,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Automount Administrators'},
+        },
+    }
+
     num_parents = 2
     label = _('Automount Keys')
     label_singular = _('Automount Key')
@@ -715,7 +790,7 @@ class automountkey(LDAPObject):
         # First we look with the information given, then try to search for
         # the right entry.
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, ['*'])
+            dn = ldap.get_entry(dn, ['*']).dn
         except errors.NotFound:
             if kwargs.get('automountinformation', False):
                 sfilter = '(&(automountkey=%s)(automountinformation=%s))' % \
@@ -732,7 +807,7 @@ class automountkey(LDAPObject):
                 raise errors.NotFound(reason=_('More than one entry with key %(key)s found, use --info to select specific entry.') % dict(key=pkey))
             if truncated:
                 raise errors.LimitsExceeded()
-            dn = entries[0][0]
+            dn = entries[0].dn
 
         return dn
 
@@ -785,9 +860,8 @@ class automountkey(LDAPObject):
                 else: return
             self.handle_duplicate_entry(location, map, self.get_pk(key, info))
 
-api.register(automountkey)
-
 
+@register()
 class automountkey_add(LDAPCreate):
     __doc__ = _('Create a new automount key.')
 
@@ -812,12 +886,11 @@ class automountkey_add(LDAPCreate):
         options[self.obj.primary_key.name] = self.obj.get_pk(key, info)
         options['add_operation'] = True
         result = super(automountkey_add, self).execute(*keys, **options)
-        result['value'] = options['automountkey']
+        result['value'] = pkey_to_value(options['automountkey'], options)
         return result
 
-api.register(automountkey_add)
-
 
+@register()
 class automountmap_add_indirect(LDAPCreate):
     __doc__ = _('Create a new indirect mount point.')
 
@@ -869,9 +942,8 @@ class automountmap_add_indirect(LDAPCreate):
             raise
         return result
 
-api.register(automountmap_add_indirect)
-
 
+@register()
 class automountkey_del(LDAPDelete):
     __doc__ = _('Delete an automount key.')
 
@@ -908,12 +980,11 @@ class automountkey_del(LDAPDelete):
                                             options['automountkey'],
                                             options.get('automountinformation', None))
         result = super(automountkey_del, self).execute(*keys, **options)
-        result['value'] = options['automountkey']
+        result['value'] = pkey_to_value([options['automountkey']], options)
         return result
 
-api.register(automountkey_del)
-
 
+@register()
 class automountkey_mod(LDAPUpdate):
     __doc__ = _('Modify an automount key.')
 
@@ -955,7 +1026,7 @@ class automountkey_mod(LDAPUpdate):
                 # automountinformation attribute of existing LDAP object needs
                 # to be retrieved so that RDN can be generated
                 dn = self.obj.get_dn(*keys, **options)
-                (dn_, entry_attrs_) = ldap.get_entry(dn, ['automountinformation'])
+                entry_attrs_ = ldap.get_entry(dn, ['automountinformation'])
                 new_info = entry_attrs_.get('automountinformation', [])[0]
 
             # automounkey attribute cannot be overwritten so that get_dn()
@@ -967,12 +1038,11 @@ class automountkey_mod(LDAPUpdate):
                 options['rename'] = new_rdn
 
         result = super(automountkey_mod, self).execute(*keys, **options)
-        result['value'] = options['automountkey']
+        result['value'] = pkey_to_value(options['automountkey'], options)
         return result
 
-api.register(automountkey_mod)
-
 
+@register()
 class automountkey_find(LDAPSearch):
     __doc__ = _('Search for an automount key.')
 
@@ -981,9 +1051,8 @@ class automountkey_find(LDAPSearch):
         '%(count)d automount keys matched', 0
     )
 
-api.register(automountkey_find)
-
 
+@register()
 class automountkey_show(LDAPRetrieve):
     __doc__ = _('Display an automount key.')
 
@@ -1011,7 +1080,5 @@ class automountkey_show(LDAPRetrieve):
                                             options.get('automountinformation', None))
 
         result = super(automountkey_show, self).execute(*keys, **options)
-        result['value'] = options['automountkey']
+        result['value'] = pkey_to_value(options['automountkey'], options)
         return result
-
-api.register(automountkey_show)
