@@ -25,6 +25,8 @@ import time
 from copy import deepcopy
 import base64
 
+import six
+
 from ipalib import api, crud, errors
 from ipalib import Method, Object, Command
 from ipalib import Flag, Int, Str
@@ -36,6 +38,9 @@ from ipalib.capabilities import client_has_capability
 from ipapython.dn import DN, RDN
 from ipapython.version import API_VERSION
 
+if six.PY3:
+    unicode = str
+
 DNA_MAGIC = -1
 
 global_output_params = (
@@ -230,7 +235,7 @@ def entry_from_entry(entry, newentry):
     """
 
     # Wipe out the current data
-    for e in entry.keys():
+    for e in list(entry):
         del entry[e]
 
     # Re-populate it with new wentry
@@ -240,10 +245,10 @@ def entry_from_entry(entry, newentry):
 def entry_to_dict(entry, **options):
     if options.get('raw', False):
         result = {}
-        for attr in entry.iterkeys():
+        for attr in entry:
             if attr.lower() == 'attributelevelrights':
                 value = entry[attr]
-            elif entry.conn.get_type(attr) is str:
+            elif entry.conn.get_attribute_type(attr) is bytes:
                 value = entry.raw[attr]
             else:
                 value = list(entry.raw[attr])
@@ -254,7 +259,7 @@ def entry_to_dict(entry, **options):
                         pass
             result[attr] = value
     else:
-        result = dict((k.lower(), v) for (k, v) in entry.iteritems())
+        result = dict((k.lower(), v) for (k, v) in entry.items())
     if options.get('all', False):
         result['dn'] = entry.dn
     return result
@@ -298,7 +303,7 @@ def wait_for_value(ldap, dn, attr, value):
         entry_attrs = ldap.get_entry(dn, ['*'])
         if attr in entry_attrs:
             if isinstance(entry_attrs[attr], (list, tuple)):
-                values = map(lambda y:y.lower(), entry_attrs[attr])
+                values = [y.lower() for y in entry_attrs[attr]]
                 if value.lower() in values:
                     break
             else:
@@ -311,7 +316,7 @@ def wait_for_value(ldap, dn, attr, value):
 def validate_externalhost(ugettext, hostname):
     try:
         validate_hostname(hostname, check_fqdn=False, allow_underscore=True)
-    except ValueError, e:
+    except ValueError as e:
         return unicode(e)
 
 
@@ -445,12 +450,12 @@ def remove_external_post_callback(ldap, dn, entry_attrs, failed, completed,
 
     # Run through the failures and gracefully remove any member defined
     # as an external member.
+    completed_external = 0
     if memberattr in failed and membertype in failed[memberattr]:
         entry_attrs_ = ldap.get_entry(dn, [externalattr])
         dn = entry_attrs_.dn
         external_entries = entry_attrs_.get(externalattr, [])
         failed_entries = []
-        completed_external = 0
 
         for entry in failed[memberattr][membertype]:
             membername = entry[0].lower()
@@ -625,23 +630,104 @@ class LDAPObject(Object):
                 )
 
     def has_objectclass(self, classes, objectclass):
-        oc = map(lambda x:x.lower(),classes)
+        oc = [x.lower() for x in classes]
         return objectclass.lower() in oc
 
     def convert_attribute_members(self, entry_attrs, *keys, **options):
         if options.get('raw', False):
             return
+
+        container_dns = {}
+        new_attrs = {}
+
         for attr in self.attribute_members:
-            for member in entry_attrs.setdefault(attr, []):
+            try:
+                value = entry_attrs.raw[attr]
+            except KeyError:
+                continue
+            del entry_attrs[attr]
+
+            for member in value:
+                memberdn = DN(member)
                 for ldap_obj_name in self.attribute_members[attr]:
                     ldap_obj = self.api.Object[ldap_obj_name]
-                    container_dn = DN(ldap_obj.container_dn, api.env.basedn)
-                    if member.endswith(container_dn):
-                        new_attr = '%s_%s' % (attr, ldap_obj.name)
-                        entry_attrs.setdefault(new_attr, []).append(
-                            ldap_obj.get_primary_key_from_dn(member)
-                        )
-            del entry_attrs[attr]
+                    try:
+                        container_dn = container_dns[ldap_obj_name]
+                    except KeyError:
+                        container_dn = DN(ldap_obj.container_dn, api.env.basedn)
+                        container_dns[ldap_obj_name] = container_dn
+
+                    if memberdn.endswith(container_dn):
+                        new_value = ldap_obj.get_primary_key_from_dn(memberdn)
+                        new_attr_name = '%s_%s' % (attr, ldap_obj.name)
+                        try:
+                            new_attr = new_attrs[new_attr_name]
+                        except KeyError:
+                            new_attr = entry_attrs.setdefault(new_attr_name, [])
+                            new_attrs[new_attr_name] = new_attr
+                        new_attr.append(new_value)
+                        break
+
+    def get_indirect_members(self, entry_attrs, attrs_list):
+        if 'memberindirect' in attrs_list:
+            self.get_memberindirect(entry_attrs)
+        if 'memberofindirect' in attrs_list:
+            self.get_memberofindirect(entry_attrs)
+
+    def get_memberindirect(self, group_entry):
+        """
+        Get indirect members
+        """
+
+        mo_filter = self.backend.make_filter({'memberof': group_entry.dn})
+        filter = self.backend.combine_filters(
+            ('(member=*)', mo_filter), self.backend.MATCH_ALL)
+        try:
+            result, truncated = self.backend.find_entries(
+                base_dn=self.api.env.basedn,
+                filter=filter,
+                attrs_list=['member'],
+                size_limit=-1, # paged search will get everything anyway
+                paged_search=True)
+            if truncated:
+                raise errors.LimitsExceeded()
+        except errors.NotFound:
+            result = []
+
+        indirect = set()
+        for entry in result:
+            indirect.update(entry.raw.get('member', []))
+        indirect.difference_update(group_entry.raw.get('member', []))
+
+        if indirect:
+            group_entry.raw['memberindirect'] = list(indirect)
+
+    def get_memberofindirect(self, entry):
+
+        dn = entry.dn
+        filter = self.backend.make_filter(
+            {'member': dn, 'memberuser': dn, 'memberhost': dn})
+        try:
+            result, truncated = self.backend.find_entries(
+                base_dn=self.api.env.basedn,
+                filter=filter,
+                attrs_list=[''])
+            if truncated:
+                raise errors.LimitsExceeded()
+        except errors.NotFound:
+            result = []
+
+        direct = set()
+        indirect = set(entry.raw.get('memberof', []))
+        for group_entry in result:
+            dn = str(group_entry.dn)
+            if dn in indirect:
+                indirect.remove(dn)
+                direct.add(dn)
+
+        entry.raw['memberof'] = list(direct)
+        if indirect:
+            entry.raw['memberofindirect'] = list(indirect)
 
     def get_password_attributes(self, ldap, dn, entry_attrs):
         """
@@ -675,7 +761,7 @@ class LDAPObject(Object):
     def handle_duplicate_entry(self, *keys):
         try:
             pkey = keys[-1]
-        except KeyError:
+        except IndexError:
             pkey = ''
         raise errors.DuplicateEntry(
             message=self.already_exists_msg % {
@@ -710,10 +796,10 @@ class LDAPObject(Object):
         attrs = self.api.Backend.ldap2.schema.attribute_types(objectclasses)
         attrlist = []
         # Go through the MUST first
-        for (oid, attr) in attrs[0].iteritems():
+        for (oid, attr) in attrs[0].items():
             attrlist.append(attr.names[0].lower())
         # And now the MAY
-        for (oid, attr) in attrs[1].iteritems():
+        for (oid, attr) in attrs[1].items():
             attrlist.append(attr.names[0].lower())
         json_dict['aciattrs'] = attrlist
         attrlist.sort()
@@ -726,7 +812,7 @@ class LDAPObject(Object):
 # addattr can cause parameters to have more than one value even if not defined
 # as multivalue, make sure this isn't the case
 def _check_single_value_attrs(params, entry_attrs):
-    for (a, v) in entry_attrs.iteritems():
+    for (a, v) in entry_attrs.items():
         if isinstance(v, (list, tuple)) and len(v) > 1:
             if a in params and not params[a].multivalue:
                 raise errors.OnlyOneValueAllowed(attr=a)
@@ -734,8 +820,8 @@ def _check_single_value_attrs(params, entry_attrs):
 # setattr or --option='' can cause parameters to be empty that are otherwise
 # required, make sure we enforce that.
 def _check_empty_attrs(params, entry_attrs):
-    for (a, v) in entry_attrs.iteritems():
-        if v is None or (isinstance(v, basestring) and len(v) == 0):
+    for (a, v) in entry_attrs.items():
+        if v is None or (isinstance(v, six.string_types) and len(v) == 0):
             if a in params and params[a].required:
                 raise errors.RequirementError(name=a)
 
@@ -756,7 +842,7 @@ def _check_limit_object_class(attributes, attrs, allow_only):
         return
     limitattrs = deepcopy(attrs)
     # Go through the MUST first
-    for (oid, attr) in attributes[0].iteritems():
+    for (oid, attr) in attributes[0].items():
         if attr.names[0].lower() in limitattrs:
             if not allow_only:
                 raise errors.ObjectclassViolation(
@@ -764,7 +850,7 @@ def _check_limit_object_class(attributes, attrs, allow_only):
                         attribute=attr.names[0].lower()))
             limitattrs.remove(attr.names[0].lower())
     # And now the MAY
-    for (oid, attr) in attributes[1].iteritems():
+    for (oid, attr) in attributes[1].items():
         if attr.names[0].lower() in limitattrs:
             if not allow_only:
                 raise errors.ObjectclassViolation(
@@ -932,9 +1018,9 @@ last, after all sets and adds."""),
         setdict = self._convert_2_dict(ldap, options.get('setattr', []))
         deldict = self._convert_2_dict(ldap, options.get('delattr', []))
 
-        setattrs = set(setdict.keys())
-        addattrs = set(adddict.keys())
-        delattrs = set(deldict.keys())
+        setattrs = set(setdict)
+        addattrs = set(adddict)
+        delattrs = set(deldict)
 
         if dn is None:
             direct_add = addattrs
@@ -946,7 +1032,7 @@ last, after all sets and adds."""),
             direct_del = setattrs & delattrs
             needldapattrs = list((addattrs | delattrs) - setattrs)
 
-        for attr, val in setdict.iteritems():
+        for attr, val in setdict.items():
             entry_attrs[attr] = val
 
         for attr in direct_add:
@@ -980,7 +1066,7 @@ last, after all sets and adds."""),
             # Provide a nice error message when user tries to delete an
             # attribute that does not exist on the entry (and user is not
             # adding it)
-            names = set(n.lower() for n in old_entry.keys())
+            names = set(n.lower() for n in old_entry)
             del_nonexisting = delattrs - (names | setattrs | addattrs)
             if del_nonexisting:
                 raise errors.ValidationError(name=del_nonexisting.pop(),
@@ -996,7 +1082,7 @@ last, after all sets and adds."""),
                     try:
                         entry_attrs[attr].remove(delval)
                     except ValueError:
-                        if isinstance(delval, str):
+                        if isinstance(delval, bytes):
                             # This is a Binary value, base64 encode it
                             delval = unicode(base64.b64encode(delval))
                         raise errors.AttrValueNotFound(attr=attr, value=delval)
@@ -1018,9 +1104,9 @@ last, after all sets and adds."""),
                 # validate, convert and encode params
                 try:
                    value = param(value)
-                except errors.ValidationError, err:
+                except errors.ValidationError as err:
                     raise errors.ValidationError(name=attr, error=err.error)
-                except errors.ConversionError, err:
+                except errors.ConversionError as err:
                     raise errors.ConversionError(name=attr, error=err.error)
                 if isinstance(value, tuple):
                     value = list(value)
@@ -1062,7 +1148,8 @@ last, after all sets and adds."""),
             while True:
                 try:
                     return func(*call_args, **call_kwargs)
-                except errors.ExecutionError, e:
+                except errors.ExecutionError as exc:
+                    e = exc
                     if not callbacks:
                         raise
                     # call exc_callback in the next loop
@@ -1146,8 +1233,8 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
                 *keys, **options)
 
         _check_single_value_attrs(self.params, entry_attrs)
-        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.limit_object_classes), entry_attrs.keys(), allow_only=True)
-        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), entry_attrs.keys(), allow_only=False)
+        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.limit_object_classes), list(entry_attrs), allow_only=True)
+        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), list(entry_attrs), allow_only=False)
 
         try:
             self._exc_wrapper(keys, options, ldap.add_entry)(entry_attrs)
@@ -1185,6 +1272,8 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
+        self.obj.get_indirect_members(entry_attrs, attrs_list)
+
         for callback in self.get_callbacks('post'):
             entry_attrs.dn = callback(
                 self, ldap, entry_attrs.dn, entry_attrs, *keys, **options)
@@ -1216,17 +1305,6 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
     def interactive_prompt_callback(self, kw):
         return
 
-    # list of attributes we want exported to JSON
-    json_friendly_attributes = (
-        'takes_args',
-    )
-
-    def __json__(self):
-        json_dict = dict(
-            (a, getattr(self, a)) for a in self.json_friendly_attributes
-        )
-        json_dict['takes_options'] = list(self.get_json_options())
-        return json_dict
 
 class LDAPQuery(BaseLDAPCommand, crud.PKQuery):
     """
@@ -1238,17 +1316,6 @@ class LDAPQuery(BaseLDAPCommand, crud.PKQuery):
         for arg in super(LDAPQuery, self).get_args():
             yield arg
 
-    # list of attributes we want exported to JSON
-    json_friendly_attributes = (
-        'takes_args',
-    )
-
-    def __json__(self):
-        json_dict = dict(
-            (a, getattr(self, a)) for a in self.json_friendly_attributes
-        )
-        json_dict['takes_options'] = list(self.get_json_options())
-        return json_dict
 
 class LDAPMultiQuery(LDAPQuery):
     """
@@ -1308,6 +1375,8 @@ class LDAPRetrieve(LDAPQuery):
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
+        self.obj.get_indirect_members(entry_attrs, attrs_list)
+
         if options.get('rights', False) and options.get('all', False):
             entry_attrs['attributelevelrights'] = get_effective_rights(
                 ldap, entry_attrs.dn)
@@ -1404,8 +1473,8 @@ class LDAPUpdate(LDAPQuery, crud.Update):
                 self, ldap, entry_attrs.dn, entry_attrs, attrs_list,
                 *keys, **options)
 
-        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.limit_object_classes), entry_attrs.keys(), allow_only=True)
-        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), entry_attrs.keys(), allow_only=False)
+        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.limit_object_classes), list(entry_attrs), allow_only=True)
+        _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), list(entry_attrs), allow_only=False)
 
         rdnupdate = False
         try:
@@ -1417,10 +1486,12 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             if self.obj.rdn_is_primary_key and self.obj.primary_key.name in entry_attrs:
                 try:
                     # RDN change
-                    self._exc_wrapper(keys, options, ldap.update_entry_rdn)(
+                    new_dn = DN((self.obj.primary_key.name,
+                                 entry_attrs[self.obj.primary_key.name]),
+                                *entry_attrs.dn[1:])
+                    self._exc_wrapper(keys, options, ldap.move_entry)(
                         entry_attrs.dn,
-                        RDN((self.obj.primary_key.name,
-                             entry_attrs[self.obj.primary_key.name])))
+                        new_dn)
 
                     rdnkeys = keys[:-1] + (entry_attrs[self.obj.primary_key.name], )
                     entry_attrs.dn = self.obj.get_dn(*rdnkeys)
@@ -1438,11 +1509,11 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             # mean an error occurred, just that there were no other updates to
             # perform.
             update = self._exc_wrapper(keys, options, ldap.get_entry)(
-                entry_attrs.dn, entry_attrs.keys())
+                entry_attrs.dn, list(entry_attrs))
             update.update(entry_attrs)
 
             self._exc_wrapper(keys, options, ldap.update_entry)(update)
-        except errors.EmptyModlist, e:
+        except errors.EmptyModlist as e:
             if not rdnupdate:
                 raise e
         except errors.NotFound:
@@ -1456,6 +1527,8 @@ class LDAPUpdate(LDAPQuery, crud.Update):
                 format=_('the entry was deleted while being modified')
             )
 
+        self.obj.get_indirect_members(entry_attrs, attrs_list)
+
         if options.get('rights', False) and options.get('all', False):
             entry_attrs['attributelevelrights'] = get_effective_rights(
                 ldap, entry_attrs.dn)
@@ -1498,6 +1571,8 @@ class LDAPDelete(LDAPMultiQuery):
 
     has_output_params = global_output_params
 
+    subtree_delete = True
+
     def execute(self, *keys, **options):
         ldap = self.obj.backend
 
@@ -1533,6 +1608,8 @@ class LDAPDelete(LDAPMultiQuery):
             except errors.NotFound:
                 self.obj.handle_not_found(*nkeys)
             except errors.NotAllowedOnNonLeaf:
+                if not self.subtree_delete:
+                    raise
                 # this entry is not a leaf entry, delete all child nodes
                 delete_subtree(dn)
 
@@ -1585,6 +1662,7 @@ class LDAPModMember(LDAPQuery):
     """
     member_attributes = ['member']
     member_param_doc = _('%s')
+    member_param_label = _('member %s')
     member_count_out = ('%i member processed.', '%i members processed.')
 
     def get_options(self):
@@ -1595,9 +1673,9 @@ class LDAPModMember(LDAPQuery):
                 ldap_obj = self.api.Object[ldap_obj_name]
                 name = to_cli(ldap_obj_name)
                 doc = self.member_param_doc % ldap_obj.object_name_plural
+                label = self.member_param_label % ldap_obj.object_name
                 yield Str('%s*' % name, cli_name='%ss' % name, doc=doc,
-                          label=_('member %s') % ldap_obj.object_name,
-                          csv=True, alwaysask=True)
+                          label=label, csv=True, alwaysask=True)
 
     def get_member_dns(self, **options):
         dns = {}
@@ -1617,7 +1695,7 @@ class LDAPModMember(LDAPQuery):
                     ldap_obj = self.api.Object[ldap_obj_name]
                     try:
                         dns[attr][ldap_obj_name].append(ldap_obj.get_dn(name))
-                    except errors.PublicError, e:
+                    except errors.PublicError as e:
                         failed[attr][ldap_obj_name].append((name, unicode(e)))
         return (dns, failed)
 
@@ -1657,7 +1735,7 @@ class LDAPAddMember(LDAPModMember):
             assert isinstance(dn, DN)
 
         completed = 0
-        for (attr, objs) in member_dns.iteritems():
+        for (attr, objs) in member_dns.items():
             for ldap_obj_name in objs:
                 for m_dn in member_dns[attr][ldap_obj_name]:
                     assert isinstance(m_dn, DN)
@@ -1665,7 +1743,7 @@ class LDAPAddMember(LDAPModMember):
                         continue
                     try:
                         ldap.add_entry_to_group(m_dn, dn, attr, allow_same=self.allow_same)
-                    except errors.PublicError, e:
+                    except errors.PublicError as e:
                         ldap_obj = self.api.Object[ldap_obj_name]
                         failed[attr][ldap_obj_name].append((
                             ldap_obj.get_primary_key_from_dn(m_dn),
@@ -1690,6 +1768,8 @@ class LDAPAddMember(LDAPModMember):
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
+        self.obj.get_indirect_members(entry_attrs, attrs_list)
+
         for callback in self.get_callbacks('post'):
             (completed, entry_attrs.dn) = callback(
                 self, ldap, completed, failed, entry_attrs.dn, entry_attrs,
@@ -1756,15 +1836,15 @@ class LDAPRemoveMember(LDAPModMember):
             assert isinstance(dn, DN)
 
         completed = 0
-        for (attr, objs) in member_dns.iteritems():
-            for ldap_obj_name, m_dns in objs.iteritems():
+        for (attr, objs) in member_dns.items():
+            for ldap_obj_name, m_dns in objs.items():
                 for m_dn in m_dns:
                     assert isinstance(m_dn, DN)
                     if not m_dn:
                         continue
                     try:
                         ldap.remove_entry_from_group(m_dn, dn, attr)
-                    except errors.PublicError, e:
+                    except errors.PublicError as e:
                         ldap_obj = self.api.Object[ldap_obj_name]
                         failed[attr][ldap_obj_name].append((
                             ldap_obj.get_primary_key_from_dn(m_dn),
@@ -1792,6 +1872,8 @@ class LDAPRemoveMember(LDAPModMember):
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
+        self.obj.get_indirect_members(entry_attrs, attrs_list)
+
         for callback in self.get_callbacks('post'):
             (completed, entry_attrs.dn) = callback(
                 self, ldap, completed, failed, entry_attrs.dn, entry_attrs,
@@ -1846,14 +1928,14 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
     takes_options = (
         Int('timelimit?',
             label=_('Time Limit'),
-            doc=_('Time limit of search in seconds'),
+            doc=_('Time limit of search in seconds (0 is unlimited)'),
             flags=['no_display'],
             minvalue=0,
             autofill=False,
         ),
         Int('sizelimit?',
             label=_('Size Limit'),
-            doc=_('Maximum number of entries returned'),
+            doc=_('Maximum number of entries returned (0 is unlimited)'),
             flags=['no_display'],
             minvalue=0,
             autofill=False,
@@ -1966,7 +2048,7 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
             config_attrs = config.get(
                 self.obj.search_attributes_config, [])
             if len(config_attrs) == 1 and (
-                isinstance(config_attrs[0], basestring)):
+                isinstance(config_attrs[0], six.string_types)):
                 search_attrs = config_attrs[0].split(',')
 
         search_kw['objectclass'] = self.obj.object_class
@@ -2012,6 +2094,7 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
 
         if not options.get('raw', False):
             for e in entries:
+                self.obj.get_indirect_members(e, attrs_list)
                 self.obj.convert_attribute_members(e, *args, **options)
 
         for (i, e) in enumerate(entries):
@@ -2037,17 +2120,6 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
     def interactive_prompt_callback(self, kw):
         return
 
-    # list of attributes we want exported to JSON
-    json_friendly_attributes = (
-        'takes_args',
-    )
-
-    def __json__(self):
-        json_dict = dict(
-            (a, getattr(self, a)) for a in self.json_friendly_attributes
-        )
-        json_dict['takes_options'] = list(self.get_json_options())
-        return json_dict
 
 class LDAPModReverseMember(LDAPQuery):
     """
@@ -2132,12 +2204,12 @@ class LDAPAddReverseMember(LDAPModReverseMember):
                         completed = completed + 1
                     else:
                         failed['member'][self.reverse_attr].append((attr, result['failed']['member'][self.member_attr][0][1]))
-                except errors.NotFound, e:
+                except errors.NotFound as e:
                     msg = str(e)
                     (attr, msg) = msg.split(':', 1)
                     failed['member'][self.reverse_attr].append((attr, unicode(msg.strip())))
 
-            except errors.PublicError, e:
+            except errors.PublicError as e:
                 failed['member'][self.reverse_attr].append((attr, unicode(msg)))
 
         # Update the member data.
@@ -2233,12 +2305,12 @@ class LDAPRemoveReverseMember(LDAPModReverseMember):
                         completed = completed + 1
                     else:
                         failed['member'][self.reverse_attr].append((attr, result['failed']['member'][self.member_attr][0][1]))
-                except errors.NotFound, e:
+                except errors.NotFound as e:
                     msg = str(e)
                     (attr, msg) = msg.split(':', 1)
                     failed['member'][self.reverse_attr].append((attr, unicode(msg.strip())))
 
-            except errors.PublicError, e:
+            except errors.PublicError as e:
                 failed['member'][self.reverse_attr].append((attr, unicode(msg)))
 
         # Update the member data.
@@ -2273,3 +2345,117 @@ class LDAPRemoveReverseMember(LDAPModReverseMember):
 
     def interactive_prompt_callback(self, kw):
         return
+
+
+class LDAPModAttribute(LDAPQuery):
+
+    attribute = None
+
+    has_output = output.standard_entry
+
+    def get_options(self):
+        for option in super(LDAPModAttribute, self).get_options():
+            yield option
+
+        option = self.obj.params[self.attribute]
+        attribute = 'virtual_attribute' not in option.flags
+        yield option.clone(attribute=attribute, alwaysask=True)
+
+    def _update_attrs(self, update, entry_attrs):
+        raise NotImplementedError("%s.update_attrs()", self.__class__.__name__)
+
+    def execute(self, *keys, **options):
+        ldap = self.obj.backend
+
+        dn = self.obj.get_dn(*keys, **options)
+        entry_attrs = ldap.make_entry(dn, self.args_options_2_entry(**options))
+
+        if options.get('all', False):
+            attrs_list = ['*', self.obj.primary_key.name]
+        else:
+            attrs_list = {self.obj.primary_key.name}
+            attrs_list.update(entry_attrs.keys())
+            attrs_list = list(attrs_list)
+
+        for callback in self.get_callbacks('pre'):
+            entry_attrs.dn = callback(
+                self, ldap, entry_attrs.dn, entry_attrs, attrs_list,
+                *keys, **options)
+
+        try:
+            update = self._exc_wrapper(keys, options, ldap.get_entry)(
+                entry_attrs.dn, list(entry_attrs))
+            self._update_attrs(update, entry_attrs)
+
+            self._exc_wrapper(keys, options, ldap.update_entry)(update)
+        except errors.NotFound:
+            self.obj.handle_not_found(*keys)
+
+        try:
+            entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
+                entry_attrs.dn, attrs_list)
+        except errors.NotFound:
+            raise errors.MidairCollision(
+                format=_('the entry was deleted while being modified')
+            )
+
+        for callback in self.get_callbacks('post'):
+            entry_attrs.dn = callback(
+                self, ldap, entry_attrs.dn, entry_attrs, *keys, **options)
+
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+
+        if self.obj.primary_key:
+            pkey = keys[-1]
+        else:
+            pkey = None
+
+        return dict(result=entry_attrs, value=pkey_to_value(pkey, options))
+
+    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys,
+                     **options):
+        assert isinstance(dn, DN)
+        return dn
+
+    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
+        assert isinstance(dn, DN)
+        return dn
+
+    def exc_callback(self, keys, options, exc, call_func, *call_args,
+                     **call_kwargs):
+        raise exc
+
+    def interactive_prompt_callback(self, kw):
+        return
+
+
+class LDAPAddAttribute(LDAPModAttribute):
+    msg_summary = _('added attribute value to entry %(value)')
+
+    def _update_attrs(self, update, entry_attrs):
+        for name, value in entry_attrs.items():
+            old_value = set(update.get(name, []))
+            value_to_add = set(value)
+
+            if not old_value.isdisjoint(value_to_add):
+                raise errors.ExecutionError(
+                    message=_('\'%s\' already contains one or more values'
+                              % name)
+                )
+
+            update[name] = list(old_value | value_to_add)
+
+
+class LDAPRemoveAttribute(LDAPModAttribute):
+    msg_summary = _('removed attribute values from entry %(value)')
+
+    def _update_attrs(self, update, entry_attrs):
+        for name, value in entry_attrs.items():
+            old_value = set(update.get(name, []))
+            value_to_remove = set(value)
+
+            if not value_to_remove.issubset(old_value):
+                raise errors.AttrValueNotFound(
+                    attr=name, value=_("one or more values to remove"))
+
+            update[name] = list(old_value - value_to_remove)
