@@ -20,6 +20,7 @@
 
 from ipalib import api
 from ipalib import Bool, Int, Str, IA5Str, StrEnum, DNParam
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib.plugins.selinuxusermap import validate_selinuxuser
 from ipalib import _
@@ -75,11 +76,9 @@ EXAMPLES:
    ipa config-mod --ipaselinuxusermaporder='guest_u:s0$xguest_u:s0$user_u:s0-s0:c0.c1023$staff_u:s0-s0:c0.c1023$unconfined_u:s0-s0:c0.c1023'
 """)
 
-def validate_searchtimelimit(ugettext, limit):
-    if limit == 0:
-        raise ValidationError(name='ipasearchtimelimit', error=_('searchtimelimit must be -1 or > 1.'))
-    return None
+register = Registry()
 
+@register()
 class config(LDAPObject):
     """
     IPA configuration object
@@ -92,7 +91,30 @@ class config(LDAPObject):
         'ipamigrationenabled', 'ipacertificatesubjectbase',
         'ipapwdexpadvnotify', 'ipaselinuxusermaporder',
         'ipaselinuxusermapdefault', 'ipaconfigstring', 'ipakrbauthzdata',
+        'ipauserauthtype'
     ]
+    container_dn = DN(('cn', 'ipaconfig'), ('cn', 'etc'))
+    permission_filter_objectclasses = ['ipaguiconfig']
+    managed_permissions = {
+        'System: Read Global Configuration': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'objectclass',
+                'ipacertificatesubjectbase', 'ipaconfigstring',
+                'ipadefaultemaildomain', 'ipadefaultloginshell',
+                'ipadefaultprimarygroup', 'ipagroupobjectclasses',
+                'ipagroupsearchfields', 'ipahomesrootdir',
+                'ipakrbauthzdata', 'ipamaxusernamelength',
+                'ipamigrationenabled', 'ipapwdexpadvnotify',
+                'ipaselinuxusermapdefault', 'ipaselinuxusermaporder',
+                'ipasearchrecordslimit', 'ipasearchtimelimit',
+                'ipauserauthtype', 'ipauserobjectclasses',
+                'ipausersearchfields', 'ipacustomfields',
+            },
+        },
+    }
 
     label = _('Configuration')
     label_singular = _('Configuration')
@@ -123,16 +145,16 @@ class config(LDAPObject):
             label=_('Default e-mail domain'),
             doc=_('Default e-mail domain'),
         ),
-        Int('ipasearchtimelimit', validate_searchtimelimit,
+        Int('ipasearchtimelimit',
             cli_name='searchtimelimit',
             label=_('Search time limit'),
-            doc=_('Maximum amount of time (seconds) for a search (> 0, or -1 for unlimited)'),
+            doc=_('Maximum amount of time (seconds) for a search (-1 or 0 is unlimited)'),
             minvalue=-1,
         ),
         Int('ipasearchrecordslimit',
             cli_name='searchrecordslimit',
             label=_('Search size limit'),
-            doc=_('Maximum number of records to search (-1 is unlimited)'),
+            doc=_('Maximum number of records to search (-1 or 0 is unlimited)'),
             minvalue=-1,
         ),
         IA5Str('ipausersearchfields',
@@ -179,7 +201,8 @@ class config(LDAPObject):
             label=_('Password plugin features'),
             doc=_('Extra hashes to generate in password plug-in'),
             values=(u'AllowNThash',
-                    u'KDC:Disable Last Success', u'KDC:Disable Lockout'),
+                    u'KDC:Disable Last Success', u'KDC:Disable Lockout',
+                    u'KDC:Disable Default Preauth for SPNs'),
             csv=True,
         ),
         Str('ipaselinuxusermaporder',
@@ -197,14 +220,21 @@ class config(LDAPObject):
             values=(u'MS-PAC', u'PAD', u'nfs:NONE'),
             csv=True,
         ),
+        StrEnum('ipauserauthtype*',
+            cli_name='user_auth_type',
+            label=_('Default user authentication types'),
+            doc=_('Default types of supported user authentication'),
+            values=(u'password', u'radius', u'otp', u'disabled'),
+            csv=True,
+        ),
     )
 
     def get_dn(self, *keys, **kwargs):
         return DN(('cn', 'ipaconfig'), ('cn', 'etc'), api.env.basedn)
 
-api.register(config)
 
 
+@register()
 class config_mod(LDAPUpdate):
     __doc__ = _('Modify configuration options.')
 
@@ -222,17 +252,28 @@ class config_mod(LDAPUpdate):
         if 'ipagroupsearchfields' in entry_attrs:
             kw['ipagroupsearchfields']  = 'ipagroupobjectclasses'
         if kw:
-            config = ldap.get_ipa_config(kw.values())
-            for (k, v) in kw.iteritems():
-                allowed_attrs = ldap.get_allowed_attributes(config[1][v])
+            config = ldap.get_ipa_config(list(kw.values()))
+            for (k, v) in kw.items():
+                allowed_attrs = ldap.get_allowed_attributes(config[v])
                 fields = entry_attrs[k].split(',')
                 for a in fields:
                     a = a.strip()
+                    a, tomato, olive = a.partition(';')
                     if a not in allowed_attrs:
                         raise errors.ValidationError(
                             name=k, error=_('attribute "%s" not allowed') % a
                         )
 
+        # Set ipasearchrecordslimit to -1 if 0 is used
+        if 'ipasearchrecordslimit' in entry_attrs:
+            if entry_attrs['ipasearchrecordslimit'] is 0:
+                 entry_attrs['ipasearchrecordslimit'] = -1
+
+        # Set ipasearchtimelimit to -1 if 0 is used
+        if 'ipasearchtimelimit' in entry_attrs:
+            if entry_attrs['ipasearchtimelimit'] is 0:
+                 entry_attrs['ipasearchtimelimit'] = -1
+
         for (attr, obj) in (('ipauserobjectclasses', 'user'),
                             ('ipagroupobjectclasses', 'group')):
             if attr in entry_attrs:
@@ -247,6 +288,7 @@ class config_mod(LDAPUpdate):
                 if self.api.Object[obj].uuid_attribute:
                     checked_attrs = checked_attrs + [self.api.Object[obj].uuid_attribute]
                 for obj_attr in checked_attrs:
+                    obj_attr, tomato, olive = obj_attr.partition(';')
                     if obj_attr in OPERATIONAL_ATTRIBUTES:
                         continue
                     if obj_attr in self.api.Object[obj].params and \
@@ -277,7 +319,7 @@ class config_mod(LDAPUpdate):
                                 error=error_message)
 
             else:
-                config = ldap.get_ipa_config()[1]
+                config = ldap.get_ipa_config()
                 defaultuser = config.get('ipaselinuxusermapdefault', [None])[0]
 
             if 'ipaselinuxusermaporder' in entry_attrs:
@@ -299,7 +341,7 @@ class config_mod(LDAPUpdate):
                                 error=error_message)
             else:
                 if not config:
-                    config = ldap.get_ipa_config()[1]
+                    config = ldap.get_ipa_config()
                 order = config['ipaselinuxusermaporder']
                 userlist = order[0].split('$')
             if defaultuser and defaultuser not in userlist:
@@ -308,10 +350,9 @@ class config_mod(LDAPUpdate):
 
         return dn
 
-api.register(config_mod)
 
 
+@register()
 class config_show(LDAPRetrieve):
     __doc__ = _('Show the current configuration.')
 
-api.register(config_show)
