@@ -18,12 +18,19 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import six
+
 from ipalib import api
 from ipalib import Int, Str
 from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
+from ipalib.plugins.idviews import remove_ipaobject_overrides
 from ipalib.plugins import baseldap
 from ipalib import _, ngettext
+
+if six.PY3:
+    unicode = str
+
 if api.env.in_server and api.env.context in ['lite', 'server']:
     try:
         import ipaserver.dcerpc
@@ -316,6 +323,10 @@ class group_del(LDAPDelete):
                 reason=_(u'privileged group'))
         if 'mepmanagedby' in group_attrs:
             raise errors.ManagedGroupError()
+
+        # Remove any ID overrides tied with this group
+        remove_ipaobject_overrides(ldap, self.obj.api, dn)
+
         return dn
 
     def post_callback(self, ldap, dn, *keys, **options):
@@ -392,7 +403,7 @@ class group_mod(LDAPUpdate):
     def exc_callback(self, keys, options, exc, call_func, *call_args, **call_kwargs):
         # Check again for GID requirement in case someone tried to clear it
         # using --setattr.
-        if call_func.func_name == 'update_entry':
+        if call_func.__name__ == 'update_entry':
             if isinstance(exc, errors.ObjectclassViolation):
                 if 'gidNumber' in exc.message and 'posixGroup' in exc.message:
                     raise errors.RequirementError(name='gid')
@@ -455,7 +466,7 @@ class group_find(LDAPSearch):
             search_kw = {}
             config = ldap.get_ipa_config()
             attrs = config.get(self.obj.search_attributes_config, [])
-            if len(attrs) == 1 and isinstance(attrs[0], basestring):
+            if len(attrs) == 1 and isinstance(attrs[0], six.string_types):
                 search_attrs = attrs[0].split(',')
                 for a in search_attrs:
                     search_kw[a] = args[-1]
@@ -515,7 +526,7 @@ class group_add_member(LDAPAddMember):
                 else:
                     try:
                         actual_sid = domain_validator.get_trusted_domain_object_sid(sid)
-                    except errors.PublicError, e:
+                    except errors.PublicError as e:
                         failed_sids.append((sid, e.strerror))
                     else:
                         sids.append(actual_sid)
@@ -572,7 +583,7 @@ class group_remove_member(LDAPRemoveMember):
                 else:
                     try:
                         actual_sid = domain_validator.get_trusted_domain_object_sid(sid)
-                    except errors.PublicError, e:
+                    except errors.PublicError as e:
                         failed_sids.append((sid, unicode(e)))
                     else:
                         sids.append(actual_sid)
