@@ -19,6 +19,7 @@
 
 from ipalib import api, errors
 from ipalib import AccessTime, Password, Str, StrEnum, Bool, DeprecatedParam
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
 
@@ -70,6 +71,7 @@ EXAMPLES:
    ipa hbacrule-del allow_server
 """)
 
+register = Registry()
 
 # AccessTime support is being removed for now.
 #
@@ -107,6 +109,7 @@ def is_all(options, attribute):
         return False
 
 
+@register()
 class hbacrule(LDAPObject):
     """
     HBAC object.
@@ -115,12 +118,13 @@ class hbacrule(LDAPObject):
     object_name = _('HBAC rule')
     object_name_plural = _('HBAC rules')
     object_class = ['ipaassociation', 'ipahbacrule']
+    permission_filter_objectclasses = ['ipahbacrule']
     default_attributes = [
         'cn', 'ipaenabledflag',
         'description', 'usercategory', 'hostcategory',
-        'sourcehostcategory', 'servicecategory', 'ipaenabledflag',
+        'servicecategory', 'ipaenabledflag',
         'memberuser', 'sourcehost', 'memberhost', 'memberservice',
-        'memberhostgroup', 'externalhost',
+        'externalhost',
     ]
     uuid_attribute = 'ipauniqueid'
     rdn_attribute = 'ipauniqueid'
@@ -130,6 +134,56 @@ class hbacrule(LDAPObject):
         'sourcehost': ['host', 'hostgroup'],
         'memberservice': ['hbacsvc', 'hbacsvcgroup'],
     }
+    managed_permissions = {
+        'System: Read HBAC Rules': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'accessruletype', 'accesstime', 'cn', 'description',
+                'externalhost', 'hostcategory', 'ipaenabledflag',
+                'ipauniqueid', 'memberhost', 'memberservice', 'memberuser',
+                'servicecategory', 'sourcehost', 'sourcehostcategory',
+                'usercategory', 'objectclass', 'member',
+            },
+        },
+        'System: Add HBAC Rule': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///ipauniqueid=*,cn=hbac,$SUFFIX")(version 3.0;acl "permission:Add HBAC rule";allow (add) groupdn = "ldap:///cn=Add HBAC rule,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'HBAC Administrator'},
+        },
+        'System: Delete HBAC Rule': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///ipauniqueid=*,cn=hbac,$SUFFIX")(version 3.0;acl "permission:Delete HBAC rule";allow (delete) groupdn = "ldap:///cn=Delete HBAC rule,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'HBAC Administrator'},
+        },
+        'System: Manage HBAC Rule Membership': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'externalhost', 'memberhost', 'memberservice', 'memberuser'
+            },
+            'replaces': [
+                '(targetattr = "memberuser || externalhost || memberservice || memberhost")(target = "ldap:///ipauniqueid=*,cn=hbac,$SUFFIX")(version 3.0;acl "permission:Manage HBAC rule membership";allow (write) groupdn = "ldap:///cn=Manage HBAC rule membership,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'HBAC Administrator'},
+        },
+        'System: Modify HBAC Rule': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'accessruletype', 'accesstime', 'cn', 'description',
+                'hostcategory', 'ipaenabledflag', 'servicecategory',
+                'sourcehost', 'sourcehostcategory', 'usercategory'
+            },
+            'replaces': [
+                '(targetattr = "servicecategory || sourcehostcategory || cn || description || ipaenabledflag || accesstime || usercategory || hostcategory || accessruletype || sourcehost")(target = "ldap:///ipauniqueid=*,cn=hbac,$SUFFIX")(version 3.0;acl "permission:Modify HBAC rule";allow (write) groupdn = "ldap:///cn=Modify HBAC rule,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'HBAC Administrator'},
+        },
+    }
 
     label = _('HBAC Rules')
     label_singular = _('HBAC Rule')
@@ -211,9 +265,9 @@ class hbacrule(LDAPObject):
         external_host_param,
     )
 
-api.register(hbacrule)
 
 
+@register()
 class hbacrule_add(LDAPCreate):
     __doc__ = _('Create a new HBAC rule.')
 
@@ -225,9 +279,9 @@ class hbacrule_add(LDAPCreate):
         entry_attrs['ipaenabledflag'] = 'TRUE'
         return dn
 
-api.register(hbacrule_add)
 
 
+@register()
 class hbacrule_del(LDAPDelete):
     __doc__ = _('Delete an HBAC rule.')
 
@@ -242,9 +296,9 @@ class hbacrule_del(LDAPDelete):
 
         return dn
 
-api.register(hbacrule_del)
 
 
+@register()
 class hbacrule_mod(LDAPUpdate):
     __doc__ = _('Modify an HBAC rule.')
 
@@ -253,7 +307,8 @@ class hbacrule_mod(LDAPUpdate):
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, attrs_list)
+            entry_attrs = ldap.get_entry(dn, attrs_list)
+            dn = entry_attrs.dn
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
@@ -265,9 +320,9 @@ class hbacrule_mod(LDAPUpdate):
             raise errors.MutuallyExclusiveError(reason=_("service category cannot be set to 'all' while there are allowed services"))
         return dn
 
-api.register(hbacrule_mod)
 
 
+@register()
 class hbacrule_find(LDAPSearch):
     __doc__ = _('Search for HBAC rules.')
 
@@ -275,15 +330,15 @@ class hbacrule_find(LDAPSearch):
         '%(count)d HBAC rule matched', '%(count)d HBAC rules matched', 0
     )
 
-api.register(hbacrule_find)
 
 
+@register()
 class hbacrule_show(LDAPRetrieve):
     __doc__ = _('Display the properties of an HBAC rule.')
 
-api.register(hbacrule_show)
 
 
+@register()
 class hbacrule_enable(LDAPQuery):
     __doc__ = _('Enable an HBAC rule.')
 
@@ -294,23 +349,26 @@ class hbacrule_enable(LDAPQuery):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
-        entry_attrs = {'ipaenabledflag': 'TRUE'}
-
         try:
-            ldap.update_entry(dn, entry_attrs)
-        except errors.EmptyModlist:
-            pass
+            entry_attrs = ldap.get_entry(dn, ['ipaenabledflag'])
         except errors.NotFound:
             self.obj.handle_not_found(cn)
 
+        entry_attrs['ipaenabledflag'] = ['TRUE']
+
+        try:
+            ldap.update_entry(entry_attrs)
+        except errors.EmptyModlist:
+            pass
+
         return dict(
             result=True,
-            value=cn,
+            value=pkey_to_value(cn, options),
         )
 
-api.register(hbacrule_enable)
 
 
+@register()
 class hbacrule_disable(LDAPQuery):
     __doc__ = _('Disable an HBAC rule.')
 
@@ -321,21 +379,23 @@ class hbacrule_disable(LDAPQuery):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
-        entry_attrs = {'ipaenabledflag': 'FALSE'}
-
         try:
-            ldap.update_entry(dn, entry_attrs)
-        except errors.EmptyModlist:
-            pass
+            entry_attrs = ldap.get_entry(dn, ['ipaenabledflag'])
         except errors.NotFound:
             self.obj.handle_not_found(cn)
 
+        entry_attrs['ipaenabledflag'] = ['FALSE']
+
+        try:
+            ldap.update_entry(entry_attrs)
+        except errors.EmptyModlist:
+            pass
+
         return dict(
             result=True,
-            value=cn,
+            value=pkey_to_value(cn, options),
         )
 
-api.register(hbacrule_disable)
 
 
 class hbacrule_add_accesstime(LDAPQuery):
@@ -355,12 +415,12 @@ class hbacrule_add_accesstime(LDAPQuery):
 
         dn = self.obj.get_dn(cn)
 
-        (dn, entry_attrs) = ldap.get_entry(dn, ['accesstime'])
+        entry_attrs = ldap.get_entry(dn, ['accesstime'])
         entry_attrs.setdefault('accesstime', []).append(
             options['accesstime']
         )
         try:
-            ldap.update_entry(dn, entry_attrs)
+            ldap.update_entry(entry_attrs)
         except errors.EmptyModlist:
             pass
         except errors.NotFound:
@@ -395,12 +455,12 @@ class hbacrule_remove_accesstime(LDAPQuery):
 
         dn = self.obj.get_dn(cn)
 
-        (dn, entry_attrs) = ldap.get_entry(dn, ['accesstime'])
+        entry_attrs = ldap.get_entry(dn, ['accesstime'])
         try:
             entry_attrs.setdefault('accesstime', []).remove(
                 options['accesstime']
             )
-            ldap.update_entry(dn, entry_attrs)
+            ldap.update_entry(entry_attrs)
         except (ValueError, errors.EmptyModlist):
             pass
         except errors.NotFound:
@@ -419,6 +479,7 @@ class hbacrule_remove_accesstime(LDAPQuery):
 #api.register(hbacrule_remove_accesstime)
 
 
+@register()
 class hbacrule_add_user(LDAPAddMember):
     __doc__ = _('Add users and groups to an HBAC rule.')
 
@@ -428,7 +489,8 @@ class hbacrule_add_user(LDAPAddMember):
     def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, self.obj.default_attributes)
+            entry_attrs = ldap.get_entry(dn, self.obj.default_attributes)
+            dn = entry_attrs.dn
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
         if 'usercategory' in entry_attrs and \
@@ -437,18 +499,18 @@ class hbacrule_add_user(LDAPAddMember):
                 reason=_("users cannot be added when user category='all'"))
         return dn
 
-api.register(hbacrule_add_user)
 
 
+@register()
 class hbacrule_remove_user(LDAPRemoveMember):
     __doc__ = _('Remove users and groups from an HBAC rule.')
 
     member_attributes = ['memberuser']
     member_count_out = ('%i object removed.', '%i objects removed.')
 
-api.register(hbacrule_remove_user)
 
 
+@register()
 class hbacrule_add_host(LDAPAddMember):
     __doc__ = _('Add target hosts and hostgroups to an HBAC rule.')
 
@@ -458,7 +520,8 @@ class hbacrule_add_host(LDAPAddMember):
     def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, self.obj.default_attributes)
+            entry_attrs = ldap.get_entry(dn, self.obj.default_attributes)
+            dn = entry_attrs.dn
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
         if 'hostcategory' in entry_attrs and \
@@ -467,18 +530,18 @@ class hbacrule_add_host(LDAPAddMember):
                 reason=_("hosts cannot be added when host category='all'"))
         return dn
 
-api.register(hbacrule_add_host)
 
 
+@register()
 class hbacrule_remove_host(LDAPRemoveMember):
     __doc__ = _('Remove target hosts and hostgroups from an HBAC rule.')
 
     member_attributes = ['memberhost']
     member_count_out = ('%i object removed.', '%i objects removed.')
 
-api.register(hbacrule_remove_host)
 
 
+@register()
 class hbacrule_add_sourcehost(LDAPAddMember):
     NO_CLI = True
 
@@ -488,9 +551,9 @@ class hbacrule_add_sourcehost(LDAPAddMember):
     def validate(self, **kw):
         raise errors.DeprecationError(name='hbacrule_add_sourcehost')
 
-api.register(hbacrule_add_sourcehost)
 
 
+@register()
 class hbacrule_remove_sourcehost(LDAPRemoveMember):
     NO_CLI = True
 
@@ -500,9 +563,9 @@ class hbacrule_remove_sourcehost(LDAPRemoveMember):
     def validate(self, **kw):
         raise errors.DeprecationError(name='hbacrule_remove_sourcehost')
 
-api.register(hbacrule_remove_sourcehost)
 
 
+@register()
 class hbacrule_add_service(LDAPAddMember):
     __doc__ = _('Add services to an HBAC rule.')
 
@@ -512,7 +575,8 @@ class hbacrule_add_service(LDAPAddMember):
     def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, self.obj.default_attributes)
+            entry_attrs = ldap.get_entry(dn, self.obj.default_attributes)
+            dn = entry_attrs.dn
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
         if 'servicecategory' in entry_attrs and \
@@ -521,13 +585,12 @@ class hbacrule_add_service(LDAPAddMember):
                 "services cannot be added when service category='all'"))
         return dn
 
-api.register(hbacrule_add_service)
 
 
+@register()
 class hbacrule_remove_service(LDAPRemoveMember):
     __doc__ = _('Remove service and service groups from an HBAC rule.')
 
     member_attributes = ['memberservice']
     member_count_out = ('%i object removed.', '%i objects removed.')
 
-api.register(hbacrule_remove_service)
