@@ -19,7 +19,6 @@
 
 from ipalib import api, errors, output, util
 from ipalib import Command, Str, Flag, Int, DeprecatedParam
-from types import NoneType
 from ipalib.cli import to_cli
 from ipalib import _, ngettext
 from ipapython.dn import DN
@@ -32,6 +31,10 @@ if api.env.in_server and api.env.context in ['lite', 'server']:
         _dcerpc_bindings_installed = False
 
 import pyhbac
+import six
+
+if six.PY3:
+    unicode = str
 
 __doc__ = _("""
 Simulate use of Host-based access controls
@@ -246,10 +249,10 @@ class hbactest(Command):
 
     has_output = (
         output.summary,
-        output.Output('warning', (list, tuple, NoneType),   _('Warning')),
-        output.Output('matched', (list, tuple, NoneType),   _('Matched rules')),
-        output.Output('notmatched', (list, tuple, NoneType), _('Not matched rules')),
-        output.Output('error', (list, tuple, NoneType), _('Non-existent or invalid rules')),
+        output.Output('warning', (list, tuple, type(None)),   _('Warning')),
+        output.Output('matched', (list, tuple, type(None)),   _('Matched rules')),
+        output.Output('notmatched', (list, tuple, type(None)), _('Not matched rules')),
+        output.Output('error', (list, tuple, type(None)), _('Non-existent or invalid rules')),
         output.Output('value',  bool, _('Result of simulation'), ['no_display']),
     )
 
@@ -462,13 +465,14 @@ class hbactest(Command):
                         matched_rules.append(ipa_rule.name)
                     if res == pyhbac.HBAC_EVAL_DENY:
                         notmatched_rules.append(ipa_rule.name)
-                except pyhbac.HbacError as (code, rule_name):
+                except pyhbac.HbacError as e:
+                    code, rule_name = e.args
                     if code == pyhbac.HBAC_EVAL_ERROR:
                         error_rules.append(rule_name)
                         self.log.info('Native IPA HBAC rule "%s" parsing error: %s' % \
                                       (rule_name, pyhbac.hbac_result_string(code)))
-                except (TypeError, IOError) as (info):
-                    self.log.error('Native IPA HBAC module error: %s' % (info))
+                except (TypeError, IOError) as info:
+                    self.log.error('Native IPA HBAC module error: %s' % info)
 
             access_granted = len(matched_rules) > 0
         else:
