@@ -18,12 +18,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import six
+
 from ipalib.plugable import Registry
-from ipalib.plugins.baseldap import *
-from ipalib import api, Int, _, ngettext, errors
+from ipalib.plugins.baseldap import (LDAPObject, LDAPCreate, LDAPRetrieve,
+                                     LDAPDelete, LDAPUpdate, LDAPSearch,
+                                     LDAPAddMember, LDAPRemoveMember,
+                                     entry_from_entry, wait_for_value)
+from ipalib import Str, api, _, ngettext, errors
 from ipalib.plugins.netgroup import NETGROUP_PATTERN, NETGROUP_PATTERN_ERRMSG
 from ipapython.dn import DN
 
+if six.PY3:
+    unicode = str
+
 __doc__ = _("""
 Groups of hosts.
 
@@ -54,8 +62,19 @@ EXAMPLES:
    ipa hostgroup-del baltimore
 """)
 
+
+def get_complete_hostgroup_member_list(hostgroup):
+    result = api.Command['hostgroup_show'](hostgroup)['result']
+    direct = list(result.get('member_host', []))
+    indirect = list(result.get('memberindirect_host', []))
+    return direct + indirect
+
+
 register = Registry()
 
+PROTECTED_HOSTGROUPS = (u'ipaservers',)
+
+
 @register()
 class hostgroup(LDAPObject):
     """
@@ -66,6 +85,7 @@ class hostgroup(LDAPObject):
     object_name_plural = _('host groups')
     object_class = ['ipaobject', 'ipahostgroup']
     permission_filter_objectclasses = ['ipahostgroup']
+    search_attributes = ['cn', 'description', 'member', 'memberof']
     default_attributes = ['cn', 'description', 'member', 'memberof',
         'memberindirect', 'memberofindirect',
     ]
@@ -103,6 +123,10 @@ class hostgroup(LDAPObject):
         },
         'System: Modify Hostgroup Membership': {
             'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                '(objectclass=ipahostgroup)',
+                '(!(cn=ipaservers))',
+            ],
             'ipapermdefaultattr': {'member'},
             'replaces': [
                 '(targetattr = "member")(target = "ldap:///cn=*,cn=hostgroups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Hostgroup membership";allow (write) groupdn = "ldap:///cn=Modify Hostgroup membership,cn=permissions,cn=pbac,$SUFFIX";)',
@@ -139,7 +163,7 @@ class hostgroup(LDAPObject):
             primary_key=True,
             normalizer=lambda value: value.lower(),
         ),
-        Str('description',
+        Str('description?',
             cli_name='desc',
             label=_('Description'),
             doc=_('A description of this host-group'),
@@ -166,7 +190,6 @@ class hostgroup(LDAPObject):
                 entry_attrs['memberof'].remove(member)
 
 
-
 @register()
 class hostgroup_add(LDAPCreate):
     __doc__ = _('Add a new hostgroup.')
@@ -186,9 +209,9 @@ class hostgroup_add(LDAPCreate):
             # when enabled, a managed netgroup is created for every hostgroup
             # make sure that the netgroup can be created
             api.Object['netgroup'].get_dn_if_exists(keys[-1])
-            raise errors.DuplicateEntry(message=unicode(_(\
-                    u'netgroup with name "%s" already exists. ' \
-                    u'Hostgroups and netgroups share a common namespace'\
+            raise errors.DuplicateEntry(message=unicode(_(
+                    u'netgroup with name "%s" already exists. '
+                    u'Hostgroups and netgroups share a common namespace'
                     ) % keys[-1]))
         except errors.NotFound:
             pass
@@ -206,14 +229,19 @@ class hostgroup_add(LDAPCreate):
         return dn
 
 
-
-
 @register()
 class hostgroup_del(LDAPDelete):
     __doc__ = _('Delete a hostgroup.')
 
     msg_summary = _('Deleted hostgroup "%(value)s"')
 
+    def pre_callback(self, ldap, dn, *keys, **options):
+        if keys[0] in PROTECTED_HOSTGROUPS:
+            raise errors.ProtectedEntryError(label=_(u'hostgroup'),
+                                             key=keys[0],
+                                             reason=_(u'privileged hostgroup'))
+
+        return dn
 
 
 @register()
@@ -228,7 +256,6 @@ class hostgroup_mod(LDAPUpdate):
         return dn
 
 
-
 @register()
 class hostgroup_find(LDAPSearch):
     __doc__ = _('Search for hostgroups.')
@@ -246,7 +273,6 @@ class hostgroup_find(LDAPSearch):
         return truncated
 
 
-
 @register()
 class hostgroup_show(LDAPRetrieve):
     __doc__ = _('Display information about a hostgroup.')
@@ -257,7 +283,6 @@ class hostgroup_show(LDAPRetrieve):
         return dn
 
 
-
 @register()
 class hostgroup_add_member(LDAPAddMember):
     __doc__ = _('Add members to a hostgroup.')
@@ -268,11 +293,22 @@ class hostgroup_add_member(LDAPAddMember):
         return (completed, dn)
 
 
-
 @register()
 class hostgroup_remove_member(LDAPRemoveMember):
     __doc__ = _('Remove members from a hostgroup.')
 
+    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
+        if keys[0] in PROTECTED_HOSTGROUPS and 'host' in options:
+            result = api.Command.hostgroup_show(keys[0])
+            hosts_left = set(result['result'].get('member_host', []))
+            hosts_deleted = set(options['host'])
+            if hosts_left.issubset(hosts_deleted):
+                raise errors.LastMemberError(key=sorted(hosts_deleted)[0],
+                                             label=_(u'hostgroup'),
+                                             container=keys[0])
+
+        return dn
+
     def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
         self.obj.suppress_netgroup_memberof(ldap, dn, entry_attrs)
