@@ -22,6 +22,7 @@
 """
 Plugins not accessible directly through the CLI, commands used internally
 """
+from __future__ import print_function
 
 import json
 
@@ -138,7 +139,7 @@ class json_metadata(Command):
         return retval
 
     def output_for_cli(self, textui, result, *args, **options):
-        print json.dumps(result, default=json_serialize)
+        print(json.dumps(result, default=json_serialize))
 
 
 @register()
@@ -190,13 +191,23 @@ class i18n_messages(Command):
             "removed": _("${count} item(s) removed"),
             "show_results": _("Show Results"),
         },
+        "authtype": {
+            "config_tooltip": _("<p>Implicit method (password) will be used if no method is chosen.</p><p><strong>Password + Two-factor:</strong> LDAP and Kerberos allow authentication with either one of the authentication types but Kerberos uses pre-authentication method which requires to use armor ccache.</p><p><strong>RADIUS with another type:</strong> Kerberos always use RADIUS, but LDAP never does. LDAP only recognize the password and two-factor authentication options.</p>"),
+            "type_otp": _("Two factor authentication (password + OTP)"),
+            "type_password": _("Password"),
+            "type_radius": _("Radius"),
+            "type_disabled": _("Disable per-user override"),
+            "user_tooltip": _("<p>Per-user setting, overwrites the global setting if any option is checked.</p><p><strong>Password + Two-factor:</strong> LDAP and Kerberos allow authentication with either one of the authentication types but Kerberos uses pre-authentication method which requires to use armor ccache.</p><p><strong>RADIUS with another type:</strong> Kerberos always use RADIUS, but LDAP never does. LDAP only recognize the password and two-factor authentication options.</p>"),
+        },
         "buttons": {
             "about": _("About"),
+            "activate": _("Activate"),
             "add": _("Add"),
             "add_and_add_another": _("Add and Add Another"),
             "add_and_close": _("Add and Close"),
             "add_and_edit": _("Add and Edit"),
             "add_many": _("Add Many"),
+            "apply": _("Apply"),
             "back": _("Back"),
             "cancel": _("Cancel"),
             "close": _("Close"),
@@ -206,6 +217,7 @@ class i18n_messages(Command):
             "filter": _("Filter"),
             "find": _("Find"),
             "get": _("Get"),
+            "hide": _("Hide"),
             "issue": _("Issue"),
             "ok": _("OK"),
             "refresh": _("Refresh"),
@@ -214,8 +226,12 @@ class i18n_messages(Command):
             "reset_password_and_login": _("Reset Password and Login"),
             "restore": _("Restore"),
             "retry": _("Retry"),
+            "revert": _("Revert"),
             "revoke": _("Revoke"),
+            "save": _("Save"),
             "set": _("Set"),
+            "show": _("Show"),
+            "unapply": ("Un-apply"),
             "update": _("Update"),
             "view": _("View"),
         },
@@ -246,6 +262,7 @@ class i18n_messages(Command):
             "remove_empty": _("Select entries to be removed."),
             "remove_title": _("Remove ${entity}"),
             "show_details": _("Show details"),
+            "success": _("Success"),
             "validation_title": _("Validation error"),
             "validation_message": _("Input form contains invalid or missing values."),
         },
@@ -276,6 +293,14 @@ class i18n_messages(Command):
             "search": _("Search"),
         },
         "false": _("False"),
+        "keytab": {
+            "add_create": _("Allow ${other_entity} to create keytab of ${primary_key}"),
+            "add_retrive": _("Allow ${other_entity} to retrieve keytab of ${primary_key}"),
+            "allowed_to_create": _("Allowed to create keytab"),
+            "allowed_to_retrieve": _("Allowed to retrieve keytab"),
+            "remove_create": _("Disallow ${other_entity} to create keytab of ${primary_key}"),
+            "remove_retrieve": _("Disallow ${other_entity} to retrieve keytab of ${primary_key}"),
+        },
         "krbauthzdata": {
             "inherited": _("Inherited from server configuration"),
             "mspac": _("MS-PAC"),
@@ -283,9 +308,9 @@ class i18n_messages(Command):
             "pad": _("PAD"),
         },
         "login": {
-            "form_auth": _("To login with username and password, enter them in the fields below then click Login."),
+            "form_auth": _("<i class=\"fa fa-info-circle\"></i> To login with <strong>username and password</strong>, enter them in the corresponding fields, then click Login."),
             "header": _("Logged In As"),
-            "krb_auth_msg": _("To login with Kerberos, please make sure you have valid tickets (obtainable via kinit) and <a href='http://${host}/ipa/config/unauthorized.html'>configured</a> the browser correctly, then click Login."),
+            "krb_auth_msg": _("<i class=\"fa fa-info-circle\"></i> To login with <strong>Kerberos</strong>, please make sure you have valid tickets (obtainable via kinit) and <a href='http://${host}/ipa/config/unauthorized.html'>configured</a> the browser correctly, then click Login."),
             "login": _("Login"),
             "logout": _("Logout"),
             "logout_error": _("Logout error"),
@@ -325,8 +350,22 @@ class i18n_messages(Command):
                 "direct": _("Direct"),
                 "indirect": _("Indirect"),
             },
+            "caacl": {
+                "any_host": _("Any Host"),
+                "any_service": _("Any Service"),
+                "any_profile": _("Any Profile"),
+                "anyone": _("Anyone"),
+                "ipaenabledflag": _("Rule status"),
+                "profile": _("Profiles"),
+                "specified_hosts": _("Specified Hosts and Groups"),
+                "specified_profiles": _("Specified Profiles"),
+                "specified_services": _("Specified Services and Groups"),
+                "specified_users": _("Specified Users and Groups"),
+                "who": _("Permitted to have certificates issued"),
+            },
             "cert": {
                 "aa_compromise": _("AA Compromise"),
+                "add_principal": _("Add principal"),
                 "affiliation_changed": _("Affiliation Changed"),
                 "ca_compromise": _("CA Compromise"),
                 "certificate": _("Certificate"),
@@ -350,6 +389,7 @@ class i18n_messages(Command):
                 "fingerprints": _("Fingerprints"),
                 "get_certificate": _("Get Certificate"),
                 "issue_certificate": _("Issue New Certificate for ${entity} ${primary_key}"),
+                "issue_certificate_generic": _("Issue New Certificate"),
                 "issued_by": _("Issued By"),
                 "issued_on": _("Issued On"),
                 "issued_to": _("Issued To"),
@@ -360,10 +400,11 @@ class i18n_messages(Command):
                 "note": _("Note"),
                 "organization": _("Organization"),
                 "organizational_unit": _("Organizational Unit"),
+                "present": _("${count} certificate(s) present"),
                 "privilege_withdrawn": _("Privilege Withdrawn"),
                 "reason": _("Reason for Revocation"),
                 "remove_from_crl": _("Remove from CRL"),
-                "request_message": _("<ol> <li>Create a certificate database or use an existing one. To create a new database:<br/> <code># certutil -N -d &lt;database path&gt;</code> </li> <li>Create a CSR with subject <em>CN=&lt;hostname&gt;,O=&lt;realm&gt;</em>, for example:<br/> <code># certutil -R -d &lt;database path&gt; -a -g &lt;key size&gt; -s 'CN=${hostname},O=${realm}'</code> </li> <li> Copy and paste the CSR (from <em>-----BEGIN NEW CERTIFICATE REQUEST-----</em> to <em>-----END NEW CERTIFICATE REQUEST-----</em>) into the text area below: </li> </ol>"),
+                "request_message": _("<ol> <li>Create a certificate database or use an existing one. To create a new database:<br/> <code># certutil -N -d &lt;database path&gt;</code> </li> <li>Create a CSR with subject <em>CN=&lt;${cn_name}&gt;,O=&lt;realm&gt;</em>, for example:<br/> <code># certutil -R -d &lt;database path&gt; -a -g &lt;key size&gt; -s 'CN=${cn},O=${realm}'</code> </li> <li> Copy and paste the CSR (from <em>-----BEGIN NEW CERTIFICATE REQUEST-----</em> to <em>-----END NEW CERTIFICATE REQUEST-----</em>) into the text area below: </li> </ol>"),
                 "requested": _("Certificate requested"),
                 "restore_certificate": _("Restore Certificate for ${entity} ${primary_key}"),
                 "restore_certificate_simple": _("Restore Certificate"),
@@ -426,6 +467,15 @@ class i18n_messages(Command):
                 "add_permission_confirm":_("Are you sure you want to add permission for DNS Zone ${object}?"),
                 "remove_permission": _("Remove Permission"),
                 "remove_permission_confirm": _("Are you sure you want to remove permission for DNS Zone ${object}?"),
+                "skip_dns_check": _("Skip DNS check"),
+                "soamname_change_message": _("Do you want to check if new authoritative nameserver address is in DNS"),
+                "soamname_change_title": _("Authoritative nameserver change"),
+            },
+            "domainlevel": {
+                "label": _("Domain Level"),
+                "label_singular": _("Domain Level"),
+                "ipadomainlevel": _("Level"),
+                "set": _("Set Domain Level"),
             },
             "group": {
                 "details": _("Group Settings"),
@@ -495,6 +545,35 @@ class i18n_messages(Command):
             "hostgroup": {
                 "identity": _("Host Group Settings"),
             },
+            "idoverrideuser": {
+                "anchor_label": _("User to override"),
+                "anchor_tooltip": _("Enter trusted or IPA user login. Note: search doesn't list users from trusted domains."),
+                "anchor_tooltip_ad": _("Enter trusted user login."),
+            },
+            "idoverridegroup": {
+                "anchor_label": _("Group to override"),
+                "anchor_tooltip": _("Enter trusted or IPA group name. Note: search doesn't list groups from trusted domains."),
+                "anchor_tooltip_ad": _("Enter trusted group name."),
+            },
+            "idview": {
+                "appliesto_tab": _("${primary_key} applies to:"),
+                "appliedtohosts": _("Applied to hosts"),
+                "appliedtohosts_title": _("Applied to hosts"),
+                "apply_hostgroups": _("Apply to host groups"),
+                "apply_hostgroups_title": _("Apply ID View ${primary_key} on hosts of ${entity}"),
+                "apply_hosts": _("Apply to hosts"),
+                "apply_hosts_title": _("Apply ID view ${primary_key} on ${entity}"),
+                "ipaassignedidview": _("Assigned ID View"),
+                "overrides_tab": _("${primary_key} overrides:"),
+                "unapply_hostgroups": _("Un-apply from host groups"),
+                "unapply_hostgroups_all_title": _("Un-apply ID Views from hosts of hostgroups"),
+                "unapply_hostgroups_title": _("Un-apply ID View ${primary_key} from hosts of ${entity}"),
+                "unapply_hosts": _("Un-apply"),
+                "unapply_hosts_all": _("Un-apply from hosts"),
+                "unapply_hosts_all_title": _("Un-apply ID Views from hosts"),
+                "unapply_hosts_confirm": _("Are you sure you want to un-apply ID view from selected entries?"),
+                "unapply_hosts_title": _("Un-apply ID View ${primary_key} from hosts"),
+            },
             "krbtpolicy": {
                 "identity": _("Kerberos Ticket Policy"),
             },
@@ -513,9 +592,17 @@ class i18n_messages(Command):
                 "users": _("Users"),
             },
             "otptoken": {
-                "details": "OTP Token Settings",
-                "disable": "Disable token",
-                "enable": "Enable token",
+                "add_token": _("Add OTP Token"),
+                "app_link": _("You can use <a href=\"${link}\" target=\"_blank\">FreeOTP<a/> as a software OTP token application."),
+                "config_title": _("Configure your token"),
+                "config_instructions": _("Configure your token by scanning the QR code below. Click on the QR code if you see this on the device you want to configure."),
+                "details": _("OTP Token Settings"),
+                "disable": _("Disable token"),
+                "enable": _("Enable token"),
+                "show_qr": _("Show QR code"),
+                "show_uri": _("Show configuration uri"),
+                "type_hotp": _("Counter-based (HOTP)"),
+                "type_totp": _("Time-based (TOTP)"),
             },
             "permission": {
                 "add_custom_attr": _("Add custom attribute"),
@@ -593,6 +680,15 @@ class i18n_messages(Command):
                 "status_new_ns": _("New: key not set"),
                 "status_new_s": _("New: key set"),
             },
+            "stageuser": {
+                "activate_confirm": _("Are you sure you want to activate selected users?"),
+                "activate_success": _("${count} user(s) activated"),
+                "label": _("Stage users"),
+                "preserved_label": _("Preserved users"),
+                "undel_confirm": _("Are you sure you want to restore selected users?"),
+                "undel_success": _("${count} user(s) restored"),
+                "user_categories": _("User categories"),
+            },
             "sudocmd": {
                 "groups": _("Groups"),
             },
@@ -620,6 +716,11 @@ class i18n_messages(Command):
                 "specified_users": _("Specified Users and Groups"),
                 "user": _("Who"),
             },
+            "topology": {
+                "segment_details": _("Segment details"),
+                "replication_config": _("Replication configuration"),
+                "insufficient_domain_level" : _("Managed topology requires minimal domain level ${domainlevel}"),
+            },
             "trust": {
                 "account": _("Account"),
                 "admin_account": _("Administrative account"),
@@ -641,12 +742,17 @@ class i18n_messages(Command):
             "user": {
                 "account": _("Account Settings"),
                 "account_status": _("Account Status"),
+                "activeuser_label": _("Active users"),
                 "contact": _("Contact Settings"),
+                "delete_mode": _("Delete mode"),
                 "employee": _("Employee Information"),
                 "error_changing_status": _("Error changing account status"),
                 "krbpasswordexpiration": _("Password expiration"),
                 "mailing": _("Mailing Address"),
                 "misc": _("Misc. Information"),
+                "mode_delete": _("delete"),
+                "mode_preserve": _("preserve"),
+                "noprivate": _("No private group"),
                 "status_confirmation": _("Are you sure you want to ${action} the user?<br/>The change will take effect immediately."),
                 "status_link": _("Click to ${action}"),
                 "unlock": _("Unlock"),
@@ -662,6 +768,7 @@ class i18n_messages(Command):
             "new_password": _("New Password"),
             "new_password_required": _("New password is required"),
             "otp": _("OTP"),
+            "otp_info": _("<i class=\"fa fa-info-circle\"></i> <strong>One-Time-Password(OTP):</strong> Generate new OTP code for each OTP field."),
             "otp_long": _("One-Time-Password"),
             "otp_sync_fail": _("Token synchronization failed"),
             "otp_sync_invalid": _("The username, password or token codes are not correct"),
@@ -713,6 +820,7 @@ class i18n_messages(Command):
             "policy": _("Policy"),
             "role": _("Role Based Access Control"),
             "sudo": _("Sudo"),
+            "topology": _("Topology"),
             "trust": _("Trusts"),
         },
         "true": _("True"),
@@ -735,7 +843,7 @@ class i18n_messages(Command):
                 "ip_v6_address": _('Not a valid IPv6 address'),
                 "max_value": _("Maximum value is ${value}"),
                 "min_value": _("Minimum value is ${value}"),
-                "net_address": _("Not a valid network address"),
+                "net_address": _("Not a valid network address (examples: 2001:db8::/64, 192.0.2.0/24)"),
                 "parse": _("Parse error"),
                 "port": _("'${port}' is not a valid port"),
                 "required": _("Required field"),
@@ -750,6 +858,6 @@ class i18n_messages(Command):
         return dict(texts=json_serialize(self.messages))
 
     def output_for_cli(self, textui, result, *args, **options):
-        print json.dumps(result, default=json_serialize)
+        print(json.dumps(result, default=json_serialize))
 
 
