@@ -18,16 +18,20 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import re
+from ldap import MOD_ADD
+from ldap import SCOPE_BASE, SCOPE_ONELEVEL, SCOPE_SUBTREE
+
+import six
 
 from ipalib import api, errors, output
-from ipalib import Command, Password, Str, Flag, StrEnum, DNParam, File
+from ipalib import Command, Password, Str, Flag, StrEnum, DNParam, File, Bool
 from ipalib.cli import to_cli
 from ipalib.plugable import Registry
 from ipalib.plugins.user import NO_UPG_MAGIC
 if api.env.in_server and api.env.context in ['lite', 'server']:
     try:
         from ipaserver.plugins.ldap2 import ldap2
-    except StandardError, e:
+    except Exception as e:
         raise e
 from ipalib import _
 from ipapython.dn import DN
@@ -35,6 +39,9 @@ from ipapython.ipautil import write_tmp_file
 import datetime
 from ipaplatform.paths import paths
 
+if six.PY3:
+    unicode = str
+
 __doc__ = _("""
 Migration to IPA
 
@@ -140,6 +147,10 @@ _dn_err_msg = _('Malformed DN')
 
 _supported_schemas = (u'RFC2307bis', u'RFC2307')
 
+# search scopes for users and groups when migrating
+_supported_scopes = {u'base': SCOPE_BASE, u'onelevel': SCOPE_ONELEVEL, u'subtree': SCOPE_SUBTREE}
+_default_scope = u'onelevel'
+
 
 def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs):
     assert isinstance(dn, DN)
@@ -165,16 +176,16 @@ def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs
                     'gidnumber', entry_attrs['gidnumber'][0], 'posixgroup',
                     [''], search_bases['group']
                 )
-                valid_gids.append(entry_attrs['gidnumber'][0])
+                valid_gids.add(entry_attrs['gidnumber'][0])
             except errors.NotFound:
                 api.log.warn('GID number %s of migrated user %s does not point to a known group.' \
                              % (entry_attrs['gidnumber'][0], pkey))
-                invalid_gids.append(entry_attrs['gidnumber'][0])
-            except errors.SingleMatchExpected, e:
+                invalid_gids.add(entry_attrs['gidnumber'][0])
+            except errors.SingleMatchExpected as e:
                 # GID number matched more groups, this should not happen
                 api.log.warn('GID number %s of migrated user %s should match 1 group, but it matched %d groups' \
                              % (entry_attrs['gidnumber'][0], pkey, e.found))
-            except errors.LimitsExceeded, e:
+            except errors.LimitsExceeded as e:
                 api.log.warn('Search limit exceeded searching for GID %s' % entry_attrs['gidnumber'][0])
 
     # We don't want to create a UPG so set the magic value in description
@@ -234,7 +245,7 @@ def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs
                         ', convert it', pkey, value, type(value), attr)
                     try:
                         value = DN(value)
-                    except ValueError, e:
+                    except ValueError as e:
                         api.log.warn('%s: skipping normalization of value %s of type %s '
                             'in attribute %s which could not be converted to DN: %s',
                                 pkey, value, type(value), attr, e)
@@ -268,7 +279,8 @@ def _pre_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwargs
 def _post_migrate_user(ldap, pkey, dn, entry_attrs, failed, config, ctx):
     assert isinstance(dn, DN)
 
-    _update_default_group(ldap, ctx, False)
+    if 'def_group_dn' in ctx:
+        _update_default_group(ldap, ctx, False)
 
     if 'description' in entry_attrs and NO_UPG_MAGIC in entry_attrs['description']:
         entry_attrs['description'].remove(NO_UPG_MAGIC)
@@ -291,31 +303,25 @@ def _update_default_group(ldap, ctx, force):
         try:
             (result, truncated) = ldap.find_entries(searchfilter,
                 [''], DN(api.env.container_user, api.env.basedn),
-                scope=ldap.SCOPE_SUBTREE, time_limit = -1)
+                scope=ldap.SCOPE_SUBTREE, time_limit=-1, size_limit=-1)
         except errors.NotFound:
             api.log.debug('All users have default group set')
             return
-        new_members = []
-        group_entry_attrs = ldap.get_entry(group_dn, ['member'])
-        existing_members = set(group_entry_attrs.get('member', []))
-        for m in result:
-            if m.dn not in existing_members:
-                new_members.append(m.dn)
 
-        if new_members:
-            members = group_entry_attrs.setdefault('member', [])
-            members.extend(new_members)
-
-            try:
-                ldap.update_entry(group_entry_attrs)
-            except errors.EmptyModlist:
-                pass
+        member_dns = [m.dn for m in result]
+        modlist = [(MOD_ADD, 'member', ldap.encode(member_dns))]
+        try:
+            with ldap.error_handler():
+                ldap.conn.modify_s(str(group_dn), modlist)
+        except errors.DatabaseError as e:
+            api.log.error('Adding new members to default group failed: %s \n'
+                          'members: %s', e, ','.join(member_dns))
 
         e = datetime.datetime.now()
         d = e - s
         mode = " (forced)" if force else ""
-        api.log.debug('Adding %d users to group%s duration %s',
-                len(new_members), mode, d)
+        api.log.info('Adding %d users to group%s duration %s',
+                      len(member_dns), mode, d)
 
 # GROUP MIGRATION CALLBACKS AND VARS
 
@@ -330,7 +336,7 @@ def _pre_migrate_group(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwarg
         for m in entry_attrs[member_attr]:
             try:
                 m = DN(m)
-            except ValueError, e:
+            except ValueError as e:
                 # This should be impossible unless the remote server
                 # doesn't enforce syntax checking.
                 api.log.error('Malformed DN %s: %s'  % (m, e))
@@ -342,11 +348,11 @@ def _pre_migrate_group(ldap, pkey, dn, entry_attrs, failed, config, ctx, **kwarg
                 continue
 
             if m.endswith(search_bases['user']):
-                api.log.info('migrating %s user %s' % (member_attr, m))
+                api.log.debug('migrating %s user %s', member_attr, m)
                 m = DN((api.Object.user.primary_key.name, rdnval),
                        api.env.container_user, api.env.basedn)
             elif m.endswith(search_bases['group']):
-                api.log.info('migrating %s group %s' % (member_attr, m))
+                api.log.debug('migrating %s group %s', member_attr, m)
                 m = DN((api.Object.group.primary_key.name, rdnval),
                        api.env.container_group, api.env.basedn)
             else:
@@ -607,6 +613,23 @@ class migrate_ds(Command):
             doc=_('Load CA certificate of LDAP server from FILE'),
             default=None
         ),
+        Bool('use_def_group?',
+            cli_name='use_default_group',
+            label=_('Add to default group'),
+            doc=_('Add migrated users without a group to a default group '
+                  '(default: true)'),
+            default=True,
+            autofill=True,
+        ),
+        StrEnum('scope',
+            cli_name='scope',
+            label=_('Search scope'),
+            doc=_('LDAP search scope for users and groups: base, onelevel, or '
+                  'subtree. Defaults to onelevel'),
+            values=tuple(_supported_scopes.keys()),
+            default=_default_scope,
+            autofill=True,
+        ),
     )
 
     has_output = (
@@ -701,6 +724,9 @@ can use their Kerberos accounts.''')
         failed = {} # {'OBJ': {'PKEY1': 'Failed 'cos blabla', ...}, ...}
         search_bases = self._get_search_bases(options, ds_base_dn, self.migrate_order)
         migration_start = datetime.datetime.now()
+
+        scope = _supported_scopes[options.get('scope')]
+
         for ldap_obj_name in self.migrate_order:
             ldap_obj = self.api.Object[ldap_obj_name]
 
@@ -717,7 +743,7 @@ can use their Kerberos accounts.''')
             try:
                 entries, truncated = ds_ldap.find_entries(
                     search_filter, ['*'], search_bases[ldap_obj_name],
-                    ds_ldap.SCOPE_ONELEVEL,
+                    scope,
                     time_limit=0, size_limit=-1,
                     search_refs=True    # migrated DS may contain search references
                 )
@@ -750,21 +776,19 @@ can use their Kerberos accounts.''')
                     blacklists[blacklist] = tuple()
 
             # get default primary group for new users
-            if 'def_group_dn' not in context:
+            if 'def_group_dn' not in context and options.get('use_def_group'):
                 def_group = config.get('ipadefaultprimarygroup')
                 context['def_group_dn'] = api.Object.group.get_dn(def_group)
                 try:
-                    g_attrs = ldap.get_entry(context['def_group_dn'], ['gidnumber', 'cn'])
+                    ldap.get_entry(context['def_group_dn'], ['gidnumber', 'cn'])
                 except errors.NotFound:
                     error_msg = _('Default group for new users not found')
                     raise errors.NotFound(reason=error_msg)
-                if 'gidnumber' in g_attrs:
-                    context['def_group_gid'] = g_attrs['gidnumber'][0]
 
             context['has_upg'] = ldap.has_upg()
 
-            valid_gids = []
-            invalid_gids = []
+            valid_gids = set()
+            invalid_gids = set()
             migrate_cnt = 0
             context['migrate_cnt'] = 0
             for entry_attrs in entries:
@@ -807,19 +831,19 @@ can use their Kerberos accounts.''')
                         )
                         if not entry_attrs.dn:
                             continue
-                    except errors.NotFound, e:
+                    except errors.NotFound as e:
                         failed[ldap_obj_name][pkey] = unicode(e.reason)
                         continue
 
                 try:
                     ldap.add_entry(entry_attrs)
-                except errors.ExecutionError, e:
+                except errors.ExecutionError as e:
                     callback = self.migrate_objects[ldap_obj_name]['exc_callback']
                     if callable(callback):
                         try:
                             callback(
                                 ldap, entry_attrs.dn, entry_attrs, e, options)
-                        except errors.ExecutionError, e:
+                        except errors.ExecutionError as e:
                             failed[ldap_obj_name][pkey] = unicode(e)
                             continue
                     else:
@@ -841,7 +865,8 @@ can use their Kerberos accounts.''')
                     api.log.info("%d %ss migrated. %s elapsed." % (migrate_cnt, ldap_obj_name, total_dur))
                 api.log.debug("%d %ss migrated, duration: %s (total %s)" % (migrate_cnt, ldap_obj_name, d, total_dur))
 
-        _update_default_group(ldap, context, True)
+        if 'def_group_dn' in context:
+            _update_default_group(ldap, context, True)
 
         return (migrated, failed)
 
@@ -859,7 +884,7 @@ can use their Kerberos accounts.''')
             return dict(result={}, failed={}, enabled=False, compat=True)
 
         # connect to DS
-        ds_ldap = ldap2(shared_instance=False, ldap_uri=ldapuri, base_dn='')
+        ds_ldap = ldap2(self.api, ldap_uri=ldapuri)
 
         cacert = None
         if options.get('cacertfile') is not None:
@@ -896,8 +921,8 @@ can use their Kerberos accounts.''')
                 try:
                     ds_base_dn = DN(entries[0]['namingcontexts'][0])
                     assert isinstance(ds_base_dn, DN)
-                except (IndexError, KeyError), e:
-                    raise StandardError(str(e))
+                except (IndexError, KeyError) as e:
+                    raise Exception(str(e))
 
         # migrate!
         (migrated, failed) = self.migrate(
