@@ -17,24 +17,33 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import print_function
+import sys
+
 from ipalib.plugins.baseldap import DN, LDAPObject, LDAPAddMember, LDAPRemoveMember
 from ipalib.plugins.baseldap import LDAPCreate, LDAPDelete, LDAPUpdate, LDAPSearch, LDAPRetrieve
 from ipalib import api, Int, Str, Bool, DateTime, Flag, Bytes, IntEnum, StrEnum, Password, _, ngettext
+from ipalib.messages import add_message, ResultFormattingError
 from ipalib.plugable import Registry
 from ipalib.errors import PasswordMismatch, ConversionError, LastMemberError, NotFound, ValidationError
 from ipalib.request import context
 from ipalib.frontend import Local
+from ipaplatform.paths import paths
+from ipapython.nsslib import NSSConnection
+from ipapython.version import API_VERSION
 
-from backports.ssl_match_hostname import match_hostname
 import base64
+import locale
 import uuid
-import urllib
-import urllib2
-import httplib
-import urlparse
 import qrcode
 import os
-import ssl
+
+import six
+from six import StringIO
+from six.moves import urllib
+
+if six.PY3:
+    unicode = str
 
 __doc__ = _("""
 OTP Tokens
@@ -90,15 +99,15 @@ class OTPTokenKey(Bytes):
         if isinstance(value, unicode):
             try:
                 value = base64.b32decode(value, True)
-            except TypeError, e:
+            except TypeError as e:
                 raise ConversionError(name=self.name, index=index, error=str(e))
 
         return super(OTPTokenKey, self)._convert_scalar(value, index)
 
 def _convert_owner(userobj, entry_attrs, options):
     if 'ipatokenowner' in entry_attrs and not options.get('raw', False):
-        entry_attrs['ipatokenowner'] = map(userobj.get_primary_key_from_dn,
-                                           entry_attrs['ipatokenowner'])
+        entry_attrs['ipatokenowner'] = [userobj.get_primary_key_from_dn(o)
+                                        for o in entry_attrs['ipatokenowner']]
 
 def _normalize_owner(userobj, entry_attrs):
     owner = entry_attrs.get('ipatokenowner', None)
@@ -161,7 +170,7 @@ class otptoken(LDAPObject):
             doc=_('Type of the token'),
             default=u'totp',
             autofill=True,
-            values=tuple(TOKEN_TYPES.keys() + [x.upper() for x in TOKEN_TYPES]),
+            values=tuple(list(TOKEN_TYPES) + [x.upper() for x in TOKEN_TYPES]),
             flags=('virtual_attribute', 'no_update'),
         ),
         Str('description?',
@@ -325,15 +334,15 @@ class otptoken_add(LDAPCreate):
         args['issuer'] = issuer
         args['secret'] = base64.b32encode(entry_attrs['ipatokenotpkey'])
         args['digits'] = entry_attrs['ipatokenotpdigits']
-        args['algorithm'] = entry_attrs['ipatokenotpalgorithm']
+        args['algorithm'] = entry_attrs['ipatokenotpalgorithm'].upper()
         if options['type'] == 'totp':
             args['period'] = entry_attrs['ipatokentotptimestep']
         elif options['type'] == 'hotp':
             args['counter'] = entry_attrs['ipatokenhotpcounter']
 
         # Build the URI
-        label = urllib.quote(entry_attrs['ipatokenuniqueid'])
-        parameters = urllib.urlencode(args)
+        label = urllib.parse.quote(entry_attrs['ipatokenuniqueid'])
+        parameters = urllib.parse.urlencode(args)
         uri = u'otpauth://%s/%s:%s?%s' % (options['type'], issuer, label, parameters)
         setattr(context, 'uri', uri)
 
@@ -346,18 +355,72 @@ class otptoken_add(LDAPCreate):
         _convert_owner(self.api.Object.user, entry_attrs, options)
         return super(otptoken_add, self).post_callback(ldap, dn, entry_attrs, *keys, **options)
 
-    def output_for_cli(self, textui, output, *args, **options):
-        uri = output['result'].get('uri', None)
-        rv = super(otptoken_add, self).output_for_cli(textui, output, *args, **options)
-
+    def _get_qrcode(self, output, uri, version):
         # Print QR code to terminal if specified
-        if uri and not options.get('no_qrcode', False):
-            print "\n"
-            qr = qrcode.QRCode()
-            qr.add_data(uri)
-            qr.make()
-            qr.print_ascii(tty=True)
-            print "\n"
+        qr_output = StringIO()
+        qr = qrcode.QRCode()
+        qr.add_data(uri)
+        qr.make()
+        qr.print_ascii(out=qr_output, tty=False)
+
+        encoding = getattr(sys.stdout, 'encoding', None)
+        if encoding is None:
+            encoding = locale.getpreferredencoding(False)
+
+        try:
+            qr_code = qr_output.getvalue().decode(encoding)
+        except UnicodeError:
+            add_message(
+                version,
+                output,
+                message=ResultFormattingError(
+                    message=_("Unable to display QR code using the configured "
+                              "output encoding. Please use the token URI to "
+                              "configure you OTP device")
+                )
+            )
+            return None
+
+        if sys.stdout.isatty():
+            output_width = self.api.Backend.textui.get_tty_width()
+            qr_code_width = len(qr_code.splitlines()[0])
+            if qr_code_width > output_width:
+                add_message(
+                    version,
+                    output,
+                    message=ResultFormattingError(
+                        message=_(
+                            "QR code width is greater than that of the output "
+                            "tty. Please resize your terminal.")
+                    )
+                )
+
+        return qr
+
+    def output_for_cli(self, textui, output, *args, **options):
+        # copy-pasted from ipalib/Frontend.__do_call()
+        # because option handling is broken on client-side
+        if 'version' in options:
+            pass
+        elif self.api.env.skip_version_check:
+            options['version'] = u'2.0'
+        else:
+            options['version'] = API_VERSION
+
+        uri = output['result'].get('uri', None)
+
+        if uri is not None and not options.get('no_qrcode', False):
+            qr = self._get_qrcode(output, uri, options['version'])
+        else:
+            qr = None
+
+        rv = super(otptoken_add, self).output_for_cli(
+                textui, output, *args, **options)
+
+        if qr is not None:
+            print("\n")
+            qr.print_ascii(tty=sys.stdout.isatty())
+            print("\n")
 
         return rv
 
@@ -467,48 +530,29 @@ class otptoken_add_managedby(LDAPAddMember):
 
 @register()
 class otptoken_remove_managedby(LDAPRemoveMember):
-    __doc__ = _('Remove hosts that can manage this host.')
+    __doc__ = _('Remove users that can manage this token.')
 
     member_attributes = ['managedby']
 
-class HTTPSConnection(httplib.HTTPConnection):
-    "Generates an SSL HTTP connection that performs hostname validation."
 
-    ssl_kwargs = ssl.wrap_socket.func_code.co_varnames[1:ssl.wrap_socket.func_code.co_argcount] #pylint: disable=E1101
-    default_port = httplib.HTTPS_PORT
-
-    def __init__(self, host, **kwargs):
-        # Strip out arguments we want to pass to ssl.wrap_socket()
-        self.__kwargs = {k: v for k, v in kwargs.items() if k in self.ssl_kwargs}
-        for k in self.__kwargs:
-            del kwargs[k]
-
-        # Can't use super() because the parent is an old-style class.
-        httplib.HTTPConnection.__init__(self, host, **kwargs)
-
-    def connect(self):
-        # Create the raw socket and wrap it in ssl.
-        httplib.HTTPConnection.connect(self)
-        self.sock = ssl.wrap_socket(self.sock, **self.__kwargs)
-
-        # Verify the remote hostname.
-        match_hostname(self.sock.getpeercert(), self.host.split(':', 1)[0])
-
-class HTTPSHandler(urllib2.HTTPSHandler):
+class HTTPSHandler(urllib.request.HTTPSHandler):
     "Opens SSL HTTPS connections that perform hostname validation."
 
     def __init__(self, **kwargs):
         self.__kwargs = kwargs
 
         # Can't use super() because the parent is an old-style class.
-        urllib2.HTTPSHandler.__init__(self)
+        urllib.request.HTTPSHandler.__init__(self)
 
     def __inner(self, host, **kwargs):
         tmp = self.__kwargs.copy()
         tmp.update(kwargs)
-        return HTTPSConnection(host, **tmp)
+        # NSSConnection doesn't support timeout argument
+        tmp.pop('timeout', None)
+        return NSSConnection(host, **tmp)
 
     def https_open(self, req):
+        # pylint: disable=no-member
         return self.do_open(self.__inner, req)
 
 @register()
@@ -532,10 +576,12 @@ class otptoken_sync(Local):
         status = {'result': {self.header: 'unknown'}}
 
         # Get the sync URI.
-        segments = list(urlparse.urlparse(self.api.env.xmlrpc_uri))
+        segments = list(urllib.parse.urlparse(self.api.env.xmlrpc_uri))
         assert segments[0] == 'https' # Ensure encryption.
         segments[2] = segments[2].replace('/xml', '/session/sync_token')
-        sync_uri = urlparse.urlunparse(segments)
+        # urlunparse *can* take one argument
+        # pylint: disable=too-many-function-args
+        sync_uri = urllib.parse.urlunparse(segments)
 
         # Prepare the query.
         query = {k: v for k, v in kwargs.items()
@@ -544,13 +590,14 @@ class otptoken_sync(Local):
             obj = self.api.Object.otptoken
             query['token'] = DN((obj.primary_key.name, args[0]),
                                 obj.container_dn, self.api.env.basedn)
-        query = urllib.urlencode(query)
+        query = urllib.parse.urlencode(query)
 
         # Sync the token.
-        handler = HTTPSHandler(ca_certs=os.path.join(self.api.env.confdir, 'ca.crt'),
-                               cert_reqs=ssl.CERT_REQUIRED,
-                               ssl_version=ssl.PROTOCOL_TLSv1)
-        rsp = urllib2.build_opener(handler).open(sync_uri, query)
+        # pylint: disable=E1101
+        handler = HTTPSHandler(dbdir=paths.IPA_NSSDB_DIR,
+                               tls_version_min=api.env.tls_version_min,
+                               tls_version_max=api.env.tls_version_max)
+        rsp = urllib.request.build_opener(handler).open(sync_uri, query)
         if rsp.getcode() == 200:
             status['result'][self.header] = rsp.info().get(self.header, 'unknown')
         rsp.close()
