@@ -20,7 +20,10 @@
 import re
 import traceback
 
+import six
+
 from ipalib.plugins import baseldap
+from ipalib.plugins.privilege import validate_permission_to_privilege
 from ipalib import errors
 from ipalib.parameters import Str, StrEnum, DNParam, Flag
 from ipalib import api, _, ngettext
@@ -30,6 +33,9 @@ from ipalib.aci import ACI
 from ipapython.dn import DN
 from ipalib.request import context
 
+if six.PY3:
+    unicode = str
+
 __doc__ = _("""
 Permissions
 """) + _("""
@@ -133,7 +139,7 @@ class DNOrURL(DNParam):
     """
 
     def _convert_scalar(self, value, index=None):
-        if isinstance(value, basestring) and value.startswith('ldap:///'):
+        if isinstance(value, six.string_types) and value.startswith('ldap:///'):
             value = strip_ldap_prefix(value)
         return super(DNOrURL, self)._convert_scalar(value, index=index)
 
@@ -302,6 +308,22 @@ class permission(baseldap.LDAPObject):
                   '(must be in the subtree, but may not yet exist)'),
         ),
 
+        DNParam(
+            'ipapermtargetto?',
+            cli_name='targetto',
+            label=_('Target DN subtree'),
+            doc=_('Optional DN subtree where an entry can be moved to '
+                  '(must be in the subtree, but may not yet exist)'),
+        ),
+
+        DNParam(
+            'ipapermtargetfrom?',
+            cli_name='targetfrom',
+            label=_('Origin DN subtree'),
+            doc=_('Optional DN subtree from where an entry can be moved '
+                  '(must be in the subtree, but may not yet exist)'),
+        ),
+
         Str('memberof*',
             label=_('Member of group'),  # FIXME: Does this label make sense?
             doc=_('Target members of a group (sets memberOf targetfilter)'),
@@ -532,6 +554,18 @@ class permission(baseldap.LDAPObject):
             aci_parts.append("(target = \"%s\")" %
                              'ldap:///%s' % ipapermtarget)
 
+        # target_to
+        ipapermtargetto = entry.single_value.get('ipapermtargetto')
+        if ipapermtargetto:
+            aci_parts.append("(target_to = \"%s\")" %
+                             'ldap:///%s' % ipapermtargetto)
+
+        # target_from
+        ipapermtargetfrom = entry.single_value.get('ipapermtargetfrom')
+        if ipapermtargetfrom:
+            aci_parts.append("(target_from = \"%s\")" %
+                             'ldap:///%s' % ipapermtargetfrom)
+
         # targetfilter
         ipapermtargetfilter = entry.get('ipapermtargetfilter')
         if ipapermtargetfilter:
@@ -620,7 +654,7 @@ class permission(baseldap.LDAPObject):
         try:
             ldap.update_entry(acientry)
         except errors.EmptyModlist:
-            self.log.info('No changes to ACI')
+            self.log.debug('No changes to ACI')
         return acientry, acistring
 
     def _get_aci_entry_and_string(self, permission_entry, name=None,
@@ -973,7 +1007,7 @@ class permission_add(baseldap.LDAPCreate):
     def post_callback(self, ldap, dn, entry, *keys, **options):
         try:
             self.obj.add_aci(entry)
-        except Exception, e:
+        except Exception as e:
             # Adding the ACI failed.
             # We want to be 100% sure the ACI is not there, so try to
             # remove it. (This is a no-op if the ACI was not added.)
@@ -1117,7 +1151,7 @@ class permission_mod(baseldap.LDAPUpdate):
         # Since `entry` only contains the attributes we are currently changing,
         # it cannot be used directly to generate an ACI.
         # First we need to copy the original data into it.
-        for key, value in old_entry.iteritems():
+        for key, value in old_entry.items():
             if (key not in options and
                     key != 'cn' and
                     key not in self.obj.attribute_members):
@@ -1156,7 +1190,7 @@ class permission_mod(baseldap.LDAPUpdate):
             context.permision_moving_aci = True
             try:
                 context.old_aci_info = self.obj.remove_aci(old_entry)
-            except errors.NotFound, e:
+            except errors.NotFound as e:
                 self.log.error('permission ACI not found: %s' % e)
 
         # To pass data to postcallback, we currently need to use the context
@@ -1165,7 +1199,7 @@ class permission_mod(baseldap.LDAPUpdate):
         return dn
 
     def exc_callback(self, keys, options, exc, call_func, *call_args, **call_kwargs):
-        if call_func.func_name == 'update_entry':
+        if call_func.__name__ == 'update_entry':
             self._revert_aci()
         raise exc
 
@@ -1258,8 +1292,7 @@ class permission_find(baseldap.LDAPSearch):
             if 'sizelimit' in options:
                 max_entries = options['sizelimit']
             else:
-                config = ldap.get_ipa_config()
-                max_entries = int(config.single_value['ipasearchrecordslimit'])
+                max_entries = self.api.Backend.ldap2.size_limit
 
             filters = ['(objectclass=ipaPermission)',
                        '(!(ipaPermissionType=V2))']
@@ -1324,7 +1357,7 @@ class permission_find(baseldap.LDAPSearch):
 
         for entry in entries:
             if options.get('pkey_only'):
-                for opt_name in entry.keys():
+                for opt_name in list(entry):
                     if opt_name != self.obj.primary_key.name:
                         del entry[opt_name]
             else:
@@ -1349,6 +1382,12 @@ class permission_add_member(baseldap.LDAPAddMember):
     """Add members to a permission."""
     NO_CLI = True
 
+    def pre_callback(self, ldap, dn, member_dns, failed, *keys, **options):
+        # We can only add permissions with bind rule type set to
+        # "permission" (or old-style permissions)
+        validate_permission_to_privilege(self.api, keys[-1])
+        return dn
+
 
 @register()
 class permission_remove_member(baseldap.LDAPRemoveMember):
