@@ -28,6 +28,11 @@ from ipapython.ipautil import run
 from ipapython.dn import DN
 from distutils import version
 
+import six
+
+if six.PY3:
+    unicode = str
+
 __doc__ = _("""
 Password policy
 
@@ -159,10 +164,15 @@ class cosentry_add(LDAPCreate):
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
+        
         # check for existence of the group
         group_dn = self.api.Object.group.get_dn(keys[-1])
-        result = ldap.get_entry(group_dn, ['objectclass'])
-        oc = map(lambda x:x.lower(),result['objectclass'])
+        try:
+            result = ldap.get_entry(group_dn, ['objectclass'])
+        except errors.NotFound:
+            self.api.Object.group.handle_not_found(keys[-1]) 
+
+        oc = [x.lower() for x in result['objectclass']]
         if 'mepmanagedentry' in oc:
             raise errors.ManagedPolicyError()
         self.obj.check_priority_uniqueness(*keys, **options)
@@ -269,9 +279,9 @@ class pwpolicy(LDAPObject):
     has_lockout = False
     lockout_params = ()
 
-    (stdout, stderr, rc) = run(['klist', '-V'], raiseonerr=False)
-    if rc == 0:
-        verstr = stdout.split()[-1]
+    result = run(['klist', '-V'], raiseonerr=False, capture_output=True)
+    if result.returncode == 0:
+        verstr = result.output.split()[-1]
         ver = version.LooseVersion(verstr)
         min = version.LooseVersion(MIN_KRB5KDC_WITH_LOCKOUT)
         if ver >= min:
@@ -363,11 +373,11 @@ class pwpolicy(LDAPObject):
         if not options.get('raw', False):
             if 'krbmaxpwdlife' in entry_attrs:
                 entry_attrs['krbmaxpwdlife'][0] = unicode(
-                    int(entry_attrs['krbmaxpwdlife'][0]) / 86400
+                    int(entry_attrs['krbmaxpwdlife'][0]) // 86400
                 )
             if 'krbminpwdlife' in entry_attrs:
                 entry_attrs['krbminpwdlife'][0] = unicode(
-                    int(entry_attrs['krbminpwdlife'][0]) / 3600
+                    int(entry_attrs['krbminpwdlife'][0]) // 3600
                 )
 
     def convert_time_on_input(self, entry_attrs):
@@ -486,7 +496,7 @@ class pwpolicy_mod(LDAPUpdate):
                 self.api.Command.cosentry_mod(
                     keys[-1], cospriority=options['cospriority']
                     )
-            except errors.EmptyModlist, e:
+            except errors.EmptyModlist as e:
                 if len(entry_attrs) == 1:   # cospriority only was passed
                     raise e
             else:
@@ -501,7 +511,7 @@ class pwpolicy_mod(LDAPUpdate):
         return dn
 
     def exc_callback(self, keys, options, exc, call_func, *call_args, **call_kwargs):
-        if call_func.func_name == 'update_entry':
+        if call_func.__name__ == 'update_entry':
             if isinstance(exc, errors.EmptyModlist):
                 entry_attrs = call_args[0]
                 cosupdate = getattr(context, 'cosupdate')
