@@ -19,6 +19,7 @@
 
 from ipalib import api, errors
 from ipalib import Str, StrEnum, Bool
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
 from ipalib.plugins.hbacrule import is_all
@@ -68,6 +69,8 @@ SEEALSO:
  and the default SELinux user are available in the config-show command.
 """)
 
+register = Registry()
+
 notboth_err = _('HBAC rule and local members cannot both be set')
 
 
@@ -112,7 +115,7 @@ def validate_selinuxuser_inlist(ldap, user):
 
     Returns nothing if the user is found, raises an exception otherwise.
     """
-    config = ldap.get_ipa_config()[1]
+    config = ldap.get_ipa_config()
     item = config.get('ipaselinuxusermaporder', [])
     if len(item) != 1:
         raise errors.NotFound(reason=_('SELinux user map list not '
@@ -126,6 +129,7 @@ def validate_selinuxuser_inlist(ldap, user):
     return
 
 
+@register()
 class selinuxusermap(LDAPObject):
     """
     SELinux User Map object.
@@ -134,11 +138,12 @@ class selinuxusermap(LDAPObject):
     object_name = _('SELinux User Map rule')
     object_name_plural = _('SELinux User Map rules')
     object_class = ['ipaassociation', 'ipaselinuxusermap']
+    permission_filter_objectclasses = ['ipaselinuxusermap']
     default_attributes = [
         'cn', 'ipaenabledflag',
         'description', 'usercategory', 'hostcategory',
         'ipaenabledflag', 'memberuser', 'memberhost',
-        'memberhostgroup', 'seealso', 'ipaselinuxuser',
+        'seealso', 'ipaselinuxuser',
     ]
     uuid_attribute = 'ipauniqueid'
     rdn_attribute = 'ipauniqueid'
@@ -146,6 +151,44 @@ class selinuxusermap(LDAPObject):
         'memberuser': ['user', 'group'],
         'memberhost': ['host', 'hostgroup'],
     }
+    managed_permissions = {
+        'System: Read SELinux User Maps': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'accesstime', 'cn', 'description', 'hostcategory',
+                'ipaenabledflag', 'ipaselinuxuser', 'ipauniqueid',
+                'memberhost', 'memberuser', 'seealso', 'usercategory',
+                'objectclass', 'member',
+            },
+        },
+        'System: Add SELinux User Maps': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///ipauniqueid=*,cn=usermap,cn=selinux,$SUFFIX")(version 3.0;acl "permission:Add SELinux User Maps";allow (add) groupdn = "ldap:///cn=Add SELinux User Maps,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'SELinux User Map Administrators'},
+        },
+        'System: Modify SELinux User Maps': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'cn', 'ipaenabledflag', 'ipaselinuxuser', 'memberhost',
+                'memberuser', 'seealso'
+            },
+            'replaces': [
+                '(targetattr = "cn || memberuser || memberhost || seealso || ipaselinuxuser || ipaenabledflag")(target = "ldap:///ipauniqueid=*,cn=usermap,cn=selinux,$SUFFIX")(version 3.0;acl "permission:Modify SELinux User Maps";allow (write) groupdn = "ldap:///cn=Modify SELinux User Maps,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'SELinux User Map Administrators'},
+        },
+        'System: Remove SELinux User Maps': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///ipauniqueid=*,cn=usermap,cn=selinux,$SUFFIX")(version 3.0;acl "permission:Remove SELinux User Maps";allow (delete) groupdn = "ldap:///cn=Remove SELinux User Maps,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'SELinux User Map Administrators'},
+        },
+    }
 
     # These maps will not show as members of other entries
 
@@ -217,13 +260,13 @@ class selinuxusermap(LDAPObject):
             return str(dn)
         except ValueError:
             try:
-                (dn, entry_attrs) = self.backend.find_entry_by_attr(
+                entry_attrs = self.backend.find_entry_by_attr(
                     self.api.Object['hbacrule'].primary_key.name,
                     seealso,
                     self.api.Object['hbacrule'].object_class,
                     [''],
                     DN(self.api.Object['hbacrule'].container_dn, api.env.basedn))
-                seealso = dn
+                seealso = entry_attrs.dn
             except errors.NotFound:
                 raise errors.NotFound(reason=_('HBAC rule %(rule)s not found') % dict(rule=seealso))
 
@@ -237,12 +280,12 @@ class selinuxusermap(LDAPObject):
             return
 
         if 'seealso' in entry_attrs:
-            (hbac_dn, hbac_attrs) = ldap.get_entry(entry_attrs['seealso'][0], ['cn'])
+            hbac_attrs = ldap.get_entry(entry_attrs['seealso'][0], ['cn'])
             entry_attrs['seealso'] = hbac_attrs['cn'][0]
 
-api.register(selinuxusermap)
 
 
+@register()
 class selinuxusermap_add(LDAPCreate):
     __doc__ = _('Create a new SELinux User Map.')
 
@@ -277,17 +320,17 @@ class selinuxusermap_add(LDAPCreate):
 
         return dn
 
-api.register(selinuxusermap_add)
 
 
+@register()
 class selinuxusermap_del(LDAPDelete):
     __doc__ = _('Delete a SELinux User Map.')
 
     msg_summary = _('Deleted SELinux User Map "%(value)s"')
 
-api.register(selinuxusermap_del)
 
 
+@register()
 class selinuxusermap_mod(LDAPUpdate):
     __doc__ = _('Modify a SELinux User Map.')
 
@@ -296,7 +339,7 @@ class selinuxusermap_mod(LDAPUpdate):
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (_dn, _entry_attrs) = ldap.get_entry(dn, attrs_list)
+            _entry_attrs = ldap.get_entry(dn, attrs_list)
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
@@ -341,9 +384,9 @@ class selinuxusermap_mod(LDAPUpdate):
         self.obj._convert_seealso(ldap, entry_attrs, **options)
         return dn
 
-api.register(selinuxusermap_mod)
 
 
+@register()
 class selinuxusermap_find(LDAPSearch):
     __doc__ = _('Search for SELinux User Maps.')
 
@@ -369,14 +412,13 @@ all=True)['result']
     def post_callback(self, ldap, entries, truncated, *args, **options):
         if options.get('pkey_only', False):
             return truncated
-        for entry in entries:
-            (dn, attrs) = entry
+        for attrs in entries:
             self.obj._convert_seealso(ldap, attrs, **options)
         return truncated
 
-api.register(selinuxusermap_find)
 
 
+@register()
 class selinuxusermap_show(LDAPRetrieve):
     __doc__ = _('Display the properties of a SELinux User Map rule.')
 
@@ -385,9 +427,9 @@ class selinuxusermap_show(LDAPRetrieve):
         self.obj._convert_seealso(ldap, entry_attrs, **options)
         return dn
 
-api.register(selinuxusermap_show)
 
 
+@register()
 class selinuxusermap_enable(LDAPQuery):
     __doc__ = _('Enable an SELinux User Map rule.')
 
@@ -398,23 +440,26 @@ class selinuxusermap_enable(LDAPQuery):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
-        entry_attrs = {'ipaenabledflag': 'TRUE'}
-
         try:
-            ldap.update_entry(dn, entry_attrs)
-        except errors.EmptyModlist:
-            raise errors.AlreadyActive()
+            entry_attrs = ldap.get_entry(dn, ['ipaenabledflag'])
         except errors.NotFound:
             self.obj.handle_not_found(cn)
 
+        entry_attrs['ipaenabledflag'] = ['TRUE']
+
+        try:
+            ldap.update_entry(entry_attrs)
+        except errors.EmptyModlist:
+            raise errors.AlreadyActive()
+
         return dict(
             result=True,
-            value=cn,
+            value=pkey_to_value(cn, options),
         )
 
-api.register(selinuxusermap_enable)
 
 
+@register()
 class selinuxusermap_disable(LDAPQuery):
     __doc__ = _('Disable an SELinux User Map rule.')
 
@@ -425,23 +470,26 @@ class selinuxusermap_disable(LDAPQuery):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
-        entry_attrs = {'ipaenabledflag': 'FALSE'}
-
         try:
-            ldap.update_entry(dn, entry_attrs)
-        except errors.EmptyModlist:
-            raise errors.AlreadyInactive()
+            entry_attrs = ldap.get_entry(dn, ['ipaenabledflag'])
         except errors.NotFound:
             self.obj.handle_not_found(cn)
 
+        entry_attrs['ipaenabledflag'] = ['FALSE']
+
+        try:
+            ldap.update_entry(entry_attrs)
+        except errors.EmptyModlist:
+            raise errors.AlreadyInactive()
+
         return dict(
             result=True,
-            value=cn,
+            value=pkey_to_value(cn, options),
         )
 
-api.register(selinuxusermap_disable)
 
 
+@register()
 class selinuxusermap_add_user(LDAPAddMember):
     __doc__ = _('Add users and groups to an SELinux User Map rule.')
 
@@ -451,7 +499,8 @@ class selinuxusermap_add_user(LDAPAddMember):
     def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, self.obj.default_attributes)
+            entry_attrs = ldap.get_entry(dn, self.obj.default_attributes)
+            dn = entry_attrs.dn
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
         if 'usercategory' in entry_attrs and \
@@ -462,18 +511,18 @@ class selinuxusermap_add_user(LDAPAddMember):
             raise errors.MutuallyExclusiveError(reason=notboth_err)
         return dn
 
-api.register(selinuxusermap_add_user)
 
 
+@register()
 class selinuxusermap_remove_user(LDAPRemoveMember):
     __doc__ = _('Remove users and groups from an SELinux User Map rule.')
 
     member_attributes = ['memberuser']
     member_count_out = ('%i object removed.', '%i objects removed.')
 
-api.register(selinuxusermap_remove_user)
 
 
+@register()
 class selinuxusermap_add_host(LDAPAddMember):
     __doc__ = _('Add target hosts and hostgroups to an SELinux User Map rule.')
 
@@ -483,7 +532,8 @@ class selinuxusermap_add_host(LDAPAddMember):
     def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, self.obj.default_attributes)
+            entry_attrs = ldap.get_entry(dn, self.obj.default_attributes)
+            dn = entry_attrs.dn
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
         if 'hostcategory' in entry_attrs and \
@@ -494,13 +544,12 @@ class selinuxusermap_add_host(LDAPAddMember):
             raise errors.MutuallyExclusiveError(reason=notboth_err)
         return dn
 
-api.register(selinuxusermap_add_host)
 
 
+@register()
 class selinuxusermap_remove_host(LDAPRemoveMember):
     __doc__ = _('Remove target hosts and hostgroups from an SELinux User Map rule.')
 
     member_attributes = ['memberhost']
     member_count_out = ('%i object removed.', '%i objects removed.')
 
-api.register(selinuxusermap_remove_host)
