@@ -23,6 +23,7 @@ import sys
 
 from ipalib import api, errors, util
 from ipalib import Str
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
 
@@ -41,8 +42,11 @@ EXAMPLES:
 
 """)
 
+register = Registry()
+
 topic = ('sudo', _('commands for controlling sudo configuration'))
 
+@register()
 class sudocmd(LDAPObject):
     """
     Sudo Command object.
@@ -51,6 +55,7 @@ class sudocmd(LDAPObject):
     object_name = _('sudo command')
     object_name_plural = _('sudo commands')
     object_class = ['ipaobject', 'ipasudocmd']
+    permission_filter_objectclasses = ['ipasudocmd']
     # object_class_config = 'ipahostobjectclasses'
     search_attributes = [
         'sudocmd', 'description',
@@ -63,6 +68,43 @@ class sudocmd(LDAPObject):
     }
     uuid_attribute = 'ipauniqueid'
     rdn_attribute = 'ipauniqueid'
+    managed_permissions = {
+        'System: Read Sudo Commands': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'description', 'ipauniqueid', 'memberof', 'objectclass',
+                'sudocmd',
+            },
+        },
+        'System: Add Sudo Command': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///sudocmd=*,cn=sudocmds,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Add Sudo command";allow (add) groupdn = "ldap:///cn=Add Sudo command,cn=permissions,cn=pbac,$SUFFIX";)',
+                '(targetfilter = "(objectclass=ipasudocmd)")(target = "ldap:///cn=sudocmds,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Add Sudo command";allow (add) groupdn = "ldap:///cn=Add Sudo command,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Sudo Administrator'},
+        },
+        'System: Delete Sudo Command': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///sudocmd=*,cn=sudocmds,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Delete Sudo command";allow (delete) groupdn = "ldap:///cn=Delete Sudo command,cn=permissions,cn=pbac,$SUFFIX";)',
+                '(targetfilter = "(objectclass=ipasudocmd)")(target = "ldap:///cn=sudocmds,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Delete Sudo command";allow (delete) groupdn = "ldap:///cn=Delete Sudo command,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Sudo Administrator'},
+        },
+        'System: Modify Sudo Command': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'description'},
+            'replaces': [
+                '(targetattr = "description")(target = "ldap:///sudocmd=*,cn=sudocmds,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Modify Sudo command";allow (write) groupdn = "ldap:///cn=Modify Sudo command,cn=permissions,cn=pbac,$SUFFIX";)',
+                '(targetfilter = "(objectclass=ipasudocmd)")(targetattr = "description")(target = "ldap:///cn=sudocmds,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Modify Sudo command";allow (write) groupdn = "ldap:///cn=Modify Sudo command,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Sudo Administrator'},
+        },
+    }
+
     label = _('Sudo Commands')
     label_singular = _('Sudo Command')
 
@@ -87,23 +129,23 @@ class sudocmd(LDAPObject):
             self.backend.get_entry(dn, [''])
         except errors.NotFound:
             try:
-                (dn, entry_attrs) = self.backend.find_entry_by_attr(
+                entry_attrs = self.backend.find_entry_by_attr(
                     'sudocmd', keys[-1], self.object_class, [''],
-                    DN(self.container_dn, api.env.basedn)
-                )
+                    DN(self.container_dn, api.env.basedn))
+                dn = entry_attrs.dn
             except errors.NotFound:
                 pass
         return dn
 
-api.register(sudocmd)
 
+@register()
 class sudocmd_add(LDAPCreate):
     __doc__ = _('Create new Sudo Command.')
 
     msg_summary = _('Added Sudo Command "%(value)s"')
 
-api.register(sudocmd_add)
 
+@register()
 class sudocmd_del(LDAPDelete):
     __doc__ = _('Delete Sudo Command.')
 
@@ -125,7 +167,7 @@ class sudocmd_del(LDAPDelete):
         except errors.NotFound:
             pass
         else:
-            for entry_dn, entry_attrs in entries:
+            for entry_attrs in entries:
                 [cn] = entry_attrs['cn']
                 dependent_sudorules.append(cn)
 
@@ -135,15 +177,15 @@ class sudocmd_del(LDAPDelete):
                 dependent=', '.join(dependent_sudorules))
         return dn
 
-api.register(sudocmd_del)
 
+@register()
 class sudocmd_mod(LDAPUpdate):
     __doc__ = _('Modify Sudo Command.')
 
     msg_summary = _('Modified Sudo Command "%(value)s"')
 
-api.register(sudocmd_mod)
 
+@register()
 class sudocmd_find(LDAPSearch):
     __doc__ = _('Search for Sudo Commands.')
 
@@ -151,9 +193,8 @@ class sudocmd_find(LDAPSearch):
         '%(count)d Sudo Command matched', '%(count)d Sudo Commands matched', 0
     )
 
-api.register(sudocmd_find)
 
+@register()
 class sudocmd_show(LDAPRetrieve):
     __doc__ = _('Display Sudo Command.')
 
-api.register(sudocmd_show)
