@@ -21,33 +21,31 @@
 """
 RPC client and shared RPC client/server functionality.
 
-This module adds some additional functionality on top of the ``xmlrpclib``
-module in the Python standard library.  For documentation on the
-``xmlrpclib`` module, see:
+This module adds some additional functionality on top of the ``xmlrpc.client``
+module in the Python standard library (``xmlrpclib`` in Python 2).
+For documentation on the ``xmlrpclib`` module, see:
 
-    http://docs.python.org/library/xmlrpclib.html
+    http://docs.python.org/2/library/xmlrpclib.html
 
 Also see the `ipaserver.rpcserver` module.
 """
 
-from types import NoneType
 from decimal import Decimal
 import sys
 import datetime
 import os
 import locale
 import base64
-import urllib
 import json
 import socket
-from urllib2 import urlparse
+import gzip
 
-from xmlrpclib import (Binary, Fault, DateTime, dumps, loads, ServerProxy,
-        Transport, ProtocolError, MININT, MAXINT)
-import kerberos
+import gssapi
 from dns import resolver, rdatatype
 from dns.exception import DNSException
 from nss.error import NSPRError
+import six
+from six.moves import urllib
 
 from ipalib.backend import Connectible
 from ipalib.constants import LDAP_GENERALIZED_TIME_FORMAT
@@ -55,7 +53,6 @@ from ipalib.errors import (public_errors, UnknownError, NetworkError,
     KerberosError, XMLRPCMarshallError, JSONError, ConversionError)
 from ipalib import errors, capabilities
 from ipalib.request import context, Connection
-from ipalib.util import get_current_principal
 from ipapython.ipa_log_manager import root_logger
 from ipapython import ipautil
 from ipapython import kernel_keyring
@@ -63,13 +60,28 @@ from ipaplatform.paths import paths
 from ipapython.cookie import Cookie
 from ipapython.dnsutil import DNSName
 from ipalib.text import _
-from ipapython.nsslib import NSSHTTPS, NSSConnection
+import ipapython.nsslib
+from ipapython.nsslib import NSSConnection
 from ipalib.krb_utils import KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, KRB5KRB_AP_ERR_TKT_EXPIRED, \
-                             KRB5_FCC_PERM, KRB5_FCC_NOFILE, KRB5_CC_FORMAT, KRB5_REALM_CANT_RESOLVE
+                             KRB5_FCC_PERM, KRB5_FCC_NOFILE, KRB5_CC_FORMAT, \
+                             KRB5_REALM_CANT_RESOLVE, KRB5_CC_NOTFOUND, get_principal
 from ipapython.dn import DN
 from ipalib.capabilities import VERSION_WITHOUT_CAPABILITIES
 from ipalib import api
 
+# The XMLRPC client is in  "six.moves.xmlrpc_client", but pylint
+# cannot handle that
+try:
+    from xmlrpclib import (Binary, Fault, DateTime, dumps, loads, ServerProxy,
+            Transport, ProtocolError, MININT, MAXINT)
+except ImportError:
+    from xmlrpc.client import (Binary, Fault, DateTime, dumps, loads, ServerProxy,
+            Transport, ProtocolError, MININT, MAXINT)
+
+
+if six.PY3:
+    unicode = str
+
 COOKIE_NAME = 'ipa_session'
 KEYRING_COOKIE_NAME = '%s_cookie:%%s' % COOKIE_NAME
 
@@ -94,7 +106,7 @@ def update_persistent_client_session_data(principal, data):
 
     try:
         keyname = client_session_keyring_keyname(principal)
-    except Exception, e:
+    except Exception as e:
         raise ValueError(str(e))
 
     # kernel_keyring only raises ValueError (why??)
@@ -110,7 +122,7 @@ def read_persistent_client_session_data(principal):
 
     try:
         keyname = client_session_keyring_keyname(principal)
-    except Exception, e:
+    except Exception as e:
         raise ValueError(str(e))
 
     # kernel_keyring only raises ValueError (why??)
@@ -126,7 +138,7 @@ def delete_persistent_client_session_data(principal):
 
     try:
         keyname = client_session_keyring_keyname(principal)
-    except Exception, e:
+    except Exception as e:
         raise ValueError(str(e))
 
     # kernel_keyring only raises ValueError (why??)
@@ -134,16 +146,16 @@ def delete_persistent_client_session_data(principal):
 
 def xml_wrap(value, version):
     """
-    Wrap all ``str`` in ``xmlrpclib.Binary``.
+    Wrap all ``str`` in ``xmlrpc.client.Binary``.
 
-    Because ``xmlrpclib.dumps()`` will itself convert all ``unicode`` instances
+    Because ``xmlrpc.client.dumps()`` will itself convert all ``unicode`` instances
     into UTF-8 encoded ``str`` instances, we don't do it here.
 
     So in total, when encoding data for an XML-RPC packet, the following
     transformations occur:
 
         * All ``str`` instances are treated as binary data and are wrapped in
-          an ``xmlrpclib.Binary()`` instance.
+          an ``xmlrpc.client.Binary()`` instance.
 
         * Only ``unicode`` instances are treated as character data. They get
           converted to UTF-8 encoded ``str`` instances (although as mentioned,
@@ -157,19 +169,19 @@ def xml_wrap(value, version):
         return tuple(xml_wrap(v, version) for v in value)
     if isinstance(value, dict):
         return dict(
-            (k, xml_wrap(v, version)) for (k, v) in value.iteritems()
+            (k, xml_wrap(v, version)) for (k, v) in value.items()
         )
-    if type(value) is str:
+    if type(value) is bytes:
         return Binary(value)
     if type(value) is Decimal:
         # transfer Decimal as a string
         return unicode(value)
-    if isinstance(value, (int, long)) and (value < MININT or value > MAXINT):
+    if isinstance(value, six.integer_types) and (value < MININT or value > MAXINT):
         return unicode(value)
     if isinstance(value, DN):
         return str(value)
 
-    # Encode datetime.datetime objects as xmlrpclib.DateTime objects
+    # Encode datetime.datetime objects as xmlrpc.client.DateTime objects
     if isinstance(value, datetime.datetime):
         if capabilities.client_has_capability(version, 'datetime_values'):
             return DateTime(value)
@@ -182,7 +194,7 @@ def xml_wrap(value, version):
         else:
             return unicode(value)
 
-    assert type(value) in (unicode, int, long, float, bool, NoneType)
+    assert type(value) in (unicode, float, bool, type(None)) + six.integer_types
     return value
 
 
@@ -193,7 +205,7 @@ def xml_unwrap(value, encoding='UTF-8'):
     When decoding data from an XML-RPC packet, the following transformations
     occur:
 
-        * The binary payloads of all ``xmlrpclib.Binary`` instances are
+        * The binary payloads of all ``xmlrpc.client.Binary`` instances are
           returned as ``str`` instances.
 
         * All ``str`` instances are treated as UTF-8 encoded Unicode strings.
@@ -211,17 +223,17 @@ def xml_unwrap(value, encoding='UTF-8'):
             return DNSName(value['__dns_name__'])
         else:
             return dict(
-                (k, xml_unwrap(v, encoding)) for (k, v) in value.iteritems()
+                (k, xml_unwrap(v, encoding)) for (k, v) in value.items()
             )
-    if type(value) is str:
+    if isinstance(value, bytes):
         return value.decode(encoding)
     if isinstance(value, Binary):
-        assert type(value.data) is str
+        assert type(value.data) is bytes
         return value.data
     if isinstance(value, DateTime):
         # xmlprc DateTime is converted to string of %Y%m%dT%H:%M:%S format
         return datetime.datetime.strptime(str(value), "%Y%m%dT%H:%M:%S")
-    assert type(value) in (unicode, int, float, bool, NoneType)
+    assert type(value) in (unicode, int, float, bool, type(None))
     return value
 
 
@@ -231,16 +243,16 @@ def xml_dumps(params, version, methodname=None, methodresponse=False,
     Encode an XML-RPC data packet, transparently wraping ``params``.
 
     This function will wrap ``params`` using `xml_wrap()` and will
-    then encode the XML-RPC data packet using ``xmlrpclib.dumps()`` (from the
+    then encode the XML-RPC data packet using ``xmlrpc.client.dumps()`` (from the
     Python standard library).
 
-    For documentation on the ``xmlrpclib.dumps()`` function, see:
+    For documentation on the ``xmlrpc.client.dumps()`` function, see:
 
-        http://docs.python.org/library/xmlrpclib.html#convenience-functions
+        http://docs.python.org/library/xmlrpc.client.html#convenience-functions
 
     Also see `xml_loads()`.
 
-    :param params: A ``tuple`` or an ``xmlrpclib.Fault`` instance.
+    :param params: A ``tuple`` or an ``xmlrpc.client.Fault`` instance.
     :param methodname: The name of the method to call if this is a request.
     :param methodresponse: Set this to ``True`` if this is a response.
     :param encoding: The Unicode encoding to use (defaults to ``'UTF-8'``).
@@ -285,7 +297,7 @@ def json_encode_binary(val, version):
     elif isinstance(val, (list, tuple)):
         new_list = [json_encode_binary(v, version) for v in val]
         return new_list
-    elif isinstance(val, str):
+    elif isinstance(val, bytes):
         return {'__base64__': base64.b64encode(val)}
     elif isinstance(val, Decimal):
         return {'__base64__': base64.b64encode(str(val))}
@@ -338,7 +350,7 @@ def json_decode_binary(val):
     elif isinstance(val, list):
         return tuple(json_decode_binary(v) for v in val)
     else:
-        if isinstance(val, basestring):
+        if isinstance(val, bytes):
             try:
                 return val.decode('utf-8')
             except UnicodeDecodeError:
@@ -352,7 +364,7 @@ def json_decode_binary(val):
 
 def decode_fault(e, encoding='UTF-8'):
     assert isinstance(e, Fault)
-    if type(e.faultString) is str:
+    if isinstance(e.faultString, bytes):
         return Fault(e.faultCode, e.faultString.decode(encoding))
     return e
 
@@ -362,9 +374,9 @@ def xml_loads(data, encoding='UTF-8'):
     Decode the XML-RPC packet in ``data``, transparently unwrapping its params.
 
     This function will decode the XML-RPC packet in ``data`` using
-    ``xmlrpclib.loads()`` (from the Python standard library).  If ``data``
-    contains a fault, ``xmlrpclib.loads()`` will itself raise an
-    ``xmlrpclib.Fault`` exception.
+    ``xmlrpc.client.loads()`` (from the Python standard library).  If ``data``
+    contains a fault, ``xmlrpc.client.loads()`` will itself raise an
+    ``xmlrpc.client.Fault`` exception.
 
     Assuming an exception is not raised, this function will then unwrap the
     params in ``data`` using `xml_unwrap()`.  Finally, a
@@ -372,9 +384,9 @@ def xml_loads(data, encoding='UTF-8'):
     and the name of the method being called.  If the packet contains no method
     name, ``methodname`` will be ``None``.
 
-    For documentation on the ``xmlrpclib.loads()`` function, see:
+    For documentation on the ``xmlrpc.client.loads()`` function, see:
 
-        http://docs.python.org/library/xmlrpclib.html#convenience-functions
+        http://docs.python.org/2/library/xmlrpclib.html#convenience-functions
 
     Also see `xml_dumps()`.
 
@@ -383,13 +395,13 @@ def xml_loads(data, encoding='UTF-8'):
     try:
         (params, method) = loads(data)
         return (xml_unwrap(params), method)
-    except Fault, e:
+    except Fault as e:
         raise decode_fault(e)
 
 
 class DummyParser(object):
     def __init__(self):
-        self.data = ''
+        self.data = b''
 
     def feed(self, data):
         self.data += data
@@ -451,14 +463,10 @@ class LanguageAwareTransport(MultiProtocolTransport):
 class SSLTransport(LanguageAwareTransport):
     """Handles an HTTPS transaction to an XML-RPC server."""
 
-    def __nss_initialized(self, dbdir):
+    def get_connection_dbdir(self):
         """
-        If there is another connections open it may have already
-        initialized NSS. This is likely to lead to an NSS shutdown
-        failure.  One way to mitigate this is to tell NSS to not
-        initialize if it has already been done in another open connection.
-
-        Returns True if another connection is using the same db.
+        If there is a connections open it may have already initialized
+        NSS database. Return the database location used by the connection.
         """
         for value in context.__dict__.values():
             if not isinstance(value, Connection):
@@ -467,63 +475,69 @@ class SSLTransport(LanguageAwareTransport):
                     getattr(value.conn, '_ServerProxy__transport', None),
                     SSLTransport):
                 continue
-            if hasattr(value.conn._ServerProxy__transport, 'dbdir') and \
-              value.conn._ServerProxy__transport.dbdir == dbdir:
-                return True
-        return False
+            if hasattr(value.conn._ServerProxy__transport, 'dbdir'):
+                return value.conn._ServerProxy__transport.dbdir
+        return None
 
     def make_connection(self, host):
         host, self._extra_headers, x509 = self.get_host_info(host)
-        # Python 2.7 changed the internal class used in xmlrpclib from
-        # HTTP to HTTPConnection. We need to use the proper subclass
 
-        # If we an existing connection exists using the same NSS database
-        # there is no need to re-initialize. Pass thsi into the NSS
-        # connection creator.
-        if sys.version_info >= (2, 7):
-            if self._connection and host == self._connection[0]:
-                return self._connection[1]
+        if self._connection and host == self._connection[0]:
+            return self._connection[1]
 
         dbdir = getattr(context, 'nss_dir', paths.IPA_NSSDB_DIR)
-        no_init = self.__nss_initialized(dbdir)
-        if sys.version_info < (2, 7):
-            conn = NSSHTTPS(host, 443, dbdir=dbdir, no_init=no_init)
+        connection_dbdir = self.get_connection_dbdir()
+
+        if connection_dbdir:
+            # If an existing connection is already using the same NSS
+            # database there is no need to re-initialize.
+            no_init = dbdir == connection_dbdir
+
         else:
-            conn = NSSConnection(host, 443, dbdir=dbdir, no_init=no_init,
-                                 tls_version_min=api.env.tls_version_min,
-                                 tls_version_max=api.env.tls_version_max)
+            # If the NSS database is already being used there is no
+            # need to re-initialize.
+            no_init = dbdir == ipapython.nsslib.current_dbdir
+
+        conn = NSSConnection(host, 443, dbdir=dbdir, no_init=no_init,
+                             tls_version_min=api.env.tls_version_min,
+                             tls_version_max=api.env.tls_version_max)
         self.dbdir=dbdir
 
         conn.connect()
-        if sys.version_info < (2, 7):
-            return conn
-        else:
-            self._connection = host, conn
-            return self._connection[1]
+
+        self._connection = host, conn
+        return self._connection[1]
 
 
 class KerbTransport(SSLTransport):
     """
     Handles Kerberos Negotiation authentication to an XML-RPC server.
     """
-    flags = kerberos.GSS_C_MUTUAL_FLAG | kerberos.GSS_C_SEQUENCE_FLAG
+    flags = [gssapi.RequirementFlag.mutual_authentication,
+             gssapi.RequirementFlag.out_of_sequence_detection]
+
+    def __init__(self, *args, **kwargs):
+        SSLTransport.__init__(self, *args, **kwargs)
+        self._sec_context = None
 
     def _handle_exception(self, e, service=None):
-        (major, minor) = ipautil.get_gsserror(e)
-        if minor[1] == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN:
+        minor = e.min_code
+        if minor == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN:
             raise errors.ServiceError(service=service)
-        elif minor[1] == KRB5_FCC_NOFILE:
+        elif minor == KRB5_FCC_NOFILE:
             raise errors.NoCCacheError()
-        elif minor[1] == KRB5KRB_AP_ERR_TKT_EXPIRED:
+        elif minor == KRB5KRB_AP_ERR_TKT_EXPIRED:
             raise errors.TicketExpired()
-        elif minor[1] == KRB5_FCC_PERM:
+        elif minor == KRB5_FCC_PERM:
             raise errors.BadCCachePerms()
-        elif minor[1] == KRB5_CC_FORMAT:
+        elif minor == KRB5_CC_FORMAT:
             raise errors.BadCCacheFormat()
-        elif minor[1] == KRB5_REALM_CANT_RESOLVE:
+        elif minor == KRB5_REALM_CANT_RESOLVE:
             raise errors.CannotResolveKDC()
+        elif minor == KRB5_CC_NOTFOUND:
+            raise errors.CCacheError()
         else:
-            raise errors.KerberosError(major=major, minor=minor)
+            raise errors.KerberosError(message=unicode(e))
 
     def get_host_info(self, host):
         """
@@ -544,32 +558,108 @@ class KerbTransport(SSLTransport):
         service = "HTTP@" + host.split(':')[0]
 
         try:
-            (rc, vc) = kerberos.authGSSClientInit(service, self.flags)
-        except kerberos.GSSError, e:
-            self._handle_exception(e)
-
-        try:
-            kerberos.authGSSClientStep(vc, "")
-        except kerberos.GSSError, e:
+            name = gssapi.Name(service, gssapi.NameType.hostbased_service)
+            self._sec_context = gssapi.SecurityContext(name=name, flags=self.flags)
+            response = self._sec_context.step()
+        except gssapi.exceptions.GSSError as e:
             self._handle_exception(e, service=service)
 
+        self._set_auth_header(extra_headers, response)
+
+        return (host, extra_headers, x509)
+
+    def _set_auth_header(self, extra_headers, token):
         for (h, v) in extra_headers:
             if h == 'Authorization':
                 extra_headers.remove((h, v))
                 break
 
-        extra_headers.append(
-            ('Authorization', 'negotiate %s' % kerberos.authGSSClientResponse(vc))
-        )
+        if token:
+            extra_headers.append(
+                ('Authorization', 'negotiate %s' % base64.b64encode(token).decode('ascii'))
+            )
 
-        return (host, extra_headers, x509)
+    def _auth_complete(self, response):
+        if self._sec_context:
+            header = response.getheader('www-authenticate', '')
+            token = None
+            for field in header.split(','):
+                k, _, v = field.strip().partition(' ')
+                if k.lower() == 'negotiate':
+                    try:
+                        token = base64.b64decode(v.encode('ascii'))
+                        break
+                    # b64decode raises TypeError on invalid input
+                    except (TypeError, UnicodeError):
+                        pass
+            if not token:
+                raise KerberosError(message="No valid Negotiate header in server response")
+            token = self._sec_context.step(token=token)
+            if self._sec_context.complete:
+                self._sec_context = None
+                return True
+            self._set_auth_header(self._extra_headers, token)
+            return False
+        return True
 
     def single_request(self, host, handler, request_body, verbose=0):
+        # Based on Python 2.7's xmllib.Transport.single_request
         try:
-            return SSLTransport.single_request(self, host, handler, request_body, verbose)
+            h = SSLTransport.make_connection(self, host)
+
+            if verbose:
+                h.set_debuglevel(1)
+
+            while True:
+                if six.PY2:
+                    self.send_request(h, handler, request_body)
+                    self.send_host(h, host)
+                    self.send_user_agent(h)
+                    self.send_content(h, request_body)
+                    response = h.getresponse(buffering=True)
+                else:
+                    self.__send_request(h, host, handler, request_body, verbose)
+                    response = h.getresponse()
+
+                if response.status != 200:
+                    if (response.getheader("content-length", 0)):
+                        response.read()
+
+                    if response.status == 401:
+                        if not self._auth_complete(response):
+                            continue
+
+                    raise ProtocolError(
+                        host + handler,
+                        response.status, response.reason,
+                        response.msg)
+
+                self.verbose = verbose
+                if not self._auth_complete(response):
+                    continue
+                return self.parse_response(response)
+        except gssapi.exceptions.GSSError as e:
+            self._handle_exception(e)
         finally:
             self.close()
 
+    if six.PY3:
+        def __send_request(self, connection, host, handler, request_body, debug):
+            # Based on xmlrpc.client.Transport.send_request
+            headers = self._extra_headers[:]
+            if debug:
+                connection.set_debuglevel(1)
+            if self.accept_gzip_encoding and gzip:
+                connection.putrequest("POST", handler, skip_accept_encoding=True)
+                connection.putheader("Accept-Encoding", "gzip")
+                headers.append(("Accept-Encoding", "gzip"))
+            else:
+                connection.putrequest("POST", handler)
+            headers.append(("User-Agent", self.user_agent))
+            self.send_headers(connection, headers)  # pylint: disable=E1101
+            self.send_content(connection, request_body)
+            return connection
+
     def store_session_cookie(self, cookie_header):
         '''
         Given the contents of a Set-Cookie header scan the header and
@@ -602,7 +692,7 @@ class KerbTransport(SSLTransport):
         try:
             session_cookie = Cookie.get_named_cookie_from_string(cookie_header,
                                                                  COOKIE_NAME, request_url)
-        except Exception, e:
+        except Exception as e:
             root_logger.error("unable to parse cookie header '%s': %s", cookie_header, e)
             return
 
@@ -613,7 +703,7 @@ class KerbTransport(SSLTransport):
         root_logger.debug("storing cookie '%s' for principal %s", cookie_string, principal)
         try:
             update_persistent_client_session_data(principal, cookie_string)
-        except Exception, e:
+        except Exception as e:
             # Not fatal, we just can't use the session cookie we were sent.
             pass
 
@@ -627,8 +717,9 @@ class DelegatedKerbTransport(KerbTransport):
     Handles Kerberos Negotiation authentication and TGT delegation to an
     XML-RPC server.
     """
-    flags = kerberos.GSS_C_DELEG_FLAG |  kerberos.GSS_C_MUTUAL_FLAG | \
-            kerberos.GSS_C_SEQUENCE_FLAG
+    flags = [gssapi.RequirementFlag.delegate_to_peer,
+             gssapi.RequirementFlag.mutual_authentication,
+             gssapi.RequirementFlag.out_of_sequence_detection]
 
 
 class RPCClient(Connectible):
@@ -650,13 +741,13 @@ class RPCClient(Connectible):
         """
         # the configured URL defines what we use for the discovered servers
         (scheme, netloc, path, params, query, fragment
-            ) = urlparse.urlparse(rpc_uri)
+            ) = urllib.parse.urlparse(rpc_uri)
         servers = []
         name = '_ldap._tcp.%s.' % self.env.domain
 
         try:
             answers = resolver.query(name, rdatatype.SRV)
-        except DNSException, e:
+        except DNSException as e:
             answers = []
 
         for answer in answers:
@@ -689,13 +780,13 @@ class RPCClient(Connectible):
         # (possibly with more than one cookie).
         try:
             cookie_string = read_persistent_client_session_data(principal)
-        except Exception, e:
+        except Exception as e:
             return None
 
         # Search for the session cookie within the cookie string
         try:
             session_cookie = Cookie.get_named_cookie_from_string(cookie_string, COOKIE_NAME)
-        except Exception, e:
+        except Exception as e:
             return None
 
         return session_cookie
@@ -738,17 +829,17 @@ class RPCClient(Connectible):
         # Decide if we should send the cookie to the server
         try:
             session_cookie.http_return_ok(original_url)
-        except Cookie.Expired, e:
+        except Cookie.Expired as e:
             self.debug("deleting session data for principal '%s': %s", principal, e)
             try:
                 delete_persistent_client_session_data(principal)
-            except Exception, e:
+            except Exception as e:
                 pass
             return original_url
-        except Cookie.URLMismatch, e:
+        except Cookie.URLMismatch as e:
             self.debug("not sending session cookie, URL mismatch: %s", e)
             return original_url
-        except Exception, e:
+        except Exception as e:
             self.error("not sending session cookie, unknown error: %s", e)
             return original_url
 
@@ -758,9 +849,11 @@ class RPCClient(Connectible):
         setattr(context, 'session_cookie', session_cookie.http_cookie())
 
         # Form the session URL by substituting the session path into the original URL
-        scheme, netloc, path, params, query, fragment = urlparse.urlparse(original_url)
+        scheme, netloc, path, params, query, fragment = urllib.parse.urlparse(original_url)
         path = self.session_path
-        session_url = urlparse.urlunparse((scheme, netloc, path, params, query, fragment))
+        # urlencode *can* take one argument
+        # pylint: disable=too-many-function-args
+        session_url = urllib.parse.urlunparse((scheme, netloc, path, params, query, fragment))
 
         return session_url
 
@@ -768,13 +861,13 @@ class RPCClient(Connectible):
                           delegate=False, nss_dir=None):
         try:
             rpc_uri = self.env[self.env_rpc_uri_key]
-            principal = get_current_principal()
+            principal = get_principal()
             setattr(context, 'principal', principal)
             # We have a session cookie, try using the session URI to see if it
             # is still valid
             if not delegate:
                 rpc_uri = self.apply_session_cookie(rpc_uri)
-        except ValueError:
+        except (errors.CCacheError, ValueError):
             # No session key, do full Kerberos auth
             pass
         # This might be dangerous. Use at your own risk!
@@ -805,7 +898,7 @@ class RPCClient(Connectible):
                 command = getattr(serverproxy, 'ping')
                 try:
                     response = command([], {})
-                except Fault, e:
+                except Fault as e:
                     e = decode_fault(e)
                     if e.faultCode in errors_by_code:
                         error = errors_by_code[e.faultCode]
@@ -818,23 +911,23 @@ class RPCClient(Connectible):
                         )
                 # We don't care about the response, just that we got one
                 break
-            except KerberosError, krberr:
+            except KerberosError as krberr:
                 # kerberos error on one server is likely on all
-                raise errors.KerberosError(major=str(krberr), minor='')
-            except ProtocolError, e:
+                raise errors.KerberosError(message=unicode(krberr))
+            except ProtocolError as e:
                 if hasattr(context, 'session_cookie') and e.errcode == 401:
                     # Unauthorized. Remove the session and try again.
                     delattr(context, 'session_cookie')
                     try:
                         delete_persistent_client_session_data(principal)
-                    except Exception, e:
+                    except Exception as e:
                         # This shouldn't happen if we have a session but it isn't fatal.
                         pass
                     return self.create_connection(ccache, verbose, fallback, delegate)
                 if not fallback:
                     raise
                 serverproxy = None
-            except Exception, e:
+            except Exception as e:
                 if not fallback:
                     raise
                 else:
@@ -847,11 +940,10 @@ class RPCClient(Connectible):
         return serverproxy
 
     def destroy_connection(self):
-        if sys.version_info >= (2, 7):
-            conn = getattr(context, self.id, None)
-            if conn is not None:
-                conn = conn.conn._ServerProxy__transport
-                conn.close()
+        conn = getattr(context, self.id, None)
+        if conn is not None:
+            conn = conn.conn._ServerProxy__transport
+            conn.close()
 
     def _call_command(self, command, params):
         """Call the command with given params"""
@@ -881,7 +973,7 @@ class RPCClient(Connectible):
         params = [args, kw]
         try:
             return self._call_command(command, params)
-        except Fault, e:
+        except Fault as e:
             e = decode_fault(e)
             self.debug('Caught fault %d from server %s: %s', e.faultCode,
                 server, e.faultString)
@@ -893,9 +985,9 @@ class RPCClient(Connectible):
                 error=e.faultString,
                 server=server,
             )
-        except NSPRError, e:
+        except NSPRError as e:
             raise NetworkError(uri=server, error=str(e))
-        except ProtocolError, e:
+        except ProtocolError as e:
             # By catching a 401 here we can detect the case where we have
             # a single IPA server and the session is invalid. Otherwise
             # we always have to do a ping().
@@ -906,7 +998,7 @@ class RPCClient(Connectible):
                 try:
                     principal = getattr(context, 'principal', None)
                     delete_persistent_client_session_data(principal)
-                except Exception, e:
+                except Exception as e:
                     # This shouldn't happen if we have a session but it isn't fatal.
                     pass
 
@@ -928,9 +1020,9 @@ class RPCClient(Connectible):
                     current_conn.conn._ServerProxy__transport.dbdir = dbdir
                 return self.forward(name, *args, **kw)
             raise NetworkError(uri=server, error=e.errmsg)
-        except socket.error, e:
+        except socket.error as e:
             raise NetworkError(uri=server, error=str(e))
-        except (OverflowError, TypeError), e:
+        except (OverflowError, TypeError) as e:
             raise XMLRPCMarshallError(error=str(e))
 
 
@@ -949,10 +1041,11 @@ class xmlclient(RPCClient):
 
 class JSONServerProxy(object):
     def __init__(self, uri, transport, encoding, verbose, allow_none):
-        type, uri = urllib.splittype(uri)
-        if type not in ("http", "https"):
+        split_uri = urllib.parse.urlsplit(uri)
+        if split_uri.scheme not in ("http", "https"):
             raise IOError("unsupported XML-RPC protocol")
-        self.__host, self.__handler = urllib.splithost(uri)
+        self.__host = split_uri.netloc
+        self.__handler = split_uri.path
         self.__transport = transport
 
         assert encoding == 'UTF-8'
@@ -960,7 +1053,7 @@ class JSONServerProxy(object):
         self.__verbose = verbose
 
         # FIXME: Some of our code requires ServerProxy internals.
-        # But, xmlrpclib.ServerProxy's _ServerProxy__transport can be accessed
+        # But, xmlrpc.client.ServerProxy's _ServerProxy__transport can be accessed
         # by calling serverproxy('transport')
         self._ServerProxy__transport = transport
 
@@ -976,13 +1069,13 @@ class JSONServerProxy(object):
         response = self.__transport.request(
             self.__host,
             self.__handler,
-            json.dumps(payload),
+            json.dumps(payload).encode('utf-8'),
             verbose=self.__verbose >= 3,
         )
 
         try:
-            response = json_decode_binary(json.loads(response))
-        except ValueError, e:
+            response = json_decode_binary(json.loads(response.decode('ascii')))
+        except ValueError as e:
             raise JSONError(str(e))
 
         if self.__verbose >= 2:
