@@ -18,15 +18,17 @@
 
 import memcache
 import random
-import errors
 import os
 import re
 import time
-from urllib2 import urlparse
-from text import _
+
+from six.moves.urllib.parse import urlparse
+
+from ipalib import errors
+from ipalib.text import _
 from ipapython.ipa_log_manager import *
 from ipalib import api, errors
-from ipalib import Command
+from ipaplatform.paths import paths
 from ipalib.krb_utils import *
 from ipapython.cookie import Cookie
 
@@ -483,7 +485,7 @@ improve authentication performance. First some definitions.
 There are 4 major players:
 
   1. client
-  2. mod_auth_kerb (in Apache process)
+  2. mod_auth_gssapi (in Apache process)
   3. wsgi handler (in IPA wsgi python process)
   4. ds (directory server)
 
@@ -505,12 +507,12 @@ This describes how things work in our current system for the web UI.
 
   2. Client sends post to /ipa/json.
 
-  3. mod_auth_kerb is configured to protect /ipa/json, replies 401
+  3. mod_auth_gssapi is configured to protect /ipa/json, replies 401
      authenticate negotiate.
 
   4. Client resends with credentials
 
-  5. mod_auth_kerb validates credentials
+  5. mod_auth_gssapi validates credentials
 
      a. if invalid replies 403 access denied (stops here)
 
@@ -549,7 +551,7 @@ A few notes about the session implementation.
 Changes to Apache's resource protection
 ---------------------------------------
 
-  * /ipa/json is no longer protected by mod_auth_kerb. This is
+  * /ipa/json is no longer protected by mod_auth_gssapi. This is
     necessary to avoid the negotiate expense in steps 3,4,5
     above. Instead the /ipa/json resource will be protected in our wsgi
     handler via the session cookie.
@@ -582,15 +584,15 @@ The new sequence is:
 
   5. client sends request to /ipa/login to obtain session credentials
 
-  6. mod_auth_kerb replies 401 negotiate on /ipa/login
+  6. mod_auth_gssapi replies 401 negotiate on /ipa/login
 
   7. client sends credentials to /ipa/login
 
-  8. mod_auth_kerb validates credentials
+  8. mod_auth_gssapi validates credentials
 
      a. if valid
 
-        - mod_auth_kerb permits access to /ipa/login. wsgi handler is
+        - mod_auth_gssapi permits access to /ipa/login. wsgi handler is
           invoked and does the following:
 
           * establishes session for client
@@ -599,7 +601,7 @@ The new sequence is:
 
      a. if invalid
 
-        - mod_auth_kerb sends 403 access denied (processing stops)
+        - mod_auth_gssapi sends 403 access denied (processing stops)
 
   9. client now posts the same data again to /ipa/json including
      session cookie. Processing repeats starting at step 2 and since
@@ -616,12 +618,12 @@ and xmlrpc API's are the same, they differ only on how their procedure
 calls are marshalled and unmarshalled.
 
 Under the new scheme /ipa/xml will continue to be Kerberos protected
-at all times. Apache's mod_auth_kerb will continue to require the
+at all times. Apache's mod_auth_gssapi will continue to require the
 client provides valid Kerberos credentials.
 
 When the WSGI handler routes to /ipa/xml the Kerberos credentials will
 be extracted from the KRB5CCNAME environment variable as provided by
-mod_auth_kerb. Everything else remains the same.
+mod_auth_gssapi. Everything else remains the same.
 
 '''
 
@@ -692,7 +694,7 @@ class SessionAuthManager(object):
     def unregister(self, name):
         self.debug('SessionAuthManager.unregister: name=%s', name)
 
-        if not self.auth_managers.has_key(name):
+        if name not in self.auth_managers:
             raise KeyError('cannot unregister auth manager named "%s", does not exist',
                            name)
         del self.auth_managers[name]
@@ -706,7 +708,7 @@ class SessionAuthManager(object):
         for auth_mgr in self.auth_managers.values():
             try:
                 auth_mgr.logout(session_data)
-            except Exception, e:
+            except Exception as e:
                 self.error('%s auth_mgr logout failed: %s', auth_mgr.name, e)
 
 #-------------------------------------------------------------------------------
@@ -795,7 +797,7 @@ class MemcacheSessionManager(SessionManager):
     session data (see `load_session_data()`)
     '''
 
-    memcached_socket_path = '/var/run/ipa_memcached/ipa_memcached'
+    memcached_socket_path = paths.VAR_RUN_IPA_MEMCACHED
     session_cookie_name = 'ipa_session'
     mc_server_stat_name_re = re.compile(r'(.+)\s+\((\d+)\)')
 
@@ -963,7 +965,7 @@ class MemcacheSessionManager(SessionManager):
 
         try:
             session_cookie = Cookie.get_named_cookie_from_string(cookie_header, self.session_cookie_name)
-        except Exception, e:
+        except Exception as e:
             session_cookie = None
         if session_cookie:
             session_id = session_cookie.value
@@ -1084,7 +1086,7 @@ class MemcacheSessionManager(SessionManager):
             expiration = None;
 
         cookie = Cookie(self.session_cookie_name, session_id,
-                        domain=urlparse.urlparse(api.env.xmlrpc_uri).netloc,
+                        domain=urlparse(api.env.xmlrpc_uri).netloc,
                         path=url_path, httponly=True, secure=True,
                         expires=expiration)
         if add_header:
@@ -1208,7 +1210,7 @@ class MemcacheSessionManager(SessionManager):
 
 
 #-------------------------------------------------------------------------------
-krbccache_dir ='/var/run/ipa_memcached'
+krbccache_dir =paths.IPA_MEMCACHED_DIR
 krbccache_prefix = 'krbcc_'
 
 def _get_krbccache_pathname():
@@ -1259,7 +1261,7 @@ def release_ipa_ccache(ccache_name):
     do we'll remove them.
     '''
 
-    if os.environ.has_key('KRB5CCNAME'):
+    if 'KRB5CCNAME' in os.environ:
         if ccache_name != os.environ['KRB5CCNAME']:
             root_logger.error('release_ipa_ccache: ccache_name (%s) != KRB5CCNAME environment variable (%s)',
                               ccache_name, os.environ['KRB5CCNAME'])
@@ -1272,37 +1274,9 @@ def release_ipa_ccache(ccache_name):
         if os.path.exists(name):
             try:
                 os.unlink(name)
-            except Exception, e:
+            except Exception as e:
                 root_logger.error('unable to delete session ccache file "%s", %s', name, e)
     else:
         raise ValueError('ccache scheme "%s" unsupported (%s)', scheme, ccache_name)
 
-
-#-------------------------------------------------------------------------------
-
-from ipalib.request import context
-
-class session_logout(Command):
-    '''
-    RPC command used to log the current user out of their session.
-    '''
-
-    def execute(self, *args, **options):
-        session_data = getattr(context, 'session_data', None)
-        if session_data is None:
-            self.debug('session logout command: no session_data found')
-        else:
-            session_id = session_data.get('session_id')
-            self.debug('session logout command: session_id=%s', session_id)
-
-            # Notifiy registered listeners
-            session_mgr.auth_mgr.logout(session_data)
-
-        return dict(result=None)
-
-api.register(session_logout)
-
-#-------------------------------------------------------------------------------
-
-
 session_mgr = MemcacheSessionManager()
