@@ -115,7 +115,13 @@ forms, see `NGettextFactory` and `NGettext`.
 import threading
 import locale
 import gettext
-from request import context
+
+import six
+
+from ipalib.request import context
+
+if six.PY3:
+    unicode = str
 
 
 def create_translation(key):
@@ -136,6 +142,9 @@ class LazyText(object):
 
     This class is not used directly.  See the `Gettext` and `NGettext`
     subclasses.
+
+    Concatenating LazyText objects with the + operator gives
+    ConcatenatedLazyText objects.
     """
 
     __slots__ = ('domain', 'localedir', 'key', 'args')
@@ -176,7 +185,14 @@ class LazyText(object):
         """
         return not self.__eq__(other)
 
+    def __add__(self, other):
+        return ConcatenatedLazyText(self) + other
 
+    def __radd__(self, other):
+        return other + ConcatenatedLazyText(self)
+
+
+@six.python_2_unicode_compatible
 class Gettext(LazyText):
     """
     Deferred translation using ``gettext.ugettext()``.
@@ -231,23 +247,30 @@ class Gettext(LazyText):
         return '%s(%r, domain=%r, localedir=%r)' % (self.__class__.__name__,
             self.msg, self.domain, self.localedir)
 
-    def __unicode__(self):
+    def as_unicode(self):
         """
         Translate this message and return as a ``unicode`` instance.
         """
         if self.key in context.__dict__:
-            g = context.__dict__[self.key].ugettext
+            t = context.__dict__[self.key]
         else:
-            g = create_translation(self.key).ugettext
-        return g(self.msg)
+            t = create_translation(self.key)
+        if six.PY2:
+            return t.ugettext(self.msg)
+        else:
+            return t.gettext(self.msg)
+
+    def __str__(self):
+        return unicode(self.as_unicode())
 
     def __json__(self):
-        return self.__unicode__()
+        return unicode(self)   #pylint: disable=no-member
 
     def __mod__(self, kw):
-        return self.__unicode__() % kw
+        return unicode(self) % kw  #pylint: disable=no-member
 
 
+@six.python_2_unicode_compatible
 class FixMe(Gettext):
     """
     Non-translated place-holder for UI labels.
@@ -293,7 +316,7 @@ class FixMe(Gettext):
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.msg)
 
-    def __unicode__(self):
+    def __str__(self):
         return u'<%s>' % self.msg
 
 
@@ -384,10 +407,51 @@ class NGettext(LazyText):
 
     def __call__(self, count):
         if self.key in context.__dict__:
-            ng = context.__dict__[self.key].ungettext
+            t = context.__dict__[self.key]
         else:
-            ng = create_translation(self.key).ungettext
-        return ng(self.singular, self.plural, count)
+            t = create_translation(self.key)
+        if six.PY2:
+            return t.ungettext(self.singular, self.plural, count)
+        else:
+            return t.ngettext(self.singular, self.plural, count)
+
+
+@six.python_2_unicode_compatible
+class ConcatenatedLazyText(object):
+    """Concatenation of multiple strings, or any objects convertible to unicode
+
+    Used to concatenate several LazyTexts together.
+    This allows large strings like help text to be split, so translators
+    do not have to re-translate the whole text when only a small part changes.
+
+    Additional strings may be added to the end with the + or += operators.
+    """
+    def __init__(self, *components):
+        self.components = list(components)
+
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, self.components)
+
+    def __str__(self):
+        return u''.join(unicode(c) for c in self.components)
+
+    def __json__(self):
+        return unicode(self)
+
+    def __mod__(self, kw):
+        return unicode(self) % kw
+
+    def __add__(self, other):
+        if isinstance(other, ConcatenatedLazyText):
+            return ConcatenatedLazyText(*self.components + other.components)
+        else:
+            return ConcatenatedLazyText(*self.components + [other])
+
+    def __radd__(self, other):
+        if isinstance(other, ConcatenatedLazyText):
+            return ConcatenatedLazyText(*other.components + self.components)
+        else:
+            return ConcatenatedLazyText(*[other] + self.components)
 
 
 class GettextFactory(object):
