@@ -31,14 +31,19 @@
 # nsscert: the certificate is an NSS Certificate object
 # rawcert: the cert is in an unknown format
 
+from __future__ import print_function
+
 import os
 import sys
 import base64
 import re
+
 import nss.nss as nss
 from nss.error import NSPRError
 from pyasn1.type import univ, namedtype, tag
 from pyasn1.codec.der import decoder, encoder
+import six
+
 from ipapython import ipautil
 from ipalib import api
 from ipalib import _
@@ -125,7 +130,11 @@ def load_certificate(data, datatype=PEM, dbdir=None):
 
     initialize_nss_database(dbdir=dbdir)
 
-    return nss.Certificate(buffer(data))
+    if six.PY2:
+        return nss.Certificate(buffer(data))  # pylint: disable=buffer-builtin
+    else:
+        # In python 3 , `bytes` has the buffer interface
+        return nss.Certificate(data)
 
 def load_certificate_from_file(filename, dbdir=None):
     """
@@ -264,7 +273,9 @@ def make_pem(data):
     Convert a raw base64-encoded blob into something that looks like a PE
     file with lines split to 64 characters and proper headers.
     """
-    pemcert = '\n'.join([data[x:x+64] for x in range(0, len(data), 64)])
+    if isinstance(data, bytes):
+        data = data.decode('ascii')
+    pemcert = '\r\n'.join([data[x:x+64] for x in range(0, len(data), 64)])
     return '-----BEGIN CERTIFICATE-----\n' + \
     pemcert + \
     '\n-----END CERTIFICATE-----'
@@ -285,7 +296,7 @@ def normalize_certificate(rawcert):
     if util.isvalid_base64(rawcert):
         try:
             dercert = base64.b64decode(rawcert)
-        except Exception, e:
+        except Exception as e:
             raise errors.Base64DecodeError(reason=str(e))
     else:
         dercert = rawcert
@@ -294,16 +305,24 @@ def normalize_certificate(rawcert):
     # was base64-encoded and now its not or it came in as DER format.
     # Let's decode it and see. Fetching the serial number will pass the
     # certificate through the NSS DER parser.
+    validate_certificate(dercert, datatype=DER)
+
+    return dercert
+
+
+def validate_certificate(cert, datatype=PEM, dbdir=None):
+    """
+    Perform certificate validation by trying to load it into NSS database
+    """
     try:
-        serial = unicode(get_serial_number(dercert, DER))
-    except NSPRError, nsprerr:
+        load_certificate(cert, datatype=datatype, dbdir=dbdir)
+    except NSPRError as nsprerr:
         if nsprerr.errno == -8183: # SEC_ERROR_BAD_DER
             raise errors.CertificateFormatError(
                 error=_('improperly formatted DER-encoded certificate'))
         else:
             raise errors.CertificateFormatError(error=str(nsprerr))
 
-    return dercert
 
 def write_certificate(rawcert, filename):
     """
@@ -318,7 +337,7 @@ def write_certificate(rawcert, filename):
         fp = open(filename, 'w')
         fp.write(make_pem(base64.b64encode(dercert)))
         fp.close()
-    except (IOError, OSError), e:
+    except (IOError, OSError) as e:
         raise errors.FileError(reason=str(e))
 
 def write_certificate_list(rawcerts, filename):
@@ -336,7 +355,7 @@ def write_certificate_list(rawcerts, filename):
                 cert = base64.b64encode(cert)
                 cert = make_pem(cert)
                 f.write(cert + '\n')
-    except (IOError, OSError), e:
+    except (IOError, OSError) as e:
         raise errors.FileError(reason=str(e))
 
 def verify_cert_subject(ldap, hostname, dercert):
@@ -399,4 +418,4 @@ if __name__ == '__main__':
 
     nsscert = load_certificate(cert)
 
-    print nsscert
+    print(nsscert)
