@@ -24,6 +24,9 @@ This module contains default platform-specific implementations of system tasks.
 
 import pwd
 import grp
+
+from pkg_resources import parse_version
+
 from ipaplatform.paths import paths
 from ipapython.ipa_log_manager import log_mgr
 from ipapython import ipautil
@@ -147,6 +150,21 @@ class BaseTaskNamespace(object):
 
         return
 
+    def backup_auth_configuration(self, path):
+        """
+        Create backup of access control configuration.
+        :param path: store the backup here. This will be passed to
+        restore_auth_configuration as well.
+        """
+        return
+
+    def restore_auth_configuration(self, path):
+        """
+        Restore backup of access control configuration.
+        :param path: restore the backup from here.
+        """
+        return
+
     def set_selinux_booleans(self, required_settings, backup_func=None):
         """Set the specified SELinux booleans
 
@@ -166,7 +184,7 @@ class BaseTaskNamespace(object):
 
         return
 
-    def create_system_user(self, name, group, homedir, shell, uid = None, gid = None, comment = None):
+    def create_system_user(self, name, group, homedir, shell, uid=None, gid=None, comment=None, create_homedir=False):
         """Create a system user with a corresponding group"""
         try:
             grp.getgrnam(group)
@@ -193,12 +211,16 @@ class BaseTaskNamespace(object):
                 '-g', group,
                 '-d', homedir,
                 '-s', shell,
-                '-M', '-r', name,
+                '-r', name,
             ]
             if uid:
                 args += ['-u', str(uid)]
             if comment:
                 args += ['-c', comment]
+            if create_homedir:
+                args += ['-m']
+            else:
+                args += ['-M']
             try:
                 ipautil.run(args)
                 log.debug('Done adding user')
@@ -208,5 +230,9 @@ class BaseTaskNamespace(object):
         else:
             log.debug('user %s exists', name)
 
-
-task_namespace = BaseTaskNamespace()
+    def parse_ipa_version(self, version):
+        """
+        :param version: textual version
+        :return: object implementing proper __cmp__ method for version compare
+        """
+        return parse_version(version)
