@@ -23,17 +23,21 @@
 This module contains default Red Hat OS family-specific implementations of
 system tasks.
 '''
+from __future__ import print_function
 
 import os
 import stat
 import socket
 import sys
-import urllib
 import base64
+from cffi import FFI
+from ctypes.util import find_library
+from functools import total_ordering
 
 from subprocess import CalledProcessError
 from nss.error import NSPRError
 from pyasn1.error import PyAsn1Error
+from six.moves import urllib
 
 from ipapython.ipa_log_manager import root_logger, log_mgr
 from ipapython import ipautil
@@ -41,14 +45,54 @@ import ipapython.errors
 
 from ipalib import x509 # FIXME: do not import from ipalib
 
+from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipaplatform.redhat.authconfig import RedHatAuthConfig
 from ipaplatform.base.tasks import BaseTaskNamespace
 
+_ffi = FFI()
+_ffi.cdef("""
+int rpmvercmp (const char *a, const char *b);
+""")
+
+# use ctypes loader to get correct librpm.so library version according to
+# https://cffi.readthedocs.org/en/latest/overview.html#id8
+_librpm = _ffi.dlopen(find_library("rpm"))
 
 log = log_mgr.get_logger(__name__)
 
 
+def selinux_enabled():
+    """
+    Check if SELinux is enabled.
+    """
+    if os.path.exists(paths.SELINUXENABLED):
+        try:
+            ipautil.run([paths.SELINUXENABLED])
+            return True
+        except ipautil.CalledProcessError:
+            # selinuxenabled returns 1 if not enabled
+            return False
+    else:
+        # No selinuxenabled, no SELinux
+        return False
+
+
+@total_ordering
+class IPAVersion(object):
+
+    def __init__(self, version):
+        self.version = version
+
+    def __eq__(self, other):
+        assert isinstance(other, IPAVersion)
+        return _librpm.rpmvercmp(self.version, other.version) == 0
+
+    def __lt__(self, other):
+        assert isinstance(other, IPAVersion)
+        return _librpm.rpmvercmp(self.version, other.version) < 0
+
+
 class RedHatTaskNamespace(BaseTaskNamespace):
 
     def restore_context(self, filepath, restorecon=paths.SBIN_RESTORECON):
@@ -60,14 +104,8 @@ class RedHatTaskNamespace(BaseTaskNamespace):
 
         ipautil.run() will do the logging.
         """
-        try:
-            if os.path.exists(paths.SELINUXENABLED):
-                ipautil.run([paths.SELINUXENABLED])
-            else:
-                # No selinuxenabled, no SELinux
-                return
-        except ipautil.CalledProcessError:
-            # selinuxenabled returns 1 if not enabled
+
+        if not selinux_enabled():
             return
 
         if (os.path.exists(restorecon)):
@@ -83,14 +121,7 @@ class RedHatTaskNamespace(BaseTaskNamespace):
         This function returns nothing but may raise a Runtime exception
         if SELinux is enabled but restorecon is not available.
         """
-        try:
-            if os.path.exists(paths.SELINUXENABLED):
-                ipautil.run([paths.SELINUXENABLED])
-            else:
-                # No selinuxenabled, no SELinux
-                return
-        except ipautil.CalledProcessError:
-            # selinuxenabled returns 1 if not enabled
+        if not selinux_enabled():
             return
 
         if not os.path.exists(restorecon):
@@ -158,10 +189,18 @@ class RedHatTaskNamespace(BaseTaskNamespace):
         auth_config.add_option("nostart")
         auth_config.execute()
 
+    def backup_auth_configuration(self, path):
+        auth_config = RedHatAuthConfig()
+        auth_config.backup(path)
+
+    def restore_auth_configuration(self, path):
+        auth_config = RedHatAuthConfig()
+        auth_config.restore(path)
+
     def reload_systemwide_ca_store(self):
         try:
             ipautil.run([paths.UPDATE_CA_TRUST])
-        except CalledProcessError, e:
+        except CalledProcessError as e:
             root_logger.error(
                 "Could not update systemwide CA trust database: %s", e)
             return False
@@ -175,7 +214,7 @@ class RedHatTaskNamespace(BaseTaskNamespace):
         if os.path.exists(new_cacert_path):
             try:
                 os.remove(new_cacert_path)
-            except OSError, e:
+            except OSError as e:
                 root_logger.error(
                     "Could not remove %s: %s", new_cacert_path, e)
                 return False
@@ -184,7 +223,7 @@ class RedHatTaskNamespace(BaseTaskNamespace):
 
         try:
             f = open(new_cacert_path, 'w')
-        except IOError, e:
+        except IOError as e:
             root_logger.info("Failed to open %s: %s" % (new_cacert_path, e))
             return False
 
@@ -198,16 +237,16 @@ class RedHatTaskNamespace(BaseTaskNamespace):
                 issuer = x509.get_der_issuer(cert, x509.DER)
                 serial_number = x509.get_der_serial_number(cert, x509.DER)
                 public_key_info = x509.get_der_public_key_info(cert, x509.DER)
-            except (NSPRError, PyAsn1Error), e:
+            except (NSPRError, PyAsn1Error, ValueError) as e:
                 root_logger.warning(
                     "Failed to decode certificate \"%s\": %s", nickname, e)
                 continue
 
-            label = urllib.quote(nickname)
-            subject = urllib.quote(subject)
-            issuer = urllib.quote(issuer)
-            serial_number = urllib.quote(serial_number)
-            public_key_info = urllib.quote(public_key_info)
+            label = urllib.parse.quote(nickname)
+            subject = urllib.parse.quote(subject)
+            issuer = urllib.parse.quote(issuer)
+            serial_number = urllib.parse.quote(serial_number)
+            public_key_info = urllib.parse.quote(public_key_info)
 
             cert = base64.b64encode(cert)
             cert = x509.make_pem(cert)
@@ -238,12 +277,12 @@ class RedHatTaskNamespace(BaseTaskNamespace):
                     ext_key_usage = {x509.EKU_PLACEHOLDER}
                 try:
                     ext_key_usage = x509.encode_ext_key_usage(ext_key_usage)
-                except PyAsn1Error, e:
+                except PyAsn1Error as e:
                     root_logger.warning(
                         "Failed to encode extended key usage for \"%s\": %s",
                         nickname, e)
                     continue
-                value = urllib.quote(ext_key_usage)
+                value = urllib.parse.quote(ext_key_usage)
                 obj = ("[p11-kit-object-v1]\n"
                        "class: x-certificate-extension\n"
                        "label: \"ExtendedKeyUsage for %(label)s\"\n"
@@ -275,7 +314,7 @@ class RedHatTaskNamespace(BaseTaskNamespace):
                 continue
             try:
                 os.remove(new_cacert_path)
-            except OSError, e:
+            except OSError as e:
                 root_logger.error(
                     "Could not remove %s: %s", new_cacert_path, e)
                 result = False
@@ -292,9 +331,9 @@ class RedHatTaskNamespace(BaseTaskNamespace):
         old_hostname = socket.gethostname()
         try:
             ipautil.run([paths.BIN_HOSTNAME, hostname])
-        except ipautil.CalledProcessError, e:
-            print >>sys.stderr, ("Failed to set this machine hostname to "
-                                 "%s (%s)." % (hostname, str(e)))
+        except ipautil.CalledProcessError as e:
+            print(("Failed to set this machine hostname to "
+                                 "%s (%s)." % (hostname, str(e))), file=sys.stderr)
 
         filepath = paths.ETC_HOSTNAME
         if os.path.exists(filepath):
@@ -330,8 +369,8 @@ class RedHatTaskNamespace(BaseTaskNamespace):
             # in /etc/sysconfig/network
             old_filepath_restore = paths.SYSCONFIG_NETWORK_IPABKP
             fstore.restore_file(old_filepath, old_filepath_restore)
-            print "Deprecated configuration file '%s' was restored to '%s'" \
-                    % (old_filepath, old_filepath_restore)
+            print("Deprecated configuration file '%s' was restored to '%s'" \
+                    % (old_filepath, old_filepath_restore))
             hostname_was_configured = True
 
         filepath = paths.ETC_HOSTNAME
@@ -350,33 +389,30 @@ class RedHatTaskNamespace(BaseTaskNamespace):
     def set_selinux_booleans(self, required_settings, backup_func=None):
         def get_setsebool_args(changes):
             args = [paths.SETSEBOOL, "-P"]
-            args.extend(["%s=%s" % update for update in changes.iteritems()])
+            args.extend(["%s=%s" % update for update in changes.items()])
 
             return args
 
-        if (os.path.exists(paths.SELINUXENABLED)):
-            try:
-                ipautil.run([paths.SELINUXENABLED])
-            except ipautil.CalledProcessError:
-                # selinuxenabled returns 1 if not enabled
-                return False
-        else:
+        if not selinux_enabled():
             return False
 
         updated_vars = {}
         failed_vars = {}
-        for setting, state in required_settings.iteritems():
+        for setting, state in required_settings.items():
             if state is None:
                 continue
             try:
-                (stdout, stderr, rc) = ipautil.run([paths.GETSEBOOL, setting])
-                original_state = stdout.split()[2]
+                result = ipautil.run(
+                    [paths.GETSEBOOL, setting],
+                    capture_output=True
+                )
+                original_state = result.output.split()[2]
                 if backup_func is not None:
                     backup_func(setting, original_state)
 
                 if original_state != state:
                     updated_vars[setting] = state
-            except ipautil.CalledProcessError, e:
+            except ipautil.CalledProcessError as e:
                 log.error("Cannot get SELinux boolean '%s': %s", setting, e)
                 failed_vars[setting] = state
 
@@ -394,7 +430,7 @@ class RedHatTaskNamespace(BaseTaskNamespace):
 
         return True
 
-    def create_system_user(self, name, group, homedir, shell, uid = None, gid = None, comment = None):
+    def create_system_user(self, name, group, homedir, shell, uid=None, gid=None, comment=None, create_homedir=False):
         """
         Create a system user with a corresponding group
 
@@ -403,19 +439,26 @@ class RedHatTaskNamespace(BaseTaskNamespace):
         This values should be constant and may be hardcoded.
         Add other values for other users when needed.
         """
-        if name == 'pkiuser':
+        if name == constants.PKI_USER:
             if uid is None:
                 uid = 17
             if gid is None:
                 gid = 17
             if comment is None:
                 comment = 'CA System User'
-        if name == 'dirsrv':
+        if name == constants.DS_USER:
             if comment is None:
                 comment = 'DS System User'
 
         super(RedHatTaskNamespace, self).create_system_user(name, group,
-            homedir, shell, uid, gid, comment)
+            homedir, shell, uid, gid, comment, create_homedir)
+
+    def parse_ipa_version(self, version):
+        """
+        :param version: textual version
+        :return: object implementing proper __cmp__ method for version compare
+        """
+        return IPAVersion(version)
 
 
 tasks = RedHatTaskNamespace()
