@@ -32,7 +32,7 @@ from ipapython import config
 from ipapython import ipa_log_manager
 
 
-class ScriptError(StandardError):
+class ScriptError(Exception):
     """An exception that records an error message and a return value
     """
     def __init__(self, msg='', rval=1):
@@ -169,7 +169,7 @@ class AdminTool(object):
             self.ask_for_options()
             self.setup_logging()
             return_value = self.run()
-        except BaseException, exception:
+        except BaseException as exception:
             traceback = sys.exc_info()[2]
             error_message, return_value = self.handle_error(exception)
             if return_value:
@@ -283,6 +283,7 @@ class AdminTool(object):
         """
         self.log.debug('%s was invoked with arguments %s and options: %s',
                 self.command_name, self.args, self.safe_options)
+        self.log.debug('IPA version %s' % version.VENDOR_VERSION)
 
     def log_failure(self, error_message, return_value, exception, backtrace):
         self.log.debug(''.join(traceback.format_tb(backtrace)))
@@ -290,6 +291,10 @@ class AdminTool(object):
             self.command_name, type(exception).__name__, exception)
         if error_message:
             self.log.error(error_message)
+        message = "The %s command failed." % self.command_name
+        if self.log_file_name:
+            message += " See %s for more information" % self.log_file_name
+        self.log.error(message)
 
     def log_success(self):
         self.log.info('The %s command was successful', self.command_name)
