@@ -172,7 +172,7 @@ FAQ (Frequently Asked Questions)
 
    See above. Logger's will never have a level less than the level of
    the handlers visible to the logger. If there are no handlers then
-   loggers can't output anything so their level is set to maxint.
+   loggers can't output anything so their level is set to maxsize.
 
 #. **I set the default_level but all the loggers are configured
    at INFO or DEBUG, what happened?**
@@ -500,6 +500,7 @@ bewildering difficult to get it do what I wanted.
 John Dennis <jdennis@redhat.com>
 
 '''
+from __future__ import print_function
 
 #-------------------------------------------------------------------------------
 import sys
@@ -509,6 +510,8 @@ import logging
 import re
 import time
 
+import six
+
 #-------------------------------------------------------------------------------
 # Default format
 LOGGING_DEFAULT_FORMAT = '%(levelname)s %(message)s'
@@ -544,7 +547,7 @@ def get_unique_levels(iterable):
     levels = set()
 
     for obj in iterable:
-        level = getattr(obj, 'level', sys.maxint)
+        level = getattr(obj, 'level', sys.maxsize)
         if level != logging.NOTSET:
             levels.add(level)
     levels = list(levels)
@@ -554,7 +557,7 @@ def get_unique_levels(iterable):
 def get_minimum_level(iterable):
     '''
     Given a iterable of objects containing a logging level return the
-    minimum level. If no levels are defined return maxint.
+    minimum level. If no levels are defined return maxsize.
     set of unique levels.
 
     :parameters:
@@ -563,10 +566,10 @@ def get_minimum_level(iterable):
     :returns:
       Ordered list (min to max) of unique levels.
     '''
-    min_level = sys.maxint
+    min_level = sys.maxsize
 
     for obj in iterable:
-        level = getattr(obj, 'level', sys.maxint)
+        level = getattr(obj, 'level', sys.maxsize)
         if level != logging.NOTSET:
             if level < min_level:
                 min_level = level
@@ -600,7 +603,7 @@ def parse_log_level(level):
     '''
     # Is it a string representation of an integer?
     # If so convert to an int.
-    if isinstance(level, basestring):
+    if isinstance(level, six.string_types):
         try:
             level = int(level)
         except:
@@ -608,7 +611,7 @@ def parse_log_level(level):
 
     # If it's a string lookup it's name and map to logging level
     # otherwise validate the integer value is in range.
-    if isinstance(level, basestring):
+    if isinstance(level, six.string_types):
         result = log_level_name_map.get(level.lower()) #pylint: disable=E1103
         if result is None:
             raise ValueError('unknown log level (%s)' % level)
@@ -936,7 +939,7 @@ class LogManager(object):
         if value is not None:
             try:
                 level = parse_log_level(value)
-            except Exception, e:
+            except Exception as e:
                 raise ValueError("could not set %s (%s)" % (attr, e))
             setattr(self, attr, level)
 
@@ -959,7 +962,7 @@ class LogManager(object):
 
                 if not isinstance(config, dict):
                     raise TypeError('expected dict for handler config, got "%s"', type(config))
-                if isinstance(logger, basestring):
+                if isinstance(logger, six.string_types):
                     logger = self.get_logger(logger)
                 else:
                     if not isinstance(logger, logging.Logger):
@@ -1148,7 +1151,7 @@ class LogManager(object):
             stream = cfg.get("stream")
             log_handler = cfg.get("log_handler")
             if filename:
-                if cfg.has_key("stream"):
+                if "stream" in cfg:
                     raise ValueError("both filename and stream are specified, must be one or the other, config: %s" % cfg)
                 path = os.path.abspath(filename)
                 filemode = cfg.get('filemode', 'a')
@@ -1167,7 +1170,7 @@ class LogManager(object):
                 user = cfg.get('user')
                 group = cfg.get('group')
                 if user is not None:
-                    if isinstance(user, basestring):
+                    if isinstance(user, six.string_types):
                         pw = pwd.getpwnam(user)
                         uid = pw.pw_uid
                     elif isinstance(user, int):
@@ -1175,7 +1178,7 @@ class LogManager(object):
                     else:
                         raise TypeError("user (%s) is not int or basestring" % user)
                 if group is not None:
-                    if isinstance(group, basestring):
+                    if isinstance(group, six.string_types):
                         pw = pwd.getpwnam(group)
                         gid = pw.pw_gid
                     elif isinstance(group, int):
@@ -1216,7 +1219,7 @@ class LogManager(object):
             datefmt = cfg.get("datefmt", None)
             formatter = logging.Formatter(format, datefmt)
             time_zone_converter = cfg.get('time_zone_converter', time.localtime)
-            if isinstance(time_zone_converter, basestring):
+            if isinstance(time_zone_converter, six.string_types):
                 converter = {'local'     : time.localtime,
                              'localtime' : time.localtime,
                              'gmt'       : time.gmtime,
@@ -1239,8 +1242,8 @@ class LogManager(object):
             if level is not None:
                 try:
                     level = parse_log_level(level)
-                except Exception, e:
-                    print >>sys.stderr, 'could not set handler log level "%s" (%s)' % (level, e)
+                except Exception as e:
+                    print('could not set handler log level "%s" (%s)' % (level, e), file=sys.stderr)
                     level = None
             if level is None:
                 level = self.default_level
@@ -1310,7 +1313,7 @@ class LogManager(object):
           List of loggers with the handler is bound to.
         '''
 
-        if isinstance(handler, basestring):
+        if isinstance(handler, six.string_types):
             handler = self.get_handler(handler)
         elif isinstance(handler, logging.Handler):
             if not handler in self.handlers.values():
@@ -1344,7 +1347,7 @@ class LogManager(object):
             use configure_state to track the state of the log manager.
         '''
 
-        if isinstance(handler, basestring):
+        if isinstance(handler, six.string_types):
             handler = self.get_handler(handler)
         elif not isinstance(handler, logging.Handler):
             raise TypeError('handler must be basestring or Handler object, got %s' % type(handler))
@@ -1439,7 +1442,7 @@ class LogManager(object):
 
         :return:
           The minimum of all the handler's levels. If no
-          handlers are defined sys.maxint will be returned.
+          handlers are defined sys.maxsize will be returned.
         '''
 
         handlers = self.get_logger_handlers(logger)
@@ -1522,7 +1525,7 @@ class LogManager(object):
         '''
 
         is_object = False
-        if isinstance(who, basestring):
+        if isinstance(who, six.string_types):
             obj_name = who
         else:
             is_object = True
