@@ -52,7 +52,7 @@ def setup_package():
     try:
         setup(
             name = "ipapython",
-            version = "4.1.4",
+            version = "4.3.1",
             license = "GPL",
             author = "Karl MacMillan, et.al.",
             author_email = "kmacmill@redhat.com",
@@ -62,10 +62,13 @@ def setup_package():
             description = DOCLINES[0],
             long_description = "\n".join(DOCLINES[2:]),
             download_url = "http://www.freeipa.org/page/Downloads",
-            classifiers=filter(None, CLASSIFIERS.split('\n')),
+            classifiers=[line for line in CLASSIFIERS.split('\n') if line],
             platforms = ["Linux", "Solaris", "Unix"],
             package_dir = {'ipapython': ''},
-            packages = [ "ipapython", "ipapython.dnssec" ],
+            packages = ["ipapython",
+                        "ipapython.dnssec",
+                        "ipapython.secrets",
+                        "ipapython.install"],
         )
     finally:
         del sys.path[0]
