@@ -31,7 +31,6 @@ from ipapython import ipautil
 from ipapython.ipa_log_manager import *
 from ipapython.dn import DN
 from ipaserver.install import installutils
-from ipaserver.plugins import ldap2
 from ipalib.util import normalize_name
 
 import os, string, struct, copy
@@ -43,7 +42,8 @@ from samba.ndr import ndr_pack, ndr_print
 from samba import net
 import samba
 import random
-from M2Crypto import RC4
+from cryptography.hazmat.primitives.ciphers import Cipher, algorithms
+from cryptography.hazmat.backends import default_backend
 try:
     from ldap.controls import RequestControl as LDAPControl #pylint: disable=F0401
 except ImportError:
@@ -55,11 +55,16 @@ from dns import resolver, rdatatype
 from dns.exception import DNSException
 import pysss_nss_idmap
 import pysss
+import six
 from ipaplatform.paths import paths
 
 from ldap.filter import escape_filter_chars
 from time import sleep
 
+if six.PY3:
+    unicode = str
+    long = int
+
 __doc__ = _("""
 Classes to manage trust joins using DCE-RPC calls
 
@@ -67,6 +72,10 @@ The code in this module relies heavily on samba4-python package
 and Samba4 python bindings.
 """)
 
+# Both constants can be used as masks against trust direction
+# because bi-directional has two lower bits set.
+TRUST_ONEWAY        = 1
+TRUST_BIDIRECTIONAL = 3
 
 def is_sid_valid(sid):
     try:
@@ -89,6 +98,10 @@ dcerpc_error_codes = {
     -1073741811: # NT_STATUS_INVALID_PARAMETER
         errors.RemoteRetrieveError(
             reason=_('AD domain controller complains about communication sequence. It may mean unsynchronized time on both sides, for example')),
+    -1073741776: # NT_STATUS_INVALID_PARAMETER_MIX, we simply will skip the binding
+        access_denied_error,
+    -1073741772: # NT_STATUS_OBJECT_NAME_NOT_FOUND
+        errors.RemoteRetrieveError(reason=_('CIFS server configuration does not allow access to \\\\pipe\\lsarpc')),
 }
 
 dcerpc_error_messages = {
@@ -100,6 +113,14 @@ dcerpc_error_messages = {
          errors.RequirementError(name=_('At least the domain or IP address should be specified')),
 }
 
+pysss_type_key_translation_dict = {
+    pysss_nss_idmap.ID_USER: 'user',
+    pysss_nss_idmap.ID_GROUP: 'group',
+    # Used for users with magic private groups
+    pysss_nss_idmap.ID_BOTH: 'both',
+}
+
+
 def assess_dcerpc_exception(num=None,message=None):
     """
     Takes error returned by Samba bindings and converts it into
@@ -113,6 +134,14 @@ def assess_dcerpc_exception(num=None,message=None):
                   message "%(message)s" (both may be "None")''') % dict(num=num, message=message)
     return errors.RemoteRetrieveError(reason=reason)
 
+
+def arcfour_encrypt(key, data):
+    algorithm = algorithms.ARC4(key)
+    cipher = Cipher(algorithm, mode=None, backend=default_backend())
+    encryptor = cipher.encryptor()
+    return encryptor.update(data)
+
+
 class ExtendedDNControl(LDAPControl):
     # This class attempts to implement LDAP control that would work
     # with both python-ldap 2.4.x and 2.3.x, thus there is mix of properties
@@ -144,6 +173,7 @@ class DomainValidator(object):
         self._domains = None
         self._info = dict()
         self._creds = None
+        self._admin_creds = None
         self._parm = None
 
     def is_configured(self):
@@ -154,7 +184,7 @@ class DomainValidator(object):
             self.sid = entry_attrs[self.ATTR_SID][0]
             self.dn = entry_attrs.dn
             self.domain = self.api.env.domain
-        except errors.NotFound, e:
+        except errors.NotFound as e:
             return False
         return True
 
@@ -186,7 +216,7 @@ class DomainValidator(object):
                     trust_partner = entry[self.ATTR_TRUST_PARTNER][0]
                     flatname_normalized = entry[self.ATTR_FLATNAME][0].lower()
                     trusted_sid = entry[self.ATTR_TRUSTED_SID][0]
-                except KeyError, e:
+                except KeyError as e:
                     # Some piece of trusted domain info in LDAP is missing
                     # Skip the domain, but leave log entry for investigation
                     api.log.warn("Trusted domain '%s' entry misses an "
@@ -196,7 +226,7 @@ class DomainValidator(object):
                 result[trust_partner] = (flatname_normalized,
                                          security.dom_sid(trusted_sid))
             return result
-        except errors.NotFound, e:
+        except errors.NotFound as e:
             return []
 
     def set_trusted_domains(self):
@@ -323,12 +353,17 @@ class DomainValidator(object):
 
         return entries
 
-    def get_trusted_domain_object_sid(self, object_name):
+    def get_trusted_domain_object_sid(self, object_name, fallback_to_ldap=True):
         result = pysss_nss_idmap.getsidbyname(object_name)
         if object_name in result and (pysss_nss_idmap.SID_KEY in result[object_name]):
             object_sid = result[object_name][pysss_nss_idmap.SID_KEY]
             return object_sid
 
+        # If fallback to AD DC LDAP is not allowed, bail out
+        if not fallback_to_ldap:
+            raise errors.ValidationError(name=_('trusted domain object'),
+               error= _('SSSD was unable to resolve the object to a valid SID'))
+
         # Else, we are going to contact AD DC LDAP
         components = normalize_name(object_name)
         if not ('domain' in components or 'flatname' in components):
@@ -351,12 +386,33 @@ class DomainValidator(object):
         try:
             test_sid = security.dom_sid(sid)
             return unicode(test_sid)
-        except TypeError, e:
+        except TypeError as e:
             raise errors.ValidationError(name=_('trusted domain object'),
                error= _('Trusted domain did not return a valid SID for the object'))
 
+    def get_trusted_domain_object_type(self, name_or_sid):
+        """
+        Return the type of the object corresponding to the given name in
+        the trusted domain, which is either 'user', 'group' or 'both'.
+        The 'both' types is used for users with magic private groups.
+        """
+
+        object_type = None
+
+        if is_sid_valid(name_or_sid):
+            result = pysss_nss_idmap.getnamebysid(name_or_sid)
+        else:
+            result = pysss_nss_idmap.getsidbyname(name_or_sid)
+
+        if name_or_sid in result:
+            object_type = result[name_or_sid].get(pysss_nss_idmap.TYPE_KEY)
+
+        # Do the translation to hide pysss_nss_idmap constants
+        # from higher-level code
+        return pysss_type_key_translation_dict.get(object_type)
+
     def get_trusted_domain_object_from_sid(self, sid):
-        root_logger.info("Converting SID to object name: %s" % sid)
+        root_logger.debug("Converting SID to object name: %s" % sid)
 
         # Check if the given SID is valid
         if not self.is_trusted_sid_valid(sid):
@@ -374,7 +430,7 @@ class DomainValidator(object):
                 return result.get(pysss_nss_idmap.NAME_KEY)
 
         # If unsuccessful, search AD DC LDAP
-        root_logger.info("Searching AD DC LDAP")
+        root_logger.debug("Searching AD DC LDAP")
 
         escaped_sid = escape_filter_chars(
             security.dom_sid(sid).__ndr_pack__(),
@@ -532,7 +588,7 @@ class DomainValidator(object):
         # Destroy the contents of the ccache
         root_logger.debug('Destroying the contents of the separate ccache')
 
-        (stdout, stderr, returncode) = ipautil.run(
+        ipautil.run(
             [paths.KDESTROY, '-A', '-c', ccache_path],
             env={'KRB5CCNAME': ccache_path},
             raiseonerr=False)
@@ -541,12 +597,56 @@ class DomainValidator(object):
         root_logger.debug('Running kinit from ipa.keytab to obtain HTTP '
                           'service principal with MS-PAC attached.')
 
-        (stdout, stderr, returncode) = ipautil.run(
+        result = ipautil.run(
             [paths.KINIT, '-kt', keytab, principal],
             env={'KRB5CCNAME': ccache_path},
             raiseonerr=False)
 
-        if returncode == 0:
+        if result.returncode == 0:
+            return (ccache_path, principal)
+        else:
+            return (None, None)
+
+    def kinit_as_administrator(self, domain):
+        """
+        Initializes ccache with http service credentials.
+
+        Applies session code defaults for ccache directory and naming prefix.
+        Session code uses krbccache_prefix+<pid>, we use
+        krbccache_prefix+<TD>+<domain netbios name> so there is no clash.
+
+        Returns tuple (ccache path, principal) where (None, None) signifes an
+        error on ccache initialization
+        """
+
+        if self._admin_creds == None:
+            return (None, None)
+
+        domain_suffix = domain.replace('.', '-')
+
+        ccache_name = "%sTDA%s" % (krbccache_prefix, domain_suffix)
+        ccache_path = os.path.join(krbccache_dir, ccache_name)
+
+        (principal, password) = self._admin_creds.split('%', 1)
+
+        # Destroy the contents of the ccache
+        root_logger.debug('Destroying the contents of the separate ccache')
+
+        ipautil.run(
+            [paths.KDESTROY, '-A', '-c', ccache_path],
+            env={'KRB5CCNAME': ccache_path},
+            raiseonerr=False)
+
+        # Destroy the contents of the ccache
+        root_logger.debug('Running kinit with credentials of AD administrator')
+
+        result = ipautil.run(
+            [paths.KINIT, principal],
+            env={'KRB5CCNAME': ccache_path},
+            stdin=password,
+            raiseonerr=False)
+
+        if result.returncode == 0:
             return (ccache_path, principal)
         else:
             return (None, None)
@@ -583,10 +683,13 @@ class DomainValidator(object):
         Returns LDAP result or None.
         """
 
-        (ccache_name, principal) = self.kinit_as_http(info['dns_domain'])
+        ccache_name = None
+
+        if self._admin_creds:
+            (ccache_name, principal) = self.kinit_as_administrator(info['dns_domain'])
 
         if ccache_name:
-            with installutils.private_ccache(path=ccache_name):
+            with ipautil.private_ccache(path=ccache_name):
                 entries = None
 
                 try:
@@ -605,7 +708,7 @@ class DomainValidator(object):
                         basedn = ipautil.realm_to_suffix(info['dns_domain'])
 
                     entries = conn.get_entries(basedn, scope, filter, attrs)
-                except Exception, e:
+                except Exception as e:
                     msg = "Search on AD DC {host}:{port} failed with: {err}"\
                           .format(host=host, port=str(port), err=str(e))
                     if quiet:
@@ -640,11 +743,11 @@ class DomainValidator(object):
         result = None
         try:
             result = netrc.finddc(domain=domain, flags=nbt.NBT_SERVER_LDAP | nbt.NBT_SERVER_GC | nbt.NBT_SERVER_CLOSEST)
-        except RuntimeError, e:
+        except RuntimeError as e:
             try:
                 # If search of closest GC failed, attempt to find any one
                 result = netrc.finddc(domain=domain, flags=nbt.NBT_SERVER_LDAP | nbt.NBT_SERVER_GC)
-            except RuntimeError, e:
+            except RuntimeError as e:
                 finddc_error = e
 
         if not self._domains:
@@ -665,7 +768,7 @@ class DomainValidator(object):
 
             try:
                 answers = resolver.query(gc_name, rdatatype.SRV)
-            except DNSException, e:
+            except DNSException as e:
                 answers = []
 
             for answer in answers:
@@ -710,7 +813,8 @@ class TrustDomainInstance(object):
        try:
            result = lsa.lsarpc(binding, self.parm, self.creds)
            return result
-       except RuntimeError, (num, message):
+       except RuntimeError as e:
+           num, message = e.args
            raise assess_dcerpc_exception(num=num, message=message)
 
     def init_lsa_pipe(self, remote_host):
@@ -728,16 +832,20 @@ class TrustDomainInstance(object):
             return
 
         attempts = 0
+        session_attempts = 0
         bindings = self.__gen_lsa_bindings(remote_host)
         for binding in bindings:
             try:
                 self._pipe = self.__gen_lsa_connection(binding)
-                if self._pipe:
+                if self._pipe and self._pipe.session_key:
                     break
-            except errors.ACIError, e:
+            except errors.ACIError as e:
                 attempts = attempts + 1
+            except RuntimeError as e:
+                # When session key is not available, we just skip this binding
+                session_attempts = session_attempts + 1
 
-        if self._pipe is None and attempts == len(bindings):
+        if self._pipe is None and (attempts + session_attempts) == len(bindings):
             raise errors.ACIError(
                 info=_('CIFS server %(host)s denied your credentials') % dict(host=remote_host))
 
@@ -745,6 +853,7 @@ class TrustDomainInstance(object):
             raise errors.RemoteRetrieveError(
                 reason=_('Cannot establish LSA connection to %(host)s. Is CIFS server running?') % dict(host=remote_host))
         self.binding = binding
+        self.session_key = self._pipe.session_key
 
     def __gen_lsa_bindings(self, remote_host):
         """
@@ -753,12 +862,11 @@ class TrustDomainInstance(object):
         Generate all we can use. init_lsa_pipe() will try them one by one until
         there is one working.
 
-        We try NCACN_NP before NCACN_IP_TCP and signed sessions before unsigned.
+        We try NCACN_NP before NCACN_IP_TCP and use SMB2 before SMB1 or defaults.
         """
         transports = (u'ncacn_np', u'ncacn_ip_tcp')
-        options = ( u',', u'')
-        binding_template=lambda x,y,z: u'%s:%s[%s]' % (x, y, z)
-        return [binding_template(t, remote_host, o) for t in transports for o in options]
+        options = ( u'smb2,print', u'print')
+        return [u'%s:%s[%s]' % (t, remote_host, o) for t in transports for o in options]
 
     def retrieve_anonymously(self, remote_host, discover_srv=False, search_pdc=False):
         """
@@ -773,7 +881,7 @@ class TrustDomainInstance(object):
                 result = netrc.finddc(domain=remote_host, flags=flags)
             else:
                 result = netrc.finddc(address=remote_host, flags=flags)
-        except RuntimeError, e:
+        except RuntimeError as e:
             raise assess_dcerpc_exception(message=str(e))
 
         if not result:
@@ -795,11 +903,11 @@ class TrustDomainInstance(object):
             (objtype, res) = conn.search_s('', _ldap.SCOPE_BASE)[0]
             search_result = res['defaultNamingContext'][0]
             self.info['dns_hostname'] = res['dnsHostName'][0]
-        except _ldap.LDAPError, e:
+        except _ldap.LDAPError as e:
             root_logger.error(
                 "LDAP error when connecting to %(host)s: %(error)s" %
                     dict(host=unicode(result.pdc_name), error=str(e)))
-        except KeyError, e:
+        except KeyError as e:
             root_logger.error("KeyError: {err}, LDAP entry from {host} "
                               "returned malformed. Your DNS might be "
                               "misconfigured."
@@ -822,7 +930,8 @@ class TrustDomainInstance(object):
         try:
             self._policy_handle = self._pipe.OpenPolicy2(u"", objectAttribute, security.SEC_FLAG_MAXIMUM_ALLOWED)
             result = self._pipe.QueryInfoPolicy2(self._policy_handle, lsa.LSA_POLICY_INFO_DNS)
-        except RuntimeError, (num, message):
+        except RuntimeError as e:
+            num, message = e.args
             raise assess_dcerpc_exception(num=num, message=message)
 
         self.info['name'] = unicode(result.name.string)
@@ -834,16 +943,13 @@ class TrustDomainInstance(object):
 
         try:
             result = self._pipe.QueryInfoPolicy2(self._policy_handle, lsa.LSA_POLICY_INFO_ROLE)
-        except RuntimeError, (num, message):
+        except RuntimeError as e:
+            num, message = e.args
             raise assess_dcerpc_exception(num=num, message=message)
 
         self.info['is_pdc'] = (result.role == lsa.LSA_ROLE_PRIMARY)
 
     def generate_auth(self, trustdom_secret):
-        def arcfour_encrypt(key, data):
-            c = RC4.RC4(key)
-            return c.update(data)
-
         password_blob = string_to_array(trustdom_secret.encode('utf-16-le'))
 
         clear_value = drsblobs.AuthInfoClear()
@@ -930,11 +1036,11 @@ class TrustDomainInstance(object):
                                                                           ftinfo, 0)
                 if collision_info:
                     root_logger.error("When setting forest trust information, got collision info back:\n%s" % (ndr_print(collision_info)))
-        except RuntimeError, e:
+        except RuntimeError as e:
             # We can ignore the error here -- setting up name suffix routes may fail
             pass
 
-    def establish_trust(self, another_domain, trustdom_secret):
+    def establish_trust(self, another_domain, trustdom_secret, trust_type='bidirectional'):
         """
         Establishes trust between our and another domain
         Input: another_domain -- instance of TrustDomainInstance, initialized with #retrieve call
@@ -952,7 +1058,9 @@ class TrustDomainInstance(object):
         info.domain_name.string = another_domain.info['dns_domain']
         info.netbios_name.string = another_domain.info['name']
         info.sid = security.dom_sid(another_domain.info['sid'])
-        info.trust_direction = lsa.LSA_TRUST_DIRECTION_INBOUND | lsa.LSA_TRUST_DIRECTION_OUTBOUND
+        info.trust_direction = lsa.LSA_TRUST_DIRECTION_INBOUND
+        if trust_type == TRUST_BIDIRECTIONAL:
+            info.trust_direction |= lsa.LSA_TRUST_DIRECTION_OUTBOUND
         info.trust_type = lsa.LSA_TRUST_TYPE_UPLEVEL
         info.trust_attributes = 0
 
@@ -961,14 +1069,16 @@ class TrustDomainInstance(object):
             dname.string = another_domain.info['dns_domain']
             res = self._pipe.QueryTrustedDomainInfoByName(self._policy_handle, dname, lsa.LSA_TRUSTED_DOMAIN_INFO_FULL_INFO)
             self._pipe.DeleteTrustedDomain(self._policy_handle, res.info_ex.sid)
-        except RuntimeError, (num, message):
+        except RuntimeError as e:
+            num, message = e.args
             # Ignore anything but access denied (NT_STATUS_ACCESS_DENIED)
             if num == -1073741790:
                 raise access_denied_error
 
         try:
             trustdom_handle = self._pipe.CreateTrustedDomainEx2(self._policy_handle, info, self.auth_info, security.SEC_STD_DELETE)
-        except RuntimeError, (num, message):
+        except RuntimeError as e:
+            num, message = e.args
             raise assess_dcerpc_exception(num=num, message=message)
 
         # We should use proper trustdom handle in order to modify the
@@ -982,7 +1092,7 @@ class TrustDomainInstance(object):
             infoclass.enc_types |= security.KERB_ENCTYPE_AES128_CTS_HMAC_SHA1_96
             infoclass.enc_types |= security.KERB_ENCTYPE_AES256_CTS_HMAC_SHA1_96
             self._pipe.SetInformationTrustedDomain(trustdom_handle, lsa.LSA_TRUSTED_DOMAIN_SUPPORTED_ENCRYPTION_TYPES, infoclass)
-        except RuntimeError, e:
+        except RuntimeError as e:
             # We can ignore the error here -- changing enctypes is for
             # improved security but the trust will work with default values as
             # well. In particular, the call may fail against Windows 2003
@@ -990,48 +1100,74 @@ class TrustDomainInstance(object):
             pass
 
         try:
-            info.trust_attributes = lsa.LSA_TRUST_ATTRIBUTE_FOREST_TRANSITIVE
+            info = self._pipe.QueryTrustedDomainInfo(trustdom_handle, lsa.LSA_TRUSTED_DOMAIN_INFO_INFO_EX)
+            info.trust_attributes |= lsa.LSA_TRUST_ATTRIBUTE_FOREST_TRANSITIVE
             self._pipe.SetInformationTrustedDomain(trustdom_handle, lsa.LSA_TRUSTED_DOMAIN_INFO_INFO_EX, info)
-        except RuntimeError, e:
+        except RuntimeError as e:
             root_logger.error('unable to set trust to transitive: %s' % (str(e)))
             pass
         if self.info['is_pdc']:
             self.update_ftinfo(another_domain)
 
     def verify_trust(self, another_domain):
-        def retrieve_netlogon_info_2(domain, function_code, data):
+        def retrieve_netlogon_info_2(logon_server, domain, function_code, data):
             try:
                 netr_pipe = netlogon.netlogon(domain.binding, domain.parm, domain.creds)
-                result = netr_pipe.netr_LogonControl2Ex(logon_server=None,
+                result = netr_pipe.netr_LogonControl2Ex(logon_server=logon_server,
                                            function_code=function_code,
                                            level=2,
                                            data=data
                                            )
                 return result
-            except RuntimeError, (num, message):
+            except RuntimeError as e:
+                num, message = e.args
                 raise assess_dcerpc_exception(num=num, message=message)
 
-        result = retrieve_netlogon_info_2(self,
+        result = retrieve_netlogon_info_2(None, self,
                                           netlogon.NETLOGON_CONTROL_TC_VERIFY,
                                           another_domain.info['dns_domain'])
-        if (result and (result.flags and netlogon.NETLOGON_VERIFY_STATUS_RETURNED)):
-            if (result.pdc_connection_status[0] != 0) and (result.tc_connection_status[0] != 0):
+
+        if result and result.flags and netlogon.NETLOGON_VERIFY_STATUS_RETURNED:
+            if result.pdc_connection_status[0] != 0 and result.tc_connection_status[0] != 0:
                 if result.pdc_connection_status[1] == "WERR_ACCESS_DENIED":
                     # Most likely AD DC hit another IPA replica which yet has no trust secret replicated
+
                     # Sleep and repeat again
                     self.validation_attempts += 1
                     if self.validation_attempts < 10:
                         sleep(5)
                         return self.verify_trust(another_domain)
-                    raise errors.ACIError(reason=_('IPA master denied trust validation requests from AD DC '
-                                                   '%(count)d times. Most likely AD DC contacted a replica '
-                                                   'that has no trust information replicated yet.' % (self.validation_attempts)))
+
+                    # If we get here, we already failed 10 times
+                    srv_record_templates = (
+                        '_ldap._tcp.%s',
+                        '_ldap._tcp.Default-First-Site-Name._sites.dc._msdcs.%s'
+                    )
+
+                    srv_records = ', '.join(
+                        [srv_record % api.env.domain
+                         for srv_record in srv_record_templates]
+                    )
+
+                    error_message = _(
+                        'IPA master denied trust validation requests from AD '
+                        'DC %(count)d times. Most likely AD DC contacted a '
+                        'replica that has no trust information replicated '
+                        'yet. Additionally, please check that AD DNS is able '
+                        'to resolve %(records)s SRV records to the correct '
+                        'IPA server.') % dict(count=self.validation_attempts,
+                                              records=srv_records)
+
+                    raise errors.ACIError(info=error_message)
+
                 raise assess_dcerpc_exception(*result.pdc_connection_status)
+
             return True
+
         return False
 
 
-def fetch_domains(api, mydomain, trustdomain, creds=None):
+def fetch_domains(api, mydomain, trustdomain, creds=None, server=None):
     trust_flags = dict(
                 NETR_TRUST_FLAG_IN_FOREST = 0x00000001,
                 NETR_TRUST_FLAG_OUTBOUND  = 0x00000002,
@@ -1072,29 +1208,41 @@ def fetch_domains(api, mydomain, trustdomain, creds=None):
     cr.set_workstation(domain_validator.flatname)
     netrc = net.Net(creds=cr, lp=td.parm)
     try:
-        result = netrc.finddc(domain=trustdomain,
-                              flags=nbt.NBT_SERVER_LDAP | nbt.NBT_SERVER_DS)
-    except RuntimeError, e:
+        if server:
+            result = netrc.finddc(address=server,
+                                  flags=nbt.NBT_SERVER_LDAP | nbt.NBT_SERVER_DS)
+        else:
+            result = netrc.finddc(domain=trustdomain,
+                                  flags=nbt.NBT_SERVER_LDAP | nbt.NBT_SERVER_DS)
+    except RuntimeError as e:
         raise assess_dcerpc_exception(message=str(e))
 
     td.info['dc'] = unicode(result.pdc_dns_name)
-    if creds is None:
+    if type(creds) is bool:
+        # Rely on existing Kerberos credentials in the environment
+        td.creds = credentials.Credentials()
+        td.creds.set_kerberos_state(credentials.MUST_USE_KERBEROS)
+        td.creds.guess(td.parm)
+        td.creds.set_workstation(domain_validator.flatname)
+        domains = communicate(td)
+    else:
+        # Attempt to authenticate as HTTP/ipa.master and use cross-forest trust
+        # or as passed-in user in case of a one-way trust
         domval = DomainValidator(api)
-        (ccache_name, principal) = domval.kinit_as_http(trustdomain)
+        ccache_name = None
+        principal = None
+        if creds:
+            domval._admin_creds = creds
+            (ccache_name, principal) = domval.kinit_as_administrator(trustdomain)
+        else:
+            (ccache_name, principal) = domval.kinit_as_http(trustdomain)
         td.creds = credentials.Credentials()
         td.creds.set_kerberos_state(credentials.MUST_USE_KERBEROS)
         if ccache_name:
-            with installutils.private_ccache(path=ccache_name):
+            with ipautil.private_ccache(path=ccache_name):
                 td.creds.guess(td.parm)
                 td.creds.set_workstation(domain_validator.flatname)
                 domains = communicate(td)
-    else:
-        td.creds = credentials.Credentials()
-        td.creds.set_kerberos_state(credentials.DONT_USE_KERBEROS)
-        td.creds.guess(td.parm)
-        td.creds.parse_string(creds)
-        td.creds.set_workstation(domain_validator.flatname)
-        domains = communicate(td)
 
     if domains is None:
         return None
@@ -1186,7 +1334,7 @@ class TrustDomainJoins(object):
         if self.local_domain.read_only:
             return
 
-	self.local_domain.ftinfo_records = []
+        self.local_domain.ftinfo_records = []
 
         realm_domains = self.api.Command.realmdomains_show()['result']
         # Use realmdomains' modification timestamp to judge records last update time
@@ -1201,7 +1349,7 @@ class TrustDomainJoins(object):
             ftinfo['rec_type'] = lsa.LSA_FOREST_TRUST_TOP_LEVEL_NAME
             self.local_domain.ftinfo_records.append(ftinfo)
 
-    def join_ad_full_credentials(self, realm, realm_server, realm_admin, realm_passwd):
+    def join_ad_full_credentials(self, realm, realm_server, realm_admin, realm_passwd, trust_type):
         if not self.configured:
             return None
 
@@ -1219,13 +1367,17 @@ class TrustDomainJoins(object):
         if not self.remote_domain.read_only:
             trustdom_pass = samba.generate_random_password(128, 128)
             self.get_realmdomains()
-            self.remote_domain.establish_trust(self.local_domain, trustdom_pass)
-            self.local_domain.establish_trust(self.remote_domain, trustdom_pass)
-            result = self.remote_domain.verify_trust(self.local_domain)
+            self.remote_domain.establish_trust(self.local_domain, trustdom_pass, trust_type)
+            self.local_domain.establish_trust(self.remote_domain, trustdom_pass, trust_type)
+            # if trust is inbound, we don't need to verify it because AD DC will respond
+            # with WERR_NO_SUCH_DOMAIN -- in only does verification for outbound trusts.
+            result = True
+            if trust_type == TRUST_BIDIRECTIONAL:
+                result = self.remote_domain.verify_trust(self.local_domain)
             return dict(local=self.local_domain, remote=self.remote_domain, verified=result)
         return None
 
-    def join_ad_ipa_half(self, realm, realm_server, trustdom_passwd):
+    def join_ad_ipa_half(self, realm, realm_server, trustdom_passwd, trust_type):
         if not self.configured:
             return None
 
@@ -1235,5 +1387,5 @@ class TrustDomainJoins(object):
         if self.remote_domain.info['dns_domain'] != self.remote_domain.info['dns_forest']:
             raise errors.NotAForestRootError(forest=self.remote_domain.info['dns_forest'], domain=self.remote_domain.info['dns_domain'])
 
-        self.local_domain.establish_trust(self.remote_domain, trustdom_passwd)
+        self.local_domain.establish_trust(self.remote_domain, trustdom_passwd, trust_type)
         return dict(local=self.local_domain, remote=self.remote_domain, verified=False)
