@@ -17,6 +17,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import print_function
+
 import os
 import errno
 import ldap
@@ -26,11 +28,13 @@ import string
 import struct
 import re
 
+import six
+
 from ipaserver.install import service
 from ipaserver.install import installutils
-from ipaserver.install.dsinstance import realm_to_serverid
 from ipaserver.install.bindinstance import get_rr, add_rr, del_rr, \
                                            dns_zone_exists
+from ipaserver.install.replication import wait_for_task
 from ipalib import errors, api
 from ipalib.util import normalize_zone
 from ipapython.dn import DN
@@ -44,8 +48,10 @@ from ipaplatform import services
 from ipaplatform.paths import paths
 from ipaplatform.tasks import tasks
 
+if six.PY3:
+    unicode = str
 
-ALLOWED_NETBIOS_CHARS = string.ascii_uppercase + string.digits
+ALLOWED_NETBIOS_CHARS = string.ascii_uppercase + string.digits + '-'
 
 UPGRADE_ERROR = """
 Entry %(dn)s does not exist.
@@ -60,9 +66,9 @@ SELINUX_BOOLEAN_SETTINGS = {'samba_portmapper': 'on'}
 def check_inst():
     for smbfile in [paths.SMBD, paths.NET]:
         if not os.path.exists(smbfile):
-            print "%s was not found on this system" % smbfile
-            print "Please install the 'samba' packages and " \
-                  "start the installation again"
+            print("%s was not found on this system" % smbfile)
+            print("Please install the 'samba' packages and " \
+                  "start the installation again")
             return False
 
     #TODO: Add check for needed samba4 libraries
@@ -72,7 +78,7 @@ def check_inst():
 def ipa_smb_conf_exists():
     try:
         conf_fd = open(paths.SMB_CONF, 'r')
-    except IOError, err:
+    except IOError as err:
         if err.errno == errno.ENOENT:
             return False
 
@@ -84,13 +90,19 @@ def ipa_smb_conf_exists():
     return False
 
 
-def check_netbios_name(s):
-    # NetBIOS names may not be longer than 15 allowed characters
-    if not s or len(s) > 15 or \
-       ''.join([c for c in s if c not in ALLOWED_NETBIOS_CHARS]):
+def check_netbios_name(name):
+    # Empty NetBIOS name is not allowed
+    if name is None:
         return False
 
-    return True
+    # NetBIOS names may not be longer than 15 allowed characters
+    invalid_netbios_name = any([
+        len(name) > 15,
+        ''.join([c for c in name if c not in ALLOWED_NETBIOS_CHARS])
+    ])
+
+    return not invalid_netbios_name
+
 
 def make_netbios_name(s):
     return ''.join([c for c in s.split('.')[0].upper() \
@@ -108,7 +120,6 @@ class ADTRUSTInstance(service.Service):
     FALLBACK_GROUP_NAME = u'Default SMB Group'
 
     def __init__(self, fstore=None):
-        self.ip_address = None
         self.netbios_name = None
         self.reset_netbios_name = None
         self.no_msdcs = None
@@ -156,7 +167,7 @@ class ADTRUSTInstance(service.Service):
         self.cifs_principal = "cifs/" + self.fqdn + "@" + self.realm
         self.suffix = ipautil.realm_to_suffix(self.realm)
         self.ldapi_socket = "%%2fvar%%2frun%%2fslapd-%s.socket" % \
-                            realm_to_serverid(self.realm)
+                            installutils.realm_to_serverid(self.realm)
 
         # DN definitions
         self.trust_dn = DN(api.env.container_trusts, self.suffix)
@@ -173,6 +184,9 @@ class ADTRUSTInstance(service.Service):
         self.cifs_agent = DN(('krbprincipalname', self.cifs_principal.lower()),
                              api.env.container_service,
                              self.suffix)
+        self.host_princ = DN(('fqdn', self.fqdn),
+                             api.env.container_host,
+                             self.suffix)
 
 
     def __gen_sid_string(self):
@@ -207,13 +221,13 @@ class ADTRUSTInstance(service.Service):
 
         try:
             admin_entry = self.admin_conn.get_entry(admin_dn)
-        except:
+        except errors.NotFound:
             self.print_msg("IPA admin object not found")
             return
 
         try:
             admin_group_entry = self.admin_conn.get_entry(admin_group_dn)
-        except:
+        except errors.NotFound:
             self.print_msg("IPA admin group object not found")
             return
 
@@ -224,7 +238,7 @@ class ADTRUSTInstance(service.Service):
                 self.admin_conn.modify_s(admin_dn, \
                             [(ldap.MOD_ADD, "objectclass", self.OBJC_USER), \
                              (ldap.MOD_ADD, self.ATTR_SID, dom_sid + "-500")])
-            except:
+            except Exception:
                 self.print_msg("Failed to modify IPA admin object")
 
         if admin_group_entry.single_value.get(self.ATTR_SID):
@@ -234,7 +248,7 @@ class ADTRUSTInstance(service.Service):
                 self.admin_conn.modify_s(admin_group_dn, \
                             [(ldap.MOD_ADD, "objectclass", self.OBJC_GROUP), \
                              (ldap.MOD_ADD, self.ATTR_SID, dom_sid + "-512")])
-            except:
+            except Exception:
                 self.print_msg("Failed to modify IPA admin group object")
 
     def __add_default_trust_view(self):
@@ -246,7 +260,7 @@ class ADTRUSTInstance(service.Service):
         except errors.NotFound:
             try:
                 self._ldap_mod('default-trust-view.ldif', self.sub_dict)
-            except Exception, e:
+            except Exception as e:
                 self.print_msg("Failed to add default trust view.")
                 raise e
         else:
@@ -290,7 +304,7 @@ class ADTRUSTInstance(service.Service):
         except errors.NotFound:
             try:
                 self._ldap_mod('default-smb-group.ldif', self.sub_dict)
-            except Exception, e:
+            except Exception as e:
                 self.print_msg("Failed to add fallback group.")
                 raise e
 
@@ -305,7 +319,7 @@ class ADTRUSTInstance(service.Service):
         try:
             mod = [(ldap.MOD_ADD, self.ATTR_FALLBACK_GROUP, fb_group_dn)]
             self.admin_conn.modify_s(self.smb_dom_dn, mod)
-        except:
+        except Exception:
             self.print_msg("Failed to add fallback group to domain object")
 
     def __add_rid_bases(self):
@@ -326,7 +340,7 @@ class ADTRUSTInstance(service.Service):
                                   r.single_value.get('ipaBaseRID'),
                                   r.single_value.get('ipaSecondaryBaseRID')))
 
-            ranges_with_no_rid_base = filter(no_rid_base_set, ranges)
+            ranges_with_no_rid_base = [r for r in ranges if no_rid_base_set(r)]
 
             # Return if no range is without RID base
             if len(ranges_with_no_rid_base) == 0:
@@ -357,7 +371,7 @@ class ADTRUSTInstance(service.Service):
                                                  str(self.rid_base)),
                                          (ldap.MOD_ADD, "ipaSecondaryBaseRID",
                                                  str(self.secondary_rid_base))])
-            except ldap.CONSTRAINT_VIOLATION, e:
+            except ldap.CONSTRAINT_VIOLATION as e:
                 self.print_msg("Failed to add RID bases to the local range "
                                "object:\n  %s" % e[0]['info'])
                 raise RuntimeError("Constraint violation.\n")
@@ -400,7 +414,7 @@ class ADTRUSTInstance(service.Service):
             except errors.NotFound:
                 try:
                     name = new_dn[1].attr
-                except Exception, e:
+                except Exception as e:
                     self.print_msg('Cannot extract RDN attribute value from "%s": %s' % \
                           (new_dn, e))
                     return
@@ -452,32 +466,34 @@ class ADTRUSTInstance(service.Service):
         """
         self.__add_plugin_conf('CLDAP', 'ipa_cldap', 'ipa-cldap-conf.ldif')
 
-    def __add_sidgen_module(self):
+    def __add_sidgen_task(self):
         """
         Add sidgen directory server plugin configuration and the related task
         if they not already exist.
         """
-        self.__add_plugin_conf('Sidgen', 'IPA SIDGEN', 'ipa-sidgen-conf.ldif')
         self.__add_plugin_conf('Sidgen task', 'ipa-sidgen-task',
                                'ipa-sidgen-task-conf.ldif')
 
     def __add_sids(self):
         """
-        Add SIDs for existing users and groups
+        Add SIDs for existing users and groups. Make sure the task is finished
+        before continuing.
         """
 
         try:
+            # Start the sidgen task
             self._ldap_mod("ipa-sidgen-task-run.ldif", self.sub_dict)
-        except:
-            pass
 
-    def __add_extdom_module(self):
-        """
-        Add directory server configuration for the extdom extended operation
-        if it not already exists.
-        """
-        self.__add_plugin_conf('Extdom', 'ipa_extdom_extop',
-                               'ipa-extdom-extop-conf.ldif')
+            # Notify the user about the possible delay
+            self.print_msg("This step may take considerable amount of time, please wait..")
+
+            # Wait for the task to complete
+            task_dn = DN('cn=sidgen,cn=ipa-sidgen-task,cn=tasks,cn=config')
+            wait_for_task(self.admin_conn, task_dn)
+
+        except Exception as e:
+            root_logger.warning("Exception occured during SID generation: {0}"
+                                .format(str(e)))
 
     def __add_s4u2proxy_target(self):
         """
@@ -504,6 +520,11 @@ class ADTRUSTInstance(service.Service):
         os.write(tmp_fd, conf)
         os.close(tmp_fd)
 
+        # Workaround for: https://fedorahosted.org/freeipa/ticket/5687
+        # We make sure that paths.SMB_CONF file exists, hence touch it
+        with open(paths.SMB_CONF, 'a'):
+            os.utime(paths.SMB_CONF, None)
+
         args = [paths.NET, "conf", "import", tmp_name]
 
         try:
@@ -511,6 +532,13 @@ class ADTRUSTInstance(service.Service):
         finally:
             os.remove(tmp_name)
 
+    def __setup_group_membership(self):
+        # Add the CIFS and host principals to the 'adtrust agents' group
+        # as 389-ds only operates with GroupOfNames, we have to use
+        # the principal's proper dn as defined in self.cifs_agent
+        service.add_principals_to_group(self.admin_conn, self.smb_dn, "member",
+                                        [self.cifs_agent, self.host_princ])
+
     def __setup_principal(self):
         try:
             api.Command.service_add(unicode(self.cifs_principal))
@@ -519,28 +547,11 @@ class ADTRUSTInstance(service.Service):
             # adtrustinstance is managed
             # That's fine, we we'll re-extract the key again.
             pass
-        except Exception, e:
+        except Exception as e:
             self.print_msg("Cannot add CIFS service: %s" % e)
 
-        # Add the principal to the 'adtrust agents' group
-        # as 389-ds only operates with GroupOfNames, we have to use
-        # the principal's proper dn as defined in self.cifs_agent
-        try:
-            current = self.admin_conn.get_entry(self.smb_dn)
-            members = current.get('member', [])
-            if not(self.cifs_agent in members):
-                current["member"] = members + [self.cifs_agent]
-                self.admin_conn.update_entry(current)
-        except errors.NotFound:
-            entry = self.admin_conn.make_entry(
-                self.smb_dn,
-                objectclass=["top", "GroupOfNames"],
-                cn=[self.smb_dn['cn']],
-                member=[self.cifs_agent],
-            )
-            self.admin_conn.add_entry(entry)
-
         self.clean_samba_keytab()
+        installutils.remove_ccache(paths.KRB5CC_SAMBA)
 
         try:
             ipautil.run(["ipa-getkeytab", "--server", self.fqdn,
@@ -555,7 +566,7 @@ class ADTRUSTInstance(service.Service):
             try:
                 ipautil.run(["ipa-rmkeytab", "--principal", self.cifs_principal,
                                          "-k", self.samba_keytab])
-            except ipautil.CalledProcessError, e:
+            except ipautil.CalledProcessError as e:
                 if e.returncode != 5:
                     root_logger.critical("Failed to remove old key for %s"
                                          % self.cifs_principal)
@@ -571,12 +582,7 @@ class ADTRUSTInstance(service.Service):
         """
 
         zone = self.domain_name
-        host, host_domain = self.fqdn.split(".", 1)
-
-        if normalize_zone(zone) == normalize_zone(host_domain):
-            host_in_rr = host
-        else:
-            host_in_rr = normalize_zone(self.fqdn)
+        host_in_rr = normalize_zone(self.fqdn)
 
         priority = 0
 
@@ -606,9 +612,10 @@ class ADTRUSTInstance(service.Service):
             self.print_msg(err_msg)
             self.print_msg("Add the following service records to your DNS " \
                            "server for DNS zone %s: " % zone)
-            for srv in ipa_srv_rec:
-                for suff in win_srv_suffix:
-                    self.print_msg(" - %s%s"  % (srv[0], suff))
+            for suff in win_srv_suffix:
+                for srv in ipa_srv_rec:
+                    self.print_msg("%s%s IN SRV %s"  % (srv[0], suff, " ".join(srv[1])))
+            self.print_msg("")
             return
 
         for (srv, rdata, port) in ipa_srv_rec:
@@ -672,7 +679,7 @@ class ADTRUSTInstance(service.Service):
 
         try:
             krb5conf = open(paths.KRB5_CONF, 'r')
-        except IOError, e:
+        except IOError as e:
             self.print_msg("Cannot open /etc/krb5.conf (%s)\n" % str(e))
             return
 
@@ -706,9 +713,9 @@ class ADTRUSTInstance(service.Service):
                             # this is CIFS service of a different host in our
                             # REALM, we need to remember it to announce via
                             # SRV records for _msdcs
-                            self.cifs_hosts.append(fqdn.split(".")[0])
+                            self.cifs_hosts.append(normalize_zone(fqdn))
 
-        except Exception, e:
+        except Exception as e:
             root_logger.critical("Checking replicas for cifs principals failed with error '%s'" % e)
 
     def __enable_compat_tree(self):
@@ -722,14 +729,14 @@ class ADTRUSTInstance(service.Service):
                 if not(config[1] in lookup_nsswitch):
                     current[lookup_nsswitch_name] = [config[1]]
                     self.admin_conn.update_entry(current)
-        except Exception, e:
+        except Exception as e:
             root_logger.critical("Enabling nsswitch support in slapi-nis failed with error '%s'" % e)
 
     def __start(self):
         try:
             self.start()
             services.service('winbind').start()
-        except:
+        except Exception:
             root_logger.critical("CIFS services failed to start")
 
     def __stop(self):
@@ -737,13 +744,19 @@ class ADTRUSTInstance(service.Service):
         try:
             services.service('winbind').stop()
             self.stop()
-        except:
+        except Exception:
             pass
 
     def __restart_dirsrv(self):
         try:
             services.knownservices.dirsrv.restart()
-        except:
+        except Exception:
+            pass
+
+    def __restart_smb(self):
+        try:
+            services.knownservices.smb.restart()
+        except Exception:
             pass
 
     def __enable(self):
@@ -756,13 +769,13 @@ class ADTRUSTInstance(service.Service):
         try:
             self.ldap_enable('ADTRUST', self.fqdn, self.dm_password, \
                              self.suffix)
-        except (ldap.ALREADY_EXISTS, errors.DuplicateEntry), e:
+        except (ldap.ALREADY_EXISTS, errors.DuplicateEntry) as e:
             root_logger.info("ADTRUST Service startup entry already exists.")
 
         try:
             self.ldap_enable('EXTID', self.fqdn, self.dm_password, \
                              self.suffix)
-        except (ldap.ALREADY_EXISTS, errors.DuplicateEntry), e:
+        except (ldap.ALREADY_EXISTS, errors.DuplicateEntry) as e:
             root_logger.info("EXTID Service startup entry already exists.")
 
     def __setup_sub_dict(self):
@@ -774,11 +787,10 @@ class ADTRUSTInstance(service.Service):
                              LDAPI_SOCKET = self.ldapi_socket,
                              FQDN = self.fqdn)
 
-    def setup(self, fqdn, ip_address, realm_name, domain_name, netbios_name,
+    def setup(self, fqdn, realm_name, domain_name, netbios_name,
               reset_netbios_name, rid_base, secondary_rid_base,
               no_msdcs=False, add_sids=False, smbd_user="samba", enable_compat=False):
         self.fqdn = fqdn
-        self.ip_address = ip_address
         self.realm = realm_name
         self.domain_name = domain_name
         self.netbios_name = netbios_name
@@ -849,14 +861,15 @@ class ADTRUSTInstance(service.Service):
         self.step("creating samba config registry", self.__write_smb_registry)
         self.step("writing samba config file", self.__write_smb_conf)
         self.step("adding cifs Kerberos principal", self.__setup_principal)
+        self.step("adding cifs and host Kerberos principals to the adtrust agents group", \
+                  self.__setup_group_membership)
         self.step("check for cifs services defined on other replicas", self.__check_replica)
         self.step("adding cifs principal to S4U2Proxy targets", self.__add_s4u2proxy_target)
         self.step("adding admin(group) SIDs", self.__add_admin_sids)
         self.step("adding RID bases", self.__add_rid_bases)
         self.step("updating Kerberos config", self.__update_krb5_conf)
         self.step("activating CLDAP plugin", self.__add_cldap_module)
-        self.step("activating sidgen plugin and task", self.__add_sidgen_module)
-        self.step("activating extdom plugin", self.__add_extdom_module)
+        self.step("activating sidgen task", self.__add_sidgen_task)
         self.step("configuring smbd to start on boot", self.__enable)
         self.step("adding special DNS service records", \
                   self.__add_dns_service_records)
@@ -876,6 +889,7 @@ class ADTRUSTInstance(service.Service):
         if self.add_sids:
             self.step("adding SIDs to existing users and groups",
                       self.__add_sids)
+        self.step("restarting smbd", self.__restart_smb)
 
         self.start_creation(show_service_name=False)
 
@@ -911,8 +925,7 @@ class ADTRUSTInstance(service.Service):
             self.print_msg('WARNING: ' + str(e))
 
         # Remove samba's credentials cache
-        krb5cc_samba = paths.KRB5CC_SAMBA
-        installutils.remove_file(krb5cc_samba)
+        installutils.remove_ccache(ccache_path=paths.KRB5CC_SAMBA)
 
         # Remove samba's configuration file
         installutils.remove_file(self.smb_conf)
