@@ -17,6 +17,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import print_function
+
 import tempfile
 import os
 import pwd
@@ -26,11 +28,10 @@ import sys
 import time
 
 import ldap
+import six
 
-import installutils
-import service
-from ipaserver.plugins import ldap2
-from ipaserver.install.dsinstance import realm_to_serverid
+from ipaserver.install import installutils
+from ipaserver.install import service
 from ipaserver.install.cainstance import IPA_CA_RECORD
 from ipapython import sysrestore, ipautil, ipaldap
 from ipapython.ipa_log_manager import *
@@ -38,14 +39,21 @@ from ipapython.dn import DN
 import ipalib
 from ipalib import api, errors
 from ipaplatform import services
+from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipaplatform.tasks import tasks
 from ipalib.util import (validate_zonemgr_str, normalize_zonemgr,
-        get_dns_forward_zone_update_policy, get_dns_reverse_zone_update_policy,
-        normalize_zone, get_reverse_zone_default, zone_is_reverse,
-        validate_dnssec_forwarder)
+                         get_dns_forward_zone_update_policy,
+                         get_dns_reverse_zone_update_policy,
+                         normalize_zone, get_reverse_zone_default,
+                         zone_is_reverse, validate_dnssec_global_forwarder,
+                         DNSSECSignatureMissingError, EDNS0UnsupportedError,
+                         UnresolvableRecordError, verify_host_resolvable)
 from ipalib.constants import CACERT
 
+if six.PY3:
+    unicode = str
+
 NAMED_CONF = paths.NAMED_CONF
 RESOLV_CONF = paths.RESOLV_CONF
 
@@ -63,33 +71,12 @@ named_conf_arg_options_template_nonstr = "%(indent)s%(name)s %(value)s;\n"
 named_conf_include_re = re.compile(r'\s*include\s+"(?P<path>)"\s*;')
 named_conf_include_template = "include \"%(path)s\";\n"
 
-def check_inst(unattended):
-    has_bind = True
-    named = services.knownservices.named
-    if not os.path.exists(named.get_binary_path()):
-        print "BIND was not found on this system"
-        print ("Please install the '%s' package and start the installation again"
-              % named.get_package_name())
-        has_bind = False
-
-    # Also check for the LDAP BIND plug-in
-    if not os.path.exists(paths.BIND_LDAP_SO) and \
-       not os.path.exists(paths.BIND_LDAP_SO_64):
-        print "The BIND LDAP plug-in was not found on this system"
-        print "Please install the 'bind-dyndb-ldap' package and start the installation again"
-        has_bind = False
-
-    if not has_bind:
-        return False
-
-    if not unattended and os.path.exists(NAMED_CONF):
-        msg = "Existing BIND configuration detected, overwrite?"
-        return ipautil.user_input(msg, False)
-
-    return True
 
 def create_reverse():
-    return ipautil.user_input("Do you want to configure the reverse zone?", True)
+    return ipautil.user_input(
+        "Do you want to search for missing reverse zones?",
+        True
+    )
 
 def named_conf_exists():
     try:
@@ -196,7 +183,7 @@ def named_conf_set_directive(name, value, section=NAMED_SECTION_IPA,
                     if name == match.group('name'):
                         matched = True
                         if value is not None:
-                            if not isinstance(value, basestring):
+                            if not isinstance(value, six.string_types):
                                 value = str(value)
                             new_conf = named_conf_arg_template \
                                     % dict(indent=last_indent,
@@ -255,7 +242,7 @@ def dns_container_exists(fqdn, suffix, dm_password=None, ldapi=False, realm=None
 
     return ret
 
-def dns_zone_exists(name):
+def dns_zone_exists(name, api=api):
     try:
         zone = api.Command.dnszone_show(unicode(name))
     except ipalib.errors.NotFound:
@@ -284,34 +271,60 @@ def verify_reverse_zone(zone, ip_address):
 
     return True
 
-def find_reverse_zone(ip_address):
+def find_reverse_zone(ip_address, api=api):
     ip = netaddr.IPAddress(ip_address)
     zone = normalize_zone(ip.reverse_dns)
 
     while len(zone) > 0:
-        if dns_zone_exists(zone):
+        if dns_zone_exists(zone, api):
             return zone
         foo, bar, zone = zone.partition('.')
 
     return None
 
-def get_reverse_zone(ip_address):
-    return find_reverse_zone(ip_address) or get_reverse_zone_default(ip_address)
 
-def read_reverse_zone(default, ip_address):
+def read_reverse_zone(default, ip_address, allow_zone_overlap=False):
     while True:
         zone = ipautil.user_input("Please specify the reverse zone name", default=default)
         if not zone:
             return None
-        if verify_reverse_zone(zone, ip_address):
-            break
-        else:
-            print "Invalid reverse zone %s for IP address %s" % (zone, ip_address)
+        if not verify_reverse_zone(zone, ip_address):
+            root_logger.error("Invalid reverse zone %s for IP address %s"
+                              % (zone, ip_address))
+            continue
+        if not allow_zone_overlap:
+            try:
+                ipautil.check_zone_overlap(zone, raise_on_error=False)
+            except ValueError as e:
+                root_logger.error("Reverse zone %s will not be used: %s"
+                                  % (zone, e))
+                continue
+        break
 
     return normalize_zone(zone)
 
+
+def get_auto_reverse_zones(ip_addresses):
+    auto_zones = []
+    for ip in ip_addresses:
+        if ipautil.reverse_record_exists(ip):
+            # PTR exist there is no reason to create reverse zone
+            root_logger.info("Reverse record for IP address %s already "
+                             "exists" % ip)
+            continue
+        default_reverse = get_reverse_zone_default(ip)
+        try:
+            ipautil.check_zone_overlap(default_reverse)
+        except ValueError:
+            root_logger.info("Reverse zone %s for IP address %s already exists"
+                             % (default_reverse, ip))
+            continue
+        auto_zones.append((ip, default_reverse))
+    return auto_zones
+
 def add_zone(name, zonemgr=None, dns_backup=None, ns_hostname=None,
-       update_policy=None, force=False):
+             update_policy=None, force=False, skip_overlap_check=False,
+             api=api):
 
     # always normalize zones
     name = normalize_zone(name)
@@ -337,11 +350,12 @@ def add_zone(name, zonemgr=None, dns_backup=None, ns_hostname=None,
                                 idnsupdatepolicy=unicode(update_policy),
                                 idnsallowquery=u'any',
                                 idnsallowtransfer=u'none',
+                                skip_overlap_check=skip_overlap_check,
                                 force=force)
     except (errors.DuplicateEntry, errors.EmptyModlist):
         pass
 
-def add_rr(zone, name, type, rdata, dns_backup=None, **kwargs):
+def add_rr(zone, name, type, rdata, dns_backup=None, api=api, **kwargs):
     addkw = { '%srecord' % str(type.lower()) : unicode(rdata) }
     addkw.update(kwargs)
     try:
@@ -351,40 +365,44 @@ def add_rr(zone, name, type, rdata, dns_backup=None, **kwargs):
     if dns_backup:
         dns_backup.add(zone, type, name, rdata)
 
-def add_fwd_rr(zone, host, ip_address):
+def add_fwd_rr(zone, host, ip_address, api=api):
     addr = netaddr.IPAddress(ip_address)
     if addr.version == 4:
-        add_rr(zone, host, "A", ip_address)
+        add_rr(zone, host, "A", ip_address, None, api)
     elif addr.version == 6:
-        add_rr(zone, host, "AAAA", ip_address)
+        add_rr(zone, host, "AAAA", ip_address, None, api)
 
-def add_ptr_rr(zone, ip_address, fqdn, dns_backup=None):
+def add_ptr_rr(zone, ip_address, fqdn, dns_backup=None, api=api):
     name = get_reverse_record_name(zone, ip_address)
-    add_rr(zone, name, "PTR", normalize_zone(fqdn), dns_backup)
+    add_rr(zone, name, "PTR", normalize_zone(fqdn), dns_backup, api)
 
-def add_ns_rr(zone, hostname, dns_backup=None, force=True):
+
+def add_ns_rr(zone, hostname, dns_backup=None, force=True, api=api):
     hostname = normalize_zone(hostname)
     add_rr(zone, "@", "NS", hostname, dns_backup=dns_backup,
-            force=force)
+           force=force, api=api)
 
-def del_rr(zone, name, type, rdata):
+
+def del_rr(zone, name, type, rdata, api=api):
     delkw = { '%srecord' % str(type.lower()) : unicode(rdata) }
     try:
         api.Command.dnsrecord_del(unicode(zone), unicode(name), **delkw)
     except (errors.NotFound, errors.AttrValueNotFound, errors.EmptyModlist):
         pass
 
-def del_fwd_rr(zone, host, ip_address):
+
+def del_fwd_rr(zone, host, ip_address, api=api):
     addr = netaddr.IPAddress(ip_address)
     if addr.version == 4:
-        del_rr(zone, host, "A", ip_address)
+        del_rr(zone, host, "A", ip_address, api=api)
     elif addr.version == 6:
-        del_rr(zone, host, "AAAA", ip_address)
+        del_rr(zone, host, "AAAA", ip_address, api=api)
 
-def del_ns_rr(zone, name, rdata):
-    del_rr(zone, name, 'NS', rdata)
 
-def get_rr(zone, name, type):
+def del_ns_rr(zone, name, rdata, api=api):
+    del_rr(zone, name, 'NS', rdata, api=api)
+
+def get_rr(zone, name, type, api=api):
     rectype = '%srecord' % unicode(type.lower())
     ret = api.Command.dnsrecord_find(unicode(zone), unicode(name))
     if ret['count'] > 0:
@@ -394,8 +412,8 @@ def get_rr(zone, name, type):
 
     return []
 
-def get_fwd_rr(zone, host):
-    return [x for t in ("A", "AAAA") for x in get_rr(zone, host, t)]
+def get_fwd_rr(zone, host, api=api):
+    return [x for t in ("A", "AAAA") for x in get_rr(zone, host, t, api)]
 
 def zonemgr_callback(option, opt_str, value, parser):
     """
@@ -410,7 +428,7 @@ def zonemgr_callback(option, opt_str, value, parser):
                 encoding = 'utf-8'
             value = value.decode(encoding)
             validate_zonemgr_str(value)
-        except ValueError, e:
+        except ValueError as e:
             # FIXME we can do this in better way
             # https://fedorahosted.org/freeipa/ticket/4804
             # decode to proper stderr encoding
@@ -422,65 +440,94 @@ def zonemgr_callback(option, opt_str, value, parser):
 
     parser.values.zonemgr = value
 
-def check_reverse_zones(ip_addresses, reverse_zones, options, unattended, search_reverse_zones=False):
-    reverse_asked = False
 
-    ret_reverse_zones = []
-    # check that there is IP address in every reverse zone
-    if reverse_zones:
-        for rz in reverse_zones:
-            for ip in ip_addresses:
-                if verify_reverse_zone(rz, ip):
-                    ret_reverse_zones.append(normalize_zone(rz))
-                    break
-            else:
-                # no ip matching reverse zone found
-                sys.exit("There is no IP address matching reverse zone %s." % rz)
-    if not options.no_reverse:
-        # check that there is reverse zone for every IP
-        for ip in ip_addresses:
-            if search_reverse_zones and find_reverse_zone(str(ip)):
-                # reverse zone is already in LDAP
+def check_reverse_zones(ip_addresses, reverse_zones, options, unattended,
+                        search_reverse_zones=False):
+    checked_reverse_zones = []
+
+    if (not options.no_reverse and not reverse_zones
+            and not options.auto_reverse):
+        if unattended:
+            options.no_reverse = True
+        else:
+            options.no_reverse = not create_reverse()
+
+    # shortcut
+    if options.no_reverse:
+        return []
+
+    # verify zones passed in options
+    for rz in reverse_zones:
+        # isn't the zone managed by someone else
+        if not options.allow_zone_overlap:
+            try:
+                ipautil.check_zone_overlap(rz)
+            except ValueError as e:
+                msg = "Reverse zone %s will not be used: %s" % (rz, e)
+                if unattended:
+                    sys.exit(msg)
+                else:
+                    root_logger.warning(msg)
                 continue
-            for rz in ret_reverse_zones:
-                if verify_reverse_zone(rz, ip):
-                    # reverse zone was entered by user
-                    break
-            else:
-                # no reverse zone for ip found
-                if not reverse_asked:
-                    if not unattended and not reverse_zones:
-                        # user did not specify reverse_zone nor no_reverse
-                        options.no_reverse = not create_reverse()
-                        if options.no_reverse:
-                            # user decided not to create reverse zone
-                            return []
-                    reverse_asked = True
-                rz = get_reverse_zone_default(str(ip))
-                if not unattended:
-                    rz = read_reverse_zone(rz, str(ip))
-                ret_reverse_zones.append(rz)
+        checked_reverse_zones.append(normalize_zone(rz))
+
+    # check that there is reverse zone for every IP
+    ips_missing_reverse = []
+    for ip in ip_addresses:
+        if search_reverse_zones and find_reverse_zone(str(ip)):
+            # reverse zone is already in LDAP
+            continue
+        for rz in checked_reverse_zones:
+            if verify_reverse_zone(rz, ip):
+                # reverse zone was entered by user
+                break
+        else:
+            ips_missing_reverse.append(ip)
+
+    # create reverse zone for IP addresses that does not have one
+    for (ip, rz) in get_auto_reverse_zones(ips_missing_reverse):
+        if options.auto_reverse:
+            root_logger.info("Reverse zone %s will be created" % rz)
+            checked_reverse_zones.append(rz)
+        elif unattended:
+            root_logger.warning("Missing reverse record for IP address %s"
+                                % ip)
+        else:
+            if ipautil.user_input("Do you want to create reverse zone for IP "
+                                  "%s" % ip, True):
+                rz = read_reverse_zone(rz, str(ip), options.allow_zone_overlap)
+                checked_reverse_zones.append(rz)
+
+    return checked_reverse_zones
 
-    return ret_reverse_zones
 
 def check_forwarders(dns_forwarders, logger):
-    print "Checking forwarders, please wait ..."
+    print("Checking DNS forwarders, please wait ...")
     forwarders_dnssec_valid = True
     for forwarder in dns_forwarders:
-        logger.debug("Checking forwarder: %s", forwarder)
-        result = validate_dnssec_forwarder(forwarder)
-        if result is None:
-            logger.error("Forwarder %s does not work", forwarder)
-            raise RuntimeError("Forwarder %s does not respond" % forwarder)
-        elif result is False:
+        logger.debug("Checking DNS server: %s", forwarder)
+        try:
+            validate_dnssec_global_forwarder(forwarder, log=logger)
+        except DNSSECSignatureMissingError as e:
             forwarders_dnssec_valid = False
-            logger.warning("DNS forwarder %s does not return DNSSEC signatures in answers", forwarder)
+            logger.warning("DNS server %s does not support DNSSEC: %s",
+                           forwarder, e)
             logger.warning("Please fix forwarder configuration to enable DNSSEC support.\n"
                 "(For BIND 9 add directive \"dnssec-enable yes;\" to \"options {}\")")
-            print ("WARNING: DNS forwarder %s does not return DNSSEC "
-                   "signatures in answers" % forwarder)
-            print "Please fix forwarder configuration to enable DNSSEC support."
-            print "(For BIND 9 add directive \"dnssec-enable yes;\" to \"options {}\")"
+            print("DNS server %s: %s" % (forwarder, e))
+            print("Please fix forwarder configuration to enable DNSSEC support.")
+            print("(For BIND 9 add directive \"dnssec-enable yes;\" to \"options {}\")")
+        except EDNS0UnsupportedError as e:
+            forwarders_dnssec_valid = False
+            logger.warning("DNS server %s does not support ENDS0 "
+                           "(RFC 6891): %s", forwarder, e)
+            logger.warning("Please fix forwarder configuration. "
+                           "DNSSEC support cannot be enabled without EDNS0")
+            print(("WARNING: DNS server %s does not support EDNS0 "
+                   "(RFC 6891): %s" % (forwarder, e)))
+        except UnresolvableRecordError as e:
+            logger.error("DNS server %s: %s", forwarder, e)
+            raise RuntimeError("DNS server %s: %s" % (forwarder, e))
 
     return forwarders_dnssec_valid
 
@@ -533,13 +580,14 @@ class DnsBackup(object):
 
 
 class BindInstance(service.Service):
-    def __init__(self, fstore=None, dm_password=None, start_tls=False):
+    def __init__(self, fstore=None, dm_password=None, api=api, ldapi=False,
+                 start_tls=False, autobind=ipaldap.AUTOBIND_DISABLED):
         service.Service.__init__(
             self, "named",
             service_desc="DNS",
             dm_password=dm_password,
-            ldapi=False,
-            autobind=ipaldap.AUTOBIND_DISABLED,
+            ldapi=ldapi,
+            autobind=autobind,
             start_tls=start_tls
         )
         self.dns_backup = DnsBackup(self)
@@ -552,6 +600,7 @@ class BindInstance(service.Service):
         self.sub_dict = None
         self.reverse_zones = []
         self.dm_password = dm_password
+        self.api = api
         self.named_regular = services.service('named-regular')
 
         if fstore:
@@ -562,7 +611,7 @@ class BindInstance(service.Service):
     suffix = ipautil.dn_attribute_property('_suffix')
 
     def setup(self, fqdn, ip_addresses, realm_name, domain_name, forwarders, ntp,
-              reverse_zones, named_user="named", zonemgr=None,
+              reverse_zones, named_user=constants.NAMED_USER, zonemgr=None,
               ca_configured=None, no_dnssec_validation=False):
         self.named_user = named_user
         self.fqdn = fqdn
@@ -584,7 +633,7 @@ class BindInstance(service.Service):
 
         self.first_instance = not dns_container_exists(
             self.fqdn, self.suffix, realm=self.realm, ldapi=True,
-            dm_password=self.dm_password)
+            dm_password=self.dm_password, autobind=self.autobind)
 
         self.__setup_sub_dict()
 
@@ -608,7 +657,7 @@ class BindInstance(service.Service):
         [bind_fd, bind_name] = tempfile.mkstemp(".db","sample.zone.")
         os.write(bind_fd, bind_txt)
         os.close(bind_fd)
-        print "Sample zone file for bind has been created in "+bind_name
+        print("Sample zone file for bind has been created in "+bind_name)
 
     def create_instance(self):
 
@@ -630,7 +679,7 @@ class BindInstance(service.Service):
         if self.first_instance:
             self.step("adding DNS container", self.__setup_dns_container)
 
-        if not dns_zone_exists(self.domain):
+        if not dns_zone_exists(self.domain, self.api):
             self.step("setting up our zone", self.__setup_zone)
         if self.reverse_zones:
             self.step("setting up reverse zone", self.__setup_reverse_zone)
@@ -661,12 +710,10 @@ class BindInstance(service.Service):
             if self.get_state("running") is None:
                 # first time store status
                 self.backup_state("running", self.is_running())
-                self.backup_state("named-regular-running",
-                                  self.named_regular.is_running())
             self.restart()
         except Exception as e:
             root_logger.error("Named service failed to start (%s)", e)
-            print "named service failed to start"
+            print("named service failed to start")
 
     def __enable(self):
         if self.get_state("enabled") is None:
@@ -684,6 +731,10 @@ class BindInstance(service.Service):
             root_logger.error("DNS service already exists")
 
         # disable named, we need to run named-pkcs11 only
+        if self.get_state("named-regular-running") is None:
+            # first time store status
+            self.backup_state("named-regular-running",
+                              self.named_regular.is_running())
         try:
             self.named_regular.stop()
         except Exception as e:
@@ -723,7 +774,7 @@ class BindInstance(service.Service):
             DOMAIN=self.domain,
             HOST=self.host,
             REALM=self.realm,
-            SERVER_ID=realm_to_serverid(self.realm),
+            SERVER_ID=installutils.realm_to_serverid(self.realm),
             FORWARDERS=fwds,
             SUFFIX=self.suffix,
             OPTIONAL_NTP=optional_ntp,
@@ -743,12 +794,12 @@ class BindInstance(service.Service):
         self.__fix_dns_privilege_members()
 
     def __fix_dns_privilege_members(self):
-        ldap = api.Backend.ldap2
+        ldap = self.api.Backend.ldap2
 
         cn = 'Update PBAC memberOf %s' % time.time()
         task_dn = DN(('cn', cn), ('cn', 'memberof task'), ('cn', 'tasks'),
                      ('cn', 'config'))
-        basedn = DN(api.env.container_privilege, api.env.basedn)
+        basedn = DN(self.api.env.container_privilege, self.api.env.basedn)
         entry = ldap.make_entry(
             task_dn,
             objectclass=['top', 'extensibleObject'],
@@ -773,24 +824,27 @@ class BindInstance(service.Service):
     def __setup_zone(self):
         # Always use force=True as named is not set up yet
         add_zone(self.domain, self.zonemgr, dns_backup=self.dns_backup,
-                ns_hostname=api.env.host, force=True)
+                 ns_hostname=self.api.env.host, force=True,
+                 skip_overlap_check=True, api=self.api)
 
-        add_rr(self.domain, "_kerberos", "TXT", self.realm)
+        add_rr(self.domain, "_kerberos", "TXT", self.realm, api=self.api)
 
     def __add_self_ns(self):
         # add NS record to all zones
-        ns_hostname = normalize_zone(api.env.host)
-        result = api.Command.dnszone_find()
+        ns_hostname = normalize_zone(self.api.env.host)
+        result = self.api.Command.dnszone_find()
         for zone in result['result']:
             zone = unicode(zone['idnsname'][0])  # we need unicode due to backup
             root_logger.debug("adding self NS to zone %s apex", zone)
-            add_ns_rr(zone, ns_hostname, self.dns_backup, force=True)
+            add_ns_rr(zone, ns_hostname, self.dns_backup, force=True,
+                      api=self.api)
 
     def __setup_reverse_zone(self):
         # Always use force=True as named is not set up yet
         for reverse_zone in self.reverse_zones:
-            add_zone(reverse_zone, self.zonemgr, ns_hostname=api.env.host,
-                dns_backup=self.dns_backup, force=True)
+            add_zone(reverse_zone, self.zonemgr, ns_hostname=self.api.env.host,
+                     dns_backup=self.dns_backup, force=True,
+                     skip_overlap_check=True, api=self.api)
 
     def __add_master_records(self, fqdn, addrs):
         host, zone = fqdn.split(".", 1)
@@ -815,25 +869,27 @@ class BindInstance(service.Service):
             )
 
         for (rname, rdata) in srv_records:
-            add_rr(self.domain, rname, "SRV", rdata, self.dns_backup)
+            add_rr(self.domain, rname, "SRV", rdata, self.dns_backup,
+                   api=self.api)
 
-        if not dns_zone_exists(zone):
-            # add DNS domain for host first
-            root_logger.debug(
-                "Host domain (%s) is different from DNS domain (%s)!" % (
-                    zone, self.domain))
-            root_logger.debug("Add DNS zone for host first.")
-
-            add_zone(zone, self.zonemgr, dns_backup=self.dns_backup,
-                     ns_hostname=self.fqdn, force=True)
+        if not dns_zone_exists(zone, self.api):
+            # check if master hostname is resolvable
+            try:
+                verify_host_resolvable(fqdn, root_logger)
+            except errors.DNSNotARecordError:
+                root_logger.warning("Master FQDN (%s) is not resolvable.",
+                                    fqdn)
 
         # Add forward and reverse records to self
         for addr in addrs:
-            add_fwd_rr(zone, host, addr)
+            try:
+                add_fwd_rr(zone, host, addr, self.api)
+            except errors.NotFound as e:
+                pass
 
-            reverse_zone = find_reverse_zone(addr)
+            reverse_zone = find_reverse_zone(addr, self.api)
             if reverse_zone:
-                add_ptr_rr(reverse_zone, addr, fqdn)
+                add_ptr_rr(reverse_zone, addr, fqdn, None, api=self.api)
 
     def __add_self(self):
         self.__add_master_records(self.fqdn, self.ip_addresses)
@@ -863,10 +919,10 @@ class BindInstance(service.Service):
             # add the CA record. So we need to find out
             root_logger.debug("Check if CA is enabled for this host")
             base_dn = DN(('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'),
-                         ('cn', 'etc'), api.env.basedn)
+                         ('cn', 'etc'), self.api.env.basedn)
             ldap_filter = '(&(objectClass=ipaConfigObject)(cn=CA))'
             try:
-                api.Backend.ldap2.find_entries(filter=ldap_filter, base_dn=base_dn)
+                self.api.Backend.ldap2.find_entries(filter=ldap_filter, base_dn=base_dn)
             except ipalib.errors.NotFound:
                 root_logger.debug("CA is not configured")
                 return
@@ -875,7 +931,7 @@ class BindInstance(service.Service):
 
         try:
             for addr in addrs:
-                add_fwd_rr(self.domain, IPA_CA_RECORD, addr)
+                add_fwd_rr(self.domain, IPA_CA_RECORD, addr, api=self.api)
         except errors.ValidationError:
             # there is a CNAME record in ipa-ca, we can't add A/AAAA records
             pass
@@ -885,11 +941,11 @@ class BindInstance(service.Service):
                                   self.ca_configured)
 
         if self.first_instance:
-            ldap = api.Backend.ldap2
+            ldap = self.api.Backend.ldap2
             try:
                 entries = ldap.get_entries(
                     DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
-                       api.env.basedn),
+                       self.api.env.basedn),
                     ldap.SCOPE_SUBTREE, '(&(objectClass=ipaConfigObject)(cn=CA))',
                     ['dn'])
             except errors.NotFound:
@@ -902,8 +958,8 @@ class BindInstance(service.Service):
                     continue
 
                 host, zone = fqdn.split('.', 1)
-                if dns_zone_exists(zone):
-                    addrs = get_fwd_rr(zone, host)
+                if dns_zone_exists(zone, self.api):
+                    addrs = get_fwd_rr(zone, host, api=self.api)
                 else:
                     addrs = installutils.resolve_host(fqdn)
 
@@ -927,7 +983,7 @@ class BindInstance(service.Service):
         # Make sure access is strictly reserved to the named user
         pent = pwd.getpwnam(self.named_user)
         os.chown(paths.NAMED_KEYTAB, pent.pw_uid, pent.pw_gid)
-        os.chmod(paths.NAMED_KEYTAB, 0400)
+        os.chmod(paths.NAMED_KEYTAB, 0o400)
 
         # modify the principal so that it is marked as an ipa service so that
         # it can host the memberof attribute, then also add it to the
@@ -940,7 +996,7 @@ class BindInstance(service.Service):
             self.admin_conn.modify_s(dns_group, mod)
         except ldap.TYPE_OR_VALUE_EXISTS:
             pass
-        except Exception, e:
+        except Exception as e:
             root_logger.critical("Could not modify principal's %s entry: %s" \
                     % (dns_principal, str(e)))
             raise
@@ -953,7 +1009,7 @@ class BindInstance(service.Service):
                (ldap.MOD_REPLACE, 'nsLookThroughLimit', '-1')]
         try:
             self.admin_conn.modify_s(dns_principal, mod)
-        except Exception, e:
+        except Exception as e:
             root_logger.critical("Could not set principal's %s LDAP limits: %s" \
                     % (dns_principal, str(e)))
             raise
@@ -1001,7 +1057,7 @@ class BindInstance(service.Service):
 
     def __generate_rndc_key(self):
         installutils.check_entropy()
-        ipautil.run(['/usr/libexec/generate-rndc-key.sh'])
+        ipautil.run([paths.GENERATE_RNDC_KEY])
 
     def add_master_dns_records(self, fqdn, ip_addresses, realm_name, domain_name,
                                reverse_zones, ntp=False, ca_configured=None):
@@ -1022,8 +1078,8 @@ class BindInstance(service.Service):
 
     def add_ipa_ca_dns_records(self, fqdn, domain_name, ca_configured=True):
         host, zone = fqdn.split(".", 1)
-        if dns_zone_exists(zone):
-            addrs = get_fwd_rr(zone, host)
+        if dns_zone_exists(zone, self.api):
+            addrs = get_fwd_rr(zone, host, api=self.api)
         else:
             addrs = installutils.resolve_host(fqdn)
 
@@ -1033,7 +1089,7 @@ class BindInstance(service.Service):
 
     def convert_ipa_ca_cnames(self, domain_name):
         # get ipa-ca CNAMEs
-        cnames = get_rr(domain_name, IPA_CA_RECORD, "CNAME")
+        cnames = get_rr(domain_name, IPA_CA_RECORD, "CNAME", api=self.api)
         if not cnames:
             return
 
@@ -1049,11 +1105,11 @@ class BindInstance(service.Service):
             cname_fqdn[cname] = fqdn
 
         # get FQDNs of all IPA masters
-        ldap = api.Backend.ldap2
+        ldap = self.api.Backend.ldap2
         try:
             entries = ldap.get_entries(
                 DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
-                   api.env.basedn),
+                   self.api.env.basedn),
                 ldap.SCOPE_ONELEVEL, None, ['cn'])
             masters = set(e['cn'][0] for e in entries)
         except errors.NotFound:
@@ -1070,7 +1126,7 @@ class BindInstance(service.Service):
 
         # delete all CNAMEs
         for cname in cnames:
-            del_rr(domain_name, IPA_CA_RECORD, "CNAME", cname)
+            del_rr(domain_name, IPA_CA_RECORD, "CNAME", cname, api=self.api)
 
         # add A/AAAA records
         for cname in cnames:
@@ -1096,32 +1152,33 @@ class BindInstance(service.Service):
         )
 
         for (record, type, rdata) in resource_records:
-            del_rr(self.domain, record, type, rdata)
+            del_rr(self.domain, record, type, rdata, api=self.api)
 
-        areclist = get_fwd_rr(zone, host)
+        areclist = get_fwd_rr(zone, host, api=self.api)
         for rdata in areclist:
-            del_fwd_rr(zone, host, rdata)
+            del_fwd_rr(zone, host, rdata, api=self.api)
 
             rzone = find_reverse_zone(rdata)
             if rzone is not None:
                 record = get_reverse_record_name(rzone, rdata)
-                del_rr(rzone, record, "PTR", normalize_zone(fqdn))
+                del_rr(rzone, record, "PTR", normalize_zone(fqdn),
+                       api=self.api)
 
     def remove_ipa_ca_dns_records(self, fqdn, domain_name):
         host, zone = fqdn.split(".", 1)
-        if dns_zone_exists(zone):
-            addrs = get_fwd_rr(zone, host)
+        if dns_zone_exists(zone, self.api):
+            addrs = get_fwd_rr(zone, host, api=self.api)
         else:
             addrs = installutils.resolve_host(fqdn)
 
         for addr in addrs:
-            del_fwd_rr(domain_name, IPA_CA_RECORD, addr)
+            del_fwd_rr(domain_name, IPA_CA_RECORD, addr, api=self.api)
 
     def remove_server_ns_records(self, fqdn):
         """
         Remove all NS records pointing to this server
         """
-        ldap = api.Backend.ldap2
+        ldap = self.api.Backend.ldap2
         ns_rdata = normalize_zone(fqdn)
 
         # find all NS records pointing to this server
@@ -1129,7 +1186,7 @@ class BindInstance(service.Service):
         search_kw['nsrecord'] = ns_rdata
         attr_filter = ldap.make_filter(search_kw, rules=ldap.MATCH_ALL)
         attributes = ['idnsname', 'objectclass']
-        dn = DN(api.env.container_dns, api.env.basedn)
+        dn = DN(self.api.env.container_dns, self.api.env.basedn)
 
         entries, truncated = ldap.find_entries(attr_filter, attributes, base_dn=dn)
 
@@ -1142,33 +1199,34 @@ class BindInstance(service.Service):
                 # zone record
                 zone = entry.single_value['idnsname']
                 root_logger.debug("zone record %s", zone)
-                del_ns_rr(zone, u'@', ns_rdata)
+                del_ns_rr(zone, u'@', ns_rdata, api=self.api)
             else:
                 zone = entry.dn[1].value  # get zone from DN
                 record = entry.single_value['idnsname']
                 root_logger.debug("record %s in zone %s", record, zone)
-                del_ns_rr(zone, record, ns_rdata)
+                del_ns_rr(zone, record, ns_rdata, api=self.api)
 
     def check_global_configuration(self):
         """
         Check global DNS configuration in LDAP server and inform user when it
         set and thus overrides his configured options in named.conf.
         """
-        result = api.Command.dnsconfig_show()
+        result = self.api.Command.dnsconfig_show()
         global_conf_set = any(param in result['result'] for \
-                              param in api.Object['dnsconfig'].params)
+                              param in self.api.Object['dnsconfig'].params)
 
         if not global_conf_set:
-            print "Global DNS configuration in LDAP server is empty"
-            print "You can use 'dnsconfig-mod' command to set global DNS options that"
-            print "would override settings in local named.conf files"
+            print("Global DNS configuration in LDAP server is empty")
+            print("You can use 'dnsconfig-mod' command to set global DNS options that")
+            print("would override settings in local named.conf files")
             return
 
-        print "Global DNS configuration in LDAP server is not empty"
-        print "The following configuration options override local settings in named.conf:"
-        print ""
-        textui = ipalib.cli.textui()
-        api.Command.dnsconfig_show.output_for_cli(textui, result, None, reverse=False)
+        print("Global DNS configuration in LDAP server is not empty")
+        print("The following configuration options override local settings in named.conf:")
+        print("")
+        textui = ipalib.cli.textui(self.api)
+        self.api.Command.dnsconfig_show.output_for_cli(textui, result, None,
+                                                       reverse=False)
 
     def uninstall(self):
         if self.is_configured():
@@ -1179,23 +1237,22 @@ class BindInstance(service.Service):
         named_regular_running = self.restore_state("named-regular-running")
         named_regular_enabled = self.restore_state("named-regular-enabled")
 
-        self.dns_backup.clear_records(api.Backend.ldap2.isconnected())
+        self.dns_backup.clear_records(self.api.Backend.ldap2.isconnected())
 
-        if not running is None:
-            self.stop()
 
         for f in [NAMED_CONF, RESOLV_CONF]:
             try:
                 self.fstore.restore_file(f)
-            except ValueError, error:
+            except ValueError as error:
                 root_logger.debug(error)
                 pass
 
-        if not enabled is None and not enabled:
-            self.disable()
+        # disabled by default, by ldap_enable()
+        if enabled:
+            self.enable()
 
-        if not running is None and running:
-            self.start()
+        if running:
+            self.restart()
 
         self.named_regular.unmask()
         if named_regular_enabled:
@@ -1203,3 +1260,6 @@ class BindInstance(service.Service):
 
         if named_regular_running:
             self.named_regular.start()
+
+        installutils.remove_keytab(paths.NAMED_KEYTAB)
+        installutils.remove_ccache(run_as=constants.NAMED_USER)
