@@ -29,7 +29,8 @@ from hashlib import sha1
 import fcntl
 import time
 import datetime
-import ConfigParser as configparser
+
+from six.moves import configparser
 
 from ipapython.ipa_log_manager import root_logger
 from ipapython import dogtag
@@ -42,6 +43,7 @@ from ipalib import pkcs10, x509, api
 from ipalib.errors import CertificateOperationError
 from ipalib.text import _
 from ipaplatform import services
+from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 
 # Apache needs access to this database so we need to create it
@@ -95,7 +97,7 @@ class CertDB(object):
         self.subject_base = subject_base
         try:
             self.cwd = os.getcwd()
-        except OSError, e:
+        except OSError as e:
             raise RuntimeError("Unable to determine the current directory: %s" % str(e))
 
         if not subject_base:
@@ -162,8 +164,8 @@ class CertDB(object):
     def gen_password(self):
         return sha1(ipautil.ipa_generate_password()).hexdigest()
 
-    def run_certutil(self, args, stdin=None):
-        return self.nssdb.run_certutil(args, stdin)
+    def run_certutil(self, args, stdin=None, **kwargs):
+        return self.nssdb.run_certutil(args, stdin, **kwargs)
 
     def run_signtool(self, args, stdin=None):
         with open(self.passwd_fname, "r") as f:
@@ -229,8 +231,9 @@ class CertDB(object):
         root_nicknames = self.find_root_cert(nickname)[:-1]
         fd = open(self.cacert_fname, "w")
         for root in root_nicknames:
-            (cert, stderr, returncode) = self.run_certutil(["-L", "-n", root, "-a"])
-            fd.write(cert)
+            result = self.run_certutil(["-L", "-n", root, "-a"],
+                                       capture_output=True)
+            fd.write(result.output)
         fd.close()
         os.chmod(self.cacert_fname, stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
         if create_pkcs12:
@@ -274,7 +277,8 @@ class CertDB(object):
         """
         try:
             args = ["-L", "-n", nickname, "-a"]
-            (cert, err, returncode) = self.run_certutil(args)
+            result = self.run_certutil(args, capture_output=True)
+            cert = result.output
             if pem:
                 return cert
             else:
@@ -293,14 +297,10 @@ class CertDB(object):
         /usr/lib[64]/ipa/certmonger.
         """
         if command is not None and not os.path.isabs(command):
-            if sys.maxsize > 2**32L:
-                libpath = 'lib64'
-            else:
-                libpath = 'lib'
-            command = paths.CERTMONGER_COMMAND_TEMPLATE % (libpath, command)
+            command = paths.CERTMONGER_COMMAND_TEMPLATE % (command)
         try:
             request_id = certmonger.start_tracking(nickname, self.secdir, password_file, command)
-        except RuntimeError, e:
+        except RuntimeError as e:
             root_logger.error("certmonger failed starting to track certificate: %s" % str(e))
             return
 
@@ -316,7 +316,7 @@ class CertDB(object):
         """
         try:
             certmonger.stop_tracking(self.secdir, nickname=nickname)
-        except RuntimeError, e:
+        except RuntimeError as e:
             root_logger.error("certmonger failed to stop tracking certificate: %s" % str(e))
 
     def create_server_cert(self, nickname, hostname, other_certdb=None, subject=None):
@@ -368,9 +368,10 @@ class CertDB(object):
                 "-z", self.noise_fname,
                 "-f", self.passwd_fname,
                 "-a"]
-        (stdout, stderr, returncode) = self.run_certutil(args)
+        result = self.run_certutil(args,
+                                   capture_output=True, capture_error=True)
         os.remove(self.noise_fname)
-        return (stdout, stderr)
+        return (result.output, result.error_output)
 
     def issue_server_cert(self, certreq_fname, cert_fname):
         self.setup_cert_request()
@@ -386,7 +387,7 @@ class CertDB(object):
         # We just want the CSR bits, make sure there is nothing else
         csr = pkcs10.strip_header(csr)
 
-        params = {'profileId': 'caIPAserviceCert',
+        params = {'profileId': dogtag.DEFAULT_PROFILE,
                 'cert_request_type': 'pkcs10',
                 'requestor_name': 'IPA Installer',
                 'cert_request': csr,
@@ -397,17 +398,13 @@ class CertDB(object):
         password = f.readline()
         f.close()
         result = dogtag.https_request(
-            self.host_name,
-            api.env.ca_ee_install_port or
-                dogtag.configured_constants().EE_SECURE_PORT,
-            "/ca/ee/ca/profileSubmitSSLClient",
+            self.host_name, 8443, "/ca/ee/ca/profileSubmitSSLClient",
             self.secdir, password, "ipaCert", **params)
-        http_status, http_reason_phrase, http_headers, http_body = result
+        http_status, http_headers, http_body = result
 
         if http_status != 200:
             raise CertificateOperationError(
-                error=_('Unable to communicate with CMS (%s)') %
-                    http_reason_phrase)
+                error=_('Unable to communicate with CMS (status %d)') % http_status)
 
         # The result is an XML blob. Pull the certificate out of that
         doc = xml.dom.minidom.parseString(http_body)
@@ -454,12 +451,9 @@ class CertDB(object):
         password = f.readline()
         f.close()
         result = dogtag.https_request(
-            self.host_name,
-            api.env.ca_ee_install_port or
-                dogtag.configured_constants().EE_SECURE_PORT,
-            "/ca/ee/ca/profileSubmitSSLClient",
+            self.host_name, 8443, "/ca/ee/ca/profileSubmitSSLClient",
             self.secdir, password, "ipaCert", **params)
-        http_status, http_reason_phrase, http_headers, http_body = result
+        http_status, http_headers, http_body = result
         if http_status != 200:
             raise RuntimeError("Unable to submit cert request")
 
@@ -518,8 +512,7 @@ class CertDB(object):
         f.write(pwdfile.read())
         f.close()
         pwdfile.close()
-        # TODO: replace explicit uid by a platform-specific one
-        self.set_perms(self.pwd_conf, uid="apache")
+        self.set_perms(self.pwd_conf, uid=constants.HTTPD_USER)
 
     def find_root_cert(self, nickname):
         """
@@ -647,11 +640,23 @@ class CertDB(object):
 
     def publish_ca_cert(self, location):
         shutil.copy(self.cacert_fname, location)
-        os.chmod(location, 0444)
+        os.chmod(location, 0o444)
 
     def export_pem_cert(self, nickname, location):
         return self.nssdb.export_pem_cert(nickname, location)
 
+    def request_service_cert(self, nickname, principal, host, pwdconf=False):
+        self.create_from_cacert(paths.IPA_CA_CRT)
+        if pwdconf:
+            self.create_password_conf()
+        reqid = certmonger.request_cert(nssdb=self.secdir,
+                                        nickname=nickname,
+                                        principal=principal,
+                                        subject=host,
+                                        passwd_fname=self.passwd_fname)
+        # Now wait for the cert to appear. Check three times then abort
+        certmonger.wait_for_request(reqid, timeout=15)
+
 
 class _CrossProcessLock(object):
     _DATETIME_FORMAT = '%Y%m%d%H%M%S%f'
