@@ -2,10 +2,8 @@
 # Copyright (C) 2014  FreeIPA Contributors see COPYING for license
 #
 
-from ipapython.dnsutil import DNSName
+from __future__ import print_function
 
-import service
-import installutils
 import os
 import pwd
 import grp
@@ -14,13 +12,17 @@ import shutil
 import stat
 
 import ldap
-import _ipap11helper
 
+from ipapython import p11helper as _ipap11helper
+from ipapython.dnsutil import DNSName
+from ipaserver.install import service
+from ipaserver.install import installutils
 from ipapython.ipa_log_manager import *
 from ipapython.dn import DN
 from ipapython import ipaldap
 from ipapython import sysrestore, ipautil
 from ipaplatform import services
+from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipalib import errors, api
 from ipalib.constants import CACERT
@@ -30,12 +32,6 @@ softhsm_token_label = u'ipaDNSSEC'
 softhsm_slot = 0
 replica_keylabel_template = u"dnssec-replica:%s"
 
-def check_inst():
-    if not os.path.exists(paths.DNSSEC_KEYFROMLABEL):
-        print ("Please install the 'bind-pkcs11-utils' package and start "
-               "the installation again")
-        return False
-    return True
 
 def dnssec_container_exists(fqdn, suffix, dm_password=None, ldapi=False,
                             realm=None, autobind=ipaldap.AUTOBIND_DISABLED):
@@ -95,9 +91,9 @@ class DNSKeySyncInstance(service.Service):
             self.named_gid = self.__get_named_gid()
 
         if not os.path.exists(paths.BIND_LDAP_DNS_IPA_WORKDIR):
-            os.mkdir(paths.BIND_LDAP_DNS_IPA_WORKDIR, 0770)
+            os.mkdir(paths.BIND_LDAP_DNS_IPA_WORKDIR, 0o770)
         # dnssec daemons require to have access into the directory
-        os.chmod(paths.BIND_LDAP_DNS_IPA_WORKDIR, 0770)
+        os.chmod(paths.BIND_LDAP_DNS_IPA_WORKDIR, 0o770)
         os.chown(paths.BIND_LDAP_DNS_IPA_WORKDIR, self.named_uid,
                  self.named_gid)
 
@@ -118,15 +114,13 @@ class DNSKeySyncInstance(service.Service):
             ldap.delete_entry(entry)
 
     def start_dnskeysyncd(self):
-        print "Restarting ipa-dnskeysyncd"
+        print("Restarting ipa-dnskeysyncd")
         self.__start()
 
     def create_instance(self, fqdn, realm_name):
         self.fqdn = fqdn
         self.realm = realm_name
         self.suffix = ipautil.realm_to_suffix(self.realm)
-        self.backup_state("enabled", self.is_enabled())
-        self.backup_state("running", self.is_running())
         try:
             self.stop()
         except:
@@ -149,14 +143,14 @@ class DNSKeySyncInstance(service.Service):
     def __get_named_uid(self):
         named = services.knownservices.named
         try:
-            return pwd.getpwnam(named.get_user_name()).pw_uid
+            return pwd.getpwnam(constants.NAMED_USER).pw_uid
         except KeyError:
             raise RuntimeError("Named UID not found")
 
     def __get_named_gid(self):
         named = services.knownservices.named
         try:
-            return grp.getgrnam(named.get_group_name()).gr_gid
+            return grp.getgrnam(constants.NAMED_GROUP).gr_gid
         except KeyError:
             raise RuntimeError("Named GID not found")
 
@@ -167,12 +161,12 @@ class DNSKeySyncInstance(service.Service):
         self.named_gid = self.__get_named_gid()
 
         try:
-            self.ods_uid = pwd.getpwnam(ods_enforcerd.get_user_name()).pw_uid
+            self.ods_uid = pwd.getpwnam(constants.ODS_USER).pw_uid
         except KeyError:
             raise RuntimeError("OpenDNSSEC UID not found")
 
         try:
-            self.ods_gid = grp.getgrnam(ods_enforcerd.get_group_name()).gr_gid
+            self.ods_gid = grp.getgrnam(constants.ODS_GROUP).gr_gid
         except KeyError:
             raise RuntimeError("OpenDNSSEC GID not found")
 
@@ -182,6 +176,9 @@ class DNSKeySyncInstance(service.Service):
         ):
             raise RuntimeError("DNS container does not exist")
 
+        # ready to be installed, storing a state is required to run uninstall
+        self.backup_state("configured", True)
+
     def __setup_dnssec_containers(self):
         """
         Setup LDAP containers for DNSSEC
@@ -205,7 +202,8 @@ class DNSKeySyncInstance(service.Service):
         # create dnssec directory
         if not os.path.exists(paths.IPA_DNSSEC_DIR):
             self.logger.debug("Creating %s directory", paths.IPA_DNSSEC_DIR)
-            os.mkdir(paths.IPA_DNSSEC_DIR, 0770)
+            os.mkdir(paths.IPA_DNSSEC_DIR)
+            os.chmod(paths.IPA_DNSSEC_DIR, 0o770)
             # chown ods:named
             os.chown(paths.IPA_DNSSEC_DIR, self.ods_uid, self.named_gid)
 
@@ -222,6 +220,7 @@ class DNSKeySyncInstance(service.Service):
         named_fd.truncate(0)
         named_fd.write(softhsm_conf_txt)
         named_fd.close()
+        os.chmod(paths.DNSSEC_SOFTHSM2_CONF, 0o644)
 
         # setting up named to use softhsm2
         if not self.fstore.has_file(paths.SYSCONFIG_NAMED):
@@ -250,7 +249,7 @@ class DNSKeySyncInstance(service.Service):
                           paths.DNSSEC_TOKENS_DIR)
         # sticky bit is required by daemon
         os.mkdir(paths.DNSSEC_TOKENS_DIR)
-        os.chmod(paths.DNSSEC_TOKENS_DIR, 0770 | stat.S_ISGID)
+        os.chmod(paths.DNSSEC_TOKENS_DIR, 0o770 | stat.S_ISGID)
         # chown to ods:named
         os.chown(paths.DNSSEC_TOKENS_DIR, self.ods_uid, self.named_gid)
 
@@ -266,7 +265,7 @@ class DNSKeySyncInstance(service.Service):
         named_fd.truncate(0)
         named_fd.write(pin)
         named_fd.close()
-        os.chmod(paths.DNSSEC_SOFTHSM_PIN, 0770)
+        os.chmod(paths.DNSSEC_SOFTHSM_PIN, 0o770)
         # chown to ods:named
         os.chown(paths.DNSSEC_SOFTHSM_PIN, self.ods_uid, self.named_gid)
 
@@ -277,7 +276,7 @@ class DNSKeySyncInstance(service.Service):
         named_fd.write(pin_so)
         named_fd.close()
         # owner must be root
-        os.chmod(paths.DNSSEC_SOFTHSM_PIN_SO, 0400)
+        os.chmod(paths.DNSSEC_SOFTHSM_PIN_SO, 0o400)
 
         # initialize SoftHSM
 
@@ -314,7 +313,7 @@ class DNSKeySyncInstance(service.Service):
                 # check if key with this ID exist in softHSM
                 # id is 16 Bytes long
                 key_id = "".join(chr(random.randint(0, 255))
-                                 for _ in xrange(0, 16))
+                                 for _ in range(0, 16))
                 replica_pubkey_dn = DN(('ipk11UniqueId', 'autogenerate'), dn_base)
 
 
@@ -403,12 +402,12 @@ class DNSKeySyncInstance(service.Service):
         for (root, dirs, files) in os.walk(paths.DNSSEC_TOKENS_DIR):
             for directory in dirs:
                 dir_path = os.path.join(root, directory)
-                os.chmod(dir_path, 0770 | stat.S_ISGID)
+                os.chmod(dir_path, 0o770 | stat.S_ISGID)
                 # chown to ods:named
                 os.chown(dir_path, self.ods_uid, self.named_gid)
             for filename in files:
                 file_path = os.path.join(root, filename)
-                os.chmod(file_path, 0770 | stat.S_ISGID)
+                os.chmod(file_path, 0o770 | stat.S_ISGID)
                 # chown to ods:named
                 os.chown(file_path, self.ods_uid, self.named_gid)
 
@@ -418,10 +417,10 @@ class DNSKeySyncInstance(service.Service):
                              self.suffix, self.extra_config)
         except errors.DuplicateEntry:
             self.logger.error("DNSKeySync service already exists")
-        self.enable()
 
     def __setup_principal(self):
         assert self.ods_gid is not None
+        installutils.remove_keytab(paths.IPA_DNSKEYSYNCD_KEYTAB)
         dnssynckey_principal = "ipa-dnskeysyncd/" + self.fqdn + "@" + self.realm
         installutils.kadmin_addprinc(dnssynckey_principal)
 
@@ -438,7 +437,7 @@ class DNSKeySyncInstance(service.Service):
 
         # Make sure access is strictly reserved to the named user
         os.chown(paths.IPA_DNSKEYSYNCD_KEYTAB, 0, self.ods_gid)
-        os.chmod(paths.IPA_DNSKEYSYNCD_KEYTAB, 0440)
+        os.chmod(paths.IPA_DNSKEYSYNCD_KEYTAB, 0o440)
 
         dns_group = DN(('cn', 'DNS Servers'), ('cn', 'privileges'),
                        ('cn', 'pbac'), self.suffix)
@@ -448,7 +447,7 @@ class DNSKeySyncInstance(service.Service):
             self.admin_conn.modify_s(dns_group, mod)
         except ldap.TYPE_OR_VALUE_EXISTS:
             pass
-        except Exception, e:
+        except Exception as e:
             self.logger.critical("Could not modify principal's %s entry: %s"
                                  % (dnssynckey_principal_dn, str(e)))
             raise
@@ -462,7 +461,7 @@ class DNSKeySyncInstance(service.Service):
                (ldap.MOD_REPLACE, 'nsLookThroughLimit', '-1')]
         try:
             self.admin_conn.modify_s(dnssynckey_principal_dn, mod)
-        except Exception, e:
+        except Exception as e:
             self.logger.critical("Could not set principal's %s LDAP limits: %s"
                                  % (dnssynckey_principal_dn, str(e)))
             raise
@@ -471,26 +470,27 @@ class DNSKeySyncInstance(service.Service):
         try:
             self.restart()
         except Exception as e:
-            print "Failed to start ipa-dnskeysyncd"
+            print("Failed to start ipa-dnskeysyncd")
             self.logger.debug("Failed to start ipa-dnskeysyncd: %s", e)
 
 
     def uninstall(self):
-        if not self.is_configured():
-            return
+        if self.is_configured():
+            self.print_msg("Unconfiguring %s" % self.service_name)
 
-        self.print_msg("Unconfiguring %s" % self.service_name)
+        # Just eat states
+        self.restore_state("running")
+        self.restore_state("enabled")
+        self.restore_state("configured")
 
-        running = self.restore_state("running")
-        enabled = self.restore_state("enabled")
-
-        if running is not None:
-            self.stop()
+        # stop and disable service (IPA service, we do not need it anymore)
+        self.stop()
+        self.disable()
 
         for f in [paths.SYSCONFIG_NAMED]:
             try:
                 self.fstore.restore_file(f)
-            except ValueError, error:
+            except ValueError as error:
                 self.logger.debug(error)
                 pass
 
@@ -502,8 +502,4 @@ class DNSKeySyncInstance(service.Service):
         except Exception:
             pass
 
-        if enabled is not None and not enabled:
-            self.disable()
-
-        if running is not None and running:
-            self.start()
+        installutils.remove_keytab(paths.IPA_DNSKEYSYNCD_KEYTAB)
