@@ -18,6 +18,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import print_function
+
 import shutil
 import pwd
 import sys
@@ -27,27 +29,30 @@ import time
 import tempfile
 import stat
 
+import ldap
+
 from ipapython.ipa_log_manager import *
 from ipapython import ipautil, sysrestore, ipaldap
-import service
-import installutils
-import certs
-import ldap
+from ipaserver.install import service
+from ipaserver.install import installutils
+from ipaserver.install import certs
 from ipaserver.install import ldapupdate
 from ipaserver.install import replication
 from ipaserver.install import sysupgrade
-from ipalib import errors, certstore
+from ipaserver.install import upgradeinstance
+from ipalib import api
+from ipalib import certstore
+from ipalib import errors
+from ipalib import constants
+from ipaplatform.constants import constants as platformconstants
 from ipaplatform.tasks import tasks
 from ipalib.constants import CACERT
 from ipapython.dn import DN
 from ipaplatform import services
 from ipaplatform.paths import paths
 
-SERVER_ROOT_64 = paths.USR_LIB_DIRSRV_64
-SERVER_ROOT_32 = paths.USR_LIB_DIRSRV
-
-DS_USER = 'dirsrv'
-DS_GROUP = 'dirsrv'
+DS_USER = platformconstants.DS_USER
+DS_GROUP = platformconstants.DS_GROUP
 
 IPA_SCHEMA_FILES = ("60kerberos.ldif",
                     "60samba.ldif",
@@ -56,25 +61,26 @@ IPA_SCHEMA_FILES = ("60kerberos.ldif",
                     "60basev3.ldif",
                     "60ipapk11.ldif",
                     "60ipadns.ldif",
+                    "60certificate-profiles.ldif",
                     "61kerberos-ipav3.ldif",
                     "65ipacertstore.ldif",
                     "65ipasudo.ldif",
                     "70ipaotp.ldif",
+                    "70topology.ldif",
                     "71idviews.ldif",
+                    "72domainlevels.ldif",
                     "15rfc2307bis.ldif",
                     "15rfc4876.ldif")
 
 ALL_SCHEMA_FILES = IPA_SCHEMA_FILES + ("05rfc2247.ldif", )
+DS_INSTANCE_PREFIX = 'slapd-'
 
 
 def find_server_root():
-    if ipautil.dir_exists(SERVER_ROOT_64):
-        return SERVER_ROOT_64
+    if ipautil.dir_exists(paths.USR_LIB_DIRSRV_64):
+        return paths.USR_LIB_DIRSRV_64
     else:
-        return SERVER_ROOT_32
-
-def realm_to_serverid(realm_name):
-    return "-".join(realm_name.split("."))
+        return paths.USR_LIB_DIRSRV
 
 def config_dirname(serverid):
     return (paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % serverid) + "/"
@@ -82,29 +88,29 @@ def config_dirname(serverid):
 def schema_dirname(serverid):
     return config_dirname(serverid) + "/schema/"
 
-def erase_ds_instance_data(serverid):
-    installutils.rmtree(paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % serverid)
 
-    installutils.rmtree(paths.USR_LIB_SLAPD_INSTANCE_TEMPLATE % serverid)
+def remove_ds_instance(serverid, force=False):
+    """A wrapper around the 'remove-ds.pl' script used by
+    389ds to remove a single directory server instance. In case of error
+    additional call with the '-f' flag is performed (forced removal). If this
+    also fails, then an exception is raised.
+    """
+    instance_name = ''.join([DS_INSTANCE_PREFIX, serverid])
+    args = [paths.REMOVE_DS_PL, '-i', instance_name]
+    if force:
+        args.append('-f')
+        root_logger.debug("Forcing instance removal")
 
-    installutils.rmtree(paths.USR_LIB_DIRSRV_SLAPD_INSTANCE_DIR_TEMPLATE % serverid)
+    try:
+        ipautil.run(args)
+    except ipautil.CalledProcessError:
+        if force:
+            root_logger.error("Instance removal failed.")
+            raise
+        root_logger.debug("'%s' failed. "
+                          "Attempting to force removal" % paths.REMOVE_DS_PL)
+        remove_ds_instance(serverid, force=True)
 
-    installutils.rmtree(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % serverid)
-
-    installutils.rmtree(paths.SLAPD_INSTANCE_LOCK_TEMPLATE % serverid)
-
-    installutils.remove_file(paths.SLAPD_INSTANCE_SOCKET_TEMPLATE % serverid)
-
-    installutils.rmtree(paths.VAR_LIB_DIRSRV_INSTANCE_SCRIPTS_TEMPLATE % serverid)
-
-    installutils.remove_file(paths.DS_KEYTAB)
-
-    installutils.remove_file(paths.SYSCONFIG_DIRSRV_INSTANCE % serverid)
-
-#    try:
-#        shutil.rmtree(paths.VAR_LOG_DIRSRV_INSTANCE_TEMPLATE % serverid)
-#    except:
-#        pass
 
 def get_ds_instances():
     '''
@@ -114,8 +120,7 @@ def get_ds_instances():
     matches 389ds behavior.
     '''
 
-    dirsrv_instance_dir=paths.ETC_DIRSRV
-    instance_prefix = 'slapd-'
+    dirsrv_instance_dir = paths.ETC_DIRSRV
 
     instances = []
 
@@ -124,9 +129,10 @@ def get_ds_instances():
         # Must be a directory
         if os.path.isdir(pathname):
             # Must start with prefix and not end with .removed
-            if basename.startswith(instance_prefix) and not basename.endswith('.removed'):
+            if (basename.startswith(DS_INSTANCE_PREFIX) and
+                    not basename.endswith('.removed')):
                 # Strip off prefix
-                instance = basename[len(instance_prefix):]
+                instance = basename[len(DS_INSTANCE_PREFIX):]
                 # Must be non-empty
                 if instance:
                     instances.append(instance)
@@ -160,6 +166,20 @@ def create_ds_user():
     )
 
 
+def get_domain_level(api=api):
+    conn = ipaldap.IPAdmin(ldapi=True, realm=api.env.realm)
+    conn.do_external_bind('root')
+
+    dn = DN(('cn', 'Domain Level'),
+            ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
+
+    try:
+        entry = conn.get_entry(dn, ['ipaDomainLevel'])
+    except errors.NotFound:
+        return constants.DOMAIN_LEVEL_0
+    return int(entry.single_value['ipaDomainLevel'])
+
+
 INF_TEMPLATE = """
 [General]
 FullMachineName=   $FQDN
@@ -187,7 +207,7 @@ info: IPA V2.0
 
 class DsInstance(service.Service):
     def __init__(self, realm_name=None, domain_name=None, dm_password=None,
-                 fstore=None):
+                 fstore=None, domainlevel=None, config_ldif=None):
         service.Service.__init__(self, "dirsrv",
             service_desc="directory server",
             dm_password=dm_password,
@@ -210,6 +230,8 @@ class DsInstance(service.Service):
         self.subject_base = None
         self.open_ports = []
         self.run_init_memberof = True
+        self.config_ldif = config_ldif  # updates for dse.ldif
+        self.domainlevel = domainlevel
         if realm_name:
             self.suffix = ipautil.realm_to_suffix(self.realm)
             self.__setup_sub_dict()
@@ -228,6 +250,9 @@ class DsInstance(service.Service):
 
         self.step("creating directory server user", create_ds_user)
         self.step("creating directory server instance", self.__create_instance)
+        if self.config_ldif:
+            self.step("updating configuration in dse.ldif", self.__update_dse_ldif)
+        self.step("restarting directory server", self.__restart_instance)
         self.step("adding default schema", self.__add_default_schemas)
         self.step("enabling memberof plugin", self.__add_memberof_module)
         self.step("enabling winsync plugin", self.__add_winsync_module)
@@ -240,6 +265,7 @@ class DsInstance(service.Service):
         self.step("configuring DNS plugin", self.__config_dns_module)
         self.step("enabling entryUSN plugin", self.__enable_entryusn)
         self.step("configuring lockout plugin", self.__config_lockout_module)
+        self.step("configuring topology plugin", self.__config_topology_module)
         self.step("creating indices", self.__create_indices)
         self.step("enabling referential integrity plugin", self.__add_referint_module)
         if enable_ssl:
@@ -248,17 +274,20 @@ class DsInstance(service.Service):
         self.step("configure autobind for root", self.__root_autobind)
         self.step("configure new location for managed entries", self.__repoint_managed_entries)
         self.step("configure dirsrv ccache", self.configure_dirsrv_ccache)
-        self.step("enable SASL mapping fallback", self.__enable_sasl_mapping_fallback)
-        self.step("restarting directory server", self.__restart_instance)
+        self.step("enabling SASL mapping fallback",
+                  self.__enable_sasl_mapping_fallback)
 
     def __common_post_setup(self):
         self.step("initializing group membership", self.init_memberof)
         self.step("adding master entry", self.__add_master_entry)
+        self.step("initializing domain level", self.__set_domain_level)
         self.step("configuring Posix uid/gid generation",
                   self.__config_uidgid_gen)
         self.step("adding replication acis", self.__add_replication_acis)
         self.step("enabling compatibility plugin",
                   self.__enable_compat_plugin)
+        self.step("activating sidgen plugin", self._add_sidgen_plugin)
+        self.step("activating extdom plugin", self._add_extdom_plugin)
         self.step("tuning directory server", self.__tuning)
 
         self.step("configuring directory to start on boot", self.__enable)
@@ -266,7 +295,7 @@ class DsInstance(service.Service):
     def init_info(self, realm_name, fqdn, domain_name, dm_password,
                   subject_base, idstart, idmax, pkcs12_info, ca_file=None):
         self.realm = realm_name.upper()
-        self.serverid = realm_to_serverid(self.realm)
+        self.serverid = installutils.realm_to_serverid(self.realm)
         self.suffix = ipautil.realm_to_suffix(self.realm)
         self.fqdn = fqdn
         self.dm_password = dm_password
@@ -291,7 +320,9 @@ class DsInstance(service.Service):
             subject_base, idstart, idmax, pkcs12_info, ca_file=ca_file)
 
         self.__common_setup()
+        self.step("restarting directory server", self.__restart_instance)
 
+        self.step("adding sasl mappings to the directory", self.__configure_sasl_mappings)
         self.step("adding default layout", self.__add_default_layout)
         self.step("adding delegation layout", self.__add_delegation_layout)
         self.step("creating container for managed entries", self.__managed_entries)
@@ -302,6 +333,9 @@ class DsInstance(service.Service):
         self.step("adding range check plugin", self.__add_range_check_plugin)
         if hbac_allow:
             self.step("creating default HBAC rule allow_all", self.add_hbac)
+        self.step("adding sasl mappings to the directory",
+                  self.__configure_sasl_mappings)
+        self.step("adding entries for topology management", self.__add_topology_entries)
 
         self.__common_post_setup()
 
@@ -318,7 +352,8 @@ class DsInstance(service.Service):
 
     def create_replica(self, realm_name, master_fqdn, fqdn,
                        domain_name, dm_password, subject_base,
-                       pkcs12_info=None, ca_file=None, ca_is_configured=None):
+                       pkcs12_info=None, ca_file=None,
+                       ca_is_configured=None, promote=False):
         # idstart and idmax are configured so that the range is seen as
         # depleted by the DNA plugin and the replica will go and get a
         # new range from the master.
@@ -340,10 +375,19 @@ class DsInstance(service.Service):
         self.master_fqdn = master_fqdn
         if ca_is_configured is not None:
             self.ca_is_configured = ca_is_configured
+        self.promote = promote
 
-        self.__common_setup(True)
+        self.__common_setup(enable_ssl=(not self.promote))
+        self.step("restarting directory server", self.__restart_instance)
+
+        if self.promote:
+            self.step("creating DS keytab", self.__get_ds_keytab)
+            if self.ca_is_configured:
+                self.step("retrieving DS Certificate", self.__get_ds_cert)
+            self.step("restarting directory server", self.__restart_instance)
 
         self.step("setting up initial replication", self.__setup_replica)
+        self.step("adding sasl mappings to the directory", self.__configure_sasl_mappings)
         self.step("updating schema", self.__update_schema)
         # See LDIFs for automember configuration during replica install
         self.step("setting Auto Member configuration", self.__add_replica_automember_config)
@@ -360,14 +404,75 @@ class DsInstance(service.Service):
             self.realm,
             self.dm_password)
 
+        # Always connect to self over ldapi
+        conn = ipaldap.IPAdmin(self.fqdn, ldapi=True, realm=self.realm)
+        conn.do_external_bind('root')
         repl = replication.ReplicationManager(self.realm,
                                               self.fqdn,
-                                              self.dm_password)
-        repl.setup_replication(self.master_fqdn,
-                               r_binddn=DN(('cn', 'Directory Manager')),
-                               r_bindpw=self.dm_password)
+                                              self.dm_password, conn=conn)
+        if self.promote:
+            repl.setup_promote_replication(self.master_fqdn)
+        else:
+            repl.setup_replication(self.master_fqdn,
+                                   r_binddn=DN(('cn', 'Directory Manager')),
+                                   r_bindpw=self.dm_password)
         self.run_init_memberof = repl.needs_memberof_fixup()
 
+        # Now that the server is up make sure all changes happen against
+        # the local server (as repica pomotion does not have the DM password.
+        if self.admin_conn:
+            self.ldap_disconnect()
+        self.ldapi = True
+
+    def __configure_sasl_mappings(self):
+        # we need to remove any existing SASL mappings in the directory as otherwise they
+        # they may conflict.
+
+        if not self.admin_conn:
+            self.ldap_connect()
+
+        try:
+            res = self.admin_conn.get_entries(
+                DN(('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')),
+                self.admin_conn.SCOPE_ONELEVEL,
+                "(objectclass=nsSaslMapping)")
+            for r in res:
+                try:
+                    self.admin_conn.delete_entry(r)
+                except Exception as e:
+                    root_logger.critical(
+                        "Error during SASL mapping removal: %s", e)
+                    raise
+        except Exception as e:
+            root_logger.critical("Error while enumerating SASL mappings %s", e)
+            raise
+
+        entry = self.admin_conn.make_entry(
+            DN(
+                ('cn', 'Full Principal'), ('cn', 'mapping'), ('cn', 'sasl'),
+                ('cn', 'config')),
+            objectclass=["top", "nsSaslMapping"],
+            cn=["Full Principal"],
+            nsSaslMapRegexString=['\(.*\)@\(.*\)'],
+            nsSaslMapBaseDNTemplate=[self.suffix],
+            nsSaslMapFilterTemplate=['(krbPrincipalName=\\1@\\2)'],
+            nsSaslMapPriority=['10'],
+        )
+        self.admin_conn.add_entry(entry)
+
+        entry = self.admin_conn.make_entry(
+            DN(
+                ('cn', 'Name Only'), ('cn', 'mapping'), ('cn', 'sasl'),
+                ('cn', 'config')),
+            objectclass=["top", "nsSaslMapping"],
+            cn=["Name Only"],
+            nsSaslMapRegexString=['^[^:@]+$'],
+            nsSaslMapBaseDNTemplate=[self.suffix],
+            nsSaslMapFilterTemplate=['(krbPrincipalName=&@%s)' % self.realm],
+            nsSaslMapPriority=['10'],
+        )
+        self.admin_conn.add_entry(entry)
+
     def __update_schema(self):
         # FIXME: https://fedorahosted.org/389/ticket/47490
         self._ldap_mod("schema-update.ldif")
@@ -394,7 +499,15 @@ class DsInstance(service.Service):
                              IDMAX=self.idmax, HOST=self.fqdn,
                              ESCAPED_SUFFIX=str(self.suffix),
                              GROUP=DS_GROUP,
-                             IDRANGE_SIZE=idrange_size
+                             IDRANGE_SIZE=idrange_size,
+                             DOMAIN_LEVEL=self.domainlevel,
+                             MAX_DOMAIN_LEVEL=constants.MAX_DOMAIN_LEVEL,
+                             MIN_DOMAIN_LEVEL=constants.MIN_DOMAIN_LEVEL,
+                             STRIP_ATTRS=" ".join(replication.STRIP_ATTRS),
+                             EXCLUDES='(objectclass=*) $ EXCLUDE ' +
+                             ' '.join(replication.EXCLUDES),
+                             TOTAL_EXCLUDES='(objectclass=*) $ EXCLUDE ' +
+                             ' '.join(replication.TOTAL_EXCLUDES),
                          )
 
     def __create_instance(self):
@@ -413,7 +526,7 @@ class DsInstance(service.Service):
         base_fd.close()
 
         # Must be readable for dirsrv
-        os.chmod(target_fname, 0440)
+        os.chmod(target_fname, 0o440)
         os.chown(target_fname, pent.pw_uid, pent.pw_gid)
 
         inf_txt = ipautil.template_str(INF_TEMPLATE, self.sub_dict)
@@ -430,28 +543,51 @@ class DsInstance(service.Service):
         try:
             ipautil.run(args)
             root_logger.debug("completed creating ds instance")
-        except ipautil.CalledProcessError, e:
-            root_logger.critical("failed to create ds instance %s" % e)
+        except ipautil.CalledProcessError as e:
+            raise RuntimeError("failed to create ds instance %s" % e)
 
         # check for open port 389 from now on
         self.open_ports.append(389)
 
-        root_logger.debug("restarting ds instance")
-        try:
-            self.__restart_instance()
-            root_logger.debug("done restarting ds instance")
-        except ipautil.CalledProcessError, e:
-            print "failed to restart ds instance", e
-            root_logger.debug("failed to restart ds instance %s" % e)
         inf_fd.close()
         os.remove(paths.DIRSRV_BOOT_LDIF)
 
+    def __update_dse_ldif(self):
+        """
+        This method updates dse.ldif right after instance creation. This is
+        supposed to allow admin modify configuration of the DS which has to be
+        done before IPA is fully installed (for example: settings for
+        replication on replicas)
+        DS must be turned off.
+        """
+        self.stop()
+
+        dse_filename = os.path.join(
+            paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % self.serverid,
+            'dse.ldif'
+        )
+
+        with tempfile.NamedTemporaryFile(delete=False) as new_dse_ldif:
+            temp_filename = new_dse_ldif.name
+            with open(dse_filename, "r") as input_file:
+                parser = installutils.ModifyLDIF(input_file, new_dse_ldif)
+                # parse modification from config ldif
+                with open(self.config_ldif, "r") as config_ldif:
+                    parser.modifications_from_ldif(config_ldif)
+                parser.parse()
+            new_dse_ldif.flush()
+        shutil.copy2(temp_filename, dse_filename)
+        try:
+            os.remove(temp_filename)
+        except OSError as e:
+            root_logger.debug("Failed to clean temporary file: %s" % e)
+
     def __add_default_schemas(self):
         pent = pwd.getpwnam(DS_USER)
         for schema_fname in IPA_SCHEMA_FILES:
             target_fname = schema_dirname(self.serverid) + schema_fname
             shutil.copyfile(ipautil.SHARE_DIR + schema_fname, target_fname)
-            os.chmod(target_fname, 0440)    # read access for dirsrv user/group
+            os.chmod(target_fname, 0o440)    # read access for dirsrv user/group
             os.chown(target_fname, pent.pw_uid, pent.pw_gid)
 
         try:
@@ -460,21 +596,28 @@ class DsInstance(service.Service):
 
             target_fname = schema_dirname(self.serverid) + "05rfc2247.ldif"
             shutil.copyfile(ipautil.SHARE_DIR + "05rfc2247.ldif", target_fname)
-            os.chmod(target_fname, 0440)
+            os.chmod(target_fname, 0o440)
             os.chown(target_fname, pent.pw_uid, pent.pw_gid)
         except IOError:
             # Does not apply with newer DS releases
             pass
 
+    def stop(self, *args, **kwargs):
+        if self.admin_conn:
+            self.ldap_disconnect()
+        super(DsInstance, self).stop(*args, **kwargs)
+
     def restart(self, instance=''):
+        if self.admin_conn:
+            self.ldap_disconnect()
         try:
             super(DsInstance, self).restart(instance)
             if not is_ds_running(instance):
                 root_logger.critical("Failed to restart the directory server. See the installation log for details.")
                 sys.exit(1)
-        except SystemExit, e:
+        except SystemExit as e:
             raise e
-        except Exception, e:
+        except Exception as e:
             # TODO: roll back here?
             root_logger.critical("Failed to restart the directory server (%s). See the installation log for details." % e)
 
@@ -507,9 +650,14 @@ class DsInstance(service.Service):
         conn.unbind()
 
     def apply_updates(self):
-        ld = ldapupdate.LDAPUpdate(dm_password=self.dm_password, sub_dict=self.sub_dict, plugins=True)
-        files = ld.get_all_files(ldapupdate.UPDATES_DIR)
-        ld.update(files, ordered=True)
+        data_upgrade = upgradeinstance.IPAUpgrade(self.realm)
+        try:
+            data_upgrade.create_instance()
+        except Exception as e:
+            # very fatal errors only will raise exception
+            raise RuntimeError("Update failed: %s" % e)
+        installutils.store_version()
+
 
     def __add_referint_module(self):
         self._ldap_mod("referint-conf.ldif")
@@ -523,6 +671,9 @@ class DsInstance(service.Service):
     def __add_master_entry(self):
         self._ldap_mod("master-entry.ldif", self.sub_dict)
 
+    def __add_topology_entries(self):
+        self._ldap_mod("topology-entries.ldif", self.sub_dict)
+
     def __add_winsync_module(self):
         self._ldap_mod("ipa-winsync-conf.ldif")
 
@@ -551,11 +702,14 @@ class DsInstance(service.Service):
     def __config_lockout_module(self):
         self._ldap_mod("lockout-conf.ldif")
 
+    def __config_topology_module(self):
+        self._ldap_mod("ipa-topology-conf.ldif", self.sub_dict)
+
     def __repoint_managed_entries(self):
         self._ldap_mod("repoint-managed-entries.ldif", self.sub_dict)
 
     def configure_dirsrv_ccache(self):
-        pent = pwd.getpwnam("dirsrv")
+        pent = pwd.getpwnam(platformconstants.DS_USER)
         ccache = paths.TMP_KRB5CC % pent.pw_uid
         filepath = paths.SYSCONFIG_DIRSRV
         if not os.path.exists(filepath):
@@ -624,7 +778,7 @@ class DsInstance(service.Service):
         conn.do_simple_bind(DN(('cn', 'directory manager')), self.dm_password)
 
         mod = [(ldap.MOD_REPLACE, "nsSSLClientAuth", "allowed"),
-               (ldap.MOD_REPLACE, "nsSSL3Ciphers", "+all"),
+               (ldap.MOD_REPLACE, "nsSSL3Ciphers", "default"),
                (ldap.MOD_REPLACE, "allowWeakCipher", "off")]
         conn.modify_s(DN(('cn', 'encryption'), ('cn', 'config')), mod)
 
@@ -745,8 +899,8 @@ class DsInstance(service.Service):
                         'LDAPTLS_CACERT':CACERT }
                 ipautil.run(args, env=env)
                 root_logger.debug("ldappasswd done")
-            except ipautil.CalledProcessError, e:
-                print "Unable to set admin password", e
+            except ipautil.CalledProcessError as e:
+                print("Unable to set admin password", e)
                 root_logger.debug("Unable to set admin password %s" % e)
 
         finally:
@@ -767,17 +921,25 @@ class DsInstance(service.Service):
         try:
             self.fstore.restore_file(paths.LIMITS_CONF)
             self.fstore.restore_file(paths.SYSCONFIG_DIRSRV)
-        except ValueError, error:
+        except ValueError as error:
             root_logger.debug(error)
             pass
 
-        if not enabled is None and not enabled:
-            self.disable()
+        # disabled during IPA installation
+        if enabled:
+            self.enable()
 
         serverid = self.restore_state("serverid")
-        if not serverid is None:
+        if serverid is not None:
             self.stop_tracking_certificates(serverid)
-            erase_ds_instance_data(serverid)
+            root_logger.debug("Removing DS instance %s" % serverid)
+            try:
+                remove_ds_instance(serverid)
+                installutils.remove_keytab(paths.DS_KEYTAB)
+                installutils.remove_ccache(run_as=DS_USER)
+            except ipautil.CalledProcessError:
+                root_logger.error("Failed to remove DS instance. You may "
+                                  "need to remove instance data manually")
 
         # At one time we removed this user on uninstall. That can potentially
         # orphan files, or worse, if another useradd runs in the intermim,
@@ -795,7 +957,7 @@ class DsInstance(service.Service):
         for ds_instance in get_ds_instances():
             try:
                 services.knownservices.dirsrv.restart(ds_instance, wait=False)
-            except Exception, e:
+            except Exception as e:
                 root_logger.error('Unable to restart ds instance %s: %s', ds_instance, e)
 
     def stop_tracking_certificates(self, serverid=None):
@@ -808,6 +970,13 @@ class DsInstance(service.Service):
             dsdb = certs.CertDB(self.realm, nssdir=dirname)
             dsdb.untrack_server_cert(self.nickname)
 
+    def start_tracking_certificates(self, serverid):
+        dirname = config_dirname(serverid)[:-1]
+        dsdb = certs.CertDB(self.realm, nssdir=dirname)
+        dsdb.track_server_cert(self.nickname, self.principal,
+                               dsdb.passwd_fname,
+                               'restart_dirsrv %s' % serverid)
+
     # we could probably move this function into the service.Service
     # class - it's very generic - all we need is a way to get an
     # instance of a particular Service
@@ -825,7 +994,7 @@ class DsInstance(service.Service):
                 root_logger.critical("The given CA cert file named [%s] could not be read" %
                                              cacert_fname)
                 return False
-        except OSError, e:
+        except OSError as e:
             root_logger.critical("The given CA cert file named [%s] could not be read: %s" %
                                          (cacert_fname, str(e)))
             return False
@@ -833,7 +1002,7 @@ class DsInstance(service.Service):
         # shutdown the server
         self.stop()
 
-        dirname = config_dirname(realm_to_serverid(self.realm))
+        dirname = config_dirname(installutils.realm_to_serverid(self.realm))
         certdb = certs.CertDB(self.realm, nssdir=dirname, subject_base=self.subject_base)
         if not cacert_name or len(cacert_name) == 0:
             cacert_name = "Imported CA"
@@ -842,7 +1011,7 @@ class DsInstance(service.Service):
         status = True
         try:
             certdb.load_cacert(cacert_fname, 'C,,')
-        except ipautil.CalledProcessError, e:
+        except ipautil.CalledProcessError as e:
             root_logger.critical("Error importing CA cert file named [%s]: %s" %
                                          (cacert_fname, str(e)))
             status = False
@@ -890,6 +1059,48 @@ class DsInstance(service.Service):
     def __add_range_check_plugin(self):
         self._ldap_mod("range-check-conf.ldif", self.sub_dict)
 
+    def _add_sidgen_plugin(self):
+        """
+        Add sidgen directory server plugin configuration if it does not already exist.
+        """
+        self.add_sidgen_plugin(self.sub_dict['SUFFIX'])
+
+    def add_sidgen_plugin(self, suffix):
+        """
+        Add sidgen plugin configuration only if it does not already exist.
+        """
+        if not self.admin_conn:
+            self.ldap_connect()
+
+        dn = DN('cn=IPA SIDGEN,cn=plugins,cn=config')
+        try:
+            self.admin_conn.get_entry(dn)
+        except errors.NotFound:
+            self._ldap_mod('ipa-sidgen-conf.ldif', dict(SUFFIX=suffix))
+        else:
+            root_logger.debug("sidgen plugin is already configured")
+
+    def _add_extdom_plugin(self):
+        """
+        Add directory server configuration for the extdom extended operation.
+        """
+        self.add_extdom_plugin(self.sub_dict['SUFFIX'])
+
+    def add_extdom_plugin(self, suffix):
+        """
+        Add extdom configuration if it does not already exist.
+        """
+        if not self.admin_conn:
+            self.ldap_connect()
+
+        dn = DN('cn=ipa_extdom_extop,cn=plugins,cn=config')
+        try:
+            self.admin_conn.get_entry(dn)
+        except errors.NotFound:
+            self._ldap_mod('ipa-extdom-extop-conf.ldif', dict(SUFFIX=suffix))
+        else:
+            root_logger.debug("extdom plugin is already configured")
+
     def replica_populate(self):
         self.ldap_connect()
 
@@ -909,3 +1120,250 @@ class DsInstance(service.Service):
             pass
 
         self.ldap_disconnect()
+
+    def find_subject_base(self):
+        """
+        Try to find the current value of certificate subject base.
+        1) Look in sysupgrade first
+        2) If no value is found there, look in DS (start DS if necessary)
+        3) Last resort, look in the certmap.conf itself
+        4) If all fails, log loudly and return None
+
+        Note that this method can only be executed AFTER the ipa server
+        is configured, the api is initialized elsewhere and
+        that a ticket already have been acquired.
+        """
+        root_logger.debug(
+            'Trying to find certificate subject base in sysupgrade')
+        subject_base = sysupgrade.get_upgrade_state(
+            'certmap.conf', 'subject_base')
+
+        if subject_base:
+            root_logger.debug(
+                'Found certificate subject base in sysupgrade: %s',
+                subject_base)
+            return subject_base
+
+        root_logger.debug(
+            'Unable to find certificate subject base in sysupgrade')
+        root_logger.debug(
+            'Trying to find certificate subject base in DS')
+
+        ds_is_running = is_ds_running()
+        if not ds_is_running:
+            try:
+                self.start()
+                ds_is_running = True
+            except ipautil.CalledProcessError as e:
+                root_logger.error('Cannot start DS to find certificate '
+                                  'subject base: %s', e)
+
+        if ds_is_running:
+            try:
+                api.Backend.ldap2.connect(autobind=True)
+                ret = api.Command['config_show']()
+                subject_base = str(
+                    ret['result']['ipacertificatesubjectbase'][0])
+                root_logger.debug(
+                    'Found certificate subject base in DS: %s', subject_base)
+            except errors.PublicError as e:
+                root_logger.error('Cannot connect to DS to find certificate '
+                                  'subject base: %s', e)
+            finally:
+                try:
+                    api.Backend.ldap2.disconnect()
+                except Exception:
+                    pass
+
+        if not subject_base:
+            root_logger.debug('Unable to find certificate subject base in DS')
+            root_logger.debug('Trying to find certificate subject base in '
+                              'certmap.conf')
+
+            certmap_dir = config_dirname(
+                installutils.realm_to_serverid(api.env.realm)
+            )
+            try:
+                with open(os.path.join(certmap_dir, 'certmap.conf')) as f:
+                    for line in f:
+                        if line.startswith('certmap ipaca'):
+                            subject_base = line.strip().split(',')[-1]
+                            root_logger.debug(
+                                'Found certificate subject base in certmap.conf: '
+                                '%s', subject_base)
+
+            except IOError as e:
+                root_logger.error('Cannot open certmap.conf to find certificate '
+                                  'subject base: %s', e.strerror)
+
+        if subject_base:
+            return subject_base
+
+        root_logger.debug('Unable to find certificate subject base in '
+                          'certmap.conf')
+        return None
+
+    def __set_domain_level(self):
+        # Create global domain level entry and set the domain level
+        if self.domainlevel is not None:
+            self._ldap_mod("domainlevel.ldif", self.sub_dict)
+
+    def __get_ds_keytab(self):
+
+        self.fstore.backup_file(paths.DS_KEYTAB)
+        try:
+            os.unlink(paths.DS_KEYTAB)
+        except OSError:
+            pass
+
+        installutils.install_service_keytab(self.principal,
+                                            self.master_fqdn,
+                                            paths.DS_KEYTAB,
+                                            force_service_add=True)
+
+        # Configure DS to use the keytab
+        vardict = {"KRB5_KTNAME": paths.DS_KEYTAB}
+        ipautil.config_replace_variables(paths.SYSCONFIG_DIRSRV,
+                                         replacevars=vardict)
+
+        # Keytab must be owned by DS itself
+        pent = pwd.getpwnam(DS_USER)
+        os.chown(paths.DS_KEYTAB, pent.pw_uid, pent.pw_gid)
+
+    def __get_ds_cert(self):
+        subject = DN(('O', self.realm))
+        nssdb_dir = config_dirname(self.serverid)
+        db = certs.CertDB(self.realm, nssdir=nssdb_dir, subject_base=subject)
+        db.request_service_cert(self.nickname, self.principal, self.fqdn)
+        db.create_pin_file()
+
+        # Connect to self over ldapi as Directory Manager and configure SSL
+        conn = ipaldap.IPAdmin(self.fqdn, ldapi=True, realm=self.realm)
+        conn.do_external_bind('root')
+
+        mod = [(ldap.MOD_REPLACE, "nsSSLClientAuth", "allowed"),
+               (ldap.MOD_REPLACE, "nsSSL3Ciphers", "default"),
+               (ldap.MOD_REPLACE, "allowWeakCipher", "off")]
+        conn.modify_s(DN(('cn', 'encryption'), ('cn', 'config')), mod)
+
+        mod = [(ldap.MOD_ADD, "nsslapd-security", "on")]
+        conn.modify_s(DN(('cn', 'config')), mod)
+
+        entry = conn.make_entry(
+            DN(('cn', 'RSA'), ('cn', 'encryption'), ('cn', 'config')),
+            objectclass=["top", "nsEncryptionModule"],
+            cn=["RSA"],
+            nsSSLPersonalitySSL=[self.nickname],
+            nsSSLToken=["internal (software)"],
+            nsSSLActivation=["on"],
+        )
+        conn.add_entry(entry)
+
+        conn.unbind()
+
+        # check for open secure port 636 from now on
+        self.open_ports.append(636)
+
+    def update_dna_shared_config(self, method="SASL/GSSAPI", protocol="LDAP"):
+
+        dna_bind_method = "dnaRemoteBindMethod"
+        dna_conn_protocol = "dnaRemoteConnProtocol"
+        dna_plugin = DN(('cn', 'Distributed Numeric Assignment Plugin'),
+                        ('cn', 'plugins'),
+                        ('cn', 'config'))
+        dna_config_base = DN(('cn', 'posix IDs'), dna_plugin)
+
+        if not self.admin_conn:
+            self.ldap_connect()
+        conn = self.admin_conn
+
+        # Check the plugin is enabled else it is useless to update
+        # the shared entry
+        try:
+            entry = conn.get_entry(dna_plugin)
+            if entry.single_value.get('nsslapd-pluginenabled') == 'off':
+                return
+        except errors.NotFound:
+            root_logger.error("Could not find DNA plugin entry: %s" %
+                              dna_config_base)
+            return
+
+        try:
+            entry = conn.get_entry(dna_config_base)
+        except errors.NotFound:
+            root_logger.error("Could not find DNA config entry: %s" %
+                              dna_config_base)
+            return
+
+        sharedcfgdn = entry.single_value.get("dnaSharedCfgDN")
+        if sharedcfgdn is not None:
+            sharedcfgdn = DN(sharedcfgdn)
+        else:
+            root_logger.error(
+                "Could not find DNA shared config DN in entry: %s" %
+                dna_config_base)
+            return
+
+        #
+        # Update the shared config entry related to that host
+        #
+        # If the shared config entry already exists (like upgrade)
+        # the update occurs immediately without sleep.
+        #
+        # If the shared config entry does not exist (fresh install)
+        # DS server waits for 30s after its startup to create it.
+        # Startup likely occurred few sec before this function is
+        # called so this loop will wait for 30s max.
+        #
+        # In case the server is not able to create the entry
+        # The loop gives a grace period of 60s before logging
+        # the failure to update the shared config entry and return
+        #
+        max_wait = 30
+        for i in range(0, max_wait + 1):
+            try:
+                entries = conn.get_entries(
+                    sharedcfgdn, scope=ldap.SCOPE_ONELEVEL,
+                    filter='dnaHostname=%s' % self.fqdn
+                )
+                break
+            except errors.NotFound:
+                root_logger.debug(
+                    "Unable to find DNA shared config entry for "
+                    "dnaHostname=%s (under %s) so far. Retry in 2 sec." %
+                    (self.fqdn, sharedcfgdn)
+                )
+                time.sleep(2)
+        else:
+            root_logger.error(
+                "Could not get dnaHostname entries in {} seconds".format(
+                    max_wait * 2)
+            )
+            return
+
+        # If there are several entries, all of them will be updated
+        # just log a debug msg. This is likely the result of #5510
+        if len(entries) != 1:
+            root_logger.debug(
+                "%d entries dnaHostname=%s under %s. One expected" %
+                (len(entries), self.fqdn, sharedcfgdn)
+            )
+
+        # time to set the bind method and the protocol in the
+        # shared config entries
+        for entry in entries:
+            mod = []
+            if entry.single_value.get(dna_bind_method) != method:
+                mod.append((ldap.MOD_REPLACE, dna_bind_method, method))
+
+            if entry.single_value.get(dna_conn_protocol) != method:
+                mod.append((ldap.MOD_REPLACE, dna_conn_protocol, protocol))
+
+            if mod:
+                try:
+                    conn.modify_s(entry.dn, mod)
+                except Exception as e:
+                    root_logger.error(
+                        "Failed to set SASL/GSSAPI bind method/protocol "
+                        "in entry {}: {}".format(entry, e)
+                    )
