@@ -17,15 +17,20 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import absolute_import
+from __future__ import print_function
+
+import errno
 import socket
 import getpass
+import gssapi
+import ldif
 import os
 import re
 import fileinput
 import sys
 import tempfile
 import shutil
-from ConfigParser import SafeConfigParser, NoOptionError
 import traceback
 import textwrap
 from contextlib import contextmanager
@@ -33,23 +38,34 @@ from contextlib import contextmanager
 from dns import resolver, rdatatype
 from dns.exception import DNSException
 import ldap
+import ldapurl
 from nss.error import NSPRError
+import six
+from six.moves.configparser import SafeConfigParser, NoOptionError
 
-from ipapython import ipautil, sysrestore, admintool, dogtag
+import ipaplatform
+
+from ipapython import ipautil, sysrestore, admintool, version
 from ipapython.admintool import ScriptError
 from ipapython.ipa_log_manager import root_logger, log_mgr
 from ipalib.util import validate_hostname
 from ipapython import config
-from ipalib import errors, x509
+from ipalib import api, errors, x509
 from ipapython.dn import DN
-from ipaserver.install import certs, service
+from ipaserver.install import certs, service, sysupgrade
 from ipaplatform import services
 from ipaplatform.paths import paths
+from ipaplatform.tasks import tasks
+from ipapython import certmonger
+
+
+if six.PY3:
+    unicode = str
 
 # Used to determine install status
 IPA_MODULES = [
-    'httpd', 'kadmin', 'dirsrv', 'pki-cad', 'pki-tomcatd', 'install',
-    'krb5kdc', 'ntpd', 'named', 'ipa_memcached']
+    'httpd', 'kadmin', 'dirsrv', 'pki-tomcatd', 'install', 'krb5kdc', 'ntpd',
+    'named', 'ipa_memcached']
 
 
 class BadHostError(Exception):
@@ -67,8 +83,29 @@ class HostReverseLookupError(HostLookupError):
 class HostnameLocalhost(HostLookupError):
     pass
 
+
+class UpgradeVersionError(Exception):
+    pass
+
+
+class UpgradePlatformError(UpgradeVersionError):
+    pass
+
+
+class UpgradeDataOlderVersionError(UpgradeVersionError):
+    pass
+
+
+class UpgradeDataNewerVersionError(UpgradeVersionError):
+    pass
+
+
+class UpgradeMissingVersionError(UpgradeVersionError):
+    pass
+
+
 class ReplicaConfig:
-    def __init__(self):
+    def __init__(self, top_dir=None):
         self.realm_name = ""
         self.domain_name = ""
         self.master_host_name = ""
@@ -78,6 +115,7 @@ class ReplicaConfig:
         self.subject_base = None
         self.setup_ca = False
         self.version = 0
+        self.top_dir = top_dir
 
     subject_base = ipautil.dn_attribute_property('_subject_base')
 
@@ -117,7 +155,7 @@ def verify_fqdn(host_name, no_host_dns=False, local_hostname=True):
     try:
         # make sure that the host name meets the requirements in ipalib
         validate_hostname(host_name)
-    except ValueError, e:
+    except ValueError as e:
         raise BadHostError("Invalid hostname '%s', %s" % (host_name, unicode(e)))
 
     if local_hostname:
@@ -130,17 +168,19 @@ def verify_fqdn(host_name, no_host_dns=False, local_hostname=True):
                         "Please check /etc/hosts or DNS name resolution" % (host_name, ex_name[0]))
         except socket.gaierror:
             pass
-        except socket.error, e:
-            root_logger.debug('socket.gethostbyaddr() error: %d: %s' % (e.errno, e.strerror))
+        except socket.error as e:
+            root_logger.debug(
+                'socket.gethostbyaddr() error: %d: %s',
+                e.errno, e.strerror)  # pylint: disable=no-member
 
     if no_host_dns:
-        print "Warning: skipping DNS resolution of host", host_name
+        print("Warning: skipping DNS resolution of host", host_name)
         return
 
     try:
         root_logger.debug('Search DNS for %s', host_name)
         hostaddr = socket.getaddrinfo(host_name, None)
-    except Exception, e:
+    except Exception as e:
         root_logger.debug('Search failed: %s', e)
         raise HostForwardLookupError("Unable to resolve host name, check /etc/hosts or DNS name resolution")
 
@@ -166,15 +206,22 @@ def verify_fqdn(host_name, no_host_dns=False, local_hostname=True):
         try:
             root_logger.debug('Check reverse address of %s', address)
             revname = socket.gethostbyaddr(address)[0]
-        except Exception, e:
+        except Exception as e:
             root_logger.debug('Check failed: %s', e)
-            raise HostReverseLookupError("Unable to resolve the reverse ip address, check /etc/hosts or DNS name resolution")
-        root_logger.debug('Found reverse name: %s', revname)
-        if revname != host_name:
-            raise HostReverseLookupError("The host name %s does not match the reverse lookup %s" % (host_name, revname))
+            root_logger.error(
+                "Unable to resolve the IP address %s to a host name, "
+                "check /etc/hosts and DNS name resolution", address)
+        else:
+            root_logger.debug('Found reverse name: %s', revname)
+            if revname != host_name:
+                root_logger.error(
+                    "The host name %s does not match the value %s obtained "
+                    "by reverse lookup on IP address %s", host_name, revname,
+                    address)
         verified.add(address)
 
-def record_in_hosts(ip, host_name=None, file=paths.HOSTS):
+
+def record_in_hosts(ip, host_name=None, conf_file=paths.HOSTS):
     """
     Search record in /etc/hosts - static table lookup for hostnames
 
@@ -184,9 +231,9 @@ def record_in_hosts(ip, host_name=None, file=paths.HOSTS):
 
     :param ip: IP address
     :param host_name: Optional hostname to search
-    :param file: Optional path to the lookup table
+    :param conf_file: Optional path to the lookup table
     """
-    hosts = open(file, 'r').readlines()
+    hosts = open(conf_file, 'r').readlines()
     for line in hosts:
         line = line.rstrip('\n')
         fields = line.partition('#')[0].split()
@@ -206,69 +253,62 @@ def record_in_hosts(ip, host_name=None, file=paths.HOSTS):
                     return None
             return (hosts_ip, names)
         except IndexError:
-            print "Warning: Erroneous line '%s' in %s" % (line, file)
+            print("Warning: Erroneous line '%s' in %s" % (line, conf_file))
             continue
 
     return None
 
-def add_record_to_hosts(ip, host_name, file=paths.HOSTS):
-    hosts_fd = open(file, 'r+')
+def add_record_to_hosts(ip, host_name, conf_file=paths.HOSTS):
+    hosts_fd = open(conf_file, 'r+')
     hosts_fd.seek(0, 2)
     hosts_fd.write(ip+'\t'+host_name+' '+host_name.split('.')[0]+'\n')
     hosts_fd.close()
 
-# TODO: Remove when removing usage from ipa-adtrust-install
-def read_ip_address(host_name, fstore):
-    while True:
-        ip = ipautil.user_input("Please provide the IP address to be used for this host name", allow_empty = False)
-        try:
-            ip_parsed = ipautil.CheckedIPAddress(ip, match_local=True)
-        except Exception, e:
-            print "Error: Invalid IP Address %s: %s" % (ip, e)
-            continue
-        else:
-            break
 
-    return ip_parsed
-
-def read_ip_addresses(host_name, fstore):
+def read_ip_addresses():
     ips = []
-    print "Enter the IP address to use, or press Enter to finish."
+    print("Enter the IP address to use, or press Enter to finish.")
     while True:
         ip = ipautil.user_input("Please provide the IP address to be used for this host name", allow_empty = True)
         if not ip:
             break
         try:
             ip_parsed = ipautil.CheckedIPAddress(ip, match_local=True)
-        except Exception, e:
-            print "Error: Invalid IP Address %s: %s" % (ip, e)
+        except Exception as e:
+            print("Error: Invalid IP Address %s: %s" % (ip, e))
             continue
         ips.append(ip_parsed)
 
     return ips
 
+
 def read_dns_forwarders():
     addrs = []
     if ipautil.user_input("Do you want to configure DNS forwarders?", True):
-        print "Enter the IP address of DNS forwarder to use, or press Enter to finish."
-
+        print("Following DNS servers are configured in /etc/resolv.conf: %s" %
+                ", ".join(resolver.get_default_resolver().nameservers))
+        if ipautil.user_input("Do you want to configure these servers as DNS "
+                "forwarders?", True):
+            addrs = resolver.default_resolver.nameservers[:]
+            print("All DNS servers from /etc/resolv.conf were added. You can "
+                  "enter additional addresses now:")
         while True:
-            ip = ipautil.user_input("Enter IP address for a DNS forwarder",
-                                    allow_empty=True)
+            ip = ipautil.user_input("Enter an IP address for a DNS forwarder, "
+                                    "or press Enter to skip", allow_empty=True)
             if not ip:
                 break
             try:
                 ip_parsed = ipautil.CheckedIPAddress(ip, parse_netmask=False)
-            except Exception, e:
-                print "Error: Invalid IP Address %s: %s" % (ip, e)
-                print "DNS forwarder %s not added" % ip
+            except Exception as e:
+                print("Error: Invalid IP Address %s: %s" % (ip, e))
+                print("DNS forwarder %s not added." % ip)
                 continue
 
-            print "DNS forwarder %s added" % ip
+            print("DNS forwarder %s added. You may add another." % ip)
             addrs.append(str(ip_parsed))
 
     if not addrs:
-        print "No DNS forwarders configured"
+        print("No DNS forwarders configured")
 
     return addrs
 
@@ -301,8 +341,8 @@ def read_password(user, confirm=True, validate=True, retry=True, validator=_read
             if validate:
                 try:
                     validator(pwd)
-                except ValueError, e:
-                    print str(e)
+                except ValueError as e:
+                    print(str(e))
                     pwd = None
                     continue
             if not confirm:
@@ -310,15 +350,15 @@ def read_password(user, confirm=True, validate=True, retry=True, validator=_read
                 continue
             pwd_confirm = get_password("Password (confirm): ")
             if pwd != pwd_confirm:
-                print "Password mismatch!"
-                print ""
+                print("Password mismatch!")
+                print("")
                 pwd = None
             else:
                 correct = True
     except EOFError:
         return None
     finally:
-        print ""
+        print("")
     return pwd
 
 def update_file(filename, orig, subst):
@@ -335,7 +375,7 @@ def update_file(filename, orig, subst):
         os.chown(filename, st.st_uid, st.st_gid) # reset perms
         return 0
     else:
-        print "File %s doesn't exist." % filename
+        print("File %s doesn't exist." % filename)
         return 1
 
 def set_directive(filename, directive, value, quotes=True, separator=' '):
@@ -438,32 +478,30 @@ def get_host_name(no_host_dns):
     verify_fqdn(hostname, no_host_dns)
     return hostname
 
-def get_server_ip_address(host_name, fstore, unattended, setup_dns, ip_addresses):
+def get_server_ip_address(host_name, unattended, setup_dns, ip_addresses):
     # Check we have a public IP that is associated with the hostname
     try:
         hostaddr = resolve_host(host_name)
     except HostnameLocalhost:
-        print >> sys.stderr, "The hostname resolves to the localhost address (127.0.0.1/::1)"
-        print >> sys.stderr, "Please change your /etc/hosts file so that the hostname"
-        print >> sys.stderr, "resolves to the ip address of your network interface."
-        print >> sys.stderr, "The KDC service does not listen on localhost"
-        print >> sys.stderr, ""
-        print >> sys.stderr, "Please fix your /etc/hosts file and restart the setup program"
+        print("The hostname resolves to the localhost address (127.0.0.1/::1)", file=sys.stderr)
+        print("Please change your /etc/hosts file so that the hostname", file=sys.stderr)
+        print("resolves to the ip address of your network interface.", file=sys.stderr)
+        print("The KDC service does not listen on localhost", file=sys.stderr)
+        print("", file=sys.stderr)
+        print("Please fix your /etc/hosts file and restart the setup program", file=sys.stderr)
         sys.exit(1)
 
-    ip_add_to_hosts = False
-
     ips = []
     if len(hostaddr):
         for ha in hostaddr:
             try:
                 ips.append(ipautil.CheckedIPAddress(ha, match_local=True))
-            except ValueError, e:
+            except ValueError as e:
                 root_logger.warning("Invalid IP address %s for %s: %s", ha, host_name, unicode(e))
 
     if not ips and not ip_addresses:
         if not unattended:
-            ip_addresses = read_ip_addresses(host_name, fstore)
+            ip_addresses = read_ip_addresses()
 
     if ip_addresses:
         if setup_dns:
@@ -473,40 +511,51 @@ def get_server_ip_address(host_name, fstore, unattended, setup_dns, ip_addresses
             if set(ip_addresses) <= set(ips):
                 ips = ip_addresses
             else:
-                print >>sys.stderr, "Error: the hostname resolves to IP address(es) that are different"
-                print >>sys.stderr, "from those provided on the command line.  Please fix your DNS"
-                print >>sys.stderr, "or /etc/hosts file and restart the installation."
-                print >>sys.stderr, "Provided but not resolved address(es): %s" % \
-                                    ", ".join(str(ip) for ip in (set(ip_addresses) - set(ips)))
+                print("Error: the hostname resolves to IP address(es) that are different", file=sys.stderr)
+                print("from those provided on the command line.  Please fix your DNS", file=sys.stderr)
+                print("or /etc/hosts file and restart the installation.", file=sys.stderr)
+                print("Provided but not resolved address(es): %s" % \
+                                    ", ".join(str(ip) for ip in (set(ip_addresses) - set(ips))), file=sys.stderr)
                 sys.exit(1)
-        ip_add_to_hosts = True
 
     if not ips:
-        print >> sys.stderr, "No usable IP address provided nor resolved."
+        print("No usable IP address provided nor resolved.", file=sys.stderr)
         sys.exit(1)
 
     for ip_address in ips:
-        # check /etc/hosts sanity, add a record when needed
+        # check /etc/hosts sanity
         hosts_record = record_in_hosts(str(ip_address))
 
-        if hosts_record is None:
-            if ip_add_to_hosts or setup_dns:
-                print "Adding ["+str(ip_address)+" "+host_name+"] to your /etc/hosts file"
-                fstore.backup_file(paths.HOSTS)
-                add_record_to_hosts(str(ip_address), host_name)
-        else:
+        if hosts_record is not None:
             primary_host = hosts_record[1][0]
             if primary_host != host_name:
-                print >>sys.stderr, "Error: there is already a record in /etc/hosts for IP address %s:" \
-                        % ip_address
-                print >>sys.stderr, hosts_record[0], " ".join(hosts_record[1])
-                print >>sys.stderr, "Chosen hostname %s does not match configured canonical hostname %s" \
-                        % (host_name, primary_host)
-                print >>sys.stderr, "Please fix your /etc/hosts file and restart the installation."
+                print("Error: there is already a record in /etc/hosts for IP address %s:" \
+                        % ip_address, file=sys.stderr)
+                print(hosts_record[0], " ".join(hosts_record[1]), file=sys.stderr)
+                print("Chosen hostname %s does not match configured canonical hostname %s" \
+                        % (host_name, primary_host), file=sys.stderr)
+                print("Please fix your /etc/hosts file and restart the installation.", file=sys.stderr)
                 sys.exit(1)
 
     return ips
 
+
+def update_hosts_file(ip_addresses, host_name, fstore):
+    """
+    Update hosts with specified addresses
+    :param ip_addresses: list of IP addresses
+    :return:
+    """
+    if not fstore.has_file(paths.HOSTS):
+        fstore.backup_file(paths.HOSTS)
+    for ip_address in ip_addresses:
+        if record_in_hosts(str(ip_address)):
+            continue
+        print("Adding [{address!s} {name}] to your /etc/hosts file".format(
+            address=ip_address, name=host_name))
+        add_record_to_hosts(str(ip_address), host_name)
+
+
 def expand_replica_info(filename, password):
     """
     Decrypt and expand a replica installation file into a temporary
@@ -514,20 +563,20 @@ def expand_replica_info(filename, password):
     """
     top_dir = tempfile.mkdtemp("ipa")
     tarfile = top_dir+"/files.tar"
-    dir = top_dir + "/realm_info"
+    dir_path = top_dir + "/realm_info"
     ipautil.decrypt_file(filename, tarfile, password, top_dir)
     ipautil.run(["tar", "xf", tarfile, "-C", top_dir])
     os.remove(tarfile)
 
-    return top_dir, dir
+    return top_dir, dir_path
 
-def read_replica_info(dir, rconfig):
+def read_replica_info(dir_path, rconfig):
     """
     Read the contents of a replica installation file.
 
     rconfig is a ReplicaConfig object
     """
-    filename = dir + "/realm_info"
+    filename = dir_path + "/realm_info"
     fd = open(filename)
     config = SafeConfigParser()
     config.readfp(fd)
@@ -544,20 +593,65 @@ def read_replica_info(dir, rconfig):
 
 def read_replica_info_dogtag_port(config_dir):
     portfile = config_dir + "/dogtag_directory_port.txt"
-    default_port = dogtag.Dogtag9Constants.DS_PORT
+    default_port = 7389
     if not ipautil.file_exists(portfile):
         dogtag_master_ds_port = default_port
     else:
         with open(portfile) as fd:
             try:
                 dogtag_master_ds_port = int(fd.read())
-            except (ValueError, IOError), e:
+            except (ValueError, IOError) as e:
                 root_logger.debug('Cannot parse dogtag DS port: %s', e)
                 root_logger.debug('Default to %d', default_port)
                 dogtag_master_ds_port = default_port
 
     return dogtag_master_ds_port
 
+
+def create_replica_config(dirman_password, filename, options):
+    top_dir = None
+    try:
+        top_dir, dir = expand_replica_info(filename, dirman_password)
+    except Exception as e:
+        root_logger.error("Failed to decrypt or open the replica file.")
+        print("ERROR: Failed to decrypt or open the replica file.")
+        print("Verify you entered the correct Directory Manager password.")
+        sys.exit(1)
+    config = ReplicaConfig(top_dir)
+    read_replica_info(dir, config)
+    root_logger.debug(
+        'Installing replica file with version %d (0 means no version in prepared file).',
+        config.version)
+    if config.version and config.version > version.NUM_VERSION:
+        root_logger.error(
+            'A replica file from a newer release (%d) cannot be installed on an older version (%d)',
+            config.version, version.NUM_VERSION)
+        sys.exit(1)
+    config.dirman_password = dirman_password
+    try:
+        host = get_host_name(options.no_host_dns)
+    except BadHostError as e:
+        root_logger.error(str(e))
+        sys.exit(1)
+    if config.host_name != host:
+        try:
+            print("This replica was created for '%s' but this machine is named '%s'" % (config.host_name, host))
+            if not ipautil.user_input("This may cause problems. Continue?", False):
+                root_logger.debug(
+                    "Replica was created for %s but machine is named %s  "
+                    "User chose to exit",
+                    config.host_name, host)
+                sys.exit(0)
+            config.host_name = host
+            print("")
+        except KeyboardInterrupt:
+            root_logger.debug("Keyboard Interrupt")
+            sys.exit(0)
+    config.dir = dir
+    config.ca_ds_port = read_replica_info_dogtag_port(config.dir)
+    return config
+
+
 def check_server_configuration():
     """
     Check if IPA server is configured on the system.
@@ -574,16 +668,18 @@ def check_server_configuration():
     if not server_fstore.has_files():
         raise RuntimeError("IPA is not configured on this system.")
 
+
 def remove_file(filename):
     """
     Remove a file and log any exceptions raised.
     """
     try:
-        if os.path.exists(filename):
+        if os.path.lexists(filename):
             os.unlink(filename)
-    except Exception, e:
+    except Exception as e:
         root_logger.error('Error removing %s: %s' % (filename, str(e)))
 
+
 def rmtree(path):
     """
     Remove a directory structure and log any exceptions raised.
@@ -591,9 +687,10 @@ def rmtree(path):
     try:
         if os.path.exists(path):
             shutil.rmtree(path)
-    except Exception, e:
+    except Exception as e:
         root_logger.error('Error removing %s: %s' % (path, str(e)))
 
+
 def is_ipa_configured():
     """
     Using the state and index install files determine if IPA is already
@@ -640,8 +737,11 @@ def run_script(main_function, operation_name, log_file_name=None,
     try:
         try:
             return_value = main_function()
-        except BaseException, e:
-            if isinstance(e, SystemExit) and (e.code is None or e.code == 0):
+        except BaseException as e:
+            if (
+                isinstance(e, SystemExit) and
+                (e.code is None or e.code == 0)  # pylint: disable=no-member
+            ):
                 # Not an error after all
                 root_logger.info('The %s command was successful',
                     operation_name)
@@ -654,7 +754,7 @@ def run_script(main_function, operation_name, log_file_name=None,
                 root_logger.debug('The %s command failed, exception: %s: %s',
                                   operation_name, type(e).__name__, e)
                 if fail_message and not isinstance(e, SystemExit):
-                    print fail_message
+                    print(fail_message)
                 raise
         else:
             if return_value:
@@ -665,10 +765,10 @@ def run_script(main_function, operation_name, log_file_name=None,
                     operation_name)
             sys.exit(return_value)
 
-    except BaseException, error:
+    except BaseException as error:
         message, exitcode = handle_error(error, log_file_name)
         if message:
-            print >> sys.stderr, message
+            print(message, file=sys.stderr)
         sys.exit(exitcode)
 
 
@@ -700,12 +800,16 @@ def handle_error(error, log_file_name=None):
     if isinstance(error, ldap.INSUFFICIENT_ACCESS):
         return "Insufficient access", 1
     if isinstance(error, ldap.LOCAL_ERROR):
-        return error.args[0]['info'], 1
+        return error.args[0].get('info', ''), 1
     if isinstance(error, ldap.SERVER_DOWN):
         return error.args[0]['desc'], 1
     if isinstance(error, ldap.LDAPError):
-        return 'LDAP error: %s\n%s' % (
-            type(error).__name__, error.args[0]['info']), 1
+        message = 'LDAP error: %s\n%s\n%s' % (
+            type(error).__name__,
+            error.args[0]['desc'].strip(),
+            error.args[0].get('info', '').strip()
+        )
+        return message, 1
 
     if isinstance(error, config.IPAConfigError):
         message = "An IPA server to update cannot be found. Has one been configured yet?"
@@ -797,7 +901,7 @@ def load_pkcs12(cert_files, key_password, key_nickname, ca_cert_files,
         for nickname in trust_chain[1:]:
             try:
                 nssdb.verify_ca_cert_validity(nickname)
-            except ValueError, e:
+            except ValueError as e:
                 raise ScriptError(
                     "CA certificate %s in %s is not valid: %s" %
                     (subject, ", ".join(cert_files), e))
@@ -825,28 +929,6 @@ def load_pkcs12(cert_files, key_password, key_nickname, ca_cert_files,
 
     return out_file, out_password, ca_cert
 
-@contextmanager
-def private_ccache(path=None):
-
-    if path is None:
-        (desc, path) = tempfile.mkstemp(prefix='krbcc')
-        os.close(desc)
-
-    original_value = os.environ.get('KRB5CCNAME', None)
-
-    os.environ['KRB5CCNAME'] = path
-
-    try:
-        yield
-    finally:
-        if original_value is not None:
-            os.environ['KRB5CCNAME'] = original_value
-        else:
-            os.environ.pop('KRB5CCNAME')
-
-        if os.path.exists(path):
-            os.remove(path)
-
 
 @contextmanager
 def stopped_service(service, instance_name=""):
@@ -883,23 +965,23 @@ def stopped_service(service, instance_name=""):
             root_logger.debug('Starting %s%s.', service, log_instance_name)
             service_obj.start(instance_name)
 
+
 def check_entropy():
-    '''
+    """
     Checks if the system has enough entropy, if not, displays warning message
-    '''
+    """
     try:
-        with open('/proc/sys/kernel/random/entropy_avail', 'r') as efname:
+        with open(paths.ENTROPY_AVAIL, 'r') as efname:
             if int(efname.read()) < 200:
                 emsg = 'WARNING: Your system is running out of entropy, ' \
                         'you may experience long delays'
                 service.print_msg(emsg)
                 root_logger.debug(emsg)
     except IOError as e:
-        root_logger.debug("Could not open /proc/sys/kernel/random/entropy_avail: %s" % \
-            e)
+        root_logger.debug(
+            "Could not open %s: %s", paths.ENTROPY_AVAIL, e)
     except ValueError as e:
-        root_logger.debug("Invalid value in /proc/sys/kernel/random/entropy_avail %s" % \
-            e)
+        root_logger.debug("Invalid value in %s %s", paths.ENTROPY_AVAIL, e)
 
 def load_external_cert(files, subject_base):
     """
@@ -958,7 +1040,7 @@ def load_external_cert(files, subject_base):
         for nickname in trust_chain:
             try:
                 nssdb.verify_ca_cert_validity(nickname)
-            except ValueError, e:
+            except ValueError as e:
                 raise ScriptError(
                     "CA certificate %s in %s is not valid: %s" %
                     (subject, ", ".join(files), e))
@@ -972,3 +1054,327 @@ def load_external_cert(files, subject_base):
     ca_file.flush()
 
     return cert_file, ca_file
+
+
+def store_version():
+    """Store current data version and platform. This is required for check if
+    upgrade is required.
+    """
+    sysupgrade.set_upgrade_state('ipa', 'data_version',
+                                 version.VENDOR_VERSION)
+    sysupgrade.set_upgrade_state('ipa', 'platform', ipaplatform.NAME)
+
+
+def check_version():
+    """
+    :raise UpgradePlatformError: if platform is not the same
+    :raise UpgradeDataOlderVersionError: if data needs to be upgraded
+    :raise UpgradeDataNewerVersionError: older version of IPA was detected than data
+    :raise UpgradeMissingVersionError: if platform or version is missing
+    """
+    platform = sysupgrade.get_upgrade_state('ipa', 'platform')
+    if platform is not None:
+        if platform != ipaplatform.NAME:
+            raise UpgradePlatformError(
+                "platform mismatch (expected '%s', current '%s')" % (
+                platform, ipaplatform.NAME)
+            )
+    else:
+        raise UpgradeMissingVersionError("no platform stored")
+
+    data_version = sysupgrade.get_upgrade_state('ipa', 'data_version')
+    if data_version is not None:
+        parsed_data_ver = tasks.parse_ipa_version(data_version)
+        parsed_ipa_ver = tasks.parse_ipa_version(version.VENDOR_VERSION)
+        if parsed_data_ver < parsed_ipa_ver:
+            raise UpgradeDataOlderVersionError(
+                "data needs to be upgraded (expected version '%s', current "
+                "version '%s')" % (version.VENDOR_VERSION, data_version)
+            )
+        elif parsed_data_ver > parsed_ipa_ver:
+            raise UpgradeDataNewerVersionError(
+                "data are in newer version than IPA (data version '%s', IPA "
+                "version '%s')" % (data_version, version.VENDOR_VERSION)
+            )
+    else:
+        raise UpgradeMissingVersionError("no data_version stored")
+
+def realm_to_serverid(realm_name):
+    return "-".join(realm_name.split("."))
+
+
+def realm_to_ldapi_uri(realm_name):
+    serverid = realm_to_serverid(realm_name)
+    socketname = paths.SLAPD_INSTANCE_SOCKET_TEMPLATE % (serverid,)
+    return 'ldapi://' + ldapurl.ldapUrlEscape(socketname)
+
+
+def install_service_keytab(principal, server, path, force_service_add=False):
+
+    try:
+        api.Backend.rpcclient.connect()
+
+        # Create services if none exists (we use the .forward method
+        # here so that we can control the client version number and avoid
+        # errors. This is a workaround until the API becomes version
+        # independent: FIXME
+
+        api.Backend.rpcclient.forward(
+            'service_add',
+            krbprincipalname=principal,
+            force=force_service_add,
+            version=u'2.112'    # All the way back to 3.0 servers
+        )
+    except errors.DuplicateEntry:
+        pass
+    finally:
+        if api.Backend.rpcclient.isconnected():
+            api.Backend.rpcclient.disconnect()
+
+    args = [paths.IPA_GETKEYTAB, '-k', path, '-p', principal, '-s', server]
+    ipautil.run(args)
+
+
+def check_creds(options, realm_name):
+
+    # Check if ccache is available
+    default_cred = None
+    try:
+        root_logger.debug('KRB5CCNAME set to %s' %
+                          os.environ.get('KRB5CCNAME', None))
+        # get default creds, will raise if none found
+        default_cred = gssapi.creds.Credentials()
+        principal = str(default_cred.name)
+    except gssapi.raw.misc.GSSError as e:
+        root_logger.debug('Failed to find default ccache: %s' % e)
+        principal = None
+
+    # Check if the principal matches the requested one (if any)
+    if principal is not None and options.principal is not None:
+        op = options.principal
+        if op.find('@') == -1:
+            op = '%s@%s' % (op, realm_name)
+        if principal != op:
+            root_logger.debug('Specified principal %s does not match '
+                              'available credentials (%s)' %
+                              (options.principal, principal))
+            principal = None
+
+    if principal is None:
+        (ccache_fd, ccache_name) = tempfile.mkstemp()
+        os.close(ccache_fd)
+        options.created_ccache_file = ccache_name
+
+        if options.principal is not None:
+            principal = options.principal
+        else:
+            principal = 'admin'
+        stdin = None
+        if principal.find('@') == -1:
+            principal = '%s@%s' % (principal, realm_name)
+        if options.admin_password is not None:
+            stdin = options.admin_password
+        else:
+            if not options.unattended:
+                try:
+                    stdin = getpass.getpass("Password for %s: " % principal)
+                except EOFError:
+                    stdin = None
+                if not stdin:
+                    root_logger.error(
+                        "Password must be provided for %s.", principal)
+                    raise ScriptError("Missing password for %s" % principal)
+            else:
+                if sys.stdin.isatty():
+                    root_logger.error("Password must be provided in " +
+                                      "non-interactive mode.")
+                    root_logger.info("This can be done via " +
+                                     "echo password | ipa-client-install " +
+                                     "... or with the -w option.")
+                    raise ScriptError("Missing password for %s" % principal)
+                else:
+                    stdin = sys.stdin.readline()
+
+            # set options.admin_password for future use
+            options.admin_password = stdin
+
+        try:
+            ipautil.kinit_password(principal, stdin, ccache_name)
+        except RuntimeError as e:
+            root_logger.error("Kerberos authentication failed: %s" % e)
+            raise ScriptError("Invalid credentials: %s" % e)
+
+        os.environ['KRB5CCNAME'] = ccache_name
+
+
+class ModifyLDIF(ldif.LDIFParser):
+    """
+    Allows to modify LDIF file.
+
+    Operations keep the order in which were specified per DN.
+    Warning: only modifications of existing DNs are supported
+    """
+    def __init__(self, input_file, output_file):
+        """
+        :param input_file: an LDIF
+        :param output_file: an LDIF file
+        """
+        ldif.LDIFParser.__init__(self, input_file)
+        self.writer = ldif.LDIFWriter(output_file)
+        self.dn_updated = set()
+
+        self.modifications = {}  # keep modify operations in original order
+
+    def add_value(self, dn, attr, values):
+        """
+        Add value to LDIF.
+        :param dn: DN of entry (must exists)
+        :param attr: attribute name
+        :param value: value to be added
+        """
+        assert isinstance(values, list)
+        self.modifications.setdefault(dn, []).append(
+            dict(
+                op="add",
+                attr=attr,
+                values=values,
+            )
+        )
+
+    def remove_value(self, dn, attr, values=None):
+        """
+        Remove value from LDIF.
+        :param dn: DN of entry
+        :param attr: attribute name
+        :param value: value to be removed, if value is None, attribute will
+        be removed
+        """
+        assert values is None or isinstance(values, list)
+        self.modifications.setdefault(dn, []).append(
+            dict(
+                op="del",
+                attr=attr,
+                values=values,
+            )
+        )
+
+    def replace_value(self, dn, attr, values):
+        """
+        Replace values in LDIF with new value.
+        :param dn: DN of entry
+        :param attr: attribute name
+        :param value: new value for atribute
+        """
+        assert isinstance(values, list)
+        self.remove_value(dn, attr)
+        self.add_value(dn, attr, values)
+
+    def modifications_from_ldif(self, ldif_file):
+        """
+        Parse ldif file. Default operation is add, only changetypes "add"
+        and "modify" are supported.
+        :param ldif_file: an opened file for read
+        :raises: ValueError
+        """
+        parser = ldif.LDIFRecordList(ldif_file)
+        parser.parse()
+
+        last_dn = None
+        for dn, entry in parser.all_records:
+            if dn is None:
+                # ldif parser return None, if records belong to previous DN
+                dn = last_dn
+            else:
+                last_dn = dn
+
+            if "replace" in entry:
+                for attr in entry["replace"]:
+                    try:
+                        self.replace_value(dn, attr, entry[attr])
+                    except KeyError:
+                        raise ValueError("replace: {dn}, {attr}: values are "
+                                         "missing".format(dn=dn, attr=attr))
+            elif "delete" in entry:
+                for attr in entry["delete"]:
+                    self.remove_value(dn, attr, entry.get(attr, None))
+            elif "add" in entry:
+                for attr in entry["add"]:
+                    try:
+                        self.replace_value(dn, attr, entry[attr])
+                    except KeyError:
+                        raise ValueError("add: {dn}, {attr}: values are "
+                                         "missing".format(dn=dn, attr=attr))
+            else:
+                root_logger.error("Ignoring entry: %s : only modifications "
+                                  "are allowed (missing \"changetype: "
+                                  "modify\")", dn)
+
+    def handle(self, dn, entry):
+        if dn in self.modifications:
+            self.dn_updated.add(dn)
+        for mod in self.modifications.get(dn, []):
+            attr_name = mod["attr"]
+            values = mod["values"]
+
+            if mod["op"] == "del":
+                # delete
+                attribute = entry.setdefault(attr_name, [])
+                if values is None:
+                    attribute = []
+                else:
+                    attribute = [v for v in attribute if v not in values]
+                if not attribute:  # empty
+                    del entry[attr_name]
+            elif mod["op"] == "add":
+                # add
+                attribute = entry.setdefault(attr_name, [])
+                attribute.extend([v for v in values if v not in attribute])
+            else:
+                assert False, "Unknown operation: %r" % mod["op"]
+
+        self.writer.unparse(dn, entry)
+
+    def parse(self):
+        ldif.LDIFParser.parse(self)
+
+        # check if there are any remaining modifications
+        remaining_changes = set(self.modifications.keys()) - self.dn_updated
+        for dn in remaining_changes:
+            root_logger.error(
+                "DN: %s does not exists or haven't been updated", dn)
+
+
+def remove_keytab(keytab_path):
+    """
+    Remove Kerberos keytab and issue a warning if the procedure fails
+
+    :param keytab_path: path to the keytab file
+    """
+    try:
+        root_logger.debug("Removing service keytab: {}".format(keytab_path))
+        os.remove(keytab_path)
+    except OSError as e:
+        if e.errno != errno.ENOENT:
+            root_logger.warning("Failed to remove Kerberos keytab '{}': "
+                                "{}".format(keytab_path, e))
+            root_logger.warning("You may have to remove it manually")
+
+
+def remove_ccache(ccache_path=None, run_as=None):
+    """
+    remove Kerberos credential cache, essentially a wrapper around kdestroy.
+
+    :param ccache_path: path to the ccache file
+    :param run_as: run kdestroy as this user
+    """
+    root_logger.debug("Removing service credentials cache")
+    kdestroy_cmd = [paths.KDESTROY]
+    if ccache_path is not None:
+        root_logger.debug("Ccache path: '{}'".format(ccache_path))
+        kdestroy_cmd.extend(['-c', ccache_path])
+
+    try:
+        ipautil.run(kdestroy_cmd, runas=run_as, env={})
+    except ipautil.CalledProcessError as e:
+        root_logger.warning(
+            "Failed to clear Kerberos credentials cache: {}".format(e))
