@@ -24,23 +24,25 @@ import tempfile
 import time
 import pwd
 from optparse import OptionGroup
-from ConfigParser import SafeConfigParser
 from ipaplatform.paths import paths
 from ipaplatform import services
 
+from six.moves.configparser import SafeConfigParser
+
 from ipalib import api, errors
 from ipapython import version
 from ipapython.ipautil import run, write_tmp_file
 from ipapython import admintool
 from ipapython.config import IPAOptionParser
 from ipapython.dn import DN
-from ipaserver.install.dsinstance import realm_to_serverid, DS_USER
 from ipaserver.install.replication import wait_for_task
 from ipaserver.install import installutils
 from ipapython import ipaldap
 from ipalib.session import ISO8601_DATETIME_FMT
 from ipalib.constants import CACERT
-from ConfigParser import SafeConfigParser
+from six.moves.configparser import SafeConfigParser
+from ipaplatform.constants import constants
+from ipaplatform.tasks import tasks
 
 """
 A test gpg can be generated like this:
@@ -63,8 +65,6 @@ EOF
       --keyring /root/backup.pub --list-secret-keys
 """
 
-BACKUP_DIR = paths.IPA_BACKUP_DIR
-
 
 def encrypt_file(filename, keyring, remove_original=True):
     source = filename
@@ -85,9 +85,9 @@ def encrypt_file(filename, keyring, remove_original=True):
     args.append('-e')
     args.append(source)
 
-    (stdout, stderr, rc) = run(args, raiseonerr=False)
-    if rc != 0:
-        raise admintool.ScriptError('gpg failed: %s' % stderr)
+    result = run(args, raiseonerr=False)
+    if result.returncode != 0:
+        raise admintool.ScriptError('gpg failed: %s' % result.error_log)
 
     if remove_original:
         os.unlink(source)
@@ -105,12 +105,10 @@ class Backup(admintool.AdminTool):
 
     dirs = (paths.IPA_HTML_DIR,
         paths.ROOT_PKI,
-        paths.ETC_PKI_CA_DIR,
         paths.PKI_TOMCAT,
         paths.SYSCONFIG_PKI,
         paths.HTTPD_ALIAS_DIR,
         paths.VAR_LIB_PKI_DIR,
-        paths.VAR_LIB_PKI_CA_DIR,
         paths.SYSRESTORE,
         paths.IPA_CLIENT_SYSRESTORE,
         paths.IPA_DNSSEC_DIR,
@@ -126,14 +124,12 @@ class Backup(admintool.AdminTool):
         paths.NAMED_CONF,
         paths.NAMED_KEYTAB,
         paths.RESOLV_CONF,
-        paths.SYSCONFIG_PKI_CA_DIR,
         paths.SYSCONFIG_PKI_TOMCAT,
         paths.SYSCONFIG_DIRSRV,
         paths.SYSCONFIG_NTPD,
         paths.SYSCONFIG_KRB5KDC_DIR,
-        paths.SYSCONFIG_PKI_CA_PKI_CA_DIR,
         paths.SYSCONFIG_IPA_DNSKEYSYNCD,
-        paths.SYSOCNFIG_IPA_ODS_EXPORTER,
+        paths.SYSCONFIG_IPA_ODS_EXPORTER,
         paths.SYSCONFIG_NAMED,
         paths.SYSCONFIG_ODS,
         paths.ETC_SYSCONFIG_AUTHCONFIG,
@@ -147,6 +143,7 @@ class Backup(admintool.AdminTool):
         paths.LIMITS_CONF,
         paths.HTTPD_PASSWORD_CONF,
         paths.IPA_KEYTAB,
+        paths.HTTPD_IPA_KDCPROXY_CONF,
         paths.HTTPD_IPA_PKI_PROXY_CONF,
         paths.HTTPD_IPA_REWRITE_CONF,
         paths.HTTPD_NSS_CONF,
@@ -160,7 +157,8 @@ class Backup(admintool.AdminTool):
         paths.NTP_CONF,
         paths.SMB_CONF,
         paths.SAMBA_KEYTAB,
-        paths.CA_AGENT_P12,
+        paths.DOGTAG_ADMIN_P12,
+        paths.KRA_AGENT_PEM,
         paths.CACERT_P12,
         paths.KRB5KDC_KDC_CONF,
         paths.SYSTEMD_IPA_SERVICE,
@@ -170,33 +168,41 @@ class Backup(admintool.AdminTool):
         paths.SVC_LIST_FILE,
         paths.OPENDNSSEC_CONF_FILE,
         paths.OPENDNSSEC_KASP_FILE,
+        paths.OPENDNSSEC_ZONELIST_FILE,
+        paths.OPENDNSSEC_KASP_DB,
         paths.DNSSEC_SOFTHSM2_CONF,
         paths.DNSSEC_SOFTHSM_PIN_SO,
         paths.IPA_ODS_EXPORTER_KEYTAB,
         paths.IPA_DNSKEYSYNCD_KEYTAB,
+        paths.HOSTS,
     ) + tuple(
-        os.path.join(base, file)
-        for base in (paths.NSS_DB_DIR, paths.IPA_NSSDB_DIR)
+        os.path.join(paths.IPA_NSSDB_DIR, file)
         for file in ('cert8.db', 'key3.db', 'secmod.db')
     )
 
     logs=(
-      paths.PKI_CA_LOG_DIR,
       paths.VAR_LOG_PKI_DIR,
-      paths.VAR_LOG_SLAPD_PKI_IPA_DIR,
       paths.VAR_LOG_HTTPD_DIR,
       paths.IPASERVER_INSTALL_LOG,
       paths.KADMIND_LOG,
-      paths.PKI_CA_INSTALL_LOG,
       paths.MESSAGES,
       paths.IPACLIENT_INSTALL_LOG,
       paths.LOG_SECURE,
       paths.IPASERVER_UNINSTALL_LOG,
-      paths.PKI_CA_UNINSTALL_LOG,
       paths.IPACLIENT_UNINSTALL_LOG,
       paths.NAMED_RUN,
     )
 
+    required_dirs=(
+      paths.TOMCAT_TOPLEVEL_DIR,
+      paths.TOMCAT_CA_DIR,
+      paths.TOMCAT_SIGNEDAUDIT_DIR,
+      paths.TOMCAT_CA_ARCHIVE_DIR,
+      paths.TOMCAT_KRA_DIR,
+      paths.TOMCAT_KRA_SIGNEDAUDIT_DIR,
+      paths.TOMCAT_KRA_ARCHIVE_DIR,
+    )
+
     def __init__(self, options, args):
         super(Backup, self).__init__(options, args)
         self._conn = None
@@ -258,14 +264,14 @@ class Backup(admintool.AdminTool):
 
         self.log.info("Preparing backup on %s", api.env.host)
 
-        pent = pwd.getpwnam(DS_USER)
+        pent = pwd.getpwnam(constants.DS_USER)
 
         self.top_dir = tempfile.mkdtemp("ipa")
         os.chown(self.top_dir, pent.pw_uid, pent.pw_gid)
-        os.chmod(self.top_dir, 0750)
+        os.chmod(self.top_dir, 0o750)
         self.dir = os.path.join(self.top_dir, "ipa")
-        os.mkdir(self.dir, 0750)
-
+        os.mkdir(self.dir)
+        os.chmod(self.dir, 0o750)
         os.chown(self.dir, pent.pw_uid, pent.pw_gid)
 
         self.header = os.path.join(self.top_dir, 'header')
@@ -290,13 +296,18 @@ class Backup(admintool.AdminTool):
                 self.log.info('Stopping IPA services')
                 run(['ipactl', 'stop'])
 
-            for instance in [realm_to_serverid(api.env.realm), 'PKI-IPA']:
-                if os.path.exists(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % instance):
-                    if os.path.exists(paths.SLAPD_INSTANCE_DB_DIR_TEMPLATE % (instance, 'ipaca')):
-                        self.db2ldif(instance, 'ipaca', online=options.online)
-                    self.db2ldif(instance, 'userRoot', online=options.online)
-                    self.db2bak(instance, online=options.online)
+            instance = installutils.realm_to_serverid(api.env.realm)
+            if os.path.exists(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE %
+                              instance):
+                if os.path.exists(paths.SLAPD_INSTANCE_DB_DIR_TEMPLATE %
+                                  (instance, 'ipaca')):
+                    self.db2ldif(instance, 'ipaca', online=options.online)
+                self.db2ldif(instance, 'userRoot', online=options.online)
+                self.db2bak(instance, online=options.online)
             if not options.data_only:
+                # create backup of auth configuration
+                auth_backup_path = os.path.join(paths.VAR_LIB_IPA, 'auth_backup')
+                tasks.backup_auth_configuration(auth_backup_path)
                 self.file_backup(options)
             self.finalize_backup(options.data_only, options.gpg, options.gpg_keyring)
 
@@ -311,7 +322,7 @@ class Backup(admintool.AdminTool):
         finally:
             try:
                 os.chdir(cwd)
-            except Exception, e:
+            except Exception as e:
                 self.log.error('Cannot change directory to %s: %s' % (cwd, e))
             shutil.rmtree(self.top_dir)
 
@@ -320,31 +331,21 @@ class Backup(admintool.AdminTool):
         '''
         Add instance-specific files and directories.
 
-        NOTE: this adds some things that may not get backed up, like the PKI-IPA
-              instance.
+        NOTE: this adds some things that may not get backed up.
         '''
-        for dir in [
-                paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % realm_to_serverid(api.env.realm),
-                paths.VAR_LIB_DIRSRV_INSTANCE_SCRIPTS_TEMPLATE % realm_to_serverid(api.env.realm),
-                paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % realm_to_serverid(api.env.realm),
-                paths.VAR_LIB_SLAPD_PKI_IPA_DIR_TEMPLATE,
-                paths.USR_LIB_SLAPD_PKI_IPA_DIR,
-                paths.ETC_SLAPD_PKI_IPA_DIR,
-                paths.VAR_LIB_SLAPD_PKI_IPA_DIR_TEMPLATE,
-                self.__find_scripts_dir('PKI-IPA'),
-            ]:
+        serverid = installutils.realm_to_serverid(api.env.realm)
+
+        for dir in [paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % serverid,
+                    paths.VAR_LIB_DIRSRV_INSTANCE_SCRIPTS_TEMPLATE % serverid,
+                    paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % serverid]:
             if os.path.exists(dir):
                 self.dirs.append(dir)
 
-        for file in [
-                paths.SYSCONFIG_DIRSRV_INSTANCE % realm_to_serverid(api.env.realm),
-                paths.SYSCONFIG_DIRSRV_PKI_IPA_DIR]:
-            if os.path.exists(file):
-                self.files.append(file)
+        file = paths.SYSCONFIG_DIRSRV_INSTANCE % serverid
+        if os.path.exists(file):
+            self.files.append(file)
 
-        for log in [
-              paths.VAR_LOG_DIRSRV_INSTANCE_TEMPLATE % realm_to_serverid(api.env.realm),]:
-            self.logs.append(log)
+        self.logs.append(paths.VAR_LOG_DIRSRV_INSTANCE_TEMPLATE % serverid)
 
 
     def get_connection(self):
@@ -362,7 +363,7 @@ class Backup(admintool.AdminTool):
         try:
             pw_name = pwd.getpwuid(os.geteuid()).pw_name
             self._conn.do_external_bind(pw_name)
-        except Exception, e:
+        except Exception as e:
             self.log.error("Unable to bind to LDAP server %s: %s" %
                 (self._conn.host, e))
 
@@ -405,20 +406,21 @@ class Backup(admintool.AdminTool):
 
             try:
                 conn.add_entry(ent)
-            except Exception, e:
+            except Exception as e:
                 raise admintool.ScriptError('Unable to add LDIF task: %s'
                     % e)
 
             self.log.info("Waiting for LDIF to finish")
             wait_for_task(conn, dn)
         else:
-            args = ['%s/db2ldif' % self.__find_scripts_dir(instance),
+            args = [paths.DB2LDIF,
+                    '-Z', instance,
                     '-r',
                     '-n', backend,
                     '-a', ldiffile]
-            (stdout, stderr, rc) = run(args, raiseonerr=False)
-            if rc != 0:
-                self.log.critical("db2ldif failed: %s", stderr)
+            result = run(args, raiseonerr=False)
+            if result.returncode != 0:
+                self.log.critical('db2ldif failed: %s', result.error_log)
 
         # Move the LDIF backup to our location
         shutil.move(ldiffile, os.path.join(self.dir, ldifname))
@@ -452,17 +454,17 @@ class Backup(admintool.AdminTool):
 
             try:
                 conn.add_entry(ent)
-            except Exception, e:
+            except Exception as e:
                 raise admintool.ScriptError('Unable to to add backup task: %s'
                     % e)
 
             self.log.info("Waiting for BAK to finish")
             wait_for_task(conn, dn)
         else:
-            args = ['%s/db2bak' % self.__find_scripts_dir(instance), bakdir]
-            (stdout, stderr, rc) = run(args, raiseonerr=False)
-            if rc != 0:
-                self.log.critical("db2bak failed: %s" % stderr)
+            args = [paths.DB2BAK, bakdir, '-Z', instance]
+            result = run(args, raiseonerr=False)
+            if result.returncode != 0:
+                self.log.critical('db2bak failed: %s', result.error_log)
 
         shutil.move(bakdir, self.dir)
 
@@ -472,13 +474,15 @@ class Backup(admintool.AdminTool):
         def verify_directories(dirs):
             return [s for s in dirs if os.path.exists(s)]
 
+        tarfile = os.path.join(self.dir, 'files.tar')
+
         self.log.info("Backing up files")
         args = ['tar',
                 '--exclude=/var/lib/ipa/backup',
                 '--xattrs',
                 '--selinux',
-                '-czf',
-                os.path.join(self.dir, 'files.tar')
+                '-cf',
+                tarfile
                ]
 
         args.extend(verify_directories(self.dirs))
@@ -487,9 +491,45 @@ class Backup(admintool.AdminTool):
         if options.logs:
             args.extend(verify_directories(self.logs))
 
-        (stdout, stderr, rc) = run(args, raiseonerr=False)
-        if rc != 0:
-            raise admintool.ScriptError('tar returned non-zero %d: %s' % (rc, stdout))
+        result = run(args, raiseonerr=False)
+        if result.returncode != 0:
+            raise admintool.ScriptError('tar returned non-zero code %d: %s' %
+                                        (result.returncode, result.error_log))
+
+        # Backup the necessary directory structure. This is a separate
+        # call since we are using the '--no-recursion' flag to store
+        # the directory structure only, no files.
+        missing_directories = verify_directories(self.required_dirs)
+
+        if missing_directories:
+            args = ['tar',
+                    '--exclude=/var/lib/ipa/backup',
+                    '--xattrs',
+                    '--selinux',
+                    '--no-recursion',
+                    '-rf',  # -r appends to an existing archive
+                    tarfile,
+                   ]
+            args.extend(missing_directories)
+
+            result = run(args, raiseonerr=False)
+            if result.returncode != 0:
+                raise admintool.ScriptError(
+                    'tar returned non-zero code %d '
+                    'when adding directory structure: %s' %
+                    (result.returncode, result.error_log))
+
+        # Compress the archive. This is done separately, since 'tar' cannot
+        # append to a compressed archive.
+        result = run(['gzip', tarfile], raiseonerr=False)
+        if result.returncode != 0:
+            raise admintool.ScriptError(
+                'gzip returned non-zero code %d '
+                'when compressing the backup: %s' %
+                (result.returncode, result.error_log))
+
+        # Rename the archive back to files.tar to preserve compatibility
+        os.rename(os.path.join(self.dir, 'files.tar.gz'), tarfile)
 
 
     def create_header(self, data_only):
@@ -516,7 +556,7 @@ class Backup(admintool.AdminTool):
         except errors.NetworkError:
             self.log.critical(
               "Unable to obtain list of master services, continuing anyway")
-        except Exception, e:
+        except Exception as e:
             self.log.error("Failed to read services from '%s': %s" %
                 (conn.host, e))
         else:
@@ -541,13 +581,14 @@ class Backup(admintool.AdminTool):
         '''
 
         if data_only:
-            backup_dir = os.path.join(BACKUP_DIR, time.strftime('ipa-data-%Y-%m-%d-%H-%M-%S'))
+            backup_dir = os.path.join(paths.IPA_BACKUP_DIR, time.strftime('ipa-data-%Y-%m-%d-%H-%M-%S'))
             filename = os.path.join(backup_dir, "ipa-data.tar")
         else:
-            backup_dir = os.path.join(BACKUP_DIR, time.strftime('ipa-full-%Y-%m-%d-%H-%M-%S'))
+            backup_dir = os.path.join(paths.IPA_BACKUP_DIR, time.strftime('ipa-full-%Y-%m-%d-%H-%M-%S'))
             filename = os.path.join(backup_dir, "ipa-full.tar")
 
-        os.mkdir(backup_dir, 0700)
+        os.mkdir(backup_dir)
+        os.chmod(backup_dir, 0o700)
 
         cwd = os.getcwd()
         os.chdir(self.dir)
@@ -558,9 +599,11 @@ class Backup(admintool.AdminTool):
                 filename,
                 '.'
                ]
-        (stdout, stderr, rc) = run(args, raiseonerr=False)
-        if rc != 0:
-            raise admintool.ScriptError('tar returned non-zero %d: %s' % (rc, stdout))
+        result = run(args, raiseonerr=False)
+        if result.returncode != 0:
+            raise admintool.ScriptError(
+                'tar returned non-zero code %s: %s' %
+                (result.returncode, result.error_log))
 
         if encrypt:
             self.log.info('Encrypting %s' % filename)
@@ -569,17 +612,3 @@ class Backup(admintool.AdminTool):
         shutil.move(self.header, backup_dir)
 
         self.log.info('Backed up to %s', backup_dir)
-
-    def __find_scripts_dir(self, instance):
-        """
-        IPA stores its 389-ds scripts in a different directory than dogtag
-        does so we need to probe for it.
-        """
-        if instance != 'PKI-IPA':
-            return os.path.join(paths.VAR_LIB_DIRSRV, 'scripts-%s' % instance)
-        else:
-            if sys.maxsize > 2**32L:
-                libpath = 'lib64'
-            else:
-                libpath = 'lib'
-            return os.path.join(paths.USR_DIR, libpath, 'dirsrv', 'slapd-PKI-IPA')
