@@ -18,27 +18,44 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import absolute_import
+from __future__ import print_function
+
 import os
 import shutil
 import tempfile
 import time
 from optparse import OptionGroup, SUPPRESS_HELP
-from ConfigParser import SafeConfigParser
 
 import dns.resolver
+from six.moves.configparser import SafeConfigParser
 
 from ipaserver.install import certs, installutils, bindinstance, dsinstance
 from ipaserver.install.replication import enable_replication_version_checking
 from ipaserver.plugins.ldap2 import ldap2
 from ipaserver.install.bindinstance import (
     add_zone, add_fwd_rr, add_ptr_rr, dns_container_exists)
-from ipapython import ipautil, admintool, dogtag
+from ipapython import ipautil, admintool
 from ipapython.dn import DN
 from ipapython import version
 from ipalib import api
 from ipalib import errors
 from ipaplatform.paths import paths
-from ipalib.constants import CACERT
+from ipalib.constants import CACERT, DOMAIN_LEVEL_0
+
+
+UNSUPPORTED_DOMAIN_LEVEL_TEMPLATE = """
+Replica creation using '{command_name}' to generate replica file
+is supported only in {domain_level}-level IPA domain.
+
+The current IPA domain level is {curr_domain_level} and thus the replica must
+be created by promoting an existing IPA client.
+
+To set up a replica use the following procedure:
+    1.) set up a client on the host using 'ipa-client-install'
+    2.) promote the client to replica running 'ipa-replica-install'
+        *without* replica file specified
+"""
 
 
 class ReplicaPrepare(admintool.AdminTool):
@@ -63,6 +80,11 @@ class ReplicaPrepare(admintool.AdminTool):
         parser.add_option("--no-reverse", dest="no_reverse",
             action="store_true", default=False,
             help="do not create reverse DNS zone")
+        parser.add_option("--auto-reverse", dest="auto_reverse", default=False,
+            action="store_true", help="create necessary DNS zones")
+        parser.add_option("--allow-zone-overlap", dest="allow_zone_overlap",
+            action="store_true", default=False, help="create DNS "
+            "zone even if it already exists")
         parser.add_option("--no-pkinit", dest="setup_pkinit",
             action="store_false", default=True,
             help="disables pkinit setup steps")
@@ -158,11 +180,13 @@ class ReplicaPrepare(admintool.AdminTool):
         api.bootstrap(in_server=True)
         api.finalize()
 
+        self.check_for_supported_domain_level()
+
         if api.env.host == self.replica_fqdn:
             raise admintool.ScriptError("You can't create a replica on itself")
 
         config_dir = dsinstance.config_dirname(
-            dsinstance.realm_to_serverid(api.env.realm))
+            installutils.realm_to_serverid(api.env.realm))
         if not ipautil.dir_exists(config_dir):
             raise admintool.ScriptError(
                 "could not find directory instance: %s" % config_dir)
@@ -195,8 +219,13 @@ class ReplicaPrepare(admintool.AdminTool):
             conn = api.Backend.ldap2
             conn.connect(bind_dn=DN(('cn', 'directory manager')),
                          bind_pw=self.dirman_password)
+
             entry_attrs = conn.get_ipa_config()
+            self.subject_base = entry_attrs.get(
+                'ipacertificatesubjectbase', [None])[0]
+
             ca_enabled = api.Command.ca_is_enabled()['result']
+
             conn.disconnect()
         except errors.ACIError:
             raise admintool.ScriptError("The password provided is incorrect "
@@ -204,7 +233,7 @@ class ReplicaPrepare(admintool.AdminTool):
         except errors.LDAPError:
             raise admintool.ScriptError(
                 "Unable to connect to LDAP server %s" % api.env.host)
-        except errors.DatabaseError, e:
+        except errors.DatabaseError as e:
             raise admintool.ScriptError(e.desc)
 
         if not ca_enabled and not options.http_cert_files:
@@ -213,15 +242,13 @@ class ReplicaPrepare(admintool.AdminTool):
                 "--http-cert-file, --dirsrv-cert-file options to provide "
                 "custom certificates.")
 
-        self.subject_base = entry_attrs.get(
-            'ipacertificatesubjectbase', [None])[0]
         if self.subject_base is not None:
             self.subject_base = DN(self.subject_base)
 
         # Validate more options using the password
         try:
             installutils.verify_fqdn(self.replica_fqdn, local_hostname=False)
-        except installutils.BadHostError, e:
+        except installutils.BadHostError as e:
             msg = str(e)
             if isinstance(e, installutils.HostLookupError):
                 if not options.ip_addresses:
@@ -229,8 +256,9 @@ class ReplicaPrepare(admintool.AdminTool):
                             api.env.host, api.env.basedn,
                             dm_password=self.dirman_password,
                             ldapi=True, realm=api.env.realm):
-                        self.log.info('Add the --ip-address argument to '
-                            'create a DNS entry.')
+                        self.log.info('You might use the --ip-address option '
+                                      'to create a DNS entry if the DNS zone '
+                                      'is managed by IPA.')
                     raise
                 else:
                     # The host doesn't exist in DNS but we're adding it.
@@ -258,6 +286,14 @@ class ReplicaPrepare(admintool.AdminTool):
             options.reverse_zones = bindinstance.check_reverse_zones(
                 options.ip_addresses, options.reverse_zones, options, False,
                 True)
+
+            host, zone = self.replica_fqdn.split('.', 1)
+            if not bindinstance.dns_zone_exists(zone, api=api):
+                self.log.error("DNS zone %s does not exist in IPA managed DNS "
+                               "server. Either create DNS zone or omit "
+                               "--ip-address option." % zone)
+                raise admintool.ScriptError("Cannot add DNS record")
+
             if disconnect:
                 api.Backend.ldap2.disconnect()
 
@@ -311,8 +347,7 @@ class ReplicaPrepare(admintool.AdminTool):
                 "Apache Server SSL certificate and Directory Server SSL "
                  "certificate are not signed by the same CA certificate")
 
-        if (not ipautil.file_exists(
-                    dogtag.configured_constants().CS_CFG_PATH) and
+        if (not ipautil.file_exists(paths.CA_CS_CFG_PATH) and
                 options.dirsrv_pin is None):
             self.log.info("If you installed IPA with your own certificates "
                 "using PKCS#12 files you must provide PKCS#12 files for any "
@@ -331,7 +366,8 @@ class ReplicaPrepare(admintool.AdminTool):
 
         self.top_dir = tempfile.mkdtemp("ipa")
         self.dir = os.path.join(self.top_dir, "realm_info")
-        os.mkdir(self.dir, 0700)
+        os.mkdir(self.dir)
+        os.chmod(self.dir, 0o700)
         try:
             self.copy_ds_certificate()
 
@@ -388,7 +424,7 @@ class ReplicaPrepare(admintool.AdminTool):
             port_fname = os.path.join(
                 self.dir, "dogtag_directory_port.txt")
             with open(port_fname, "w") as fd:
-                fd.write("%s\n" % str(dogtag.configured_constants().DS_PORT))
+                fd.write("389\n")
 
     def copy_httpd_certificate(self):
         options = self.options
@@ -425,15 +461,10 @@ class ReplicaPrepare(admintool.AdminTool):
         self.log.info("Copying additional files")
 
         self.copy_info_file(CACERT, "ca.crt")
-        preferences_filename = paths.PREFERENCES_HTML
-        if ipautil.file_exists(preferences_filename):
-            self.copy_info_file(preferences_filename, "preferences.html")
-        jar_filename = paths.CONFIGURE_JAR
-        if ipautil.file_exists(jar_filename):
-            self.copy_info_file(jar_filename, "configure.jar")
         cacert_filename = paths.CACERT_PEM
         if ipautil.file_exists(cacert_filename):
             self.copy_info_file(cacert_filename, "cacert.pem")
+        self.copy_info_file(paths.IPA_DEFAULT_CONF, "default.conf")
 
     def save_config(self):
         self.log.info("Finalizing configuration")
@@ -460,7 +491,7 @@ class ReplicaPrepare(admintool.AdminTool):
         ipautil.encrypt_file(
             replicafile, encfile, self.dirman_password, self.top_dir)
 
-        os.chmod(encfile, 0600)
+        os.chmod(encfile, 0o600)
 
         installutils.remove_file(replicafile)
 
@@ -474,11 +505,6 @@ class ReplicaPrepare(admintool.AdminTool):
             api.Backend.ldap2.connect(
                 bind_dn=DN(('cn', 'Directory Manager')),
                 bind_pw=self.dirman_password)
-        try:
-            add_zone(domain)
-        except errors.PublicError, e:
-            raise admintool.ScriptError(
-                "Could not create forward DNS zone for the replica: %s" % e)
 
         for reverse_zone in options.reverse_zones:
             self.log.info("Adding reverse zone %s", reverse_zone)
@@ -488,17 +514,17 @@ class ReplicaPrepare(admintool.AdminTool):
             ip_address = str(ip)
             try:
                 add_fwd_rr(domain, name, ip_address)
-            except errors.PublicError, e:
+            except errors.PublicError as e:
                 raise admintool.ScriptError(
-                    "Could not add forward DNS record for the replica: %s" % e)
+                    "Could not add A/AAAA DNS record for the replica: %s" % e)
 
             if not options.no_reverse:
                 reverse_zone = bindinstance.find_reverse_zone(ip)
                 try:
                     add_ptr_rr(reverse_zone, ip_address, self.replica_fqdn)
-                except errors.PublicError, e:
+                except errors.PublicError as e:
                     raise admintool.ScriptError(
-                        "Could not add reverse DNS record for the replica: %s"
+                        "Could not add PTR DNS record for the replica: %s"
                         % e)
 
     def check_dns(self, replica_fqdn):
@@ -535,7 +561,7 @@ class ReplicaPrepare(admintool.AdminTool):
 
         self.log.info('Waiting for %s A or AAAA record to be resolvable',
                       replica_fqdn)
-        print 'This can be safely interrupted (Ctrl+C)'
+        print('This can be safely interrupted (Ctrl+C)')
 
         try:
             while not self.check_dns(replica_fqdn):
@@ -555,7 +581,7 @@ class ReplicaPrepare(admintool.AdminTool):
         self.log.debug('Copying %s to %s', source, dest_path)
         try:
             shutil.copy(source, dest_path)
-        except IOError, e:
+        except IOError as e:
             raise admintool.ScriptError("File copy failed: %s" % e)
 
     def remove_info_file(self, filename):
@@ -599,7 +625,7 @@ class ReplicaPrepare(admintool.AdminTool):
                         nickname, os.path.join(self.dir, "kdc.pem"))
                 else:
                     db.export_pkcs12(pkcs12_fname, passwd_fname, nickname)
-            except ipautil.CalledProcessError, e:
+            except ipautil.CalledProcessError as e:
                 self.log.info("error exporting Server certificate: %s", e)
                 installutils.remove_file(pkcs12_fname)
                 installutils.remove_file(passwd_fname)
@@ -615,7 +641,7 @@ class ReplicaPrepare(admintool.AdminTool):
             orig_filename = passwd_fname + ".orig"
             if ipautil.file_exists(orig_filename):
                 installutils.remove_file(orig_filename)
-        except errors.CertificateOperationError, e:
+        except errors.CertificateOperationError as e:
             raise admintool.ScriptError(str(e))
 
     def export_ra_pkcs12(self):
@@ -633,7 +659,7 @@ class ReplicaPrepare(admintool.AdminTool):
             os.remove(agent_name)
 
     def update_pki_admin_password(self):
-        ldap = ldap2(shared_instance=False)
+        ldap = ldap2(api)
         ldap.connect(
             bind_dn=DN(('cn', 'directory manager')),
             bind_pw=self.dirman_password
@@ -662,3 +688,26 @@ class ReplicaPrepare(admintool.AdminTool):
             '-w', dm_pwd_fd.name,
             '-o', ca_file
         ])
+
+    def check_for_supported_domain_level(self):
+        """
+        check if we are in 0-level topology. If not, raise an error pointing
+        the user to the replica promotion pathway
+        """
+
+        domain_level = dsinstance.get_domain_level(api)
+        if domain_level > DOMAIN_LEVEL_0:
+            self.log.error(
+                UNSUPPORTED_DOMAIN_LEVEL_TEMPLATE.format(
+                    command_name=self.command_name,
+                    domain_level=DOMAIN_LEVEL_0,
+                    curr_domain_level=domain_level
+                )
+            )
+            raise errors.InvalidDomainLevelError(
+                reason="'{command}' is allowed only in domain level "
+                "{prep_domain_level}".format(
+                    command=self.command_name,
+                    prep_domain_level=DOMAIN_LEVEL_0
+                )
+            )
