@@ -23,26 +23,29 @@ import shutil
 import tempfile
 import time
 import pwd
-from ConfigParser import SafeConfigParser
 import ldif
 import itertools
+import locale
 
-from ipalib import api, errors, constants
-from ipapython import version, ipautil, certdb, dogtag
+from six.moves.configparser import SafeConfigParser
+import six
+
+from ipalib import api, errors
+from ipalib.constants import FQDN
+from ipapython import version, ipautil, certdb
 from ipapython.ipautil import run, user_input
 from ipapython import admintool
 from ipapython.dn import DN
-from ipaserver.install.dsinstance import (realm_to_serverid,
-                                          create_ds_user, DS_USER)
-from ipaserver.install.cainstance import PKI_USER, create_ca_user
+from ipaserver.install.dsinstance import create_ds_user
+from ipaserver.install.cainstance import create_ca_user
 from ipaserver.install.replication import (wait_for_task, ReplicationManager,
                                            get_cs_replication_manager)
 from ipaserver.install import installutils
 from ipaserver.install import dsinstance, httpinstance, cainstance
 from ipapython import ipaldap
 import ipapython.errors
+from ipaplatform.constants import constants
 from ipaplatform.tasks import tasks
-from ipaserver.install.ipa_backup import BACKUP_DIR
 from ipaplatform import services
 from ipaplatform.paths import paths
 
@@ -59,10 +62,10 @@ def recursive_chown(path, uid, gid):
     for root, dirs, files in os.walk(path):
         for dir in dirs:
             os.chown(os.path.join(root, dir), uid, gid)
-            os.chmod(os.path.join(root, dir), 0750)
+            os.chmod(os.path.join(root, dir), 0o750)
         for file in files:
             os.chown(os.path.join(root, file), uid, gid)
-            os.chmod(os.path.join(root, file), 0640)
+            os.chmod(os.path.join(root, file), 0o640)
 
 
 def decrypt_file(tmpdir, filename, keyring):
@@ -89,9 +92,9 @@ def decrypt_file(tmpdir, filename, keyring):
     args.append('-d')
     args.append(source)
 
-    (stdout, stderr, rc) = run(args, raiseonerr=False)
-    if rc != 0:
-        raise admintool.ScriptError('gpg failed: %s' % stderr)
+    result = run(args, raiseonerr=False)
+    if result.returncode != 0:
+        raise admintool.ScriptError('gpg failed: %s' % result.error_log)
 
     return dest
 
@@ -106,7 +109,7 @@ class RemoveRUVParser(ldif.LDIFParser):
         objectclass = None
         nsuniqueid = None
 
-        for name, value in entry.iteritems():
+        for name, value in entry.items():
             name = name.lower()
             if name == 'objectclass':
                 objectclass = [x.lower() for x in value]
@@ -130,6 +133,14 @@ class Restore(admintool.AdminTool):
 
     description = "Restore IPA files and databases."
 
+    # directories and files listed here will be removed from filesystem before
+    # files from backup are copied
+    DIRS_TO_BE_REMOVED = [
+        paths.DNSSEC_TOKENS_DIR,
+    ]
+
+    FILES_TO_BE_REMOVED = []
+
     def __init__(self, options, args):
         super(Restore, self).__init__(options, args)
         self._conn = None
@@ -173,7 +184,7 @@ class Restore(admintool.AdminTool):
 
         dirname = self.args[0]
         if not os.path.isabs(dirname):
-            dirname = os.path.join(BACKUP_DIR, dirname)
+            dirname = os.path.join(paths.IPA_BACKUP_DIR, dirname)
         if not os.path.isdir(dirname):
             parser.error("must provide path to backup directory")
 
@@ -205,10 +216,10 @@ class Restore(admintool.AdminTool):
 
         self.backup_dir = self.args[0]
         if not os.path.isabs(self.backup_dir):
-            self.backup_dir = os.path.join(BACKUP_DIR, self.backup_dir)
+            self.backup_dir = os.path.join(paths.IPA_BACKUP_DIR, self.backup_dir)
 
         self.log.info("Preparing restore from %s on %s",
-                      self.backup_dir, constants.FQDN)
+                      self.backup_dir, FQDN)
 
         self.header = os.path.join(self.backup_dir, 'header')
 
@@ -271,10 +282,10 @@ class Restore(admintool.AdminTool):
         self.log.info("Performing %s restore from %s backup" %
                       (restore_type, self.backup_type))
 
-        if self.backup_host != constants.FQDN:
+        if self.backup_host != FQDN:
             raise admintool.ScriptError(
                 "Host name %s does not match backup name %s" %
-                (constants.FQDN, self.backup_host))
+                (FQDN, self.backup_host))
 
         if self.backup_ipa_version != str(version.VERSION):
             self.log.warning(
@@ -287,15 +298,15 @@ class Restore(admintool.AdminTool):
                 raise admintool.ScriptError("Aborted")
 
         create_ds_user()
-        pent = pwd.getpwnam(DS_USER)
+        pent = pwd.getpwnam(constants.DS_USER)
 
         # Temporary directory for decrypting files before restoring
         self.top_dir = tempfile.mkdtemp("ipa")
         os.chown(self.top_dir, pent.pw_uid, pent.pw_gid)
-        os.chmod(self.top_dir, 0750)
+        os.chmod(self.top_dir, 0o750)
         self.dir = os.path.join(self.top_dir, "ipa")
-        os.mkdir(self.dir, 0750)
-
+        os.mkdir(self.dir)
+        os.chmod(self.dir, 0o750)
         os.chown(self.dir, pent.pw_uid, pent.pw_gid)
 
         cwd = os.getcwd()
@@ -358,15 +369,17 @@ class Restore(admintool.AdminTool):
                     dirsrv.start(capture_output=False)
             else:
                 self.log.info('Stopping IPA services')
-                (stdout, stderr, rc) = run(['ipactl', 'stop'], raiseonerr=False)
-                if rc not in [0, 6]:
-                    self.log.warn('Stopping IPA failed: %s' % stderr)
+                result = run(['ipactl', 'stop'], raiseonerr=False)
+                if result.returncode not in [0, 6]:
+                    self.log.warn('Stopping IPA failed: %s' % result.error_log)
 
                 self.restore_selinux_booleans()
 
+            http = httpinstance.HTTPInstance()
 
             # We do either a full file restore or we restore data.
             if restore_type == 'FULL':
+                self.remove_old_files()
                 if 'CA' in self.backup_services:
                     create_ca_user()
                 self.cert_restore_prepare()
@@ -374,12 +387,11 @@ class Restore(admintool.AdminTool):
                 self.cert_restore()
                 if 'CA' in self.backup_services:
                     self.__create_dogtag_log_dirs()
+                if http.is_kdcproxy_configured():
+                    httpinstance.create_kdcproxy_user()
 
             # Always restore the data from ldif
-            # If we are restoring PKI-IPA then we need to restore the
-            # userRoot backend in it and the main IPA instance. If we
-            # have a unified instance we need to restore both userRoot and
-            # ipaca.
+            # We need to restore both userRoot and ipaca.
             for instance, backend in databases:
                 self.ldif2db(instance, backend, online=options.online)
 
@@ -388,6 +400,10 @@ class Restore(admintool.AdminTool):
                     self.log.info('Starting Directory Server')
                     dirsrv.start(capture_output=False)
             else:
+                # restore access controll configuration
+                auth_backup_path = os.path.join(paths.VAR_LIB_IPA, 'auth_backup')
+                if os.path.exists(auth_backup_path):
+                    tasks.restore_auth_configuration(auth_backup_path)
                 # explicitly enable then disable the pki tomcatd service to
                 # re-register its instance. FIXME, this is really wierd.
                 services.knownservices.pki_tomcatd.enable()
@@ -398,12 +414,11 @@ class Restore(admintool.AdminTool):
                 self.log.info('Restarting SSSD')
                 sssd = services.service('sssd')
                 sssd.restart()
-                http = httpinstance.HTTPInstance()
                 http.remove_httpd_ccache()
         finally:
             try:
                 os.chdir(cwd)
-            except Exception, e:
+            except Exception as e:
                 self.log.error('Cannot change directory to %s: %s' % (cwd, e))
             shutil.rmtree(self.top_dir)
 
@@ -412,6 +427,13 @@ class Restore(admintool.AdminTool):
         '''
         Create an ldapi connection and bind to it using autobind as root.
         '''
+        instance_name = installutils.realm_to_serverid(api.env.realm)
+
+        if not services.knownservices.dirsrv.is_running(instance_name):
+            raise admintool.ScriptError(
+                "directory server instance is not running/configured"
+            )
+
         if self._conn is not None:
             return self._conn
 
@@ -423,7 +445,7 @@ class Restore(admintool.AdminTool):
         try:
             pw_name = pwd.getpwuid(os.geteuid()).pw_name
             self._conn.do_external_bind(pw_name)
-        except Exception, e:
+        except Exception as e:
             raise admintool.ScriptError('Unable to bind to LDAP server: %s'
                 % e)
         return self._conn
@@ -437,14 +459,14 @@ class Restore(admintool.AdminTool):
         '''
         try:
             conn = self.get_connection()
-        except Exception, e :
+        except Exception as e:
             self.log.error('Unable to get connection, skipping disabling agreements: %s' % e)
             return
         masters = []
         dn = DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
         try:
             entries = conn.get_entries(dn, conn.SCOPE_ONELEVEL)
-        except Exception, e:
+        except Exception as e:
             raise admintool.ScriptError(
                 "Failed to read master data: %s" % e)
         else:
@@ -457,7 +479,7 @@ class Restore(admintool.AdminTool):
             try:
                 repl = ReplicationManager(api.env.realm, master,
                                           self.dirman_password)
-            except Exception, e:
+            except Exception as e:
                 self.log.critical("Unable to disable agreement on %s: %s" % (master, e))
                 continue
 
@@ -482,7 +504,7 @@ class Restore(admintool.AdminTool):
                 try:
                     repl = get_cs_replication_manager(api.env.realm, master,
                                                       self.dirman_password)
-                except Exception, e:
+                except Exception as e:
                     self.log.critical("Unable to disable agreement on %s: %s" % (master, e))
                     continue
 
@@ -513,8 +535,9 @@ class Restore(admintool.AdminTool):
         srcldiffile = os.path.join(self.dir, ldifname)
 
         if not os.path.exists(ldifdir):
-            pent = pwd.getpwnam(DS_USER)
-            os.mkdir(ldifdir, 0770)
+            pent = pwd.getpwnam(constants.DS_USER)
+            os.mkdir(ldifdir)
+            os.chmod(ldifdir, 0o770)
             os.chown(ldifdir, pent.pw_uid, pent.pw_gid)
 
         ipautil.backup_file(ldiffile)
@@ -539,19 +562,25 @@ class Restore(admintool.AdminTool):
 
             try:
                 conn.add_entry(ent)
-            except Exception, e:
+            except Exception as e:
                 self.log.error("Unable to bind to LDAP server: %s" % e)
                 return
 
             self.log.info("Waiting for LDIF to finish")
             wait_for_task(conn, dn)
         else:
-            args = ['%s/ldif2db' % self.__find_scripts_dir(instance),
+            try:
+                os.makedirs(paths.VAR_LOG_DIRSRV_INSTANCE_TEMPLATE % instance)
+            except OSError as e:
+                pass
+
+            args = [paths.LDIF2DB,
+                    '-Z', instance,
                     '-i', ldiffile,
                     '-n', backend]
-            (stdout, stderr, rc) = run(args, raiseonerr=False)
-            if rc != 0:
-                self.log.critical("ldif2db failed: %s" % stderr)
+            result = run(args, raiseonerr=False)
+            if result.returncode != 0:
+                self.log.critical("ldif2db failed: %s" % result.error_log)
 
 
     def bak2db(self, instance, backend, online=True):
@@ -564,7 +593,7 @@ class Restore(admintool.AdminTool):
 
         instance here is a loaded term. It can mean either a separate
         389-ds install instance or a separate 389-ds backend. We only need
-        to treat PKI-IPA and ipaca specially.
+        to treat ipaca specially.
         '''
         if backend is not None:
             self.log.info('Restoring %s in %s' % (backend, instance))
@@ -591,21 +620,22 @@ class Restore(admintool.AdminTool):
 
             try:
                 conn.add_entry(ent)
-            except Exception, e:
+            except Exception as e:
                 raise admintool.ScriptError('Unable to bind to LDAP server: %s'
                     % e)
 
             self.log.info("Waiting for restore to finish")
             wait_for_task(conn, dn)
         else:
-            args = ['%s/bak2db' % self.__find_scripts_dir(instance),
+            args = [paths.BAK2DB,
+                    '-Z', instance,
                     os.path.join(self.dir, instance)]
             if backend is not None:
                 args.append('-n')
                 args.append(backend)
-            (stdout, stderr, rc) = run(args, raiseonerr=False)
-            if rc != 0:
-                self.log.critical("bak2db failed: %s" % stderr)
+            result = run(args, raiseonerr=False)
+            if result.returncode != 0:
+                self.log.critical("bak2db failed: %s" % result.error_log)
 
 
     def restore_default_conf(self):
@@ -625,12 +655,31 @@ class Restore(admintool.AdminTool):
                 paths.IPA_DEFAULT_CONF[1:],
                ]
 
-        (stdout, stderr, rc) = run(args, raiseonerr=False)
-        if rc != 0:
+        result = run(args, raiseonerr=False)
+        if result.returncode != 0:
             self.log.critical('Restoring %s failed: %s' %
-                              (paths.IPA_DEFAULT_CONF, stderr))
+                              (paths.IPA_DEFAULT_CONF, result.error_log))
         os.chdir(cwd)
 
+    def remove_old_files(self):
+        """
+        Removes all directories, files or temporal files that should be
+        removed before backup files are copied, to prevent errors.
+        """
+        for d in self.DIRS_TO_BE_REMOVED:
+            try:
+                shutil.rmtree(d)
+            except OSError as e:
+                if e.errno != 2:  # 2: dir does not exist
+                    self.log.warning("Could not remove directory: %s (%s)",
+                                     d, e)
+
+        for f in self.FILES_TO_BE_REMOVED:
+            try:
+                os.remove(f)
+            except OSError as e:
+                if e.errno != 2:  # 2: file does not exist
+                    self.log.warning("Could not remove file: %s (%s)", f, e)
 
     def file_restore(self, nologs=False):
         '''
@@ -652,9 +701,9 @@ class Restore(admintool.AdminTool):
             args.append('--exclude')
             args.append('var/log')
 
-        (stdout, stderr, rc) = run(args, raiseonerr=False)
-        if rc != 0:
-            self.log.critical('Restoring files failed: %s', stderr)
+        result = run(args, raiseonerr=False)
+        if result.returncode != 0:
+            self.log.critical('Restoring files failed: %s', result.error_log)
 
         os.chdir(cwd)
 
@@ -711,7 +760,7 @@ class Restore(admintool.AdminTool):
                ]
         run(args)
 
-        pent = pwd.getpwnam(DS_USER)
+        pent = pwd.getpwnam(constants.DS_USER)
         os.chown(self.top_dir, pent.pw_uid, pent.pw_gid)
         recursive_chown(self.dir, pent.pw_uid, pent.pw_gid)
 
@@ -719,21 +768,6 @@ class Restore(admintool.AdminTool):
             # We can remove the decoded tarball
             os.unlink(filename)
 
-
-    def __find_scripts_dir(self, instance):
-        """
-        IPA stores its 389-ds scripts in a different directory than dogtag
-        does so we need to probe for it.
-        """
-        if instance != 'PKI-IPA':
-            return os.path.join(paths.VAR_LIB_DIRSRV, 'scripts-%s' % instance)
-        else:
-            if sys.maxsize > 2**32L:
-                libpath = 'lib64'
-            else:
-                libpath = 'lib'
-            return os.path.join(paths.USR_DIR, libpath, 'dirsrv', 'slapd-PKI-IPA')
-
     def __create_dogtag_log_dirs(self):
         """
         If we are doing a full restore and the dogtag log directories do
@@ -743,11 +777,6 @@ class Restore(admintool.AdminTool):
         or a d10-based installation.
         """
         dirs = []
-        # dogtag 9
-        if (os.path.exists(paths.VAR_LIB_PKI_CA_DIR) and
-                not os.path.exists(paths.PKI_CA_LOG_DIR)):
-            dirs += [paths.PKI_CA_LOG_DIR,
-                     os.path.join(paths.PKI_CA_LOG_DIR, 'signedAudit')]
         # dogtag 10
         if (os.path.exists(paths.VAR_LIB_PKI_TOMCAT_DIR) and
                 not os.path.exists(paths.TOMCAT_TOPLEVEL_DIR)):
@@ -757,18 +786,20 @@ class Restore(admintool.AdminTool):
                      paths.TOMCAT_SIGNEDAUDIT_DIR]
 
         try:
-            pent = pwd.getpwnam(PKI_USER)
+            pent = pwd.getpwnam(constants.PKI_USER)
         except KeyError:
-            self.log.debug("No %s user exists, skipping CA directory creation" % PKI_USER)
+            self.log.debug("No %s user exists, skipping CA directory creation",
+                           constants.PKI_USER)
             return
         self.log.debug('Creating log directories for dogtag')
         for dir in dirs:
             try:
                 self.log.debug('Creating %s' % dir)
-                os.mkdir(dir, 0770)
+                os.mkdir(dir)
+                os.chmod(dir, 0o770)
                 os.chown(dir, pent.pw_uid, pent.pw_gid)
                 tasks.restore_context(dir)
-            except Exception, e:
+            except Exception as e:
                 # This isn't so fatal as to side-track the restore
                 self.log.error('Problem with %s: %s' % (dir, e))
 
@@ -787,12 +818,11 @@ class Restore(admintool.AdminTool):
             self.log.error('%s', e)
 
     def cert_restore_prepare(self):
-        cainstance.stop_tracking_certificates(
-            dogtag.configured_constants())
+        cainstance.CAInstance().stop_tracking_certificates()
         httpinstance.HTTPInstance().stop_tracking_certificates()
         try:
             dsinstance.DsInstance().stop_tracking_certificates(
-                realm_to_serverid(api.env.realm))
+                installutils.realm_to_serverid(api.env.realm))
         except OSError:
             # When IPA is not installed, DS NSS DB does not exist
             pass
@@ -807,23 +837,10 @@ class Restore(admintool.AdminTool):
         tasks.remove_ca_certs_from_systemwide_ca_store()
 
     def cert_restore(self):
-        if not os.path.exists(os.path.join(paths.IPA_NSSDB_DIR, 'cert8.db')):
-            certdb.create_ipa_nssdb()
-            ipa_db = certdb.NSSDatabase(paths.IPA_NSSDB_DIR)
-            sys_db = certdb.NSSDatabase(paths.NSS_DB_DIR)
-            for nickname, trust_flags in (('IPA CA', 'CT,C,C'),
-                                          ('External CA cert', 'C,,')):
-                try:
-                    cert = sys_db.get_cert(nickname)
-                except RuntimeError:
-                    pass
-                else:
-                    try:
-                        ipa_db.add_cert(cert, nickname, trust_flags)
-                    except ipautil.CalledProcessError as e:
-                        self.log.error(
-                            "Failed to add %s to %s: %s" %
-                            (nickname, paths.IPA_NSSDB_DIR, e))
+        try:
+            certdb.update_ipa_nssdb()
+        except RuntimeError as e:
+            self.log.error("%s", e)
 
         tasks.reload_systemwide_ca_store()
 
@@ -833,5 +850,5 @@ class Restore(admintool.AdminTool):
         api.bootstrap(in_server=False, context='restore', **overrides)
         api.finalize()
 
-        self.instances = [realm_to_serverid(api.env.realm), 'PKI-IPA']
+        self.instances = [installutils.realm_to_serverid(api.env.realm)]
         self.backends = ['userRoot', 'ipaca']
