@@ -24,6 +24,7 @@ import os.path
 import pwd
 import optparse
 
+from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipapython import admintool
 from ipapython.dn import DN
@@ -31,7 +32,7 @@ from ipapython.ipautil import user_input, write_tmp_file
 from ipalib import api, errors
 from ipalib.constants import CACERT
 from ipaserver.install import certs, dsinstance, httpinstance, installutils
-from ipaserver.plugins.ldap2 import ldap2
+
 
 class ServerCertInstall(admintool.AdminTool):
     command_name = 'ipa-server-certinstall'
@@ -114,7 +115,7 @@ class ServerCertInstall(admintool.AdminTool):
         conn.disconnect()
 
     def install_dirsrv_cert(self):
-        serverid = dsinstance.realm_to_serverid(api.env.realm)
+        serverid = installutils.realm_to_serverid(api.env.realm)
         dirname = dsinstance.config_dirname(serverid)
 
         conn = api.Backend.ldap2
@@ -147,11 +148,11 @@ class ServerCertInstall(admintool.AdminTool):
                                    'NSSNickname', server_cert)
 
         # Fix the database permissions
-        os.chmod(os.path.join(dirname, 'cert8.db'), 0640)
-        os.chmod(os.path.join(dirname, 'key3.db'), 0640)
-        os.chmod(os.path.join(dirname, 'secmod.db'), 0640)
+        os.chmod(os.path.join(dirname, 'cert8.db'), 0o640)
+        os.chmod(os.path.join(dirname, 'key3.db'), 0o640)
+        os.chmod(os.path.join(dirname, 'secmod.db'), 0o640)
 
-        pent = pwd.getpwnam("apache")
+        pent = pwd.getpwnam(constants.HTTPD_USER)
         os.chown(os.path.join(dirname, 'cert8.db'), 0, pent.pw_gid)
         os.chown(os.path.join(dirname, 'key3.db'), 0, pent.pw_gid)
         os.chown(os.path.join(dirname, 'secmod.db'), 0, pent.pw_gid)
@@ -177,7 +178,7 @@ class ServerCertInstall(admintool.AdminTool):
             if ca_enabled:
                 cdb.track_server_cert(server_cert, principal, cdb.passwd_fname,
                                       command)
-        except RuntimeError, e:
+        except RuntimeError as e:
             raise admintool.ScriptError(str(e))
 
         return server_cert
