@@ -17,6 +17,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import absolute_import
+from __future__ import print_function
+
 import shutil
 import fileinput
 import re
@@ -26,44 +29,27 @@ import pwd
 import socket
 import dns.name
 
-import service
-import installutils
+from ipaserver.install import service
+from ipaserver.install import installutils
 from ipapython import sysrestore
 from ipapython import ipautil
-from ipapython import services as ipaservices
 from ipapython import kernel_keyring
 from ipalib import errors
+from ipalib.constants import CACERT
 from ipapython.ipa_log_manager import *
 from ipapython.dn import DN
 
 from ipaserver.install import replication
-from ipaserver.install import dsinstance
+from ipaserver.install import ldapupdate
 
 import pyasn1.codec.ber.decoder
 import struct
 
-import certs
+from ipaserver.install import certs
 from distutils import version
-
-def update_key_val_in_file(filename, key, val):
-    if os.path.exists(filename):
-        pattern = "^[\s#]*%s\s*=\s*%s\s*" % (re.escape(key), re.escape(val))
-        p = re.compile(pattern)
-        for line in fileinput.input(filename):
-            if p.search(line):
-                fileinput.close()
-                return
-        fileinput.close()
-
-        pattern = "^[\s#]*%s\s*=" % re.escape(key)
-        p = re.compile(pattern)
-        for line in fileinput.input(filename, inplace=1):
-            if not p.search(line):
-                sys.stdout.write(line)
-        fileinput.close()
-    f = open(filename, "a")
-    f.write("%s=%s\n" % (key, val))
-    f.close()
+from ipaplatform.constants import constants
+from ipaplatform.tasks import tasks
+from ipaplatform.paths import paths
 
 class KpasswdInstance(service.SimpleServiceInstance):
     def __init__(self):
@@ -87,7 +73,7 @@ class KrbInstance(service.Service):
         if fstore:
             self.fstore = fstore
         else:
-            self.fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
+            self.fstore = sysrestore.FileStore(paths.SYSRESTORE)
 
     suffix = ipautil.dn_attribute_property('_suffix')
     subject_base = ipautil.dn_attribute_property('_subject_base')
@@ -125,13 +111,18 @@ class KrbInstance(service.Service):
             ipauniqueid=['autogenerate'],
             managedby=[host_dn],
         )
-        if 'krbpasswordexpiration' in service_entry.toDict():
+        if 'krbpasswordexpiration' in service_entry:
             host_entry['krbpasswordexpiration'] = service_entry[
                 'krbpasswordexpiration']
-        if 'krbticketflags' in service_entry.toDict():
+        if 'krbticketflags' in service_entry:
             host_entry['krbticketflags'] = service_entry['krbticketflags']
         self.admin_conn.add_entry(host_entry)
 
+        # Add the host to the ipaserver host group
+        ld = ldapupdate.LDAPUpdate(ldapi=True)
+        ld.update([os.path.join(paths.UPDATES_DIR,
+                                '20-ipaservers_hostgroup.update')])
+
     def __common_setup(self, realm_name, host_name, domain_name, admin_password):
         self.fqdn = host_name
         self.realm = realm_name.upper()
@@ -166,7 +157,6 @@ class KrbInstance(service.Service):
 
         self.__common_setup(realm_name, host_name, domain_name, admin_password)
 
-        self.step("adding sasl mappings to the directory", self.__configure_sasl_mappings)
         self.step("adding kerberos container to the directory", self.__add_krb_container)
         self.step("configuring KDC", self.__configure_instance)
         self.step("initialize kerberos container", self.__init_ipa_kdb)
@@ -189,22 +179,25 @@ class KrbInstance(service.Service):
                        master_fqdn, host_name,
                        domain_name, admin_password,
                        setup_pkinit=False, pkcs12_info=None,
-                       subject_base=None):
+                       subject_base=None, promote=False):
         self.pkcs12_info = pkcs12_info
         self.subject_base = subject_base
         self.master_fqdn = master_fqdn
 
         self.__common_setup(realm_name, host_name, domain_name, admin_password)
 
-        self.step("adding sasl mappings to the directory", self.__configure_sasl_mappings)
-        self.step("writing stash file from DS", self.__write_stash_from_ds)
         self.step("configuring KDC", self.__configure_instance)
-        self.step("creating a keytab for the directory", self.__create_ds_keytab)
-        self.step("creating a keytab for the machine", self.__create_host_keytab)
+        if not promote:
+            self.step("creating a keytab for the directory",
+                      self.__create_ds_keytab)
+            self.step("creating a keytab for the machine",
+                      self.__create_host_keytab)
         self.step("adding the password extension to the directory", self.__add_pwd_extop_module)
         if setup_pkinit:
             self.step("installing X509 Certificate for PKINIT", self.__setup_pkinit)
-        self.step("enable GSSAPI for replication", self.__convert_to_gssapi_replication)
+        if not promote:
+            self.step("enable GSSAPI for replication",
+                      self.__convert_to_gssapi_replication)
 
         self.__common_post_setup()
 
@@ -234,8 +227,13 @@ class KrbInstance(service.Service):
                              SUFFIX=self.suffix,
                              DOMAIN=self.domain,
                              HOST=self.host,
-                             SERVER_ID=dsinstance.realm_to_serverid(self.realm),
-                             REALM=self.realm)
+                             SERVER_ID=installutils.realm_to_serverid(self.realm),
+                             REALM=self.realm,
+                             KRB5KDC_KADM5_ACL=paths.KRB5KDC_KADM5_ACL,
+                             DICT_WORDS=paths.DICT_WORDS,
+                             KRB5KDC_KADM5_KEYTAB=paths.KRB5KDC_KADM5_KEYTAB,
+                             KDC_PEM=paths.KDC_PEM,
+                             CACERT_PEM=paths.CACERT_PEM)
 
         # IPA server/KDC is not a subdomain of default domain
         # Proper domain-realm mapping needs to be specified
@@ -262,59 +260,13 @@ class KrbInstance(service.Service):
             root_logger.debug("Persistent keyring CCACHE is not enabled")
             self.sub_dict['OTHER_LIBDEFAULTS'] = ''
 
-    def __configure_sasl_mappings(self):
-        # we need to remove any existing SASL mappings in the directory as otherwise they
-        # they may conflict.
-
-        try:
-            res = self.admin_conn.get_entries(
-                DN(('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')),
-                self.admin_conn.SCOPE_ONELEVEL,
-                "(objectclass=nsSaslMapping)")
-            for r in res:
-                try:
-                    self.admin_conn.delete_entry(r)
-                except Exception, e:
-                    root_logger.critical(
-                        "Error during SASL mapping removal: %s", e)
-                    raise
-        except Exception, e:
-            root_logger.critical("Error while enumerating SASL mappings %s", e)
-            raise
-
-        entry = self.admin_conn.make_entry(
-            DN(
-                ('cn', 'Full Principal'), ('cn', 'mapping'), ('cn', 'sasl'),
-                ('cn', 'config')),
-            objectclass=["top", "nsSaslMapping"],
-            cn=["Full Principal"],
-            nsSaslMapRegexString=['\(.*\)@\(.*\)'],
-            nsSaslMapBaseDNTemplate=[self.suffix],
-            nsSaslMapFilterTemplate=['(krbPrincipalName=\\1@\\2)'],
-            nsSaslMapPriority=['10'],
-        )
-        self.admin_conn.add_entry(entry)
-
-        entry = self.admin_conn.make_entry(
-            DN(
-                ('cn', 'Name Only'), ('cn', 'mapping'), ('cn', 'sasl'),
-                ('cn', 'config')),
-            objectclass=["top", "nsSaslMapping"],
-            cn=["Name Only"],
-            nsSaslMapRegexString=['^[^:@]+$'],
-            nsSaslMapBaseDNTemplate=[self.suffix],
-            nsSaslMapFilterTemplate=['(krbPrincipalName=&@%s)' % self.realm],
-            nsSaslMapPriority=['10'],
-        )
-        self.admin_conn.add_entry(entry)
-
     def __add_krb_container(self):
         self._ldap_mod("kerberos.ldif", self.sub_dict)
 
     def __add_default_acis(self):
         self._ldap_mod("default-aci.ldif", self.sub_dict)
 
-    def __template_file(self, path, chmod=0644):
+    def __template_file(self, path, chmod=0o644):
         template = os.path.join(ipautil.SHARE_DIR, os.path.basename(path) + ".template")
         conf = ipautil.template_file(template, self.sub_dict)
         self.fstore.backup_file(path)
@@ -325,6 +277,9 @@ class KrbInstance(service.Service):
             os.chmod(path, chmod)
 
     def __init_ipa_kdb(self):
+        # kdb5_util may take a very long time when entropy is low
+        installutils.check_entropy()
+
         #populate the directory with the realm structure
         args = ["kdb5_util", "create", "-s",
                                        "-r", self.realm,
@@ -337,22 +292,23 @@ class KrbInstance(service.Service):
         )
         try:
             ipautil.run(args, nolog=(self.master_password,), stdin=''.join(dialogue))
-        except ipautil.CalledProcessError, e:
-            print "Failed to initialize the realm container"
+        except ipautil.CalledProcessError as e:
+            print("Failed to initialize the realm container")
 
     def __configure_instance(self):
-        self.__template_file("/var/kerberos/krb5kdc/kdc.conf", chmod=None)
-        self.__template_file("/etc/krb5.conf")
-        self.__template_file("/usr/share/ipa/html/krb5.ini")
-        self.__template_file("/usr/share/ipa/html/krb.con")
-        self.__template_file("/usr/share/ipa/html/krbrealm.con")
+        self.__template_file(paths.KRB5KDC_KDC_CONF, chmod=None)
+        self.__template_file(paths.KRB5_CONF)
+        self.__template_file(paths.HTML_KRB5_INI)
+        self.__template_file(paths.KRB_CON)
+        self.__template_file(paths.HTML_KRBREALM_CON)
 
         MIN_KRB5KDC_WITH_WORKERS = "1.9"
         cpus = os.sysconf('SC_NPROCESSORS_ONLN')
         workers = False
-        (stdout, stderr, rc) = ipautil.run(['klist', '-V'], raiseonerr=False)
-        if rc == 0:
-            verstr = stdout.split()[-1]
+        result = ipautil.run(['klist', '-V'],
+                             raiseonerr=False, capture_output=True)
+        if result.returncode == 0:
+            verstr = result.output.split()[-1]
             ver = version.LooseVersion(verstr)
             min = version.LooseVersion(MIN_KRB5KDC_WITH_WORKERS)
             if ver >= min:
@@ -363,35 +319,10 @@ class KrbInstance(service.Service):
         appendvars = {}
         if workers and cpus > 1:
             appendvars = {'KRB5KDC_ARGS': "'-w %s'" % str(cpus)}
-        ipautil.backup_config_and_replace_variables(self.fstore, "/etc/sysconfig/krb5kdc",
+        ipautil.backup_config_and_replace_variables(self.fstore, paths.SYSCONFIG_KRB5KDC_DIR,
                                                     replacevars=replacevars,
                                                     appendvars=appendvars)
-        ipaservices.restore_context("/etc/sysconfig/krb5kdc")
-
-    def __write_stash_from_ds(self):
-        try:
-            entries = self.admin_conn.get_entries(
-                self.get_realm_suffix(), self.admin_conn.SCOPE_SUBTREE)
-            # TODO: Ensure we got only one entry
-            entry = entries[0]
-        except errors.NotFound, e:
-            root_logger.critical("Could not find master key in DS")
-            raise e
-
-        krbMKey = pyasn1.codec.ber.decoder.decode(
-            entry.single_value('krbmkey', None))
-        keytype = int(krbMKey[0][1][0])
-        keydata = str(krbMKey[0][1][1])
-
-        format = '=hi%ss' % len(keydata)
-        s = struct.pack(format, keytype, len(keydata), keydata)
-        try:
-            fd = open("/var/kerberos/krb5kdc/.k5."+self.realm, "w")
-            fd.write(s)
-            fd.close()
-        except os.error, e:
-            root_logger.critical("failed to write stash file")
-            raise e
+        tasks.restore_context(paths.SYSCONFIG_KRB5KDC_DIR)
 
     #add the password extop module
     def __add_pwd_extop_module(self):
@@ -402,23 +333,25 @@ class KrbInstance(service.Service):
         installutils.kadmin_addprinc(ldap_principal)
         self.move_service(ldap_principal)
 
-        self.fstore.backup_file("/etc/dirsrv/ds.keytab")
-        installutils.create_keytab("/etc/dirsrv/ds.keytab", ldap_principal)
+        self.fstore.backup_file(paths.DS_KEYTAB)
+        installutils.create_keytab(paths.DS_KEYTAB, ldap_principal)
 
-        update_key_val_in_file("/etc/sysconfig/dirsrv", "KRB5_KTNAME", "/etc/dirsrv/ds.keytab")
-        pent = pwd.getpwnam(dsinstance.DS_USER)
-        os.chown("/etc/dirsrv/ds.keytab", pent.pw_uid, pent.pw_gid)
+        vardict = {"KRB5_KTNAME": paths.DS_KEYTAB}
+        ipautil.config_replace_variables(paths.SYSCONFIG_DIRSRV,
+                                         replacevars=vardict)
+        pent = pwd.getpwnam(constants.DS_USER)
+        os.chown(paths.DS_KEYTAB, pent.pw_uid, pent.pw_gid)
 
     def __create_host_keytab(self):
         host_principal = "host/" + self.fqdn + "@" + self.realm
         installutils.kadmin_addprinc(host_principal)
 
-        self.fstore.backup_file("/etc/krb5.keytab")
-        installutils.create_keytab("/etc/krb5.keytab", host_principal)
+        self.fstore.backup_file(paths.KRB5_KEYTAB)
+        installutils.create_keytab(paths.KRB5_KEYTAB, host_principal)
 
         # Make sure access is strictly reserved to root only for now
-        os.chown("/etc/krb5.keytab", 0, 0)
-        os.chmod("/etc/krb5.keytab", 0600)
+        os.chown(paths.KRB5_KEYTAB, 0, 0)
+        os.chmod(paths.KRB5_KEYTAB, 0o600)
 
         self.move_service_to_host(host_principal)
 
@@ -429,13 +362,13 @@ class KrbInstance(service.Service):
         if self.pkcs12_info:
             ca_db.install_pem_from_p12(self.pkcs12_info[0],
                                        self.pkcs12_info[1],
-                                       "/var/kerberos/krb5kdc/kdc.pem")
+                                       paths.KDC_PEM)
         else:
             raise RuntimeError("PKI not supported yet\n")
 
         # Finally copy the cacert in the krb directory so we don't
         # have any selinux issues with the file context
-        shutil.copyfile("/etc/ipa/ca.crt", "/var/kerberos/krb5kdc/cacert.pem")
+        shutil.copyfile(CACERT, paths.CACERT_PEM)
 
     def __add_anonymous_pkinit_principal(self):
         princ = "WELLKNOWN/ANONYMOUS"
@@ -468,18 +401,19 @@ class KrbInstance(service.Service):
         except:
             pass
 
-        for f in ["/var/kerberos/krb5kdc/kdc.conf", "/etc/krb5.conf"]:
+        for f in [paths.KRB5KDC_KDC_CONF, paths.KRB5_CONF]:
             try:
                 self.fstore.restore_file(f)
-            except ValueError, error:
+            except ValueError as error:
                 root_logger.debug(error)
                 pass
 
-        if not enabled is None and not enabled:
-            self.disable()
+        # disabled by default, by ldap_enable()
+        if enabled:
+            self.enable()
 
-        if not running is None and running:
-            self.start()
+        if running:
+            self.restart()
 
         self.kpasswd = KpasswdInstance()
         self.kpasswd.uninstall()
