@@ -17,25 +17,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from ipaserver.install.plugins import MIDDLE, LAST
-from ipaserver.install.plugins.baseupdate import PostUpdate
 from ipalib import api, errors
+from ipalib import Updater
 from ipapython.dn import DN
 from ipapython.ipa_log_manager import *
 
 
-class update_idrange_type(PostUpdate):
+class update_idrange_type(Updater):
     """
     Update all ID ranges that do not have ipaRangeType attribute filled.
     This applies to all ID ranges prior to IPA 3.3.
     """
 
-    order = MIDDLE
-
     def execute(self, **options):
-        ldap = self.obj.backend
+        ldap = self.api.Backend.ldap2
 
-        base_dn = DN(api.env.container_ranges, api.env.basedn)
+        base_dn = DN(self.api.env.container_ranges, self.api.env.basedn)
         search_filter = ("(&(objectClass=ipaIDrange)(!(ipaRangeType=*)))")
         root_logger.debug("update_idrange_type: search for ID ranges with no "
                           "type set")
@@ -51,18 +48,18 @@ class update_idrange_type(PostUpdate):
             except errors.NotFound:
                 root_logger.debug("update_idrange_type: no ID range without "
                                   "type set found")
-                return (False, False, [])
+                return False, []
 
-            except errors.ExecutionError, e:
+            except errors.ExecutionError as e:
                 root_logger.error("update_idrange_type: cannot retrieve list "
                                   "of ranges with no type set: %s", e)
-                return (False, False, [])
+                return False, []
 
             if not entries:
                 # No entry was returned, rather break than continue cycling
                 root_logger.debug("update_idrange_type: no ID range was "
                                   "returned")
-                return (False, False, [])
+                return False, []
 
             root_logger.debug("update_idrange_type: found %d "
                               "idranges to update, truncated: %s",
@@ -92,7 +89,7 @@ class update_idrange_type(PostUpdate):
                     ldap.update_entry(entry)
                 except (errors.EmptyModlist, errors.NotFound):
                     pass
-                except errors.ExecutionError, e:
+                except errors.ExecutionError as e:
                     root_logger.debug("update_idrange_type: cannot "
                                       "update idrange type: %s", e)
                     error = True
@@ -101,29 +98,27 @@ class update_idrange_type(PostUpdate):
                 # Exit loop to avoid infinite cycles
                 root_logger.error("update_idrange_type: error(s) "
                                   "detected during idrange type update")
-                return (False, False, [])
+                return False, []
 
             elif not truncated:
                 # All affected entries updated, exit the loop
                 root_logger.debug("update_idrange_type: all affected idranges "
                                   "were assigned types")
-                return (False, False, [])
+                return False, []
 
-        return (False, False, [])
+        return False, []
 
 
-class update_idrange_baserid(PostUpdate):
+class update_idrange_baserid(Updater):
     """
     Update ipa-ad-trust-posix ranges' base RID to 0. This applies to AD trust
     posix ranges prior to IPA 4.1.
     """
 
-    order = LAST
-
     def execute(self, **options):
-        ldap = self.obj.backend
+        ldap = self.api.Backend.ldap2
 
-        base_dn = DN(api.env.container_ranges, api.env.basedn)
+        base_dn = DN(self.api.env.container_ranges, self.api.env.basedn)
         search_filter = ("(&(objectClass=ipaTrustedADDomainRange)"
                          "(ipaRangeType=ipa-ad-trust-posix)"
                          "(!(ipaBaseRID=0)))")
@@ -140,12 +135,12 @@ class update_idrange_baserid(PostUpdate):
         except errors.NotFound:
             root_logger.debug("update_idrange_baserid: no AD domain "
                               "range with posix attributes found")
-            return (False, False, [])
+            return False, []
 
-        except errors.ExecutionError, e:
+        except errors.ExecutionError as e:
             root_logger.error("update_idrange_baserid: cannot retrieve "
                               "list of affected ranges: %s", e)
-            return (False, False, [])
+            return False, []
 
         root_logger.debug("update_idrange_baserid: found %d "
                           "idranges possible to update",
@@ -157,12 +152,12 @@ class update_idrange_baserid(PostUpdate):
         for entry in entries:
             entry['ipabaserid'] = 0
             try:
-                root_logger.info("Updating existing idrange: %s" % (entry.dn))
+                root_logger.debug("Updating existing idrange: %s" % (entry.dn))
                 ldap.update_entry(entry)
                 root_logger.info("Done")
             except (errors.EmptyModlist, errors.NotFound):
                 pass
-            except errors.ExecutionError, e:
+            except errors.ExecutionError as e:
                 root_logger.debug("update_idrange_type: cannot "
                                   "update idrange: %s", e)
                 error = True
@@ -175,7 +170,7 @@ class update_idrange_baserid(PostUpdate):
             root_logger.debug("update_idrange_baserid: all affected "
                               "idranges updated")
 
-        return (False, False, [])
+        return False, []
 
 api.register(update_idrange_type)
 api.register(update_idrange_baserid)
