@@ -24,10 +24,10 @@ import time
 import datetime
 import traceback
 
-from ipapython import sysrestore, ipautil, dogtag, ipaldap
+from ipapython import sysrestore, ipautil, ipaldap
 from ipapython.dn import DN
 from ipapython.ipa_log_manager import *
-from ipalib import errors, certstore
+from ipalib import api, errors, certstore
 from ipaplatform import services
 from ipaplatform.paths import paths
 
@@ -40,7 +40,9 @@ SERVICE_LIST = {
     'DNS': ('named', 30),
     'MEMCACHE': ('ipa_memcached', 39),
     'HTTP': ('httpd', 40),
-    'CA': ('%sd' % dogtag.configured_constants().PKI_INSTANCE_NAME, 50),
+    'KEYS': ('ipa-custodia', 41),
+    'CA': ('pki-tomcatd', 50),
+    'KRA': ('pki-tomcatd', 51),
     'ADTRUST': ('smb', 60),
     'EXTID': ('winbind', 70),
     'OTPD': ('ipa-otpd', 80),
@@ -70,6 +72,61 @@ def format_seconds(seconds):
             parts[-1] += 's'
     return ' '.join(parts)
 
+def add_principals_to_group(admin_conn, group, member_attr, principals):
+    """Add principals to a GroupOfNames LDAP group
+    admin_conn  -- LDAP connection with admin rights
+    group       -- DN of the group
+    member_attr -- attribute to represent members
+    principals  -- list of DNs to add as members
+    """
+    try:
+        current = admin_conn.get_entry(group)
+        members = current.get(member_attr, [])
+        if len(members) == 0:
+            current[member_attr] = []
+        for amember in principals:
+            if not(amember in members):
+                current[member_attr].extend([amember])
+        admin_conn.update_entry(current)
+    except errors.NotFound:
+        entry = admin_conn.make_entry(
+                group,
+                objectclass=["top", "GroupOfNames"],
+                cn=[group['cn']],
+                member=principals,
+        )
+        admin_conn.add_entry(entry)
+    except errors.EmptyModlist:
+        # If there are no changes just pass
+        pass
+
+
+def find_providing_server(svcname, conn, host_name=None, api=api):
+    """
+    :param svcname: The service to find
+    :param conn: a connection to the LDAP server
+    :param host_name: the preferred server
+    :return: the selected host name
+
+    Find a server that is a CA.
+    """
+    dn = DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
+    query_filter = conn.make_filter({'objectClass': 'ipaConfigObject',
+                                     'ipaConfigString': 'enabledService',
+                                     'cn': svcname}, rules='&')
+    try:
+        entries, trunc = conn.find_entries(filter=query_filter, base_dn=dn)
+    except errors.NotFound:
+        return None
+    if len(entries):
+        if host_name is not None:
+            for entry in entries:
+                if entry.dn[1].value == host_name:
+                    return host_name
+        # if the preferred is not found, return the first in the list
+        return entries[0].dn[1].value
+    return None
+
 
 class Service(object):
     def __init__(self, service_name, service_desc=None, sstore=None,
@@ -117,7 +174,7 @@ class Service(object):
                 conn = ipaldap.IPAdmin(self.fqdn, port=389)
 
             conn.do_bind(self.dm_password, autobind=self.autobind)
-        except Exception, e:
+        except Exception as e:
             root_logger.debug("Could not connect to the Directory Server on %s: %s" % (self.fqdn, str(e)))
             raise
 
@@ -127,7 +184,7 @@ class Service(object):
         self.admin_conn.unbind()
         self.admin_conn = None
 
-    def _ldap_mod(self, ldif, sub_dict=None):
+    def _ldap_mod(self, ldif, sub_dict=None, raise_on_err=False):
         pw_name = None
         fd = None
         path = ipautil.SHARE_DIR + ldif
@@ -170,7 +227,9 @@ class Service(object):
         try:
             try:
                 ipautil.run(args, nolog=nologlist)
-            except ipautil.CalledProcessError, e:
+            except ipautil.CalledProcessError as e:
+                if raise_on_err:
+                    raise
                 root_logger.critical("Failed to load %s: %s" % (ldif, str(e)))
         finally:
             if pw_name:
@@ -254,7 +313,7 @@ class Service(object):
         entry.setdefault('userCertificate', []).append(self.dercert)
         try:
             self.admin_conn.update_entry(entry)
-        except Exception, e:
+        except Exception as e:
             root_logger.critical("Could not add certificate to service %s entry: %s" % (self.principal, str(e)))
 
     def import_ca_certs(self, db, ca_is_configured, conn=None):
@@ -367,7 +426,7 @@ class Service(object):
                     end_message = "Done configuring %s." % self.service_desc
 
         if runtime > 0:
-            self.print_msg('%s: Estimated time %s' % (start_message,
+            self.print_msg('%s. Estimated time: %s' % (start_message,
                                                       format_seconds(runtime)))
         else:
             self.print_msg(start_message)
@@ -388,7 +447,8 @@ class Service(object):
                 run_step(full_msg, method)
                 step += 1
         except BaseException as e:
-            if not (isinstance(e, SystemExit) and e.code == 0):
+            if not (isinstance(e, SystemExit) and
+                    e.code == 0):  # pylint: disable=no-member
                 # show the traceback, so it's not lost if cleanup method fails
                 root_logger.debug("%s" % traceback.format_exc())
                 self.print_msg('  [error] %s: %s' % (type(e).__name__, e))
@@ -448,7 +508,7 @@ class Service(object):
 
         try:
             self.admin_conn.add_entry(entry)
-        except (errors.DuplicateEntry), e:
+        except (errors.DuplicateEntry) as e:
             root_logger.debug("failed to add service %s startup entry", name)
             raise e
 
@@ -490,6 +550,19 @@ class Service(object):
 
         root_logger.debug("service %s startup entry disabled", name)
 
+    def ldap_remove_service_container(self, name, fqdn, ldap_suffix):
+        if not self.admin_conn:
+            self.ldap_connect()
+
+        entry_dn = DN(('cn', name), ('cn', fqdn), ('cn', 'masters'),
+                        ('cn', 'ipa'), ('cn', 'etc'), ldap_suffix)
+        try:
+            self.admin_conn.delete_entry(entry_dn)
+        except errors.NotFound:
+            root_logger.debug("service %s container already removed", name)
+        else:
+            root_logger.debug("service %s container sucessfully removed", name)
+
 
 class SimpleServiceInstance(Service):
     def create_instance(self, gensvc_name=None, fqdn=None, dm_password=None, ldap_suffix=None, realm=None):
@@ -512,7 +585,6 @@ class SimpleServiceInstance(Service):
         self.restart()
 
     def __enable(self):
-        self.enable()
         self.backup_state("enabled", self.is_enabled())
         if self.gensvc_name == None:
             self.enable()
@@ -524,11 +596,14 @@ class SimpleServiceInstance(Service):
         if self.is_configured():
             self.print_msg("Unconfiguring %s" % self.service_name)
 
-        running = self.restore_state("running")
-        enabled = not self.restore_state("enabled")
+        self.stop()
+        self.disable()
 
-        if not running is None and not running:
-            self.stop()
-        if not enabled is None and not enabled:
-            self.disable()
-            self.remove()
+        running = self.restore_state("running")
+        enabled = self.restore_state("enabled")
+
+        # restore the original state of service
+        if running:
+            self.start()
+        if enabled:
+            self.enable()
