@@ -30,11 +30,12 @@ Backend plugin for LDAP.
 import os
 import pwd
 
-import krbV
 import ldap as _ldap
 
+from ipalib import krb_utils
 from ipapython.dn import DN
-from ipapython.ipaldap import SASL_GSSAPI, IPASimpleLDAPObject, LDAPClient
+from ipapython.ipaldap import (LDAPClient, AUTOBIND_AUTO, AUTOBIND_ENABLED,
+                               AUTOBIND_DISABLED)
 
 
 try:
@@ -56,35 +57,60 @@ from ipalib.crud import CrudBackend
 from ipalib.request import context
 
 
-class ldap2(LDAPClient, CrudBackend):
+class ldap2(CrudBackend, LDAPClient):
     """
     LDAP Backend Take 2.
     """
 
-    def __init__(self, shared_instance=True, ldap_uri=None, base_dn=None,
-                 schema=None):
-        try:
-            ldap_uri = ldap_uri or api.env.ldap_uri
-        except AttributeError:
-            ldap_uri = 'ldap://example.com'
+    def __init__(self, api, ldap_uri=None):
+        if ldap_uri is None:
+            ldap_uri = api.env.ldap_uri
 
-        CrudBackend.__init__(self, shared_instance=shared_instance)
-        LDAPClient.__init__(self, ldap_uri)
+        force_schema_updates = api.env.context in ('installer', 'updates')
 
-        try:
-            if base_dn is not None:
-                self.base_dn = DN(base_dn)
-            else:
-                self.base_dn = DN(api.env.basedn)
-        except AttributeError:
-            self.base_dn = DN()
+        CrudBackend.__init__(self, api)
+        LDAPClient.__init__(self, ldap_uri,
+                            force_schema_updates=force_schema_updates)
 
-    def _init_connection(self):
+        self.__time_limit = None
+        self.__size_limit = None
+
+    @property
+    def time_limit(self):
+        if self.__time_limit is None:
+            return float(self.get_ipa_config().single_value.get(
+                'ipasearchtimelimit', 2))
+        return self.__time_limit
+
+    @time_limit.setter
+    def time_limit(self, val):
+        self.__time_limit = float(val)
+
+    @time_limit.deleter
+    def time_limit(self):
+        self.__time_limit = None
+
+    @property
+    def size_limit(self):
+        if self.__size_limit is None:
+            return int(self.get_ipa_config().single_value.get(
+                'ipasearchrecordslimit', 0))
+        return self.__size_limit
+
+    @size_limit.setter
+    def size_limit(self, val):
+        self.__size_limit = int(val)
+
+    @size_limit.deleter
+    def size_limit(self):
+        self.__size_limit = None
+
+    def _connect(self):
         # Connectible.conn is a proxy to thread-local storage;
         # do not set it
         pass
 
-    def __del__(self):
+    def close(self):
         if self.isconnected():
             self.disconnect()
 
@@ -93,13 +119,14 @@ class ldap2(LDAPClient, CrudBackend):
 
     def create_connection(self, ccache=None, bind_dn=None, bind_pw='',
             tls_cacertfile=None, tls_certfile=None, tls_keyfile=None,
-            debug_level=0, autobind=False, serverctrls=None, clientctrls=None):
+            debug_level=0, autobind=AUTOBIND_AUTO, serverctrls=None,
+            clientctrls=None, time_limit=None, size_limit=None):
         """
         Connect to LDAP server.
 
         Keyword arguments:
         ldapuri -- the LDAP server to connect to
-        ccache -- Kerberos V5 ccache object or name
+        ccache -- Kerberos ccache name
         bind_dn -- dn used to bind to the server
         bind_pw -- password used to bind to the server
         debug_level -- LDAP debug level option
@@ -120,15 +147,19 @@ class ldap2(LDAPClient, CrudBackend):
         if tls_keyfile is not None:
             _ldap.set_option(_ldap.OPT_X_TLS_KEYFILE, tls_keyfile)
 
+        if time_limit is not None:
+            self.time_limit = time_limit
+        if size_limit is not None:
+            self.size_limit = size_limit
+
         if debug_level:
             _ldap.set_option(_ldap.OPT_DEBUG_LEVEL, debug_level)
 
-        with self.error_handler():
-            force_updates = api.env.context in ('installer', 'updates')
-            conn = IPASimpleLDAPObject(
-                self.ldap_uri, force_schema_updates=force_updates)
-            if self.ldap_uri.startswith('ldapi://') and ccache:
-                conn.set_option(_ldap.OPT_HOST_NAME, api.env.host)
+        client = LDAPClient(self.ldap_uri,
+                            force_schema_updates=self._force_schema_updates)
+        conn = client._conn
+
+        with client.error_handler():
             minssf = conn.get_option(_ldap.OPT_X_SASL_SSF_MIN)
             maxssf = conn.get_option(_ldap.OPT_X_SASL_SSF_MAX)
             # Always connect with at least an SSF of 56, confidentiality
@@ -138,171 +169,62 @@ class ldap2(LDAPClient, CrudBackend):
                 conn.set_option(_ldap.OPT_X_SASL_SSF_MIN, minssf)
                 if maxssf < minssf:
                     conn.set_option(_ldap.OPT_X_SASL_SSF_MAX, minssf)
-            if ccache is not None:
-                if isinstance(ccache, krbV.CCache):
-                    principal = ccache.principal().name
-                    # Get a fully qualified CCACHE name (schema+name)
-                    # As we do not use the krbV.CCache object later,
-                    # we can safely overwrite it
-                    ccache = "%(type)s:%(name)s" % dict(type=ccache.type,
-                                                        name=ccache.name)
-                else:
-                    principal = krbV.CCache(name=ccache,
-                        context=krbV.default_context()).principal().name
 
-                os.environ['KRB5CCNAME'] = ccache
-                conn.sasl_interactive_bind_s(None, SASL_GSSAPI,
-                                             serverctrls=serverctrls,
-                                             clientctrls=clientctrls)
-                setattr(context, 'principal', principal)
+        ldapi = self.ldap_uri.startswith('ldapi://')
+
+        if bind_pw:
+            client.simple_bind(bind_dn, bind_pw,
+                               server_controls=serverctrls,
+                               client_controls=clientctrls)
+        elif autobind != AUTOBIND_DISABLED and os.getegid() == 0 and ldapi:
+            try:
+                pw_name = pwd.getpwuid(os.geteuid()).pw_name
+                client.external_bind(pw_name,
+                                     server_controls=serverctrls,
+                                     client_controls=clientctrls)
+            except errors.NotFound:
+                if autobind == AUTOBIND_ENABLED:
+                    # autobind was required and failed, raise
+                    # exception that it failed
+                    raise
+        else:
+            if ldapi:
+                with client.error_handler():
+                    conn.set_option(_ldap.OPT_HOST_NAME, self.api.env.host)
+            if ccache is None:
+                os.environ.pop('KRB5CCNAME', None)
             else:
-                # no kerberos ccache, use simple bind or external sasl
-                if autobind:
-                    pent = pwd.getpwuid(os.geteuid())
-                    auth_tokens = _ldap.sasl.external(pent.pw_name)
-                    conn.sasl_interactive_bind_s(None, auth_tokens,
-                                                 serverctrls=serverctrls,
-                                                 clientctrls=clientctrls)
-                else:
-                    conn.simple_bind_s(bind_dn, bind_pw,
-                                       serverctrls=serverctrls,
-                                       clientctrls=clientctrls)
+                os.environ['KRB5CCNAME'] = ccache
+
+            principal = krb_utils.get_principal(ccache_name=ccache)
+
+            client.gssapi_bind(server_controls=serverctrls,
+                               client_controls=clientctrls)
+            setattr(context, 'principal', principal)
 
         return conn
 
     def destroy_connection(self):
         """Disconnect from LDAP server."""
         try:
-            self.conn.unbind_s()
-        except _ldap.LDAPError:
+            if self.conn is not None:
+                self.unbind()
+        except errors.PublicError:
             # ignore when trying to unbind multiple times
             pass
 
+        del self.time_limit
+        del self.size_limit
 
-    def find_entries(self, filter=None, attrs_list=None, base_dn=None,
-                     scope=_ldap.SCOPE_SUBTREE, time_limit=None,
-                     size_limit=None, search_refs=False, paged_search=False):
-
-        def _get_limits():
-            """Get configured global limits, caching them for more calls"""
-            if not _lims:
-                config = self.get_ipa_config()
-                _lims['time'] = config.get('ipasearchtimelimit', [None])[0]
-                _lims['size'] = config.get('ipasearchrecordslimit', [None])[0]
-            return _lims
-        _lims = {}
-
-        if time_limit is None:
-            time_limit = _get_limits()['time']
-        if size_limit is None:
-            size_limit = _get_limits()['size']
-
-        has_memberindirect = False
-        has_memberofindirect = False
-        if attrs_list:
-            new_attrs_list = []
-            for attr_name in attrs_list:
-                if attr_name == 'memberindirect':
-                    has_memberindirect = True
-                elif attr_name == 'memberofindirect':
-                    has_memberofindirect = True
-                else:
-                    new_attrs_list.append(attr_name)
-            attrs_list = new_attrs_list
-
-        res, truncated = super(ldap2, self).find_entries(
-            filter=filter, attrs_list=attrs_list, base_dn=base_dn, scope=scope,
-            time_limit=time_limit, size_limit=size_limit,
-            search_refs=search_refs, paged_search=paged_search)
-
-        if has_memberindirect or has_memberofindirect:
-
-            # For the memberof searches, we want to apply the global limit
-            # if it's larger than the requested one, so decreasing limits on
-            # the individual query only affects the query itself.
-            # See https://fedorahosted.org/freeipa/ticket/4398
-            def _max_with_none(a, b):
-                """Maximum of a and b, treating None as infinity"""
-                if a is None or b is None:
-                    return None
-                else:
-                    return max(a, b)
-            time_limit = _max_with_none(time_limit, _get_limits()['time'])
-            size_limit = _max_with_none(size_limit, _get_limits()['size'])
-
-            for entry in res:
-                if has_memberindirect:
-                    self._process_memberindirect(
-                        entry, time_limit=time_limit, size_limit=size_limit)
-                if has_memberofindirect:
-                    self._process_memberofindirect(
-                        entry, time_limit=time_limit, size_limit=size_limit)
-
-        return (res, truncated)
-
-    def _process_memberindirect(self, group_entry, time_limit=None,
-                                size_limit=None):
-        filter = self.make_filter({'memberof': group_entry.dn})
-        try:
-            result, truncated = self.find_entries(
-                base_dn=self.api.env.basedn,
-                filter=filter,
-                attrs_list=['member'],
-                time_limit=time_limit,
-                size_limit=size_limit,
-                paged_search=True)
-            if truncated:
-                raise errors.LimitsExceeded()
-        except errors.NotFound:
-            result = []
-
-        indirect = set()
-        for entry in result:
-            indirect.update(entry.get('member', []))
-        indirect.difference_update(group_entry.get('member', []))
-
-        if indirect:
-            group_entry['memberindirect'] = list(indirect)
-
-    def _process_memberofindirect(self, entry, time_limit=None,
-                                  size_limit=None):
-        dn = entry.dn
-        filter = self.make_filter(
-            {'member': dn, 'memberuser': dn, 'memberhost': dn})
-        try:
-            result, truncated = self.find_entries(
-                base_dn=self.api.env.basedn,
-                filter=filter,
-                attrs_list=[''],
-                time_limit=time_limit,
-                size_limit=size_limit)
-            if truncated:
-                raise errors.LimitsExceeded()
-        except errors.NotFound:
-            result = []
-
-        direct = set()
-        indirect = set(entry.get('memberof', []))
-        for group_entry in result:
-            dn = group_entry.dn
-            if dn in indirect:
-                indirect.remove(dn)
-                direct.add(dn)
-
-        entry['memberof'] = list(direct)
-        if indirect:
-            entry['memberofindirect'] = list(indirect)
-
-    config_defaults = {'ipasearchtimelimit': [2], 'ipasearchrecordslimit': [0]}
     def get_ipa_config(self, attrs_list=None):
         """Returns the IPA configuration entry (dn, entry_attrs)."""
 
-        dn = api.Object.config.get_dn()
+        dn = self.api.Object.config.get_dn()
         assert isinstance(dn, DN)
 
         try:
             config_entry = getattr(context, 'config_entry')
-            if config_entry.conn is self.conn:
+            if config_entry.conn.conn is self.conn:
                 return config_entry
         except AttributeError:
             # Not in our context yet
@@ -317,9 +239,7 @@ class ldap2(LDAPClient, CrudBackend):
             config_entry = entries[0]
         except errors.NotFound:
             config_entry = self.make_entry(dn)
-        for a in self.config_defaults:
-            if a not in config_entry:
-                config_entry[a] = self.config_defaults[a]
+
         context.config_entry = config_entry
         return config_entry
 
@@ -334,12 +254,14 @@ class ldap2(LDAPClient, CrudBackend):
         """
 
         upg_dn = DN(('cn', 'UPG Definition'), ('cn', 'Definitions'), ('cn', 'Managed Entries'),
-                    ('cn', 'etc'), api.env.basedn)
+                    ('cn', 'etc'), self.api.env.basedn)
 
         try:
-            upg_entries = self.conn.search_s(upg_dn, _ldap.SCOPE_BASE,
-                                             attrlist=['*'])
-        except _ldap.NO_SUCH_OBJECT:
+            with self.error_handler():
+                upg_entries = self.conn.search_s(str(upg_dn), _ldap.SCOPE_BASE,
+                                                 attrlist=['*'])
+                upg_entries = self._convert_result(upg_entries)
+        except errors.NotFound:
             upg_entries = None
         if not upg_entries or 'originfilter' not in upg_entries[0]:
             raise errors.ACIError(info=_(
@@ -359,7 +281,7 @@ class ldap2(LDAPClient, CrudBackend):
 
         principal = getattr(context, 'principal')
         entry = self.find_entry_by_attr("krbprincipalname", principal,
-            "krbPrincipalAux", base_dn=api.env.basedn)
+            "krbPrincipalAux", base_dn=self.api.env.basedn)
         sctrl = [GetEffectiveRightsControl(True, "dn: " + str(entry.dn))]
         self.conn.set_option(_ldap.OPT_SERVER_CONTROLS, sctrl)
         try:
@@ -448,14 +370,15 @@ class ldap2(LDAPClient, CrudBackend):
             pw = old_pass
             if (otp):
                 pw = old_pass+otp
-            with self.error_handler():
-                conn = IPASimpleLDAPObject(
-                    self.ldap_uri, force_schema_updates=False)
-                conn.simple_bind_s(dn, pw)
-                conn.unbind_s()
+
+            with LDAPClient(self.ldap_uri, force_schema_updates=False) as conn:
+                conn.simple_bind(dn, pw)
+                conn.unbind()
 
         with self.error_handler():
-            self.conn.passwd_s(dn, old_pass, new_pass)
+            old_pass = self.encode(old_pass)
+            new_pass = self.encode(new_pass)
+            self.conn.passwd_s(str(dn), old_pass, new_pass)
 
     def add_entry_to_group(self, dn, group_dn, member_attr='member', allow_same=False):
         """
@@ -487,7 +410,9 @@ class ldap2(LDAPClient, CrudBackend):
         # update group entry
         try:
             with self.error_handler():
-                self.conn.modify_s(group_dn, modlist)
+                modlist = [(a, self.encode(b), self.encode(c))
+                           for a, b, c in modlist]
+                self.conn.modify_s(str(group_dn), modlist)
         except errors.DatabaseError:
             raise errors.AlreadyGroupMember()
 
@@ -507,7 +432,9 @@ class ldap2(LDAPClient, CrudBackend):
         # update group entry
         try:
             with self.error_handler():
-                self.conn.modify_s(group_dn, modlist)
+                modlist = [(a, self.encode(b), self.encode(c))
+                           for a, b, c in modlist]
+                self.conn.modify_s(str(group_dn), modlist)
         except errors.MidairCollision:
             raise errors.NotGroupMember()
 
@@ -561,7 +488,7 @@ class ldap2(LDAPClient, CrudBackend):
                (_ldap.MOD_REPLACE, 'krblastpwdchange', None)]
 
         with self.error_handler():
-            self.conn.modify_s(dn, mod)
+            self.conn.modify_s(str(dn), mod)
 
     # CrudBackend methods
 
