@@ -35,19 +35,20 @@ from ipalib import Backend
 from ipalib import errors
 from ipaserver.install import certs
 import os
+from ipaplatform.paths import paths
 
 class rabase(Backend):
     """
     Request Authority backend plugin.
     """
-    def __init__(self):
+    def __init__(self, api):
         if api.env.in_tree:
             self.sec_dir = api.env.dot_ipa + os.sep + 'alias'
             self.pwd_file = self.sec_dir + os.sep + '.pwd'
         else:
-            self.sec_dir = "/etc/httpd/alias"
-            self.pwd_file = "/etc/httpd/alias/pwdfile.txt"
-        super(rabase, self).__init__()
+            self.sec_dir = paths.HTTPD_ALIAS_DIR
+            self.pwd_file = paths.ALIAS_PWDFILE_TXT
+        super(rabase, self).__init__(api)
 
 
     def check_request_status(self, request_id):
@@ -66,11 +67,12 @@ class rabase(Backend):
         """
         raise errors.NotImplementedError(name='%s.get_certificate' % self.name)
 
-    def request_certificate(self, csr, request_type='pkcs10'):
+    def request_certificate(self, csr, profile_id, request_type='pkcs10'):
         """
         Submit certificate signing request.
 
         :param csr: The certificate signing request.
+        :param profile_id: Profile to use for this request.
         :param request_type: The request type (defaults to ``'pkcs10'``).
         """
         raise errors.NotImplementedError(name='%s.request_certificate' % self.name)
