@@ -53,7 +53,7 @@ def setup_package():
     try:
         setup(
             name = "ipatests",
-            version = "4.1.4",
+            version = "4.3.1",
             license = "GPL",
             author = "FreeIPA Developers",
             author_email = "freeipa-devel@redhat.com",
@@ -63,9 +63,10 @@ def setup_package():
             description = DOCLINES[0],
             long_description = "\n".join(DOCLINES[2:]),
             download_url = "http://www.freeipa.org/page/Downloads",
-            classifiers=filter(None, CLASSIFIERS.split('\n')),
+            classifiers=[line for line in CLASSIFIERS.split('\n') if line],
             package_dir = {'ipatests': ''},
             packages = ["ipatests",
+                        "ipatests.pytest_plugins",
                         "ipatests.test_cmdline",
                         "ipatests.test_install",
                         "ipatests.test_integration",
@@ -75,13 +76,16 @@ def setup_package():
                         "ipatests.test_ipaserver.test_install",
                         "ipatests.test_pkcs10",
                         "ipatests.test_webui",
-                        "ipatests.test_xmlrpc"],
+                        "ipatests.test_xmlrpc",
+                        "ipatests.test_xmlrpc.tracker"],
             scripts=['ipa-run-tests', 'ipa-test-config', 'ipa-test-task'],
             package_data = {
+                'ipatests': ['pytest.ini'],
                 'ipatests.test_install': ['*.update'],
                 'ipatests.test_integration': ['scripts/*'],
                 'ipatests.test_pkcs10': ['*.csr'],
                 "ipatests.test_ipaserver": ['data/*'],
+                'ipatests.test_xmlrpc': ['data/*'],
             }
         )
     finally:
