@@ -22,39 +22,40 @@ Test `ipa-getkeytab`
 
 import os
 import shutil
-from cmdline import cmdline_test
+import tempfile
+
+import gssapi
+import nose
+import pytest
+
 from ipalib import api
 from ipalib import errors
-import tempfile
-from ipapython import ipautil
-import nose
-import tempfile
-import krbV
+from ipapython import ipautil, ipaldap
 from ipaserver.plugins.ldap2 import ldap2
 from ipapython.dn import DN
+from ipatests.test_cmdline.cmdline import cmdline_test
 
 def use_keytab(principal, keytab):
     try:
         tmpdir = tempfile.mkdtemp(prefix = "tmp-")
         ccache_file = 'FILE:%s/ccache' % tmpdir
-        krbcontext = krbV.default_context()
-        principal = str(principal)
-        keytab = krbV.Keytab(name=keytab, context=krbcontext)
-        principal = krbV.Principal(name=principal, context=krbcontext)
+        name = gssapi.Name(principal, gssapi.NameType.kerberos_principal)
+        store = {'ccache': ccache_file,
+                 'client_keytab': keytab}
         os.environ['KRB5CCNAME'] = ccache_file
-        ccache = krbV.CCache(name=ccache_file, context=krbcontext, primary_principal=principal)
-        ccache.init(principal)
-        ccache.init_creds_keytab(keytab=keytab, principal=principal)
-        conn = ldap2(shared_instance=False, ldap_uri=api.env.ldap_uri, base_dn=api.env.basedn)
-        conn.connect(ccache=ccache)
+        gssapi.Credentials(name=name, usage='initiate', store=store)
+        conn = ldap2(api)
+        conn.connect(autobind=ipaldap.AUTOBIND_DISABLED)
         conn.disconnect()
-    except krbV.Krb5Error, e:
-        raise StandardError('Unable to bind to LDAP. Error initializing principal %s in %s: %s' % (principal.name, keytab, str(e)))
+    except gssapi.exceptions.GSSError as e:
+        raise Exception('Unable to bind to LDAP. Error initializing principal %s in %s: %s' % (principal, keytab, str(e)))
     finally:
-        del os.environ['KRB5CCNAME']
+        os.environ.pop('KRB5CCNAME', None)
         if tmpdir:
             shutil.rmtree(tmpdir)
 
+
+@pytest.mark.tier0
 class test_ipagetkeytab(cmdline_test):
     """
     Test `ipa-getkeytab`.
@@ -85,8 +86,11 @@ class test_ipagetkeytab(cmdline_test):
                     "-p", "test/notfound.example.com",
                     "-k", self.keytabname,
                    ]
-        (out, err, rc) = ipautil.run(new_args, stdin=None, raiseonerr=False)
+        result = ipautil.run(new_args, stdin=None, raiseonerr=False,
+                             capture_error=True)
+        err = result.error_output
         assert 'Failed to parse result: PrincipalName not found.\n' in err, err
+        rc = result.returncode
         assert rc > 0, rc
 
     def test_2_run(self):
@@ -107,11 +111,12 @@ class test_ipagetkeytab(cmdline_test):
                     "-k", self.keytabname,
                    ]
         try:
-            (out, err, rc) = ipautil.run(new_args, None)
+            result = ipautil.run(new_args, None, capture_error=True)
             expected = 'Keytab successfully retrieved and stored in: %s\n' % (
                 self.keytabname)
-            assert expected in err, 'Success message not in output:\n%s' % err
-        except ipautil.CalledProcessError, e:
+            assert expected in result.error_output, (
+                'Success message not in output:\n%s' % result.error_output)
+        except ipautil.CalledProcessError as e:
             assert (False)
 
     def test_3_use(self):
@@ -141,7 +146,7 @@ class test_ipagetkeytab(cmdline_test):
         """
         try:
             use_keytab(self.service_princ, self.keytabname)
-        except StandardError, errmsg:
+        except Exception as errmsg:
             assert('Unable to bind to LDAP. Error initializing principal' in str(errmsg))
 
     def test_9_cleanup(self):
