@@ -24,6 +24,7 @@ import unittest
 import os
 
 import nose
+import pytest
 
 from ipalib import api
 from ipalib import errors
@@ -45,6 +46,8 @@ have occurred as expected.
 The DM password needs to be set in ~/.ipa/.dmpw
 """
 
+
+@pytest.mark.tier0
 class test_update(unittest.TestCase):
     """
     Test the LDAP updater.
@@ -59,7 +62,7 @@ class test_update(unittest.TestCase):
             fp.close()
         else:
             raise nose.SkipTest("No directory manager password")
-        self.updater = LDAPUpdate(dm_password=self.dm_password, sub_dict={}, live_run=True)
+        self.updater = LDAPUpdate(dm_password=self.dm_password, sub_dict={})
         self.ld = ipaldap.IPAdmin(fqdn)
         self.ld.do_simple_bind(bindpw=self.dm_password)
         if ipautil.file_exists("0_reset.update"):
