@@ -24,6 +24,7 @@ import base64
 import glob
 import contextlib
 import nose
+import pytest
 
 from ipalib import x509
 from ipapython import ipautil
@@ -31,7 +32,6 @@ from ipaplatform.paths import paths
 from ipapython.dn import DN
 from ipatests.test_integration.base import IntegrationTest
 from ipatests.test_integration import tasks
-from ipatests.order_plugin import ordered
 
 _DEFAULT = object()
 
@@ -67,8 +67,8 @@ def assert_error(result, stderr_text, returncode=None):
 
 class CALessBase(IntegrationTest):
     @classmethod
-    def install(cls):
-        super(CALessBase, cls).install()
+    def install(cls, mh):
+        super(CALessBase, cls).install(mh)
         cls.cert_dir = tempfile.mkdtemp(prefix="ipatest-")
         cls.pem_filename = os.path.join(cls.cert_dir, 'root.pem')
         scriptfile = os.path.join(os.path.dirname(__file__),
@@ -108,17 +108,11 @@ class CALessBase(IntegrationTest):
                 host.transport.put_file(source, dest)
 
     @classmethod
-    def uninstall(cls):
+    def uninstall(cls, mh):
         # Remove the NSS database
         shutil.rmtree(cls.cert_dir)
 
-        # Remove CA cert in /etc/pki/nssdb, in case of failed (un)install
-        for host in cls.get_all_hosts():
-            cls.master.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
-                                    '-n', 'External CA cert'],
-                                   raiseonerr=False)
-
-        super(CALessBase, cls).uninstall()
+        super(CALessBase, cls).uninstall(mh)
 
     @classmethod
     def install_server(cls, host=None,
@@ -146,17 +140,17 @@ class CALessBase(IntegrationTest):
         for filename in set(files_to_copy):
             cls.copy_cert(host, filename)
 
-        cls.collect_log(host, paths.IPASERVER_INSTALL_LOG)
-        cls.collect_log(host, paths.IPACLIENT_INSTALL_LOG)
+        host.collect_log(paths.IPASERVER_INSTALL_LOG)
+        host.collect_log(paths.IPACLIENT_INSTALL_LOG)
         inst = host.domain.realm.replace('.', '-')
-        cls.collect_log(host, paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
-        cls.collect_log(host, paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
+        host.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
+        host.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
 
         args = [
             'ipa-server-install',
-            '--http_pkcs12', http_pkcs12,
-            '--dirsrv_pkcs12', dirsrv_pkcs12,
-            '--root-ca-file', root_ca_file,
+            '--http-cert-file', http_pkcs12,
+            '--dirsrv-cert-file', dirsrv_pkcs12,
+            '--ca-cert-file', root_ca_file,
             '--ip-address', host.ip,
             '-r', host.domain.name,
             '-p', host.config.dirman_password,
@@ -166,9 +160,9 @@ class CALessBase(IntegrationTest):
         ]
 
         if http_pin is not None:
-            args.extend(['--http_pin', http_pin])
+            args.extend(['--http-pin', http_pin])
         if dirsrv_pin is not None:
-            args.extend(['--dirsrv_pin', dirsrv_pin])
+            args.extend(['--dirsrv-pin', dirsrv_pin])
         if unattended:
             args.extend(['-U'])
 
@@ -217,11 +211,11 @@ class CALessBase(IntegrationTest):
                 os.path.join(self.cert_dir, filename),
                 os.path.join(master.config.test_dir, filename))
 
-        self.collect_log(replica, paths.IPAREPLICA_INSTALL_LOG)
-        self.collect_log(replica, paths.IPACLIENT_INSTALL_LOG)
+        replica.collect_log(paths.IPAREPLICA_INSTALL_LOG)
+        replica.collect_log(paths.IPACLIENT_INSTALL_LOG)
         inst = replica.domain.realm.replace('.', '-')
-        self.collect_log(replica, paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
-        self.collect_log(replica, paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
+        replica.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
+        replica.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
 
         args = [
             'ipa-replica-prepare',
@@ -230,13 +224,13 @@ class CALessBase(IntegrationTest):
         ]
 
         if http_pkcs12:
-            args.extend(['--http_pkcs12', http_pkcs12])
+            args.extend(['--http-cert-file', http_pkcs12])
         if dirsrv_pkcs12:
-            args.extend(['--dirsrv_pkcs12', dirsrv_pkcs12])
+            args.extend(['--dirsrv-cert-file', dirsrv_pkcs12])
         if http_pin is not None:
-            args.extend(['--http_pin', http_pin])
+            args.extend(['--http-pin', http_pin])
         if dirsrv_pin is not None:
-            args.extend(['--dirsrv_pin', dirsrv_pin])
+            args.extend(['--dirsrv-pin', dirsrv_pin])
 
         args.extend([replica.hostname])
 
@@ -343,12 +337,6 @@ class TestServerInstall(CALessBase):
     def tearDown(self):
         self.uninstall_server()
 
-        # Remove CA cert in /etc/pki/nssdb, in case of failed (un)install
-        for host in self.get_all_hosts():
-            self.master.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
-                                     '-n', 'External CA cert'],
-                                    raiseonerr=False)
-
     def test_nonexistent_ca_pem_file(self):
         "IPA server install with non-existent CA PEM file "
 
@@ -428,8 +416,8 @@ class TestServerInstall(CALessBase):
 
         result = self.install_server(http_pin=None)
         assert_error(result,
-                     'ipa-server-install: error: You must specify --http_pin '
-                     'with --http_pkcs12')
+                     'ipa-server-install: error: You must specify --http-pin '
+                     'with --http-cert-file')
 
     def test_missing_ds_password(self):
         "IPA server install with missing DS PKCS#12 password (unattended)"
@@ -441,7 +429,7 @@ class TestServerInstall(CALessBase):
         result = self.install_server(dirsrv_pin=None)
         assert_error(result,
                      'ipa-server-install: error: You must specify '
-                     '--dirsrv_pin with --dirsrv_pkcs12')
+                     '--dirsrv-pin with --dirsrv-cert-file')
 
     def test_incorect_http_pin(self):
         "IPA server install with incorrect HTTP PKCS#12 password"
@@ -769,12 +757,7 @@ class TestReplicaInstall(CALessBase):
         self.master.run_command(['ipa', 'host-del', replica.hostname],
                                 raiseonerr=False)
 
-        replica.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
-                             '-n', 'External CA cert'], raiseonerr=False)
-
         self.uninstall_server()
-        self.master.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
-                                 '-n', 'External CA cert'], raiseonerr=False)
 
     def test_no_certs(self):
         "IPA replica install without certificates"
@@ -784,8 +767,9 @@ class TestReplicaInstall(CALessBase):
                                          raiseonerr=False)
         assert result.returncode > 0
         assert ('Cannot issue certificates: a CA is not installed. Use the '
-                '--http_pkcs12, --dirsrv_pkcs12 options to provide custom '
-                'certificates.' in result.stderr_text), result.stderr_text
+                '--http-cert-file, --dirsrv-cert-file options to provide '
+                'custom certificates.' in result.stderr_text), \
+               result.stderr_text
 
     def test_nonexistent_http_pkcs12_file(self):
         "IPA replica install with non-existent HTTP PKCS#12 file"
@@ -1142,11 +1126,10 @@ class TestClientInstall(CALessBase):
         self.verify_installation()
 
 
-@ordered
 class TestIPACommands(CALessBase):
     @classmethod
-    def install(cls):
-        super(TestIPACommands, cls).install()
+    def install(cls, mh):
+        super(TestIPACommands, cls).install(mh)
 
         cls.export_pkcs12('ca1/server')
         with open(cls.pem_filename, 'w') as f:
@@ -1168,18 +1151,16 @@ class TestIPACommands(CALessBase):
         result = self.master.run_command(['ipa', command], raiseonerr=False)
         assert_error(result, "ipa: ERROR: unknown command '%s'" % command)
 
-    def test_cert_commands_unavailable(self):
-        for cmd in (
-                'cert-status',
-                'cert-show',
-                'cert-find',
-                'cert-revoke',
-                'cert-remove-hold',
-                'cert-status'):
-            func = lambda: self.check_ipa_command_not_available(cmd)
-            func.description = 'Verify that %s command is not available' % cmd
-            func.test_argument = cmd
-            yield (func, )
+    @pytest.mark.parametrize('command', (
+        'cert-status',
+        'cert-show',
+        'cert-find',
+        'cert-revoke',
+        'cert-remove-hold',
+        'cert-status'))
+    def test_cert_commands_unavailable(self, command):
+        result = self.master.run_command(['ipa', command], raiseonerr=False)
+        assert_error(result, "ipa: ERROR: unknown command '%s'" % command)
 
     def test_cert_help_unavailable(self):
         "Verify that cert plugin help is not available"
@@ -1247,7 +1228,7 @@ class TestIPACommands(CALessBase):
 
 class TestCertinstall(CALessBase):
     @classmethod
-    def install(cls):
+    def install(cls, mh):
         super(TestCertinstall, cls).install()
 
         cls.export_pkcs12('ca1/server')
@@ -1479,7 +1460,7 @@ class TestCertinstall(CALessBase):
 
         args = ['ipa-server-certinstall',
                 '-w', 'server.p12',
-                '--http_pin', self.cert_password]
+                '--http-pin', self.cert_password]
 
         result = self.certinstall('w', 'ca1/server', args=args)
         assert result.returncode == 0
@@ -1490,7 +1471,7 @@ class TestCertinstall(CALessBase):
 
         args = ['ipa-server-certinstall',
                 '-d', 'server.p12',
-                '--dirsrv_pin', self.cert_password]
+                '--dirsrv-pin', self.cert_password]
         stdin_text = self.master.config.dirman_password + '\n'
 
         result = self.certinstall('d', 'ca1/server',
