@@ -32,7 +32,11 @@ from ipatests.util import TempDir, TempHome
 from ipalib.constants import TYPE_ERROR, OVERRIDE_ERROR, SET_ERROR, DEL_ERROR
 from ipalib.constants import NAME_REGEX, NAME_ERROR
 from ipalib import config, constants, base
+from ipaplatform.paths import paths
 
+import pytest
+
+pytestmark = pytest.mark.tier0
 
 # Valid environment variables in (key, raw, value) tuples:
 #    key: the name of the environment variable
@@ -43,8 +47,7 @@ good_vars = (
     ('trailing_whitespace', u' value  ', u'value'),
     ('an_int', 42, 42),
     ('int_repr', ' 42 ', 42),
-    ('a_float', 3.14, 3.14),
-    ('float_repr', ' 3.14 ', 3.14),
+    ('not_a_float', '3.14', u'3.14'),
     ('true', True, True),
     ('true_repr', ' True ', True),
     ('false', False, False),
@@ -166,7 +169,7 @@ class test_Env(ClassChecker):
         assert o.__islocked__() is False
         o.__lock__()
         assert o.__islocked__() is True
-        e = raises(StandardError, o.__lock__)
+        e = raises(Exception, o.__lock__)
         assert str(e) == 'Env.__lock__() already called'
 
         # Also test with base.lock() function:
@@ -298,7 +301,7 @@ class test_Env(ClassChecker):
         """
         o = self.cls()
         assert len(o) == 0
-        for i in xrange(1, 11):
+        for i in range(1, 11):
             key = 'key%d' % i
             value = u'value %d' % i
             o[key] = value
@@ -345,7 +348,7 @@ class test_Env(ClassChecker):
         expected.update(dict(group1))
         assert list(o) == sorted(expected)
         assert expected['key2'] == 'value 2'  # And not 'Value 2'
-        for (key, value) in expected.iteritems():
+        for (key, value) in expected.items():
             assert getattr(o, key) is value
             assert o[key] is value
         assert o._merge(**expected) == (0, 6)
@@ -390,7 +393,7 @@ class test_Env(ClassChecker):
         for (k, v) in orig.items():
             assert o[k] is v
         assert list(o) == sorted(keys + ('key0', 'key1', 'key2', 'key3', 'config_loaded'))
-        for i in xrange(4):
+        for i in range(4):
             assert o['key%d' % i] == ('var%d' % i)
         keys = tuple(o)
 
@@ -406,7 +409,7 @@ class test_Env(ClassChecker):
         assert o.yes is True
         assert o.no is False
         assert o.number == 42
-        assert o.floating == 3.14
+        assert o.floating == '3.14'
 
     def new(self, in_tree=False):
         """
@@ -429,7 +432,7 @@ class test_Env(ClassChecker):
         assert o._isdone('_bootstrap') is False
         o._bootstrap(**overrides)
         assert o._isdone('_bootstrap') is True
-        e = raises(StandardError, o._bootstrap)
+        e = raises(Exception, o._bootstrap)
         assert str(e) == 'Env._bootstrap() already called'
         return (o, home)
 
@@ -449,8 +452,8 @@ class test_Env(ClassChecker):
         assert o.dot_ipa == home.join('.ipa')
         assert o.in_tree is False
         assert o.context == 'default'
-        assert o.confdir == '/etc/ipa'
-        assert o.conf == '/etc/ipa/default.conf'
+        assert o.confdir == paths.ETC_IPA
+        assert o.conf == paths.IPA_DEFAULT_CONF
         assert o.conf_default == o.conf
 
         # Test overriding values created by _bootstrap()
@@ -462,11 +465,11 @@ class test_Env(ClassChecker):
         assert o.in_tree is False
         assert o.context == 'default'
         assert o.conf == '/my/wacky/whatever.conf'
-        assert o.conf_default == '/etc/ipa/default.conf'
+        assert o.conf_default == paths.IPA_DEFAULT_CONF
         (o, home) = self.bootstrap(conf_default='/my/wacky/default.conf')
         assert o.in_tree is False
         assert o.context == 'default'
-        assert o.conf == '/etc/ipa/default.conf'
+        assert o.conf == paths.IPA_DEFAULT_CONF
         assert o.conf_default == '/my/wacky/default.conf'
 
         # Test various overrides and types conversion
@@ -512,7 +515,7 @@ class test_Env(ClassChecker):
             assert key in o
 
         # Check that it can't be called twice:
-        e = raises(StandardError, o._finalize_core)
+        e = raises(Exception, o._finalize_core)
         assert str(e) == 'Env._finalize_core() already called'
 
         return (o, home)
@@ -586,7 +589,7 @@ class test_Env(ClassChecker):
         assert o._isdone('_finalize') is True
 
         # Check that it can't be called twice:
-        e = raises(StandardError, o._finalize)
+        e = raises(Exception, o._finalize)
         assert str(e) == 'Env._finalize() already called'
 
         # Check that _finalize() calls __lock__()
@@ -594,7 +597,7 @@ class test_Env(ClassChecker):
         assert o.__islocked__() is False
         o._finalize()
         assert o.__islocked__() is True
-        e = raises(StandardError, o.__lock__)
+        e = raises(Exception, o.__lock__)
         assert str(e) == 'Env.__lock__() already called'
 
         # Check that **lastchance works
