@@ -27,6 +27,10 @@ from ipalib.frontend import Command
 from ipalib import _
 from ipapython.version import API_VERSION
 
+import pytest
+
+pytestmark = pytest.mark.tier0
+
 class test_Output(ClassChecker):
     """
     Test the `ipalib.output.Output` class.
@@ -71,9 +75,10 @@ class test_ListOfEntries(ClassChecker):
         """
         Test the `ipalib.output.ListOfEntries.validate` method.
         """
+        api = 'the api instance'
         class example(Command):
             pass
-        cmd = example()
+        cmd = example(api)
         inst = self.cls('stuff')
 
         okay = dict(foo='bar')
