@@ -22,12 +22,19 @@
 Test the `ipalib.parameters` module.
 """
 
+# FIXME: Pylint errors
+# pylint: disable=no-member
+
 import datetime
 import re
 import sys
-from types import NoneType
 from decimal import Decimal
 from inspect import isclass
+from six.moves.xmlrpc_client import MAXINT, MININT
+import pytest
+
+import six
+
 from ipatests.util import raises, ClassChecker, read_only
 from ipatests.util import dummy_ugettext, assert_equal
 from ipatests.data import binary_bytes, utf8_bytes, unicode_str
@@ -35,9 +42,15 @@ from ipalib import parameters, text, errors, config
 from ipalib.constants import TYPE_ERROR, CALLABLE_ERROR
 from ipalib.errors import ValidationError, ConversionError
 from ipalib import _
-from xmlrpclib import MAXINT, MININT
 
-NULLS = (None, '', u'', tuple(), [])
+if six.PY3:
+    unicode = str
+    long = int
+
+NULLS = (None, b'', u'', tuple(), [])
+
+pytestmark = pytest.mark.tier0
+
 
 class test_DefaultFrom(ClassChecker):
     """
@@ -147,8 +160,12 @@ def test_parse_param_spec():
     assert f('name^') == ('name^', dict(required=True, multivalue=False))
 
     # Test that TypeError is raised if spec isn't an str:
-    e = raises(TypeError, f, u'name?')
-    assert str(e) == TYPE_ERROR % ('spec', str, u'name?', unicode)
+    if six.PY2:
+        bad_value = u'name?'
+    else:
+        bad_value = b'name?'
+    e = raises(TypeError, f, bad_value)
+    assert str(e) == TYPE_ERROR % ('spec', str, bad_value, type(bad_value))
 
 
 class DummyRule(object):
@@ -219,12 +236,12 @@ class test_Param(ClassChecker):
 
         # Test that ValueError is raised when a kwarg from a subclass
         # conflicts with an attribute:
-        class Subclass(self.cls):
+        class Subclass1(self.cls):
             kwargs = self.cls.kwargs + (
                 ('convert', callable, None),
             )
-        e = raises(ValueError, Subclass, name)
-        assert str(e) == "kwarg 'convert' conflicts with attribute on Subclass"
+        e = raises(ValueError, Subclass1, name)
+        assert str(e) == "kwarg 'convert' conflicts with attribute on Subclass1"
 
         # Test type validation of keyword arguments:
         class Subclass(self.cls):
@@ -471,10 +488,10 @@ class test_Param(ClassChecker):
 
         # Test with wrong (scalar) type:
         e = raises(TypeError, o.validate, (None, None, 42, None), 'cli')
-        assert str(e) == TYPE_ERROR % ('my_param', NoneType, 42, int)
+        assert str(e) == TYPE_ERROR % ('my_param', type(None), 42, int)
         o = self.cls('my_param')
         e = raises(TypeError, o.validate, 'Hello', 'cli')
-        assert str(e) == TYPE_ERROR % ('my_param', NoneType, 'Hello', str)
+        assert str(e) == TYPE_ERROR % ('my_param', type(None), 'Hello', str)
 
         class Example(self.cls):
             type = int
@@ -593,6 +610,8 @@ class test_Param(ClassChecker):
             type = unicode
 
             def __init__(self, name, **kw):
+                # (Pylint complains because the superclass is unknowm)
+                # pylint: disable=bad-super-call, super-on-old-class
                 self._convert_scalar = PassThrough()
                 super(Str, self).__init__(name, **kw)
 
@@ -639,7 +658,7 @@ class test_Flag(ClassChecker):
 
         # Test that TypeError is raise if default is not a bool:
         e = raises(TypeError, self.cls, 'my_flag', default=None)
-        assert str(e) == TYPE_ERROR % ('default', bool, None, NoneType)
+        assert str(e) == TYPE_ERROR % ('default', bool, None, type(None))
 
         # Test with autofill=False, default=True
         o = self.cls('my_flag', autofill=False, default=True)
@@ -674,7 +693,7 @@ class test_Data(ClassChecker):
         Test the `ipalib.parameters.Data.__init__` method.
         """
         o = self.cls('my_data')
-        assert o.type is NoneType
+        assert o.type is type(None)
         assert o.password is False
         assert o.rules == tuple()
         assert o.class_rules == tuple()
@@ -682,7 +701,6 @@ class test_Data(ClassChecker):
         assert o.minlength is None
         assert o.maxlength is None
         assert o.length is None
-        assert o.pattern is None
 
         # Test mixing length with minlength or maxlength:
         o = self.cls('my_data', length=5)
@@ -694,7 +712,7 @@ class test_Data(ClassChecker):
         ]
         for kw in permutations:
             o = self.cls('my_data', **kw)
-            for (key, value) in kw.iteritems():
+            for (key, value) in kw.items():
                 assert getattr(o, key) == value
             e = raises(ValueError, self.cls, 'my_data', length=5, **kw)
             assert str(e) == \
@@ -728,7 +746,7 @@ class test_Bytes(ClassChecker):
         Test the `ipalib.parameters.Bytes.__init__` method.
         """
         o = self.cls('my_bytes')
-        assert o.type is str
+        assert o.type is bytes
         assert o.password is False
         assert o.rules == tuple()
         assert o.class_rules == tuple()
@@ -755,7 +773,7 @@ class test_Bytes(ClassChecker):
             assert len(o.class_rules) == len(kw)
             assert len(o.rules) == 0
             assert len(o.all_rules) == len(kw)
-            for (key, value) in kw.iteritems():
+            for (key, value) in kw.items():
                 assert getattr(o, key) == value
             e = raises(ValueError, self.cls, 'my_bytes', length=5, **kw)
             assert str(e) == \
@@ -789,12 +807,12 @@ class test_Bytes(ClassChecker):
         assert dummy.translation is translation
 
         # Test with passing values:
-        for value in ('abc', 'four', '12345'):
+        for value in (b'abc', b'four', b'12345'):
             assert rule(dummy, value) is None
             assert dummy.called() is False
 
         # Test with failing values:
-        for value in ('', 'a', '12'):
+        for value in (b'', b'a', b'12'):
             assert_equal(
                 rule(dummy, value),
                 translation % dict(minlength=3)
@@ -815,12 +833,12 @@ class test_Bytes(ClassChecker):
         assert dummy.translation is translation
 
         # Test with passing values:
-        for value in ('ab', '123', 'four'):
+        for value in (b'ab', b'123', b'four'):
             assert rule(dummy, value) is None
             assert dummy.called() is False
 
         # Test with failing values:
-        for value in ('12345', 'sixsix'):
+        for value in (b'12345', b'sixsix'):
             assert_equal(
                 rule(dummy, value),
                 translation % dict(maxlength=4)
@@ -841,12 +859,12 @@ class test_Bytes(ClassChecker):
         assert dummy.translation is translation
 
         # Test with passing values:
-        for value in ('1234', 'four'):
+        for value in (b'1234', b'four'):
             assert rule(dummy, value) is None
             assert dummy.called() is False
 
         # Test with failing values:
-        for value in ('ab', '123', '12345', 'sixsix'):
+        for value in (b'ab', b'123', b'12345', b'sixsix'):
             assert_equal(
                 rule(dummy, value),
                 translation % dict(length=4),
@@ -860,9 +878,9 @@ class test_Bytes(ClassChecker):
         Test the `ipalib.parameters.Bytes._rule_pattern` method.
         """
         # Test our assumptions about Python re module and Unicode:
-        pat = '\w+$'
+        pat = b'\w+$'
         r = re.compile(pat)
-        assert r.match('Hello_World') is not None
+        assert r.match(b'Hello_World') is not None
         assert r.match(utf8_bytes) is None
         assert r.match(binary_bytes) is None
 
@@ -874,12 +892,12 @@ class test_Bytes(ClassChecker):
         dummy = dummy_ugettext(translation)
 
         # Test with passing values:
-        for value in ('HELLO', 'hello', 'Hello_World'):
+        for value in (b'HELLO', b'hello', b'Hello_World'):
             assert rule(dummy, value) is None
             assert dummy.called() is False
 
         # Test with failing values:
-        for value in ('Hello!', 'Hello World', utf8_bytes, binary_bytes):
+        for value in (b'Hello!', b'Hello World', utf8_bytes, binary_bytes):
             assert_equal(
                 rule(dummy, value),
                 translation % dict(pattern=pat),
@@ -915,7 +933,7 @@ class test_Str(ClassChecker):
         mthd = o._convert_scalar
         for value in (u'Hello', 42, 1.2, unicode_str):
             assert mthd(value) == unicode(value)
-        bad = [True, 'Hello', dict(one=1), utf8_bytes]
+        bad = [True, b'Hello', dict(one=1), utf8_bytes]
         for value in bad:
             e = raises(errors.ConversionError, mthd, value)
             assert e.name == 'my_str'
@@ -1019,7 +1037,10 @@ class test_Str(ClassChecker):
         pat = '\w{5}$'
         r1 = re.compile(pat)
         r2 = re.compile(pat, re.UNICODE)
-        assert r1.match(unicode_str) is None
+        if six.PY2:
+            assert r1.match(unicode_str) is None
+        else:
+            assert r1.match(unicode_str) is not None
         assert r2.match(unicode_str) is not None
 
         # Create instance:
@@ -1155,8 +1176,8 @@ class test_StrEnum(EnumChecker):
     _name = 'my_strenum'
     _datatype = unicode
     _test_values = u'Hello', u'naughty', u'nurse!'
-    _bad_type_values = u'Hello', 'naughty', u'nurse!'
-    _bad_type = str
+    _bad_type_values = u'Hello', b'naughty', u'nurse!'
+    _bad_type = bytes
     _translation = u"values='Hello', 'naughty', 'nurse!'"
     _bad_values = u'Howdy', u'quiet', u'library!'
     _single_value_translation = u"value='Hello'"
@@ -1175,9 +1196,9 @@ def check_int_scalar_conversions(o):
         assert e.name == 'my_number'
         assert e.index is None
     # Assure large magnitude values are handled correctly
-    assert type(o._convert_scalar(sys.maxint * 2)) == long
-    assert o._convert_scalar(sys.maxint * 2) == sys.maxint * 2
-    assert o._convert_scalar(unicode(sys.maxint * 2)) == sys.maxint * 2
+    assert type(o._convert_scalar(sys.maxsize * 2)) == long
+    assert o._convert_scalar(sys.maxsize * 2) == sys.maxsize * 2
+    assert o._convert_scalar(unicode(sys.maxsize * 2)) == sys.maxsize * 2
     assert o._convert_scalar(long(16)) == 16
     # Assure normal conversions produce expected result
     assert o._convert_scalar(u'16.99') == 16
@@ -1185,6 +1206,7 @@ def check_int_scalar_conversions(o):
     assert o._convert_scalar(u'16') == 16
     assert o._convert_scalar(u'0x10') == 16
     assert o._convert_scalar(u'020') == 16
+    assert o._convert_scalar(u'0o20') == 16
 
 
 class test_IntEnum(EnumChecker):
@@ -1220,7 +1242,7 @@ class test_Number(ClassChecker):
         Test the `ipalib.parameters.Number.__init__` method.
         """
         o = self.cls('my_number')
-        assert o.type is NoneType
+        assert o.type is type(None)
         assert o.password is False
         assert o.rules == tuple()
         assert o.class_rules == tuple()
@@ -1241,7 +1263,7 @@ class test_Int(ClassChecker):
         # Test with no kwargs:
         o = self.cls('my_number')
         assert o.type == int
-        assert o.allowed_types == (int, long)
+        assert o.allowed_types == six.integer_types
         assert isinstance(o, parameters.Int)
         assert o.minvalue == int(MININT)
         assert o.maxvalue == int(MAXINT)
@@ -1419,8 +1441,7 @@ class test_Decimal(ClassChecker):
         param = self.cls('my_number', precision=1)
         e = raises(ConversionError, param, '123456789012345678901234567890')
 
-        assert str(e) == \
-        "invalid 'my_number': quantize result has too many digits for current context"
+        assert str(e).startswith("invalid 'my_number': ")
 
     def test_exponential(self):
         """
@@ -1541,7 +1562,11 @@ def test_create_param():
         assert p.multivalue is kw['multivalue']
 
     # Test that TypeError is raised when spec is neither a Param nor a str:
-    for spec in (u'one', 42, parameters.Param, parameters.Str):
+    if six.PY2:
+        bad_value = u'one'
+    else:
+        bad_value = b'one'
+    for spec in (bad_value, 42, parameters.Param, parameters.Str):
         e = raises(TypeError, f, spec)
         assert str(e) == \
             TYPE_ERROR % ('spec', (str, parameters.Param), spec, type(spec))
@@ -1580,7 +1605,10 @@ class test_IA5Str(ClassChecker):
             e = raises(errors.ConversionError, mthd, value)
             assert e.name == 'my_str'
             assert e.index is None
-            assert_equal(e.error, "The character '\\xc3' is not allowed.")
+            if six.PY2:
+                assert_equal(e.error, "The character '\\xc3' is not allowed.")
+            else:
+                assert_equal(e.error, "The character 'á' is not allowed.")
 
 
 class test_DateTime(ClassChecker):
@@ -1625,4 +1653,4 @@ class test_DateTime(ClassChecker):
                       u'1991-31-12Z',
                       u'1991-12-07T25:30:05Z',
             ):
-            raises(ConversionError, o.convert, value)
\ No newline at end of file
+            raises(ConversionError, o.convert, value)
