@@ -20,13 +20,18 @@
 """
 Test the `ipalib.text` module.
 """
+from __future__ import print_function
 
 import os
 import shutil
 import tempfile
 import re
+
 import nose
 import locale
+import six
+import pytest
+
 from ipatests.util import raises, assert_equal
 from ipatests.i18n import create_po, po_file_iterate
 from ipalib.request import context
@@ -34,6 +39,11 @@ from ipalib import request
 from ipalib import text
 from ipapython.ipautil import file_exists
 
+if six.PY3:
+    unicode = str
+
+pytestmark = pytest.mark.tier0
+
 singular = '%(count)d goose makes a %(dish)s'
 plural = '%(count)d geese make a %(dish)s'
 
@@ -46,7 +56,7 @@ def test_create_translation():
 
 
 class test_TestLang(object):
-    def setUp(self):
+    def setup(self):
         self.tmp_dir = None
         self.saved_lang  = None
 
@@ -85,7 +95,7 @@ class test_TestLang(object):
 
         self.po_file_iterate = po_file_iterate
 
-    def tearDown(self):
+    def teardown(self):
         if self.saved_lang is not None:
             os.environ['LANG'] = self.saved_lang
 
@@ -93,7 +103,7 @@ class test_TestLang(object):
             shutil.rmtree(self.tmp_dir)
 
     def test_test_lang(self):
-        print "test_test_lang"
+        print("test_test_lang")
         # The test installs the test message catalog under the xh_ZA
         # (e.g. Zambia Xhosa) language by default. It would be nice to
         # use a dummy language not associated with any real language,
