@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 # Authors:
 #   Jan Cholasta <jcholast@redhat.com>
 #
@@ -20,20 +22,29 @@
 Test the `ipapython/ipautil.py` module.
 """
 
+import sys
+
 import nose
+import pytest
+import six
 
 from ipapython import ipautil
 
-class CheckIPAddress:
-    def __init__(self, addr):
-        self.description = "Test IP address parsing and verification (%s)" % addr
+import pytest
 
-    def __call__(self, addr, words=None, prefixlen=None):
+pytestmark = pytest.mark.tier0
+
+
+def make_ipaddress_checker(addr, words=None, prefixlen=None):
+    def check_ipaddress():
         try:
             ip = ipautil.CheckedIPAddress(addr, match_local=False)
             assert ip.words == words and ip.prefixlen == prefixlen
         except:
             assert words is None and prefixlen is None
+    check_ipaddress.description = "Test IP address parsing and verification (%s)" % addr
+    return check_ipaddress
+
 
 def test_ip_address():
     addrs = [
@@ -66,7 +77,7 @@ def test_ip_address():
     ]
 
     for addr in addrs:
-        yield (CheckIPAddress(addr[0]),) + addr
+        yield make_ipaddress_checker(*addr)
 
 
 class TestCIDict(object):
@@ -115,27 +126,37 @@ class TestCIDict(object):
         nose.tools.assert_equal("newval4", self.cidict["key4"])
 
     def test_del(self):
-        assert self.cidict.has_key("Key1")
+        assert "Key1" in self.cidict
         del(self.cidict["Key1"])
-        assert not self.cidict.has_key("Key1")
+        assert "Key1" not in self.cidict
 
-        assert self.cidict.has_key("key2")
+        assert "key2" in self.cidict
         del(self.cidict["KEY2"])
-        assert not self.cidict.has_key("key2")
+        assert "key2" not in self.cidict
 
     def test_clear(self):
         nose.tools.assert_equal(3, len(self.cidict))
         self.cidict.clear()
         nose.tools.assert_equal(0, len(self.cidict))
+        assert self.cidict == {}
+        assert list(self.cidict) == []
+        assert list(self.cidict.values()) == []
+        assert list(self.cidict.items()) == []
+        if six.PY2:
+            assert self.cidict.keys() == []
+            assert self.cidict.values() == []
+            assert self.cidict.items() == []
+        assert self.cidict._keys == {}
 
     def test_copy(self):
         copy = self.cidict.copy()
         assert copy == self.cidict
         nose.tools.assert_equal(3, len(copy))
-        assert copy.has_key("Key1")
-        assert copy.has_key("key1")
+        assert "Key1" in copy
+        assert "key1" in copy
         nose.tools.assert_equal("val1", copy["Key1"])
 
+    @pytest.mark.skipif(not six.PY2, reason="Python 2 only")
     def test_haskey(self):
         assert self.cidict.has_key("KEY1")
         assert self.cidict.has_key("key2")
@@ -151,15 +172,15 @@ class TestCIDict(object):
         assert "Key4" not in self.cidict
 
     def test_items(self):
-        items = self.cidict.items()
+        items = list(self.cidict.items())
         nose.tools.assert_equal(3, len(items))
         items_set = set(items)
         assert ("Key1", "val1") in items_set
         assert ("key2", "val2") in items_set
         assert ("KEY3", "VAL3") in items_set
 
-        assert self.cidict.items() == list(self.cidict.iteritems()) == zip(
-            self.cidict.iterkeys(), self.cidict.itervalues())
+        assert list(self.cidict.items()) == list(self.cidict.iteritems()) == list(zip(
+            self.cidict.keys(), self.cidict.values()))
 
     def test_iter(self):
         items = []
@@ -197,24 +218,24 @@ class TestCIDict(object):
         assert "VAL3" in values_set
 
     def test_keys(self):
-        keys = self.cidict.keys()
+        keys = list(self.cidict.keys())
         nose.tools.assert_equal(3, len(keys))
         keys_set = set(keys)
         assert "Key1" in keys_set
         assert "key2" in keys_set
         assert "KEY3" in keys_set
 
-        assert self.cidict.keys() == list(self.cidict.iterkeys())
+        assert list(self.cidict.keys()) == list(self.cidict.iterkeys())
 
     def test_values(self):
-        values = self.cidict.values()
+        values = list(self.cidict.values())
         nose.tools.assert_equal(3, len(values))
         values_set = set(values)
         assert "val1" in values_set
         assert "val2" in values_set
         assert "VAL3" in values_set
 
-        assert self.cidict.values() == list(self.cidict.itervalues())
+        assert list(self.cidict.values()) == list(self.cidict.itervalues())
 
     def test_update(self):
         newdict = { "KEY2": "newval2",
@@ -222,7 +243,7 @@ class TestCIDict(object):
         self.cidict.update(newdict)
         nose.tools.assert_equal(4, len(self.cidict))
 
-        items = self.cidict.items()
+        items = list(self.cidict.items())
         nose.tools.assert_equal(4, len(items))
         items_set = set(items)
         assert ("Key1", "val1") in items_set
@@ -263,22 +284,22 @@ class TestCIDict(object):
     def test_setdefault(self):
         nose.tools.assert_equal("val1", self.cidict.setdefault("KEY1", "default"))
 
-        assert not self.cidict.has_key("KEY4")
+        assert "KEY4" not in self.cidict
         nose.tools.assert_equal("default", self.cidict.setdefault("KEY4", "default"))
-        assert self.cidict.has_key("KEY4")
+        assert "KEY4" in self.cidict
         nose.tools.assert_equal("default", self.cidict["key4"])
 
-        assert not self.cidict.has_key("KEY5")
+        assert "KEY5" not in self.cidict
         nose.tools.assert_equal(None, self.cidict.setdefault("KEY5"))
-        assert self.cidict.has_key("KEY5")
+        assert "KEY5" in self.cidict
         nose.tools.assert_equal(None, self.cidict["key5"])
 
     def test_pop(self):
         nose.tools.assert_equal("val1", self.cidict.pop("KEY1", "default"))
-        assert not self.cidict.has_key("key1")
+        assert "key1" not in self.cidict
 
         nose.tools.assert_equal("val2", self.cidict.pop("KEY2"))
-        assert not self.cidict.has_key("key2")
+        assert "key2" not in self.cidict
 
         nose.tools.assert_equal("default", self.cidict.pop("key4", "default"))
         with nose.tools.assert_raises(KeyError):
@@ -306,15 +327,7 @@ class TestCIDict(object):
     def test_fromkeys(self):
         dct = ipautil.CIDict.fromkeys(('A', 'b', 'C'))
         assert sorted(dct.keys()) == sorted(['A', 'b', 'C'])
-        assert sorted(dct.values()) == [None] * 3
-
-    def test_clear(self):
-        self.cidict.clear()
-        assert self.cidict == {}
-        assert self.cidict.keys() == []
-        assert self.cidict.values() == []
-        assert self.cidict.items() == []
-        assert self.cidict._keys == {}
+        assert list(dct.values()) == [None] * 3
 
 
 class TestTimeParser(object):
@@ -373,6 +386,8 @@ class TestTimeParser(object):
         nose.tools.assert_equal(800000, time.microsecond)
 
     def test_time_zones(self):
+        # pylint: disable=no-member
+
         timestr = "20051213141205Z"
 
         time = ipautil.parse_generalized_time(timestr)
@@ -406,3 +421,62 @@ class TestTimeParser(object):
         nose.tools.assert_equal(-30, time.tzinfo.minoffset)
         offset = time.tzinfo.utcoffset(time.tzinfo.dst())
         nose.tools.assert_equal(((24 - 9) * 60 * 60) - (30 * 60), offset.seconds)
+
+
+def test_run():
+    result = ipautil.run(['echo', 'foo\x02bar'],
+                         capture_output=True,
+                         capture_error=True)
+    assert result.returncode == 0
+    assert result.output == 'foo\x02bar\n'
+    assert result.raw_output == b'foo\x02bar\n'
+    assert result.error_output == ''
+    assert result.raw_error_output == b''
+
+
+def test_run_no_capture_output():
+    result = ipautil.run(['echo', 'foo\x02bar'])
+    assert result.returncode == 0
+    assert result.output is None
+    assert result.raw_output == b'foo\x02bar\n'
+    assert result.error_output is None
+    assert result.raw_error_output == b''
+
+
+def test_run_bytes():
+    result = ipautil.run(['echo', b'\x01\x02'], capture_output=True)
+    assert result.returncode == 0
+    assert result.raw_output == b'\x01\x02\n'
+
+
+def test_run_decode():
+    result = ipautil.run(['echo', u'á'.encode('utf-8')],
+                         encoding='utf-8', capture_output=True)
+    assert result.returncode == 0
+    if six.PY3:
+        assert result.output == 'á\n'
+    else:
+        assert result.output == 'á\n'.encode('utf-8')
+
+
+def test_run_decode_bad():
+    if six.PY3:
+        with pytest.raises(UnicodeDecodeError):
+            ipautil.run(['echo', b'\xa0\xa1'],
+                        capture_output=True,
+                        encoding='utf-8')
+    else:
+        result = ipautil.run(['echo', '\xa0\xa1'],
+                             capture_output=True,
+                             encoding='utf-8')
+        assert result.returncode == 0
+        assert result.output == '\xa0\xa1\n'
+
+
+def test_backcompat():
+    result = out, err, rc = ipautil.run(['echo', 'foo\x02bar'],
+                                        capture_output=True,
+                                        capture_error=True)
+    assert rc is result.returncode
+    assert out is result.output
+    assert err is result.error_output
