@@ -22,12 +22,19 @@ Test the `ipaserver.rpc` module.
 """
 
 import json
+import pytest
+
+import six
 
 from ipatests.util import create_test_api, assert_equal, raises, PluginTester
 from ipatests.data import unicode_str
 from ipalib import errors, Command
 from ipaserver import rpcserver
 
+if six.PY3:
+    unicode = str
+
+pytestmark = pytest.mark.tier0
 
 class StartResponse(object):
     def __init__(self):
@@ -47,7 +54,8 @@ class StartResponse(object):
 
 
 def test_not_found():
-    f = rpcserver.HTTP_Status()
+    api = 'the api instance'
+    f = rpcserver.HTTP_Status(api)
     t = rpcserver._not_found_template
     s = StartResponse()
 
@@ -72,7 +80,8 @@ def test_not_found():
 
 
 def test_bad_request():
-    f = rpcserver.HTTP_Status()
+    api = 'the api instance'
+    f = rpcserver.HTTP_Status(api)
     t = rpcserver._bad_request_template
     s = StartResponse()
 
@@ -85,7 +94,8 @@ def test_bad_request():
 
 
 def test_internal_error():
-    f = rpcserver.HTTP_Status()
+    api = 'the api instance'
+    f = rpcserver.HTTP_Status(api)
     t = rpcserver._internal_error_template
     s = StartResponse()
 
@@ -98,7 +108,8 @@ def test_internal_error():
 
 
 def test_unauthorized_error():
-    f = rpcserver.HTTP_Status()
+    api = 'the api instance'
+    f = rpcserver.HTTP_Status(api)
     t = rpcserver._unauthorized_template
     s = StartResponse()
 
@@ -139,7 +150,8 @@ class test_session(object):
                 [environ[k] for k in ('SCRIPT_NAME', 'PATH_INFO')]
             )
 
-        inst = self.klass()
+        api = 'the api instance'
+        inst = self.klass(api)
         inst.mount(app1, '/foo/stuff')
         inst.mount(app2, '/bar')
 
@@ -157,13 +169,14 @@ class test_session(object):
             pass
 
         # Test that mount works:
-        inst = self.klass()
+        api = 'the api instance'
+        inst = self.klass(api)
         inst.mount(app1, 'foo')
         assert inst['foo'] is app1
         assert list(inst) == ['foo']
 
-        # Test that StandardError is raise if trying override a mount:
-        e = raises(StandardError, inst.mount, app2, 'foo')
+        # Test that Exception is raise if trying override a mount:
+        e = raises(Exception, inst.mount, app2, 'foo')
         assert str(e) == '%s.mount(): cannot replace %r with %r at %r' % (
             'wsgi_dispatch', app1, app2, 'foo'
         )
