@@ -20,21 +20,26 @@
 Test the `pkcs10.py` module.
 """
 
-import os
-import sys
+# FIXME: Pylint errors
+# pylint: disable=no-member
+
+import binascii
+
 import nose
-from ipatests.util import raises, PluginTester
 from ipalib import pkcs10
 from ipapython import ipautil
 import nss.nss as nss
 from nss.error import NSPRError
+import pytest
 
+
+@pytest.mark.tier0
 class test_update(object):
     """
     Test the PKCS#10 Parser.
     """
 
-    def setUp(self):
+    def setup(self):
         nss.nss_init_nodb()
         if ipautil.file_exists("test0.csr"):
             self.testdir="./"
@@ -108,7 +113,7 @@ class test_update(object):
 
         try:
             request = pkcs10.load_certificate_request(csr)
-        except NSPRError, nsprerr:
+        except NSPRError as nsprerr:
             # (SEC_ERROR_BAD_DER) security library: improperly formatted DER-encoded message.
             assert(nsprerr. errno== -8183)
 
@@ -119,5 +124,5 @@ class test_update(object):
         csr = self.read_file("test4.csr")
         try:
             request = pkcs10.load_certificate_request(csr)
-        except TypeError, typeerr:
+        except (TypeError, binascii.Error) as typeerr:
             assert(str(typeerr) == 'Incorrect padding')
