@@ -22,8 +22,17 @@ Test the `tests.util` module.
 """
 
 import re
-import util
-from util import raises, TYPE, VALUE, LEN, KEYS
+
+import six
+
+from ipatests import util
+from ipatests.util import raises, TYPE, VALUE, LEN, KEYS
+import pytest
+
+if six.PY3:
+    unicode = str
+
+pytestmark = pytest.mark.tier0
 
 
 class Prop(object):
@@ -107,7 +116,7 @@ class test_Fuzzy(object):
     def test_eq(self):
         assert (self.klass('bar') == u'foobar') is True
         assert (self.klass('^bar') == u'foobar') is False
-        assert (self.klass('bar', type=str) == u'foobar') is False
+        assert (self.klass('bar', type=bytes) == u'foobar') is False
 
         assert ('18' == self.klass()) is True
         assert ('18' == self.klass(type=int)) is False
@@ -121,7 +130,7 @@ class test_Fuzzy(object):
 
         t = lambda other: other.endswith('bar')
         assert (self.klass(test=t) == 'foobar') is True
-        assert (self.klass(test=t, type=unicode) == 'foobar') is False
+        assert (self.klass(test=t, type=unicode) == b'foobar') is False
         assert (self.klass(test=t) == 'barfoo') is False
 
         assert (False == self.klass()) is True
@@ -149,9 +158,9 @@ def test_assert_deepequal():
         'foo', u'hello', u'world', tuple()
     )
 
-    e = raises(AssertionError, f, 'hello', u'hello', 'foo')
+    e = raises(AssertionError, f, b'hello', u'hello', 'foo')
     assert str(e) == TYPE % (
-        'foo', str, unicode, 'hello', u'hello', tuple()
+        'foo', bytes, unicode, b'hello', u'hello', tuple()
     )
 
     e = raises(AssertionError, f, 18, 18.0, 'foo')
@@ -174,23 +183,23 @@ def test_assert_deepequal():
 
     # Test with bad compound values:
     b = [
-        'hello',
+        b'hello',
         dict(naughty=u'nurse'),
         18,
     ]
     e = raises(AssertionError, f, a, b, 'foo')
     assert str(e) == TYPE % (
-        'foo', unicode, str, u'hello', 'hello', (2,)
+        'foo', unicode, bytes, u'hello', b'hello', (2 if six.PY2 else 0,)
     )
 
     b = [
         u'hello',
-        dict(naughty='nurse'),
+        dict(naughty=b'nurse'),
         18,
     ]
     e = raises(AssertionError, f, a, b, 'foo')
     assert str(e) == TYPE % (
-        'foo', unicode, str, u'nurse', 'nurse', (1, 'naughty')
+        'foo', unicode, bytes, u'nurse', b'nurse', (1, 'naughty')
     )
 
     b = [
@@ -200,7 +209,7 @@ def test_assert_deepequal():
     ]
     e = raises(AssertionError, f, a, b, 'foo')
     assert str(e) == TYPE % (
-        'foo', int, float, 18, 18.0, (0,)
+        'foo', int, float, 18, 18.0, (0 if six.PY2 else 2,)
     )
 
     # List length mismatch
