@@ -29,8 +29,10 @@ import ipatests.test_webui.data_netgroup as netgroup
 import ipatests.test_webui.data_hbac as hbac
 import ipatests.test_webui.test_rbac as rbac
 import ipatests.test_webui.data_sudo as sudo
+import pytest
 
 
+@pytest.mark.tier1
 class test_hostgroup(UI_driver):
 
     @screenshot
@@ -48,7 +50,8 @@ class test_hostgroup(UI_driver):
         Hostgroup associations
         """
         self.init_app()
-        host = host_tasks(self.driver, self.config)
+        host = host_tasks()
+        host.setup(self.driver, self.config)
 
         # prepare
         # -------
@@ -89,7 +92,8 @@ class test_hostgroup(UI_driver):
         Hostgroup indirect associations
         """
         self.init_app()
-        host = host_tasks(self.driver, self.config)
+        host = host_tasks()
+        host.setup(self.driver, self.config)
 
         # add
         # ---
