@@ -22,15 +22,23 @@ Base class for UI integration tests.
 
 Contains browser driver and common tasks.
 """
+from __future__ import print_function
 
 import nose
+from datetime import datetime
 import time
 import re
 import os
+from functools import wraps
+from nose.plugins.skip import SkipTest
 
 try:
     from selenium import webdriver
     from selenium.common.exceptions import NoSuchElementException
+    from selenium.common.exceptions import InvalidElementStateException
+    from selenium.common.exceptions import StaleElementReferenceException
+    from selenium.common.exceptions import WebDriverException
+    from selenium.webdriver.common.action_chains import ActionChains
     from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
     from selenium.webdriver.common.keys import Keys
     from selenium.webdriver.common.by import By
@@ -45,7 +53,8 @@ try:
     NO_YAML = False
 except ImportError:
     NO_YAML = True
-from urllib2 import URLError
+from six.moves.urllib.error import URLError
+from ipaplatform.paths import paths
 
 ENV_MAP = {
     'MASTER': 'ipa_server',
@@ -76,22 +85,51 @@ DEFAULT_PORT = 4444
 DEFAULT_TYPE = 'local'
 
 
+def screenshot(fn):
+    """
+    Decorator for saving screenshot on exception (test fail)
+    Should be applied on methods of UI_driver subclasses
+    """
+    @wraps(fn)
+    def screenshot_wrapper(*args):
+        try:
+            return fn(*args)
+        except SkipTest:
+            raise
+        except Exception:
+            self = args[0]
+            name = '%s_%s_%s' % (
+                datetime.now().isoformat(),
+                self.__class__.__name__,
+                fn.__name__)
+            self.take_screenshot(name)
+            raise
+
+    return screenshot_wrapper
+
+
 class UI_driver(object):
     """
     Base class for all UI integration tests
     """
 
     @classmethod
-    def setUpClass(cls):
+    def setup_class(cls):
         if NO_SELENIUM:
             raise nose.SkipTest('Selenium not installed')
 
-    def __init__(self, driver=None, config=None):
+    def setup(self, driver=None, config=None):
         self.request_timeout = 30
         self.driver = driver
         self.config = config
         if not config:
             self.load_config()
+        if not self.driver:
+            self.driver = self.get_driver()
+        self.driver.maximize_window()
+
+    def teardown(self):
+        self.driver.quit()
 
     def load_config(self):
         """
@@ -107,9 +145,9 @@ class UI_driver(object):
             try:
                 with open(path, 'r') as conf:
                     self.config = yaml.load(conf)
-            except yaml.YAMLError, e:
+            except yaml.YAMLError as e:
                 raise nose.SkipTest("Invalid Web UI config.\n%s" % e)
-            except IOError, e:
+            except IOError as e:
                 raise nose.SkipTest("Can't load Web UI test config: %s" % e)
         else:
             self.config = {}
@@ -117,7 +155,7 @@ class UI_driver(object):
         c = self.config
 
         # override with environmental variables
-        for k, v in ENV_MAP.iteritems():
+        for k, v in ENV_MAP.items():
             val = os.environ.get(k)
             if val is not None:
                 c[v] = val
@@ -130,19 +168,6 @@ class UI_driver(object):
         if 'type' not in c:
             c['type'] = DEFAULT_TYPE
 
-    def setUp(self):
-        """
-        Test setup
-        """
-        if not self.driver:
-            self.driver = self.get_driver()
-
-    def tearDown(self):
-        """
-        Test clean up
-        """
-        self.driver.quit()
-
     def get_driver(self):
         """
         Get WebDriver according to configuration
@@ -155,7 +180,7 @@ class UI_driver(object):
 
         if browser == 'chromium':
             options = ChromeOptions()
-            options.binary_location = '/usr/bin/chromium-browser'
+            options.binary_location = paths.CHROMIUM_BROWSER
 
         if driver_type == 'remote':
             if not 'host' in self.config:
@@ -174,9 +199,9 @@ class UI_driver(object):
                 driver = webdriver.Remote(
                     command_executor='http://%s:%d/wd/hub' % (host, port),
                     desired_capabilities=capabilities)
-            except URLError, e:
+            except URLError as e:
                 raise nose.SkipTest('Error connecting to selenium server: %s' % e)
-            except RuntimeError, e:
+            except RuntimeError as e:
                 raise nose.SkipTest('Error while establishing webdriver: %s' % e)
         else:
             try:
@@ -189,9 +214,9 @@ class UI_driver(object):
                     if "ff_profile" in self.config:
                         fp = webdriver.FirefoxProfile(self.config["ff_profile"])
                     driver = webdriver.Firefox(fp)
-            except URLError, e:
+            except URLError as e:
                 raise nose.SkipTest('Error connecting to selenium server: %s' % e)
-            except RuntimeError, e:
+            except RuntimeError as e:
                 raise nose.SkipTest('Error while establishing webdriver: %s' % e)
 
         return driver
@@ -234,7 +259,7 @@ class UI_driver(object):
         """
         Test if dependencies were loaded. (Checks if UI has been rendered)
         """
-        indicator = self.find("span.network-activity-indicator", By.CSS_SELECTOR)
+        indicator = self.find(".global-activity-indicator", By.CSS_SELECTOR)
         return indicator is not None
 
     def has_ca(self):
@@ -259,9 +284,15 @@ class UI_driver(object):
         """
         Check if there is running AJAX request
         """
-        indicator = self.find("span.network-activity-indicator", By.CSS_SELECTOR)
-        displayed = indicator and indicator.is_displayed()
-        return displayed
+        global_indicators = self.find(".global-activity-indicator", By.CSS_SELECTOR, many=True)
+        for el in global_indicators:
+            try:
+                if not self.has_class(el, 'closed'):
+                    return True
+            except StaleElementReferenceException:
+                # we don't care. Happens when indicator is part of removed dialog.
+                continue
+        return False
 
     def wait(self, seconds=0.2):
         """
@@ -320,19 +351,19 @@ class UI_driver(object):
             if not new_password:
                 new_password = password
 
-            auth = self.get_auth_dialog()
+            auth = self.get_login_screen()
             login_tb = self.find("//input[@type='text'][@name='username']", 'xpath', auth, strict=True)
             psw_tb = self.find("//input[@type='password'][@name='password']", 'xpath', auth, strict=True)
             login_tb.send_keys(login)
             psw_tb.send_keys(password)
             psw_tb.send_keys(Keys.RETURN)
             self.wait(0.5)
-            self.wait_for_request()
+            self.wait_for_request(n=2)
 
             # reset password if needed
-            if self.get_auth_dialog():
-                newpw_tb = self.find("//input[@type='password'][@name='new_password']", 'xpath', auth, strict=True)
-                verify_tb = self.find("//input[@type='password'][@name='verify_password']", 'xpath', auth, strict=True)
+            newpw_tb = self.find("//input[@type='password'][@name='new_password']", 'xpath', auth)
+            verify_tb = self.find("//input[@type='password'][@name='verify_password']", 'xpath', auth)
+            if newpw_tb and newpw_tb.is_displayed():
                 newpw_tb.send_keys(new_password)
                 verify_tb.send_keys(new_password)
                 verify_tb.send_keys(Keys.RETURN)
@@ -343,27 +374,34 @@ class UI_driver(object):
         """
         Check if user is logged in
         """
-        login_as = self.find('header-loggedinas', 'class name')
-        visible_name = login_as and login_as.is_displayed()
-        logged_in = not self.auth_dialog_opened() and visible_name
+        login_as = self.find('loggedinas', 'class name')
+        visible_name = len(login_as.text) > 0
+        logged_in = not self.login_screen_visible() and visible_name
         return logged_in
 
     def logout(self):
-        btn = self.find('logout', 'class name')
-        btn.click()
+        self.profile_menu_action('logout')
 
-    def get_auth_dialog(self):
+    def get_login_screen(self):
         """
-        Get reference to authentication dialog
+        Get reference of login screen
         """
-        return self.find('unauthorized_dialog', 'id')
+        return self.find('rcue-login-screen', 'id')
 
-    def auth_dialog_opened(self):
+    def login_screen_visible(self):
         """
-        Check if authentication dialog is opened
+        Check if login screen is visible
         """
-        dialog = self.get_auth_dialog()
-        return dialog and dialog.is_displayed()
+        screen = self.get_login_screen()
+        return screen and screen.is_displayed()
+
+    def take_screenshot(self, name):
+        if self.config.get('save_screenshots'):
+            scr_dir = self.config.get('screenshot_dir')
+            path = name + '.png'
+            if scr_dir:
+                path = os.path.join(scr_dir, path)
+            self.driver.get_screenshot_as_file(path)
 
     def navigate_to_entity(self, entity, facet=None):
         self.driver.get(self.get_url(entity, facet))
@@ -380,9 +418,9 @@ class UI_driver(object):
                 parent = parts[0:-1]
                 self.navigate_by_menu('/'.join(parent), complete)
 
-        s = ".navigation a[href='#%s']" % item
+        s = ".navbar a[href='#%s']" % item
         link = self.find(s, By.CSS_SELECTOR, strict=True)
-        assert link.is_displayed(), 'Navigation link is not displayed'
+        assert link.is_displayed(), 'Navigation link is not displayed: %s' % item
         link.click()
         self.wait_for_request()
         self.wait_for_request(0.4)
@@ -392,7 +430,7 @@ class UI_driver(object):
         Navigate by breadcrumb navigation
         """
         facet = self.get_facet()
-        nav = self.find('div.breadcrumb', By.CSS_SELECTOR, facet, strict=True)
+        nav = self.find('.breadcrumb', By.CSS_SELECTOR, facet, strict=True)
         a = self.find(item, By.LINK_TEXT, nav, strict=True)
         a.click()
         self.wait_for_request()
@@ -467,9 +505,9 @@ class UI_driver(object):
         """
         Get all dialogs in DOM
         """
-        s = 'div[role=dialog]'
+        s = '.modal-dialog'
         if name:
-            s += " div[data-name='%s'" % name
+            s += "[data-name='%s']" % name
         dialogs = self.find(s, By.CSS_SELECTOR, many=True)
         if strict:
             assert dialogs, "No dialogs found"
@@ -489,7 +527,7 @@ class UI_driver(object):
         """
         Get last opened error dialog or None.
         """
-        s = "div[role=dialog] div[data-name='%s']" % dialog_name
+        s = ".modal-dialog[data-name='%s']" % dialog_name
         dialogs = self.find(s, By.CSS_SELECTOR, many=True)
         dialog = None
         if dialogs:
@@ -505,10 +543,10 @@ class UI_driver(object):
 
         info = None
         if dialog:
-            content = self.find('div.ui-dialog-content', By.CSS_SELECTOR, dialog, strict=True)
+            body = self.find('.modal-body', By.CSS_SELECTOR, dialog, strict=True)
             info = {
-                'name': content.get_attribute('data-name'),
-                'text': content.text,
+                'name': dialog.get_attribute('data-name'),
+                'text': body.text,
             }
         return info
 
@@ -521,9 +559,9 @@ class UI_driver(object):
         var args = arguments[1];
         var options = arguments[2];
         var callback = arguments[arguments.length - 1];
-        var IPA = require('freeipa/ipa');
+        var rpc = require('freeipa/rpc');
 
-        var cmd = IPA.command({
+        var cmd = rpc.command({
             method: method,
             args: args,
             options: options,
@@ -552,7 +590,7 @@ class UI_driver(object):
         Click on facet button with given name
         """
         facet = self.get_facet()
-        s = ".facet-controls a[name=%s]" % name
+        s = ".facet-controls button[name=%s]" % name
         self._button_click(s, facet, name)
 
     def dialog_button_click(self, name, dialog=None):
@@ -564,7 +602,7 @@ class UI_driver(object):
         if not dialog:
             dialog = self.get_dialog(strict=True)
 
-        s = "div.ui-dialog-buttonset button[name=%s]" % name
+        s = ".rcue-dialog-buttons button[name='%s']" % name
         self._button_click(s, dialog, name)
 
     def action_button_click(self, name, parent):
@@ -584,17 +622,31 @@ class UI_driver(object):
         if not parent:
             parent = self.get_form()
 
-        s = "a[name='%s'].ui-button" % name
+        s = "[name='%s'].btn" % name
         self._button_click(s, parent, name)
 
     def _button_click(self, selector, parent, name=''):
         btn = self.find(selector, By.CSS_SELECTOR, parent, strict=True)
-        disabled = 'ui-state-disabled' in btn.get_attribute("class").split()
+        ActionChains(self.driver).move_to_element(btn).perform()
+        disabled = btn.get_attribute("disabled")
         assert btn.is_displayed(), 'Button is not displayed: %s' % name
         assert not disabled, 'Invalid button state: disabled. Button: %s' % name
         btn.click()
         self.wait_for_request()
 
+    def profile_menu_action(self, name):
+        """
+        Execute action from profile menu
+        """
+        menu_toggle = self.find('[name=profile-menu] > a', By.CSS_SELECTOR)
+        menu_toggle.click()
+        s = "[name=profile-menu] a[href='#%s']" % name
+        btn = self.find(s, By.CSS_SELECTOR, strict=True)
+        btn.click()
+        # action is usually followed by opening a dialog, add wait to compensate
+        # possible dialog transition effect
+        self.wait(0.5)
+
     def get_form(self):
         """
         Get last dialog or visible facet
@@ -621,8 +673,12 @@ class UI_driver(object):
         if not parent:
             parent = self.get_form()
         tb = self.find(selector, By.CSS_SELECTOR, parent, strict=True)
-        tb.clear()
-        tb.send_keys(value)
+        try:
+            tb.clear()
+            tb.send_keys(value)
+        except InvalidElementStateException as e:
+            msg = "Invalid Element State, el: %s, value: %s, error: %s" % (selector, value, e)
+            assert False, msg
 
     def fill_input(self, name, value, input_type="text", parent=None):
         """
@@ -658,7 +714,7 @@ class UI_driver(object):
             parent = self.get_form()
         s = "div[name='%s'].multivalued-widget" % name
         w = self.find(s, By.CSS_SELECTOR, parent, strict=True)
-        add_btn = self.find("Add", By.LINK_TEXT, w, strict=True)
+        add_btn = self.find("button[name=add]", By.CSS_SELECTOR, w, strict=True)
         add_btn.click()
         s = "div[name=value] input"
         inputs = self.find(s, By.CSS_SELECTOR, w, many=True)
@@ -680,7 +736,7 @@ class UI_driver(object):
             val = i.get_attribute('value')
             n = i.get_attribute('name')
             if val == value:
-                s = "input[name='%s'] ~ a[name=remove]" % n
+                s = "input[name='%s'] ~ .input-group-btn button[name=remove]" % n
                 link = self.find(s, By.CSS_SELECTOR, w, strict=True)
                 link.click()
                 self.wait()
@@ -702,7 +758,8 @@ class UI_driver(object):
 
     def check_option(self, name, value=None, parent=None):
         """
-        Find checkbox or radio with name which matches ^NAME\d$
+        Find checkbox or radio with name which matches ^NAME\d$ and
+        check it by clicking on a label.
         """
         if not parent:
             parent = self.get_form()
@@ -710,15 +767,16 @@ class UI_driver(object):
         if value is not None:
             s += "[@value='%s']" % value
         opts = self.find(s, "xpath", parent, many=True)
-        opt = None
+        label = None
         # Select only the one which matches exactly the name
         for o in opts:
             n = o.get_attribute("name")
-            if n == name or re.match("^%s\d$" % name, n):
-                opt = o
+            if n == name or re.match("^%s\d+$" % name, n):
+                s = "label[for='%s']" % o.get_attribute("id")
+                label = self.find(s, By.CSS_SELECTOR, parent, strict=True)
                 break
-        assert opt is not None, "Option not found: %s" % name
-        opt.click()
+        assert label is not None, "Option not found: %s" % name
+        label.click()
 
     def select_combobox(self, name, value, parent=None):
         """
@@ -728,12 +786,13 @@ class UI_driver(object):
             parent = self.get_form()
         s = "[name='%s'].combobox-widget" % name
         cb = self.find(s, By.CSS_SELECTOR, parent, strict=True)
-        open_btn = self.find('a[name=open] span', By.CSS_SELECTOR, cb, strict=True)
+        open_btn = self.find('a[name=open] i', By.CSS_SELECTOR, cb, strict=True)
         open_btn.click()
         self.wait()
         self.wait_for_request()
 
-        search_btn = self.find('a[name=search] span', By.CSS_SELECTOR, cb, strict=True)
+        list_cnt = self.find('.combobox-widget-list', By.CSS_SELECTOR, cb, strict=True)
+        search_btn = self.find('a[name=search] i', By.CSS_SELECTOR, cb, strict=True)
         opt_s = "select[name=list] option[value='%s']" % value
         option = self.find(opt_s, By.CSS_SELECTOR, cb)
         if not option:
@@ -747,7 +806,7 @@ class UI_driver(object):
         option.click()
 
         # Chrome does not close search area on click
-        if search_btn.is_displayed():
+        if list_cnt.is_displayed():
             self.driver.switch_to_active_element().send_keys(Keys.RETURN)
 
         self.wait()
@@ -766,13 +825,13 @@ class UI_driver(object):
         value = el.get_attribute('value')
         return value
 
-    def get_field_text(self, name, parent=None, element='label'):
+    def get_field_text(self, name, parent=None, element='p'):
 
-        s = "div[name='%s'] %s[name='%s']" % (name, element, name)
+        s = ".controls %s[name='%s']" % (element, name)
         return self.get_text(s, parent)
 
     def get_field_value(self, name, parent=None, element='input'):
-        s = "div[name='%s'] %s[name='%s']" % (name, element, name)
+        s = ".controls %s[name='%s']" % (element, name)
         return self.get_value(s, parent)
 
     def get_multivalued_value(self, name, parent=None):
@@ -787,7 +846,7 @@ class UI_driver(object):
     def get_field_checked(self, name, parent=None):
         if not parent:
             parent = self.get_form()
-        s = "div[name='%s'] input[name='%s']" % (name, name)
+        s = "div[name='%s'] input[name^='%s']" % (name, name)
         els = self.find(s, By.CSS_SELECTOR, parent, strict=True, many=True)
         values = []
         for el in els:
@@ -813,7 +872,7 @@ class UI_driver(object):
         """
         if not parent:
             parent = self.get_form()
-        s = "div[name='%s'].field span.undo" % (field)
+        s = ".controls div[name='%s'] .btn.undo" % (field)
         undos = self.find(s, By.CSS_SELECTOR, parent, strict=True, many=True)
         return undos
 
@@ -861,7 +920,7 @@ class UI_driver(object):
         s = "table"
         if name:
             s += "[name='%s']" % name
-        s += '.search-table'
+        s += '.table'
         return s
 
     def select_record(self, pkey, parent=None, table_name=None):
@@ -872,9 +931,18 @@ class UI_driver(object):
             parent = self.get_form()
 
         s = self.get_table_selector(table_name)
-        s += " tbody td input[value='%s']" % pkey
-        checkbox = self.find(s, By.CSS_SELECTOR, parent, strict=True)
-        checkbox.click()
+        input_s = s + " tbody td input[value='%s']" % pkey
+        checkbox = self.find(input_s, By.CSS_SELECTOR, parent, strict=True)
+        checkbox_id = checkbox.get_attribute('id')
+        label_s = s + " tbody td label[for='%s']" % checkbox_id
+        print(label_s)
+        label = self.find(label_s, By.CSS_SELECTOR, parent, strict=True)
+        try:
+            ActionChains(self.driver).move_to_element(label).click().perform()
+        except WebDriverException as e:
+            assert False, 'Can\'t click on checkbox label: %s \n%s' % (s, e)
+        self.wait()
+        assert checkbox.is_selected(), 'Record was not checked: %s' % input_s
         self.wait()
 
     def get_record_value(self, pkey, column, parent=None, table_name=None):
@@ -939,7 +1007,7 @@ class UI_driver(object):
             if table_name and parent:
                 s = self.get_table_selector(table_name)
                 table = self.find(s, By.CSS_SELECTOR, parent, strict=True)
-                self.action_button_click('remove', table)
+                self.button_click('remove', table)
             else:
                 self.facet_button_click('remove')
             if fields:
@@ -978,7 +1046,7 @@ class UI_driver(object):
             key = field[1]
             val = field[2]
 
-            if undo:
+            if undo and not hasattr(key, '__call__'):
                 self.assert_undo_button(key, False, parent)
 
             if widget_type == 'textbox':
@@ -990,7 +1058,9 @@ class UI_driver(object):
             elif widget_type == 'radio':
                 self.check_option(key, val, parent)
             elif widget_type == 'checkbox':
-                self.check_option(key, parent=parent)
+                self.check_option(key, val, parent=parent)
+            elif widget_type == 'selectbox':
+                self.select('select[name=%s]' % key, val, parent)
             elif widget_type == 'combobox':
                 self.select_combobox(key, val, parent)
             elif widget_type == 'add_table_record':
@@ -1001,8 +1071,13 @@ class UI_driver(object):
                 self.fill_multivalued(key, val, parent)
             elif widget_type == 'table':
                 self.select_record(val, parent, key)
+            # this meta field specifies a function, to extend functionality of
+            # field checking
+            elif widget_type == 'callback':
+                if hasattr(key, '__call__'):
+                    key(val)
             self.wait()
-            if undo:
+            if undo and not hasattr(key, '__call__'):
                 self.assert_undo_button(key, True, parent)
 
     def validate_fields(self, fields, parent=None):
@@ -1144,7 +1219,7 @@ class UI_driver(object):
             new_count = len(self.get_rows())
             self.assert_row_count(count, new_count)
 
-    def mod_record(self, entity, data, facet='details', facet_btn='update'):
+    def mod_record(self, entity, data, facet='details', facet_btn='save'):
         """
         Mod record
 
@@ -1169,7 +1244,7 @@ class UI_driver(object):
                    add_facet_btn='add',
                    add_dialog_btn='add',
                    add_dialog_name='add',
-                   update_btn='update',
+                   update_btn='save',
                    breadcrumb=None,
                    navigate=True,
                    delete=True):
@@ -1244,7 +1319,7 @@ class UI_driver(object):
             parent = self.get_form()
         s = self.get_table_selector(name)
         table = self.find(s, By.CSS_SELECTOR, parent, strict=True)
-        s = "a[name=%s] span.button-label" % 'add'
+        s = ".btn[name=%s]" % 'add'
         btn = self.find(s, By.CSS_SELECTOR, table, strict=True)
         btn.click()
         self.wait()
@@ -1260,6 +1335,7 @@ class UI_driver(object):
             self.switch_to_facet(facet)
 
         self.facet_button_click('add')
+        self.wait()
         self.wait_for_request()
 
         for key in pkeys:
@@ -1285,7 +1361,7 @@ class UI_driver(object):
         s = self.get_table_selector(table_name)
         table = self.find(s, By.CSS_SELECTOR, parent, strict=True)
 
-        s = "a[name=%s] span.button-label" % 'add'
+        s = "button[name='%s']" % 'add'
         btn = self.find(s, By.CSS_SELECTOR, table, strict=True)
         btn.click()
         self.wait_for_request(0.4)
@@ -1293,6 +1369,7 @@ class UI_driver(object):
         for key in pkeys:
             self.select_record(key, table_name='available')
             self.button_click('add')
+            self.wait()
 
         self.dialog_button_click('add')
         self.wait_for_request(n=2)
@@ -1304,14 +1381,18 @@ class UI_driver(object):
             for key in pkeys:
                 self.assert_record(key, parent, table_name, negative=True)
 
-    def action_list_action(self, name):
+    def action_list_action(self, name, confirm=True, confirm_btn="ok"):
         """
         Execute action list action
         """
-        cont = self.find(".active-facet .facet-action-list", By.CSS_SELECTOR, strict=True)
-        select = self.find("select[name=action]", By.CSS_SELECTOR, cont, strict=True)
-        Select(select).select_by_value(name)
-        self.button_click('apply', cont)
+        cont = self.find(".active-facet .facet-actions", By.CSS_SELECTOR, strict=True)
+        expand = self.find(".dropdown-toggle", By.CSS_SELECTOR, cont, strict=True)
+        expand.click()
+        action_link = self.find("li[data-name=%s] a" % name, By.CSS_SELECTOR, cont, strict=True)
+        action_link.click()
+        if confirm:
+            self.wait(0.5)  # wait for dialog
+            self.dialog_button_click(confirm_btn)
         self.wait()
 
     def action_panel_action(self, panel_name, action):
@@ -1381,7 +1462,7 @@ class UI_driver(object):
 
         # update
         self.assert_rule_tables_enabled(t_list, False)
-        self.facet_button_click('update')
+        self.facet_button_click('save')
         self.wait_for_request(n=3, d=0.3)
         self.assert_rule_tables_enabled(t_list, False)
 
@@ -1400,7 +1481,7 @@ class UI_driver(object):
         for cat in categories:
             self.check_option(cat, '')
         self.assert_rule_tables_enabled(t_list, True)
-        self.facet_button_click('update')
+        self.facet_button_click('save')
         self.wait_for_request(n=3, d=0.3)
         self.assert_rule_tables_enabled(t_list, True)
 
@@ -1409,6 +1490,12 @@ class UI_driver(object):
             # add multiple at once and test table delete button
             self.add_table_associations(table, keys, delete=True)
 
+    def has_class(self, el, cls):
+        """
+        Check if el has CSS class
+        """
+        return cls in el.get_attribute("class").split()
+
     def skip(self, reason):
         """
         Skip tests
@@ -1463,19 +1550,18 @@ class UI_driver(object):
 
     def assert_button_enabled(self, name, context_selector=None, enabled=True):
         """
-        Assert that button is enabled or disabled
+        Assert that button is enabled or disabled (expects that element will be
+        <button>)
         """
         s = ""
         if context_selector:
             s = context_selector
-        s += "a[name=%s]" % name
+        s += "button[name=%s]" % name
         facet = self.get_facet()
         btn = self.find(s, By.CSS_SELECTOR, facet, strict=True)
-        cls = 'action-button-disabled'
-        has_cls = cls in btn.get_attribute("class").split()
-        valid = enabled ^ has_cls
+        valid = enabled == btn.is_enabled()
         assert btn.is_displayed(), 'Button is not displayed'
-        assert valid, 'Button has incorrect enabled state.'
+        assert valid, 'Button (%s) has incorrect enabled state (enabled==%s).' % (s, enabled)
 
     def assert_facet_button_enabled(self, name, enabled=True):
         """
@@ -1527,17 +1613,31 @@ class UI_driver(object):
         else:
             assert visible, "Element not visible: %s" % selector
 
+    def assert_disabled(self, selector, parent=None, negative=False):
+        """
+        Assert that element defined by selector is disabled
+        """
+        if not parent:
+            parent = self.get_form()
+        el = self.find(selector, By.CSS_SELECTOR, parent, strict=True)
+        dis = self.find(selector+"[disabled]", By.CSS_SELECTOR, parent)
+        if negative:
+            assert dis is None, "Element is disabled: %s" % selector
+        else:
+            assert dis, "Element is not disabled: %s" % selector
+
     def assert_record(self, pkey, parent=None, table_name=None, negative=False):
         """
         Assert that record is in current search table
         """
         has = self.has_record(pkey, parent, table_name)
+        has |= self.has_record(pkey.lower(), parent, table_name)
         if negative:
             assert not has, "Record exists when it shouldn't: %s" % pkey
         else:
             assert has, 'Record does not exist: %s' % pkey
 
-    def assert_indirect_record(self, pkey, entity, facet, negative=False, switch=True, lower=True):
+    def assert_indirect_record(self, pkey, entity, facet, negative=False, switch=True):
         """
         Switch to indirect facet and assert record.
 
@@ -1549,8 +1649,6 @@ class UI_driver(object):
             self.check_option(radio_name, 'indirect')
             self.wait_for_request(n=2)
         key = pkey
-        if lower:
-            key = key.lower()
         self.assert_record(key, negative=negative)
 
     def assert_record_value(self, expected, pkey, column, parent=None, table_name=None):
@@ -1564,7 +1662,7 @@ class UI_driver(object):
         """
         Assert that element has certain class
         """
-        valid = cls in element.get_attribute('class').split()
+        valid = self.has_class(element, cls)
         if negative:
             assert not valid, "Element contains unwanted class: %s" % cls
         else:
@@ -1600,7 +1698,7 @@ class UI_driver(object):
         is_visible = link is not None and link.is_displayed()
         is_enabled = False
         if is_visible:
-            is_enabled = 'disabled' not in link.get_attribute("class").split()
+            is_enabled = not self.has_class(link, 'disabled')
 
         assert is_visible == visible, ('Invalid visibility of action button: %s. '
                                        'Expected: %s') % (action, str(visible))
@@ -1608,3 +1706,27 @@ class UI_driver(object):
         if is_visible:
             assert is_enabled == enabled, ('Invalid enabled state of action button %s. '
                                            'Expected: %s') % (action, str(visible))
+
+    def assert_action_list_action(self, action, visible=True, enabled=True, parent=None):
+        """
+        Assert that action dropdown action is visible/hidden, and enabled/disabled
+
+        Enabled is checked only if action is visible.
+        """
+        if not parent:
+            parent = self.get_form()
+
+        s = ".facet-actions li[data-name='%s']" % action
+        li = self.find(s, By.CSS_SELECTOR, parent)
+        link = self.find("a", By.CSS_SELECTOR, li)
+
+        is_visible = li is not None and link is not None
+        is_enabled = False
+
+        assert is_visible == visible, ('Invalid visibility of action item: %s. '
+                                       'Expected: %s') % (action, str(visible))
+
+        if is_visible:
+            is_enabled = not self.has_class(li, 'disabled')
+            assert is_enabled == enabled, ('Invalid enabled state of action item %s. '
+                                           'Expected: %s') % (action, str(visible))
