@@ -201,3 +201,24 @@ idoverridegroup = [
     u'top',
     u'ipaGroupOverride',
 ]
+
+servicedelegationrule = [
+    u'top',
+    u'groupofprincipals',
+    u'ipakrb5delegationacl',
+]
+
+servicedelegationtarget = [
+    u'top',
+    u'groupofprincipals',
+]
+
+certprofile = [
+    u'top',
+    u'ipacertprofile',
+]
+
+caacl = [
+    u'ipaassociation',
+    u'ipacaacl'
+]
