@@ -24,10 +24,12 @@ Test the `ipalib/plugins/automember.py` module.
 from ipalib import api, errors
 from ipapython.dn import DN
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid, \
-    fuzzy_automember_dn, fuzzy_automember_message
+from ipatests.test_xmlrpc.xmlrpc_test import (
+    Declarative, fuzzy_digits, fuzzy_uuid, fuzzy_automember_dn,
+    fuzzy_automember_message)
 from ipatests.test_xmlrpc.test_user_plugin import get_user_result
 
+import pytest
 
 user1 = u'tuser1'
 user_does_not_exist = u'does_not_exist'
@@ -65,6 +67,7 @@ hostgroup_exclude_regex2 = u'^www5'
 hostgroup_exclude_regex3 = u'^webserver5'
 
 
+@pytest.mark.tier1
 class test_automember(Declarative):
 
     cleanup_commands = [
