@@ -25,15 +25,21 @@ import sys
 import textwrap
 import tempfile
 import shutil
+import pytest
 
 from ipalib import api
 from ipalib import errors
 from ipapython.dn import DN
+import six
 
 from nose.tools import raises, assert_raises  # pylint: disable=E0611
-from xmlrpc_test import XMLRPC_test, assert_attr_equal
+from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test, assert_attr_equal
+from ipaplatform.paths import paths
 from ipatests.util import assert_deepequal
 
+if six.PY3:
+    unicode = str
+
 
 class MockTextui(list):
     """Collects output lines"""
@@ -44,6 +50,10 @@ class MockTextui(list):
 
 class AutomountTest(XMLRPC_test):
     """Provides common functionality for automount tests"""
+
+    locname = u'testlocation'
+    tofiles_output = ''  # To be overridden
+
     def check_tofiles(self):
         """Check automountlocation_tofiles output against self.tofiles_output
         """
@@ -51,7 +61,7 @@ class AutomountTest(XMLRPC_test):
 
         mock_ui = MockTextui()
         command = api.Command['automountlocation_tofiles']
-        command.output_for_cli(mock_ui, res, self.locname)
+        command.output_for_cli(mock_ui, res, self.locname, version=u'2.88')
         expected_output = self.tofiles_output
         assert_deepequal(expected_output, u'\n'.join(mock_ui))
 
@@ -79,6 +89,8 @@ class AutomountTest(XMLRPC_test):
                 break
             else:
                 current_file.write(line + '\n')
+        assert current_file is not None, ('The input file does not contain any'
+                                          'records of files to be opened.')
         current_file.close()
 
         self.failsafe_add(api.Object.automountlocation, self.locname)
@@ -87,7 +99,8 @@ class AutomountTest(XMLRPC_test):
             # Feed the files to automountlocation_import & check
             master_file = u'%s/auto.master' % conf_directory
             automountlocation_import = api.Command['automountlocation_import']
-            res = automountlocation_import(self.locname, master_file)
+            res = automountlocation_import(self.locname, master_file,
+                                           version=u'2.88')
             assert_deepequal(dict(
                 result=dict(
                     keys=lambda k: k,
@@ -100,16 +113,17 @@ class AutomountTest(XMLRPC_test):
         finally:
             res = api.Command['automountlocation_del'](self.locname)['result']
             assert res
-            assert_attr_equal(res, 'failed', '')
+            assert not res['failed']
 
         # Success; delete the temporary directory
         shutil.rmtree(conf_directory)
 
+
+@pytest.mark.tier1
 class test_automount(AutomountTest):
     """
     Test the `automount` plugin.
     """
-    locname = u'testlocation'
     mapname = u'testmap'
     keyname = u'testkey'
     keyname_rename = u'testkey_rename'
@@ -220,7 +234,7 @@ class test_automount(AutomountTest):
         assert_attr_equal(res, 'automountinformation', self.newinfo)
         assert_attr_equal(res, 'automountkey', self.keyname_rename)
 
-    def test_a_automountmap_mod(self):
+    def test_a1_automountmap_mod(self):
         """
         Test the `xmlrpc.automountmap_mod` method.
         """
@@ -233,7 +247,8 @@ class test_automount(AutomountTest):
         """
         Test the `automountlocation_tofiles` command.
         """
-        res = api.Command['automountlocation_tofiles'](self.locname)
+        res = api.Command['automountlocation_tofiles'](self.locname,
+                                                       version=u'2.88')
         assert_deepequal(dict(
             result=dict(
                 keys={'auto.direct': ()},
@@ -241,7 +256,7 @@ class test_automount(AutomountTest):
                     dn=DN(('automountmapname', self.mapname),
                           ('cn', self.locname),
                           ('cn', 'automount'), api.env.basedn),
-                    description=(u'description of map',),
+                    description=(u'new description',),
                     automountmapname=(u'testmap',)),),
                 orphankeys=[(
                     dict(
@@ -285,7 +300,7 @@ class test_automount(AutomountTest):
         delkey_kw={'automountkey': self.keyname_rename, 'automountinformation' : self.newinfo}
         res = api.Command['automountkey_del'](self.locname, self.mapname, **delkey_kw)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verify that it is gone
         with assert_raises(errors.NotFound):
@@ -297,7 +312,7 @@ class test_automount(AutomountTest):
         """
         res = api.Command['automountlocation_del'](self.locname)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verify that it is gone
         with assert_raises(errors.NotFound):
@@ -313,11 +328,11 @@ class test_automount(AutomountTest):
             api.Command['automountkey_show'](self.locname, self.mapname, **key_kw)
 
 
+@pytest.mark.tier1
 class test_automount_direct(AutomountTest):
     """
     Test the `automount` plugin indirect map functionality.
     """
-    locname = u'testlocation'
     mapname = u'auto.direct2'
     keyname = u'/-'
     direct_kw = { 'key' : keyname }
@@ -367,7 +382,7 @@ class test_automount_direct(AutomountTest):
         """
         res = api.Command['automountlocation_del'](self.locname)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verity that it is gone
         with assert_raises(errors.NotFound):
@@ -379,11 +394,11 @@ class test_automount_direct(AutomountTest):
         self.check_import_roundtrip()
 
 
+@pytest.mark.tier1
 class test_automount_indirect(AutomountTest):
     """
     Test the `automount` plugin indirect map functionality.
     """
-    locname = u'testlocation'
     mapname = u'auto.home'
     keyname = u'/home'
     parentmap = u'auto.master'
@@ -443,7 +458,7 @@ class test_automount_indirect(AutomountTest):
         """
         res = api.Command['automountkey_del'](self.locname, self.parentmap, **self.key_kw)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verify that it is gone
         with assert_raises(errors.NotFound):
@@ -455,7 +470,7 @@ class test_automount_indirect(AutomountTest):
         """
         res = api.Command['automountmap_del'](self.locname, self.mapname)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verify that it is gone
         with assert_raises(errors.NotFound):
@@ -467,7 +482,7 @@ class test_automount_indirect(AutomountTest):
         """
         res = api.Command['automountlocation_del'](self.locname)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verity that it is gone
         with assert_raises(errors.NotFound):
@@ -478,11 +493,12 @@ class test_automount_indirect(AutomountTest):
         """
         self.check_import_roundtrip()
 
+
+@pytest.mark.tier1
 class test_automount_indirect_no_parent(AutomountTest):
     """
     Test the `automount` plugin Indirect map function.
     """
-    locname = u'testlocation'
     mapname = u'auto.home'
     keyname = u'/home'
     mapname2 = u'auto.direct2'
@@ -551,7 +567,7 @@ class test_automount_indirect_no_parent(AutomountTest):
         delkey_kw={'automountkey': self.keyname, 'automountinformation': self.mapname}
         res = api.Command['automountkey_del'](self.locname, self.parentmap, **delkey_kw)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verify that it is gone
         with assert_raises(errors.NotFound):
@@ -563,7 +579,7 @@ class test_automount_indirect_no_parent(AutomountTest):
         """
         res = api.Command['automountmap_del'](self.locname, self.mapname)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verify that it is gone
         with assert_raises(errors.NotFound):
@@ -575,7 +591,7 @@ class test_automount_indirect_no_parent(AutomountTest):
         """
         res = api.Command['automountlocation_del'](self.locname)['result']
         assert res
-        assert_attr_equal(res, 'failed', '')
+        assert not res['failed']
 
         # Verity that it is gone
         with assert_raises(errors.NotFound):
