@@ -24,10 +24,13 @@ Test the `ipalib/plugins/batch.py` module.
 from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.util import assert_equal, Fuzzy, assert_deepequal
-from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
+                                              fuzzy_uuid)
 from ipapython.dn import DN
+import pytest
 
 group1 = u'testgroup1'
+first1 = u'John'
 
 
 def deepequal_list(*expected):
@@ -46,6 +49,7 @@ def deepequal_list(*expected):
     return checker
 
 
+@pytest.mark.tier1
 class test_batch(Declarative):
 
     cleanup_commands = [
@@ -182,7 +186,7 @@ class test_batch(Declarative):
                 # missing required argument
                 dict(method='user_add', params=([], dict())),
                 # missing required option
-                dict(method='group_add', params=([group1], dict())),
+                dict(method='user_add', params=([], dict(givenname=first1))),
                 # bad type
                 dict(method='group_add', params=([group1], dict(
                         description=u't', gidnumber=u'bad'))),
@@ -216,7 +220,7 @@ class test_batch(Declarative):
                         error_code=3007,
                     ),
                     dict(
-                        error=u"'description' is required",
+                        error=u"'sn' is required",
                         error_name=u'RequirementError',
                         error_code=3007,
                     ),
