@@ -22,11 +22,18 @@ Test integration of DNS and realmdomains.
 2. realmdomains_mod should add a _kerberos TXT record in the DNS zone.
 """
 
+import six
+
 from ipalib import api, errors
+from ipalib.util import normalize_zone
 from ipapython.dn import DN
 from ipapython.dnsutil import DNSName
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import Declarative, fuzzy_digits
+from ipatests.test_xmlrpc.xmlrpc_test import Declarative, fuzzy_digits
+import pytest
+
+if six.PY3:
+    unicode = str
 
 
 cn = u'Realm Domains'
@@ -43,6 +50,8 @@ dnszone_2_absolute = "%s." % dnszone_2
 dnszone_2_dn = DN(('idnsname', dnszone_2_absolute), api.env.container_dns,
                   api.env.basedn)
 
+self_server_ns = normalize_zone(api.env.host)
+self_server_ns_dnsname = DNSName(self_server_ns)
 
 def assert_realmdomain_and_txt_record_present(response):
     zone = response['value']
@@ -80,6 +89,7 @@ def assert_realmdomain_and_txt_record_not_present(response):
         return True
 
 
+@pytest.mark.tier1
 class test_dns_realmdomains_integration(Declarative):
     cleanup_commands = [
         ('realmdomains_mod', [], {'associateddomain': [our_domain]}),
@@ -92,9 +102,7 @@ class test_dns_realmdomains_integration(Declarative):
                  'during dnszone_add',
             command=(
                 'dnszone_add', [dnszone_1], {
-                    'idnssoamname': idnssoamname,
                     'idnssoarname': idnssoarname,
-                    'ip_address': u'1.2.3.4',
                 }
             ),
             expected={
@@ -104,8 +112,8 @@ class test_dns_realmdomains_integration(Declarative):
                     'dn': dnszone_1_dn,
                     'idnsname': [DNSName(dnszone_1_absolute)],
                     'idnszoneactive': [u'TRUE'],
-                    'idnssoamname': [DNSName(idnssoamname)],
-                    'nsrecord': [idnssoamname],
+                    'idnssoamname': [self_server_ns_dnsname],
+                    'nsrecord': lambda x: True,
                     'idnssoarname': [DNSName(idnssoarname)],
                     'idnssoaserial': [fuzzy_digits],
                     'idnssoarefresh': [fuzzy_digits],
@@ -131,11 +139,9 @@ class test_dns_realmdomains_integration(Declarative):
                  'during dnszone_add for forwarded zone',
             command=(
                 'dnszone_add', [dnszone_2], {
-                    'idnssoamname': idnssoamname,
                     'idnssoarname': idnssoarname,
                     'idnsforwarders': u'1.2.3.4',
                     'idnsforwardpolicy': u'only',
-                    'force': True,
                 }
             ),
             expected={
@@ -154,10 +160,10 @@ class test_dns_realmdomains_integration(Declarative):
                     'dn': dnszone_2_dn,
                     'idnsname': [DNSName(dnszone_2_absolute)],
                     'idnszoneactive': [u'TRUE'],
-                    'idnssoamname': [DNSName(idnssoamname)],
+                    'idnssoamname': [self_server_ns_dnsname],
                     'idnsforwarders': [u'1.2.3.4'],
                     'idnsforwardpolicy': [u'only'],
-                    'nsrecord': [idnssoamname],
+                    'nsrecord': lambda x: True,
                     'idnssoarname': [DNSName(idnssoarname)],
                     'idnssoaserial': [fuzzy_digits],
                     'idnssoarefresh': [fuzzy_digits],
