@@ -25,7 +25,9 @@ import nose
 from ipalib import api
 from ipapython.dn import DN
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import Declarative, fuzzy_uuid, fuzzy_user_or_group_sid
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_uuid,
+                                              fuzzy_user_or_group_sid)
+import pytest
 
 group_name = u'external_group'
 group_desc = u'Test external group'
@@ -41,10 +43,11 @@ def get_trusted_group_name():
     return u'%s\Domain Admins' % ad_netbios
 
 
+@pytest.mark.tier1
 class test_external_members(Declarative):
     @classmethod
-    def setUpClass(cls):
-        super(test_external_members, cls).setUpClass()
+    def setup_class(cls):
+        super(test_external_members, cls).setup_class()
         if not api.Backend.rpcclient.isconnected():
             api.Backend.rpcclient.connect(fallback=False)
 
