@@ -21,13 +21,20 @@
 Test the `ipalib/plugins/group.py` module.
 """
 
+import functools
+import pytest
+
 from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import (Declarative, fuzzy_digits, fuzzy_uuid, fuzzy_set_ci,
-                         add_sid, add_oc)
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits, fuzzy_uuid, fuzzy_set_ci,
+                         add_sid, add_oc, XMLRPC_test, raises_exact)
 from ipapython.dn import DN
 from ipatests.test_xmlrpc.test_user_plugin import get_user_result
 
+from ipatests.test_xmlrpc.tracker.user_plugin import UserTracker
+from ipatests.util import assert_deepequal
+
+
 group1 = u'testgroup1'
 group2 = u'testgroup2'
 group3 = u'testgroup3'
@@ -45,6 +52,8 @@ external_sid1=u'S-1-1-123456-789-1'
 def get_group_dn(cn):
     return DN(('cn', cn), api.env.container_group, api.env.basedn)
 
+
+@pytest.mark.tier1
 class test_group(Declarative):
     cleanup_commands = [
         ('group_del', [group1], {}),
@@ -1011,6 +1020,8 @@ class test_group(Declarative):
         ),
     ]
 
+
+@pytest.mark.tier1
 class test_group_remove_group_from_protected_group(Declarative):
     cleanup_commands = [
         ('group_del', [group1], {}),
@@ -1078,3 +1089,74 @@ class test_group_remove_group_from_protected_group(Declarative):
             ),
         ),
     ]
+
+
+@pytest.mark.tier1
+class test_group_full_set_of_objectclass_not_available_post_detach(Declarative):
+    # https://fedorahosted.org/freeipa/ticket/4909#comment:1
+    cleanup_commands = [
+        ('group_del', [user1], {}),
+        ('user_del', [user1], {}),
+    ]
+
+    tests = [
+        dict(
+            desc='Create %r' % user1,
+            command=(
+                'user_add', [], dict(givenname=u'Test', sn=u'User1')
+            ),
+            expected=dict(
+                value=user1,
+                summary=u'Added user "%s"' % user1,
+                result=get_user_result(user1, u'Test', u'User1', 'add'),
+            ),
+        ),
+
+        dict(
+            desc='Detach managed group %r' % user1,
+            command=('group_detach', [user1], {}),
+            expected=dict(
+                result=True,
+                value=user1,
+                summary=u'Detached group "%s" from user "%s"' % (user1, user1),
+            ),
+        ),
+
+        dict(
+            desc='Show group - check objectclass',
+            command=('group_show', [user1], dict(all=True)),
+            expected=dict(
+                value=user1,
+                result={
+                    'cn':[user1],
+                    'description': [u'User private group for tuser1'],
+                    'gidnumber': [fuzzy_digits],
+                    'dn': get_group_dn('tuser1'),
+                    'ipauniqueid': [fuzzy_uuid],
+                    'objectclass': objectclasses.posixgroup,
+                },
+                summary=None,
+            ),
+        ),
+
+        dict(
+            desc='Add member back to the detached group',
+            command=('group_add_member', [user1], dict(user=user1)),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    member=dict(
+                        group=tuple(),
+                        user=tuple(),
+                    ),
+                ),
+                result={
+                        'dn': get_group_dn('tuser1'),
+                        'member_user': [user1],
+                        'gidnumber': [fuzzy_digits],
+                        'cn': [user1],
+                        'description': [u'User private group for tuser1'],
+                },
+            ),
+        ),
+    ]
