@@ -22,21 +22,20 @@ Test the `ipalib/plugins/netgroup.py` module.
 """
 
 import nose
-import krbV
+
 from ipalib import api
 from ipalib import errors
 from ipaserver.plugins.ldap2 import ldap2
-from xmlrpc_test import (Declarative, fuzzy_digits, fuzzy_uuid,
-                         fuzzy_netgroupdn, add_sid, add_oc)
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
+                                              fuzzy_uuid, fuzzy_netgroupdn)
 from ipatests.test_xmlrpc import objectclasses
 from ipapython.dn import DN
+from ipatests.test_xmlrpc.test_user_plugin import get_user_result
+import pytest
 
 # Global so we can save the value between tests
 netgroup_dn = None
 
-# See if our LDAP server is up and we can talk to it over GSSAPI
-ccache = krbV.default_context().default_ccache().name
-
 netgroup1 = u'netgroup1'
 netgroup2 = u'netgroup2'
 netgroup_single = u'a'
@@ -65,6 +64,8 @@ invalidnisdomain1=u'domain1,domain2'
 invalidnisdomain2=u'+invalidnisdomain'
 invalidhost=u'+invalid&host'
 
+
+@pytest.mark.tier1
 class test_netgroup(Declarative):
     """
     Test the `netgroup` plugin.
@@ -198,9 +199,9 @@ class test_netgroup(Declarative):
             desc='Delete %r' % netgroup_single,
             command=('netgroup_del', [netgroup_single], {}),
             expected=dict(
-                value=netgroup_single,
+                value=[netgroup_single],
                 summary=u'Deleted netgroup "%s"' % netgroup_single,
-                result=dict(failed=u''),
+                result=dict(failed=[]),
             ),
         ),
 
@@ -272,30 +273,7 @@ class test_netgroup(Declarative):
             expected=dict(
                 value=user1,
                 summary=u'Added user "%s"' % user1,
-                result=add_sid(dict(
-                    gecos=[u'Test User1'],
-                    givenname=[u'Test'],
-                    homedirectory=[u'/home/%s' % user1],
-                    krbprincipalname=[u'%s@%s' % (user1, api.env.realm)],
-                    loginshell=[u'/bin/sh'],
-                    objectclass=add_oc(objectclasses.user, u'ipantuserattrs'),
-                    sn=[u'User1'],
-                    uid=[user1],
-                    uidnumber=[fuzzy_digits],
-                    gidnumber=[fuzzy_digits],
-                    mail=[u'%s@%s' % (user1, api.env.domain)],
-                    displayname=[u'Test User1'],
-                    cn=[u'Test User1'],
-                    initials=[u'TU'],
-                    ipauniqueid=[fuzzy_uuid],
-                    mepmanagedentry=[DN(('cn',user1),('cn','groups'),('cn','accounts'),
-                                        api.env.basedn)],
-                    memberof_group=[u'ipausers'],
-                    has_keytab=False,
-                    has_password=False,
-                    dn=DN(('uid',user1),('cn','users'),('cn','accounts'),
-                          api.env.basedn),
-                )),
+                result=get_user_result(user1, u'Test', u'User1', 'add'),
             ),
         ),
 
@@ -307,30 +285,7 @@ class test_netgroup(Declarative):
             expected=dict(
                 value=user2,
                 summary=u'Added user "%s"' % user2,
-                result=add_sid(dict(
-                    gecos=[u'Test User2'],
-                    givenname=[u'Test'],
-                    homedirectory=[u'/home/%s' % user2],
-                    krbprincipalname=[u'%s@%s' % (user2, api.env.realm)],
-                    loginshell=[u'/bin/sh'],
-                    objectclass=add_oc(objectclasses.user, u'ipantuserattrs'),
-                    sn=[u'User2'],
-                    uid=[user2],
-                    uidnumber=[fuzzy_digits],
-                    gidnumber=[fuzzy_digits],
-                    mail=[u'%s@%s' % (user2, api.env.domain)],
-                    displayname=[u'Test User2'],
-                    cn=[u'Test User2'],
-                    initials=[u'TU'],
-                    ipauniqueid=[fuzzy_uuid],
-                    mepmanagedentry=[DN(('cn',user2),('cn','groups'),('cn','accounts'),
-                                        api.env.basedn)],
-                    memberof_group=[u'ipausers'],
-                    has_keytab=False,
-                    has_password=False,
-                    dn=DN(('uid',user2),('cn','users'),('cn','accounts'),
-                          api.env.basedn),
-                )),
+                result=get_user_result(user2, u'Test', u'User2', 'add'),
             ),
         ),
 
@@ -385,8 +340,8 @@ class test_netgroup(Declarative):
             desc='Add invalid host %r to netgroup %r' % (invalidhost, netgroup1),
             command=('netgroup_add_member', [netgroup1], dict(host=invalidhost)),
             expected=errors.ValidationError(name='host',
-             error='only letters, numbers, _, and - are allowed. ' +
-                    u'DNS label may not start or end with -'),
+             error=u"only letters, numbers, '_', '-' are allowed. " +
+                    u"DNS label may not start or end with '-'"),
         ),
 
 
@@ -826,8 +781,8 @@ class test_netgroup(Declarative):
                 dict(setattr='externalhost=%s' % invalidhost)
             ),
             expected=errors.ValidationError(name='externalhost',
-                error='only letters, numbers, _, and - are allowed. ' +
-                    'DNS label may not start or end with -'),
+                error=u"only letters, numbers, '_', '-' are allowed. " +
+                    u"DNS label may not start or end with '-'"),
         ),
 
         dict(
@@ -1325,9 +1280,9 @@ class test_netgroup(Declarative):
             desc='Delete %r' % netgroup1,
             command=('netgroup_del', [netgroup1], {}),
             expected=dict(
-                value=netgroup1,
+                value=[netgroup1],
                 summary=u'Deleted netgroup "%s"' % netgroup1,
-                result=dict(failed=u''),
+                result=dict(failed=[]),
             ),
         ),
 
@@ -1341,8 +1296,8 @@ class test_netgroup(Declarative):
 #        """
 #        # Do an LDAP query to the compat area and verify that the entry
 #        # is correct
-#        conn = ldap2(shared_instance=False, ldap_uri=api.env.ldap_uri, base_dn=api.env.basedn)
-#        conn.connect(ccache=ccache)
+#        conn = ldap2(api)
+#        conn.connect()
 #        try:
 #            entries = conn.find_entries('cn=%s' % self.ng_cn,
 #                      base_dn='cn=ng,cn=compat,%s' % api.env.basedn)
@@ -1350,7 +1305,7 @@ class test_netgroup(Declarative):
 #            raise nose.SkipTest('compat and nis are not enabled, skipping test')
 #        finally:
 #            conn.disconnect()
-#        triples = entries[0][0][1]['nisnetgrouptriple']
+#        triples = entries[0][0]['nisnetgrouptriple']
 #
 #        # This may not prove to be reliable since order is not guaranteed
 #        # and even which user gets into which triple can be random.
