@@ -27,8 +27,10 @@ http://www.freeipa.org/page/V3/Permissions_V2
 
 from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
+                                              fuzzy_uuid)
 from ipapython.dn import DN
+import pytest
 
 permission1 = u'testperm'
 permission1_dn = DN(('cn',permission1),
@@ -76,6 +78,8 @@ permission3_attributelevelrights = {
                                      'ipapermright': u'rscwo',
                                      'ipapermtarget': u'rscwo',
                                      'ipapermtargetfilter': u'rscwo',
+                                     'ipapermtargetto': u'rscwo',
+                                     'ipapermtargetfrom': u'rscwo',
                                    }
 
 privilege1 = u'testpriv1'
@@ -88,6 +92,8 @@ users_dn = DN(api.env.container_user, api.env.basedn)
 groups_dn = DN(api.env.container_group, api.env.basedn)
 hbac_dn = DN(api.env.container_hbac, api.env.basedn)
 
+
+@pytest.mark.tier1
 class test_old_permission(Declarative):
     default_version = u'2.65'
 
@@ -519,6 +525,13 @@ class test_old_permission(Declarative):
                         'subtree': u'ldap:///%s' % users_dn,
                     },
                 ],
+                messages=({
+                    'message': (u'Search result has been truncated to '
+                                'configured search limit.'),
+                    'code': 13017,
+                    'type': u'warning',
+                    'name': u'SearchResultTruncated'
+                },),
             ),
         ),
 
@@ -572,6 +585,13 @@ class test_old_permission(Declarative):
                     DN(res['dn']).endswith(DN(api.env.container_permission,
                                               api.env.basedn)) and
                     'ipapermission' in res['objectclass']],
+                messages=({
+                    'message': (u'Search result has been truncated to '
+                                'configured search limit.'),
+                    'code': 13017,
+                    'type': u'warning',
+                    'name': u'SearchResultTruncated'
+                },),
             ),
         ),
 
@@ -1028,7 +1048,7 @@ class test_old_permission(Declarative):
                 'permission_add', [permission3], dict(
                      type=u'user',
                      permissions=u'write',
-		     attrs=[u'cn']
+                     attrs=[u'cn']
                 )
             ),
             expected=dict(
@@ -1055,7 +1075,7 @@ class test_old_permission(Declarative):
                 value=permission3,
                 summary=None,
                 result=dict(
-		    dn=permission3_dn,
+                    dn=permission3_dn,
                     cn=[permission3],
                     objectclass=objectclasses.permission,
                     type=u'user',
@@ -1078,7 +1098,7 @@ class test_old_permission(Declarative):
                 value=permission3,
                 summary=u'Modified permission "%s"' % permission3,
                 result=dict(
-		    dn=permission3_dn,
+                    dn=permission3_dn,
                     cn=[permission3],
                     objectclass=objectclasses.permission,
                     type=u'user',
