@@ -24,8 +24,10 @@ Test the `ipalib/plugins/role.py` module.
 
 from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
+                                              fuzzy_uuid)
 from ipapython.dn import DN
+import pytest
 
 search = u'test-role'
 
@@ -47,6 +49,8 @@ privilege1 = u'r,w privilege 1'
 privilege1_dn = DN(('cn', privilege1), DN(api.env.container_privilege),
                    api.env.basedn)
 
+
+@pytest.mark.tier1
 class test_role(Declarative):
 
     cleanup_commands = [
