@@ -22,9 +22,12 @@ Test the `ipalib/plugins/selinuxusermap.py` module.
 
 from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid, add_sid, add_oc
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
+                                              fuzzy_uuid)
 from ipapython.dn import DN
 from ipatests.util import Fuzzy
+from ipatests.test_xmlrpc.test_user_plugin import get_user_result
+import pytest
 
 rule1 = u'selinuxrule1'
 selinuxuser1 = u'guest_u:s0'
@@ -53,6 +56,7 @@ fuzzy_hbacruledn = Fuzzy(
 allow_all_rule_dn = api.Command['hbacrule_show'](u'allow_all')['result']['dn']
 
 
+@pytest.mark.tier1
 class test_selinuxusermap(Declarative):
     cleanup_commands = [
         ('selinuxusermap_del', [rule1], {}),
@@ -200,30 +204,7 @@ class test_selinuxusermap(Declarative):
             expected=dict(
                 value=user1,
                 summary=u'Added user "%s"' % user1,
-                result=add_sid(dict(
-                    gecos=[u'Test User1'],
-                    givenname=[u'Test'],
-                    homedirectory=[u'/home/%s' % user1],
-                    krbprincipalname=[u'%s@%s' % (user1, api.env.realm)],
-                    loginshell=[u'/bin/sh'],
-                    objectclass=add_oc(objectclasses.user, u'ipantuserattrs'),
-                    sn=[u'User1'],
-                    uid=[user1],
-                    uidnumber=[fuzzy_digits],
-                    gidnumber=[fuzzy_digits],
-                    mail=[u'%s@%s' % (user1, api.env.domain)],
-                    displayname=[u'Test User1'],
-                    cn=[u'Test User1'],
-                    initials=[u'TU'],
-                    ipauniqueid=[fuzzy_uuid],
-                    mepmanagedentry=[DN(('cn', user1), ('cn', 'groups'),
-                        ('cn', 'accounts'), api.env.basedn)],
-                    memberof_group=[u'ipausers'],
-                    dn=DN(('uid', user1), ('cn', 'users'), ('cn', 'accounts'),
-                          api.env.basedn),
-                    has_keytab=False,
-                    has_password=False,
-                )),
+                result=get_user_result(user1, u'Test', u'User1', 'add'),
             ),
         ),
 
@@ -594,8 +575,8 @@ class test_selinuxusermap(Declarative):
             desc='Try to delete similarly named HBAC rule %r' % hbacrule2,
             command=('hbacrule_del', [hbacrule2], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=hbacrule2,
+                result=dict(failed=[]),
+                value=[hbacrule2],
                 summary=u'Deleted HBAC rule "%s"' % hbacrule2,
             )
         ),
@@ -606,8 +587,8 @@ class test_selinuxusermap(Declarative):
             desc='Delete %r' % rule1,
             command=('selinuxusermap_del', [rule1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=rule1,
+                result=dict(failed=[]),
+                value=[rule1],
                 summary=u'Deleted SELinux User Map "%s"' % rule1,
             )
         ),
@@ -802,8 +783,8 @@ class test_selinuxusermap(Declarative):
             desc='Delete %r' % rule1,
             command=('selinuxusermap_del', [rule1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=rule1,
+                result=dict(failed=[]),
+                value=[rule1],
                 summary=u'Deleted SELinux User Map "%s"' % rule1,
             )
         ),
@@ -845,8 +826,8 @@ class test_selinuxusermap(Declarative):
             desc='Delete %r' % rule1,
             command=('selinuxusermap_del', [rule1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=rule1,
+                result=dict(failed=[]),
+                value=[rule1],
                 summary=u'Deleted SELinux User Map "%s"' % rule1,
             )
         ),
@@ -887,8 +868,8 @@ class test_selinuxusermap(Declarative):
             desc='Delete %r' % rule1,
             command=('selinuxusermap_del', [rule1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=rule1,
+                result=dict(failed=[]),
+                value=[rule1],
                 summary=u'Deleted SELinux User Map "%s"' % rule1,
             )
         ),
@@ -921,8 +902,8 @@ class test_selinuxusermap(Declarative):
             desc='Delete %r' % rule1,
             command=('selinuxusermap_del', [rule1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=rule1,
+                result=dict(failed=[]),
+                value=[rule1],
                 summary=u'Deleted SELinux User Map "%s"' % rule1,
             )
         ),
