@@ -22,8 +22,10 @@ Test the `ipalib/plugins/sudocmdgroup.py` module.
 
 from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import Declarative, fuzzy_uuid, fuzzy_sudocmddn
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_uuid,
+                                              fuzzy_sudocmddn)
 from ipapython.dn import DN
+import pytest
 
 sudocmdgroup1 = u'testsudocmdgroup1'
 sudocmdgroup2 = u'testsudocmdgroup2'
@@ -50,6 +52,8 @@ def create_command(sudocmd):
         ),
     )
 
+
+@pytest.mark.tier1
 class test_sudocmdgroup(Declarative):
     cleanup_commands = [
         ('sudocmdgroup_del', [sudocmdgroup1], {}),
@@ -600,8 +604,8 @@ class test_sudocmdgroup(Declarative):
             desc='Delete %r' % sudocmdgroup1,
             command=('sudocmdgroup_del', [sudocmdgroup1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=sudocmdgroup1,
+                result=dict(failed=[]),
+                value=[sudocmdgroup1],
                 summary=u'Deleted Sudo Command Group "testsudocmdgroup1"',
             )
         ),
@@ -638,8 +642,8 @@ class test_sudocmdgroup(Declarative):
             desc='Delete %r' % sudocmdgroup2,
             command=('sudocmdgroup_del', [sudocmdgroup2], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=sudocmdgroup2,
+                result=dict(failed=[]),
+                value=[sudocmdgroup2],
                 summary=u'Deleted Sudo Command Group "testsudocmdgroup2"',
             )
         ),
@@ -676,8 +680,8 @@ class test_sudocmdgroup(Declarative):
             desc='Now delete the sudo command %r' % sudocmd1,
             command=('sudocmd_del', [sudocmd1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=sudocmd1,
+                result=dict(failed=[]),
+                value=[sudocmd1],
                 summary=u'Deleted Sudo Command "%s"' % sudocmd1,
             )
         ),
