@@ -22,11 +22,18 @@ Test the `ipalib/plugins/sudorule.py` module.
 """
 
 from nose.tools import raises, assert_raises  # pylint: disable=E0611
+import six
 
-from xmlrpc_test import XMLRPC_test, assert_attr_equal
+from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test, assert_attr_equal
 from ipalib import api
 from ipalib import errors
+import pytest
 
+if six.PY3:
+    unicode = str
+
+
+@pytest.mark.tier1
 class test_sudorule(XMLRPC_test):
     """
     Test the `sudorule` plugin.
@@ -494,10 +501,10 @@ class test_sudorule(XMLRPC_test):
                 self.rule_name,
                 addattr='externalhost=%s' % self.test_invalid_host
             )
-        except errors.ValidationError, e:
+        except errors.ValidationError as e:
             assert unicode(e) == ("invalid 'externalhost': only letters, " +
-                "numbers, _, and - are allowed. " +
-                "DNS label may not start or end with -")
+                "numbers, '_', '-' are allowed. " +
+                "DNS label may not start or end with '-'")
         else:
             assert False
 
@@ -769,6 +776,10 @@ class test_sudorule(XMLRPC_test):
         with assert_raises(errors.ValidationError):
             api.Command['sudorule_mod'](self.rule_name, sudoorder=0)
 
+        # Try unsetting sudoorder from both rules
+        api.Command['sudorule_mod'](self.rule_name, sudoorder=None)
+        api.Command['sudorule_mod'](self.rule_name2, sudoorder=None)
+
 
     def test_m_sudorule_del(self):
         """
