@@ -21,11 +21,18 @@ Test the `ipalib/plugins/trust.py` module.
 """
 
 import nose
+import six
+
 from ipalib import api, errors
 from ipapython.dn import DN
 from ipatests.test_xmlrpc import objectclasses
-from xmlrpc_test import (Declarative, fuzzy_guid, fuzzy_domain_sid, fuzzy_string,
-        fuzzy_uuid, fuzzy_digits)
+from ipatests.test_xmlrpc.xmlrpc_test import (
+    Declarative, fuzzy_guid, fuzzy_domain_sid, fuzzy_string, fuzzy_uuid,
+    fuzzy_digits)
+import pytest
+
+if six.PY3:
+    unicode = str
 
 
 trustconfig_ad_config = DN(('cn', api.env.domain),
@@ -36,11 +43,13 @@ testgroup_dn = DN(('cn', testgroup), api.env.container_group, api.env.basedn)
 default_group = u'Default SMB Group'
 default_group_dn = DN(('cn', default_group), api.env.container_group, api.env.basedn)
 
+
+@pytest.mark.tier1
 class test_trustconfig(Declarative):
 
     @classmethod
-    def setUpClass(cls):
-        super(test_trustconfig, cls).setUpClass()
+    def setup_class(cls):
+        super(test_trustconfig, cls).setup_class()
         if not api.Backend.rpcclient.isconnected():
             api.Backend.rpcclient.connect(fallback=False)
         try:
