@@ -24,16 +24,27 @@ Common utility functions and classes for unit tests.
 import inspect
 import os
 from os import path
-import ldap
-import ldap.sasl
-import ldap.modlist
 import tempfile
 import shutil
 import re
+import uuid
+from contextlib import contextmanager
+
+import six
+import ldap
+import ldap.sasl
+import ldap.modlist
+
 import ipalib
+from ipalib import api
 from ipalib.plugable import Plugin
 from ipalib.request import context
 from ipapython.dn import DN
+from ipapython.ipautil import private_ccache, kinit_password, run
+from ipaplatform.paths import paths
+
+if six.PY3:
+    unicode = str
 
 
 class TempDir(object):
@@ -213,7 +224,7 @@ class Fuzzy(object):
         :param test: A callable used to perform equality test, e.g.
             ``lambda other: other >= 18``
         """
-        assert regex is None or isinstance(regex, basestring)
+        assert regex is None or isinstance(regex, six.string_types)
         assert test is None or callable(test)
         if regex is None:
             self.re = None
@@ -221,7 +232,7 @@ class Fuzzy(object):
             self.re = re.compile(regex)
             if type is None:
                 type = unicode
-            assert type in (unicode, str, basestring)
+            assert type in (unicode, bytes, six.string_types)
         self.regex = regex
         self.type = type
         self.test = test
@@ -309,7 +320,7 @@ def assert_deepequal(expected, got, doc='', stack=tuple()):
     if isinstance(got, tuple):
         got = list(got)
     if isinstance(expected, DN):
-        if isinstance(got, basestring):
+        if isinstance(got, six.string_types):
             got = DN(got)
     if not (isinstance(expected, Fuzzy) or callable(expected) or type(expected) is type(got)):
         raise AssertionError(
@@ -325,8 +336,14 @@ def assert_deepequal(expected, got, doc='', stack=tuple()):
             s_got = got
             s_expected = expected
         else:
-            s_got = sorted(got)
-            s_expected = sorted(expected)
+            try:
+                s_got = sorted(got)
+            except TypeError:
+                s_got = got
+            try:
+                s_expected = sorted(expected)
+            except TypeError:
+                s_expected = expected
         for (i, e_sub) in enumerate(s_expected):
             g_sub = s_got[i]
             assert_deepequal(e_sub, g_sub, doc, stack + (i,))
@@ -358,14 +375,11 @@ def raises(exception, callback, *args, **kw):
     Tests that the expected exception is raised; raises ExceptionNotRaised
     if test fails.
     """
-    raised = False
     try:
         callback(*args, **kw)
-    except exception, e:
-        raised = True
-    if not raised:
-        raise ExceptionNotRaised(exception)
-    return e
+    except exception as e:
+        return e
+    raise ExceptionNotRaised(exception)
 
 
 def getitem(obj, key):
@@ -430,7 +444,7 @@ class ClassChecker(object):
 
     def __get_cls(self):
         if self.__cls is None:
-            self.__cls = self._cls
+            self.__cls = self._cls  # pylint: disable=E1101
         assert inspect.isclass(self.__cls)
         return self.__cls
     cls = property(__get_cls)
@@ -443,31 +457,18 @@ class ClassChecker(object):
     subcls = property(__get_subcls)
 
     def get_subcls(self):
-        raise NotImplementedError(
+        raise AttributeError(
             self.__class__.__name__,
             'get_subcls()'
         )
 
-    def tearDown(self):
+    def teardown(self):
         """
         nose tear-down fixture.
         """
         context.__dict__.clear()
 
 
-def check_TypeError(value, type_, name, callback, *args, **kw):
-    """
-    Tests a standard TypeError raised with `errors.raise_TypeError`.
-    """
-    e = raises(TypeError, callback, *args, **kw)
-    assert e.value is value
-    assert e.type is type_
-    assert e.name == name
-    assert type(e.name) is str
-    assert str(e) == ipalib.errors.TYPE_ERROR % (name, type_, value)
-    return e
-
-
 def get_api(**kw):
     """
     Returns (api, home) tuple.
@@ -478,7 +479,7 @@ def get_api(**kw):
     home = TempHome()
     api = ipalib.create_api(mode='unit_test')
     api.env.in_tree = True
-    for (key, value) in kw.iteritems():
+    for (key, value) in kw.items():
         api.env[key] = value
     return (api, home)
 
@@ -493,7 +494,7 @@ def create_test_api(**kw):
     home = TempHome()
     api = ipalib.create_api(mode='unit_test')
     api.env.in_tree = True
-    for (key, value) in kw.iteritems():
+    for (key, value) in kw.items():
         api.env[key] = value
     return (api, home)
 
@@ -503,7 +504,7 @@ class PluginTester(object):
 
     def __get_plugin(self):
         if self.__plugin is None:
-            self.__plugin = self._plugin
+            self.__plugin = self._plugin  # pylint: disable=E1101
         assert issubclass(self.__plugin, Plugin)
         return self.__plugin
     plugin = property(__get_plugin)
@@ -518,9 +519,9 @@ class PluginTester(object):
         :param kw: Additional \**kw args to pass to `create_test_api`.
         """
         (api, home) = create_test_api(**kw)
-        api.register(self.plugin)
+        api.add_plugin(self.plugin)
         for p in plugins:
-            api.register(p)
+            api.add_plugin(p)
         return (api, home)
 
     def finalize(self, *plugins, **kw):
@@ -533,7 +534,7 @@ class PluginTester(object):
         o = api[namespace][self.plugin.__name__]
         return (o, api, home)
 
-    def tearDown(self):
+    def teardown(self):
         """
         nose tear-down fixture.
         """
@@ -660,3 +661,54 @@ class MockLDAP(object):
     def unbind(self):
         if self.connection is not None:
             self.connection.unbind_s()
+
+
+def prepare_config(template, values):
+    with open(template) as f:
+        template = f.read()
+
+    with tempfile.NamedTemporaryFile(mode='w', delete=False) as config:
+        config.write(template.format(**values))
+
+    return config.name
+
+
+def unlock_principal_password(user, oldpw, newpw):
+    userdn = "uid={},{},{}".format(
+        user, api.env.container_user, api.env.basedn)
+
+    args = [paths.LDAPPASSWD, '-D', userdn, '-w', oldpw, '-a', oldpw,
+            '-s', newpw, '-x']
+    return run(args)
+
+
+@contextmanager
+def change_principal(user, password, client=None, path=None):
+
+    if path:
+        ccache_name = path
+    else:
+        ccache_name = os.path.join('/tmp', str(uuid.uuid4()))
+
+    if client is None:
+        client = api
+
+
+    client.Backend.rpcclient.disconnect()
+
+    with private_ccache(ccache_name):
+        kinit_password(user, password, ccache_name)
+        client.Backend.rpcclient.connect()
+
+        try:
+            yield
+        finally:
+            client.Backend.rpcclient.disconnect()
+
+    client.Backend.rpcclient.connect()
+
+def get_group_dn(cn):
+    return DN(('cn', cn), api.env.container_group, api.env.basedn)
+
+def get_user_dn(uid):
+    return DN(('uid', uid), api.env.container_user, api.env.basedn)
